﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * CallContext
 *
 * Description:
 *
 *  nn::nex::CallContext クラスの高度な使い方のサンプルデモです。
 *  事前に CallContextBasic サンプルを参照してください。
 *
 *  本サンプルでは以下の例を示します。
 *
 *  - 非同期処理をキャンセルする。
 *  - 複数の非同期処理を並列に実行する。
 *
 *  本サンプルでは nex_demo_util::CallContextTrainer という非同期処理練習用のクラスを用います。
 */

#include <nex.h>
#include <NexDemoUtil/NexUtil.h>
#include <NexDemoUtil/CallContextTrainer.h>

namespace
{

/*!
    @brief 非同期処理完了時に呼び出されるコールバック関数。
*/
void CompletionFunction(nn::nex::CallContext* pCallContext)
{
    if (pCallContext->GetState() == nn::nex::CallContext::CallCancelled)
    {
        NEX_DEMO_LOG("In completion callback. Asynchronous call cancelled.\n");
        return;
    }
}

/*
    @brief 非同期処理をキャンセルする。

    非同期処理を途中で終了するにはCallContext::Cancel()を使います。
    CallContext::Cancel()を呼び出すとout引数を解放することができます。

    CallContext::Cancel()を呼び出した後は、実際にどこまで非同期処理が進んだかが不明な状態になります。
    例えばサーバーにRMCを発行する非同期処理をキャンセルした場合は、サーバーでそのRMCが処理されたかどうかが不明になります。
    そのため、RMCの結果を参照してその後のゲームの進行を決定する場合は、キャンセルすると問題が発生します。
    ゲームサーバーからログアウトするためRMCの結果が不要になったといったケースでCallContext::Cancel()を使用してください。
*/
void CallContextUsageCancel()
{
    nex_demo_util::CallContextTrainer trainer;
    trainer.SetDelay(1000);

    nn::nex::CallContext callContext;

    // 設定は任意ですが、完了時に呼び出されるコールバックを設定してキャンセルされたことを表示します。
    callContext.RegisterCompletionCallbackGeneric(&CompletionFunction);

    qInt32* pSum = new qInt32(0);
    trainer.Add(&callContext, 1, 2, pSum);

    // 1000ミリ秒で非同期処理が完了するところ、500ミリ秒で待機を打ち切ります。待機を打ち切っても非同期処理は継続しています。
    callContext.Wait(500);

    // 非同期処理をキャンセルします。
    callContext.Cancel();

    // キャンセル後はout引数を解放できます。
    delete pSum;

    // キャンセルしたCallContextのステートはCallContext::CallCancelled、結果はQERROR(Core, Cancelled)になります。
    nex_demo_util::ReportQResultReturnCode(callContext.GetOutcome(), "CallContext::Add cancelled.");


    // 実アプリケーションでは不要ですが、ログ出力を時系列順に行うために次の例に行く前に一度ディスパッチを行います。
    // CallContextTrainer::Add()がキャンセルされたことを受けて、このディスパッチ中に"Asynchronous call has been cancelled."と表示されます。
    nn::nex::Scheduler::GetInstance()->DispatchAll();

}

/*
    @brief 複数の非同期処理を並列に実行する。

    複数の非同期処理を並列に実行すると二番目以降に起動する非同期処理の待ち時間が隠蔽され、逐次実行するより全体が早く完了します。
 */
void CallContextUsageParallel()
{
    nex_demo_util::CallContextTrainer trainer;
    trainer.SetDelay(1000);

    nn::nex::CallContext callContext1;
    nn::nex::CallContext callContext2;
    nn::nex::CallContext callContext3;
    qInt32 sum1 = 0;
    qInt32 sum2 = 0;
    qInt32 sum3 = 0;

    // 連続で非同期処理を起動します。
    // 任意の個数の非同期処理を起動可能です。
    // 一般に非同期処理が完了する順番は不定ですが、
    // ProtocolCallContextを使いRMCを発行する場合は呼び出した順にサーバーで処理され、応答が返ってくるという特性があります。
    trainer.Add(&callContext1, 1, 2, &sum1);
    trainer.Add(&callContext2, 2, 3, &sum2);
    trainer.Add(&callContext3, 3, 5, &sum3);

    // 三つの非同期処理は並列に実行されるため、この三つのWait()がすべて完了するまでの時間は3秒ではなく1秒です。
    callContext1.Wait();
    callContext2.Wait();
    callContext3.Wait();

    if (callContext1.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(callContext1.GetOutcome(), "CallContext::Add Failed.");
        return;
    }
    if (callContext2.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(callContext2.GetOutcome(), "CallContext::Add Failed.");
        return;
    }
    if (callContext3.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(callContext3.GetOutcome(), "CallContext::Add Failed.");
        return;
    }
    NEX_DEMO_LOG("sum1: %d\n", sum1);
    NEX_DEMO_LOG("sum2: %d\n", sum2);
    NEX_DEMO_LOG("sum3: %d\n", sum3);
}

/*!
    @brief NEXの初期化後に様々な方法で非同期処理を行います。
 */
qInt NEXSampleMain()
{
    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);

    {
        // 様々な手法で非同期処理を行います。
        CallContextUsageCancel();
        CallContextUsageParallel();
    }

    // NEXの終了処理では、起動した全ての非同期処理が完了するのを待つようになっています。
    // そのため、事前に非同期処理が完了するまで待つか、CallContext::Cancel() でキャンセルするようにしてください。
    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("  CallContextAdvanced sample\n");
    NEX_DEMO_LOG("*******************************\n\n");


    qInt returnCode = NEXSampleMain();


    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
