﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * CallContext
 *
 * Description:
 *
 *  nn::nex::CallContext クラスの使い方のサンプルデモです。
 *  NEX では非同期処理の管理に nn::nex::CallContext を使います。
 *  非同期処理とはネットワーク通信などの即座に完了しないタスクをノンブロッキングで行うことを指します。
 *  非同期処理を伴う関数の第一引数には nn::nex::CallContext もしくはその派生クラスのオブジェクトを取るようになっています。
 *  nn::nex::CallContext を使うとコールバック、ポーリング、ブロッキングなど様々なスタイルで非同期処理を行うことができます。
 *
 *  本サンプルでは以下の方法で非同期処理を行う例を示します。
 *
 *  - 関数ポインタを設定して非同期処理の完了時にコールバックする。
 *  - メンバ関数ポインタを設定して非同期処理の完了時にコールバックする。
 *  - 関数オブジェクトを設定して非同期処理の完了時にコールバックする。
 *  - ラムダ式を設定して非同期処理の完了時にコールバックする。(ラムダ式はC++11対応コンパイラ使用時のみ利用可能)
 *  - ポーリングして非同期処理の完了をチェックする。
 *  - ブロッキングして非同期処理の完了を待つ。
 *  - 非同期処理のエラーハンドリングを行う。
 *  - 非同期処理の完了を待たずに処理を進める。
 *
 *  本サンプルでは nex_demo_util::CallContextTrainer という非同期処理練習用のクラスを用います。
 */

#include <nex.h>
#include <NexDemoUtil/NexUtil.h>
#include <NexDemoUtil/CallContextTrainer.h>

namespace
{
// 非同期処理に設定した遅延時間に加えて待つ猶予時間(ミリ秒)。
const nn::nex::TimeInterval COMPLETION_MARGIN_MS = 100;

/*!
    @brief 非同期処理完了時に呼び出されるコールバック関数。

    引数にnn::nex::CallContext*を取る関数である必要があります。
    非同期処理に使用するnn::nex::CallContextがnn::nex::ProtocolCallContextなどの派生型である場合はその型を引数に取ることもできるため、
    アプリケーション側でダイナミックキャストする必要はありません。
*/
void CompletionFunction(nn::nex::CallContext* pCallContext)
{
    if (pCallContext->GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(pCallContext->GetOutcome(), "CallContext::Log Failed.");
        return;
    }
    NEX_DEMO_LOG("Succeeded to Log.\n");
}

/*!
    @brief コールバック関数を登録して非同期処理の完了を待つ。

    この方法はアプリケーションでスレッドを作らずに非同期処理を実行する場合に適しています。
*/
void CallContextUsageCallbackFunction()
{
    const nn::nex::TimeInterval delayMs = 1000;

    {
        // 非同期処理の特徴を明示するため、このスコープ内で非同期処理を呼び出します。
        // このスコープでは非同期処理の開始のみを行い、完了はしません。

        nex_demo_util::CallContextTrainer trainer;
        trainer.SetDelay(delayMs); // 擬似的に非同期処理が完了するまでの遅延を発生させます。

        // 非同期処理を管理するオブジェクトを生成する。
        // CallContextオブジェクトは非同期処理が完了するまで保持する必要があるため、ここでは動的に生成します。
        nn::nex::CallContext* pCallContext = new nn::nex::CallContext();

        // 非同期処理完了時に呼び出されるコールバック関数を登録します。
        pCallContext->RegisterCompletionCallbackGeneric(&CompletionFunction);

        // 非同期処理完了時に自動的にpContextがdeleteされるように設定します。
        // 非同期処理ではローカルスコープ内で処理が完了しないケースが多いのでこの設定は便利です。
        pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);

        // 非同期処理を開始します。非同期処理の開始処理はブロッキングせず即座に制御を戻します。
        // この例ではLog()の呼び出しから1秒後にデバッグログに"Hallo asynchronous call"と表示されます。
        trainer.Log(pCallContext, "Hallo asynchronous call");
    }

    // 非同期処理の開始は以上で完了しています。
    // これ以降は別の関数やスレッドでScheudler::DispatchAll()を呼び出している想定です。
    // この非同期処理が完了するとScheudler::DispatchAll()からコールバックが呼び出されます。
    nn::nex::Time startedTime = nn::nex::SystemClock::GetTime();
    while (true)
    {
        nn::nex::Platform::Sleep(nex_demo_util::SLEEP_DURATION);
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // 非同期処理が完了したと思われる時間が経過したら抜ける
        if (nn::nex::SystemClock::GetTime() - startedTime >= delayMs + COMPLETION_MARGIN_MS)
        {
            break;
        }
    }
}


// コールバックが呼び出されるクラス
class SomeClass
{
public:
    SomeClass()
    {
    }
    void CallbackFunction(nn::nex::CallContext* pCallContext)
    {
        if (pCallContext->GetState() != nn::nex::CallContext::CallSuccess)
        {
            nex_demo_util::ReportQResultReturnCode(pCallContext->GetOutcome(), "CallContext::Log Failed.");
            return;
        }
        NEX_DEMO_LOG("Succeeded to Log.\n");
    }
};

/*!
    @brief メンバ関数をコールバックに登録して非同期処理の完了を待つ。

    CallContextUsageCallbackFunction()での例とほぼ同じですが、こちらはメンバ関数をコールバックに登録する例です。
    この方法はアプリケーションでスレッドを作らずに非同期処理を実行する場合に適しています。
*/
void CallContextUsageCallbackMemberFunction()
{
    const nn::nex::TimeInterval delayMs = 1000;

    SomeClass someClass;
    {
        nex_demo_util::CallContextTrainer trainer;
        trainer.SetDelay(delayMs); // 擬似的に非同期処理が完了するまでの遅延を発生させます。

        nn::nex::CallContext* pCallContext = new nn::nex::CallContext();

        // 非同期処理完了時に呼び出されるコールバック関数を登録します。
        // メンバ関数を登録する例です。
#ifdef NEX_LAMBDA_AND_FUNCTION_FEATURE_SUPPORT
        // C++11ではstd::bind()を利用してメンバ関数を登録できます。
        pCallContext->RegisterCompletionCallbackGeneric(std::bind(&SomeClass::CallbackFunction, &someClass, std::placeholders::_1));
#else
        // C++03では関数アダプタを組み合わせることでメンバ関数を登録できます。
        pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&SomeClass::CallbackFunction), &someClass));
#endif
        pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
        trainer.Log(pCallContext, "Hallo asynchronous call");
    }

    // 非同期処理の開始は以上で完了しています。
    // これ以降は別の関数やスレッドでScheudler::DispatchAll()を呼び出している想定です。
    // この非同期処理が完了するとScheudler::DispatchAll()からコールバックが呼び出されます。
    nn::nex::Time startedTime = nn::nex::SystemClock::GetTime();
    while (true)
    {
        nn::nex::Platform::Sleep(nex_demo_util::SLEEP_DURATION);
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // 非同期処理が完了したと思われる時間が経過したら抜ける
        if (nn::nex::SystemClock::GetTime() - startedTime >= delayMs + COMPLETION_MARGIN_MS)
        {
            break;
        }
    }
}

/*!
    @brief コールバック用の関数オブジェクト。
*/
struct CompletionCallbackFunctor
{
    CompletionCallbackFunctor()
    {
    }

    void operator()(nn::nex::CallContext* pCallContext)
    {
        if (pCallContext->GetState() != nn::nex::CallContext::CallSuccess)
        {
            nex_demo_util::ReportQResultReturnCode(pCallContext->GetOutcome(), "CallContext::Add Failed.");
            return;
        }
        NEX_DEMO_LOG("sum: %d\n", m_sum);
    }

    qInt32* GetSumPointer()
    {
        return &m_sum;
    }
private:
    // m_sumのアドレスを非同期処理のoutパラメータに設定しているため、安全のためコピー禁止にしています。
    CompletionCallbackFunctor(const CompletionCallbackFunctor&);

    qInt32 m_sum;
};

/*!
    @brief コールバックに関数オブジェクトを登録して非同期処理の完了を待つ。

    この方法はアプリケーションでスレッドを作らずに非同期処理を実行する場合に適しています。
*/
void CallContextUsageCallbackFunctor()
{
    const nn::nex::TimeInterval delayMs = 1000;
    {
        nex_demo_util::CallContextTrainer trainer;
        trainer.SetDelay(delayMs);

        nn::nex::CallContext* pCallContext = new nn::nex::CallContext();
        CompletionCallbackFunctor* pCallbackFunctor = new CompletionCallbackFunctor();

        // 非同期処理完了時に呼び出される関数オブジェクトを登録します。
        // 関数オブジェクトへのポインタを登録した場合、第二引数をtrueにするとコールバック後に自動的にpCallbackFunctorをdeleteします。
        pCallContext->RegisterCompletionCallbackGeneric(pCallbackFunctor, true);
        pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);

        // 非同期処理を行う関数のoutパラメータ (Add()では第四引数) は非同期処理が完了したときに
        // 値が書き込まれるため、呼び出し側で実体を保持しておく必要があります。
        // この例ではpCallbackFunctorオブジェクト内の変数に直接書き込むように設定しています。
        trainer.Add(pCallContext, 1, 2, pCallbackFunctor->GetSumPointer());
    }

    // 非同期処理の開始は以上で完了しています。
    // これ以降は別の関数やスレッドでScheudler::DispatchAll()を呼び出している想定です。
    // この非同期処理が完了するとScheudler::DispatchAll()からコールバックが呼び出されます。
    nn::nex::Time startedTime = nn::nex::SystemClock::GetTime();
    while (true)
    {
        nn::nex::Platform::Sleep(nex_demo_util::SLEEP_DURATION);
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // 非同期処理が完了したと思われる時間が経過したら抜ける
        if (nn::nex::SystemClock::GetTime() - startedTime >= delayMs + COMPLETION_MARGIN_MS)
        {
            break;
        }
    }
}

#ifdef NEX_LAMBDA_AND_FUNCTION_FEATURE_SUPPORT
/*!
    @brief ラムダ式を登録して非同期処理の完了を待つ。C++11対応コンパイラ向けです。

    この方法はアプリケーションでスレッドを作らずに非同期処理を実行する場合に適しています。
*/
void CallContextUsageCallbackLambda()
{
    const nn::nex::TimeInterval delayMs = 1000;
    {
        nex_demo_util::CallContextTrainer trainer;
        trainer.SetDelay(delayMs);

        nn::nex::CallContext* pCallContext = new nn::nex::CallContext();

        // outパラメータは非同期処理の完了まで実体を保持する必要があるため、ここでは動的に生成し、
        // 完了時のコールバック内で削除しています。
        qInt32* pSum = new qInt32(0);

        // ラムダ式をコールバックとして登録します。
        pCallContext->RegisterCompletionCallbackGeneric([pSum](nn::nex::CallContext* pCallContext)
        {
            if (pCallContext->GetState() == nn::nex::CallContext::CallSuccess)
            {
                NEX_DEMO_LOG("sum: %d\n", *pSum);
            }
            else
            {
                nex_demo_util::ReportQResultReturnCode(pCallContext->GetOutcome(), "CallContext::Add Failed.");
            }
            delete pSum;
        });
        pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
        trainer.Add(pCallContext, 1, 2, pSum);
    }

    // 非同期処理の開始は以上で完了しています。
    // これ以降は別の関数やスレッドでScheudler::DispatchAll()を呼び出している想定です。
    // この非同期処理が完了するとScheudler::DispatchAll()からコールバックが呼び出されます。
    nn::nex::Time startedTime = nn::nex::SystemClock::GetTime();
    while (true)
    {
        nn::nex::Platform::Sleep(nex_demo_util::SLEEP_DURATION);
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // 非同期処理が完了したと思われる時間が経過したら抜ける
        if (nn::nex::SystemClock::GetTime() - startedTime >= delayMs + COMPLETION_MARGIN_MS)
        {
            break;
        }
    }
}
#endif

/*
    @brief ポーリングで非同期処理の完了を待つ。

    この方法はアプリケーションでスレッドを作らずに非同期処理を実行する場合に適しています。
    一般にポーリングは能動的にチェックする手間がかかるため、コールバック方式の方が優れています。
*/
void CallContextUsagePolling()
{
    const nn::nex::TimeInterval delayMs = 1000;
    qInt32 sum = 0;
    nn::nex::CallContext callContext;
    {
        nex_demo_util::CallContextTrainer trainer;
        trainer.SetDelay(delayMs);

        trainer.Add(&callContext, 1, 2, &sum);
    }

    while (true)
    {
        nn::nex::Platform::Sleep(nex_demo_util::SLEEP_DURATION);
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // CallContext::GetState()で非同期処理が完了したかどうかをポーリングできます。
        // CallContext::GetOutcome()を使用してのポーリングは適切ではありませんのでご注意ください。
        if (callContext.GetState() != nn::nex::CallContext::CallInProgress)
        {
            if (callContext.GetState() != nn::nex::CallContext::CallSuccess)
            {
                nex_demo_util::ReportQResultReturnCode(callContext.GetOutcome(), "CallContext::Add Failed.");
                break;
            }
            NEX_DEMO_LOG("sum: %d\n", sum);
            break;;
        }
    }
}

/*
    @brief 同期的に非同期処理を実行する。

    この方法はアプリケーションで独自にスレッドを作り、そこで実行する場合に適しています。
*/
void CallContextUsageBlocking()
{
    nex_demo_util::CallContextTrainer trainer;
    trainer.SetDelay(1000);

    nn::nex::CallContext callContext;
    qInt32 sum = 0;
    trainer.Add(&callContext, 1, 2, &sum);

    // CallContext::Wait()を呼び出すと、紐づけられた非同期処理が完了するまでブロッキングし、内部でScheduler::Dispatch()を呼び出し処理を進めます。
    // CallContext::Wait()は紐づけられた非同期処理だけでなくNEXの内部処理全てを進めます。
    callContext.Wait();
    if (callContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(callContext.GetOutcome(), "CallContext::Add Failed.");
        return;
    }
    NEX_DEMO_LOG("sum: %d\n", sum);
}

/*
    @brief 非同期処理を実行するが、その結果を無視する。

    この方法は非同期処理の結果が必要ない場合に適しています。
    非同期処理の成功/失敗を問わず、outパラメータも受け取る必要がないケースで利用できます。

    信頼性のあるプロトコルを使用した通信を行う非同期処理でこの方法を採った場合は、
    通信の切断がない限りいずれ相手にリクエストが到達することは保証されます。(NGSとの通信は信頼性のある通信によって行われます。)
    そのため、例えば MatchmakeExtensionClient::EndParticipation() を使ってマッチメイクセッションからの退出は確実に行うが、
    その完了まで待たなくて良いケースなどで利用できます。
*/
void CallContextUsageDontCareOutcome()
{
    const nn::nex::TimeInterval delayMs = 1000;
    {
        nex_demo_util::CallContextTrainer trainer;
        trainer.SetDelay(delayMs);

        nn::nex::CallContext* pCallContext = new nn::nex::CallContext();
        pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);

        // マニュアルに特別記載がない限り、非同期処理のoutパラメータにNULLを指定することで
        // 非同期処理が完了した際に値を格納しないようにできます。
        trainer.Add(pCallContext, 1, 2, NULL);
    }

    // 非同期処理の開始は以上で完了しています。
    // これ以降は別の関数やスレッドでScheudler::DispatchAll()を呼び出している想定です。
    // この非同期処理が完了するとScheudler::DispatchAll()からコールバックが呼び出されます。
    nn::nex::Time startedTime = nn::nex::SystemClock::GetTime();
    while (true)
    {
        nn::nex::Platform::Sleep(nex_demo_util::SLEEP_DURATION);
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // 非同期処理が完了したと思われる時間が経過したら抜ける
        if (nn::nex::SystemClock::GetTime() - startedTime >= delayMs + COMPLETION_MARGIN_MS)
        {
            break;
        }
    }
}

/*
    @brief 非同期処理のエラーをハンドリングする方法を示します。
*/
void CallContextUsageErrorHandling()
{
    nex_demo_util::CallContextTrainer trainer;
    trainer.SetDelay(1000);

    // 擬似的に非同期処理の結果がエラーになるように設定します。
    trainer.SetResult(QRESULT_ERROR(Core, InvalidArgument));

    nn::nex::CallContext callContext;
    qInt32 sum = 0;
    trainer.Add(&callContext, 1, 2, &sum);
    callContext.Wait();
    if (callContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(callContext.GetOutcome(), "CallContext::Add Failed. (But expected)");

        // if文で特定の結果であることを確認するにはnn::nex::qResultとQERRORマクロで生成したリターンコードを比較します。
        if (callContext.GetOutcome() == QERROR(Core, InvalidArgument))
        {
            NEX_DEMO_LOG("This is a Core::InvalidArgument error. (Checked by if statement.)\n");
        }

        // switch文で特定の結果であることを確認するにはqResult::GetReturnCode()で取得したリターンコードと
        // QERRORマクロで生成したリターンコードを比較します。
        switch (callContext.GetOutcome().GetReturnCode())
        {
        case QERROR(Core, InvalidArgument):
            NEX_DEMO_LOG("This is a Core::InvalidArgument error. (Checked by switch statement.)\n");
            break;
        case QERROR(Core, Timeout):
            break;
        case QERROR(DataStore, NotFound):
            break;
        default:
            break;
        }
        return;
    }

    // ここには到達しません。
    NEX_DEMO_LOG("sum: %d\n", sum);
}

/*!
    @brief NEXの初期化後に様々な方法で非同期処理を行います。
 */
qInt NEXSampleMain()
{
    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);

    {
        // 様々な方法で非同期処理を行います。
        CallContextUsageCallbackFunction();
        CallContextUsageCallbackMemberFunction();
        CallContextUsageCallbackFunctor();
#ifdef NEX_LAMBDA_AND_FUNCTION_FEATURE_SUPPORT
        CallContextUsageCallbackLambda();
#endif
        CallContextUsagePolling();
        CallContextUsageBlocking();
        CallContextUsageErrorHandling();
        CallContextUsageDontCareOutcome();
    }

    // NEXの終了処理では、起動した全ての非同期処理が完了するのを待つようになっています。
    // そのため、事前に非同期処理が完了するまで待つか、CallContext::Cancel() でキャンセルするようにしてください。
    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("    CallContextBasic sample\n");
    NEX_DEMO_LOG("*******************************\n\n");


    qInt returnCode = NEXSampleMain();


    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
