﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * ThreadMode
 *
 * Description:
 *
 *  スレッドモードの違いを説明するためのサンプルデモです。
 *  NEXではアプリケーションの都合に応じて様々なスレッドモードを選択出来ます。
 *  代表的なスレッドモードはnn::nex::Core::ThreadModeSafeTransportBufferとnn::nex::Core::ThreadModeSafeTransportBufferです。
 *  この二つのスレッドモードを使い非同期処理をどのように行うかを例示します。
 *
 *  本サンプルでは nex_demo_util::CallContextTrainer という非同期処理練習用のクラスを用います。
 */

#include <nex.h>
#include <NexDemoUtil/NexUtil.h>
#include <NexDemoUtil/CallContextTrainer.h>

namespace
{
// 非同期処理に設定した遅延時間に加えて待つ猶予時間(ミリ秒)。
const nn::nex::TimeInterval COMPLETION_MARGIN_MS = 100;

/*!
    @brief コールバック用の関数オブジェクト。
*/
struct CompletionCallbackFunctor
{
    CompletionCallbackFunctor()
    {
    }

    void operator()(nn::nex::CallContext* pCallContext)
    {
        nn::nex::StringStream ss;

        // nn::nex::ObjectThreadRoot::GetCurrentThreadID() は現在のスレッドIDを取得するNEXの非公開関数です。
        // 本サンプルではコールバックするスレッドがスレッドモードによって異なる事を示すために使用しています。
        // 通常のアプリケーションでは使用しないでください。
        ss << "Completion callback was called. Called thread ID is " << nn::nex::hex << nn::nex::ObjectThreadRoot::GetCurrentThreadID();
        NEX_DEMO_LOG(STRING_FORMAT "\n", ss.CStr());

        if (pCallContext->GetState() != nn::nex::CallContext::CallSuccess)
        {
            nex_demo_util::ReportQResultReturnCode(pCallContext->GetOutcome(), "CallContext::Add Failed.");
            return;
        }
        NEX_DEMO_LOG("sum: %d\n", m_sum);
    }

    qInt32* GetSumPointer()
    {
        return &m_sum;
    }
private:
    // m_sumのアドレスを非同期処理のoutパラメータに設定しているため、安全のためコピー禁止にしています。
    CompletionCallbackFunctor(const CompletionCallbackFunctor&);

    qInt32 m_sum;
};

/*!
    @brief 非インターナルスレッドモードで非同期処理を行います。

    完了時のコールバックがScheduler::DispatchAll()から行われることを確認してください。
*/
void SampleForNonInternalThreadMode()
{
    const nn::nex::TimeInterval delayMs = 1000;
    {
        nex_demo_util::CallContextTrainer trainer;
        trainer.SetDelay(delayMs);

        nn::nex::CallContext* pCallContext = new nn::nex::CallContext();
        CompletionCallbackFunctor* pCallbackFunctor = new CompletionCallbackFunctor();

        // 非同期処理完了時に呼び出される関数オブジェクトを登録します。
        // 関数オブジェクトへのポインタを登録した場合、第二引数をtrueにするとコールバック後に自動的にpCallbackFunctorをdeleteします。
        pCallContext->RegisterCompletionCallbackGeneric(pCallbackFunctor, true);
        pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);

        // 非同期処理を行う関数のoutパラメータ (Add()では第四引数) は非同期処理が完了したときに
        // 値が書き込まれるため、呼び出し側で実体を保持しておく必要があります。
        // この例ではpCallbackFunctorオブジェクト内の変数に直接書き込むように設定しています。
        trainer.Add(pCallContext, 1, 2, pCallbackFunctor->GetSumPointer());
    }

    // 非同期処理の開始は以上で完了しています。
    // これ以降は別の関数やスレッドでScheudler::DispatchAll()を呼び出している想定です。
    // この非同期処理が完了するとScheudler::DispatchAll()からコールバックが呼び出されます。
    nn::nex::Time startedTime = nn::nex::SystemClock::GetTime();
    while (true)
    {
        nn::nex::Platform::Sleep(nex_demo_util::SLEEP_DURATION);

        // NEXの内部処理を進めるために定期的にScheduler::Dispatch()もしくはDispatchAll()の呼び出しが必要です。
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // 非同期処理が完了したと思われる時間が経過したら抜ける
        if (nn::nex::SystemClock::GetTime() - startedTime >= delayMs + COMPLETION_MARGIN_MS)
        {
            break;
        }
    }
}

/*!
    @brief インターナルスレッドモードで非同期処理を行います。

    完了時のコールバックがNEX内部のスレッドから行われることを確認してください。
*/
void SampleForInternalThreadMode()
{
    const nn::nex::TimeInterval delayMs = 1000;
    {
        nex_demo_util::CallContextTrainer trainer;
        trainer.SetDelay(delayMs);

        nn::nex::CallContext* pCallContext = new nn::nex::CallContext();
        CompletionCallbackFunctor* pCallbackFunctor = new CompletionCallbackFunctor();

        // 非同期処理完了時に呼び出される関数オブジェクトを登録します。
        // 関数オブジェクトへのポインタを登録した場合、第二引数をtrueにするとコールバック後に自動的にpCallbackFunctorをdeleteします。
        pCallContext->RegisterCompletionCallbackGeneric(pCallbackFunctor, true);
        pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);

        // 非同期処理を行う関数のoutパラメータ (Add()では第四引数) は非同期処理が完了したときに
        // 値が書き込まれるため、呼び出し側で実体を保持しておく必要があります。
        // この例ではpCallbackFunctorオブジェクト内の変数に直接書き込むように設定しています。
        trainer.Add(pCallContext, 1, 2, pCallbackFunctor->GetSumPointer());
    }

    // インターナルスレッドモードではScheduler::Dispatch()やDispatchAll()の呼び出しは不要です。
    // NEX内部のスレッドによりScheduler::DispatchAll()と同等の処理が行われます。
    // ここでスリープしている間にNEX内部のスレッドから完了を通知するコールバックが行われます。
    // 競合を回避するために適宜排他処理が必要になります。
    nn::nex::Platform::Sleep(delayMs + COMPLETION_MARGIN_MS);
}

/*!
    @brief NEXの初期化後に様々な方法で非同期処理を行います。
 */
qInt NEXSampleMain()
{
    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // スレッドモードの違いによりコールバックするスレッドが異なることを分かりやすくするために
    // スレッドIDを出力するようにログ出力フォーマットを変更します。
    nn::nex::EventLog::GetInstance()->GetOutputFormat()->ShowThreadID(true);
    {
        // スレッドモードの設定と初期化
        nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
        nn::nex::Initialize(true);

        // 非インターナルスレッドモードでのNEXの使い方を示します。
        SampleForNonInternalThreadMode();

        nn::nex::Finalize(true);
    }

    {
        // スレッドモードの設定と初期化
        nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeInternalTransportBuffer);
        nn::nex::Initialize(true);

        // インターナルスレッドモードでのNEXの使い方を示します。
        SampleForInternalThreadMode();

        nn::nex::Finalize(true);
    }


    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("     ThreadMode sample\n");
    NEX_DEMO_LOG("*******************************\n\n");


    qInt returnCode = NEXSampleMain();


    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
