﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * CtrLogin
 *
 * Description:
 *
 *  CTR からゲームサーバーにログインするサンプルです。
 *
 */

#include <nex.h>
#include <NexDemoUtil/NexUtil.h>

namespace
{

/*!
    @brief NGSから同期的にログインします。

    @param[in,out] pNgsFacade ログインをまだ行っていないオブジェクトを指定してください。

    @return ログインに成功したかどうか。
 */
qBool Login(nn::nex::NgsFacade *pNgsFacade)
{
    nn::friends::Initialize();

    // フレンドサーバーへのログインとゲームサーバーへのログインを続けて行います。
    {
        // nex_demo_util::LoginUtility::LoginToNfsAndNgsBlock()はフレンドサーバーへのログイン処理と
        // NgsFacade::Login()を一括で行う関数で、そのままアプリケーションで利用できます。
        // nex_demo_util::LoginUtility::LoginToNfsAndNgsBlock()はログインが完了するまで制御を戻しません。(ブロッキング関数)
        // ブロッキングせずに処理を行いたい場合(つまりいわゆる非同期処理をしたい場合)は
        // nex_demo_util::LoginUtility::LoginToNfsAndNgs()を使用してください。
        // 非同期処理についての詳細はCallContextBasicサンプルデモを参照してください。
        //
        // ゲームサーバーIDとアクセスキーはサンプルデモ用のものを使用しています。
        // 実際のゲームではOMASから発行されたものを使用してください。
        qInt32 errorCode = nex_demo_util::LoginUtility::LoginToNfsAndNgsBlock(pNgsFacade,
                                                                              nex_demo_util::GAME_SERVER_ID,
                                                                              nex_demo_util::NGS_ACCESS_KEY,
                                                                              nex_demo_util::RECOMMENDED_LOGIN_TIMEOUT_MS);
        if (errorCode != 0)
        {
            // エラーコードをエラー・EULAアプレットで表示してください。
            // (バックグラウンドで先行してログインするケースなど、ユーザーに通知しなくても続行可能なケースは除きます)
            return false;
        }
    }

    NEX_DEMO_LOG("Succeeded to Login to NGS\n");
    return true;
}

/*!
    @brief NGSから同期的にログアウトします。

    @param[in,out] pNgsFacade ログイン済みのオブジェクトを指定してください。
 */
void Logout(nn::nex::NgsFacade *pNgsFacade)
{
    // ゲームサーバーとフレンドサーバーからログアウトします
    nn::nex::ProtocolCallContext logoutCallContext;
    pNgsFacade->Logout(&logoutCallContext);
    logoutCallContext.Wait();

    nn::friends::Logout();
}

/*!
    @brief NEXの初期化後にNGSへログインします。
 */
qInt NEXSampleMain()
{
    // インターネットに接続します
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit failed.\n");
        return 1;
    }

    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // ログレベルの設定(設定を行うかどうかは任意です)
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);

    {
        // ゲームサーバーへログインするためのクライアントを作成します。
        nn::nex::NgsFacade ngsFacade;

        // ゲームサーバーにログインします。
        if (Login(&ngsFacade))
        {
            // ログアウトします。
            Logout(&ngsFacade);
        }
    }
    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    // インターネット接続を終了します。
    nex_demo_util::NetTerm();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("     CtrLogin sample\n");
    NEX_DEMO_LOG("*******************************\n\n");

    qInt returnCode = NEXSampleMain();

    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
