﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * Basic
 *
 * Description:
 *
 *  ゲームサーバーにログインし、ゲームサーバーに対してRMCを発行するサンプルです。
 *
 *  RMCとはリモートメソッドコールの略で、いわゆるRPC(リモートプロシージャコール)を実現します。
 *  本サンプルではRMCの例としてランキングを使用しています。ランキングについての詳細はRankingサンプルを参照してください。
 */

#include <nex.h>
#include <NexDemoUtil/NexUtil.h>

namespace
{
// 使用するランキングのカテゴリ。(Rankingサンプルと同一の値となっています。)
const qUnsignedInt32 RANKING_CATEGORY = 0;

/*!
    @brief NgsFacadeに定義されているRMCをゲームサーバーに発行します。

    @param[in] pNgsFacade ログイン済みのオブジェクトを指定してください。
 */
void CallNgsFacadeRmc(nn::nex::NgsFacade *pNgsFacade)
{
    // NgsFacade::TestConnectivity()を呼び出します。
    // これはNGSとの疎通を確認するRMCです。
    // NgsFacadeに定義されているRMCは他のクライアントと異なりBind()などが不要です。

    // 非同期処理を管理するオブジェクトを生成します。
    nn::nex::ProtocolCallContext callContext;

    // RMCを発行します。この処理は即座に制御を戻します。この関数から戻った段階ではまだ通信が完了していません。
    pNgsFacade->TestConnectivity(&callContext);

    // 通信が完了するのを待ちます。
    // ここではCallContext::Wait()を呼び出し、非同期処理を同期的に処理します。
    // 完了時にコールバックをするなど、他の手法を使い非同期処理を行いたい場合はCallContextBasicサンプルを参照してください。
    callContext.Wait();

    // 非同期処理の結果を取得します。
    nn::nex::qResult result = callContext.GetOutcome();
    if (!result)
    {
        nex_demo_util::ReportQResultReturnCode(result, "TestConnectivity() failed.");
        return;
    }
    NEX_DEMO_LOG("Succeeded to call NEX RMC\n");
}

/*!
    @brief ランキングのリストを表示する。

    @param[in] rankDataList 表示したいランキングデータ。
 */
void PrintRankDataList(const nn::nex::qVector<nn::nex::RankingRankData>& rankDataList)
{
    if(rankDataList.empty())
    {
        NEX_DEMO_LOG("RankDataList is empty.\n");
    }

    nn::nex::qVector<nn::nex::RankingRankData>::const_iterator it = rankDataList.begin();
    while(it != rankDataList.end())
    {
        NEX_DEMO_LOG("** [%2d] -------------------------------\n", static_cast<int>(it - rankDataList.begin()));
        NEX_DEMO_LOG("** principalId    = " NEX_PID_FORMAT "\n", it->GetPrincipalId());
        NEX_DEMO_LOG("** order          = %d\n", it->GetOrder());
        NEX_DEMO_LOG("** category       = %d\n", it->GetCategory());
        const nn::nex::DateTime& updateTime = it->GetUpdateTime();
        (void)updateTime;
        NEX_DEMO_LOG("** updateTime     = %04d-%02d-%02d %02d:%02d:%02d\n",
                updateTime.GetYear(), updateTime.GetMonth(), updateTime.GetDay(),
                updateTime.GetHour(), updateTime.GetMinute(), updateTime.GetSecond());
        NEX_DEMO_LOG("** score          = %d\n", it->GetScore());

        ++it;
    }
}

/*!
    @brief RMC呼び出しの例としてランキングのRMCを呼び出します。

    @param[in,out] pNgsFacade ログイン済みのオブジェクトを指定してください。
 */
void CallRankingRmc(nn::nex::NgsFacade *pNgsFacade)
{
    // ランキングのスコアを参照するRMCを呼び出します。
    // (参照した結果、空のデータがサーバーから返される事があります。
    // ランキングスコアをアップロードするにはRankingのサンプルデモを参照・実行してください。)

    // ゲームサーバー向けのRMCは用途ごとに各種クライアントクラスに分類されています。
    // ランキングの場合はRankingClientにまとめられています。
    // このクライアントを生成し、NgsFacade::GetCredentials()で取得した証明書をServiceClient::Bind()に渡すことで
    // そのユーザーとしてRMCを呼び出すことができるようになります。
    nn::nex::RankingClient rankingClient;
    rankingClient.Bind(pNgsFacade->GetCredentials());  // ここでセットしたユーザーの証明書でRMCを呼び出します。

    // 非同期処理を管理するオブジェクトを生成します。
    nn::nex::ProtocolCallContext oContext;

    // サーバへ送信するパラメータを生成します。
    // これは順位計算パラメータですが、初期値のまま送信します。
    nn::nex::RankingOrderParam orderParam;

    // ランキング取得モード。
    nn::nex::RankingConstants::RankingMode rankingMode = nn::nex::RankingConstants::RANKING_MODE_RANGE;

    // サーバから結果を受け取る変数を初期化します。
    // 非同期処理の完了後、ここにランキングの取得結果が格納されます。
    nn::nex::RankingResult rankingResult;

    // ランキングリストを取得するRMCを発行します。
    // この処理は即座に制御を戻します。この関数から戻った段階ではまだ通信が完了していません。
    rankingClient.GetRanking(&oContext, rankingMode, RANKING_CATEGORY, orderParam, &rankingResult);

    // 通信が完了するのを待ちます。
    // ここではCallContext::Wait()を呼び出し、非同期処理を同期的に処理します。
    // 完了時にコールバックをするなど、他の手法を使い非同期処理を行いたい場合はCallContextBasicサンプルを参照してください。
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to GetRanking()\n");
        return;
    }
    // 結果を出力します
    NEX_DEMO_LOG("GetRanking(RANKING_MODE_RANGE) Result:\n");
    PrintRankDataList(rankingResult.GetRankDataList());
}

/*!
    @brief NEXの初期化後にNGSへログインし、RMCを発行します。
 */
qInt NEXSampleMain()
{
    // インターネットに接続します
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit failed.\n");
        return 1;
    }

    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // ログレベルの設定(設定を行うかどうかは任意です)
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);

    {
        // ゲームサーバーへログインするためのクライアントを用意します。
        nn::nex::NgsFacade ngsFacade;
        nex_demo_util::LoginUtilityForSampleDemo loginUtilityForSampleDemo;

        // ゲームサーバーにログインします
        if (loginUtilityForSampleDemo.PerformTotalLogin(&ngsFacade))
        {
            // NgsFacadeに定義されているRMCをゲームサーバーに発行します。
            CallNgsFacadeRmc(&ngsFacade);

            // RMC呼び出しの例としてランキングのRMCを呼び出します。
            CallRankingRmc(&ngsFacade);

            // ゲームサーバーからログアウトします
            nn::nex::ProtocolCallContext logoutCallContext;
            ngsFacade.Logout(&logoutCallContext);
            logoutCallContext.Wait();
        }

    }
    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    // インターネット接続を終了します。
    nex_demo_util::NetTerm();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("     Rmc sample\n");
    NEX_DEMO_LOG("*******************************\n\n");


    qInt returnCode = NEXSampleMain();


    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
