﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * AutoMmMessaging
 *
 * Description:
 *
 *  条件を指定して自動的にマッチメイクしてメッセージングによる通信を行うサンプルです。
 *  本サンプルは非同期的(コールバックを使用)にRMCを呼び出す形式で記述されています。
 *  同期的にオートマッチメイクしてメッセージングを行うAutoMmMessagingEasyサンプルも合わせて参照してください。
 * 
 *  最小・最大参加人数、ゲームモード、マッチメイクシステムタイプが一致する条件でオートマッチメイクを行い、マッチメイクセッションに参加します。
 *  その後、MessagingClientを使いお互いにメッセージをやりとりします。
 *  最後にマッチメイクセッションから退出します。
 *  マッチメイクセッションの参加者がいなくなったときにマッチメイクセッションは自動的に削除されます。
 *  
 *  MatchmakeExtensionClient::AutoMatchmake() と MatchmakeExtensionClient::EndParticipation() の呼び出しはシーケンス処理上逐次処理され、
 *  呼び出しが完了するまでステートが変わりません。
 *  一方 MatchmakeExtensionClient::CloseParticipation() についてはメッセージングによる通信が可能なステートならいつでも呼び出せるように配慮されています。
 *  この非同期処理中にエラー等が発生し AutoMatchmakeSample を破棄しなければならなくなった際は全ての未完了の非同期処理がキャンセルされます。
 *  このような成功を前提とする非同期APIの呼び出しを行う場合は処理の待ち合わせを行う必要はありませんが、コールバック関数が呼び出される
 *  都合上 AutoMatchmakeSample クラスが破棄される前に全ての非同期処理が完了もしくはキャンセルされている必要があることにご注意ください。
 *  
 *  サンプルに実装されているシーケンスは以下の通りです。
 * 
 * 1. NGSにログイン
 * 2. オートマッチメイクを行いマッチメイクセッションへ入室
 * 6. メッセージングで通信を行う
 * 7. マッチメイクセッションから退出
 * 8. NGSからログアウト
 * 
 * アプリケーションへの組み込み方の例:
 * 
 *  AutoMatchmakeGameLoop() は実際のゲームにおけるゲームループを模しています。この処理をゲームループに実装してください。
 *  nn::nex::Scheduler::DispatchAll() や各種計算処理はここに記載されている順に呼び出すことを推奨します。
 *  ゲームループから AutoMatchmakeSample クラスの関数を呼び出し、シーケンス処理を進めます。
 *  シーケンス処理は送受信処理とは分離されています。Communicate() を編集することにより任意の送受信処理を行えます。
 *  
 *  本サンプルではメッセージングによる通信を行っています。具体的な送受信処理については MessageSender.h を参照してください。
 *  
 *  シーケンス処理や通信処理中に発生するエラーは、本サンプルで使用しているAPIや実装しているシーケンスの範囲において全て適切に補足されるように実装されています。
 *  ただし、シーケンスを増やしたり使用するAPIを変えた場合は、それによりNEX内部で発生するエラーが変わったり追加のユーザー定義結果が必要になる場合があります。
 *  アプリケーションでは MessageSender::GetFirstError() と AutoMatchmakeSample::GetLastResult()
 *  で取得した最終的なエラーを画面に表示するなどしてプレイヤーに通知するか、アプリケーションで適切に処理してください。
 */

#include <nex.h>

#include <NexDemoUtil/NexUtil.h>
#include <NexDemoUtil/MessageSender.h>

namespace
{

// マッチメイクで使用するゲームモード
const nn::nex::qUnsignedInt32 MATCHMAKE_GAME_MODE = 1002;

// 参加人数
const nn::nex::qUnsignedInt16 MATCHMAKE_MAX_PARTICIPANTS = 3;

// メッセージング通信で送受信を行う時間(フレーム)
const nn::nex::qUnsignedInt32 COMMUNICATION_DURATION = 60 * 60;  // 60秒

// ユーザー定義の結果コード
enum UserDefinedResultCode
{
    USER_DEFINED_RESULT_CODE_SUCCESS                         =  0,  // 処理に成功し、エラーがない状態
    USER_DEFINED_RESULT_CODE_GAME_OVER                       = -1,  // ゲームオーバーが通知された (このケースでは正常終了します)
    USER_DEFINED_RESULT_CODE_NETWORK_DISCONNECTED            = -2,  // ネットワークの切断を検出
    USER_DEFINED_RESULT_CODE_MATCHMAKESESSION_UNREGISTERED   = -3,  // 意図せずマッチメイクセッションが削除された
    USER_DEFINED_RESULT_CODE_UNEXPECTED_HOST_CHANGE          = -4,  // 意図せずホストの変更が通知された
    USER_DEFINED_RESULT_CODE_UNEXPECTED_RMC_FAILURE          = -5   // 意図せずRMCが失敗した
};

/*!
    @brief オートマッチメイクを行うステートマシン。
 */
class AutoMatchmakeSample
{
public:
    /*!
        @brief コンストラクタ。

        @param[in] pNgsFacade ログイン済みのオブジェクトを指定してください。
     */
    AutoMatchmakeSample(nn::nex::NgsFacade* pNgsFacade)
        : m_state(STATE_AUTOMATCHMAKE)
        , m_finished(false)
        , m_errorOccurred(false)
        , m_unexpectedRmcFailureOccurred(false)
        , m_lastResult()
        , m_myPid(INVALID_PRINCIPALID)
        , m_ownerPid(INVALID_PRINCIPALID)
        , m_gid(INVALID_GATHERINGID)
        , m_pNgsFacade(pNgsFacade)
        , m_matchMakingClient()
        , m_notificationHandler(this)
        , m_joinedMatchmakeSession()
    {
        nn::nex::Credentials* pCred = m_pNgsFacade->GetCredentials();
        NEX_DEMO_ASSERT(pCred != NULL);
        m_myPid = pCred->GetPrincipalID();

        // 各Bind()処理はpCredがNULLでない場合は必ず成功します
        NEX_DEMO_VERIFY(m_matchMakingClient.Bind(pCred));

        // NGSからの通知をハンドリングするイベントハンドラを設定します。
        m_pNgsFacade->RegisterNotificationEventHandler(&m_notificationHandler);
    };

    /*!
        @brief デストラクタ。
     */
    virtual ~AutoMatchmakeSample()
    {
        // 処理が完了していない非同期処理を全てキャンセルする
        m_callContextStore.CancelAll();

        m_pNgsFacade->UnregisterNotificationEventHandler(&m_notificationHandler);
    };
private:
    enum State
    {
        STATE_AUTOMATCHMAKE,
        STATE_COMMUNICATE,
        STATE_ENDPARTICIPATION,
        STATE_RMC_RESULT_WAITING,
        STATE_FINISH
    };

    /*!
        @brief NGSから通知を処理するイベントハンドラ。
     */
    class MyNotificationEventHandler : public nn::nex::NotificationEventHandler
    {
    public:
        MyNotificationEventHandler(AutoMatchmakeSample* sample):m_pSample(sample){};
        virtual ~MyNotificationEventHandler(){};

        // NGSから通知を受け取った際にコールバックされる関数
        void ProcessNotificationEvent(const nn::nex::NotificationEvent& oEvent)
        {
            nn::nex::StringStream ss;
            ss << oEvent;
            NEX_DEMO_LOG("[Notified] " STRING_FORMAT "\n", ss.CStr());

            // MatchmakeExtensionClient::AutoMatchmake() などのRMCを呼び出し中に先に nn::nex::ParticipationEvents::Participate 通知などを受け取ることがあるため
            // いったんキューに保存し、RMC完了後に処理するようにします。
            EnqueueNotificationEvent(oEvent);
        }

        // 未処理の通知があるかどうかを返す
        qBool HasNotificationEvent() const { return !m_notificationQueue.empty(); }

        // 通知を一つ取り出して処理する
        void DequeueAndProcessNotificationEvent() { ProcessNotificationEventImpl(m_notificationQueue.front()); m_notificationQueue.pop_front(); }
    private:
        void EnqueueNotificationEvent(const nn::nex::NotificationEvent& oEvent) { m_notificationQueue.push_back(oEvent); }
        void ProcessNotificationEventImpl(const nn::nex::NotificationEvent& oEvent)
        {
            nn::nex::StringStream ss;
            ss << oEvent;
            NEX_DEMO_LOG("[Notified] " STRING_FORMAT "\n", ss.CStr());

            // 通知毎の処理
            // 必要があれば通知をいったんキューに入れ、都合の良いタイミングで取り出して処理する等行ってください
            switch (oEvent.GetType())
            {
            case nn::nex::NotificationEvents::ParticipationEvent:
                switch (oEvent.GetSubType())
                {
                case nn::nex::ParticipationEvents::Participate:
                    NEX_DEMO_LOG("  ParticipationEvents::Participate: [PID " NEX_PID_FORMAT "] join [gathering ID %u]\n",
                        static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()), static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                    break;
                case nn::nex::ParticipationEvents::EndParticipation:
                    NEX_DEMO_LOG("  ParticipationEvents::EndParticipation: [PID " NEX_PID_FORMAT "] end [gathering ID %u]\n",
                        static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()), static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                    break;
                case nn::nex::ParticipationEvents::Disconnect:
                    NEX_DEMO_LOG("  ParticipationEvents::Disconnect: [PID " NEX_PID_FORMAT "] disconnect [gathering ID %u]\n",
                        static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()), static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                    break;
                default:
                    NEX_DEMO_LOG("  ParticipationEvents: unsupported participation event\n");
                    break;
                }
                break;
            case nn::nex::NotificationEvents::OwnershipChangeEvent:
                NEX_DEMO_LOG("  NotificationEvents::OwnershipChangeEvent: [gathering ID %u] changed owner to [PID " NEX_PID_FORMAT "]\n",
                    static_cast<nn::nex::GatheringID>(oEvent.GetParam1()), static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()));
                m_pSample->SetOwnerPid(static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()));
                break;
            case nn::nex::NotificationEvents::GatheringUnregistered:
                NEX_DEMO_LOG("  NotificationEvents::GatheringUnregistered: [gathering ID %u] unregistered\n", static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                // 本サンプルでは自分が参加している限りマッチメイクセッションは削除されない。
            default:
                NEX_DEMO_LOG("  NotificationEvents: unsupported notification event\n");
                break;
            }
        }
    private:
        AutoMatchmakeSample* m_pSample;
        nn::nex::qList<nn::nex::NotificationEvent> m_notificationQueue; // 通知キュー
    };

public:
    qBool ProcessState();
    qBool CanCommunicate() const { return m_state == STATE_COMMUNICATE; }
    void FinishCommunication() { SetState(STATE_ENDPARTICIPATION); }
    nex_demo_util::ResultContainer GetLastResult() const { return m_lastResult; }
    void CloseParticipation();
    nn::nex::PrincipalID GetMyPid() const {return m_myPid;};
    nn::nex::PrincipalID GetOwnerPid() const {return m_ownerPid;};
    nn::nex::GatheringID GetJoinedMatchmakeSessionId() const {return m_joinedMatchmakeSession.GetID();}
private:
    void UnexpectedRmcFailureOccurred() { m_unexpectedRmcFailureOccurred = true; }
    nex_demo_util::ResultContainer CheckCondition();
    void ErrorOccurred(const nex_demo_util::ResultContainer& result);
    void SetState(State state){m_state = state;};
    State GetState() const {return m_state;};
    qBool CanProcessNotificationEvent() { return m_state != STATE_RMC_RESULT_WAITING; }
    void SetOwnerPid(nn::nex::PrincipalID pid){m_ownerPid = pid;};
    void SetGatheringID(nn::nex::GatheringID gid){m_gid = gid;};
    nn::nex::GatheringID GetGatheringID() const {return m_gid;};
    void AutoMatchmake();
    void OnCompletionAutoMatchmake(nn::nex::ProtocolCallContext* pCallContext);
    void EndParticipation();
    void OnCompletionEndParticipation(nn::nex::ProtocolCallContext* pCallContext);
    void OnCompletionCloseParticipation(nn::nex::ProtocolCallContext* pCallContext);

    State m_state;                                      // シーケンス処理の状態
    qBool m_finished;                                   // シーケンス処理が完了したか
    qBool m_errorOccurred;                              // エラーが発生した状態か
    qBool m_unexpectedRmcFailureOccurred;               // 予期しないRMC呼び出しの失敗があったか
    nex_demo_util::ResultContainer m_lastResult;        // シーケンス処理の最終結果
    nex_demo_util::CallContextStore m_callContextStore; // 非同期処理実行中のCallContextが保存されるコンテナです。キャンセル処理が必要なもののみ格納されます。
    nn::nex::PrincipalID m_myPid;                       // 自分のプリンシパルID
    nn::nex::PrincipalID m_ownerPid;                    // マッチメイクセッションのオーナーのプリンシパルID
    nn::nex::GatheringID m_gid;                         // 参加したギャザリングID
    nn::nex::NgsFacade* m_pNgsFacade;                   // ログイン済みのNgsFacadeオブジェクトへのポインタ
    nn::nex::MatchmakeExtensionClient m_matchMakingClient;      // マッチメイク関連のAPI呼び出しに使用するオブジェクト
    MyNotificationEventHandler m_notificationHandler;           // NGSからの通知をハンドリングするオブジェクト
    nn::nex::MatchmakeSession m_joinedMatchmakeSession;         // 参加したマッチメイクセッション
};

/*!
    @brief 接続状態をチェックする。STATE_COMMUNICATEのときのみ使用される。ユーザー定義エラーもしくはqResultを返す。

    @retval nex_demo_util::ResultContainer(USER_DEFINED_RESULT_CODE_SUCCESS) 正常。
    @retval それ以外 切断された場合など、対戦を続行できない状態。
 */
nex_demo_util::ResultContainer AutoMatchmakeSample::CheckCondition()
{
    // ゲームサーバから切断されていないか
    nn::nex::qResult ngsResult = m_pNgsFacade->IsConnected();
    if(!ngsResult)
    {
        NEX_DEMO_LOG("Disconnected from NGS.\n");
        return nex_demo_util::ResultContainer(ngsResult);
    }

    // 意図しない非同期処理の失敗があったかチェック
    if (m_unexpectedRmcFailureOccurred)
    {
        NEX_DEMO_LOG("Detected unexpected failure.\n");
        return nex_demo_util::ResultContainer(USER_DEFINED_RESULT_CODE_UNEXPECTED_RMC_FAILURE);
    }

    // ネットワークの接続性確認
    if(!nex_demo_util::IsNetworkConnected())
    {
        NEX_DEMO_LOG("Network connection unavailable.\n");
        return nex_demo_util::ResultContainer(USER_DEFINED_RESULT_CODE_NETWORK_DISCONNECTED);
    }
    return nex_demo_util::ResultContainer(USER_DEFINED_RESULT_CODE_SUCCESS);
}

/*!
    @brief オートマッチメイクを行う。 マッチメイクセッションに参加または作成する。
 */
void AutoMatchmakeSample::AutoMatchmake()
{
    NEX_DEMO_LOG("\nCall AutoMatchmake().\n");

    // 作成するマッチメイクセッションのパラメータをセットアップ
    nn::nex::MatchmakeSession matchmakeSession;

    // 締切状態の設定
    matchmakeSession.SetOpenParticipation(true);

    // 最小・最大参加人数
    matchmakeSession.SetMinParticipants(1);
    matchmakeSession.SetMaxParticipants(MATCHMAKE_MAX_PARTICIPANTS);

    // ゲームモード
    matchmakeSession.SetGameMode(MATCHMAKE_GAME_MODE);

    // マッチメイクシステムタイプ
    matchmakeSession.SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

    // オーナーがマッチメイクセッションから退出したときにオーナーの委譲を行うか
    matchmakeSession.SetFlag(nn::nex::GatheringFlags::MigrateOwner);

    // 他人がオーナー権限を奪えるか。MatchmakingClient::StaticUpdateSessionHostWithOwner() を使うため必要。
    matchmakeSession.SetFlag(nn::nex::GatheringFlags::ChangeOwnerByOtherHost);

    // 参加などの通知をマッチメイクセッションの参加者全員に行いたい場合はこのフラグをセットする
    // すでに参加しているユーザーの nn::nex::ParticipationEvents::Participate 通知を受け取りたい場合は
    // nn::nex::GatheringFlags::NotifyParticipationEventsToAllParticipantsReproducibly を利用することもできます。

    // オートマッチメイクのパラメータをセットアップ
    // AutoMatchmakeParam::SetSearchCriteria() や AutoMatchmakeParam::SetTargetGatheringId() を使用していないため、
    // AutoMatchmakeParam::SetSourceMatchmakeSession() で指定したパラメータから自動的に検索条件が作成されます。
    nn::nex::AutoMatchmakeParam autoMatchmakeParam;
    autoMatchmakeParam.SetSourceMatchmakeSession(matchmakeSession);
    autoMatchmakeParam.SetJoinMessage(NEX_T("auto matchmake session"));

    // RMC呼び出し時に完了コールバックが呼ばれる場合があるので先にステートを変更する
    SetState(STATE_RMC_RESULT_WAITING);

    // オートマッチメイクを行う
    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
    pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&AutoMatchmakeSample::OnCompletionAutoMatchmake), this));
    m_matchMakingClient.AutoMatchmake(pCallContext, autoMatchmakeParam, &m_joinedMatchmakeSession);
}

/*!
    @brief オートマッチメイクが完了した時に呼び出されるコールバック関数。

    @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
 */
void AutoMatchmakeSample::OnCompletionAutoMatchmake(nn::nex::ProtocolCallContext* pCallContext)
{
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("     AutoMatchmake() failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        ErrorOccurred(nex_demo_util::ResultContainer(result));
        return;
    }

    NEX_DEMO_LOG("     AutoMatchmake() succeeded.\n");
    SetGatheringID(m_joinedMatchmakeSession.GetID());
    SetOwnerPid(m_joinedMatchmakeSession.GetOwnerPID());
    NEX_DEMO_LOG("[PID Info >> JoinedGID:%u OwnerPID:" NEX_PID_FORMAT " MyPID:" NEX_PID_FORMAT "]\n",
                GetGatheringID(), 
                GetOwnerPid(),
                GetMyPid());

    SetState(STATE_COMMUNICATE);
}

/*!
    @brief マッチメイクセッション退出。
 */
void AutoMatchmakeSample::EndParticipation()
{
    if (GetGatheringID() == INVALID_GATHERINGID)
    {
        // 参加していないのでEndParticipation()は呼び出さない。
        SetState(STATE_FINISH);
        return;
    }
    NEX_DEMO_LOG("\nCall EndParticipation().\n");

    // RMC呼び出し時に完了コールバックが呼ばれる場合があるので先にステートを変更する
    SetState(STATE_RMC_RESULT_WAITING);
    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
    pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&AutoMatchmakeSample::OnCompletionEndParticipation), this));
    m_matchMakingClient.EndParticipation(pCallContext, GetGatheringID());
    SetGatheringID(INVALID_GATHERINGID);
}

/*!
    @brief マッチメイクセッション退出完了時に呼び出されるコールバック関数。

    @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
 */
void AutoMatchmakeSample::OnCompletionEndParticipation(nn::nex::ProtocolCallContext* pCallContext)
{
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (!result && !(result == QERROR(RendezVous, SessionVoid)))
    {
        // すでにサーバーから切断されているときなどは失敗する
        NEX_DEMO_LOG("     EndParticipation() failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
    }
    else
    {
        NEX_DEMO_LOG("     EndParticipation() succeeded.\n");
    }
    SetState(STATE_FINISH);
}

/*!
    @brief マッチメイクセッションの締め切り処理を行う。

    ステートの変化には依存しない形で呼び出しを行います。AutoMatchmakeSampleオブジェクトを破棄するときに処理中だった場合は自動的にキャンセルされます。
 */
void AutoMatchmakeSample::CloseParticipation()
{
    NEX_DEMO_LOG("\nCall CloseParticipation().\n");

    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
    pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&AutoMatchmakeSample::OnCompletionCloseParticipation), this));

    // あとで非同期処理をキャンセルできるようにするため、生成したCallContextを非同期処理実行中のみm_callContextStoreに保存するようにします。
    nex_demo_util::CallContextStoreAssociator::Register(&m_callContextStore, pCallContext);
    m_matchMakingClient.CloseParticipation(pCallContext, GetGatheringID());
}

/*!
    @brief マッチメイクセッション締め切り完了時に呼び出されるコールバック関数。

    @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
 */
void AutoMatchmakeSample::OnCompletionCloseParticipation(nn::nex::ProtocolCallContext* pCallContext)
{
    switch (pCallContext->GetState())
    {
    case nn::nex::CallContext::CallSuccess:
        NEX_DEMO_LOG("     CloseParticipation() succeeded.\n");
        break;
    case nn::nex::CallContext::CallCancelled:
        // キャンセルされるのは ~AutoMatchmakeSample() からのみなのでなにもしない
        NEX_DEMO_LOG("     CloseParticipation() cancelled.\n");
        break;
    case nn::nex::CallContext::CallFailure:
        NEX_DEMO_LOG("     CloseParticipation() failed.\n");
        nex_demo_util::ReportQResultReturnCode(pCallContext->GetOutcome());
        UnexpectedRmcFailureOccurred();
        break;
    default:
        NEX_DEMO_ASSERT(false);
        UnexpectedRmcFailureOccurred();
        break;
    }
}

/*!
    @brief 初回エラー発生時にエラーステートに遷移する。

    @param[in] result 発生したエラーが格納された結果。
 */
void AutoMatchmakeSample::ErrorOccurred(const nex_demo_util::ResultContainer& result)
{
    // すでにエラー状態ならなにもしない
    if (m_errorOccurred)
    {
        return;
    }

    // 処理が完了していない非同期処理を全てキャンセルする
    m_callContextStore.CancelAll();

    m_errorOccurred = true;
    m_lastResult = result;

    // マッチメイクセッション参加後
    if (m_joinedMatchmakeSession.GetID() != INVALID_GATHERINGID)
    {
        SetState(STATE_ENDPARTICIPATION);
    }
    // マッチメイクセッション未参加
    else
    {
        SetState(STATE_FINISH);
    }
}

/*!
    @brief ステートを遷移させる。
 */
qBool AutoMatchmakeSample::ProcessState()
{
    // 通知を処理する。
    // 非同期処理中は処理しないようにする。特に MatchmakeExtensionClient::AutoMatchmake() を実行中に通知されることがあるので、その間は処理しないようにする。
    if (CanProcessNotificationEvent())
    {
        while (m_notificationHandler.HasNotificationEvent())
        {
            m_notificationHandler.DequeueAndProcessNotificationEvent();
        }
    }

    // 通信状態などをチェックする
    nex_demo_util::ResultContainer result = CheckCondition();
    if (!result)
    {
        ErrorOccurred(result);
    }

    switch(GetState())
    {
    // オートマッチメイク
    case STATE_AUTOMATCHMAKE:
        AutoMatchmake();
        break;
    // メッセージングによる通信が可能なステート
    case STATE_COMMUNICATE:
        {
        }
        break;
    // マッチメイクセッション退出
    case STATE_ENDPARTICIPATION:
        EndParticipation();
        break;
    // RMC (AutoMatchmake(), EndParticipation()) 処理結果待ち
    case STATE_RMC_RESULT_WAITING:
        break;
    // 全ての処理が完了
    case STATE_FINISH:
        m_finished = true;
        break;
    }

    return m_finished;
}

/*!
    @brief メッセージングで送受信する。

    @param[in] sender 送受信を行うユーティリティオブジェクト。
    @param[in] sample オートマッチメイクで使用したオブジェクトを指定してください。
    @param[in] communicationFrame ゲームループを回るごとにインクリメントされる値。

    @retval true 送受信処理を終了する。
    @retval false 送受信処理を継続する。
 */
qBool Communicate(nex_demo_util::MessageSender& sender, const AutoMatchmakeSample& sample, nn::nex::qUnsignedInt32 communicationFrame)
{
    const nn::nex::String helloMessage = NEX_T("Hi everyone.");
    const nn::nex::String gameOverMessage = NEX_T("Game over.");
    qBool isGameOver = false;

    // 受信処理
    while (sender.HasReceivedMessage())
    {
        nn::nex::TextMessage message = sender.DequeueReceivedMessage();

        // もし挨拶を受けとったら送り主に返事する
        if (message.GetTextBody() == helloMessage)
        {
            nn::nex::qChar8 responseMessage[64];
            snprintf(responseMessage, sizeof(responseMessage) - 1, "Hello " NEX_PID_FORMAT, message.GetSenderID());
            sender.SendInstant(message.GetSenderID(), nn::nex::String(responseMessage));
        }
        // もしゲームオーバーを受け取ったら終了する
        else if (message.GetTextBody() == gameOverMessage)
        {
            isGameOver = true;
        }
    }

    // 送信処理
    // オーナーが自分を含む参加者全員にメッセージを送信する
    qBool isMatchOwner = sample.GetOwnerPid() == sample.GetMyPid();
    if (isMatchOwner && communicationFrame % 180 == 0)
    {
        if (communicationFrame >= COMMUNICATION_DURATION)
        {
            // 一定時間経っていればゲームオーバーを告知する
            sender.SendChat(sample.GetJoinedMatchmakeSessionId(), gameOverMessage);
        }
        else
        {
            // 一定時間経っていなければ挨拶を送る
            sender.SendChat(sample.GetJoinedMatchmakeSessionId(), NEX_T("Hi everyone."));
        }
    }
    return isGameOver;
}

/*!
    @brief オートマッチメイクゲームループ。

    @param[in] pNgsFacade ログイン済みのオブジェクトを指定してください。
 */
qBool AutoMatchmakeGameLoop(nn::nex::NgsFacade* pNgsFacade)
{
    AutoMatchmakeSample sample(pNgsFacade);
    nex_demo_util::MessageSender sender(pNgsFacade);
    nn::nex::qUnsignedInt32 communicationFrame = 0;
    nn::nex::qResult communicationResult(true);


    for(;;)
    {
        // ゲームフレームのスタート

        // このDispatchAllでは主にパケットの受信が行われる
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // ここでゲームのキー入力を行う
        // ここでゲームの計算処理を行う

        // ステートの遷移
        qBool finished = sample.ProcessState();

        // 終了確認
        if(finished)
        {
            break;
        }

        // マッチメイクセッションの参加者と通信が可能な状態での処理
        if (sample.CanCommunicate())
        {
            // 送受信処理を行う
            qBool isGameOver = Communicate(sender, sample, communicationFrame++);
            communicationResult = sender.GetFirstError();
            if (isGameOver || !communicationResult)
            {
                // ゲームオーバーもしくはsenderにエラーが発生していたら通信を終了する
                sample.FinishCommunication();
            }
        }

        // パケット処理の遅延を減らすため、1ゲームフレームでゲームの計算処理の前と後の二回 Scheduler::DispatchAll() を呼び出すのが望ましい
        // このDispatchAllでは主にパケットの送信が行われる
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // ここでゲームの描画処理を行う

        // ゲームでは適宜フレームの同期を行ってください
        nn::nex::Platform::Sleep(nex_demo_util::SLEEP_DURATION);
    }

    // エラー終了した場合はエラー内容を表示
    nex_demo_util::ResultContainer result = sample.GetLastResult();
    if (!result)
    {
        // 必要に応じてエラー・EULAアプレット/エラービューアでエラーメッセージを表示してください。
        result.ReportError("An error occurred in AutoMatchmakeSample.\n");
    }

    return true;
}

/*!
    @brief NEXの初期化後にNGSへログインし、マッチメイク用のゲームループに遷移します。
 */
qInt NEXSampleMain()
{

    // インターネットに接続します
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit failed.\n");
        return 1;
    }

    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // ログのレベルを設定します
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);
    {
        // ゲームサーバーへログインするためのクライアントを用意します。
        nn::nex::NgsFacade ngsFacade;
        nex_demo_util::LoginUtilityForSampleDemo loginUtilityForSampleDemo;

        // ゲームサーバーにログインします
        if (loginUtilityForSampleDemo.PerformTotalLogin(&ngsFacade))
        {
            // オートマッチメイクゲームループ
            if (!AutoMatchmakeGameLoop(&ngsFacade))
            {
                NEX_DEMO_LOG("AutoMatchmakeGameLoop finished with error.\n");
            }

            // ゲームサーバーからログアウトします
            nn::nex::ProtocolCallContext logoutCallContext;
            ngsFacade.Logout(&logoutCallContext);
            logoutCallContext.Wait();
        }
    }

    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    // インターネット接続を終了します。
    nex_demo_util::NetTerm();

    NEX_DEMO_LOG("Leaving application...\n");

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG(" AutoMmMessaging sample\n");
    NEX_DEMO_LOG("*******************************\n\n");


    qInt returnCode = NEXSampleMain();


    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
