﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <demo.h>

namespace nexdemo
{
class Console
{
public:
    // フォントサイズ
    static const u8 FONT_SIZE = 8;

public:
    /*!
        @brief コンストラクタです。
    */
    Console();

    /*!
        @brief デストラクタです。
    */
    virtual ~Console();

    /*!
        @brief 初期化処理を行います。

        @param[in]  x                   ウィンドウのX座標位置。
        @param[in]  y                   ウィンドウのY座標位置。
        @param[in]  windowCharsPerLine  ウィンドウの1行あたりの文字数。
        @param[in]  windowLines         ウィンドウの行数。
        @param[in]  bufferCharsPerLine  バッファの1行あたりの文字数。
        @param[in]  bufferLines         バッファの行数。
        @param[in]  lineSpace           行間。
    */
    void Initialize(s16 x, s16 y, u16 windowCharsPerLine, u16 windowLines, u32 bufferCharsPerLine, u32 bufferLines, u8 lineSpace = 2);

    /*!
        @brief 初期化処理を行います。

        @param[in]  x             ウィンドウのX座標位置。
        @param[in]  y             ウィンドウのY座標位置。
        @param[in]  charsPerLine  1行あたりの文字数。
        @param[in]  lines         行数。
        @param[in]  lineSpace     行間。
    */
    void Initialize(s16 x, s16 y, u16 charsPerLine, u16 lines, u8 lineSpace = 2)
    {
        Initialize(x, y, charsPerLine, lines, charsPerLine, lines, lineSpace);
    }

    /*!
        @brief 終了処理を行います。
    */
    void Finalize();

    /*!
        @brief コンソールの内容をクリアします。
    */
    void Clear();

    /*!
        @brief 文字列を出力します。

        @param[in]  pFormat  フォーマット文字列。
        @param[in]  ...      可変長引数。
    */
    void Printf(const char* pFormat, ...);

    /*!
        @brief 文字列を出力します。

        @param[in]  pString  文字列。
    */
    void PutString(const char* pString);

    /*!
        @brief コンソールの内容を描画します。

        @param[in]  pRenderSystem  レンダラ。
    */
    void Render(demo::RenderSystemDrawing* pRenderSystem);

    /*!
        @brief コンソールの内容をスクロールします。

        @param[in]  scrollX  X座標のスクロール量。
        @param[in]  scrollY  Y座標のスクロール量。
    */
    void Scroll(s16 scrollX, s16 scrollY);

    /*!
        @brief ウィンドウのX座標位置を取得します。
    */
    s16 GetX() const
    {
        return m_x;
    }

    /*!
        @brief ウィンドウのY座標位置を取得します。
    */
    s16 GetY() const
    {
        return m_y;
    }

    /*!
        @brief X座標のスクロール位置を取得します。

        @return X座標のスクロール位置。
    */
    s32 GetScrollX() const
    {
        return m_scrollX;
    }

    /*!
        @brief Y座標のスクロール位置を取得します。

        @return Y座標のスクロール位置。
    */
    s32 GetScrollY() const
    {
        return m_scrollY;
    }

    /*!
        @brief ウィンドウの1行あたりの文字数を取得します。

        @return ウィンドウの1行あたりの文字数。
    */
    u16 GetWindowCharsPerLine() const
    {
        return m_windowCharsPerLine;
    }

    /*!
        @brief ウィンドウの行数を取得します。

        @return ウィンドウの行数。
    */
    u16 GetWindowLines() const
    {
        return m_windowLines;
    }

    /*!
        @brief 画面バッファの1行あたりの文字数を取得します。

        @return 画面バッファの1行あたりの文字数。
    */
    u32 GetBufferCharsPerLine() const
    {
        return m_bufferCharsPerLine;
    }

    /*!
        @brief 画面バッファの行数を取得します。

        @return 画面バッファの行数。
    */
    u32 GetBufferLines() const
    {
        return m_bufferLines;
    }

    /*!
        @brief 1番多く使用された画面バッファの1行あたりの文字数を取得します。

        @return 1番多く使用された画面バッファの1行あたりの文字数。
    */
    u32 GetUsedBufferCharsPerLine() const
    {
        return m_usedBufferCharsPerLine;
    }

    /*!
        @brief 使用された画面バッファの行数を取得します。

        @return 使用された画面バッファの行数。
    */
    u32 GetUsedBufferLines() const
    {
        return m_usedBufferLines;
    }

    /*!
        @brief 1行の高さを取得します。

        @return 1行の高さ。
    */
    u16 GetLineHeight() const
    {
        return FONT_SIZE + m_lineSpace;
    }

    /*!
        @brief 上下の余白サイズを取得します。

        @brief 余白サイズ。
    */
    u8 GetMargin() const
    {
        return m_lineSpace / 2;
    }

    /*!
        @brief 初期化済みかどうかを取得します。

        @return 初期化済みなら true。
    */
    bool IsInitialized() const
    {
        return m_isInitialized;
    }

    /*!
        @brief X座標のスクロール位置を設定します。

        @return X座標のスクロール位置。
    */
    void SetScrollX(s32 scrollX);

    /*!
        @brief Y座標のスクロール位置を設定します。

        @return Y座標のスクロール位置。
    */
    void SetScrollY(s32 scrollY);

protected:
    // 文字をバッファに追加する。
    void PutChar(char c);

protected:
    // バッファ
    char** m_ppBuffer;
    char*  m_pRenderBuffer;
    // 座標
    u16 m_x;
    u16 m_y;
    // ウィンドウ
    u16 m_windowCharsPerLine;
    u16 m_windowLines;
    // 画面バッファ
    u16 m_bufferCharsPerLine;
    u16 m_bufferLines;
    // 使用された画面バッファ
    u16 m_usedBufferCharsPerLine;
    u16 m_usedBufferLines;
    // 行間
    u8 m_lineSpace;
    // カーソル位置
    u32 m_cursorX;
    u32 m_cursorY;
    // スクロール位置
    s32 m_scrollX;
    s32 m_scrollY;
    // フラグ
    bool m_isInitialized;
    bool m_isOver;
    bool m_isFull;
};
}

