﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

/*
 * NEX Sample
 *
 * PersistentGathering
 *
 * Description:
 *
 *  永続ギャザリングの使用方法についてのサンプルです。
 *  永続ギャザリングの検索・参加・作成と永続ギャザリングマッチの実行など、基本的な処理を行います。
 *
 *  本サンプルデモは以下の処理を行います。
 *
 *  1. 永続ギャザリングコードについてのデモを行う。
 *  2. NGSにログインする。
 *  3. 自分が参加している永続ギャザリングと、指定したPIDのユーザが参加している永続ギャザリングを検索する。
 *  4. 検索結果から一つの永続ギャザリングを選び、その永続ギャザリングに参加していない場合は参加する。
 *  5. 2. で永続ギャザリングが見つからなかった場合は、自分で永続ギャザリングを作成する。
 *  6. 3 or 4 で選択した永続ギャザリングの永続ギャザリングマッチを行う。(実際にP2P通信は行いません)
 *  7. ログアウトする。
 */

#include <nex.h>
#include <NexDemoUtil/NexUtil.h>

namespace
{

// 特定のユーザと永続ギャザリングマッチを実行する場合はここでPIDを設定してください
const nn::nex::PrincipalID TARGET_PID = 1001;

// アプリケーション固有のゲームモード
const qUnsignedInt32 MATCHMAKE_GAME_MODE = 1004;

/*!
    @brief NGSから通知を処理するイベントハンドラ。
 */
class MyNotificationEventHandler : public nn::nex::NotificationEventHandler
{
public:
    void ProcessNotificationEvent(const nn::nex::NotificationEvent &oEvent)
    {
        nn::nex::StringStream ss;
        ss << oEvent;
        NEX_DEMO_LOG("Notification " STRING_FORMAT "\n\n",ss.CStr());

        // メッセージ毎の処理
        switch (oEvent.GetType())
        {
        case nn::nex::NotificationEvents::ParticipationEvent:
            switch (oEvent.GetSubType())
            {
            case nn::nex::ParticipationEvents::Participate:
                NEX_DEMO_LOG("PARTICIPATION_EVENT PARTICIPATE: [PID " NEX_PID_FORMAT "] join [session %u]\n",
                    static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()), static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                break;

            case nn::nex::ParticipationEvents::EndParticipation:
                NEX_DEMO_LOG("PARTICIPATION_EVENT END: [PID " NEX_PID_FORMAT "] end [session %u]\n",
                    static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()), static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                break;

            case nn::nex::ParticipationEvents::Disconnect:
                NEX_DEMO_LOG("PARTICIPATION_EVENT DISCONNECT: [PID " NEX_PID_FORMAT "] disconnect [session %u]\n",
                    static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()), static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                break;

            default:
                NEX_DEMO_LOG("PARTICIPATION_EVENT: unsupported participation event\n");
                break;
            }
            break;

        case nn::nex::NotificationEvents::OwnershipChangeEvent:
            NEX_DEMO_LOG("NOTIFICATION_EVENT OWNERSHIPCHANGE: [session %u] changed owner to [PID " NEX_PID_FORMAT "]\n",
                static_cast<nn::nex::GatheringID>(oEvent.GetParam1()), static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()));
            break;

        case nn::nex::NotificationEvents::GatheringUnregistered:
            NEX_DEMO_LOG("NOTIFICATION_EVENT GATHERINGUNREGISTERED: [session %u] unregistered\n", static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));

            break;

        default:
            NEX_DEMO_LOG("NOTIFICATION_EVENT: unsupported notification event\n");
            break;
        }
    }
};

/*!
    @brief 永続ギャザリングのリストを表示。

    @param[in] persistentGatheringList 表示するオブジェクトのリスト。
 */
void PrintPersistentGatheringList(const nn::nex::qList<nn::nex::PersistentGathering>& persistentGatheringList)
{
    if(persistentGatheringList.empty())
    {
        NEX_DEMO_LOG("PersistentGatheringList is empty.\n");
    }

    nn::nex::qList<nn::nex::PersistentGathering>::const_iterator it = persistentGatheringList.begin();
    while(it != persistentGatheringList.end())
    {
        NEX_DEMO_LOG("-------------------------------------------\n");
        NEX_DEMO_LOG("GatheringID : %u\n", it->GetID());
        NEX_DEMO_LOG("  PersistentGatheringCode : %llu\n", nn::nex::MatchmakeExtensionClient::GatheringIdToPersistentGatheringCode(it->GetID()));
        NEX_DEMO_LOG("  PersistentGatheringType : %u\n", it->GetPersistentGatheringType());
        NEX_DEMO_LOG("  Owner : " NEX_PID_FORMAT "\n", it->GetOwnerPID());
        NEX_DEMO_LOG("  MaxParticipants : %u\n", it->GetMaxParticipants());
        NEX_DEMO_LOG("  Attribute_0 : %u\n", it->GetAttribute(0));
        NEX_DEMO_LOG("  Attribute_1 : %u\n", it->GetAttribute(1));
        NEX_DEMO_LOG("  Attribute_2 : %u\n", it->GetAttribute(2));
        NEX_DEMO_LOG("  Attribute_3 : %u\n", it->GetAttribute(3));
        NEX_DEMO_LOG("  Attribute_4 : %u\n", it->GetAttribute(4));
        NEX_DEMO_LOG("  Attribute_5 : %u\n", it->GetAttribute(5));
        NEX_DEMO_LOG("  Participation Count : %u\n", it->GetParticipationCount());
        NEX_DEMO_LOG("  MatchmakeSession Count : %u\n", it->GetMatchmakeSessionCount());
        NEX_DEMO_LOG("-------------------------------------------\n");

        ++it;
    }
}

/*!
    @brief 永続ギャザリングを参加者で検索する。

    @param[in] pMatchmakeClient Credentials をバインド済みのオブジェクトを指定してください。
    @param[in] principalId 検索する参加者。
    @param[out] pPersistentGatheringList 見つかった永続ギャザリングを格納するポインタ。

    @return 処理に成功したかどうか。
 */
qBool DoFindPersistentGatheringByParticipant(nn::nex::MatchmakeExtensionClient* pMatchmakeClient,
                                 nn::nex::PrincipalID principalId,
                                 nn::nex::qList<nn::nex::PersistentGathering>* pPersistentGatheringList)
{
    nn::nex::ProtocolCallContext oContext;

    NEX_DEMO_LOG("FindPersistentGatheringByParticipant pid = " NEX_PID_FORMAT "\n", principalId);
    pMatchmakeClient->FindPersistentGatheringByParticipant(&oContext,
            principalId,
            nn::nex::ResultRange(),
            pPersistentGatheringList);
    oContext.Wait();
    if(!oContext.GetOutcome())
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to FindPersistentGatheringByParticipant().\n");
        return false;
    }

    PrintPersistentGatheringList(*pPersistentGatheringList);
    return true;
}

/*!
    @brief 永続ギャザリングに参加する。

    @param[in] pMatchmakeClient Credentials をバインド済みのオブジェクトを指定してください。
    @param[in] persistentGatheringId 参加する永続ギャザリングのギャザリングID。

    @return 処理に成功したかどうか。
 */
qBool DoJoinPersistentGathering(nn::nex::MatchmakeExtensionClient* pMatchmakeClient, nn::nex::GatheringID persistentGatheringId)
{
    nn::nex::ProtocolCallContext oContext;

    NEX_DEMO_LOG("Join PersistentGathering %u\n", persistentGatheringId);
    pMatchmakeClient->JoinPersistentGathering(&oContext, persistentGatheringId);
    oContext.Wait();
    if(!oContext.GetOutcome())
    {
        // アプリでハンドリングすべきエラーについては個別に対応(アプリ定義のエラーメッセージを表示するなど)
        // 本サンプルでは1度でも参加に失敗した段階で終了させる
        if(oContext.GetOutcome() == QERROR(RendezVous, SessionVoid))
        {
            NEX_DEMO_LOG("PersistentGathering %u is already unregistered.\n", persistentGatheringId);
        }
        else if(oContext.GetOutcome() == QERROR(RendezVous, SessionFull))
        {
            NEX_DEMO_LOG("PersistentGathering %u is already full.\n", persistentGatheringId);
        }
        else if(oContext.GetOutcome() == QERROR(RendezVous, PersistentGatheringParticipationMax))
        {
            NEX_DEMO_LOG("You exceed persistentGathering participation max\n");
        }
        else if(oContext.GetOutcome() == QERROR(RendezVous, InvalidPassword))
        {
            NEX_DEMO_LOG("Bad persistentGathering password\n");
        }
        else if(oContext.GetOutcome() == QERROR(RendezVous, WithoutParticipationPeriod))
        {
            NEX_DEMO_LOG("This persistentGathering is without participation period.\n");
        }
        else
        {
            nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to JoinPersistentGathering().\n");
        }
        return false;
    }

    NEX_DEMO_LOG("Success JoinPersistentGathering %u\n", persistentGatheringId);
    return true;
}

/*!
    @brief 永続ギャザリングを作成して参加する。

    @param[in] pMatchmakeClient Credentials をバインド済みのオブジェクトを指定してください。
    @param[out] pPersistentGatheringGatheringId 作成した永続ギャザリングのギャザリングID。

    @return 処理に成功したかどうか。
 */
qBool DoCreatePersistentGathering(nn::nex::MatchmakeExtensionClient* pMatchmakeClient, nn::nex::GatheringID* pPersistentGatheringGatheringId)
{
    nn::nex::ProtocolCallContext oContext;

    // 永続ギャザリングのパラメータ設定
    nn::nex::PersistentGathering persistentGathering;

    // オープン永続ギャザリング
    persistentGathering.SetPersistentGatheringType(nn::nex::PersistentGatheringType::Open);

    // 最大参加人数 0 をセットすることで無制限にできます
    persistentGathering.SetMaxParticipants(0);

    // 説明文字列
    persistentGathering.SetDescription(NEX_T("this is test persistentGathering."));

    // 属性値。テストとしてランダムな値をセットする
    persistentGathering.SetAttribute(0, nn::nex::Platform::GetRandomNumber(1000));

    // 参加可能になる日時(UTC時刻) nn::nex::DateTime::Never を指定することで無制限になる
    persistentGathering.SetParticipationStartDate(nn::nex::DateTime::Never);

    // 参加出来なくなる日時(UTC時刻) nn::nex::DateTime::Never を指定することで無制限になる
    persistentGathering.SetParticipationEndDate(nn::nex::DateTime::Never);

    pMatchmakeClient->CreatePersistentGathering(&oContext, persistentGathering, pPersistentGatheringGatheringId);
    oContext.Wait();
    if(!oContext.GetOutcome())
    {
        // アプリでハンドリングすべきエラーについては個別に対応(アプリ定義のエラーメッセージを表示するなど)
        // 本サンプルでは1度でも参加に失敗した段階で終了させる
        if(oContext.GetOutcome() == QERROR(RendezVous, PersistentGatheringParticipationMax))
        {
            NEX_DEMO_LOG("You exceed persistentGathering participation max\n");
        }
        else if(oContext.GetOutcome() == QERROR(RendezVous, PersistentGatheringCreationMax))
        {
            NEX_DEMO_LOG("You exceed persistentGathering creation max\n");
        }
        else
        {
            nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to CreatePersistentGathering().\n");
        }
        return false;
    }

    NEX_DEMO_LOG("Success CreatePersistentGathering %u\n", *pPersistentGatheringGatheringId);
    return true;
}

/*!
    @brief 引数で与えられた永続ギャザリングの永続ギャザリングマッチを開催する。

    @param[in] pMatchmakeClient Credentials をバインド済みのオブジェクトを指定してください。
    @param[in] persistentGatheringId 永続ギャザリングマッチを開催する永続ギャザリングのギャザリングID。
 */
void RunPersistentGatheringMatch(nn::nex::MatchmakeExtensionClient* pMatchmakeClient, nn::nex::GatheringID persistentGatheringId)
{
    NEX_DEMO_LOG("Run PersistentGatheringMatch for PersistentGathering %u\n", persistentGatheringId);
    nn::nex::MatchmakeSession joinedMatchmakeSession;

    // オートマッチメイクで永続ギャザリングのギャザリングIDを指定してマッチメイクセッションに入る。
    {
        nn::nex::ProtocolCallContext oContext;
        nn::nex::MatchmakeSession matchmakeSession;
        matchmakeSession.SetGameMode(MATCHMAKE_GAME_MODE);

        // 永続ギャザリングマッチなので MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING を指定する
        matchmakeSession.SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING);

        // 0番目の属性値に永続ギャザリングのギャザリングIDを指定する
        matchmakeSession.SetAttribute(0, persistentGatheringId);

        // 同様に検索条件を設定
        nn::nex::MatchmakeSessionSearchCriteria criteria;
        criteria.SetGameMode(matchmakeSession.GetGameMode());
        criteria.SetMatchmakeSystemType(matchmakeSession.GetMatchmakeSystemType());
        criteria.SetAttribute(0, matchmakeSession.GetAttribute(0));

        nn::nex::qList<nn::nex::MatchmakeSessionSearchCriteria> criteriaList;
        criteriaList.push_back(criteria);

        nn::nex::AutoMatchmakeParam autoMatchmakeParam;
        autoMatchmakeParam.SetSourceMatchmakeSession(matchmakeSession);
        autoMatchmakeParam.SetSearchCriteria(criteriaList);

        pMatchmakeClient->AutoMatchmake(&oContext, autoMatchmakeParam, &joinedMatchmakeSession);
        oContext.Wait();
        if(!oContext.GetOutcome())
        {
            nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to AutoMatchmake().\n");
            return;
        }
        NEX_DEMO_LOG("Success AutoMatchmake id:%u owner:" NEX_PID_FORMAT "\n", joinedMatchmakeSession.GetID(), joinedMatchmakeSession.GetOwnerPID());
    }

    // 60秒程度待ってから退出する。
    NEX_DEMO_LOG("Wait 60 seconds.\n");
    nn::nex::Time tiWaitStart = nn::nex::SystemClock::GetTime();
    while(true)
    {
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        nn::nex::Time currentTime = nn::nex::SystemClock::GetTime();
        if(currentTime - tiWaitStart > 60 * 1000)
        {
            break;
        }
        nn::nex::Platform::Sleep(nex_demo_util::SLEEP_DURATION);
    }

    // マッチメイクセッションから退出する
    {
        nn::nex::ProtocolCallContext oContext;
        pMatchmakeClient->EndParticipation(&oContext, joinedMatchmakeSession.GetID());
        oContext.Wait();
        if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
        {
            if(oContext.GetOutcome() == QERROR(RendezVous, SessionVoid))
            {
                NEX_DEMO_LOG("Already unregisterd. Don't care.");
                return;
            }

            nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to EndParticipation\n");
            return;
        }
    }
}

/*!
    @brief 永続ギャザリングへ参加し、さらに永続ギャザリングを使用したマッチメイクを行う。

    @param[in] pNgsFacade ログイン済みのオブジェクトを指定してください。
 */
void JoinPersistentGatheringAndDoPersistentGatheringMatch(nn::nex::NgsFacade* pNgsFacade)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::qList<nn::nex::PersistentGathering> myPersistentGatheringList;
    nn::nex::qList<nn::nex::PersistentGathering> targetPersistentGatheringList;
    nn::nex::MatchmakeExtensionClient matchmakeClient;
    matchmakeClient.Bind(pNgsFacade->GetCredentials());

    nn::nex::GatheringID persistentGatheringId = INVALID_GATHERINGID;
    qUnsignedInt32 maxMatchmakeSessionCount = 0;

    // 自分が参加している永続ギャザリングと、検索対象のユーザが参加している永続ギャザリングの中から、
    // 永続ギャザリングマッチが最も多く開催されている永続ギャザリングを選ぶ

    // 自分が参加している永続ギャザリングを取得
    if(!DoFindPersistentGatheringByParticipant(&matchmakeClient,
                                   matchmakeClient.GetCredentials()->GetPrincipalID(),
                                   &myPersistentGatheringList))
    {
        return;
    }

    if(!myPersistentGatheringList.empty())
    {
        persistentGatheringId = myPersistentGatheringList.front().GetID();
        maxMatchmakeSessionCount = myPersistentGatheringList.front().GetMatchmakeSessionCount();

        nn::nex::qList<nn::nex::PersistentGathering>::const_iterator it = myPersistentGatheringList.begin();
        while(it != myPersistentGatheringList.end())
        {
            if(it->GetMatchmakeSessionCount() > maxMatchmakeSessionCount)
            {
                maxMatchmakeSessionCount = it->GetMatchmakeSessionCount();
                persistentGatheringId = it->GetID();
            }
            ++it;
        }
    }

    // 指定したユーザが参加している永続ギャザリングを検索
    if(!DoFindPersistentGatheringByParticipant(&matchmakeClient,
                                   TARGET_PID,
                                   &targetPersistentGatheringList))
    {
        return;
    }

    if(!targetPersistentGatheringList.empty())
    {
        // 自分が永続ギャザリングに参加していない場合
        if(persistentGatheringId == INVALID_GATHERINGID)
        {
            persistentGatheringId = targetPersistentGatheringList.front().GetID();
            maxMatchmakeSessionCount = targetPersistentGatheringList.front().GetMatchmakeSessionCount();
        }

        nn::nex::qList<nn::nex::PersistentGathering>::const_iterator it = targetPersistentGatheringList.begin();
        while(it != targetPersistentGatheringList.end())
        {
            if(it->GetMatchmakeSessionCount() > maxMatchmakeSessionCount)
            {
                maxMatchmakeSessionCount = it->GetMatchmakeSessionCount();
                persistentGatheringId = it->GetID();
            }
            ++it;
        }
    }

    // 自分が参加していない永続ギャザリングであれば参加
    if(persistentGatheringId != INVALID_GATHERINGID)
    {
        qBool isJoined = false;
        nn::nex::qList<nn::nex::PersistentGathering>::const_iterator it = myPersistentGatheringList.begin();
        while(it != myPersistentGatheringList.end())
        {
            if(it->GetID() == persistentGatheringId)
            {
                isJoined = true;
            }
            ++it;
        }
        if(!isJoined)
        {
            if(!DoJoinPersistentGathering(&matchmakeClient, persistentGatheringId))
            {
                return;
            }
        }
    }

    // 参加可能な永続ギャザリングが無ければ自分で作成する
    if(persistentGatheringId == INVALID_GATHERINGID)
    {
        if(!DoCreatePersistentGathering(&matchmakeClient, &persistentGatheringId))
        {
            return;
        }
    }

    // 永続ギャザリングマッチを開催する
    RunPersistentGatheringMatch(&matchmakeClient, persistentGatheringId);
}

/*!
    @brief GatheringCodeを生成するサンプルです。
 */
void DoPersistentGatheringCodeSample()
{
    // 永続ギャザリングコード生成のための初期化を行う。
    // SetPersistentGatheringCodeKey() には自タイトルのアクセスキーを指定してください。
    nn::nex::MatchmakeExtensionClient::SetPersistentGatheringCodeKey(nex_demo_util::NGS_ACCESS_KEY);

    // 永続ギャザリングコードの変換は MatchmakeExtensionClient::Bind() を実行しなくても良い
    {
        const nn::nex::GatheringID gatheringId = 12345;
        qUnsignedInt64 persistentGatheringCode = nn::nex::MatchmakeExtensionClient::GatheringIdToPersistentGatheringCode(gatheringId);

        NEX_DEMO_LOG("GateringID:%u PersistentGatheringCode:%llu\n", gatheringId, persistentGatheringCode);

        NEX_DEMO_ASSERT(nn::nex::MatchmakeExtensionClient::IsValidPersistentGatheringCode(persistentGatheringCode));

        NEX_DEMO_ASSERT(gatheringId == nn::nex::MatchmakeExtensionClient::PersistentGatheringCodeToGatheringId(persistentGatheringCode));

        // Release ビルドでの警告を回避する。
        (void)persistentGatheringCode;
    }

    // ライブラリの終了時など、不要になったタイミングで ClearPersistentGatheringCodeKey() を呼びだしてクリアする。
    nn::nex::MatchmakeExtensionClient::ClearPersistentGatheringCodeKey();
}

/*!
    @brief NEXの初期化後にNGSへログインし、永続ギャザリングに関するAPIを実行します。
 */
qInt NEXSampleMain()
{
    // インターネットに接続します
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit failed.\n");
        return 1;
    }

    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // ログレベルの設定(設定を行うかどうかは任意です)
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);

    {
        // 永続ギャザリングコードについてのデモを実行する。
        DoPersistentGatheringCodeSample();

        // ゲームサーバーへログインするためのクライアントを用意します。
        nn::nex::NgsFacade ngsFacade;
        nex_demo_util::LoginUtilityForSampleDemo loginUtilityForSampleDemo;

        // NGSからの通知を処理するイベントハンドラを登録
        MyNotificationEventHandler myNotificationEventHandler;
        ngsFacade.RegisterNotificationEventHandler(&myNotificationEventHandler);

        // ゲームサーバーにログインします
        if (loginUtilityForSampleDemo.PerformTotalLogin(&ngsFacade))
        {
            // 永続ギャザリングへ参加したり、永続ギャザリングマッチを行う。
            JoinPersistentGatheringAndDoPersistentGatheringMatch(&ngsFacade);

            // ゲームサーバーからログアウトします
            nn::nex::ProtocolCallContext logoutCallContext;
            ngsFacade.Logout(&logoutCallContext);
            logoutCallContext.Wait();
        }

        // イベントハンドラを削除
        ngsFacade.UnregisterNotificationEventHandler(&myNotificationEventHandler);
    }
    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    // インターネット接続を終了します。
    nex_demo_util::NetTerm();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("   PersistentGathering sample\n");
    NEX_DEMO_LOG("*******************************\n\n");


    qInt returnCode = NEXSampleMain();


    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
