﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

/*
 * NEX Sample
 *
 * Ranking2
 *
 * Description:
 *
 *  Ranking2 のサンプルです。
 *  サンプルに実装されているシーケンスは以下の通りです。
 *
 * 1. NGSにログイン
 * 2. Ranking2 の一連の処理を実行 (詳細はRanking2Sampleクラスを参照)
 * 3. NGSからログアウト
 *
 */

#include <algorithm>
#include <functional>

#include <nex.h>
#include <NexDemoUtil/NexUtil.h>

    

namespace
{

/*!
    @brief Ranking2Info の内容をデバッグログに表示する。

    @param[in] rankingInfo 表示するオブジェクト。
 */
void PrintRanking(const nn::nex::Ranking2Info& rankingInfo)
{
    NEX_DEMO_LOG("PrintRanking()\n  numRankedIn: %u\n", rankingInfo.GetNumRankedIn());
    NEX_DEMO_LOG("  lowestRank: %u\n", rankingInfo.GetLowestRank());
    NEX_DEMO_LOG("  season: %u\n", rankingInfo.GetSeason());

    nn::nex::qVector<nn::nex::Ranking2RankData> rankDataList = rankingInfo.GetRankDataList();
    nn::nex::qVector<nn::nex::Ranking2RankData>::iterator it;
    
#if defined(CAFE_TARGET)
    FFLStoreData storeData;
#elif defined(CTR_TARGET)
    nn::mii::StoreData storeData;
#endif
    
    for (it = rankDataList.begin(); it != rankDataList.end(); ++it)
    {
        NEX_DEMO_LOG("  Rank: %u\n", it->GetRank());
        NEX_DEMO_LOG("    Score:       %u\n", it->GetScore());
        NEX_DEMO_LOG("    PrincipalId: " NEX_PID_FORMAT "\n", it->GetPrincipalId());
        NEX_DEMO_LOG("    NexUniqueId: %llu\n", it->GetNexUniqueId());
        NEX_DEMO_LOG("    UserName:    " STRING_FORMAT "\n", it->GetCommonData().GetUserName().CStr());


        // Miiデータの表示
#if defined(CAFE_TARGET) || defined(CTR_TARGET)
        if (!it->GetCommonData().GetMii(&storeData))
        {
            NEX_DEMO_LOG("    Mii:         not available\n");
            continue;
        }
        qUnsignedInt32 size = sizeof(storeData.data);
        NEX_DEMO_LOG("    Mii (%u bytes):", size);
        for (qUnsignedInt32 i = 0; i < size; i++)
        {
            // 32byteごとに改行する
            if (i != 0 && i % 32 == 0)
            {
                NEX_DEMO_LOG("\n                   ");
            }
            NEX_DEMO_LOG("%02X ", storeData.data[i]);
        }
        NEX_DEMO_LOG("\n");
#endif

    }
}

/*!
    @brief Ranking2ChartInfo の内容をデバッグログに表示する。

    @param[in] chartInfo 表示するオブジェクト。
 */
void PrintRankingChart(const nn::nex::Ranking2ChartInfo& chartInfo)
{
    const qUnsignedInt32 MAX_BAR_NUM = 80;

    NEX_DEMO_LOG("RankChart ");
    NEX_DEMO_LOG(" Category:%u", chartInfo.GetCategory());
    NEX_DEMO_LOG(" Season:%d", chartInfo.GetSeason());
    NEX_DEMO_LOG("\n");
    if (!chartInfo.IsValid())
    {
        NEX_DEMO_LOG("Not Found Chart\n");
        return;
    }
    qUnsignedInt32 maxQuantity = *std::max_element(chartInfo.GetQuantities().begin(), chartInfo.GetQuantities().end());
    for (qInt i = 0; i < static_cast<qInt>(chartInfo.GetQuantities().size()); i++)
    {
        if (chartInfo.GetScoreOrder() == nn::nex::Ranking2Constants::SCORE_ORDER_DESC)
        {
            NEX_DEMO_LOG(" [%10u,%10u):", chartInfo.GetHighestBinsScore() - (i + 1)*chartInfo.GetBinsWidth(), chartInfo.GetHighestBinsScore() - i * chartInfo.GetBinsWidth());
        }
        else
        {
            NEX_DEMO_LOG(" (%10u,%10u]:", chartInfo.GetHighestBinsScore() + i * chartInfo.GetBinsWidth(), chartInfo.GetHighestBinsScore() + (i + 1) * chartInfo.GetBinsWidth());
        }

        if (chartInfo.GetQuantities()[i] > 0) // 一つでも存在したら点を一個打つ
        {
            NEX_DEMO_LOG("*");
        }
        for (qUnsignedInt32 j = 0; j < chartInfo.GetQuantities()[i] * (MAX_BAR_NUM - 1) / maxQuantity; j++)
        {
            NEX_DEMO_LOG("*");
        }
        NEX_DEMO_LOG("\n");
    }
}

/*!
    @brief Ranking2 の各種APIを実行するクラス
 */
class Ranking2Sample
{
public:
    Ranking2Sample(nn::nex::NgsFacade* pNgsFacade)
        : m_pNgsFacade(pNgsFacade)
        , m_ranking2Client()
    {
        nn::nex::Credentials* pCred = m_pNgsFacade->GetCredentials();
        NEX_DEMO_ASSERT(pCred != NULL);

        // Bind()処理はpCredがNULLでない場合は必ず成功する
        NEX_DEMO_VERIFY(m_ranking2Client.Bind(pCred));
    }

    void CallRanking2Apis();
private:
    static const qUnsignedInt32 MU = 2500; // 正規分布の平均
    static const qUnsignedInt32 SIGMA = 1250; // 正規分布の標準偏差
    static const qUnsignedInt32 CATEGORY = 1; // スコアを登録、取得するカテゴリ
    qBool PutCommonData();
    qBool PutScore();
    qBool GetRanking();
    qBool GetRankingChart();

    qUnsignedInt32 GetRandomScore() {return nex_demo_util::GetRandNormal(MU, SIGMA);}

    nn::nex::NgsFacade* m_pNgsFacade;                       // ログイン済みのNgsFacadeオブジェクトへのポインタ
    nn::nex::Ranking2Client m_ranking2Client;               // Ranking2 関連のAPI呼び出しに使用するオブジェクト
};

/*!
    @brief ユーザー共通データを登録する。

    @return 処理が成功したかどうか。
 */
qBool Ranking2Sample::PutCommonData()
{
    nn::nex::Ranking2CommonData commonData;
    nn::nex::String userName = NEX_T("TestUser");
    commonData.SetUserName(userName);
    NEX_DEMO_LOG("UserName:    " STRING_FORMAT " (length=" NEX_SIZE_T_FORMAT ")\n",
        commonData.GetUserName().CStr(), commonData.GetUserName().GetLength());
    commonData.Trace();

    nn::nex::ProtocolCallContext oContext;
    m_ranking2Client.PutCommonData(&oContext, commonData);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to PutCommonData()\n");
        return false;
    }
    return true;
}

/*!
    @brief スコアを登録する。

    @return 処理が成功したかどうか。
 */
qBool Ranking2Sample::PutScore()
{
    nn::nex::Ranking2ScoreData scoreData;
    scoreData.SetCategory(CATEGORY);
    scoreData.SetScore(GetRandomScore());
    NEX_DEMO_LOG("PutScore(Score = %u)\n", scoreData.GetScore());
    scoreData.Trace();

    nn::nex::ProtocolCallContext oContext;
    m_ranking2Client.PutScore(&oContext, scoreData);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to PutScore()\n");
        return false;
    }
    return true;
}

/*!
    @brief ランキングの取得を行う。

    @return 処理が成功したかどうか。
 */
qBool Ranking2Sample::GetRanking()
{
    nn::nex::Ranking2GetParam getParam;
    // 周囲 10 人のランキングを取得する
    getParam.SetMode(nn::nex::Ranking2Mode::NEAR_RANKING);
    getParam.SetLength(10);
    getParam.SetCategory(CATEGORY);
    
#if defined(CAFE_TARGET) || defined(CTR_TARGET)
    // Mii を取得する
    getParam.SetOptionFlags(nn::nex::Ranking2GetOptionFlags::MII_REQUIRED);
#endif
    
    getParam.Trace();

    nn::nex::ProtocolCallContext oContext;
    nn::nex::Ranking2Info rankingInfo;
    m_ranking2Client.GetRanking(&oContext, getParam, &rankingInfo);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to GetRanking()\n");
        return false;
    }
    PrintRanking(rankingInfo);
    return true;
}

/*!
    @brief ランキングチャートデータの取得を行う。

    @return 処理が成功したかどうか。
*/
qBool Ranking2Sample::GetRankingChart()
{
    nn::nex::Ranking2ChartInfoInput infoInput;

    infoInput.SetChartIndex(0);
    infoInput.Trace();

    nn::nex::ProtocolCallContext oContext;
    nn::nex::Ranking2ChartInfo chartInfo;
    m_ranking2Client.GetRankingChart(&oContext, infoInput, &chartInfo);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to GetRankingChart()\n");
        return false;
    }
    PrintRankingChart(chartInfo);
    return true;
}

/*!
    ランキング2の各種APIを呼び出します。
 */
void Ranking2Sample::CallRanking2Apis()
{
    if (!PutCommonData())
    {
        return;
    }
    if (!PutScore())
    {
        return;
    }
    if (!GetRanking())
    {
        return;
    }
    if (!GetRankingChart())
    {
        return;
    }
}

/*!
    @brief NEXの初期化後にNGSへログインし、ランキング2の各種APIを呼び出します。
 */
qInt NEXSampleMain()
{
    // インターネットに接続します
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit failed.\n");
        return 1;
    }

    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // ログレベルの設定(設定を行うかどうかは任意です)
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);

    {
        // ゲームサーバーへログインするためのクライアントを用意します。
        nn::nex::NgsFacade ngsFacade;
        nex_demo_util::LoginUtilityForSampleDemo loginUtilityForSampleDemo;

        // ゲームサーバーにログインします
        if (loginUtilityForSampleDemo.PerformTotalLogin(&ngsFacade))
        {
            {
                // Ranking2のAPI呼び出し
                Ranking2Sample sample(&ngsFacade);
                sample.CallRanking2Apis();
            }

            // ゲームサーバーからログアウトします
            nn::nex::ProtocolCallContext logoutCallContext;
            ngsFacade.Logout(&logoutCallContext);
            logoutCallContext.Wait();
        }
    }
    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    // インターネット接続を終了します。
    nex_demo_util::NetTerm();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("     Ranking2 sample\n");
    NEX_DEMO_LOG("*******************************\n\n");


    qInt returnCode = NEXSampleMain();


    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
