﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

/*
 * NEX Sample
 *
 * VSocket
 *
 * Description:
 *
 *  VSocketによる通信を行うサンプルデモです。
 *  VSocketはUDPによるソケット通信に似たインターフェースを提供する低レベルな通信ライブラリです。
 *  VSocketを使うとNEXのサーバー機能やNATトラバーサル機能や暗号化・改ざん検知機能を利用しつつ
 *  クライアント同士の P2P 通信に任意のサードパーティー製のネットワークエンジンを利用できるようになります。
 *  サードパーティー製のネットワークエンジンからソケットAPIを呼び出す代わりに
 *  VSocketのAPIを呼び出すように変更して使用されることを想定しています。
 *
 *  サンプルデモに実装されているシーケンスは以下の通りです。
 *
 * 1. NGSへログインする。
 * 2. オートマッチメイクを行いマッチメイクセッションへ入室する。
 * 3. NATトラバーサルを行い、プレイヤーの端末同士の直接接続を確立する。
 * 4. VSocketを使ったデータの送受信を行う。.
 * 5. NGSからログアウトする。
 *
 *  NATトラバーサルにより直接接続を確立した後はいつでもNGSから切断する事ができます。
 */

#include <nex.h>
#include <NexDemoUtil/NexUtil.h>

// この定義を有効にするとサーバークライアント(スター)型ネットワークトポロジを構築します。
// 中心となる端末はマッチメイクセッションに設定されたホストです。
// (MatchMakingClient::UpdateSessionHost(), MatchMakingClient::GetSessionURLs()を参照してください。)
// この定義をしない場合はフルメッシュ型ネットワークトポロジを構築します。
#define USE_SERVER_CLIENT_NETWORK_TOPOLOGY

namespace {
//-----------------------------------------------------------------------------
// Constants
//-----------------------------------------------------------------------------

// マッチメイクで使用する、アプリケーション定義のゲームモード
const qUnsignedInt32 MATCHMAKE_GAME_MODE = 1100;

// マッチメイクセッションの最大参加人数
const qUnsignedInt16 MATCHMAKE_MAX_PARTICIPANTS = 3;

// 使用するマッチメイクセッションの属性インデックス
const qUnsignedInt32 MATCHMAKE_ATTR_INDEX = 0;

// 使用するマッチメイクセッションの属性パラメータ
const qUnsignedInt32 MATCHMAKE_ATTR_VALUE = 1;

// 本サンプルデモがVSocketによる送受信を行う時間(ミリ秒)
const nn::nex::TimeInterval COMMUNICATION_PERIOD = 60000;

// データの送受信に使用される内部バッファサイズです。.
// もし以下のログが出力されるようでしたらこのバッファサイズを増やすか、送信サイズもしくは頻度を下げてください。
// "(Warning) PacketBuffer OutofMemory allocSize:xxx More pool required. see PacketBufferManager."
const size_t TRANSPORT_BUFFER_SIZE = 1 * 1024 * 1024;   // 1MB

// VSocketで使用する仮想的なポート番号です。0から15の範囲を指定できます。
const nn::nex::qUnsignedInt8 VSOCKET_VIRTUAL_PORT = 1;

// VSocketによる通信を暗号化するかどうか。
// 呼び出し側で暗号化処理を行う場合はVSocketによる暗号化をオフにする事が出来ます。
const bool ENABLE_ENCRYPTION = true;

/*!
    @brief NGSから通知を処理するイベントハンドラ。
 */
class MyNotificationEventHandler : public nn::nex::NotificationEventHandler
{
public:
    void ProcessNotificationEvent(const nn::nex::NotificationEvent &oEvent)
    {
        nn::nex::StringStream ss;
        ss << oEvent;
        NEX_DEMO_LOG("Notification " STRING_FORMAT "\n\n", ss.CStr());

        // Switch by notification type.
        switch (oEvent.GetType())
        {
        case nn::nex::NotificationEvents::ParticipationEvent:
            switch (oEvent.GetSubType())
            {
            case nn::nex::ParticipationEvents::Participate:
                NEX_DEMO_LOG("PARTICIPATION_EVENT PARTICIPATE: [PID " NEX_PID_FORMAT "] join [session %u]\n",
                    static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()), static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                break;

            case nn::nex::ParticipationEvents::EndParticipation:
                NEX_DEMO_LOG("PARTICIPATION_EVENT END: [PID " NEX_PID_FORMAT "] end [session %u]\n",
                    static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()), static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                break;

            case nn::nex::ParticipationEvents::Disconnect:
                NEX_DEMO_LOG("PARTICIPATION_EVENT DISCONNECT: [PID " NEX_PID_FORMAT "] disconnect [session %u]\n",
                    static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()), static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                break;

            default:
                NEX_DEMO_LOG("PARTICIPATION_EVENT: unsupported participation event\n");
                break;
            }
            break;

        case nn::nex::NotificationEvents::OwnershipChangeEvent:
            // This is only notified if nn::nex::GatheringFlags::MigrateOwner is specified to MatchmakeSession::SetFlag()
            NEX_DEMO_LOG("NOTIFICATION_EVENT OWNERSHIPCHANGE: [session %u] changed owner to [PID " NEX_PID_FORMAT "]\n", 
                static_cast<nn::nex::GatheringID>(oEvent.GetParam1()), static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()));
            break;

        case nn::nex::NotificationEvents::GatheringUnregistered:
            NEX_DEMO_LOG("NOTIFICATION_EVENT GATHERINGUNREGISTERED: [session %u] unregistered\n",
                static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
            break;

        default:
            NEX_DEMO_LOG("NOTIFICATION_EVENT: unsupported notification event\n");
            break;
        }
    }
};

/*!
    @brief VSocketで直接送受信するための経路を確立するため、NATトラバーサルを行います。

    まずNATトラバーサルの対象となる端末の情報(StationURL)をNGSから取得します。
    そのアドレス情報を元にVSocket::ProbeStations()を呼び出し、NATトラバーサルを行います。
    最後にNATトラバーサルの結果を確認します。

    @param[in] pMatchMakingClient Credentials をバインド済みのオブジェクトを指定してください。
    @param[in] joinedGatheringId 参加しているマッチメイクセッションのギャザリングID。
    @param[out] plstPeerURL NATトラバーサル済みの通信相手のIPアドレスとポート情報が含まれるリスト。

    @retval true NATトラバーサルが成功した。
                 (USE_SERVER_CLIENT_NETWORK_TOPOLOGYが有効の場合はマッチメイクセッションのホストと、
                  無効の場合は全ての参加者と成功した。)
    @retval false いずれかの端末とのNATトラバーサルが失敗した。
 */
bool DoNatTraversal(nn::nex::MatchmakeExtensionClient *pMatchMakingClient, nn::nex::GatheringID joinedGatheringId, nn::nex::qList<nn::nex::StationURL> *plstPeerURL)
{
    // NATトラバーサルを行う相手の情報を受け取るための変数
    nn::nex::qList<nn::nex::StationURL> lstTargetURLs;
    nn::nex::ProtocolCallContext oContext;
#ifdef USE_SERVER_CLIENT_NETWORK_TOPOLOGY   // サーバークライアント(スター)型ネットワークトポロジ向け
    // ホストの情報を取得します。
    // (一端末に付き複数のStationURLが返却されます。)
    pMatchMakingClient->GetSessionURLs(&oContext, joinedGatheringId, &lstTargetURLs);
#else   // フルメッシュ型ネットワークトポロジ向け
    // 参加者全ての情報を取得します。
    pMatchMakingClient->GetParticipantsURLs(&oContext, joinedGatheringId, &lstTargetURLs);
#endif
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Error querying participants list");
        return false;
    }

    // NATトラバーサルする相手をリストアップします。
    // サーバークライアント(スター)型ネットワークトポロジにおいてはホストの端末のみをリストアップします。
    // フルメッシュ型ネットワークトポロジにおいては自分自身より先にマッチメイクセッションに参加した端末をリストアップします。
    // GetParticipantsURLs()はマッチメイクセッションへの参加順にStationURLを
    // 返すことを利用して自分自身のプリンシパルIDが出現するまでをリストアップします。
    // (参加者全員をリストアップしても正しく処理が完了しますが、そうすると参加タイミングによっては
    //  同じ相手と二回NATトラバーサルを行い余分な時間を消費するケースが出てきます。
    //  NATトラバーサルは片方向で成功すれば双方向で通信が可能です。
    //  自分自身より先に参加した端末をリストアップすることで、同じ相手と二回NATトラバーサルをすることを防ぎます。)
    nn::nex::PrincipalID myPid = pMatchMakingClient->GetCredentials()->GetPrincipalID();
    nn::nex::qList<nn::nex::StationURL> lstStationURLsToProbe;
    nn::nex::qList<nn::nex::StationURL>::iterator itURLs = lstTargetURLs.begin();
    while (itURLs != lstTargetURLs.end() && itURLs->GetPrincipalID() != myPid)
    {
        lstStationURLsToProbe.push_back(*itURLs);
        NEX_DEMO_LOG("TargetURL = " STRING_FORMAT "\n", itURLs->CStr());
        ++itURLs;
    }

    // NATトラバーサルに関する情報を初期化します。NATトラバーサルを行った結果がここに格納されます。
    nn::nex::StationProbeList lstStationProbe(lstStationURLsToProbe);
    nn::nex::CallContext oProbeCallContext;

    // NATトラバーサルを行います。
    nn::nex::VSocket::ProbeStations(&oProbeCallContext, &lstStationProbe, 0, false);
    oProbeCallContext.Wait();
    if (oProbeCallContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oProbeCallContext.GetOutcome(), "Failed NAT traversal.");

        // VSocket::ProbeStations()の非同期処理に成功した時点で
        // 指定した全ての相手とNATトラバーサルが成功したことを示します。
        // VSocket::ProbeStations()の第四引数にfalseを指定した場合(デフォルト)は、
        // 誰か一人とNATトラバーサルが失敗した時点で非同期処理が終了しQERROR(Transport, NatTraversalError)の結果になるため、
        // 具体的にどの相手が原因で失敗したかは正確には分かりません。
        // VSocket::ProbeStations()の第四引数にtrueを指定した場合は、全ての相手に対してNATトラバーサルを試行し、成功もしくは失敗するまで待つため
        // QERROR(Transport, NatTraversalError)の結果になった後に具体的にどの相手と失敗したかを調べる事ができます。

        // 全員とのNATトラバーサルは失敗したが、一部成功している可能性があるのでプリンシパルIDごとの結果を表示します。
        // (部分的に成功した相手と通信を継続したい場合は、VSocket::ProbeStations()の第四引数にtrueを指定してください。)
        nn::nex::StationProbeList::iterator itProbe = lstStationProbe.begin();
        while (itProbe != lstStationProbe.end())
        {
            // NATトラバーサルに失敗した場合はPingTimeがInfiniteになります。
            if (itProbe->GetPingTime() == nn::nex::Infinite)
            {
                NEX_DEMO_LOG("  Partially failed PrincipalID = " NEX_PID_FORMAT "\n", itProbe->GetPrincipalId());
            }
            else
            {
                NEX_DEMO_LOG("  Partially succeeded PrincipalID = " NEX_PID_FORMAT "\n", itProbe->GetPrincipalId());
            }
            ++itProbe;
        }
        return false;
    }
    NEX_DEMO_LOG("Suceeded NAT traversal.\n");

    // 通信相手の内、最もRTTが長い端末のRTTをログ出力します。
    NEX_DEMO_LOG("Ping time to game session = %ums\n", lstStationProbe.GetPingStats().GetMax());

    // VSocketでの通信時に宛先として必要となるため、接続を確立した相手のIPアドレスとポート情報をplstPeerURLに格納します。
    NEX_DEMO_LOG("The peer URLs:\n");
    nn::nex::StationProbeList::iterator itProbe = lstStationProbe.begin();
    while (itProbe != lstStationProbe.end())
    {
        plstPeerURL->push_back(itProbe->GetPreferredURL());
        NEX_DEMO_LOG("  Peer URL = " STRING_FORMAT "\n", itProbe->GetPreferredURL().CStr());
        ++itProbe;
    }
    return true;
}

/*!
    @brief 対戦相手を探すため、オートマッチメイクを行います。 マッチメイクセッションに参加または作成します。

    任意のマッチメイク機能とVSocketを組み合わせることができます。
    マッチメイクについての詳細はMatchmmaking以下のサンプルデモを参照してください。

    @param[in] pMatchMakingClient Credentials をバインド済みのオブジェクトを指定してください。
    @param[out] joinedGatheringId 参加したマッチメイクセッションのギャザリングID。
    @param[out] pOwnerPID 参加したマッチメイクセッションのオーナーのプリンシパルID。

    @return オートマッチメイクに成功したかどうか。
 */
bool AutoMatchmake(nn::nex::MatchmakeExtensionClient *pMatchMakingClient, nn::nex::GatheringID* joinedGatheringId, nn::nex::PrincipalID* pOwnerPID)
{
    NEX_DEMO_LOG("Call AutoMatchmake\n");

    *pOwnerPID = INVALID_PRINCIPALID;

    // オートマッチメイクのパラメータをセットアップします。
    nn::nex::AutoMatchmakeParam autoMatchmakeParam;

    // 検索条件を設定します。
    {
        nn::nex::qList<nn::nex::MatchmakeSessionSearchCriteria> criteriaList;
        nn::nex::MatchmakeSessionSearchCriteria criteria;

        // 最大参加人数を設定します。
        criteria.SetMinParticipants(1);
        criteria.SetMaxParticipants(MATCHMAKE_MAX_PARTICIPANTS);

        // アプリケーション定義のゲームモードを設定します。
        criteria.SetGameMode(MATCHMAKE_GAME_MODE);

        // アプリケーション定義の検索属性値を設定します。
        criteria.SetAttribute(MATCHMAKE_ATTR_INDEX, MATCHMAKE_ATTR_VALUE);

        // 誰でも参加できるようにします。
        criteria.SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

        // 検索条件をAutoMatchmakeParamに設定します。
        criteriaList.push_back(criteria);
        autoMatchmakeParam.SetSearchCriteria(criteriaList);
    }

    // 検索条件に合うマッチメイクセッションが見つからなかったときに作成するマッチメイクセッションを設定します。
    {
        nn::nex::MatchmakeSession matchmakeSession;
        matchmakeSession.SetOpenParticipation(true);

        // 任意の説明文字列を設定します。
        matchmakeSession.SetDescription(NEX_T("VSocket"));

        // 最大参加人数を設定します。
        matchmakeSession.SetMinParticipants(1);
        matchmakeSession.SetMaxParticipants(MATCHMAKE_MAX_PARTICIPANTS);

        // アプリケーション定義のゲームモードを設定します。
        matchmakeSession.SetGameMode(MATCHMAKE_GAME_MODE);

        // アプリケーション定義の検索属性値を設定します。
        matchmakeSession.SetAttribute(MATCHMAKE_ATTR_INDEX, MATCHMAKE_ATTR_VALUE);

        // 誰でも参加できるようにします。
        matchmakeSession.SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

        // アプリケーション定義のデータを設定します。
        nn::nex::qVector<nn::nex::qUnsignedInt8> applicationBuffer;
        applicationBuffer.push_back(1);
        applicationBuffer.push_back(2);
        applicationBuffer.push_back(3);
        matchmakeSession.SetApplicationBuffer(applicationBuffer);

        // オーナーマイグレーションするかどうかを設定します。
        // オーナーが退出した後にマッチメイクセッションを存続させるにはこの設定を行ってください。
        // 設定をしない場合はオーナーが退出したときにマッチメイクセッションが削除されます。
        //matchmakeSession.SetFlag(nn::nex::GatheringFlags::MigrateOwner);

        autoMatchmakeParam.SetSourceMatchmakeSession(matchmakeSession);
    }

    autoMatchmakeParam.SetJoinMessage(NEX_T("join message"));
    nn::nex::MatchmakeSession joinedMatchmakeSession;

    // マッチメイクセッションを作成、または参加します。
    nn::nex::ProtocolCallContext oContext;
    pMatchMakingClient->AutoMatchmake(&oContext, autoMatchmakeParam, &joinedMatchmakeSession);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to AutoMatchmake");
        return false;
    }

    // 成功したケース

    // 改ざん検知のための鍵を設定します。.
    nn::nex::Network::GetInstance()->SetP2PDataPacketSessionSignatureKey(joinedMatchmakeSession.GetSessionKey());

    // VSocketによる通信を暗号化する場合はその鍵を設定します。
    if (ENABLE_ENCRYPTION)
    {
        nn::nex::Stream::GetSettings(nn::nex::Stream::Game).SetIsEncryptionRequired(true);
        nn::nex::Stream::GetSettings(nn::nex::Stream::Game).SetEncryptionKey(joinedMatchmakeSession.GetSessionKey());
    }

    *joinedGatheringId = joinedMatchmakeSession.GetID();
    *pOwnerPID = joinedMatchmakeSession.GetOwnerPID();

    NEX_DEMO_LOG("[PID Info >> JoinedGID:%u OwnerPID:" NEX_PID_FORMAT " HostPID:" NEX_PID_FORMAT " MyPID:" NEX_PID_FORMAT "]\n", *joinedGatheringId, *pOwnerPID, joinedMatchmakeSession.GetHostPID(), pMatchMakingClient->GetCredentials()->GetPrincipalID());

    return true;
}


/*!
    @brief マッチメイクセッションから退出します。

    @param[in] pMatchMakingClient Credentials をバインド済みのオブジェクトを指定してください。
    @param[in] gatheringID 参加しているマッチメイクセッションのギャザリングID。

    @return 退出に成功したかどうか。
 */
bool EndParticipation(nn::nex::MatchmakeExtensionClient *pMatchMakingClient, nn::nex::GatheringID gatheringID)
{
    NEX_DEMO_LOG("Call EndParticipation\n");

    nn::nex::ProtocolCallContext oContext;

    pMatchMakingClient->EndParticipation(&oContext, gatheringID);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        // Regard as success if the MatchmakeSession has already been unregistered.
        if(oContext.GetOutcome() == QERROR(RendezVous, SessionVoid))
        {
            NEX_DEMO_LOG("matchmake session is already unregistered. don't care\n");
        }
        else
        {
            nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to end participation");
            return false;
        }
    }
    return true;
}

/*!
    @brief VSocketを使って送受信する。

    @param[in] bHost 自分自身がホストかどうか。
    @param[in] lstPeerURL NATトラバーサルを行った結果の情報。IPアドレスやポート情報が含まれる。
 */
void CommunicateByVSocket(bool bHost, nn::nex::qList<nn::nex::StationURL> &lstPeerURL)
{
    NEX_DEMO_LOG("CommunicateByVSocket Start\n");

    nn::nex::VSocket vSocket;

    if(!vSocket.Open())
    {
        NEX_DEMO_LOG("Cannnot open.");
        return;
    }

    // Bind()にはVSocketで使用する仮想的なポート番号を指定します。
    // 0 - 15の任意の数値を指定してください。
    // 送信側と受信側で一致している必要があります。
    if(!vSocket.Bind(VSOCKET_VIRTUAL_PORT))
    {
        NEX_DEMO_LOG("Cannnot bind.");
        return;
    }
    bool bJoined = false;

    if (bHost)
    {
        NEX_DEMO_LOG("Waiting to receive incoming data...\n");
    }

    nn::nex::Time tiStart = nn::nex::Time::GetTime();
    while (true)
    {
        nn::nex::Time now = nn::nex::Time::GetTime();
        if (now - tiStart >= COMMUNICATION_PERIOD)
        {
            // 一定時間経過後に抜けます。
            break;
        }

        if (bHost)
        {
            // VSocket::RecvFrom()を使い他の参加者からデータを受信します。
            nn::nex::VSocket::InetAddress oSourceAddr; // 送信者のアドレス情報を格納する変数
            size_t uiRecvBufferSize = 0;
            qByte aRecvBuffer[nn::nex::VSocket::MAX_DATA_SIZE] = { 0 };
            while (vSocket.RecvFrom(aRecvBuffer, sizeof(aRecvBuffer), &oSourceAddr, &uiRecvBufferSize) ==
                    nn::nex::VSocket::Success)
            {
                NEX_DEMO_LOG("Received the following data: %s\n", reinterpret_cast<char*>(aRecvBuffer));
                bJoined = true;
            }

            if (bJoined)
            {
                // データを受信したらその送信者にデータを送り返します。
                // ホストは参加者からデータが送信されるまで相手のアドレス情報が分かりません。
                // 宛先はRecvFrom()で受け取ったものを使用します。
                char sendBuf[] = "This is game data from the host.";
                size_t uiSentBufferSize = 0;

                vSocket.SendTo(reinterpret_cast<qByte*>(sendBuf),
                               sizeof(sendBuf),
                               oSourceAddr,
                               &uiSentBufferSize);
            }
        }
        else
        {
            NEX_DEMO_LOG("send to URL:" STRING_FORMAT "\n", lstPeerURL.front().CStr());

            // 送信先のアドレスを設定します。
            // IPアドレスとポート番号はNATトラバーサルの結果を利用します。
            // ここではリストの先頭の相手に送信します。
            // (サーバークライアント(スター)型ネットワークトポロジを使用している場合はホストとのみNATトラバーサルを行うためリストのサイズは1になります。)
            // VPortNumberはVSocket::Bind()で指定した値と同じものを指定してください。
            nn::nex::VSocket::InetAddress oDestinationAddress;
            oDestinationAddress.SetAddress(lstPeerURL.front().GetInetAddress().GetAddress());
            oDestinationAddress.SetPortNumber(lstPeerURL.front().GetInetAddress().GetPortNumber());
            oDestinationAddress.SetVPortNumber(VSOCKET_VIRTUAL_PORT);
            size_t uiSentBufferSize = 0;
            char sendBuf[] = "This is game data from the joiner.";

            // データを送信します。
            // nn::nex::VSocket::Successが返された場合はuiSentBufferSizeは常に第二引数で指定したサイズと同じになり、それ以外が返された場合は必ず0となります。
            // データが分割して送信される事はありません。
            if (vSocket.SendTo(reinterpret_cast<qByte*>(sendBuf),
                               sizeof(sendBuf),
                               oDestinationAddress,
                               &uiSentBufferSize) != nn::nex::VSocket::Success)
            {
                NEX_DEMO_LOG("Faild to SendTo\n");
            }

            nn::nex::VSocket::InetAddress oSourceAddr;
            size_t uiRecvBufferSize = 0;
            qByte aRecvBuffer[nn::nex::VSocket::MAX_DATA_SIZE] = { 0 };

            // データを受信します。
            // 受信するデータは常に送信されたときのサイズと同じになります。分割されて受信する事はありません。
            while (vSocket.RecvFrom(aRecvBuffer, sizeof(aRecvBuffer), &oSourceAddr, &uiRecvBufferSize) ==
                    nn::nex::VSocket::Success)
            {
                NEX_DEMO_LOG("Received the following data: %s\n", reinterpret_cast<char*>(aRecvBuffer));
            }
        }

        // NEXの内部処理を進めます。(VScoketやゲームサーバーとの送受信処理や非同期処理などが行われます。
        // レイテンシを最小にするため、以下のような流れでゲームループ内で二回この関数を呼ぶことを推奨します。
        // 1. nn::nex::Scheduler::GetInstance()->DispatchAll() (ここでは主に受信処理が行われる)
        // 2. nn::nex::VSocket::RecvFrom()を呼び出し、受信したデータをゲームのオブジェクトに反映させる。
        // 3. ゲームの計算処理を行う。
        // 4. nn::nex::VSocket::SendTo()を呼び出し、直近の状態を他のプレイヤーに伝える。
        // 5. nn::nex::Scheduler::GetInstance()->DispatchAll() (ここでは主に送信処理が行われる)
        // 6. ゲーム画面を描画する。
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // ここではビジーループを防ぐためスリープします。
        // 実際のゲームでは垂直同期を待つ等してください。
        nn::nex::Platform::Sleep(nex_demo_util::SLEEP_DURATION);
    }
}

/*!
    @brief マッチメイク、NATトラバーサル、VSocketによる通信を順に行います。

    @param[in] pNgsFacade ログイン済みのオブジェクトを指定してください。
 */
void RunMatchmakingAndCommunication(nn::nex::NgsFacade* pNgsFacade)
{
    nn::nex::MatchmakeExtensionClient matchMakingClient;
    matchMakingClient.Bind(pNgsFacade->GetCredentials());
    nn::nex::PrincipalID ownerPrincipalID = INVALID_PRINCIPALID;
    nn::nex::GatheringID joinedGatheringId = INVALID_GATHERINGID;

    // 対戦相手を探すためマッチメイクセッションに参加します。
    if (!AutoMatchmake(&matchMakingClient, &joinedGatheringId, &ownerPrincipalID))
    {
        return;
    }

    nn::nex::qList<nn::nex::StationURL> lstPeerURL;
    qBool amIHost = matchMakingClient.GetCredentials()->GetPrincipalID() == ownerPrincipalID;

    // NATトラバーサルを行う準備をします。
    {
        nn::nex::CallContext oContext;
        nn::nex::ConnectivityManager::GetInstance()->StartNATSession(&oContext);
        oContext.Wait();
        if(oContext.GetState() != nn::nex::CallContext::CallSuccess)
        {
            nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to StartNatSession");
            goto END;
        }
    }

    if (amIHost)
    {
        // 自分自身がホストの場合は、自分が最初の参加者であるためDoNatTraversal()を呼び出す必要はありません。
        // VSocketによる通信シーケンスに移行します。
        CommunicateByVSocket(amIHost, lstPeerURL);
    }
    else if (DoNatTraversal(&matchMakingClient, joinedGatheringId, &lstPeerURL))
    {
        // 自分自身がホストではない場合はNATトラバーサルをした後にVSocketによる通信シーケンスに移行します。
        CommunicateByVSocket(amIHost, lstPeerURL);
    }
    else
    {
        NEX_DEMO_LOG("Failed DoNatTraversal()\n");
    }

    // NATトラバーサルのセッションを終了します。VSocketオブジェクトを破棄、生成する度にStartNATSession()及びStopNATSession()の呼び出しが必要です。
    nn::nex::ConnectivityManager::GetInstance()->StopNATSession();

END:
    // マッチメイクセッションから退出します。
    EndParticipation(&matchMakingClient, joinedGatheringId);
}

/*!
    @brief NEXの初期化後にNGSへログインし、VSocketによる通信を行います。
 */
qInt NEXSampleMain()
{
    // インターネットに接続します
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit failed.\n");
        return 1;
    }

    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // ログのレベルを設定します。
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // P2P通信がうまくいかない場合には、以下のコメントアウトを外して
    // NATトラバーサルログを出力してください。
    //nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS|TRACE_NAT);

    // VSocketが使用する通信用のバッファサイズを設定します。
    {
        nn::nex::qVector<nn::nex::PacketBufferManager::UnitHeapParam> param;
        nn::nex::PacketBufferManager::GetSimpleUnitHeapParam(TRANSPORT_BUFFER_SIZE, nn::nex::VSocket::MAX_DATA_SIZE, param);
        nn::nex::PacketBufferManager::SetUnitHeapParam(param);
    }

    // 設定値の表示。
    NEX_DEMO_LOG("***** Max participants: %d *****\n", MATCHMAKE_MAX_PARTICIPANTS);

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);

    {
        // ゲームサーバーへログインするためのクライアントを用意します。
        nn::nex::NgsFacade ngsFacade;
        nex_demo_util::LoginUtilityForSampleDemo loginUtilityForSampleDemo;

        // NGSからの通知を処理するイベントハンドラを登録
        MyNotificationEventHandler myNotificationEventHandler;
        ngsFacade.RegisterNotificationEventHandler(&myNotificationEventHandler);

        // ゲームサーバーにログインします
        if (loginUtilityForSampleDemo.PerformTotalLogin(&ngsFacade))
        {
            nn::nex::NATTraversalClient natTraversalClient;
            natTraversalClient.Bind(ngsFacade.GetCredentials());

            // 本サンプルデモのメイン処理である、マッチメイクとVSocketによる通信を行います。
            RunMatchmakingAndCommunication(&ngsFacade);

            // ゲームサーバーからログアウトします
            nn::nex::ProtocolCallContext logoutCallContext;
            ngsFacade.Logout(&logoutCallContext);
            logoutCallContext.Wait();
        }
        // イベントハンドラを削除
        ngsFacade.UnregisterNotificationEventHandler(&myNotificationEventHandler);
    }
    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    // インターネット接続を終了します。
    nex_demo_util::NetTerm();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("     VSocket sample\n");
    NEX_DEMO_LOG("*******************************\n\n");


    qInt returnCode = NEXSampleMain();


    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
