﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

/*
 * NEX Sample
 *
 * BlockList
 *
 * Description:
 *
 *  ブロックリストの使用方法についてのサンプルです。
 *  ゲームサーバーにログイン後、ゲームサーバーに登録したブロックリストを取得し、
 *  BLOCK_LIST_TARGET_1, BLOCK_LIST_TARGET_2 で指定したユーザーをブロックリストに登録したのち削除します。
 *
 *  ブロックリストにユーザーを登録すると、マッチメイクを行う際に以下のような設定をすることができます。
 *  (1) 自分の参加しているマッチメイクセッションにそのユーザーが入ってこられないようにする。
 *  (2) そのユーザーが参加しているマッチメイクセッションに自分が参加しないようにする。
 *  詳細は AutoMatchmakeParam::SetBlockListParam() や JoinMatchmakeSessionParam::SetBlockListParam() などの API リファレンスを参照してください。
 *
 *  MatchmakeExtensionClient::FindMatchmakeSessionByParticipant() といった、ユーザーを指定してマッチメイクセッションを検索する関数において、
 *  自身がブロックリストに登録されている場合は非同期処理は正常に完了しますが、
 *  そのユーザーは検索対象から除外され検索結果には含まれない(空が返る)ようになります。
 *  これらの関数では、マッチメイクセッションの参加者全てのブロックリストをチェックするものではないため、取得できたマッチメイクセッションの
 *  参加者の中に自分をブロックリストに登録しているユーザがいる場合もあり得ます。
 *
 * Note:
 *  自分のフレンドをブロックリストに登録することはできません。
 *
 *  ブロックリストに登録したユーザーとフレンドになることができますが、ゲームサーバーでは
 *  ブロックリストが優先されるため、ブロックリストから明示的に削除する必要があります。
 */

#include <nex.h>
#include <NexDemoUtil/NexUtil.h>

namespace
{
// ブロックリスト登録対象
const nn::nex::PrincipalID BLOCK_LIST_TARGET_1 = 1001;
const nn::nex::PrincipalID BLOCK_LIST_TARGET_2 = 1002;

/*!
    @brief ゲームサーバから自分がブロックリストに登録したユーザ一覧を取得して表示します。

    @param[in] pMatchmakeClient Credentials をバインド済みのオブジェクトを指定してください。
 */
void PrintBlockList(nn::nex::MatchmakeExtensionClient *pMatchmakeClient)
{
    nn::nex::ProtocolCallContext oContext;

    // 自分が登録しているブロックリストを表示する
    nn::nex::qList<nn::nex::PrincipalID> registerdBlockList;
    pMatchmakeClient->GetMyBlockList(&oContext, &registerdBlockList);
    oContext.Wait();
    nn::nex::qResult result = oContext.GetOutcome();
    if(!result)
    {
        nex_demo_util::ReportQResultReturnCode(result, "Failed to obtain a list of my blocklist.");
        return;
    }

    NEX_DEMO_LOG("REGISTERD BLOCKLIST\n");
    for(nn::nex::qList<nn::nex::PrincipalID>::iterator it = registerdBlockList.begin(); it != registerdBlockList.end(); ++it)
    {
        NEX_DEMO_LOG("   " NEX_PID_FORMAT "\n", *it);
    }
}

/*!
    @brief ブロックリストの操作をします。

    @param[in] pNgsFacade ログイン済みのオブジェクトを指定してください。
 */
void OperateBlockList(nn::nex::NgsFacade* pNgsFacade)
{
    nn::nex::MatchmakeExtensionClient matchmakeClient;
    nn::nex::qList<nn::nex::PrincipalID> newUserList;
    nn::nex::ProtocolCallContext callContext;

    matchmakeClient.Bind(pNgsFacade->GetCredentials());

    // ブロックリスト登録済みユーザーを表示
    PrintBlockList(&matchmakeClient);

    // ブロックリストに追加する
    newUserList.push_back(BLOCK_LIST_TARGET_1);
    newUserList.push_back(BLOCK_LIST_TARGET_2);

    matchmakeClient.AddToBlockList(&callContext, newUserList);
    callContext.Wait();
    if(callContext.GetOutcome())
    {
        NEX_DEMO_LOG("AddToBlockList success\n");
    }
    else
    {
        nex_demo_util::ReportQResultReturnCode(callContext.GetOutcome(), "Failed to obtain add to blocklist.");
    }

    // ブロックリスト登録済みユーザーを表示
    // BLOCK_LIST_TARGET_1, BLOCK_LIST_TARGET_2 が追加されているはず
    PrintBlockList(&matchmakeClient);

    // 同じユーザーを削除する
    //matchmakeClient.ClearBlockList(&callContext);  // ClearBlockList を使用すると全件削除になる
    callContext.Reset();
    matchmakeClient.RemoveFromBlockList(&callContext, newUserList);
    callContext.Wait();
    if(callContext.GetOutcome())
    {
        NEX_DEMO_LOG("RemoveFromBlockList success\n");
    }
    else
    {
        nex_demo_util::ReportQResultReturnCode(callContext.GetOutcome(), "Failed to obtain remove from blocklist\n");
    }

    // ブロックリスト登録済みユーザーを表示
    // BLOCK_LIST_TARGET_1, BLOCK_LIST_TARGET_2 が削除されているはず
    PrintBlockList(&matchmakeClient);
}

/*!
    @brief NEXの初期化後にNGSへログインし、ブロックリストを操作します。
 */
qInt NEXSampleMain()
{
    // インターネットに接続します
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit failed.\n");
        return 1;
    }

    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // ログレベルの設定(設定を行うかどうかは任意です)
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);

    {
        // ゲームサーバーへログインするためのクライアントを用意します。
        nn::nex::NgsFacade ngsFacade;
        nex_demo_util::LoginUtilityForSampleDemo loginUtilityForSampleDemo;

        // ゲームサーバーにログインします
        if (loginUtilityForSampleDemo.PerformTotalLogin(&ngsFacade))
        {
            OperateBlockList(&ngsFacade);

            // ゲームサーバーからログアウトします
            nn::nex::ProtocolCallContext logoutCallContext;
            ngsFacade.Logout(&logoutCallContext);
            logoutCallContext.Wait();
        }
    }
    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    // インターネット接続を終了します。
    nex_demo_util::NetTerm();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("     BlockList sample\n");
    NEX_DEMO_LOG("*******************************\n\n");

    qInt returnCode = NEXSampleMain();

    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
