﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * IndependentServer
 *
 * Description:
 *
 *  独自サーバの認証トークンの取得を行うサンプルです。
 *
 * Note:
 *
 *  認証トークンの取得とゲームサーバのログイン処理を両方行う場合、
 *  NgsFacade::LoginAndRequestAuthenticationToken() を使用することで、
 *  2つの処理を1回で行うことができます。
 *
 *  認証トークンの取得処理はサーバの負荷になるため、認証トークンが有効期限内
 *  の時に再度、認証トークンを取得するというような処理は控えて下さい。
 */

#include <nn/friends.h>
#include <nex.h>
#include <NexDemoUtil/NexUtil.h>

namespace
{

/*!
    @brief NGSへのログインをせず、認証トークンの取得のみを行います。
 */
void AquireAuthenticationToken()
{
    // 認証トークン
    qChar8 authToken[nn::nex::IndependentServer::AUTH_TOKEN_SIZE] = {0};

    // IndependentServer::RequestAuthenticationToken() を呼び出す前にNFSへのログインは不要です。
    NEX_DEMO_LOG("Request authentication token...\n");

    nn::nex::IndependentServer oIndependentServer;
    nn::nex::ProtocolCallContext oContext;

    // RequestAuthenticationTokenがfalseとなる値を返すケースはプログラミングエラーです。
    NEX_DEMO_VERIFY(oIndependentServer.RequestAuthenticationToken(&oContext,
                                                                  nex_demo_util::GAME_SERVER_ID,
                                                                  nex_demo_util::KEY_HASH,
                                                                  authToken,
                                                                  true));
    oContext.Wait();

    if (oContext.GetOutcome())
    {
        // 実行時に PrincipalID が 0 であっても非同期処理が成功していれば PrincipalID が取得できている
        NEX_DEMO_LOG("My PrincipalId %u\n", nn::friends::GetMyPrincipalId());
        NEX_DEMO_LOG("authentication token = %s\n", authToken);
    }
    else
    {
        NEX_DEMO_LOG("IndependentServer::RequestAuthenticationToken() failed. Error code : %d\n", oIndependentServer.GetLastErrorCode());
    }
}

/*!
    @brief NGSへログインすると同時に認証トークンの取得を行います。
 */
void LoginToNgsAndAquireAuthenticationToken()
{
    // 認証トークン
    qChar8 authToken[nn::nex::IndependentServer::AUTH_TOKEN_SIZE] = {0};

    // ゲームサーバへのログインと同時に認証トークンを取得する場合は
    // フレンドサーバへのログインが必須になる
    NEX_DEMO_LOG("Logging in to Friend Server...\n");
    {
        nn::Result result = nex_demo_util::LoginUtility::LoginToNfsBlock();
        if (result.IsSuccess())
        {
            NEX_DEMO_LOG("My PrincipalId %u\n", nn::friends::GetMyPrincipalId());
            NEX_DEMO_LOG("nn::friends::Login() succeeded.\n");
        }
        else
        {
            NEX_DEMO_LOG("nn::friends::Login() failed. Error code: %d\n", nn::friends::ResultToErrorCode(result));
            NN_DBG_PRINT_RESULT(result);
            return;
        }
    }

    {
        NEX_DEMO_LOG("Logging in to Nintendo Game Server as well...\n");

        nn::nex::ProtocolCallContext oContext;
        nn::nex::NgsFacade oNgsFacade;

        NEX_DEMO_VERIFY(oNgsFacade.LoginAndRequestAuthenticationToken(&oContext,
                                                                      nex_demo_util::GAME_SERVER_ID,
                                                                      nex_demo_util::NGS_ACCESS_KEY,
                                                                      nex_demo_util::KEY_HASH, authToken));
        oContext.Wait();

        if (oContext.GetOutcome())
        {
            NEX_DEMO_LOG("authentication token = %s\n", authToken);
        }
        else
        {
            NEX_DEMO_LOG("Ngsfacade::LoginAndRequestAuthenticationToken() failed. Error code : %d\n", oNgsFacade.GetLastLoginErrorCode());
        }

        NEX_DEMO_LOG("Logging out from Game Server...\n");
        {
            // ゲームサーバからログアウト
            nn::nex::ProtocolCallContext ccLogout;
            oNgsFacade.Logout(&ccLogout);

            ccLogout.Wait();
        }

        NEX_DEMO_LOG("Logging out from Friend Server...\n");
        {
            // フレンドサーバからログアウト
            nn::friends::Logout();
        }

        NEX_DEMO_LOG("Logged out.\n");
    }
}

/*!
    @brief 認証トークンの取得を二つの方法で行います。
 */
void AquireAuthenticationTokenInTwoWays()
{
    // フレンドライブラリを初期化する
    // フレンドサーバへのログインが不要であっても、認証トークンを取得する際にライブラリ内部で nn::friends の関数を
    // 呼び出すため、必ず初期化が必要となります。
    nn::friends::Initialize();
    NEX_DEMO_LOG("My PrincipalId " NEX_PID_FORMAT "\n", nn::friends::GetMyPrincipalId());

    // 認証トークンの取得のみを行う例です。
    AquireAuthenticationToken();

    // NGSへのログインと認証トークンの取得を同時に行う例です。
    LoginToNgsAndAquireAuthenticationToken();

    nn::friends::Finalize();
}

/*!
    @brief NEXの初期化後、認証トークンの取得を行います。
 */
qInt NEXSampleMain()
{
    // インターネットに接続します
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit failed.\n");
        return 1;
    }

    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // ログレベルの設定(設定を行うかどうかは任意です)
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);

    AquireAuthenticationTokenInTwoWays();

    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    // インターネット接続を終了します。
    nex_demo_util::NetTerm();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("     IndependentServer sample\n");
    NEX_DEMO_LOG("*******************************\n\n");

    qInt returnCode = NEXSampleMain();

    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
