﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * MultiUnitHeap
 *
 * Description:
 *
 *  UnitHeapを複数使用した、比較的高速なメモリアロケーターのサンプルです。
 *
 * Note:
 *
 *  NEXはメモリ確保・開放を非常に高頻度で行います。
 *  NEXのCPU負荷軽減を検討する場合には、本サンプルのようなMultiUnitHeapの採用を推奨します。
 *  詳細な実装は MultiUnitHeapTemplate.h をご覧下さい。
 *
 *  本サンプルで紹介するMultiUnitHeapはUnitHeapを複数使い、
 *  要求されたアロケーションサイズを確保可能な最適なUnitHeapを検索しています。
 *
 *  本サンプルのMultiUnitHeapを改変して、 アプリケーションに組み込んでいただいても構いません。
 *  Test関数は簡単なMultiUnitHeapの機能テストを行なっており、テストに失敗するとAssertで止まります。
 *  改変後の機能テストにもお役立てください。
 */
#include <nex.h>
#include <NexDemoUtil/NexUtil.h>
#include "MultiUnitHeapTemplate.h"

namespace
{
typedef nn::fnd::UnitHeap TSampleUnitHeap; // MultiUnitHeap内で使用するUnitHeapタイプ
typedef nn::fnd::ExpHeap TSampleReservedHeap; // MultiUnitHeapが枯渇したときに内で使用するリザーブヒープタイプ
const size_t UNIT_HEAP_NUM = 5; // MultiUnitHeapが扱うUnitHeap数
typedef MultiUnitHeapTemplate<UNIT_HEAP_NUM, TSampleUnitHeap, TSampleReservedHeap> TSampleMultiUnitHeap;

// ルートヒープ。このヒープからMultiUnitHeapを作成します。
const size_t ROOT_HEAP_BYTES = 1024 * 1024 * 8;
qByte s_RootHeapBuffer[ROOT_HEAP_BYTES];
nn::fnd::ExpHeap s_RootHeap;

// 各UnitHeapのユニットサイズ
const size_t s_UnitSize[UNIT_HEAP_NUM] =
{
    100,
    500,
    1000,
    2000,
    5000
};

// 各UnitHeapのユニット数
const size_t s_UnitObjs[UNIT_HEAP_NUM] =
{
    4000,
    2000,
    1000,
    500,
    20
};

TSampleMultiUnitHeap s_MultiUnitHeap;

/*!
    @brief MultiUnitHeapの簡単な機能テストを行います。
 */
void Test(TSampleMultiUnitHeap& muh)
{
    NEX_DEMO_LOG("\nTest start\n");
    TSampleMultiUnitHeap::DebugCount debugCount[UNIT_HEAP_NUM]= {};

    muh.ResetDebugCount();

    {
        // 一番小さいUnitHeapをいっぱいにするテスト

        void *p[4000]; // 最大数分を確保
        for (size_t i=0 ; i<s_UnitObjs[0] ; ++i)
        {
            p[i] = muh.Allocate(1); // ユニットサイズが一番小さいUnitHeapから確保されます
            NEX_DEMO_ASSERT(p[i]);

            muh.GetDebugCount(debugCount);
            NEX_DEMO_ASSERT(debugCount[0].use == i+1);
            NEX_DEMO_ASSERT(debugCount[0].peak == i+1);
        }

        muh.GetDebugCount(debugCount);
        NEX_DEMO_ASSERT(debugCount[0].peak == s_UnitObjs[0]);


        // ユニットサイズが一番小さいUnitHeapは使いきっていますが、次のUnitHeapから確保するよう実装しています
        void* pp = muh.Allocate(1);
        NEX_DEMO_ASSERT(pp);
        muh.GetDebugCount(debugCount);
        NEX_DEMO_ASSERT(debugCount[1].use == 1);
        muh.Free(pp);

        for (size_t i=0 ; i<s_UnitObjs[0] ; ++i)
        {
            muh.Free(p[i]);
        }
    }

    {
        // 各UnitHeapのユニットサイズの境界をAllocするテスト

        for (size_t i=0 ; i<UNIT_HEAP_NUM ; ++i)
        {
            size_t allocSize = s_UnitSize[i];
            void *p1 = muh.Allocate(allocSize);
            NEX_DEMO_ASSERT(p1);

            allocSize += 1; // 次に大きなユニットサイズのUnitHeapから確保される
            void *p2 = muh.Allocate(allocSize);
            NEX_DEMO_ASSERT(p2);


            if (allocSize > s_UnitSize[UNIT_HEAP_NUM-1])
            {
                // 緊急用ヒープが使われた
                NEX_DEMO_ASSERT(muh.IsOutOfMemory() == true);
            }
            else
            {
                NEX_DEMO_ASSERT(muh.IsOutOfMemory() == false);
            }

            muh.Free(p1);
            muh.Free(p2);
        }
    }
    NEX_DEMO_ASSERT(muh.IsClean());

    muh.ResetDebugCount();
    NEX_DEMO_LOG("Test end\n");
}

/*!
    @brief NEX用メモリアロケータ関数。

    @param[in] size 確保するメモリのサイズ。
 */
void* AllocFromMultiUnitHeap(size_t size)
{
    return s_MultiUnitHeap.Allocate(size);
}

/*!
    @brief NEX用メモリ解放関数。

    @param[in] p 解放するアドレス。
 */
void FreeToMultiUnitHeap(void* p)
{
    s_MultiUnitHeap.Free(p);
}

/*!
    @brief NEXにアプリケーション独自のアロケータをセットします。
 */
void NexAllocatorSetup()
{
    nn::nex::MemoryManager::SetBasicMemoryFunctions(AllocFromMultiUnitHeap, FreeToMultiUnitHeap);

    nn::nex::Initialize(true);
    NEX_DEMO_LOG("\nMultiUnitHeap dump, after nn::nex::Initialize().\n");
    s_MultiUnitHeap.Dump(); // 内部のデバッグカウンタをプリントします

    nn::nex::Finalize(true);
    NEX_DEMO_LOG("\nMultiUnitHeap dump, after nn::nex::Finalize().\n");
    s_MultiUnitHeap.Dump();
}

/*!
    @brief MultiUnitHeapのセットアップを行います。
 */
qInt NEXSampleMain()
{
    // RootHeapの準備
    NEX_DEMO_LOG("ROOT_HEAP_BYTES : %u[kbyte]\n", ROOT_HEAP_BYTES / 1024);
    s_RootHeap.Initialize(reinterpret_cast<uptr>(s_RootHeapBuffer), ROOT_HEAP_BYTES);

    {
        // MultiUnitHeap枯渇時のリザーブヒープ準備
        const size_t RESERVED_HEAP_BYTES = 1024 * 512;
        NEX_DEMO_LOG("RESERVED_HEAP_BYTES : %u[kbyte]\n", RESERVED_HEAP_BYTES / 1024);
        void* pReservedHeapBuffer = s_RootHeap.Allocate(RESERVED_HEAP_BYTES);
        NEX_DEMO_ASSERT(pReservedHeapBuffer);
        TSampleReservedHeap reservedHeap;
        reservedHeap.Initialize(reinterpret_cast<uptr>(pReservedHeapBuffer), RESERVED_HEAP_BYTES);

        {
            // MultiUnitHeapの準備
            const size_t MULTI_UNIT_HEAP_BYTES = s_MultiUnitHeap.GetRequiredMemorySize(s_UnitSize, s_UnitObjs);
            NEX_DEMO_LOG("MULTI_UNIT_HEAP_BYTES : %u[kbyte]\n", MULTI_UNIT_HEAP_BYTES / 1024 );

            void* pMultiUnitHeapBuffer = s_RootHeap.Allocate(MULTI_UNIT_HEAP_BYTES);
            NEX_DEMO_ASSERT(pMultiUnitHeapBuffer);

            NEX_DEMO_ASSERT(s_MultiUnitHeap.IsInitialized() == false);
            s_MultiUnitHeap.Initialize(
                pMultiUnitHeapBuffer,
                MULTI_UNIT_HEAP_BYTES,
                s_UnitSize,
                s_UnitObjs,
                &reservedHeap
            );
            NEX_DEMO_ASSERT(s_MultiUnitHeap.IsInitialized() == true);

            Test(s_MultiUnitHeap);

            NexAllocatorSetup();

            s_MultiUnitHeap.Finalize();
            NEX_DEMO_ASSERT(s_MultiUnitHeap.IsInitialized() == false);
            s_RootHeap.Free(pMultiUnitHeapBuffer);
        }

        // MultiUnitHeap枯渇時のリザーブヒープ破棄
        reservedHeap.Finalize();
        s_RootHeap.Free(pReservedHeapBuffer);
    }

    s_RootHeap.Finalize();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("     MultiUnitHeap sample\n");
    NEX_DEMO_LOG("*******************************\n\n");

    qInt returnCode = NEXSampleMain();

    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
