﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#include <nn.h>
#include <nn/version.h>
#include <nn/init.h>
#include <nn/os.h>
#include <nn/ac.h>
#include <nn/socket_Initialize.h>
#include <nn/http_Initialize.h>

#include "NetInit.h"
#include "NexUtilDefs.h"

using namespace nn;
using namespace nn::socket;
using namespace nn::os;

namespace nex_demo_util
{

namespace
{
// 1スレッドあたりのソケットバッファサイズ
// nn::socket::Initializeは4096Byteアラインメントされたバッファが必要
const size_t SOCKBUF_SIZE    = 64 * 1024;

// NEXは最大4スレッドからソケットを使用します
const size_t SOCKET_THREADS  = 4;

// ソケットバッファ
NN_ATTRIBUTE_ALIGN(4096) char s_sockBuf[SOCKBUF_SIZE * SOCKET_THREADS];

#ifndef NN_SWITCH_DISABLE_DEBUG_PRINT
const char* GetAddressString(nn::socket::InAddr addr)
{
    static char buffer[16];
    u8* p = reinterpret_cast<u8*>(&addr);
    std::snprintf(buffer, sizeof(buffer), "%d.%d.%d.%d", p[0], p[1], p[2], p[3]);
    return buffer;
}
#endif
nn::os::Event s_acDisconnectedEvent;
qBool s_IsSocketInitialized = false;
} // namespace

// インターネットへの接続を開始します。
qBool NetInit()
{
    Result result;
    nn::os::Thread::Sleep(nn::fnd::TimeSpan::FromMicroSeconds(2000));

    NN_LOG("Starting NetInit (waiting) %s\n", __FILE__);

    NN_LOG("NetInit:nn::ac::Initialize()\n");
    nn::fs::Initialize();
    result = nn::ac::Initialize();
    if ( result.IsSuccess() )
    {
        nn::ac::Config config;
        nn::ac::CreateDefaultConfig( &config );

        NN_LOG("Client: Connecting..\n");
        result = nn::ac::Connect( config );
        if ( result.IsSuccess() )
        {
            NN_LOG("Client: Connection succeeded.\n");
        }
        else
        {
            NN_LOG("Client: Connection failed.\n");
            NN_DBG_PRINT_RESULT(result);
            return false;
        }
    }
    else
    {
        NN_LOG("Client: Initialization failed.\n");
        NN_DBG_PRINT_RESULT(result);
        return false;
    }

    // Initializing socket library
    NN_LOG("Initializing socket..\n");

    // NEXでは4つのスレッドからソケットAPIを呼び出すことがあります。
    size_t sizeSockBuf = nn::socket::GetRequiredMemorySize(SOCKBUF_SIZE, SOCKET_THREADS);
    result = nn::socket::Initialize(reinterpret_cast<uptr>(s_sockBuf), sizeSockBuf, SOCKBUF_SIZE, SOCKET_THREADS);
    NN_ASSERT(sizeSockBuf < sizeof(s_sockBuf));
    NN_UTIL_PANIC_IF_FAILED(result);

    if (result.IsSuccess())
    {
        {
            s32 ret;
            nn::socket::InAddr addr, netmask;
            ret = nn::socket::GetPrimaryAddress(reinterpret_cast<u8*>(&addr), reinterpret_cast<u8*>(&netmask));
            NN_ASSERT(ret == 0);
            NN_LOG("host    : %s\n", GetAddressString(addr));
            NN_LOG("netmask : %s\n", GetAddressString(netmask));

            nn::socket::InAddr dns1, dns2;
            ret = nn::socket::GetResolver(reinterpret_cast<u8*>(&dns1), reinterpret_cast<u8*>(&dns2));
            NN_ASSERT(ret == 0);
            NN_LOG("dns1    : %s\n", GetAddressString(dns1));
            NN_LOG("dns2    : %s\n", GetAddressString(dns2));

            nn::socket::InAddr gateway;
            ret = nn::socket::GetDefaultGateway(reinterpret_cast<u8*>(&gateway));
            NN_ASSERT(ret == 0);
            NN_LOG("gateway : %s\n", GetAddressString(gateway));

#ifndef NN_SWITCH_DISABLE_DEBUG_PRINT
            NN_LOG("\n");
            nn::socket::DumpRoutingTable();
#endif

            NN_UNUSED_VAR(ret);
        }

        // Eventを登録して、AC切断を検知できるようにします
        s_acDisconnectedEvent.Initialize(true);
        result = nn::ac::CTR::RegisterDisconnectEvent(&s_acDisconnectedEvent);
        if (!result.IsSuccess())
        {
            NN_LOG("nn::ac::CTR::RegisterDisconnectEvent() failed.\n");
            NN_DBG_PRINT_RESULT(result);
            return false;
        }

        s_IsSocketInitialized = true;

        NN_LOG("NetInit() succeeded.\n");
        return true;
    }
    else
    {
        NN_LOG("NetInit() failed (socket::Initialize).\n");
        NN_DBG_PRINT_RESULT(result);
        return false;
    }
}

// インターネット接続を終了します。
void NetTerm()
{
    Result result;

    if (s_IsSocketInitialized)
    {
        NN_LOG("Finalizing socket.\n");

        // Finalizing socket library
        result = socket::Finalize();
        NN_UTIL_PANIC_IF_FAILED(result);

        if (result.IsFailure())
        {
            NN_LOG("NetTerm() failed (socket::Finalize).\n");
            NN_DBG_PRINT_RESULT(result);
        }
        s_IsSocketInitialized = false;
    }

    NN_LOG("Close ac.\n");
    nn::ac::Close();

    NN_LOG("Finalizing ac.\n");
    nn::ac::Finalize();

    // Eventを破棄します。
    s_acDisconnectedEvent.Finalize();
}

// ネットワークの接続性をチェックします。
qBool IsNetworkConnected()
{
    // このAPIは毎ゲームフレーム呼び出すには重い処理です。
    //return nn::ac::CTR::IsConnected();

    // イベントのシグナルをチェックします。
    return !s_acDisconnectedEvent.Wait(0);
}

// HTTPライブラリを初期化します。
qBool InitializeHttpLibrary()
{
    if(nn::http::Initialize().IsFailure())
    {
        NEX_DEMO_LOG("nn::http::initialize() failed.\n");
        return false;
    }
    return true;
}

// HTTPライブラリを終了します。
void FinalizeHttpLibrary()
{
    nn::http::Finalize();
}

}   // end of namespace nex_demo_util
