﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#include "NexUtil.h"
#include "CtrNexUtil.h"



namespace nex_demo_util
{

const qUnsignedInt32 GAME_SERVER_ID     = 0x25E5BE00;
const qChar* NGS_ACCESS_KEY             = NEX_T("12345678");
const qChar8* KEY_HASH                  = "deed2a88";
const qUnsignedInt32 TITLE_UNIQUE_ID    = 0xFE003;

nn::Result LoginUtility::LoginToNfsBlock()
{
    // フレンドサーバにログインする
    nn::os::Event event(true);
    nn::Result result = nn::friends::Login(&event);
    if (result.IsFailure())
    {
        NEX_DEMO_LOG("nn::friends::Login() failed.\n");
        return result;
    }
    event.Wait();
    return nn::friends::GetLastResponseResult();
}

class LoginToNfsFunctor
{
public:
    LoginToNfsFunctor(nn::Result* pResult)
        : m_pResult(pResult)
    {
    }

    qBool operator()()
    {
        *m_pResult = LoginUtility::LoginToNfsBlock();
        return m_pResult->IsSuccess();
    }
private:
    nn::Result* m_pResult;
};

qBool LoginUtility::LoginToNfs(nn::nex::CallContext* pContext, nn::Result* pResult)
{
    nn::nex::ThreadedJob threadedJob;
    return threadedJob.Start(pContext, LoginToNfsFunctor(pResult));
}



qInt32 LoginUtility::LoginToNfsAndNgsBlock(nn::nex::NgsFacade* pNgsFacade,
    qUnsignedInt32 gameServerId,
    const qChar* pAccessKey,
    nn::nex::TimeInterval timeoutMs)
{

    // NFSへログイン
    {
        nn::Result friendResult = LoginToNfsBlock();
        if (friendResult.IsFailure())
        {
            return static_cast<qInt32>(nn::friends::ResultToErrorCode(friendResult));
        }
    }

    // NGSへログイン
    {
        nn::nex::ProtocolCallContext context;
        nn::nex::qResult loginReturnValue = pNgsFacade->Login(&context, gameServerId, pAccessKey, timeoutMs, nn::friends::GetMyPrincipalId());

        // Login()の返値が成功とならないケースは全てプログラミングエラーです。
        NEX_DEMO_VERIFY(loginReturnValue == true);
        if (!loginReturnValue)
        {
            ReportQResultReturnCode(loginReturnValue, "NgsFacade::Login returned an error.");
            return nn::nex::ErrorCodeConverter::ConvertToNetworkErrorCode(loginReturnValue);
        }

        context.Wait();
        if (context.GetState() != nn::nex::CallContext::CallSuccess)
        {
            // Login() の非同期処理に失敗した場合は CallContext::GetOutcome() で結果を参照するのではなく、
            // NgsFacade::GetLastLoginErrorCode() を使用してください。
            return pNgsFacade->GetLastLoginErrorCode();
        }
    }
    return 0;
}

class LoginToNfsAndNgsFunctor
{
public:
    LoginToNfsAndNgsFunctor(nn::nex::NgsFacade* pNgsFacade,
                            qUnsignedInt32 gameServerId,
                            const qChar* pAccessKey,
                            nn::nex::TimeInterval timeoutMs,
                            qInt32* pErrorCode)
        : m_pNgsFacade(pNgsFacade)
        , m_gameServerId(gameServerId)
        , m_accessKey(pAccessKey)
        , m_timeoutMs(timeoutMs)
        , m_pErrorCode(pErrorCode)
    {
    }

    qBool operator()()
    {
        *m_pErrorCode = LoginUtility::LoginToNfsAndNgsBlock(m_pNgsFacade, m_gameServerId, m_accessKey.CStr(), m_timeoutMs);
        return *m_pErrorCode == 0;
    }
private:
    nn::nex::NgsFacade* m_pNgsFacade;
    qUnsignedInt32 m_gameServerId;
    nn::nex::String m_accessKey;
    nn::nex::TimeInterval m_timeoutMs;
    qInt32* m_pErrorCode;
};

qBool LoginUtility::LoginToNfsAndNgs(nn::nex::CallContext* pContext,
    nn::nex::NgsFacade* pNgsFacade,
    qUnsignedInt32 gameServerId,
    const qChar* pAccessKey,
    nn::nex::TimeInterval timeoutMs,
    qInt32* pErrorCode)
{
    // 本関数は独自に起動したスレッドからのNgsFacade::Login()呼び出しを含みます。
    // Scheduler::Dispatch()との競合を防ぐためスレッドセーフモードである必要があります。
    NEX_DEMO_ASSERT(nn::nex::Core::IsThreadSafe());

    nn::nex::ThreadedJob threadedJob;
    return threadedJob.Start(pContext, LoginToNfsAndNgsFunctor(pNgsFacade, gameServerId, pAccessKey, timeoutMs, pErrorCode));
}

qBool LoginUtilityForSampleDemo::PerformTotalLogin(nn::nex::NgsFacade* pNgsFacade)
{

    return PerformTotalLogin(pNgsFacade, GAME_SERVER_ID, NGS_ACCESS_KEY, RECOMMENDED_LOGIN_TIMEOUT_MS);
}

qBool LoginUtilityForSampleDemo::PerformTotalLogin(nn::nex::NgsFacade* pNgsFacade,
    qUnsignedInt32 gameServerId,
    const qChar* pAccessKey,
    nn::nex::TimeInterval timeoutMs)
{
    // 本関数の最後で正しくエラーが発生したかどうかを判断できるようにするために最初に前回発生したエラーをリセットします。
    pNgsFacade->ResetLastLoginError();

    qInt32 loginNnResult;
    nn::nex::CallContext callContext;
    NEX_DEMO_VERIFY(LoginUtility::LoginToNfsAndNgs(&callContext, pNgsFacade, gameServerId, pAccessKey, timeoutMs, &loginNnResult));
    callContext.Wait();
    if (callContext.GetState() == nn::nex::CallContext::CallSuccess)
    {
        return true;
    }

    // 必要に応じてエラーコードをエラー・EULAアプレットで表示してください。
    // ここではNEXに関するエラーの場合のみデバッグログにエラーコードを表示します。
    if (pNgsFacade->HasErrorOccuredLastLogin())
    {
        NEX_DEMO_LOG("NgsFacade::Login() failed. Error code : %07d\n", pNgsFacade->GetLastLoginErrorCode());
    }

    return false;
}

}   // end of namespace nex_demo_util
