﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <nn/applet.h>
#include <nn/friends.h>
#include <nex.h>
#include "NexUtilDefs.h"

namespace nex_demo_util
{

/// NGSへのログインに使用するゲームサーバーID
extern const qUnsignedInt32 GAME_SERVER_ID;

/// NGSへのログインに使用するアクセスキー
extern const qChar* NGS_ACCESS_KEY;

/// 独自サーバーで使用するキーハッシュ値
extern const qChar8* KEY_HASH;

/// タイトルのユニーク ID です。一部の機能向けの定数です。
extern const qUnsignedInt32 TITLE_UNIQUE_ID;

/*!
   NGS へのログインに関するユーティリティを定義します。
   
   NGS へのログインするまでの流れは以下の通りです。
   1. nn::friend ライブラリを使い NFS へログインする。
   2. NgsFacade::Login() を使い NGS へログインする。
   
   本クラスを使用すると上記の各ステップを実行できます。
 */
class LoginUtility
{
public:
    /*!
        @brief   NFS (フレンドサーバー) へ同期的にログインします。

        @return 処理の結果。
    */
    static nn::Result LoginToNfsBlock();

    /*!
        @brief   NFS (フレンドサーバー) へ非同期的にログインします。

        @param[in, out] pContext 非同期処理を管理するオブジェクト。
        @param[out] pResult 処理の結果を格納するオブジェクト。非同期処理の成功/失敗に関わらず必ず格納されます。

        @retval true 非同期処理の開始に成功。
        @retval false pContext が NULL もしくはすでに進行中状態のものだったため非同期処理の開始に失敗。
    */
    static qBool LoginToNfs(nn::nex::CallContext* pContext, nn::Result* pResult);

    /*!
        @brief   NFS へのログインと NGS へのログインを同期的に行います。

             返値が 0 以外だった場合はそれをエラー・EULAアプレットで表示してください。

        @param[in, out] pNgsFacade NGS へのログインに使用するオブジェクト。
        @param[in] gameServerId ログインする NGS のゲームサーバー ID。
        @param[in] pAccessKey ログインする NGS のアクセスキー。
        @param[in] timeoutMs タイムアウト時間(ミリ秒)。

        @return エラーコード。成功した場合は 0。
    */
    static qInt32 LoginToNfsAndNgsBlock(nn::nex::NgsFacade* pNgsFacade,
        qUnsignedInt32 gameServerId,
        const qChar* pAccessKey,
        nn::nex::TimeInterval timeoutMs);

    /*!
        @brief   NFS へのログインと NGS へのログインを非同期的に行います。

        非同期処理が CallContext::CallFailure で終了した場合は pErrorCode をエラー・EULAアプレットで表示してください。

        @param[in, out] pContext 非同期処理を管理するオブジェクト。
        @param[in, out] pNgsFacade NGS へのログインに使用するオブジェクト。
        @param[in] gameServerId ログインする NGS のゲームサーバー ID。
        @param[in] pAccessKey ログインする NGS のアクセスキー。
        @param[in] timeoutMs タイムアウト時間(ミリ秒)。
        @param[out] pErrorCode 結果となるエラーコードが格納されます。成功した場合は 0 が格納されます。非同期処理の成功/失敗に関わらず必ず格納されます。

        @retval true 非同期処理の開始に成功。
        @retval false pContext が NULL もしくはすでに進行中状態のものだったため非同期処理の開始に失敗。
    */
    static qBool LoginToNfsAndNgs(nn::nex::CallContext* pContext,
        nn::nex::NgsFacade* pNgsFacade,
        qUnsignedInt32 gameServerId,
        const qChar* pAccessKey,
        nn::nex::TimeInterval timeoutMs,
        qInt32* pErrorCode);
};

/*!
   サンプルデモ専用の NGS へのログインに関するユーティリティを定義します。
 */
class LoginUtilityForSampleDemo
{
public:
    LoginUtilityForSampleDemo()
    {
        nn::friends::Initialize();
    }
    virtual ~LoginUtilityForSampleDemo()
    {
        nn::friends::Finalize();
    }

    /*!
        @brief   NFS へのログインと NGS へのログインを同期的に行います。

        ログインのためのパラメータはデフォルト値が使用されます。

        @param[in, out] pNgsFacade NGS へのログインに使用するオブジェクト。

        @return 一連の処理が成功したかどうか。
    */
    qBool PerformTotalLogin(nn::nex::NgsFacade* pNgsFacade);

    /*!
        @brief   NFS へのログインと NGS へのログインを同期的に行います。

        @param[in, out] pNgsFacade NGS へのログインに使用するオブジェクト。
        @param[in] gameServerId ログインする NGS のゲームサーバー ID。
        @param[in] pAccessKey ログインする NGS のアクセスキー。
        @param[in] timeoutMs タイムアウト時間(ミリ秒)。

        @return 一連の処理が成功したかどうか。
    */
    qBool PerformTotalLogin(nn::nex::NgsFacade* pNgsFacade,
        qUnsignedInt32 gameServerId,
        const qChar* pAccessKey,
        nn::nex::TimeInterval timeoutMs);
};


}   // end of namespace nex_demo_util

/// nnMain() を定義するためのマクロです。
#define NEX_MAIN_WRAPPER(USERMAIN) \
    extern "C" void nnMain() { \
        nn::applet::Enable(); \
        USERMAIN(); \
    }


