﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
#include <cmath>
#include <limits>

#include "NexUtil.h"
#include "DemoMem.h"

namespace nex_demo_util
{

namespace
{
// メモリサイズは、サンプルデモ用に大きめの値を設定しています。
// 使用用途に応じて調整して下さい。

// NEXで使用するメモリブロックのサイズ
const size_t s_workMemorySize = 4 * 1024 * 1024;

// NEXで使用するメモリブロックへのポインタ
nn::nex::qUnsignedInt8* s_pMemory         = NULL;

DemoMem s_demoMem;
}


// アプリケーション定義のメモリアロケータを NEX に設定します。
void InitializeMemoryAllocator()
{
    // NEXで使用するメモリブロックを確保する
    s_pMemory = new nn::nex::qUnsignedInt8[s_workMemorySize];

    // ヒープ用メモリアロケータユーティリティを初期化する
    s_demoMem.Initialize(s_pMemory, s_workMemorySize);

    // メモリアロケータ関数をNEXに登録する
    nn::nex::MemoryManager::SetBasicMemoryFunctions(AllocFunction, FreeFunction);
}

// InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
void FinalizeMemoryAllocator()
{
    // メモリアロケータ関数を登録解除します。
    nn::nex::MemoryManager::SetBasicMemoryFunctions(NULL, NULL);

    // メモリリークしていないかチェックする
    if (!s_demoMem.IsClean())
    {
        NEX_DEMO_LOG("Memory leak is detected.\n");
        s_demoMem.Dump();

        // リークしているメモリを無効化する
        s_demoMem.Invalidate();
    }

    // ヒープ用メモリアロケータユーティリティを破棄する
    s_demoMem.Finalize();
    PrintMemoryUsage();

    // ワークメモリを破棄する
    delete[] s_pMemory;
}

// qResultのリターンコードをデバッグログに表示します。
void ReportQResultReturnCode(const nn::nex::qResult& result, const char* pHeaderText)
{
    (void)result;
    if (pHeaderText)
    {
        NEX_DEMO_LOG("%s ", pHeaderText);
    }

    NEX_DEMO_LOG("qResult Return Code: 0x%x " STRING_FORMAT "\n", result.GetReturnCode(), result.GetReturnCodeString());
}


// メモリ使用状況をデバッグログに表示します。
void PrintMemoryUsage()
{
    NEX_DEMO_LOG("---------------------------------------\n");
    NEX_DEMO_LOG("Memory usage\n");
    NEX_DEMO_LOG("---------------------------------------\n");
    NEX_DEMO_LOG("Current memory usage = " NEX_SIZE_T_FORMAT "\n", s_demoMem.GetUsage());
    NEX_DEMO_LOG("Max memory usage     = " NEX_SIZE_T_FORMAT "\n", s_demoMem.GetMaxUsage());
    NEX_DEMO_LOG("Current alloc count  = " NEX_SIZE_T_FORMAT "\n", s_demoMem.GetCurrentAllocCount());
    NEX_DEMO_LOG("Total alloc count    = " NEX_SIZE_T_FORMAT "\n", s_demoMem.GetTotalAllocCount());
    NEX_DEMO_LOG("---------------------------------------\n");
}

// サンプルデモで使用するアプリケーション定義のメモリ確保関数です。
void* AllocFunction(size_t size)
{
    return s_demoMem.AllocFunction(size);
}

// サンプルデモで使用するアプリケーション定義のメモリ解放関数です。
void FreeFunction(void *pointer)
{
    s_demoMem.FreeFunction(pointer);
}

// 正規分布で乱数を生成する関数です。
qUnsignedInt32 GetRandNormal(double mu, double sigma)
{
    const double PI = 3.141592653589793f;
    double z = std::sqrt( -2.0 * std::log(nn::nex::Platform::GetRealRandomNumber(1.0)) ) * std::sin( 2.0 * PI * nn::nex::Platform::GetRealRandomNumber(1.0) );
    double v = mu + sigma * z;

    if (v < 0.0)
    {
        return 0;
    }
    if (v >= static_cast<double>(std::numeric_limits<qUnsignedInt32>::max()) )
    {
        return std::numeric_limits<qUnsignedInt32>::max() - 1;
    }

    return static_cast<qUnsignedInt32>(v);
 }


}   // end of namespace nex_demo_util
