﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * Description:
 *
 * P2PInitializeApiChain, P2PStartApiChain, P2PShutdownApiChain が定義されています。
 * これらはそれぞれ一連の定型処理を行います。
 * 
 * 1. P2PInitializeApiChain P2P通信の初期化を行います。
 * 2. (P2PStartApiChain)    P2Pセッションへの参加を行い、P2Pで送受信できる状態にします。
 * 3. P2PShutdownApiChain   P2P通信の終了処理を行います。
 * 
 * の順に利用します。P2PInitializeApiChain の後に P2PStartApiChain は呼び出さなくても P2PShutdownApiChain を利用できます。
 * P2PStartApiChain 完了後に DirectStream, DOの共有, RMC 等の各種送受信処理を行えます。
 * また、ホストマイグレーション拡張を用いた自動的なホスト情報とオーナーの移譲処理も行います。
 * ホスト(セッションマスター)が P2P セッションから切断した場合に、自動的に関連するマッチメイクセッションのホスト情報とオーナーを変更します。
 * 
 * 本機能はそのままアプリケーションで利用したり、必要に応じて修正して組み込んだりすることが可能です。
 */

#include <nex.h>

#include "NexUtil.h"
#include "NexUtilNetZ.h"

// SetDOClassInitializationCallback() のための宣言
namespace nn { namespace nex {
extern void InitDOClasses();
}}

namespace nex_demo_util
{

// P2P関係の初期化を行います。
// NATトラバーサル処理の初期化、NetZオブジェクトの生成、ホストマイグレーション拡張の初期化を行います。
class P2PInitializeApiChain : public nex_demo_util::ApiChain
{
public:
    /*
     @brief    P2P関係の初期化を行うためのオブジェクトを生成します

        この処理が成功した場合は、終了させるために P2PShutdownApiChain::Call() の呼び出しが必要です。

     @param[out]    ppNetZ 処理成功後にNetZオブジェクトを代入するアドレスへのポインタ。
     @param[out]    ppHostMigrationCallback 処理成功後にHostMigrationCallbackオブジェクトを代入するアドレスへのポインタ。
     @param[in]     pMatchMakingClient Credentials が Bind() されたオブジェクト。ホストマイグレーション発生時の UpdateSessionHost() 呼び出しに利用されます。
     @param[in]     sessionKey P2P通信の暗号化と署名に使用する鍵。サーバーから取得したMatchmakeSessionに含まれているものを指定してください。
     */
    P2PInitializeApiChain(nn::nex::NetZ** ppNetZ,
                     nn::nex::HostMigrationCallback** ppHostMigrationCallback,
                     nn::nex::MatchmakeExtensionClient* pMatchMakingClient,
                     const nn::nex::qVector<qByte> sessionKey)
        : m_ppNetZ(ppNetZ)
        , m_ppHostMigrationCallback(ppHostMigrationCallback)
        , m_pMatchMakingClient(pMatchMakingClient)
        , m_sessionKey(sessionKey)
    {
        NEX_DEMO_ASSERT(m_ppNetZ != NULL);
        NEX_DEMO_ASSERT(*m_ppNetZ == NULL);
        NEX_DEMO_ASSERT(m_ppHostMigrationCallback != NULL);
        NEX_DEMO_ASSERT(*m_ppHostMigrationCallback == NULL);
        NEX_DEMO_ASSERT(m_pMatchMakingClient != NULL);
        NEX_DEMO_ASSERT(!m_sessionKey.empty());
    }

    /*
     @brief    処理を実行します。
     */
    void Call()
    {
        NEX_DEMO_LOG("\nCall P2PInitializeApiChain.\n");

        // NAT トラバーサルを有効にする
        nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
        pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
        pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&P2PInitializeApiChain::OnCompletionStartNATSession), this));
        QVERIFY(nn::nex::ConnectivityManager::GetInstance()->StartNATSession(pCallContext));
    }

private:
    void OnCompletionStartNATSession(nn::nex::CallContext* pCallContext)
    {
        nn::nex::qResult result = pCallContext->GetOutcome();
        if (!result)
        {
            NEX_DEMO_LOG("     StartNATSession() failed.\n");
            nex_demo_util::ReportQResultReturnCode(result);
            Complete(result);
            return;
        }

        // DOやホストマイグレーション拡張を使う場合に必要な処理
        nn::nex::NetZ::SetDOClassInitializationCallback(&nn::nex::InitDOClasses);

        // NetZ オブジェクトの作成
        *m_ppNetZ = new nn::nex::NetZ;
        QVERIFY((*m_ppNetZ)->IsValid());

        // NetZの通信の暗号化を有効にする
        nn::nex::Stream::GetSettings().SetIsEncryptionRequired(true);
        nn::nex::Stream::GetSettings().SetEncryptionKey(m_sessionKey);

        // P2P通信用の暗号鍵をセット
        nn::nex::Network::GetInstance()->SetP2PDataPacketSessionSignatureKey(m_sessionKey);

        // Stationに表示される名前のセットアップ
        nex_demo_util::SetupLocalPlayerName();

        // DirectStreamの送受信を有効にします
        // DirectStream::Enable() を行うよりも前にパケットを受信した場合、ライブラリ側で破棄されることに注意してください
        nn::nex::DirectStream::GetInstance()->Enable();

        // ホストマイグレーション拡張機能を有効にする
        // ホストマイグレーション拡張機能とはP2PのホストがP2Pセッションを抜けたときにサーバーのマッチメイクセッションのホスト情報を自動的に更新する機能です。
        // このサンプルではホストとオーナーを一致させるようにします。
        // StaticUpdateSessionHostWithOwner() を使いホスト情報を更新すると同時にオーナーの変更も行います。
        // P2Pセッションのホストとなったユーザーがマッチメイクセッションのオーナーとなります。
        // HostMigrationExtension は NetZ オブジェクトが破棄されたときに自動で Unregister されます
        nn::nex::HostMigrationExtension::Register();
        *m_ppHostMigrationCallback = new nn::nex::HostMigrationCallback(m_pMatchMakingClient, nn::nex::MatchMakingClient::StaticUpdateSessionHostWithOwner);
        Complete(nn::nex::qResult(true));
    }

private:
    nn::nex::NetZ** m_ppNetZ;
    nn::nex::HostMigrationCallback** m_ppHostMigrationCallback;
    nn::nex::MatchmakeExtensionClient* m_pMatchMakingClient;
    nn::nex::qVector<qByte> m_sessionKey;
};


// P2Pセッションへの参加を行い、P2Pで送受信できる状態にします。
// 自分自身がマッチメイクセッションのオーナーなら自分がP2Pセッションのホストとなり、
// そうでないならマッチメイクセッションからホスト情報を取得してP2Pセッションへ参加します。
class P2PStartApiChain : public nex_demo_util::ApiChain
{
public:
    /*
     @brief    P2Pセッションへの参加を行い、P2Pで送受信できる状態にするためのオブジェクトを生成します。

     @param[in]     pMatchMakingClient Credentials が Bind() されたオブジェクト。ホストマイグレーション発生時の UpdateSessionHost() 呼び出しに利用されます。
     @param[in]     pHostMigrationCallback ホストマイグレーション拡張の初期化を行うオブジェクト。
     @param[in]     isOwner trueなら自身でP2Pセッションを作成します。falseならP2Pセッションに参加します。
     @param[in]     gid P2Pセッションに関連付けるマッチメイクセッションのギャザリング ID
     */
    P2PStartApiChain(nn::nex::MatchmakeExtensionClient* pMatchMakingClient,
                     nn::nex::HostMigrationCallback* pHostMigrationCallback,
                     qBool isOwner,
                     nn::nex::GatheringID gid)
        : m_pMatchMakingClient(pMatchMakingClient)
        , m_pHostMigrationCallback(pHostMigrationCallback)
        , m_isOwner(isOwner)
        , m_gid(gid)
        , m_lstSessionHostURL()
    {
        NEX_DEMO_ASSERT(m_pMatchMakingClient != NULL);
        NEX_DEMO_ASSERT(m_pHostMigrationCallback != NULL);
        NEX_DEMO_ASSERT(m_gid != INVALID_GATHERINGID);
    }

    /*
     @brief    処理を実行します。
     
        事前に P2PInitializeApiChain::Call() を行っている必要があります。この処理の成功/失敗を問わずその後 P2PShutdownApiChain::Call() を行う必要があります。

     */
    void Call()
    {
        NEX_DEMO_LOG("\nCall P2PStartApiChain(). m_isOwner: %d, m_gid: %u\n", m_isOwner, m_gid);

        // 誤って別のマッチメイクセッションに参加しているクライアントが P2P通信に参加してしまわないよう、
        // SetLocalIdentificationToken() で参加したギャザリングIDをセットする。
        // ここでセットした値を CheckJoiningProcesses() でチェックしている
        nn::nex::StringStream password;
        password << m_gid;
        nn::nex::Station::SetLocalIdentificationToken(password.CStr());

        if (m_isOwner)
        {
            // マッチメイクセッションのオーナーならP2Pセッションを作りホストとなる

            // P2Pセッションを作成
            QVERIFY(nn::nex::Session::CreateSession(NULL));

            // ホストマイグレーションコールバックを登録
            m_pHostMigrationCallback->Register(m_gid);

            // P2Pホストが自分自身であることをマッチメイクセッションに登録する
            nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
            pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
            pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&P2PStartApiChain::OnCompletionUpdateSessionHost), this));
            m_pMatchMakingClient->UpdateSessionHost(pCallContext, m_gid);
            return;
        }
        else
        {
            // マッチメイクセッションのオーナーでなければP2Pセッションに参加する

            // P2Pセッションに参加するために必要なP2Pホストの情報をマッチメイクセッションから取得する
            nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
            pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
            pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&P2PStartApiChain::OnCompletionGetSessionURLs), this));
            m_pMatchMakingClient->GetSessionURLs(pCallContext, m_gid, &m_lstSessionHostURL);
            return;
        }
    }

private:
    void OnCompletionUpdateSessionHost(nn::nex::CallContext* pCallContext)
    {
        nn::nex::qResult result = pCallContext->GetOutcome();
        if (!result)
        {
            NEX_DEMO_LOG("     UpdateSessionHost() failed.\n");
            nex_demo_util::ReportQResultReturnCode(result);
            Complete(result);
            return;
        }
        Complete(nn::nex::qResult(true));
    }

    void OnCompletionGetSessionURLs(nn::nex::CallContext* pCallContext)
    {
        nn::nex::qResult result = pCallContext->GetOutcome();
        if (!result)
        {
            NEX_DEMO_LOG("     GetSessionURLs() failed.\n");
            nex_demo_util::ReportQResultReturnCode(result);
            Complete(result);
            return;
        }

        // P2Pセッションに参加する
        nn::nex::CallContext* pNewCallContext = new nn::nex::CallContext();
        pNewCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
        pNewCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&P2PStartApiChain::OnCompletionJoinSession), this));
        nn::nex::Session::JoinSession(pNewCallContext, m_lstSessionHostURL);
    }

    void OnCompletionJoinSession(nn::nex::CallContext* pCallContext)
    {
        nn::nex::qResult result = pCallContext->GetOutcome();
        if (!result)
        {
            NEX_DEMO_LOG("     JoinSession() failed.\n");
            nex_demo_util::ReportQResultReturnCode(result);
            Complete(result);
            return;
        }
        // ホストマイグレーションコールバックを登録
        m_pHostMigrationCallback->Register(m_gid);
        Complete(nn::nex::qResult(true));
    }
    nn::nex::MatchmakeExtensionClient* m_pMatchMakingClient;
    nn::nex::HostMigrationCallback* m_pHostMigrationCallback;
    qBool m_isOwner;
    nn::nex::GatheringID m_gid;
    nn::nex::qList<nn::nex::StationURL> m_lstSessionHostURL;
};


// P2P通信の終了処理を行います。
// ホストマイグレーション拡張の終了処理、NetZオブジェクトの破棄、NATトラバーサル処理の終了を行います。
// P2PInitializeApiChainを行った場合はこの処理が必要です。P2PInitializeApiChainを行わずにこの処理を行うのは不正です。
// 非同期処理が失敗することはありません。
class P2PShutdownApiChain : public nex_demo_util::ApiChain
{
public:
    /*
     @brief    P2P通信の終了処理を行うためのオブジェクトを生成します

        それぞれ P2PInitializeApiChain で指定したパラメータを指定してください。

     @param[out]    ppNetZ 破棄するNetZオブジェクトのアドレスへのポインタ。有効なNetZオブジェクトを指定する必要があります。処理完了時にNULLが代入されます。
     @param[out]    ppHostMigrationCallback 破棄するHostMigrationCallbackオブジェクトのアドレスへのポインタ。処理完了時にNULLが代入されます。
     @param[in]     gid P2Pセッションに関連付けられたマッチメイクセッションのギャザリング ID。
     */
    P2PShutdownApiChain(nn::nex::NetZ** ppNetZ,
                        nn::nex::HostMigrationCallback** ppHostMigrationCallback,
                        nn::nex::GatheringID gid)
        : m_ppNetZ(ppNetZ)
        , m_ppHostMigrationCallback(ppHostMigrationCallback)
        , m_gid(gid)
    {
        NEX_DEMO_ASSERT(m_ppNetZ != NULL);
        NEX_DEMO_ASSERT(*m_ppNetZ != NULL);
        NEX_DEMO_ASSERT(m_ppHostMigrationCallback != NULL);
        NEX_DEMO_ASSERT(*m_ppHostMigrationCallback != NULL);
        NEX_DEMO_ASSERT(m_gid != INVALID_GATHERINGID);
    }

    /*
     @brief    処理を実行します。事前に P2PInitializeApiChain::Call() を行っている必要があります。
     */
    void Call()
    {
        NEX_DEMO_LOG("\nCall P2PShutdownApiChain().\n");

        // ホストマイグレーション拡張機能を無効にする。HostMigrationCallback::Register()を呼び出していなくてもUnregister()を呼び出すことは可能です。
        (*m_ppHostMigrationCallback)->Unregister(m_gid);
        delete (*m_ppHostMigrationCallback);
        *m_ppHostMigrationCallback = NULL;

        // 送受信を無効にします
        nn::nex::DirectStream::GetInstance()->Disable();

        // NetZ の終了
        nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
        pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
        pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&P2PShutdownApiChain::OnCompletionTerminate), this));
        (*m_ppNetZ)->Terminate(pCallContext);
    }

private:
    void OnCompletionTerminate(nn::nex::CallContext* pCallContext)
    {
        nn::nex::qResult result = pCallContext->GetOutcome();
        if (!result)
        {
            NEX_DEMO_LOG("     Terminate() failed.\n");
            nex_demo_util::ReportQResultReturnCode(result);
            // NetZ::Terminate() は失敗しない前提。終了処理を続行する。
        }

        // NetZオブジェクトはP2Pセッションを超えて使い回せないため、都度削除します。
        delete *m_ppNetZ;
        *m_ppNetZ = NULL;

        // NAT トラバーサルを無効にする
        nn::nex::ConnectivityManager::GetInstance()->StopNATSession();
        Complete(nn::nex::qResult(true));
    }
    nn::nex::NetZ** m_ppNetZ;
    nn::nex::HostMigrationCallback** m_ppHostMigrationCallback;
    nn::nex::GatheringID m_gid;
};

}   // end of namespace nex_demo_util
