﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <NetZ/src/Core/DOCore/DuplicatedObject.h>
#include <OnlineCore/src/Platform/Core/EventHandler.h>
#include <NetZ/src/Core/DOCore/DOCallContext.h>

#include <OnlineCore/src/Platform/Core/qList.h>

BEGIN_NAMESPACE_NEX
class Event;

// Group=Results Classes

/*
Summary: This class defines the context of a call to a duplicated object.
Remarks: This class is used to define the call context of calls made to fetch and migrate
duplicated objects and of RMCs.
*/

/*!
   @brief DuplicatedObject 用 の CallContext です。
   @internal
 */
class ActiveDOCallContext: public DOCallContext
{
    //Group=Type Definitions
public:
    ActiveDOCallContext(DOHandle hTargetStation=INVALID_DOHANDLE,
                        qBool bSynchronousCall=true);

    virtual ~ActiveDOCallContext();

#ifdef NEX_MOVE_SEMANTICS_SUPPORT

    /*!
    @brief      ムーブコンストラクタです。

    @param[in]  rhs    ProtocolCallContext オブジェクト。
    */
    ActiveDOCallContext(ActiveDOCallContext&& rhs) NEX_NOEXCEPT
        : DOCallContext()
        , m_pCallMessage(NULL)
    {
        *this = std::move(rhs);
    }

    /*!
    @brief      ムーブ代入演算子です。

    @param[in]  rhs    ProtocolCallContext オブジェクト。
    */
    ActiveDOCallContext& operator=(ActiveDOCallContext&& rhs) NEX_NOEXCEPT
    {
        if (this == &rhs)
        {
            return *this;
        }
        this->Swap(rhs);
        return *this;
    }
#endif

    // {secret}
    inline qBool KeepCallMessage() const;   //n1769: Added const.

    // {secret}
    qBool PerformCallAndWait();
    // {secret}
    qBool PerformCall();
    // {secret}
    virtual qResult SendCallMessage(Message* pCallMessage);

    // {secret}
    qResult SendCallMessageToStation(Message* pCallMessage, DOHandle hStation, qUnsignedInt32 uiFlags);

    // {secret}
    Message* GetCallMessage();

    void Swap(ActiveDOCallContext& rhs) NEX_NOEXCEPT
    {
        DOCallContext::Swap(rhs);
        std::swap(m_pCallMessage, rhs.m_pCallMessage);
    }

    /*!
    @brief      この @ref ActiveDOCallContext の完了時に実行するコールバックを設定します。

    設定したコールバックはこの @ref ActiveDOCallContext が完了状態に遷移したときに呼び出されます。
    このコールバックを使用することで、 @ref ActiveDOCallContext が完了したかどうかを確認するためにポーリングする手間を省くことができます。

    callback には @ref ActiveDOCallContext* を引数に取る関数ポインタ、関数オブジェクト、ラムダ式を指定できます。
    AddCallContextArgumentAdapter() を併用すると引数を一つも取らない関数を渡すこともできます。

    関数オブジェクトを渡す場合は値もしくは参照型で渡す方法とポインタ型で渡す方法の二種類があり、それぞれ挙動が異なります。
    値もしくは参照型で渡した場合は内部で関数オブジェクトがコピーもしくはムーブされ、コールバックはそのコピーもしくはムーブされたオブジェクトに対して行われます。
    そのため、本関数呼び出し後は指定したオブジェクトを即座に破棄する事ができます。
    ポインタ型で渡した場合は、コールバックはそのポインタが指すオブジェクトに対して行われます。
    そのため、コールバックが呼び出されるまでその関数オブジェクトを呼び出し側で保持しておく必要があります。

    本関数を複数回呼び出すことで複数のコールバックを登録することが出来ます。
    複数のコールバックを登録した場合はデフォルトでは登録順にコールバックの呼び出しが行われ、最初に登録されたコールバックは最初に呼び出されます。
    この順序を変更したい場合は、コールバックを登録する際に bAddToEnd パラメータを設定する必要があります。

    callback に関数オブジェクトをポインタで渡した場合は deleteCallbackAfterCall を true にする事で
    callback の呼び出し後に自動的に callback を delete することができます。
    もし callback を delete するのではなくアプリケーション独自の解放処理を行いたい場合は、
    再度本関数を呼び出しそのための関数オブジェクトを追加する事で実装が可能です。

    コールバック関数内では NEX のブロック関数を呼び出さないようにしてください。

    @param[in]  callback コールバックする関数ポインタもしくは関数オブジェクトもしくはラムダ式。
    @param[in]  deleteCallbackAfterCall callback の実行が完了した後自動的に callback を delete するかどうか。
    @param[in]  bAddToEnd true（デフォルト）であれば、コールバックはリストの最後に追加されます。falseであればリストの先頭に追加されます。
    */
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    template<typename TCallback>
    void RegisterCompletionCallbackGeneric(TCallback&& callback, qBool deleteCallbackAfterCall=false, qBool bAddToEnd=true)
    {
        RegisterCompletionCallbackGenericImpl<ActiveDOCallContext>(std::forward<TCallback>(callback), deleteCallbackAfterCall, bAddToEnd);
    }
#else
    template<typename TCallback>
    void RegisterCompletionCallbackGeneric(const TCallback& callback, qBool deleteCallbackAfterCall=false, qBool bAddToEnd=true)
    {
        RegisterCompletionCallbackGenericImpl<ActiveDOCallContext>(callback, deleteCallbackAfterCall, bAddToEnd);
    }
#endif

private:
    // Deactivate copy-ctor + op=
    // {secret}
    ActiveDOCallContext(const ActiveDOCallContext&);
    // {secret}
    ActiveDOCallContext& operator=(const ActiveDOCallContext&);

    qResult _SendCallMessage(Message* pCallMessage);

    qBool IsOverReliableSendBufferLimit(const Station *station, qUnsignedInt32 uiFlags) const;
    qBool IsOverReliableSendBufferLimit(const DORef &station, qUnsignedInt32 uiFlags) const;
    qBool IsOverReliableSendBufferLimit(const Selection *pTargets, qUnsignedInt32 uiFlags) const;

    qBool IsInvalidSubStreamID(const Station *station) const;
    qBool IsInvalidSubStreamID(const DORef &station) const;
    qBool IsInvalidSubStreamID(const Selection *pTargets) const;


    qBool IsUserDO();
#ifdef NEX_CLIENT_PBPOOL
    void SetMessageTargetType(Message*);
#endif // NEX_CLIENT_PBPOOL

protected:
    // Messages...
    // {secret}
    Message* m_pCallMessage;
};


END_NAMESPACE_NEX

