﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DOOperation.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <NetZ/src/Core/DOCore/Station.h>
#include <NetZ/src/Core/DOCore/MasterStationRef.h>

BEGIN_NAMESPACE_NEX

//Group = System Operations

/*
Description: This DOOperation is performed whenever the Duplication master of a duplicated object migrates.
Remarks: A ChangeMasterStationOperation will occur as a result of the migration of the Duplication master
of a duplicated object. As a result of this operation, the Role of the object on the local station may
change. The object may either be promoted from a Duplica to a Duplication master, demoted from a
Duplication master to a Duplica, or remain a Duplica.

When an object migrates, the ChangeMasterStationOperation is called on each Station where the
Duplication master or one of its Duplica s of the object is located. For details as to exactly when this
operation is called during object migration, refer to the bottom of the Duplicated Object Overview page.

When a ChangeMasterStationOperation is called the following sequence of events occurs:
    o   The SafetyExecutive is called to determine if the object may migrate. If so, we continue, if not we exit.
    o   The method DataSet::OperationBegin is called on all datasets,
    o   The method DuplicatedObject::OperationBegin is called,
    o   The object migrates to a new Station and changes its master StationID. The Role changes that accompany this change are also performed,
    o   The $PRODUCTNAME$ Duplication master and Duplica object lists are updated,
    o   The method DataSet::OperationEnd is called on all datasets,
    o   The method DuplicatedObject::OperationEnd is called.

This operation will not be called on a DuplicatedObject when the system state is locked via Scheduler::SystemLock.
See Also: Duplicated Object Overview
*/

/*!
 * @brief   このDOOperationは複製マスターのマイグレーションが発生するときに実行されます。
 *
 * マイグレーションとは、複製オブジェクトのコントロール権限がステーション間で移行することを言います。
 * このクラスで実行される操作は、複製マスターのマイグレーションが発生した結果として発生します。
 * この操作の結果として、ローカルステーションが保持している複製オブジェクトに対する役割が変化します。
 * 変化は以下のいずれかとなります。
 *
 * @li デュプリカ(@ref DuplicatedObject::IsADuplica)から複製マスター(@ref DuplicatedObject::IsADuplicationMaster)へ、権限が昇格する(@ref IsAPromotion)。
 * @li 複製マスターからデュプリカへ、権限が降格する(@ref IsADemotion)。
 * @li デュプリカのまま変化しない(@ref IsARoleChange 関数がfalseを返すとき)。
 *
 * 複製オブジェクトに対してマイグレーションが発生するとき、ChangeMasterStationOperationが、
 * その複製オブジェクトを保持している(複製マスターであるかデュプリカであるかは問わない)、
 * それぞれのステーションに対して呼び出されます。
 *
 * ChangeMasterStationOperationが呼び出されたとき、以下の順序に従ってイベントが発生します。
 *
 * @li @ref SafetyExecutive が実行されてこのマイグレーションが妥当なものであるかどうかが判定されます。もしそうでなければここで終了します。
 * @li @ref DataSet::OperationBegin 関数がすべてのデータセットに対して呼び出されます。
 * @li @ref DuplicatedObject::OperationBegin 関数が呼び出されます。
 * @li オブジェクトが新しいステーションへマイグレーションを行い、複製マスターの@ref StationID (ステーションの複製オブジェクトID) が更新されます。
 * @li 複製マスターとデュプリカのリスト(複製オブジェクトストア)が更新されます。
 * @li @ref DataSet::OperationEnd 関数がすべてのデータセットに対して呼び出されます。
 * @li @ref DuplicatedObject::OperationEnd 関数が呼び出されます。
 *
 * この操作はシステム状態がロックされているとき(@ref Scheduler::SystemLock)に@ref DuplicatedObject に対して呼び出されません。
 */
class ChangeMasterStationOperation: public DOOperation
{
    //Group=Methods
public:
    //{secret}
    enum Context
    {
        FaultRecovery,
        MigrationOnTermination,
        LoadBalancing,
        UserRequest,
        Unknown /* TODO: Eventually remove this context! */
    };
    //{secret}
    ChangeMasterStationOperation(DOHandle hOperationOrigin,
                                 DuplicatedObject* pDO,
                                 DOHandle hOldMaster,
                                 const MasterStationRef& refNewMaster,
                                 const qList<DOHandle> *plstDupSet,
                                 Context oContext);
    //{secret}
    virtual ~ChangeMasterStationOperation();

    /*
    Description: Returns a DOHandle to the Station where the Duplication master of the object resided
    before this ChangeMasterStationOperation was performed.
    Returns: A DOHandle to the Station where the Duplication master of the object resided
    before this ChangeMasterStationOperation was performed.
    See Also: GetNewMasterStation
    */

    /*!
     * @brief   この操作が実行される前に、オブジェクトの複製マスターとなっていたStationの複製オブジェクトハンドルを取得します。
     *
     * 操作対象となっている複製オブジェクトのマスターとなっているStationの
     * 複製オブジェクトハンドル(@ref DOHandle)を取得します。
     * セッションに存在するオブジェクトはすべて複製オブジェクトハンドルによって一意に表すことができ、
     * Net-Zでは参加者自体も@ref Station として複製オブジェクト扱いとなります。
     *
     * @return  この操作が実行される前に、複製マスターとなっていたStationの複製オブジェクトハンドルが返ります。
     * @see     GetNewMasterStation, Station::GetLocalStation
     */
    DOHandle GetOldMasterStation() const
    {
        return m_refOldMaster.GetHandle();
    }

    /*
    Description: Returns a DOHandle to the Station where the Duplication master of the object currently
    resides.
    Returns: A DOHandle to the Station where the Duplication master of the object currently resides
    after this ChangeMasterStationOperation was performed.
    See Also: GetOldMasterStation
    */

    /*!
     * @brief   この操作が実行された後に、新しくオブジェクトの複製マスターとなるStationの複製オブジェクトハンドルを取得します。
     *
     * 操作対象となっている複製オブジェクトの新しいマスターとなるStationの
     * 複製オブジェクトハンドル(@ref DOHandle)を取得します。
     * セッションに存在するオブジェクトはすべて複製オブジェクトハンドルによって一意に表すことができ、
     * Net-Zでは参加者自体も@ref Station として複製オブジェクト扱いとなります。
     *
     * @return  この操作が実行された後に、新しくオブジェクトの複製マスターとなるStationの複製オブジェクトハンドルが返ります。
     * @see     GetOldMasterStation, Station::GetLocalStation
     */
    DOHandle GetNewMasterStation() const
    {
        return m_refNewMaster.GetHandle();
    }

    // {secret}
    virtual DOHandle GetImplicitStationConnection() const;

    //{secret}
    virtual DOOperation* Clone() const;

    // {secret}
    LogicalClock GetNewMasterVersion() const
    {
        return m_refNewMaster.GetVersion();
    }
    // {secret}
    Context GetContext() const
    {
        return m_oContext;
    }
    // {secret}
    const qList<DOHandle>* GetDupSet() const
    {
        return m_plstDupSet;
    }

    // {secret}
    const MasterStationRef& NewMaster() const
    {
        return m_refNewMaster;
    }

    /*
    Description: Returns whether or not any role change associated with this operation is a promotion.
    Returns: true if the ChangeMasterStationOperation results in the object on the local station
    being promoted from a Duplica to a Duplication master, false if it is demoted or no role change
    occurs.
    Remarks: When a Duplication master migrates, the object on the local station may either be promoted
    from a Duplica to a Duplication master, demoted from a Duplication master to a Duplica, or remain
    a Duplica.
    See Also: IsADemotion, IsARoleChange
    */

    /*!
     * @brief   この操作によって複製オブジェクトに対する権限が昇格するかどうかを取得します。
     *
     * 権限の昇格とは、デュプリカ(コントロール権限がない状態、ただのオブジェクトコピー)から、
     * 複製マスター(コントロール権がある状態、コピー元)になることを言います。
     * 複製オブジェクトのマイグレーション(コントロール権限のステーション間移動)が発生するとき、
     * 複製オブジェクトに対する役割は、権限が昇格する、権限が降格する、デュプリカのままである
     * (別のステーションから別のステーションへの権限移動)の、
     * いずれかひとつとの状態となります。
     *
     * @retval  true    実行したステーションにとって、この操作実行時に複製オブジェクトが
     *                  デュプリカから複製マスターに権限昇格するときに返ります。
     * @retval  false   実行したステーションにとって、この操作実行時に複製オブジェクトが
     *                  複製マスターからデュプリカになるか、あるいはデュプリカのままであるかのときに返ります。
     * @see     IsADemotion, IsARoleChange, DuplicatedObject::IsADuplicationMaster, DuplicatedObject::IsADuplica
     */
    qBool IsAPromotion() const
    {
        return m_refNewMaster.GetHandle()==Station::GetLocalStation();
    }

    /*
    Description: Returns whether or not any role change associated with this operation is a demotion.
    Returns: true if the ChangeMasterStationOperation results in the object on the local station
    being demoted from a Duplication master to a Duplica, false if it is promoted or no role change
    occurs.
    Remarks: When a Duplication master migrates, the object on the local station may either be promoted
    from a Duplica to a Duplication master, demoted from a Duplication master to a Duplica, or remain
    a Duplica.
    See Also: IsAPromotion, IsARoleChange
    */

    /*!
     * @brief   この操作によって複製オブジェクトに対する権限が降格するかどうかを取得します。
     *
     * 権限の降格とは、複製マスター(コントロール権がある状態、コピー元)から、
     * デュプリカ(コントロール権がない状態、ただのオブジェクトコピー)になることを言います。
     * 複製オブジェクトのマイグレーション(コントロール権限のステーション間移動)が発生するとき、
     * 複製オブジェクトに対する役割は、権限が昇格する、権限が降格する、デュプリカのままである
     * (別のステーションから別のステーションへの権限移動)の、
     * いずれかひとつとの状態となります。
     *
     * @retval  true    実行したステーションにとって、この操作実行時に複製オブジェクトが
     *                  複製マスターからデュプリカに、権限降格するときに返ります。
     *
     * @retval  false   実行したステーションにとって、この操作実行時に複製オブジェクトが
     *                  デュプリカから複製マスターになるか、デュプリカのままであるときに返ります。
     *
     * @see     IsAPromotion, IsARoleChange, DuplicatedObject::IsADuplicationMaster, DuplicatedObject::IsADuplica
     */
    qBool IsADemotion() const
    {
        return m_refOldMaster.GetHandle()==Station::GetLocalStation();
    }

    /*
    Description: Returns whether or not any role change is associated with this operation.
    Returns: true if the ChangeMasterStationOperation results in the object on the local station
    being promoted from a Duplica to a Duplication master or being demoted from a Duplication master
    to a Duplica, false if no role change occurs.
    Remarks: When a Duplication master migrates, the object on the local station may either be promoted
    from a Duplica to a Duplication master, demoted from a Duplication master to a Duplica, or remain
    a Duplica.
    See Also: IsAPromotion, IsADemotion
    */

    /*!
     * @brief   この操作によって、複製オブジェクトに対する権限に変更が出るかどうかを取得します。
     *
     * 権限の変化とは、昇格(@ref IsAPromotion 関数を参照：デュプリカから複製マスターになること)または
     * 降格(@ref IsADemotion 関数を参照：複製マスターからデュプリカになること)が発生することをいいます。
     * 複製オブジェクトのマイグレーション(コントロール権限のステーション間移動)が発生するとき、
     * 複製オブジェクトに対する役割は、権限が昇格する、権限が降格する、デュプリカのままである
     * (別のステーションから別のステーションへの権限移動)の、
     * いずれかひとつとの状態となります。
     *
     * @retval  true    権限の変化が発生するときに返ります。
     * @retval  false   権限の変化が発生しないときに返ります。
     *                  これは、このマイグレーションが自分に関係のないところで発生することを意味します。
     *                  複製オブジェクトはデュプリカの状態から変化しません。
     *
     * @see     IsAPromotion, IsADemotion, DuplicatedObject::IsADuplicationMaster, DuplicatedObject::IsADuplica
     */
    qBool IsARoleChange()
    {
        return IsAPromotion() || IsADemotion();
    }

    NEX_IMPL_OPERATION_COMMON_METHODS(ChangeMasterStation);

private:

#ifdef NEX_TRACE_ENABLED    // n2370
    virtual void TraceImpl(Event eEvent, qUnsignedInt64 uiTraceFlags) const;
#endif  // NEX_TRACE_ENABLED

    //{secret}
    virtual qBool CallsBackOnDataSet()
    {
        return true;
    }
    //{secret}
    virtual qBool CallsBackOnDataSet(qByte /*byIndex*/)
    {
        return true;
    }

private:

    Station::Ref m_refOldMaster;
    MasterStationRef m_refNewMaster;
    const qList<DOHandle> *m_plstDupSet;
    Context m_oContext;
};

END_NAMESPACE_NEX

