﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DatasetCGMacros.h>
#include <NetZ/src/Core/DOCore/StationUpdateFilter.h>
#include <NetZ/src/Core/DOCore/GlobalUpdateFilter.h>

BEGIN_NAMESPACE_NEX

class Message;


// This class implements various built-in dataset properties via both the GlobalUpdateFilter
// and the StationUpdateFilter classes. If you wish to implement your own update filter,
// refer to the implementations given here for examples of how.

//
// Constant
//

//$begin GlobalUpdateFilter1$

// This is an example of the implementation of a constant filter where the
// dataset is never updated after discovery. UpdateSent is not implemented
// as it is not required.
class ConstFilter: public GlobalUpdateFilter
{
public:
    ConstFilter() {};
    virtual ~ConstFilter() {};

    // This method always returns false as after discovery we never
    // need to update the dataset
    virtual qBool UpdateRequired(DuplicatedObject* pDO,
                                 void* pDS,
                                 Time tTimestamp)
    {
        // All params are dummy as we don't need them here.
        DUMMY_PARAM(pDO);
        DUMMY_PARAM(pDS);
        DUMMY_PARAM(tTimestamp);
        return false;
    }
};

// Define the updatefilter macro
#define _PR_constant_filterdecl(DS) ConstFilter

//$end GlobalUpdateFilter1$

//
// upon_request_filter
//

// This is an example of the implementation of an upon request filter
// where an update is only sent if we specifically request it. This filter
// is implemented as a template and as such may be used for any dataset.
template<class DS>
class UponRequestFilter: public GlobalUpdateFilter
{
public:
    UponRequestFilter() {};
    virtual ~UponRequestFilter() {};

    virtual qBool UpdateRequired(DuplicatedObject* pDO,
                                 void* pDS,
                                 Time tTimestamp)
    {
        // Some params are dummy - we don't need them here.
        DUMMY_PARAM(pDO);
        DUMMY_PARAM(tTimestamp);

        return static_cast<DS*>(pDS)->UpdateRequested();
    }
    virtual void UpdateSent(DuplicatedObject* pDO,
                            void* pDS,
                            Time tTimestamp,
                            qUnsignedInt32 /*uiNbRecipients*/)
    {
        // Some params are dummy - we don't need them here.
        DUMMY_PARAM(pDO);
        DUMMY_PARAM(tTimestamp);

        static_cast<DS*>(pDS)->ClearUpdateRequestFlag();
    }
    virtual void UpdateNotSent(DuplicatedObject* pDO,
                            void* pDS,
                            Time tTimestamp,
                            qBool bNoDuplica)
    {
        // Some params are dummy - we don't need them here.
        DUMMY_PARAM(pDO);
        DUMMY_PARAM(tTimestamp);
        if(bNoDuplica)
        {
            // As there's no duplica, RequestUpdate was pointless.
            static_cast<DS*>(pDS)->ClearUpdateRequestFlag();
        }
    }
};

// Define the updatefilter macro
#define _PR_upon_request_filter_filterdecl(DS)  UponRequestFilter< DS >

// Define the code macros. Here we define the various methods used to flag a
// dataset as needing to be updated or not.
#define _PR_upon_request_filter_pudecl()    \
    qBool RequestUpdate();                  \
    void ClearUpdateRequestFlag();      \
    qBool UpdateRequested() {return m_dsUpdateRequestFlag.GetFlag();}
#define _PR_upon_request_filter_prdecl()    \
    DataSetFlag m_dsUpdateRequestFlag;
#define _PR_upon_request_filter_impl(DS)    \
    qBool DATASET(DS)::RequestUpdate() {DSCHECK_ROLE_IS_DM(false); m_dsUpdateRequestFlag.SetFlag(); return true;}   \
    void DATASET(DS)::ClearUpdateRequestFlag() {m_dsUpdateRequestFlag.ResetFlag();}


//
// Station filter
//

// This filter updates datasets according to the response returned by
// the UpdateIsRequired method. This filter is implemented as a template
// and as such may be used for any dataset.
template<class DS>
class StationFilter: public StationUpdateFilter
{
public:
    StationFilter() {};
    virtual ~StationFilter() {};

    virtual qBool UpdateRequired(DuplicatedObject* pDO,
                                 void* pDS,
                                 Time tTimestamp,
                                 Station* pTargetStation)
    {
        // Some params are dummy - we don't need them here.
        DUMMY_PARAM(pDO);
        DUMMY_PARAM(tTimestamp);

        return static_cast<DS*>(pDS)->UpdateIsRequired(pDO,pTargetStation);
    }
};

// Define the updatefilter macro
#define _PR_station_filter_filterdecl(DS)   StationFilter< DS >

#ifdef NEX_PERFCOUNTER_ENABLED

//
// update_perf_counter
//

template<class DS>
class UpdatePerfCounter: public GlobalUpdateFilter
{
public:
    UpdatePerfCounter() {};
    virtual ~UpdatePerfCounter() {};

    virtual qBool UpdateRequired(DuplicatedObject* /*pDO*/,
                                 void* /*pDS*/,
                                 Time /*tTimestamp*/)
    {
        return true;
    }

    virtual void UpdateSent(DuplicatedObject* pDO,
                            void* pDS,
                            Time tTimestamp,
                            qUnsignedInt32 uiNbRecipients)
    {
        // Some params are dummy - we don't need them here.
        DUMMY_PARAM(pDS);
        DUMMY_PARAM(pDO);
        DUMMY_PARAM(tTimestamp);

        static_cast<DS*>(pDS)->GetPerformanceCounters()->Update(uiNbRecipients);
    }
};

#define _PR_update_perf_counter_filterdecl(DS)  UpdatePerfCounter< DS >

#define _PR_update_perf_counter_pudecl()    \
    static DataSetPerfCounters* GetPerformanceCounters() {return &s_oPerfDataSet;}  \
    static void RegisterCounters(qUnsignedInt32 uiOp);

#define _PR_update_perf_counter_prdecl()    \
    static DataSetPerfCounters s_oPerfDataSet;                                      \
    static qBool s_bDataSetRegistered;

#define _PR_update_perf_counter_impl(DS)    \
    DataSetPerfCounters DATASET(DS)::s_oPerfDataSet(NEX_T(#DS));   \
    qBool DATASET(DS)::s_bDataSetRegistered=false;      \
    void DATASET(DS)::RegisterCounters(qUnsignedInt32 uiOp) {   \
        switch (uiOp) {             \
            case DATASET_OP_REGISTER:           \
                                                        if (!s_bDataSetRegistered) {    \
                                                        s_oPerfDataSet.Register();  \
                                                    s_bDataSetRegistered=true;  \
                                                        }                               \
                                                        break;                          \
            case DATASET_OP_UNREGISTER:         \
                                                        if (s_bDataSetRegistered) {     \
                                                    s_oPerfDataSet.Unregister();    \
                                                    s_bDataSetRegistered=false; \
                                                        }   \
                                                        break;  \
            default:    \
                                SYSTEMCHECK(false); \
        }   \
    }

#else // NEX_PERFCOUNTER_ENABLED

template<class DS>
class UpdatePerfCounter: public GlobalUpdateFilter
{
public:
    UpdatePerfCounter() {};
    virtual ~UpdatePerfCounter() {};

    virtual qBool UpdateRequired(DuplicatedObject* /*pDO*/,
                                 void* /*pDS*/,
                                 Time /*tTimestamp*/)
    {
        return true;
    }

    virtual void UpdateSent(DuplicatedObject* /*pDO*/,
                            void* /*pDS*/,
                            Time /*tTimestamp*/,
                            qUnsignedInt32 /*uiNbRecipients*/)
    {
    }
};

#define _PR_update_perf_counter_filterdecl(DS) UpdatePerfCounter< DS >

#define _PR_update_perf_counter_pudecl()

#define _PR_update_perf_counter_prdecl()

#define _PR_update_perf_counter_impl(DS)

#endif // NEX_PERFCOUNTER_ENABLED

//
// unreliable
//

#define _PR_unreliable_pudecl()             qBool ReliableUpdate() const {return false;}
#define _PR_unreliable_prdecl()
#define _PR_unreliable_impl(DS)

//
// buffered
//

template<class DS>
class BufferedDataset: public DS
{
public:
    // Inherits members from DS.
    // Inherits AddSourceTo from DS.

    void ExtractFrom(Message *pMessage)
    {
        m_dsBuffer.ExtractFrom(pMessage);
    }
    qBool Refresh(const Time& /*tRefreshTime*/)
    {
        return Refresh();
    }
    qBool Refresh()
    {
        DS* pMyselfAsDS = static_cast<DS*>(this);
        *pMyselfAsDS = m_dsBuffer;
        return true;
    }
private:
    DS m_dsBuffer;
};

#define _PR_buffered_dsdecl(DS) BufferedDataset< DS>

END_NAMESPACE_NEX


