﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/ParseTree/DOClassID.h>
#include <NetZ/src/Core/DOCore/DOID.h>
#include <OnlineCore/src/Transport/Interface/Message.h>
#include <OnlineCore/src/Platform/Core/StringStream.h>

BEGIN_NAMESPACE_NEX

// Group = Miscellaneous Types and Macros

// {Secret}
enum
{
    CLASSID_BITS=10, // 1024 Class ID...
    OBJECTID_BITS=(32-CLASSID_BITS),

    MAX_NB_CLASSID=(1<<CLASSID_BITS),
    MAX_NB_OBJECTID=(1<<OBJECTID_BITS),

    MAX_NB_WELLKNOWNDO=256,

    OBJECTID_MASK=((1<<OBJECTID_BITS)-1),
    CLASSID_MASK=(((qUnsignedInt32)-1)^OBJECTID_MASK)
};

// TODO: This should be moved elsewhere???

/*
$ifdef SYNCSIM$
{secret}
$else$
Summary:  The session's unique global station identifier.
Remarks: The StationID of a Station object is equivalent to the value of the object's DOHandle.
StationID's are not reused. When a station leaves the session, its
ID will not be reused. An implication of this is that the valid StationID is not
limited by the session size. For example, in a session with /n/ stations, some
stations may have an ID greater than /n/.
$endif$
*/

/*!
 * @brief   セッションの中で一意なステーションの識別子です。
 *
 * @ref Station オブジェクトのStationIDとは、複製オブジェクトハンドルに対する@ref DOHandle::GetValue() の値です。
 * セッションからステーションが離脱しても、StationIDはセッションの中で使いまわされることはありません。
 * セッションから離脱したあと、セッションに別のステーションが入っても別のIDが割り当てられます。
 * StationIDは、マスター交代後の新規ステーションにアサインされる値が飛ぶ可能性があり、連続値である保証はありません。
 */
typedef qUnsignedInt32 StationID;

/*!
 @defgroup  nex_define nex_define
 @{

*/

/*!
 * @brief DOHandleで複製オブジェクトのローカルインスタンスを参照していない場合の定義値です。
 *
 */

#define INVALID_DOHANDLE_VALUE 0

/*
Summary: This constant is used for a DOHandle that does not reference a duplicated object.
Remarks: With the use of the SelectionIteratorTemplate you can iterate methods over collections
of duplicated objects with a stop condition. When the stop condition is satisfied,
the SelectionIteratorTemplate method will return the DOHandle of the duplicated object that
the iteration with a stop condition found.
Returns: INVALID_DOHANDLE if the iteration stop condition is never satisfied
throughout the iteration.
See Also:  DOHandle
*/


/*!
 * @brief 複製オブジェクトのローカルインスタンスを参照していない場合のDOHandle定義です。
 *
 */
#define INVALID_DOHANDLE NAMESPACE_NEX DOHandle(INVALID_DOHANDLE_VALUE)


/*!
 @}
 */

#define MAX_DOHANDLE    NAMESPACE_NEX DOHandle((qUnsignedInt32)-1)

// Group = ID Classes

/*
Summary: Refers to a local instance of a duplicated object.
Remarks: The DOHandle is a 32-bit value associated to a given object which is guaranteed to be unique for
each object and valid across all stations. A DOHandle is made up of the object's DOClassID and its DOID.
The DOClassID is generated by the DDL compiler and the DOID may be either generated automatically by
$PRODUCTNAME$ or assigned by the user. As the DOClassID is generated by $PRODUCTNAME$, it and therefore, also
the DOHandle may change when you recompile your project, alter your DDL file, or upgrade to a new version of
$PRODUCTNAME$.

A DOHandle can be passed across the network, either as an action argument or as a dataset variable. For example, process A is
Duplication master for the duplicated object O, while process B contains a Duplica of this object called O'. When a DOHandle on O is
passed from process A to process B, the DOHandle now refers to the local instance of the object O'. Similarly, if B sends a
DOHandle on O' to  process A, the received DOHandle now refers to the Duplication master O.

One important use of a DOHandle is that you can construct a Ref out of it in order to call methods on a given duplicated object.

See Also: DuplicatedObject::GetHandle, RefTemplate, GetDOClassID, GetDOID
*/

/*!
 * @brief   複製オブジェクトのローカルインスタンスを参照する、複製オブジェクトハンドルです。
 *
 * 複製オブジェクトハンドルは32bitの値で、ひとつのオブジェクトに対して、セッションに参加するすべての
 * ステーションの間において唯一であることを保証する値となります。複製オブジェクトハンドルは、
 * 複製オブジェクトクラスID (@ref DOClassID)と複製オブジェクトID(@ref DOID)によって構成されています。
 *
 * @see     DuplicatedObject::GetHandle
 */
class DOHandle: public RootObject
{
    //Group=Methods
public:

    /*
    Description: Creates a DOHandle object.
    Parameters:
    uiValue: The value to which the DOHandle instance will be initialized. The default is INVALID_DOHANDLE_VALUE.
    dohOther: An already existing DOHandle.
    Remarks: If no arguments are passed to the constructor, the DOHandle instance
    will initialize the value of the object to INVALID_DOHANDLE_VALUE. An existing DOHandle
    can be passed to the constructor as a parameter, which will create a DOHandle that
    references the same object as is referenced by the dohOther DOHandle.
    See Also: RefTemplate
    */

    /*!
     * @brief     コンストラクタ。複製オブジェクトハンドルを作成します。
     *
     * 指定された値で複製オブジェクトハンドルのインスタンスを初期化します。
     * もしコンストラクタになにも引数が渡されなければ、
     * 複製オブジェクトハンドルのインスタンスは値をINVALID_DOHANDLE_VALUEとして初期化します。
     *
     * @param[in] uiValue 生成される複製オブジェクトハンドルの初期値を指定します(デフォルトはINVALID_DOHANDLE_VALUE)。
     */
    DOHandle(qUnsignedInt32 uiValue=INVALID_DOHANDLE_VALUE):m_uiValue(uiValue) { }

    /*!
     * @brief     コンストラクタ。複製オブジェクトハンドルを作成します。
     *
     * 指定された@a dohOtherが指し示す複製オブジェクト(@ref DuplicatedObject)と
     * 同一の複製オブジェクトインスタンスを指し示す値として
     * 複製オブジェクトハンドルを初期化します。
     *
     * @param[in] dohOther すでに存在している複製オブジェクトハンドルを指定します。
     */
    DOHandle(const DOHandle& dohOther):RootObject(), m_uiValue(dohOther.m_uiValue) { }

    // The following operators provide an ordering of the DOHandles.

    qBool operator==(const DOHandle& dohOther) const
    {
        return m_uiValue==dohOther.m_uiValue;
    }
    qBool operator!=(const DOHandle& dohOther) const
    {
        return m_uiValue!=dohOther.m_uiValue;
    }
    qBool operator<(const DOHandle& dohOther) const
    {
        return m_uiValue<dohOther.m_uiValue;
    }

    DOHandle& operator=(qUnsignedInt32 uiValue)
    {
        m_uiValue=uiValue;
        return *this;
    }

    DOHandle& operator=(const DOHandle& hDO)
    {
        m_uiValue=hDO.m_uiValue;
        return *this;
    }

    /*
    Description: Returns the value of the DOHandle as a qUnsignedInt.
    Returns: The DOHandle as a qUnsignedInt.
    */

    /*!
     * @brief     複製オブジェクトハンドルを@ref qUnsignedInt32 型として取得します。
     * @return    複製オブジェクトハンドルを@ref qUnsignedInt32 型としたものが返ります。
     */
    qUnsignedInt32 GetValue() const
    {
        return m_uiValue;
    }

    operator qUnsignedInt32() const
    {
        return GetValue();
    }

    /*
    Description: Returns whether or not an object is well-known.
    Returns: true if the duplicated object that is referenced by
    the DOHandle is a well-known object, false if not or an error occurred.
    See Also:  DDL File Syntax, WKHandle
    */

    /*!
     * @brief     このハンドルが指し示す複製オブジェクトが既知オブジェクトであるかを取得します。
     *
     * 既知オブジェクトについての詳細はNEXプログラミングマニュアルを
     * 参照してください。
     *
     * @return    true  ハンドルが示す複製オブジェクトは既知オブジェクトです。
     * @return    false ハンドルが示す複製オブジェクトは既知オブジェクトではないか、エラーが発生しました。
     * @see       WKHandle
     * @see       IsAUserDO
     * @see       IsACoreDO
     */
    qBool IsAWKHandle() const;

    /*
    Description: Returns whether or not the DOHandle refers to a CoreDO or not.
    Returns: true if the DOHandle refers to a core object created by $PRODUCTNAME$, false if it does not.

    $ifdef ETERNA$
    A CoreDO is a DuplicatedObject created by $PRODUCTNAME$ when a Session is created or when
    Station s join or leave the Session. These objects are essential for the correct operation of
    $PRODUCTNAME$ and include the root classes for all global and non-global DuplicatedObject s,
    the Session, SessionClock, Station, and IDGenerator classes, system classes involved
    in fault tolerance, fault recovery, and multicast messaging mechanisms, a default cell class if a
    DuplicationSpace uses Cell s, and the LoadBalancingDomain and other classes involved in load balancing.
    $else$
    A CoreDO is a DuplicatedObject created by $PRODUCTNAME$ when a Session is created or when
    Station s join or leave the Session. These objects are essential for the correct operation of
    $PRODUCTNAME$ and include the root classes for all global and non-global DuplicatedObject s,
    the Session, SessionClock, Station, and IDGenerator classes, system classes involved in fault tolerance
    and fault recovery mechanisms, and a default cell class if DuplicationSpaces are used.
    $endif$

    See Also: IsAUserDO, IsAWKHandle
    */

    /*!
     * @brief     このハンドルが指し示す複製オブジェクトがCoreDOであるかどうかを取得します。
     *
     * Net-Zによって自動的に作成される複製オブジェクトのことをCoreDOと呼びます。
     * 詳しくはNEXプログラミングマニュアルも参照してください。
     *
     * @retval    true  このハンドルがNet-Zによって作られたことを示します。
     * @retval    false このハンドルがNet-Zによって作られていないことを示します。
     * @see       IsAUserDO
     * @see       IsAWKHandle
     */
    qBool IsACoreDO();

    /*
    Description: Returns whether or not the DOHandle refers to a UserDO or not.
    Returns: true if the DOHandle refers to a user-defined object, false if it does not.
    Remarks: A UserDO is a user-defined DuplicatedObject declared by the user in the DDL.
    See Also: IsACoreDO, IsAWKHandle
    */

    /*!
     * @brief     このハンドルが指し示す複製オブジェクトがユーザー定義複製オブジェクトであるかどうかを取得します。
     *
     * ユーザー定義複製オブジェクトとは、ユーザーがDDL(データ定義言語)よって宣言した
     * 複製オブジェクトのことを指します。
     *
     * @retval    true  ユーザー定義複製オブジェクトであるときに返ります。
     * @retval    false ユーザー定義複製オブジェクトでないときに返ります。
     */
    qBool IsAUserDO();

public:

    // Manipulation of the handle details...

    /*
    Description: Returns the DOID of this object.
    Returns: The DOID of this object.
    Remarks: The DOHandle of an object is made up of the object's DOClassID and its DOID.
    See Also: GetDOClassID, DOClassID, DOID
    */

    /*!
     * @brief     この複製オブジェクトハンドルの複製オブジェクトIDを取得します。
     *
     * 複製オブジェクトハンドルは、複製オブジェクトクラスID(@ref DOClassID)と
     * 複製オブジェクトID(@ref DOID)から成り立っています。
     *
     * 複製オブジェクトクラスIDの値はDOCLASSIDマクロを利用することで取得できます。
     *
     * @return    この複製オブジェクトハンドルの複製オブジェクトIDが返ります。
     * @see       GetDOClassID
     */
    DOID GetDOID() const
    {
        return DOID( m_uiValue & OBJECTID_MASK );
    }

    //{secret}
    qUnsignedInt32 GetShiftedClassID() const
    {
        return m_uiValue & CLASSID_MASK;
    }

    /*
    Description: Returns the DOClassID of this object.
    Returns: The DOClassID of this object.
    Remarks: The DOHandle of an object is made up of the object's DOClassID and its DOID.
    See Also: DOClassID, GetDOID, DOID
    */

    /*!
     * @brief     この複製オブジェクトハンドルの複製オブジェクトクラスIDを取得します。
     *
     * 複製オブジェクトハンドルは、複製オブジェクトクラスID(@ref DOClassID)と
     * 複製オブジェクトID(@ref DOID)から成り立っています。
     *
     * 複製オブジェクトクラスIDの値はDOCLASSIDマクロを利用することで取得できます。
     *
     * @return    この複製オブジェクトハンドルの複製オブジェクトクラスIDが返ります。
     * @see       GetDOID
     */
    qUnsignedInt32 GetDOClassID() const
    {
        return GetShiftedClassID()>>OBJECTID_BITS;
    }

    /*
    Description: Returns whether or not an object belongs to a given class.
    Returns: true if the duplicated object that is referenced by
    the DOHandle is an instance of the specified doclass, false if not or an error occurred.
    Parameters:
    idDOCID:  The DOClassID of the duplicated object class.
    See Also:  IsAKindOf, DuplicatedObject::IsA
    */

    /*!
     * @brief     指定された複製オブジェクトクラスIDのクラスに、
     *            このハンドルが示す複製オブジェクトのインスタンスが属するかどうかを調べます。
     *
     * 複製オブジェクトクラスIDの値はDOCLASSIDマクロを利用することで取得できます。
     *
     * @param[in] idDOCID 調べる対象の複製オブジェクトクラスIDを指定します。
     * @see       IsAKindOf
     * @see       DuplicatedObject::IsA
     */
    qBool IsA(DOClassID idDOCID);

    /*
    Description: Returns whether or not an object belongs to or inherits from a given class.
    Returns: true if the duplicated object that is referenced by the DOHandle is an instance of
    or an instance of a child class of the specified doclass, false if not or an error occurred.
    Parameters:
    idDOCID:  The DOClassID of the duplicated object class.
    See Also: IsA, DuplicatedObject::IsAKindOf
    */

    /*!
     * @brief     指定された複製オブジェクトクラスIDのクラスあるいはその派生関係にあるクラスに、
     *            このハンドルが示す複製オブジェクトのインスタンスが属するかどうかを調べます。
     *
     * 複製オブジェクトクラスIDの値はDOCLASSIDマクロを利用することで取得できます。
     *
     * @param[in] idDOCID 調べる対象の複製オブジェクトクラスIDを指定します。
     * @see       IsA
     * @see       DuplicatedObject::IsAKindOf
     */
    qBool IsAKindOf(DOClassID idDOCID);

    /*
    Description: Returns the class name of the DuplicatedObject to which this
    handle refers, as a string.
    Returns: A string that represents the class name of the DuplicatedObject to which this
    handle refers.
    See Also: DuplicatedObject::GetClassNameString
    */

    /*!
     * @brief     このハンドルが示す複製オブジェクトのクラス名を文字列として取得します。
     * @return    複製オブジェクトのクラス名を示す文字列へのポインタが返ります。
     * @see       DuplicatedObject::GetClassNameString
     */
    const qChar* GetClassNameString() const;

    /*
    Description: Returns the class name of the specified dataset associated to this Well-known duplicated object.
    Parameters:
    byDatasetIndex: The index of the dataset.
    Returns: The class name of the dataset associated to this Well-known duplicated object specified by the given index as a string.
    Remarks: This method is useful for obtaining the dataset class name in traces. $PRODUCTNAME$ sequentially assigns a dataset index
    to each dataset associated to a given duplicated object as declared in the DDL file. The index is a positive
    integer starting at one.
    See Also: DuplicatedObject::GetDatasetNameString
    */

    /*!
     * @brief     このハンドルが示す複製オブジェクトの、
     *            指定された番号に対応するデータセットの名称を文字列として取得します。
     *
     * @param[in] byDatasetIndex データセットのインデックス番号を指定します。
     * @return    データセットの名称の文字列へのポインタが返ります。
     * @see       DuplicatedObject::GetdatasetNameString
     */
    const qChar* GetDatasetNameString(qByte byDatasetIndex) const;

    //{secret}
    void SetDOID(DOID uiObjectID)
    {
        SYSTEMCHECK((uiObjectID & OBJECTID_MASK)==uiObjectID);
        m_uiValue=GetShiftedClassID() | uiObjectID;
    }

    //{secret}
    void SetDOClassID(DOClassID uiClassID)
    {
        SYSTEMCHECK(uiClassID<MAX_NB_CLASSID);
        m_uiValue=GetDOID() | (uiClassID << OBJECTID_BITS);
    }

public:
    // Sending and receiving DOHandles.
    //{secret}
    void ExtractFrom(Message *pMessage);
    //{secret}
    void AddSourceTo(Message *pMessage, qBool bCompleteUpdate=false);
    //{secret}
    void SetValue(qUnsignedInt32 uiValue);

private:

    qUnsignedInt32 m_uiValue;
};

// String streaming.
//{secret}
extern StringStream& operator<<(StringStream& oStream, const DOHandle& hValue);

//{secret}
ByteStream& operator<<(ByteStream &refMsg, DOHandle &refDoh);
//{secret}
ByteStream& operator>>(ByteStream &refMsg, DOHandle &refDoh);

END_NAMESPACE_NEX

