﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Core/Scheduler.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>


BEGIN_NAMESPACE_NEX


class Message;
class DuplicatedObject;

// Group=Template Classes

#ifdef NEX_DEBUG_VERSION
class DOSelections;
#endif

//n1589:memo:DORefの操作はSystemLockが必要です
/*
Description: Creates a reference (accessor) to a duplicated object.
Remarks: References are implemented via a template that is detailed under RefTemplate.
See Also: RefTemplate
*/

/*!
 * @brief   複製オブジェクトの安全な参照(アクセサ)を作成します。
 *
 * 複製オブジェクトへの参照は、@ref RefTemplate によって詳細な実装が行われます。
 * @ref RefTemplate は薄いテンプレートクラスの構造になっており、
 * 共通部分はこのDORefによって実装されます。
 */
class DORef: public RootObject
{
    // Group=Methods
public:

    /*
    Description: Creates a reference to a duplicated object.
    Parameters:
    hDO: A DOHandle that refers to the duplicated object to which a
    reference is wanted.
    oRef: A reference to a duplicated object.
    pDO: A pointer to a duplicated object.
    Remarks: The method IsValid should be used to check if the reference was correctly constructed.
    See Also: Duplicated Object Overview, RefTemplate
    */

    /*!
     * @brief     無効な参照として初期化します。
     */
    DORef();
    /*!
     * @brief     複製オブジェクトハンドルを利用して初期化します。
     *
     * 複製オブジェクトハンドル(@ref DOHandle)は、セッション中に存在する複製オブジェクトを
     * 一意に表すことが出来るハンドルです。このコンストラクタでは複製オブジェクトハンドルから
     * 複製オブジェクトへの安全な参照を作成します。
     *
     * 正しく複製オブジェクトのハンドルの参照が構築できた場合、
     * @ref IsValid() がtrueを返します。
     *
     * @param[in] hDO 参照対象の複製オブジェクトのハンドルを指定します。
     */
    DORef(DOHandle hDO);
    /*!
     * @brief     コピーコンストラクタです。
     *
     * 指定した参照の指す複製オブジェクトを参照して初期化します。
     *
     * 正しく複製オブジェクトのハンドルの参照が構築できた場合、
     * @ref IsValid() がtrueを返します。
     *
     * @param[in] oRef 参照のコピー元を指定します。
     */
    DORef(const DORef& oRef);
    /*!
     * @brief     複製オブジェクトへのポインタを指定して初期化します。
     *
     * 指定された複製オブジェクトへのポインタを参照するように初期化します。
     *
     * 正しく複製オブジェクトのハンドルの参照が構築できた場合、
     * @ref IsValid() がtrueを返します。
     *
     * @param[in] pDO 参照先の複製オブジェクトへのポインタを指定します。
     */
    DORef(DuplicatedObject* pDO);

    virtual ~DORef();

    // {secret}
    void EmptyInit();

    DORef& operator=(const DOHandle& hDO);
    DORef& operator=(const DORef& oRef);
    DORef& operator=(DuplicatedObject* pDO);

    // Description: Returns the DOHandle used to construct this reference.
    // Returns: The DOHandle used to construct this reference.
    // See Also: Duplicated Object Overview, RefTemplate
    /*!
     * @brief     この参照が示す複製オブジェクトのハンドルを取得します。
     * @return    この参照が示す複製オブジェクトハンドルが返ります。
     */
    DOHandle GetHandle() const
    {
        return m_hReferencedDO;
    }

    // {secret}
    qUnsignedInt32 GetHandleValue() const
    {
        return m_hReferencedDO.GetValue();
    }

    /*
    Description: Returns whether or not this DORef is a Soft reference to a duplicated object.
    Returns: true if this DORef is a Soft reference to a duplicated object, false if it is a Hard reference or
    an error occurred.
    Remarks: A reference to a duplicated object may be either a Hard reference, which is the default, or a
    Soft reference. Refer to SetSoft for further information about hard and soft references.
    See Also: IsHard, SetSoft, SetHard, Soft reference
    */

    /*!
    @brief このDORefが、複製オブジェクトへの「弱い」参照であるかどうかを返します。

    複製オブジェクトへの参照は、「弱い」参照と「強い」参照の2種類があります（デフォルトは「強い」参照）。
    参照の強弱についての詳細は、 @ref SetSoft をご覧下さい。

    @see   IsHard, SetSoft, SetHard
    @return 「弱い」参照であればtrue。「強い」参照であったり、エラーが発生した場合にはfalse。
     */
    qBool IsSoft() const
    {
        return !m_bLockRelevance;
    }

    /*
    Description: Returns whether or not this DORef is a Hard reference to a duplicated object.
    Returns: true if this DORef is a Hard reference to a duplicated object, false if it is a Soft reference or
    an error occurred.
    Remarks: A reference to a duplicated object may be either a Hard reference, which is the default, or a
    Soft reference. Refer to SetHard for further information about hard and soft references.
    See Also: IsSoft, SetSoft, SetHard, Hard reference
    */

    /*!
    @brief このDORefが、複製オブジェクトへの「強い」参照であるかどうかを返します。

    複製オブジェクトへの参照は、「弱い」参照と「強い」参照の2種類があります（デフォルトは「強い」参照）。
    参照の強弱についての詳細は、 @ref SetSoft をご覧下さい。
    @see   IsSoft, SetSoft, SetHard
    @return 「強い」参照であればtrue。「弱い」参照であったり、エラーが発生した場合にはfalse。
     */
    qBool IsHard() const
    {
        return m_bLockRelevance;
    }

    /*
    Description: Sets this DORef to a Soft reference.
    Remarks: A reference to a duplicated object may be either a Hard reference, which is the default, or a
    Soft reference. A given reference may be changed from hard to soft and vice versa during run time.
    A Hard reference ensures that an object stays relevant and therefore, kept up-to-date on the local
    Station as long as DuplicatedObject::DeleteMainRef has not been called on the object. This sort of
    reference is generally used when you want to fetch an object (via DuplicatedObject::FetchDuplica) that
    is not currently relevant on the local Station. In contrast, a Soft reference has no effect on an object's
    Relevance and therefore, a duplica will only be updated if the object is considered relevant due to a
    Duplication space match condition. The choice of using a hard or soft reference depends on whether or
    not you wish to keep the referenced object updated.

    For example, if you want to keep a list of your teammates and their current status (position,
    inventory, /etc./) even when they are in a different part of the virtual world to your avatar,
    you will want to keep Hard reference s to their avatars so that the avatar duplicas will always
    be updated. On the other hand, if you wish to keep a list of all the objects of a given type,
    say enemy avatars, that exist in the world, you will probably want to use Soft reference s
    so you don't update these objects unecessarily on your station. When a Soft reference is used, a given
    object will only be updated when it becomes relevant on the station, due to a Duplication space match condition.
    See Also: IsHard, IsSoft, SetHard, Soft reference
    */

    /*!
    @brief DORefを「弱い」参照に設定します。

    複製オブジェクトへの参照は、「弱い」参照と「強い」参照の2種類があります（デフォルトは「強い」参照）。
    この参照は、実行時に強い参照から弱い参照に変更することもできますし、その逆もまた可能です。
    強い参照は、オブジェクトが常に関連していることを保証し、DuplicatedObject::DeleteMainRefが
    呼び出されない限り、ローカルステーションにて最新の状態を保ち続けます。
    この参照の使い分けは、現時点ではローカルステーションに関連性の無かったオブジェクトを取ってくる際
    (DuplicatedObject::FetchDuplica)によく使われます。
    対照的に、弱い参照はオブジェクトの関連性に影響を及ぼさないので、デュプリカは複製スペース機能によって
    関連性があると判断されたときにのみ更新されます。
    強い参照と弱い参照の選択は、参照されたオブジェクトの状態を最新のものにキープしておきたいかどうかによって
    決まります。

    例を挙げます。仮想世界において、プレイヤーとは別世界にいるチームメイトの状態（位置、所持品など）の
    リストを最新の状態にキープしておきたいならば、それらのオブジェクト（デュプリカ）が常に最新の状態に
    更新されるようにするため、強い参照を使うでしょう。
    一方、敵キャラクターのようなケースでは、ローカルステーションで無駄に状態の更新はしないでしょうから、
    おそらく弱い参照を使うでしょう。
    弱い参照を使う場合は、そのオブジェクトはローカルステーションに関連が出てくるようになってから
    （複製スペースの条件にマッチしたら）更新の対象となります。

    @see IsHard, IsSoft, SetHard
     */
    void SetSoft();

    /*
    Description: Sets this DORef to a Hard reference.
    Remarks: A reference to a duplicated object may be either a Hard reference, which is the default, or a
    Soft reference. A given reference may be changed from hard to soft and vice versa during run time.
    A Hard reference ensures that an object stays relevant and, therefore, kept up-to-date on the local
    Station as long as DuplicatedObject::DeleteMainRef has not been called on the object. This sort of
    reference is generally used when you want to fetch an object (via DuplicatedObject::FetchDuplica) that
    is not currently relevant on the local Station. In contrast, a Soft reference has no effect on an object's
    Relevance and therefore, a duplica will only be updated if the object is considered relevant due to a
    Duplication space match condition. The choice of using a hard or soft reference depends on whether or
    not you wish to keep the referenced object updated.

    For example, if you want to keep a list of your
    teammates and their current status (position, inventory, /etc./) even when they are in a different part
    of the virtual world to your avatar, you will want to keep Hard reference s to their avatars so that
    the avatar duplicas will always be updated. On the other hand, if you wish to keep a list of all the objects
    of a given type, say enemy avatars, that exist in the world, you will probably want to use Soft reference s
    so you don't update these objects unecessarily on your station. When a Soft reference is used, a given
    object will only be updated when it becomes relevant on the station, due to a Duplication space match condition.
    See Also: IsHard, IsSoft, SetSoft, Hard reference
    */

    /*!
    @brief DORefを「強い」参照に設定します。

    複製オブジェクトへの参照は、「弱い」参照と「強い」参照の2種類があります（デフォルトは「強い」参照）。
    この参照は、実行時に強い参照から弱い参照に変更することもできますし、その逆もまた可能です。
    強い参照は、オブジェクトが常に関連していることを保証し、DuplicatedObject::DeleteMainRefが
    呼び出されない限り、ローカルステーションにて最新の状態を保ち続けます。
    この参照の使い分けは、現時点ではローカルステーションに関連性の無かったオブジェクトを取ってくる際
    (DuplicatedObject::FetchDuplica)によく使われます。
    対照的に、弱い参照はオブジェクトの関連性に影響を及ぼさないので、デュプリカは複製スペース機能によって
    関連性があると判断されたときにのみ更新されます。
    強い参照と弱い参照の選択は、参照されたオブジェクトの状態を最新のものにキープしておきたいかどうかによって
    決まります。

    例を挙げます。仮想世界において、プレイヤーとは別世界にいるチームメイトの状態（位置、所持品など）の
    リストを最新の状態にキープしておきたいならば、それらのオブジェクト（デュプリカ）が常に最新の状態に
    更新されるようにするため、強い参照を使うでしょう。
    一方、敵キャラクターのようなケースでは、ローカルステーションで無駄に状態の更新はしないでしょうから、
    おそらく弱い参照を使うでしょう。
    弱い参照を使う場合は、そのオブジェクトはローカルステーションに関連が出てくるようになってから
    （複製スペースの条件にマッチしたら）更新の対象となります。

    @see IsHard, IsSoft, SetSoft
     */
    void SetHard();

    /*!
     * @brief     この参照が示す複製オブジェクトのポインタを取得します。
     * @return    この参照が示す複製オブジェクトのポインタが返ります。参照がない場合は、NULLが返ります。
     */
    DuplicatedObject* GetDOPtr() const
    {
        return m_poReferencedDO;
    }

    /*
    Summary: Returns whether or not the DORef instance is valid.
    Returns: true if the reference construction is valid, false if not or an error occurred.
    Remarks: This method will return false if you pass a reference or a DOHandle to an invalid or
    unknown object or to an object of the wrong type. The possible errors that will be returned
    are SYSTEMERROR_OBJDUP_DO_NOT_FOUND and SYSTEMERROR_GEN_INVALID_OPERATION. If you create a
    reference by specifying a DOHandle to the object, the error SYSTEMERROR_OBJDUP_MISMATCHED_DOCLASS
    is also possible.
    See Also: Duplicated Object Overview, RefTemplate
    */

    /*!
     * @brief   DORefのインスタンスが有効であるかどうかを取得します。
     *
     * この関数は、コンストラクタにて無効なオブジェクトに対する@ref DOHandle 、不明なオブジェクトに対する
     * @ref DOHandle 、異なるオブジェクトに対する@ref DOHandle が渡されたときにfalseを返します。
     * 失敗したときに、SYSTEMERROR_OBJDUP_DO_NOT_FOUNDか、SYSTEMERROR_GEN_INVALID_OPERATIONのエラーを
     * シグナルすることがあります。もしもオブジェクトにへの@ref DOHandle を明示して参照を作っていた場合、
     * SYSTEMERROR_OBJDUP_MISMATCHED_DOCLASSのエラーをシグナルすることもあります。
     *
     * @retval  true    有効であるときに返ります。
     * @retval  false   無効であるかエラーが発生したときに返ります。
     */
    qBool IsValid() const
    {
        return GetDOPtr()!=NULL;
    }

    //Description: Tests whether the reference is valid.
    //
    //Returns: true if the DORef is now valid, false if not or an error occurred.
    //See Also: Wait, IsValid
    /*!
     * @brief   DORefの参照が有効かどうかを試行します。
     * @retval  true    DORefが現在有効であれば返ります。
     * @retval  false   DORefが現在有効でないか、あるいはエラーが発生したときに返ります。
     * @see     Wait, Valid
     */
    qBool Poll()
    {
        if (m_poReferencedDO==NULL)
        {
            Acquire();
        }
        return IsValid();
    }


    /*
    Description: Cause the system to wait until the DORef is valid or the timeout is exceeded.
    Parameters:
    uiTimeout: The maximum time in milliseconds that the system will wait. The default uiTimeout is infinite.
    Returns: true if the DORef is valid, false if not or the timeout is exceeded.
    Remarks: The error SYSTEMERROR_GEN_TIMEOUT will be signalled if the timeout is exceeded.
    See Also: Poll
    */

    /*!
     * @brief     DORefが有効になるか、タイムアウトが経過するまで待ちます。
     * @param[in] uiTimeout 待ち続ける最大の時間をミリ秒単位で指定します。
     *                      デフォルトでは無限に待つ(タイムアウトを設けない)となります。
     * @retval    true      待った結果、DORefが有効になったときに返ります。
     * @retval    false     タイムアウトが経過したときに返ります。
     */
    qBool Wait(qUnsignedInt32 uiTimeout=WAIT_INFINITE_TIMEOUT);

public:
    // TODO: Should be protected.

    // {secret}
    void Release();


#ifdef DEBUG_REFERENCES
    /*
    Description: Traces all the references to a given duplicated object.
    Parameters:
    uiTraceFlag: The trace flag used when tracing. To ensure tracing always takes
    place TRACE_ALWAYS can be used.
    hDO: A handle to the duplicated object for which the references will be traced.
    Remarks: In the debug version, to help debug reference leaks, the name of a DORef can be traced
    via this method. For each reference, the name specified via NameReference will be traced.
    In the release version, this method has no effect.
    See Also: NameReference
    */
    static void TraceReferences(qUnsignedInt64 uiTraceFlag, DOHandle hDO);
    // {secret}
    static void TraceAllReferences(qUnsignedInt64 uiTraceFlag);
    //{secret}
    static qUnsignedInt32 CountReferences(DOHandle hDO);
    //{secret}
    static void TraceReferencesImpl(qUnsignedInt64 uiTraceFlag, DOHandle hDO);
    //{secret}
    static qUnsignedInt32 CountReferencesImpl(DOHandle hDO);

    /*
    Description: Sets the name of a reference.
    Remarks: In the debug version, to help debug reference leaks, the name of a DORef can be traced
    via TraceReferences. In the release version, this method has no effect.
    See Also: TraceReferences
    */
    void NameReference(const qChar* szReferenceName)
    {
        m_szReferenceName = szReferenceName;
    }
#else
    //{secret}
    void NameReference(const qChar* /*szReferenceName*/) {}
    //{secret}
    static void TraceReferences(qUnsignedInt64 /*uiTraceFlag*/, DOHandle /*hDO*/) {}
#endif

protected:
    void SetPointer(DuplicatedObject* pDO);
    void ReleasePointer();
    void Acquire();

private:
    inline void AcquireImpl();

#ifdef DEBUG_REFERENCES
    const qChar* m_szReferenceName;
    static ForcedCriticalSection s_debugCS;
#endif

    DuplicatedObject* m_poReferencedDO;
    DOHandle m_hReferencedDO;

    qBool m_bLockRelevance;

#ifdef NEX_DEBUG_VERSION
    DOSelections* m_poStore;
#endif
};

END_NAMESPACE_NEX

