﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Core/Operation.h>
#include <OnlineCore/src/Core/PseudoSingleton.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Platform/Core/RootObject.h>
#include <OnlineCore/src/Transport/Interface/SubStreamID.h>

BEGIN_NAMESPACE_NEX

class Counter;
class DuplicatedObject;

#ifdef NEX_PERFCOUNTER_ENABLED
class DataSetPerfCounters;
#endif // NEX_PERFCOUNTER_ENABLED

class Station;
class DOOperation;



// Group=Miscellaneous Types and Macros
/*!
 * @brief   複製オブジェクトの推測航法のデータセットの更新遅延を示す列挙子です。
 */
enum DATASET_UPDATE_DELAY
{
    /*
    Summary: Dataset - Indicates that there are no maximum update delays.
    See Also: DataSet::SetMaximumUpdateDelay, DataSet::GetMaximumUpdateDelay, DEFAULT_MAXIMUM_UPDATE_DELAY
    */
    NO_MAXIMUM_UPDATE_DELAY=0, //!< 無効な最大更新遅延を示します。

    /*
    Summary: Dataset - Defines the default value for the maximum update delay.
    Remarks: The maximum update delay is the maximum time allowed between the sending of updates
    of a dataset to remote stations. The default value is 2000 ms.
    See Also: DataSet::SetMaximumUpdateDelay, DataSet::GetMaximumUpdateDelay, NO_MAXIMUM_UPDATE_DELAY
    */
    DEFAULT_MAXIMUM_UPDATE_DELAY=2000, //!< 最大更新遅延のデフォルト値です。2000です。

    /*
    Summary: Dataset - Indicates that there are no minimum update delays.
    See Also: DataSet::SetMinimumUpdateDelay, DataSet::GetMinimumUpdateDelay, DEFAULT_MINIMUM_UPDATE_DELAY
    */
    NO_MINIMUM_UPDATE_DELAY=0, //!< 無効な最小更新遅延を示します。

    /*
    Summary: Dataset - Defines the default value for the minimum update delay.
    Remarks: The minimum update delay is the minimum time allowed between the sending of updates
    of a dataset to remote stations. The default value is 66 ms, which is equivalent to approximately
    6 updates/second.
    See Also: DataSet::SetMinimumUpdateDelay, DataSet::GetMinimumUpdateDelay, NO_MINIMUM_UPDATE_DELAY
    */
    DEFAULT_MINIMUM_UPDATE_DELAY=66 //!< 最小更新遅延のデフォルト値です。66です。
};


/*!
 @defgroup  nex_define nex_define
 @{

*/


//group=DDL Output

/*
Description: Dataset macro. Generates the name of the base class from
which your dataset implementation inherits.
Remarks: Generates the name of the base class from which the dataset implementation inherits.

For example, lets say a dataset is declared in a DDL file as follows :

      dataset Position {
          real x;
          real y;
          real z;
      };

Then the user-defined /Position/ class is declared as follows :

      class Position : public  DATASET(Position) {
          void Set( qReal x, qReal y, qReal z );
      };

Note that the /Set/ method is part of the user implementation.
*/

/*!
 * @brief ベースのデータセットクラスを参照するときに用いるマクロです。
 *
 * DDLが以下のように定義されているときに、<br>
dataset Position {<br>
real x;<br>
real y;<br>
real z;<br>
};<br>
 * アプリケーションでは、Positionクラスを以下のように定義します。<br>
 *
class Position : public  DATASET(Position) {<br>
    void Set( qReal x, qReal y, qReal z );<br>
};<br>
 *
 * @param C クラス名
 */
#define DATASET(C) _DS_##C

/*!
 *
 * @brief データセットの値が、データが生成されたDDLのリビジョン番号で有効な値か判定するマクロです。
 *
 * 戻り値として、生成されたリビジョン番号で有効な値か否かをqBoolで返します。
 *
 * @param[in] VAR 取得する値の名前を指定します。
 */
#define DATASET_ISVALID(VAR) _ds_isValid_ ## VAR()

/*!
 *
 * @brief データセットの値が、どのDDLのリビジョン番号で有効か取得するマクロです。
 *
 * 戻り値として、リビジョン番号をqByteで返します。
 *
 * @param[in] VAR 取得する値の名前を指定します。
 */

#define DATASET_VALID_REVISION(VAR) _ds_validRevision_ ## VAR

#define DATASET_VALID_REVISION_DEF(VAR, VAL) enum { _ds_validRevision_ ## VAR = VAL };

// Group = Dead Reckoning

/*
Description: Accesses a dataset extrapolation variable on a Duplica.
Parameters:
DATASET: A reference to the dataset containing the variable.
VAR: The variable itself.
Returns: The error SYSTEMERROR_GEN_INVALID_OPERATION is signalled
if the macro is called on a dataset that is not extrapolated.
Remarks:
For example, if we declare the following in the DDL:

        dataset Position {
            qReal x;
            qReal y;
            qReal z;
        } extrapolation_filter, unreliable;

        dataset Posture {
            qReal v[3];
        } extrapolation_filter, unreliable;

        doclass Avatar {
            Position m_Pos;
            Posture m_Posture;
        };

Then an object of the Avatar class may use the extrapolation variables to predict
its Position or its Posture 100ms in the future via the following:

        qReal Avatar::PredictX(TimeInterval tiInterval) {
            return m_Pos.PredictValue(MODEL(m_Pos, x),GetTimestamp()+tiInterval);
        }

        void Avatar::PredictPosture(qReal dv0, qReal dv1, qReal dv2) {
            // Predicts the posture in 1 second.
            dv0=m_Posture.PredictValue(MODEL(m_Posture,v[0]),GetTimestamp()+100);
            dv1=m_Posture.PredictValue(MODEL(m_Posture,v[1]),GetTimestamp()+100);
            dv2=m_Posture.PredictValue(MODEL(m_Posture,v[2]),GetTimestamp()+100);
        }

Note: This macro is only generated for a dataset that includes the *extrapolation_filter*
DDL property in its DDL declarations.

See also: DataSet::SetModel, DataSet::PredictValue, DataSet::PredictRateOfChange,
DataSet::PredictAcceleration
*/

/*!
 *
 * @brief デュプリカのdataset推定値にアクセスするマクロです。
 * @param[in] DATASET データセット名を指定します。
 * @param[in] VAR 取得する値の名前尾を指定します。
 */
#define MODEL(DATASET, VAR) (DATASET).GetPHBDRModel()->VAR

/*
Description: Accesses a dataset extrapolation variable on a Duplication master
that uses Loopback.
Parameters:
DATASET: A reference to the dataset containing the variable.
VAR: The variable itself.
Returns: The error SYSTEMERROR_GEN_INVALID_OPERATION is signalled
if the macro is called on a dataset that is not extrapolated.
Remarks:
For example, if we declare the following in the DDL:

        dataset ExtrapolatedPosition {
            float m_fX;
            float m_fY;
            float m_fZ;
        } extrapolation_filter, loopback, unreliable;

        doclass Avatar {
            ExtrapolatedPosition m_dsPosition;
        };

Then an object of the Avatar class may use the extrapolation
variables to extrapolate its position in the past via the following:

        Vector3D Duplicated3DObject::GetLoopbackPosition() {
            return Vector3D(m_dsPosition.PredictLoopbackValue(LOOPBACKMODEL(m_dsPosition,m_fX)),
                            m_dsPosition.PredictLoopbackValue(LOOPBACKMODEL(m_dsPosition,m_fY)),
                            m_dsPosition.PredictLoopbackValue(LOOPBACKMODEL(m_dsPosition,m_fZ)));
        }

For a complete example of how to use this macro refer to the /Duplicated3DObject.cpp/ file
of the Real-time 3D framework included with the $PRODUCTNAME$ distribution.

Note: This macro is only generatedd for a dataset that includes the *extrapolation_filter* and
the *loopback* DDL property in its DDL declarations.

See Also: DataSet::HasLoopback
*/

/*! @brief DOマスターのdataset推定値にアクセスするマクロです。
 * @param[in] DATASET データセット名を指定します。
 * @param[in] VAR 取得する値の名前尾を指定します。
 */
#define LOOPBACKMODEL(DATASET, VAR) (DATASET).GetLoopbackModel()->VAR

/*!
 @}
 */

// Group=DDL Output

/*
Description: Describes duplicated object data.
Remarks:  A DataSet is defined in the DDL and groups several dataset variables, all of which share a common
update policy. This class is at the root of the game specific DataSet class hierarchy that is generated
according to the DDL declarations. When the method DuplicatedObject::Update is called on a Duplication master,
the data contained in all of the datasets of its duplicas will be updated, according to the update policy specified
in the DDL.

It should be noted that the majority of the methods of this class are only generated when a given
DDL property is included in the dataset's DDL declarations. Therefore, if a given method is called and
its associated DDL property has not been declared, a compiler error will occur. For example, all the methods
listed below under "Extrapolated Dataset Specific Methods" can only be called on a dataset that declared an
extrapolation_filter in its update policy. Similarly, GetPerformanceCounters will only function for a dataset
that declared the update_perf_counter DDL property. Refer to the notes included with each method as to
the restrictions that apply to the given method.

If you wish to implement a cheat detection system on datasets, which verifies that a given variable does not change
by more than an allowable amount or remains within certain thresholds, then you can use the OperationBegin and
OperationEnd methods to do so. In general, in OperationBegin you would retrieve the current dataset value and then
in OperationEnd verify that the new value of the dataset is acceptable. In addition, if the dataset uses an extrapolation
filter you can easily calculate the rate of change of a variable by looking at the stored extrapolation points.
See also: DDL File Syntax, Class Generation
*/

/*!
 * @brief   複製オブジェクトのデータセットを表します。
 *
 * DataSetはDDL(データ定義言語)によって定義される、いくつかの共通の更新ポリシーを持つ値のグループです。
 * C++プログラミング言語における構造体のような存在と考えることが出来ます。
 * このクラスは、DDL宣言によって生成されるアプリケーション固有のデータセットクラス階層構造の基底クラスとなります。
 * 複製マスター(@ref DuplicatedObject::IsADuplicationMaster())にて@ref DuplicatedObject::Update() が呼び出されるとき、
 * その複製オブジェクトのデュプリカに含まれるすべてのデータセットが、DLLに定義された
 * 更新ポリシーにしたがって更新されます。
 *
 * データセットのDDL宣言に与えられたDDLプロパティが含まれるときに、
 * この関数の多くの関数が生成されることに注意してください。
 * したがって、ここに載っている関数を呼び出したとしても、それに関連するDDLプロパティが
 * DDLファイルに宣言されていないとき、コンパイルエラーとなってしまいます。
 * たとえば、extrapolation_filterというDDLプロパティが宣言されていない場合は、
 * @ref SetModel() 関数は生成されないので利用することが出来ません。
 *
 * もしもデータセットに与えられた、変化可能な最大数に基づいて事前チェックを行うような
 * チート行為検出機能を実装したいと考えているのであれば、@ref OperationBegin() と
 * @ref OperationEnd() を利用してください。たいていの場合、@ref OperationBegin() にて、
 * 現在のデータセットの値を読み出して保持しておき、@ref OperationEnd() にて新しい値を見て、
 * それが受け入れられるかを判定してください。加えて、もしデータセットが補完機能を利用すれば、
 * 値の変化の割合を保存されている補完点から簡単に計算することが出来ます。
 */
class DataSet: public RootObject
{
public:

#ifdef FOR_DOC_PURPOSE

    /*
    {Group:Dataset Extrapolation Methods}

    Description: Sets a new extrapolation model via the Duplication master.
    Parameters:
    v: Dataset extrapolation variable for which the extrapolation model is
    changed.  This value is obtained using the MODEL macro.
    dValue: Value of the variable, at time tTimestamp.
    dRateOfChange: Rate of change of the variable, in units per millisecond, at time tTimestamp.
    dAcceleration: Acceleration of the variable, in units per millisecond^2, at time tTimestamp.
    tTimestamp: (optional) Specifies the point in (absolute) time at which the
    model is valid. Otherwise, the current time as given by the SessionClock is
    used.
    Returns: The error SYSTEMERROR_GEN_INVALID_OPERATION is signalled when this method is
    called on a dataset that has not been extrapolated.
    SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA is signalled if the method is called on a Duplica.
    Remarks: This method is called on a Duplication master when a sudden change occurs to a dataset to provide $PRODUCTNAME$
    with additional information about the sudden change.  It is then used in conjunction with the continuity
    break flag CONTINUITY_BREAK_UPDATED_MODEL. The method may be used to temporarily correct an inaccurate extrapolation.

    For example, when a ball hits a wall, the duplicas' network latency extrapolation on the ball's position may lead to the
    ball being displayed past the wall. The SetModel method can be used to temporarily adjust the model
    on the duplicas, taking into account the effect of the collision.  For example, if you used IndicateContinuityBreak
    to signal a CONTINUITY_BREAK_TELEPORT, then the dupica will displace itself to the new position
    immediately (without interpolation) but
    will remain stopped until it receives the next three updates, which can last 2 frames.
    You can recover the 2 frames and ensure that the Duplica will move correctly
    with the right velocity and acceleration immediately after the teleport by using SetModel.

    Refer to CONTINUITY_BREAK_UPDATED_MODEL for an example of this method in use.

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter*
    DDL property in its DDL declarations.

    See also: MODEL, IndicateContinuityBreak
    */

    /*!
     * @brief   複製マスターを通して新しい補完モデルをセットします。
     *
     * ※この関数は、DataSetクラスが提供する関数ではありません。
     * DDL(データ定義言語)宣言に、このデータセットがextrapolation_filterプロパティを持つように
     * 指示されたときのみ、DDLコンパイラがこの関数のためのコードを生成します。
     *
     * この関数は、データセットの値が突然変化したときに複製マスターによって呼び出され、
     * システムに対し突然の変更があったということを通知します。
     * 連続性が打ち切られたことを示すCONTINUITY_BREAK_UPDATED_MODELフラグとともに使われます。
     * この関数は、誤った補完情報を一時的に訂正するために利用されます。
     *
     * ゲーム中のボールの位置について推定航法(パケットのレイテンシなどを考慮して値を補完する機能)が有効である場合、
     * ボールが壁にぶつかって跳ね返ると、デュプリカではボールが壁の奥にめり込んで表示されてしまうようなことが起こります。
     * SetModel()は、一時的にデュプリカのモデルを調整して、当たり判定への影響に対する計算を行います。
     * たとえば、もしもCONTINUITY_BREAK_TELEPORTをシグナルするために@ref IndicateContinuityBreak() を使っていて、
     * デュプリカはそれ自身の居場所を失って新しい位置へ(補完なしに)即座に移動してしまいます。
     * しかし、ボールは次の3回の更新を受け取るまで(たいていのゲームの場合2フレーム)の間は同じ位置に残り続けることになります。
     * SetModel()によって、この2フレームを回復し、デュプリカが瞬間移動後に正しい速度と加速度を即座に与えることが出来ます。
     *
     * この関数を利用すると以下のエラーがシグナルされることがあります。
     *
     * @li 補完機能が有効になっていないデータセットでこの関数が呼び出された場合、SYSTEMERROR_GEN_INVALID_OPERATIONがシグナルされます。
     * @li デュプリカ(@ref DuplicatedObject::IsADuplica())がこの関数を呼び出した場合、SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICAがシグナルされます。
     *
     * この機能について詳しくはNEXプログラミングマニュアルも参照してください。
     *
     * @param[in]   v               データセットの変数の中で、補完モデルを変更するものを指定します。MODELマクロを使って指定してください。
     * @param[in]   dValue          現在のセッション時間における変数の値を指定します。
     * @param[in]   dRateOfChange   現在のセッション時間における変数の変化の割合をミリ秒単位で指定します。
     * @param[in]   dAcceleration   現在のセッション時間における変数の加速度を、ミリ秒の二乗の単位で指定します。
     * @param[in]   tTimestamp      現在のセッション時間を指定します(デフォルト引数として指定されます)。
     *
     * @see     IndicateContinuityBreak
     */
    void SetModel(PHBDRVar_qReal& v, qReal dValue, qReal dRateOfChange, qReal dAcceleration,
                  Time tTimestamp=GetTimestamp())
    {
        SYSTEMCHECK(false);
    }

    /*!
     * @brief   複製マスターを通して新しい補完モデルをセットします。
     *
     * ※この関数は、DataSetクラスが提供する関数ではありません。
     * DDL(データ定義言語)宣言に、このデータセットがextrapolation_filterプロパティを持つように
     * 指示されたときのみ、DDLコンパイラがこの関数のためのコードを生成します。
     *
     * この関数は、データセットの値が突然変化したときに複製マスターによって呼び出され、
     * システムに対し突然の変更があったということを通知します。
     * 連続性が打ち切られたことを示すCONTINUITY_BREAK_UPDATED_MODELフラグとともに使われます。
     * この関数は、誤った補完情報を一時的に訂正するために利用されます。
     *
     * ゲーム中のボールの位置について推定航法(パケットのレイテンシなどを考慮して値を補完する機能)が有効である場合、
     * ボールが壁にぶつかって跳ね返ると、デュプリカではボールが壁の奥にめり込んで表示されてしまうようなことが起こります。
     * SetModel()は、一時的にデュプリカのモデルを調整して、当たり判定への影響に対する計算を行います。
     * たとえば、もしもCONTINUITY_BREAK_TELEPORTをシグナルするために@ref IndicateContinuityBreak() を使っていて、
     * デュプリカはそれ自身の居場所を失って新しい位置へ(補完なしに)即座に移動してしまいます。
     * しかし、ボールは次の3回の更新を受け取るまで(たいていのゲームの場合2フレーム)の間は同じ位置に残り続けることになります。
     * SetModel()によって、この2フレームを回復し、デュプリカが瞬間移動後に正しい速度と加速度を即座に与えることが出来ます。
     *
     * この関数を利用すると以下のエラーがシグナルされることがあります。
     *
     * @li 補完機能が有効になっていないデータセットでこの関数が呼び出された場合、SYSTEMERROR_GEN_INVALID_OPERATIONがシグナルされます。
     * @li デュプリカ(@ref DuplicatedObject::IsADuplica())がこの関数を呼び出した場合、SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICAがシグナルされます。
     *
     * この機能について詳しくはNEXプログラミングマニュアルも参照してください。
     *
     * @param[in]   v               データセットの変数の中で、補完モデルを変更するものを指定します。MODELマクロを使って指定してください。
     * @param[in]   dValue          現在のセッション時間における変数の値を指定します。
     * @param[in]   dRateOfChange   現在のセッション時間における変数の変化の割合をミリ秒単位で指定します。
     * @param[in]   dAcceleration   現在のセッション時間における変数の加速度を、ミリ秒の二乗の単位で指定します。
     * @param[in]   tTimestamp      現在のセッション時間を指定します(デフォルト引数として指定されます)。
     *
     * @see     IndicateContinuityBreak
     */
    void SetModel(PHBDRVar_qFoat& v, qReal dValue, qReal dRateOfChange, qReal dAcceleration,
                  Time tTimestamp=GetTimestamp())
    {
        SYSTEMCHECK(false);
    }

    /*!
     * @brief   複製マスターを通して新しい補完モデルをセットします。
     *
     * ※この関数は、DataSetクラスが提供する関数ではありません。
     * DDL(データ定義言語)宣言に、このデータセットがextrapolation_filterプロパティを持つように
     * 指示されたときのみ、DDLコンパイラがこの関数のためのコードを生成します。
     *
     * この関数は、データセットの値が突然変化したときに複製マスターによって呼び出され、
     * システムに対し突然の変更があったということを通知します。
     * 連続性が打ち切られたことを示すCONTINUITY_BREAK_UPDATED_MODELフラグとともに使われます。
     * この関数は、誤った補完情報を一時的に訂正するために利用されます。
     *
     * ゲーム中のボールの位置について推定航法(パケットのレイテンシなどを考慮して値を補完する機能)が有効である場合、
     * ボールが壁にぶつかって跳ね返ると、デュプリカではボールが壁の奥にめり込んで表示されてしまうようなことが起こります。
     * SetModel()は、一時的にデュプリカのモデルを調整して、当たり判定への影響に対する計算を行います。
     * たとえば、もしもCONTINUITY_BREAK_TELEPORTをシグナルするために@ref IndicateContinuityBreak() を使っていて、
     * デュプリカはそれ自身の居場所を失って新しい位置へ(補完なしに)即座に移動してしまいます。
     * しかし、ボールは次の3回の更新を受け取るまで(たいていのゲームの場合2フレーム)の間は同じ位置に残り続けることになります。
     * SetModel()によって、この2フレームを回復し、デュプリカが瞬間移動後に正しい速度と加速度を即座に与えることが出来ます。
     *
     * この関数を利用すると以下のエラーがシグナルされることがあります。
     *
     * @li 補完機能が有効になっていないデータセットでこの関数が呼び出された場合、SYSTEMERROR_GEN_INVALID_OPERATIONがシグナルされます。
     * @li デュプリカ(@ref DuplicatedObject::IsADuplica())がこの関数を呼び出した場合、SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICAがシグナルされます。
     *
     * この機能について詳しくはNEXプログラミングマニュアルも参照してください。
     *
     * @param[in]   v               データセットの変数の中で、補完モデルを変更するものを指定します。MODELマクロを使って指定してください。
     * @param[in]   dValue          現在のセッション時間における変数の値を指定します。
     * @param[in]   dRateOfChange   現在のセッション時間における変数の変化の割合をミリ秒単位で指定します。
     * @param[in]   dAcceleration   現在のセッション時間における変数の加速度を、ミリ秒の二乗の単位で指定します。
     * @param[in]   tTimestamp      現在のセッション時間を指定します(デフォルト引数として指定されます)。
     *
     * @see     IndicateContinuityBreak
     */
    void SetModel(PHBDRVar_qInt32& v, qReal dValue, qReal dRateOfChange, qReal dAcceleration,
                  Time tTimestamp=GetTimestamp())
    {
        SYSTEMCHECK(false);
    }

    /*
    {Group:Dataset Extrapolation Methods}

    Description: Sets a new extrapolation model locally on a Duplica.
    Parameters:
    v: Dataset extrapolation variable for which the extrapolation model is
    changed.  This value is obtained using the MODEL macro.
    dValue: Value of the variable, at time tTimestamp.
    dRateOfChange: Rate of change of the variable, in units per millisecond, at time tTimestamp.
    dAcceleration: Acceleration of the variable, in units per millisecond^2, at time tTimestamp.
    tTimestamp: (optional) Specifies the point in (absolute) time at which the
    local correction is valid. Otherwise the current time as given by the
    SessionClock is used.
    Remarks: This method is only called on an individual Duplica. The method may be
    used to temporarily correct an inaccurate extrapolation.  For example, when a ball hits a wall
    due to network latency, extrapolation of the position of the ball on a Duplica may lead to the
    ball being displayed past the wall. The SetLocalCorrection method can be used to temporarily
    adjust the model on the individual Duplica, taking into account the effect of the collision.
    As a result, updates from the Duplication master are ignored and the new local extrapolation
    model is used until two points from the duplication master have been received. Once two points
    are received, $PRODUCTNAME$ has enough information to recommence the extrapolation.

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter*
    DDL property in its DDL declarations.

    Returns: The error SYSTEMERROR_GEN_INVALID_OPERATION is signalled when this method is
    called on a dataset that has not been extrapolated.
    SYSTEMERROR_OBJDUP_INVALID_ON_MASTER is signalled if the method is called on a
    Duplication master
    See also: MODEL, IndicateContinuityBreak, SetModel
    */

    /*!
     * @brief   デュプリカに対してローカルな新しい補完モデルを設定します。
     *
     * ※この関数は、DataSetクラスが提供する関数ではありません。
     * DDL(データ定義言語)宣言に、このデータセットがextrapolation_filterプロパティを持つように
     * 指示されたときのみ、DDLコンパイラがこの関数のためのコードを生成します。
     *
     * この関数は個人のデュプリカ(@ref DuplicatedObject::IsADuplica)に対してのみ実行できます。
     * この関数は、誤った補完情報を一時的に訂正することが出来ます。
     * たとえば、ゲーム中のボールが壁にあたって跳ね返った場合、ネットワーク遅延によって
     * デュプリカのボールの位置の補完によって壁にめり込んだ状態で描画されてしまうことがあります。
     * このSetLocalCorrection()は一時的に個人のデュプリカに対して
     * 当たり判定計算の影響を適用するために補完モデルを調整使うことが出来ます。
     *
     * 複製マスターからの更新情報を無視して、次に複製マスターから2回の更新信号を受け取るまで
     * ローカルの補完モデルを元に補完を実行します。複製マスターから2回の更新を受け取ったら、
     * システムを通じた補完を再開します。
     *
     * この関数は以下のエラーをシグナルすることがあります。
     *
     * @li このデータセットで補完機能が有効でないとき、SYSTEMERROR_GEN_INVALID_OPERATIONがシグナルされます。
     * @li この関数が複製マスター(@ref DuplicatedObject::IsADuplicationMaster())で実行された場合、SYSTEMERROR_OBJDUP_INVALID_ON_MASTERがシグナルされます。
     *
     * @param[in]   v               データセットの変数の中で、補完モデルを変更するものを指定します。MODELマクロを使って指定してください。
     * @param[in]   dValue          現在のセッション時間における変数の値を指定します。
     * @param[in]   dRateOfChange   現在のセッション時間における変数の変化の割合をミリ秒単位で指定します。
     * @param[in]   dAcceleration   現在のセッション時間における変数の加速度を、ミリ秒の二乗の単位で指定します。
     * @param[in]   tTimestamp      現在のセッション時間を指定します(デフォルト引数として指定されます)。
     *
     * @see     IndicateContinuityBreak
     *
     */
    void SetLocalCorrection(PHBDRVar_qReal& v, qReal dValue, qReal dRateOfChange, qReal dAcceleration,
                            Time tTimestamp=GetTimestamp())
    {
        SYSTEMCHECK(false);
    }

    /*!
     * @brief   デュプリカに対してローカルな新しい補完モデルを設定します。
     *
     * ※この関数は、DataSetクラスが提供する関数ではありません。
     * DDL(データ定義言語)宣言に、このデータセットがextrapolation_filterプロパティを持つように
     * 指示されたときのみ、DDLコンパイラがこの関数のためのコードを生成します。
     *
     * この関数は個人のデュプリカ(@ref DuplicatedObject::IsADuplica)に対してのみ実行できます。
     * この関数は、誤った補完情報を一時的に訂正することが出来ます。
     * たとえば、ゲーム中のボールが壁にあたって跳ね返った場合、ネットワーク遅延によって
     * デュプリカのボールの位置の補完によって壁にめり込んだ状態で描画されてしまうことがあります。
     * このSetLocalCorrection()は一時的に個人のデュプリカに対して
     * 当たり判定計算の影響を適用するために補完モデルを調整使うことが出来ます。
     *
     * 複製マスターからの更新情報を無視して、次に複製マスターから2回の更新信号を受け取るまで
     * ローカルの補完モデルを元に補完を実行します。複製マスターから2回の更新を受け取ったら、
     * システムを通じた補完を再開します。
     *
     * この関数は以下のエラーをシグナルすることがあります。
     *
     * @li このデータセットで補完機能が有効でないとき、SYSTEMERROR_GEN_INVALID_OPERATIONがシグナルされます。
     * @li この関数が複製マスター(@ref DuplicatedObject::IsADuplicationMaster())で実行された場合、SYSTEMERROR_OBJDUP_INVALID_ON_MASTERがシグナルされます。
     *
     * @param[in]   v               データセットの変数の中で、補完モデルを変更するものを指定します。MODELマクロを使って指定してください。
     * @param[in]   dValue          現在のセッション時間における変数の値を指定します。
     * @param[in]   dRateOfChange   現在のセッション時間における変数の変化の割合をミリ秒単位で指定します。
     * @param[in]   dAcceleration   現在のセッション時間における変数の加速度を、ミリ秒の二乗の単位で指定します。
     * @param[in]   tTimestamp      現在のセッション時間を指定します(デフォルト引数として指定されます)。
     *
     * @see     IndicateContinuityBreak
     *
     */

    void SetLocalCorrection(PHBDRVar_qReal& v, qReal dValue, qReal dRateOfChange, qReal dAcceleration,
                            Time tTimestamp=GetTimestamp())
    {
        SYSTEMCHECK(false);
    }
    /*!
     * @brief   デュプリカに対してローカルな新しい補完モデルを設定します。
     *
     * ※この関数は、DataSetクラスが提供する関数ではありません。
     * DDL(データ定義言語)宣言に、このデータセットがextrapolation_filterプロパティを持つように
     * 指示されたときのみ、DDLコンパイラがこの関数のためのコードを生成します。
     *
     * この関数は個人のデュプリカ(@ref DuplicatedObject::IsADuplica)に対してのみ実行できます。
     * この関数は、誤った補完情報を一時的に訂正することが出来ます。
     * たとえば、ゲーム中のボールが壁にあたって跳ね返った場合、ネットワーク遅延によって
     * デュプリカのボールの位置の補完によって壁にめり込んだ状態で描画されてしまうことがあります。
     * このSetLocalCorrection()は一時的に個人のデュプリカに対して
     * 当たり判定計算の影響を適用するために補完モデルを調整使うことが出来ます。
     *
     * 複製マスターからの更新情報を無視して、次に複製マスターから2回の更新信号を受け取るまで
     * ローカルの補完モデルを元に補完を実行します。複製マスターから2回の更新を受け取ったら、
     * システムを通じた補完を再開します。
     *
     * この関数は以下のエラーをシグナルすることがあります。
     *
     * @li このデータセットで補完機能が有効でないとき、SYSTEMERROR_GEN_INVALID_OPERATIONがシグナルされます。
     * @li この関数が複製マスター(@ref DuplicatedObject::IsADuplicationMaster())で実行された場合、SYSTEMERROR_OBJDUP_INVALID_ON_MASTERがシグナルされます。
     *
     * @param[in]   v               データセットの変数の中で、補完モデルを変更するものを指定します。MODELマクロを使って指定してください。
     * @param[in]   dValue          現在のセッション時間における変数の値を指定します。
     * @param[in]   dRateOfChange   現在のセッション時間における変数の変化の割合をミリ秒単位で指定します。
     * @param[in]   dAcceleration   現在のセッション時間における変数の加速度を、ミリ秒の二乗の単位で指定します。
     * @param[in]   tTimestamp      現在のセッション時間を指定します(デフォルト引数として指定されます)。
     *
     * @see     IndicateContinuityBreak
     *
     */
    void SetLocalCorrection(PHBDRVar_qInt32& v, qReal dValue, qReal dRateOfChange, qReal dAcceleration,
                            Time tTimestamp=GetTimestamp())
    {
        SYSTEMCHECK(false);
    }


    /*!
     *  @brief 受信したデータセットのリビジョンを取得します。
     *
     *  ※この関数は、DataSetクラスが提供する関数ではありません。
     *  DDLコンパイラがこの関数のためのコードを生成します。
     *
     *  @return リビジョン番号
     */

    qByte GetRevision();

#endif // FOR_DOC_PURPOSE

    //group=Dataset Methods

    /*
    Remarks: Datasets should never be directly instantiated. They are accessible
    as data members of duplicated objects.
    See Also: DuplicatedObject.
    */

    /*!
     * @brief   このコンストラクタはアプリケーションから呼び出さないでください。
     *
     * データセットは複製オブジェクトのデータメンバーとしてアクセスすることが出来ます。
     *
     * @see     DuplicatedObject
     */
    DataSet();

    /*
    Remarks: Datasets should never be directly destroyed. When the duplicated object
    to which they are associated to is destroyed, the dataset is automatically destroyed.
    See Also: DuplicatedObject
    */

    /*!
     * @brief   このデストラクタはアプリケーションから呼び出さないでください。
     *
     * データセットに関連付けられている複製オブジェクトが削除されたときにこのインスタンスも自動的に削除されます。
     *
     * @see     DuplicatedObject
     */
    virtual ~DataSet();

    //{secret}
    qBool SomeUpdatesRequired();

    /*
    {Group:Dataset System Callbacks}

    Description: Called by $PRODUCTNAME$ each time a dataset that uses a station filter is updated.
    Parameters:
    pDO: A pointer to the DuplicatedObject that this DataSet belongs to.
    pStation: A pointer to the Station where this DataSet will be updated.
    Returns: true if this DataSet must be updated on pStation, false if not.
    By default this system callback returns true.
    Remarks: If a DataSet specifies the use of a station_filter in its update policy then this method
    will be called each time the DataSet is updated to determine which stations to update. The two major
    reasons to use a station filter are to apply update (bandwidth) optimizations or security restrictions.
    In the first case, a station filter can be used to prevent unecessary information being sent across the
    network. For example, for an object that is a long way away you most likely do not need to know the object's
    orientation (you will probably only represent the object as a point after all) and therefore, this DataSet
    does not need to be updated thereby saving bandwidth. In the second case we could define data as being either
    public, semi-private, or private (or any other categorization you desire) and limit the stations where certain
    data is updated accordingly. For example, as the position of your avatar needs to be known by all players this
    should be public and updated on all stations. Data pertaining to your inventory only needs to be known by
    your fellow guild members so this data should be semi-private and only updated on guild members' stations. Other
    information could be made private and either not shared at all or only with the server. The example given below
    illustrates how this last condition could be implemented.

            qBool UpdateIsRequired(const DuplicatedObject* pDO, const Station* pStation) {
                Avatar(pDO);
                // Only update this DataSet if the Station is a server.
                return pStation->GetProcessType()==Station::ServerProcess;
            }

    Note:  This method is only generated for a dataset that includes the *station_filter* DDL property in
    its DDL declarations.

    See Also: DDL File Syntax
    */

    /*!
     * @brief   データセットを更新する対象のステーションを選択します。
     *
     * DDL(データ定義言語)宣言において、データセットの更新ポリシーにstation_filterがつけられている場合、
     * データセットを更新しようとする度にシステム側からこの関数が呼び出され、
     * どのステーションに対して更新を行うかどうかを選択することが出来ます。
     * station_filterによって更新を行うステーションを絞り込むことは二つの意味があります。
     *
     * ひとつは、更新を行うステーションを絞り込むことによってネットワーク帯域を稼いで
     * 最適化することが出来ます。たとえば、とても遠くにいるオブジェクトは
     * そのオブジェクトの正確な位置や振る舞いなどが分からなくても問題にならないことがありますので、
     * そのような伝送をカットすることで帯域幅を節約することが出来ます。
     *
     * もうひとつは、セキュリティに対する理由です。
     * たとえば、このデータセットが、公開情報なのか、一部にのみ公開されている情報なのか、
     * 公開されていない情報なのかによって(あるいは、アプリケーションによってはもっと別の状態があるかもしれません)、
     * すべてのステーションに対して更新処理を行うべきかそうでないのかを決定することが出来ます。
     * たとえば、カードゲームにおける手札の状態や、戦略ゲームにおけるステルス戦闘機の位置などは
     * すべてのプレイヤーに公開されませんし、ロールプレイングゲームでプレイヤーが持っているアイテムは、
     * そのプレイヤーか、そのプレイヤーの同一のギルドに所属するメンバーのステーションのみが
     * 知ることができます(※この例はゲームの仕様により変化しますので必ずしもそうであるとは限りません)。
     *
     * @param[in]   pDO         このデータセットに関連付けられている複製オブジェクトへのポインタが渡されます。
     * @param[in]   pStation    データセットを更新しようとしているステーションへのポインタが渡されます。
     *
     * @retval  true    このデータセットを、@a pStationにおいて更新するとき返ります(デフォルトでは常にtrueが返ります)。
     * @retval  false   このデータセットを、@a pStationにおいて更新しないときに返ります。
     */
    qBool UpdateIsRequired(const DuplicatedObject *pDO, const Station *pStation);

    //{secret}
    void FinishedAllUpdates();

    //{secret}
    qBool Refresh(const Time& tRefreshTime);

    /*
    {Group:Dataset Methods}

    Description: Returns whether or not updates for this dataset are sent over a reliable channel.
    Returns: true if updates for this dataset are sent over a reliable channel, false if they
    are sent over an unreliable channel or an error occurred.
    Remarks: Dataset updates may be sent over a reliable channel, where delivery is guaranteed, or an
    unreliable channel, where some packets may be lost. The channel that dataset updates are sent over is
    defined by the update policy for a given dataset, which is declared in the DDL. By default, updates are sent
    over a reliable channel unless the unreliable DDL property is included in the DDL declarations for the given
    dataset. Refer to Dataset DDL File Syntax for further details.
    See Also: Dataset DDL File Syntax
    */

    /*!
     * @brief   リライアブルチャンネルを利用してデータセットを更新するかそうでないかを返します。
     *
     * データセットは、リライアブル(到達保証のある)チャンネルか、
     * アンリライアブル(到達保証のない)チャンネルを通じて伝送されます。
     * 到達保証のないアンリライアブルチャンネルは、ネットワークの途中でパケットを喪失すると、
     * その内容は相手側に二度と伝わらなくなります。
     * このデータセットが利用するチャンネルは、DDL(データ定義言語)にて宣言された
     * このデータセットに対する更新ポリシーによって与えられます。
     * デフォルトでは、更新はリライアブル(到達保証のある)チャンネルを利用します。
     * DDLファイルにunreliableプロパティがつけられた場合、アンリライアブル(到達保証のない)チャンネルが利用されます。
     *
     * 一般的に、リライアブル通信は伝送コストが高く、アンリライアブル通信は伝送コストが高くありません。
     * リライアブル通信は更新頻度が低いが確実に伝送されなければならない情報
     * (たとえば、プレイヤーの体力や持っているアイテムなど)に対して利用し、
     * アンリライアブル通信は更新頻度が高いが、確実に伝わらなくても大きな支障をきたさない情報
     * (たとえば、プレイヤーの座標や向いている方向など)に対して利用してください。
     *
     * @retval  true    このデータセットがリライアブルチャンネルを通じて更新されるときに返ります。
     * @retval  false   このデータセットがアンリライアブルチャンネルを通じて更新されるときに返ります。
     */
    qBool ReliableUpdate() const
    {
        return true;
    }

#ifdef FOR_DOC_PURPOSE
    /*
    {Group:Dataset System Callbacks}

    Description: Called by $PRODUCTNAME$ each time a DOOperation that affects a DataSet begins.
    Parameters:
    pOperation: A pointer to the DOOperation that just began.
    Remarks: This method is called at the beginning of certain DOOperation s with a
    reference to the operation that just began as a parameter. The operations can be
    AddToStoreOperation, RemoveFromStoreOperation, ChangeMasterStationOperation,
    ChangeDupSetOperation, and UpdateDataSetOperation.
    See DataSet::OperationEnd for an example of how to use this method.
    See Also:  System Operations, OperationEnd, OperationManager
    */

    /*!
     * @brief   システム側から、@ref DOOperation がデータセットに対して影響を与えるときに呼び出されます。
     *
     * この関数は、それぞれの複製オブジェクトに対するシステム操作(@ref DOOperation)が実行されはじめたときに
     * システム側から呼び出されます。実行される複製オブジェクトシステム操作は、
     * @ref AddToStoreOperation , @ref RemoveFromStoreOperation , @ref ChangeMasterStationOperation ,
     * @ref ChangeDupSetOperation , @ref UpdateDataSetOperation のうちのいずれかです。
     *
     * @param[in]   pOperation  実行が開始されようとしている複製オブジェクトシステム操作が指定されます。
     * @see     OperationEnd, DuplicatedObject::OperationBegin
     */
    void OperationBegin(const DOOperation* pOperation) {}
    /*
    {Group:Dataset System Callbacks}

    Description: Called by $PRODUCTNAME$ each time a DOOperation that affects a DataSet terminates.
    Parameters:
    pOperation: A pointer to the DOOperation that just terminated.
    Remarks: This method is called at the end of certain DOOperation s with a reference to
    the operation that just ended as a parameter. The operations can be
    AddToStoreOperation, RemoveFromStoreOperation, ChangeMasterStationOperation,
    ChangeDupSetOperation, and UpdateDataSetOperation.

    For example, say you wish to be notified when a given dataset is
    updated, then you could implement the following:

            void Inventory::OperationEnd(DOOperation* pOperation){
                switch (pOperation->GetType()) {
                    case Operation::UpdateDataSet: {
                        // dataset updated. Send notification or
                        // whatever else needs to be done.
                    }
                    break;
                default:
                    break;
                }
            }

    Note: To avoid deadlocks, while within an OperationEnd context you
    should not perform any blocking calls,  such as CallContext::Wait.

    See Also: System Operations, OperationBegin, OperationManager
    */

    /*!
     * @brief   システム側から、@ref DOOperation がデータセットに対して影響を与えたあとに呼び出されます。
     *
     * この関数は、それぞれの複製オブジェクトシステム操作(@ref DOOperation)が実行されたあとで、
     * システム側から呼び出されます。実行される複製オブジェクトシステム操作は、
     * @ref AddToStoreOperation , @ref RemoveFromStoreOperation , @ref ChangeMasterStationOperation ,
     * @ref ChangeDupSetOperation , @ref UpdateDataSetOperation のうちのいずれかです。
     *
     * @param[in]   pOperation  実行された複製オブジェクトシステム操作が指定されます。
     * @see     OperationBegin, DuplicatedObject::OperationEnd
     */
    void OperationEnd(const DOOperation* pOperation) {}
#else
    void OperationBegin(const DOOperation* /*pOperation*/) {}
    void OperationEnd(const DOOperation* /*pOperation*/) {}

#endif //FOR_DOC_PURPOSE

    //{secret}
    void CallOperationOnVars(Operation::Event eEvent, void* pDOOperation);

    //{secret}
    void RegisterDOContainer(DuplicatedObject* pDOContainer);

    // {Secret}
    qBool UsesBufferedRefresh() const
    {
        return false;
    }

    // {Secret}
    static Time GetTimestamp();

public:


#ifdef FOR_DOC_PURPOSE
    //group=Dataset Extrapolation Methods

    /*
    Summary: Returns a pointer to the PHBDRParameters for the given dataset class.
    Returns: A pointer to the PHBDRParameters for the given dataset or NULL if the
    call signals an error.
    Remarks: These parameters are used to fine-tune the dead reckoning algorithm.
    SYSTEMERROR_GEN_INVALID_OPERATION is signalled if the method is called on a non-extrapolated dataset.

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter*
    DDL property in its DDL declarations.

    See Also: DDL File Syntax
    */

    /*!
     * @brief   このデータセットクラスに対する推定航法パラメータへのポインタを取得します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにextrapolation_filterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     *
     * PHBDRParametersの値は、推定航法アルゴリズムをチューニングするために使われます。
     * このデータセットが推定航法を利用しない設定の場合、
     * SYSTEMERROR_GEN_INVALID_OPERATIONがシグナルされます。
     *
     * @return  このデータセットに対する推定航法パラメータが返ります。
     * @retval  NULL エラーが発生したときに返ります。
     */
    static PHBDRParameters* GetPHBDRParameters();

    /*
    Summary: Returns a pointer to the ErrorToleranceFunction for the given dataset
    class.
    Returns: A pointer to the ErrorToleranceFunction for the dataset or NULL if the call signals
    an error.  SYSTEMERROR_GEN_INVALID_OPERATION is signalled if the dataset is not
    extrapolated.

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter*
    DDL property in its DDL declarations.

    See Also: DDL File Syntax
    */

    /*!
     * @brief   このデータセットに対するErrorToleranceFunctionへのポインタを取得します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにextrapolation_filterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     *
     * @return  このデータセットに対するErrorToleranceFunctionへのポインタが返ります。
     */
    static ErrorToleranceFunction* GetErrorToleranceFunction();

    //group=Dataset Methods

#ifdef NEX_PERFCOUNTER_ENABLED
    /*
    Description: Returns a pointer to the DataSetPerfCounters for the given dataset class.
    Returns: A pointer to the DataSetPerfCounters for the given dataset class.
    Remarks: This pointer points to the performance counters created by $PRODUCTNAME$ for each DataSet
    declared in the DDL file that specifies the update_perf_counter DDL property. Various performance
    data pertaining to the dataset can be recorded to aid in fine-tuning and optimizing network performance.
    Refer to DataSetPerfCounters for full details.

    Note: This method is only generated for a dataset that includes the *update_perf_counter* DDL property in
    its DDL declarations.

       See Also: DataSetPerfCounters, Dataset DDL File Syntax
       */

    /*!
     * @brief   このデータセットに対するパフォーマンスカウンタ(DataSetPerfCounters)へのポインタを取得します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにupdate_perf_counterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     *
     * DDL(データ定義言語)ファイルに、update_perf_counterプロパティが宣言されている
     * それぞれのデータセットに対してパフォーマンスカウンターが作成されます。
     * この関数を利用すると、パフォーマンスカウンターに対するポインタが取得できます。
     * さまざまなパフォーマンス情報をこのデータセットに対して記録することができ、
     * その情報をネットワークパフォーマンスの最適化とチューニングに利用することが出来ます。
     * 詳細は、DataSetPerfCountersを参照してください。
     *
     * @return  パフォーマンスカウンタへのポインタが返ります。
     */
    static DataSetPerfCounters* GetPerformanceCounters();
#endif // NEX_PERFCOUNTER_ENABLED

#endif //FOR_DOC_PURPOSE

#ifdef FOR_DOC_PURPOSE
    /*
    Summary: Indicates that the next update must be performed .
    Returns: SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA if this method is called on a Duplica.
    Remarks: This method is available for datasets declared with the
    upon_request_filter DDL property.  When this dataset property is specified,
    the call to update a dataset (see DuplicatedObject::Update) will
    not generate a message unless it has been requested via
    RequestUpdate. This method is only valid when called on a Duplication master.

    Note:  This method is only generated for a dataset that includes the *upon_request_filter* DDL property in
    its DDL declarations.

    See Also: DDL File Syntax
    */

    /*!
     * @brief   次の更新のときは確実に実行する必要があることを通知します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにupon_request_filterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     *
     * 複製オブジェクトで推定航法を利用する場合、推定されているデータの
     * 予測モデルを維持するために毎フレーム@ref DuplicatedObject::Update() を実行する必要があります。
     * upon_request_filterプロパティがセットされていると、@ref DuplicatedObject::Update() を実行しても
     * 必ずしもネットワークメッセージが作成されなくなります。
     * @ref DuplicatedObject::Update() を実行したときに
     * 事前にこの関数を呼び出しておくことで確実に次の更新時にメッセージを送るようになります。
     */
    void RequestUpdate();

    //group=Dataset Extrapolation Methods

    /*
    Summary: Indicates that the dataset values changed in a non-continuous
    way.  Indicating continuity breaks helps $PRODUCTNAME$ perform a smoother
    dead reckoning.
    Parameters:
    byBreak: Type of continuity break that occurred.
    Remarks: This method is available for datasets declared with the
    extrapolation_filter DDL property.  When the dataset values change
    smoothly, in a continuous way, an update message needs to be sent only
    when the extrapolation error is higher than the specified threshold (refer to
    Dead Reckoning Overview). When a continuity break occurs, the next few calls to
    DuplicatedObject::Update will generate an update of the dataset, regardless of the
    extrapolation error, to provide the duplicas with information about the change.
    This update will be sent over a reliable channel, regardless of whether updates for
    the dataset are normally sent over a reliable or unreliable channel.

    When a call to IndicateContinuityBreak is made, it will have an effect on the
    next few calls to DuplicatedObject::Update.  The type of continuity break
    dictates the number of updates that will be affected. This call only
    makes sense on a DataSet that belongs to a Duplication master.

    The valid values for continuity breaks are CONTINUITY_BREAK_TELEPORT,
    CONTINUITY_BREAK_STOP, and CONTINUITY_BREAK_SUDDEN_CHANGE. The flag
    CONTINUITY_BREAK_UPDATED_MODEL can be added to
    CONTINUITY_BREAK_TELEPORT and CONTINUITY_BREAK_SUDDEN_CHANGE.

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter*
    DDL property in its DDL declarations.

    Returns: SYSTEMERROR_GEN_INVALID_OPERATION will be signalled when called on a
    dataset that is not declared with the extrapolation_filter DDL property.
    SYSTEMERROR_GEN_OUT_OF_RANGE will be signalled when
    called with an invalid value for the continuity break.  SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA
    will be signalled if the method is called on a Duplica.
    See Also: DDL File Syntax
    */

    /*!
     * @brief   データセットが非連続的な方法で値が変更されたことを通知します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにextrapolation_filterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     *
     * データセットが非連続な方法で値が変更されたことを通知すると、
     * システムはよりスムーズな推定航法を行うことが出来ます。
     * このような、非連続的な値の変化は、ゲーム中に存在するワープゾーンに
     * プレイヤーが入って瞬間移動を行ったり、あるいはゲーム中に出てくるボールが
     * 壁に当たって跳ね返ったりするときに発生します。
     *
     * このデータセットの値が連続でスムーズに変化していくとき、
     * 更新メッセージは、推定誤差が一定値以上たまった場合にのみ送られます
     * (詳しくはNEXプログラミングマニュアルの推定航法の章も参照してください)。
     * 非連続な値の変化を通知すると、次のいくつかの@ref DuplicatedObject::Update() の
     * 呼び出しは、推定誤差を考えず、常にデュプリカに更新に関する情報を送信します。
     * この更新情報は、DDLプロパティに設定されたデータセット更新の送信方法が
     * アンリライアブル(ネットワークパケットの到達保証のない)チャンネルを利用するか
     * リライアブル(ネットワークパケットの到達保証のある)チャンネルを利用するかの設定にかかわらず
     * 常にリライアブルチャンネルを利用して送信されます。
     *
     * IndicateContinuityBreak()を呼び出したら、次のいくつかの@ref DuplicatedObject::Update() 呼び出しに
     * 影響を与えます。そのため、IndicateContinuityBreak()は複製マスター
     * (@ref DuplicatedObject::IsADuplicationMaster())に所属するデータセットに対して呼び出して初めて意味をなします。
     * @ref DuplicatedObject::Update() は複製マスター以外が呼び出すことは出来ません。
     *
     * extrapolation_filterがDDLプロパティに定義されていないときに、SYSTEMERROR_GEN_INVALID_OPERATIONがシグナルされます。
     * @a byBreakで指定された値が無効なものであるとき、SYSTEMERROR_GEN_OUT_OF_RANGEがシグナルされます。
     * この関数が、デュプリカ(@ref DuplicatedObject::IsADuplica)に所属するデータセットに対して
     * 呼び出された場合、SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICAがシグナルされます。
     *
     * @param[in]   byBreak どのように連続性が失われたのかを指定します。<br>
     *                      CONTINUITY_BREAK_TELEPORT は、物体が瞬間移動したことをシステムに通知します。<br>
     *                      CONTINUITY_BREAK_STOP は、物体が急に停止したことをシステムに通知します。<br>
     *                      CONTINUITY_BREAK_SUDDEN_CHANGE は、物体の動きが突然変化したことをシステムに通知します。
     * @param[in]   bReliable   データセットの変化をリライアブル通信で送信するかどうかを指定します。<br>
     *                          trueだとリライアブル送信、falseだとアンリライアブル送信を行います。
     *
     * @see     SetModel
     */
    void IndicateContinuityBreak(qByte byBreak, qBool bReliable = true);

    // A few methods for dead reckoning...


    /*
    Description: Sets the maximum delay between updates of a dataset.
    Parameters:
    tiMaximumUpdateDelay: Maximum update delay in milliseconds.
    Remarks: This method is only valid on an extrapolated dataset.
    When an extrapolated dataset uses unreliable communication, the loss of a
    message may cause a significant drift in the value of the dataset. The maximum update
    delay provides a solution to this by periodically sending messages to
    the remote station even if the local prediction model is correct. If reliable
    communication is used, the maximum update delay has no effect.

    The maximum update delay is the maximum time allowed between updates for a dataset.
    If a dataset is not updated within the maximum delay time, then an update of the dataset
    is automatically sent to the remote stations, even if the value of the dataset has not changed
    since the previous update.

    The initial value for the maximum update delay of 2000 ms is given by
    DEFAULT_MAXIMUM_UPDATE_DELAY. The constant
    NO_MAXIMUM_UPDATE_DELAY can
    be used to indicate that a message is never sent as a result of an
    expired update delay.

    The maximum update delay is a static dataset variable.  All instances
    of a single dataset class share the same value however, datasets of different classes
    may have different maximum update delays.

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter*
    DDL property in its DDL declarations.

    See also: GetMaximumUpdateDelay, DEFAULT_MAXIMUM_UPDATE_DELAY, NO_MAXIMUM_UPDATE_DELAY,
    ReliableUpdate, GetMinimumUpdateDelay, SetMinimumUpdateDelay
    */

    /*!
     * @brief   データセットの最大更新遅延時間を設定します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにextrapolation_filterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     * DataSet::SetMaximumUpdateDelay()は利用することが出来ません。
     * 宣言したデータセットクラスに対してSetMaximumUpdateDelayを利用してください。
     *
     * 推定航法を使うデータセットがアンリライアブル(ネットワークパケットの到達保証のないチャンネル)を
     * 利用するとき、メッセージパケットが途中で喪失すると、データセットの値が著しい変化を起こす場合があります。
     * 最大更新遅延はこのような問題を、ローカルの推定モデルが正しかったとしても
     * 一定間隔でメッセージを相手のステーションに対して送信していくことによって解決します。
     * データセットの更新がリライアブル(ネットワークパケットの到達保証のあるチャンネル)を
     * 利用しているとき、最大更新遅延設定は効果を発揮しません。
     *
     * この最大更新遅延はデータセットに対する更新間隔の許容できる最大の時間です。
     * もしデータセットが最大遅延時間の間、更新を行わなかった場合、
     * 前回の更新時から値がなにも変わっていなかったとしても
     * 相手先のステーションに自動的にメッセージが送信されます。
     *
     * この最大遅延時間は静的(static)な値であり、同一のデータセットは
     * 異なるインスタンスでも同一の最大遅延時間を持ちます。
     * ただし、データセットが異なれば、違う最大遅延時間を持ちます。
     *
     * @param[in]   tiMaximumUpdateDelay    設定する最大遅延時間を指定します。デフォルト値はDEFAULT_MAXIMUM_UPDATE_DELAYです。
     *                                      NO_MAXIMUM_UPDATE_DELAYは特殊な値であり、最大遅延時間によってはメッセージが自動的に送られないことを意味します。
     *
     * @see     GetMaximumUpdateDelay
     */
    static void SetMaximumUpdateDelay(TimeInterval tiMaximumUpdateDelay);

    /*
    Description: Returns the current maximum update delay in milliseconds.
    Returns:  The current maximum update delay in milliseconds.

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter*
    DDL property in its DDL declarations.

    See also: SetMaximumUpdateDelay, GetMinimumUpdateDelay, SetMinimumUpdateDelay
    */

    /*!
     * @brief   データセットの最大更新遅延時間を取得します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにextrapolation_filterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     * DataSet::GetMaximumUpdateDelay()は利用することが出来ません。
     * 宣言したデータセットクラスに対してGetMaximumUpdateDelayを利用してください。
     *
     * 推定航法を使うデータセットがアンリライアブル(ネットワークパケットの到達保証のないチャンネル)を
     * 利用するとき、メッセージパケットが途中で喪失すると、データセットの値が著しい変化を起こす場合があります。
     * 最大更新遅延はこのような問題を、ローカルの推定モデルが正しかったとしても
     * 一定間隔でメッセージを相手のステーションに対して送信していくことによって解決します。
     * データセットの更新がリライアブル(ネットワークパケットの到達保証のあるチャンネル)を
     * 利用しているとき、最大更新遅延設定は効果を発揮しません。
     *
     * この最大更新遅延はデータセットに対する更新間隔の許容できる最大の時間です。
     * もしデータセットが最大遅延時間の間、更新を行わなかった場合、
     * 前回の更新時から値がなにも変わっていなかったとしても
     * 相手先のステーションに自動的にメッセージが送信されます。
     *
     * この最大遅延時間は静的(static)な値であり、同一のデータセットは
     * 異なるインスタンスでも同一の最大遅延時間を持ちます。
     * ただし、データセットが異なれば、違う最大遅延時間を持ちます。
     *
     * @return  最大更新遅延時間を返します。
     */
    static TimeInterval GetMaximumUpdateDelay();

    /*
    Description: Sets the minimum delay between updates of a dataset.
    Parameters:
    tiMinimumUpdateDelay: Minimum update delay in milliseconds.
    Remarks: This method is only valid on an extrapolated dataset.
    When the defined error tolerance (see ErrorToleranceFunction) of an extrapolated
    dataset is low the dataset may be updated too frequently whereby using a significant amount
    of bandwidth. By setting a minimum update delay the developer can control the maximum frequency
    at which a dataset is updated.

    The minimum update delay is the minimum time allowed between updates for a dataset. A dataset
    will not be updated before the minimum update delay has expired. If DuplicatedObject::Update is
    called more frequently than the minimum update delay, the dataset updates will be dropped.
    However, if a continuity break is indicated via IndicateContinuityBreak, then the minimum update
    delay will be ignored for the next few updates to enable $PRODUCTNAME$ to accurately extrapolate
    the dataset values.

    The initial value for the minimum update delay of 66 ms, which is equivalent to approximately
    6 updates/second, is given by DEFAULT_MINIMUM_UPDATE_DELAY. The constant NO_MINIMUM_UPDATE_DELAY can
    be used to indicate that a message is never dropped as a result of an unexpired minimum
    update delay.

    The minimum update delay is a static dataset variable.  All instances
    of a single dataset class share the same value however, datasets of different classes
    may have different minimum update delays.

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter*
    DDL property in its DDL declarations.

    See also: GetMinimumUpdateDelay, DEFAULT_MINIMUM_UPDATE_DELAY, NO_MINIMUM_UPDATE_DELAY,
    ReliableUpdate, GetMaximumUpdateDelay, SetMaximumUpdateDelay
    */

    /*!
     * @brief   このデータセットの最小更新遅延時間を設定します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにextrapolation_filterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     * DataSet::SetMinimumUpdateDelay()は利用することが出来ません。
     * 宣言したデータセットクラスに対してSetMinimumUpdateDelayを利用してください。
     *
     * 推定航法を利用するデータセットに定義された誤差許容(ErrorToleranceFunction)が
     * 低いとき、頻繁に更新を行うとネットワーク帯域幅を大きく圧迫することがあります。
     * この最小遅延時間を設定することで、アプリケーション開発者は
     * データセットが更新される最大の頻度を設定することが出来ます。
     *
     * 最小更新遅延時間は、データセットに許容される最小の更新時間です。
     * データセットは、この最小更新遅延時間が経過するまでは更新されることはありません。
     * もしも@ref DuplicatedObject::Update() がここで設定された時間よりも
     * 頻繁に呼び出されたときは、@ref DuplicatedObject::Update() の呼び出しがされなかったことになります。
     * ただし、連続性の切断(@ref IndicateContinuityBreak() )が起こった場合はこの限りではありません。
     *
     * この最小遅延時間は静的(static)な値であり、同一のデータセットは
     * 異なるインスタンスでも同一の最小遅延時間を持ちます。
     * ただし、データセットが異なれば、違う最小遅延時間を持ちます。
     *
     * @param[in]   tiMinimumUpdateDelay        設定する最小更新遅延時間を指定します。
     *                                      デフォルトはDEFAULT_MINIMUM_UPDATE_DELAYになります。
     *                                      NO_MINIMUM_UPDATE_DELAYを指定すると最小時間によって更新が破棄されないことを示します。
     *
     * @see     GetMinimumUpdateDelay
     */
    static void SetMinimumUpdateDelay(TimeInterval tiMinimumUpdateDelay);

    /*
    Description: Returns the current minimum update delay in milliseconds.
    Returns: The current minimum update delay in milliseconds.

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter*
    DDL property in its DDL declarations.

    See also: SetMinimumUpdateDelay, GetMaximumUpdateDelay, SetMaximumUpdateDelay
    */

    /*!
     * @brief   このデータセットの最小更新遅延時間を取得します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにextrapolation_filterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     * DataSet::GetMinimumUpdateDelay()は利用することが出来ません。
     * 宣言したデータセットクラスに対してGetMinimumUpdateDelayを利用してください。
     *
     * 推定航法を利用するデータセットに定義された誤差許容(ErrorToleranceFunction)が
     * 低いとき、頻繁に更新を行うとネットワーク帯域幅を大きく圧迫することがあります。
     * この最小遅延時間を設定することで、アプリケーション開発者は
     * データセットが更新される最大の頻度を設定することが出来ます。
     *
     * 最小更新遅延時間は、データセットに許容される最小の更新時間です。
     * データセットは、この最小更新遅延時間が経過するまでは更新されることはありません。
     * もしも@ref DuplicatedObject::Update() がここで設定された時間よりも
     * 頻繁に呼び出されたときは、@ref DuplicatedObject::Update() の呼び出しがされなかったことになります。
     * ただし、連続性の切断(@ref IndicateContinuityBreak() )が起こった場合はこの限りではありません。
     *
     * この最小遅延時間は静的(static)な値であり、同一のデータセットは
     * 異なるインスタンスでも同一の最小遅延時間を持ちます。
     * ただし、データセットが異なれば、違う最小遅延時間を持ちます。
     *
     * @return  設定されている最小更新遅延時間が返ります。
     */
    static TimeInterval GetMinimumUpdateDelay();


    /*
    Summary: Predicts the value of an extrapolated variable based on current information.

    Parameters:
    v: Dataset extrapolation variable for which the prediction is done.  This
    value is obtained using the MODEL macro.
    tPredictionTime: The time for which the prediction is made, in milliseconds.
    If the time is not specified, the method predicts the value for the current time.

    Returns: When sccessful, a prediction of what the value of the referenced
    extrapolated variable will be at the specified time. Note that a valid input
    variable of type qDouble returns a bvalue of type qDouble while all valid integer
    input variables return a qFloat value.
    If called on a Duplication master, this method returns 0 and signals the error
    SYSTEMERROR_OBJDUP_INVALID_ON_MASTER.
    If called on a dataset that is not extrapolated, it returns 0 and signals the error
    SYSTEMERROR_GEN_INVALID_OPERATION.

    Remarks: This method can only be called on a Duplica. It is used to obtain the predicted
    values of a dataset (as given by its MODEL) at the specified time. A dead reckoning prediction is always
    based on the current value of a dataset. Predictions change as new values are received from
    the Duplication master.

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter*
    DDL property in its DDL declarations.

    See also: MODEL, PredictRateOfChange, PredictAcceleration
    */

    /*!
     * @brief   推定航法を行っている値を、現在の情報に基づいて予測して取得します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにextrapolation_filterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     *
     * この関数はデュプリカ(@ref DuplicatedObject::IsADuplica())のみが利用できます。
     * この関数を呼び出すと、指定された時間における、前もって予測されたデータセット(与えられたMODELとして)の
     * 値を得ることが出来ます。推定航法の事前予測は現在のデータセットの値に基づいて
     * 行われます。事前予測は複製マスター送られてくる新しい情報によって変化していきます。
     *
     * この関数が、複製マスター(@ref DuplicatedObject::IsADuplicationMaster())に
     * 所属するデータセットで実行された場合、SYSTEMERROR_OBJDUP_INVALID_ON_MASTERがシグナルされます。
     * この関数が、推定航法を利用していないデータセットで実行された場合、
     * SYSTEMERROR_GEN_INVALID_OPERATIONがシグナルされます。
     *
     * @param[in]   v   推定値を取得するデータセットの変数を指定します。これはMODELマクロによって取得できます。
     * @param[in]   tPredictionTime 推定値を取得する時刻を指定します。デフォルトでは現在のセッション時間になります。
     * @return  予測値が返ります。
     * @see     SessionClock::GetTime()
     */
    qDouble PredictValue(PHBDRVar_qDouble& v, Time tPredictionTime=SessionClock::GetTime());
    /*!
     * @brief   推定航法を行っている値を、現在の情報に基づいて予測して取得します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにextrapolation_filterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     *
     * この関数はデュプリカ(@ref DuplicatedObject::IsADuplica())のみが利用できます。
     * この関数を呼び出すと、指定された時間における、前もって予測されたデータセット(与えられたMODELとして)の
     * 値を得ることが出来ます。推定航法の事前予測は現在のデータセットの値に基づいて
     * 行われます。事前予測は複製マスター送られてくる新しい情報によって変化していきます。
     *
     * この関数が、複製マスター(@ref DuplicatedObject::IsADuplicationMaster())に
     * 所属するデータセットで実行された場合、SYSTEMERROR_OBJDUP_INVALID_ON_MASTERがシグナルされます。
     * この関数が、推定航法を利用していないデータセットで実行された場合、
     * SYSTEMERROR_GEN_INVALID_OPERATIONがシグナルされます。
     *
     * @param[in]   v   推定値を取得するデータセットの変数を指定します。これはMODELマクロによって取得できます。
     * @param[in]   tPredictionTime 推定値を取得する時刻を指定します。デフォルトでは現在のセッション時間になります。
     * @return  予測値が返ります。
     * @see     SessionClock::GetTime()
     */
    qFloat PredictValue(PHBDRVar_qFloat& v, Time tPredictionTime=SessionClock::GetTime());
    /*!
     * @brief   推定航法を行っている値を、現在の情報に基づいて予測して取得します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにextrapolation_filterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     *
     * この関数はデュプリカ(@ref DuplicatedObject::IsADuplica())のみが利用できます。
     * この関数を呼び出すと、指定された時間における、前もって予測されたデータセット(与えられたMODELとして)の
     * 値を得ることが出来ます。推定航法の事前予測は現在のデータセットの値に基づいて
     * 行われます。事前予測は複製マスター送られてくる新しい情報によって変化していきます。
     *
     * この関数が、複製マスター(@ref DuplicatedObject::IsADuplicationMaster())に
     * 所属するデータセットで実行された場合、SYSTEMERROR_OBJDUP_INVALID_ON_MASTERがシグナルされます。
     * この関数が、推定航法を利用していないデータセットで実行された場合、
     * SYSTEMERROR_GEN_INVALID_OPERATIONがシグナルされます。
     *
     * @param[in]   v   推定値を取得するデータセットの変数を指定します。これはMODELマクロによって取得できます。
     * @param[in]   tPredictionTime 推定値を取得する時刻を指定します。デフォルトでは現在のセッション時間になります。
     * @return  予測値が返ります。
     * @see     SessionClock::GetTime()
     */
    qFloat PredictValue(PHBDRVar_qInt32& v, Time tPredictionTime=SessionClock::GetTime());
    /*!
     * @brief   推定航法を行っている値を、現在の情報に基づいて予測して取得します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにextrapolation_filterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     *
     * この関数はデュプリカ(@ref DuplicatedObject::IsADuplica())のみが利用できます。
     * この関数を呼び出すと、指定された時間における、前もって予測されたデータセット(与えられたMODELとして)の
     * 値を得ることが出来ます。推定航法の事前予測は現在のデータセットの値に基づいて
     * 行われます。事前予測は複製マスター送られてくる新しい情報によって変化していきます。
     *
     * この関数が、複製マスター(@ref DuplicatedObject::IsADuplicationMaster())に
     * 所属するデータセットで実行された場合、SYSTEMERROR_OBJDUP_INVALID_ON_MASTERがシグナルされます。
     * この関数が、推定航法を利用していないデータセットで実行された場合、
     * SYSTEMERROR_GEN_INVALID_OPERATIONがシグナルされます。
     *
     * @param[in]   v   推定値を取得するデータセットの変数を指定します。これはMODELマクロによって取得できます。
     * @param[in]   tPredictionTime 推定値を取得する時刻を指定します。デフォルトでは現在のセッション時間になります。
     * @return  予測値が返ります。
     * @see     SessionClock::GetTime()
     */
    qFloat PredictValue(PHBDRVar_qInt16& v, Time tPredictionTime=SessionClock::GetTime());
    /*!
     * @brief   推定航法を行っている値を、現在の情報に基づいて予測して取得します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにextrapolation_filterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     *
     * この関数はデュプリカ(@ref DuplicatedObject::IsADuplica())のみが利用できます。
     * この関数を呼び出すと、指定された時間における、前もって予測されたデータセット(与えられたMODELとして)の
     * 値を得ることが出来ます。推定航法の事前予測は現在のデータセットの値に基づいて
     * 行われます。事前予測は複製マスター送られてくる新しい情報によって変化していきます。
     *
     * この関数が、複製マスター(@ref DuplicatedObject::IsADuplicationMaster())に
     * 所属するデータセットで実行された場合、SYSTEMERROR_OBJDUP_INVALID_ON_MASTERがシグナルされます。
     * この関数が、推定航法を利用していないデータセットで実行された場合、
     * SYSTEMERROR_GEN_INVALID_OPERATIONがシグナルされます。
     *
     * @param[in]   v   推定値を取得するデータセットの変数を指定します。これはMODELマクロによって取得できます。
     * @param[in]   tPredictionTime 推定値を取得する時刻を指定します。デフォルトでは現在のセッション時間になります。
     * @return  予測値が返ります。
     * @see     SessionClock::GetTime()
     */
    qFloat PredictValue(PHBDRVar_qInt8& v, Time tPredictionTime=SessionClock::GetTime());

    /*
    Summary: Predicts the rate of change of an extrapolated variable based on current information.
    Parameters:
    v: Dataset extrapolation variable for which prediction is done.  This
    value is obtained using the MODEL macro.
    tPredictionTime: The time for which the prediction is made, in milliseconds. If the time is not specified, the method
    predicts the rate of change at the current time.

    Remarks: This method predicts the rate of change, expressed in units per millisecond, of an
    extrapolated variable, based on current information and its extrapolation MODEL and the specified time.
    It can only be called on a Duplica.
    A dead reckoning prediction is always based on the current value of a dataset.
    Predictions change as new values are received from the duplication master.

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter*
    DDL property in its DDL declarations.

    Returns: When sccessful, returns the rate of change (expressed in units per millisecond)
    of the referenced extrapolated variable predicted for the specified time. Note that the
    value returned for a qDouble type variable is qDouble, but it returns a qFloat value for
    all permitted integer variable types.
    If called on a non-extrapolated dataset, this method returns 0 and signals
    the error SYSTEMERROR_GEN_INVALID_OPERATION. If called on a Duplication master,
    it returns 0 and signals a SYSTEMERROR_OBJDUP_INVALID_ON_MASTER.

    See also: MODEL, PredictValue, PredictAcceleration
    */

    /*!
     * @brief   推定航法を利用している変数の変化の割合を現在の情報に基づいて予測して取得します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにextrapolation_filterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     *
     * この関数は、推定航法を利用している変数のミリ秒単位ごとの変化の割合を
     * 現在の情報と推定モデルと与えられた時刻において予測します。
     * この関数はデュプリカ(@ref DuplicatedObject::IsADuplica())のみが利用できます。
     * 推定航法の事前予測は現在のデータセットの値に基づいて
     * 行われます。事前予測は複製マスター送られてくる新しい情報によって変化していきます。
     *
     * @param[in]   v   推定値を取得するデータセットの変数を指定します。これはMODELマクロによって取得できます。
     * @param[in]   tPredictionTime 推定値を取得する時刻を指定します。デフォルトでは現在のセッション時間になります。
     * @return  予測された変化の割合が返ります。
     * @see     SessionClock::GetTime()
     */
    qDouble PredictRateOfChange(PHBDRVar_qDouble& v, Time tPredictionTime=SessionClock::GetTime());
    /*!
     * @brief   推定航法を利用している変数の変化の割合を現在の情報に基づいて予測して取得します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにextrapolation_filterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     *
     * この関数は、推定航法を利用している変数のミリ秒単位ごとの変化の割合を
     * 現在の情報と推定モデルと与えられた時刻において予測します。
     * この関数はデュプリカ(@ref DuplicatedObject::IsADuplica())のみが利用できます。
     * 推定航法の事前予測は現在のデータセットの値に基づいて
     * 行われます。事前予測は複製マスター送られてくる新しい情報によって変化していきます。
     *
     * @param[in]   v   推定値を取得するデータセットの変数を指定します。これはMODELマクロによって取得できます。
     * @param[in]   tPredictionTime 推定値を取得する時刻を指定します。デフォルトでは現在のセッション時間になります。
     * @return  予測された変化の割合が返ります。
     * @see     SessionClock::GetTime()
     */
    qFloat PredictRateOfChange(PHBDRVar_qFloat& v, Time tPredictionTime=SessionClock::GetTime());
    /*!
     * @brief   推定航法を利用している変数の変化の割合を現在の情報に基づいて予測して取得します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにextrapolation_filterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     *
     * この関数は、推定航法を利用している変数のミリ秒単位ごとの変化の割合を
     * 現在の情報と推定モデルと与えられた時刻において予測します。
     * この関数はデュプリカ(@ref DuplicatedObject::IsADuplica())のみが利用できます。
     * 推定航法の事前予測は現在のデータセットの値に基づいて
     * 行われます。事前予測は複製マスター送られてくる新しい情報によって変化していきます。
     *
     * @param[in]   v   推定値を取得するデータセットの変数を指定します。これはMODELマクロによって取得できます。
     * @param[in]   tPredictionTime 推定値を取得する時刻を指定します。デフォルトでは現在のセッション時間になります。
     * @return  予測された変化の割合が返ります。
     * @see     SessionClock::GetTime()
     */
    qFloat PredictRateOfChange(PHBDRVar_qInt32& v, Time tPredictionTime=SessionClock::GetTime());
    /*!
     * @brief   推定航法を利用している変数の変化の割合を現在の情報に基づいて予測して取得します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにextrapolation_filterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     *
     * この関数は、推定航法を利用している変数のミリ秒単位ごとの変化の割合を
     * 現在の情報と推定モデルと与えられた時刻において予測します。
     * この関数はデュプリカ(@ref DuplicatedObject::IsADuplica())のみが利用できます。
     * 推定航法の事前予測は現在のデータセットの値に基づいて
     * 行われます。事前予測は複製マスター送られてくる新しい情報によって変化していきます。
     *
     * @param[in]   v   推定値を取得するデータセットの変数を指定します。これはMODELマクロによって取得できます。
     * @param[in]   tPredictionTime 推定値を取得する時刻を指定します。デフォルトでは現在のセッション時間になります。
     * @return  予測された変化の割合が返ります。
     * @see     SessionClock::GetTime()
     */
    qFloat PredictRateOfChange(PHBDRVar_qInt16& v, Time tPredictionTime=SessionClock::GetTime());
    /*!
     * @brief   推定航法を利用している変数の変化の割合を現在の情報に基づいて予測して取得します。
     *
     * ※DDL(データ定義言語)宣言にて、このデータセットにextrapolation_filterプロパティが
     * ついているときのみ、DDLコンパイラによってこの関数が生成されます。
     *
     * この関数は、推定航法を利用している変数のミリ秒単位ごとの変化の割合を
     * 現在の情報と推定モデルと与えられた時刻において予測します。
     * この関数はデュプリカ(@ref DuplicatedObject::IsADuplica())のみが利用できます。
     * 推定航法の事前予測は現在のデータセットの値に基づいて
     * 行われます。事前予測は複製マスター送られてくる新しい情報によって変化していきます。
     *
     * @param[in]   v   推定値を取得するデータセットの変数を指定します。これはMODELマクロによって取得できます。
     * @param[in]   tPredictionTime 推定値を取得する時刻を指定します。デフォルトでは現在のセッション時間になります。
     * @return  予測された変化の割合が返ります。
     * @see     SessionClock::GetTime()
     */
    qFloat PredictRateOfChange(PHBDRVar_qInt8& v, Time tPredictionTime=SessionClock::GetTime());

    /*
    Summary: Predicts the acceleration of an extrapolated variable based on current information.

    Parameters:
    v: Dataset extrapolation variable for which prediction is done.  This
    value is obtained using the MODEL macro.
    tPredictionTime: The time for which the prediction is made, in milliseconds. If the time is not specified, the method
    will predict the acceleration at the current time.

    Remarks: This method predicts the acceleration, expressed in units per millisecond^2, of an
    extrapolated variable based on current information, its extrapolation MODEL and the specified time.

    The method can only be called on a Duplica. A dead reckoning prediction is always based
    on the current value of the dataset. Predictions change as new values are received from
    the Duplication master.

    $PRODUCTNAME$ uses a first or second order function to predict variable
    values and therefore, the acceleration is constant. However, when convergence
    occurs, at the point where the convergence curve meets the tracking
    curve a change in acceleration may occur.

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter*
    DDL property in its DDL declarations.

    Returns: When sccessful, returns the acceleration of the referenced extrapolated
    variable expressed in units per millisecond predicted for the specified time.
    If called on a dataset that has not been extrapolated, this method returns 0 and signals
    the error SYSTEMERROR_GEN_INVALID_OPERATION. If called on a Duplication master,
    it returns 0 and signals the error SYSTEMERROR_OBJDUP_INVALID_ON_MASTER.
    See also: MODEL, PredictValue, PredictRateOfChange
    */
    qDouble PredictAcceleration(PHBDRVar_qDouble& v, Time tPredictionTime=SessionClock::GetTime());
    qFloat PredictAcceleration(PHBDRVar_qFloat& v, Time tPredictionTime=SessionClock::GetTime());
    qFloat PredictAcceleration(PHBDRVar_qInt32& v, Time tPredictionTime=SessionClock::GetTime());
    qFloat PredictAcceleration(PHBDRVar_qInt16& v, Time tPredictionTime=SessionClock::GetTime());
    qFloat PredictAcceleration(PHBDRVar_qInt8& v, Time tPredictionTime=SessionClock::GetTime());

    //group=Dataset Loopback Methods

    /*
    Description: Sets the loopback delay for this dataset.
    Parameters:
    tiLoopbackDelay: The time delay, in milliseconds, at which the loopback on the Duplication master
    will be calculated. The default value is 0ms.
    Remarks: The tiLoopbackDelay is equivalent to the point in time in the past ( /i.e./ the number
    of milliseconds before the current time) for which the dataset will be extrapolated.
    A dataset must use an extrapolation filter to use loopback.

    To ensure synchronization between a Duplication master and its Duplicas, loopback should be used in
    conjunction with dead reckoning with an extrapolation delay applied, which is set via
    PHBDRParameters::SetExtrapolationDelay. If not, the degree of synchronization will be sensitive to
    variations in latency. An extrapolation delay ensures that regardless of the time a message takes to
    traverse the network the time that the outcome is calculated on the duplication master and on the
    duplica is consistent. For perfect synchronization, the extrapolation delay and loopback delay
    should be set to the same value.

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter* and
    the *loopback* DDL property in its DDL declarations.

    See Also: GetLoopbackDelay, Dead Reckoning Overview, Loopback
    */


    /*! @brief datasetのループバック遅延を指定します。
     *
     *  @param[in] tiLoopbackDelay ループバック遅延時間をミリ秒単位で設定します。デフォルト値は、0msecです。
     *
     *  @see GetLoopbackDelay()
     */

    static void SetLoopbackDelay(TimeInterval tiLoopbackDelay);

    /*
    Description: Returns the loopback delay for this dataset.
    Returns: The loopback delay of this dataset as a TimeInterval in milliseconds.

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter* and
    the *loopback* DDL property in its DDL declarations.

    See Also: SetLoopbackDelay, Loopback
    */

    /*! @brief datasetのループバック遅延を取得します。
     *
     *  @return ループバック遅延時間をミリ秒単位で取得します。
     *
     *  @see SetLoopbackDelay()
     */
    static TimeInterval GetLoopbackDelay();

    /*
    Description: Sets the sampling interval for the Loopback.
    Parameters:
    tiSamplingInterval: The interval, in milliseconds, at which the actual values of the dataset
    will be stored in a queue. The stored values will then be used to extrapolate the
    dataset values at the time specified via SetLoopbackDelay.
    The default value is 50ms, which is equivalent to sampling 20 points per second.
    Remarks:

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter* and
    the *loopback* DDL property in its DDL declarations.

    See Also: GetLoopbackSamplingInterval, Dead Reckoning Overview, Loopback
    */

    /*! @brief ループバックのサンプリング間隔を設定します。
     *
     *  すべての値をループバックのためにキューに保管するのではなく、
     *  サンプリング値が保管されます。
     *  保存された値は、SetLoopbackDelay後に反映されます。
     *
     *  @param[in] tiSamplingInterval サンプリング間隔をミリ秒単位で設定します。
     *             デフォルト値は、50msecで、1秒あたり20ポイントのサンプリングとなります。
     *
     * @see GetLoopbackSamplingInterval()
     */


    void SetLoopbackSamplingInterval(TimeInterval tiSamplingInterval);

    /*
    Description: Returns the loopback sampling interval for this dataset.
    Returns: The loopback sampling interval of this dataset as a TimeInterval in milliseconds.

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter* and
    the *loopback* DDL property in its DDL declarations.

    See Also: SetLoopbackSamplingInterval, Loopback
    */

    /*!
     *  @brief ループバックのサンプリング間隔を取得します。
     *
     *  @return サンプリング間隔をミリ秒単位で取得します。
     *
     *  @see SetLoopbackSamplingInterval()
     */
    static TimeInterval GetLoopbackSamplingInterval();

    /*
    Description: Updates the LOOPBACKMODEL with the current values of the dataset.
    Remarks: This method can only be called on a Duplication master. When a dataset changes,
    to update the LOOPBACKMODEL on the Duplication master you need to call UpdateLoopback
    to add the new values to the model, then RefreshLoopback to recalculate the model using
    the new values. You can then use the PredictLoopbackValue, or similar to return the
    values predicted by the LOOPBACKMODEL at the specified time. For example, say you use Loopback
    on the Position dataset, then on the Duplication master you would typically call
    something like the following:

            // Set the new values in the Position dataset
            pObject->SetPosition(oNewPosition);
            // Update the values. This will trigger a network message to the duplicas
            pObject->Update();
            // Update the values in the LOOPBACKMODEL
            pObject->m_dsPosition.UpdateLoopback();
            // Recalculate the LOOPBACKMODEL
            pObject->m_dsPosition.RefreshLoopback();
            // Return the new predicted loopbacked values, where GetLoopbackPosition uses the
            // PredictLoopbackValue method to return the predicted values for the Position dataset.
            Vector3D vLoopbackedPosition = pObject->GetLoopbackPosition();

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter* and
    the *loopback* DDL property in its DDL declarations.

    See Also: RefreshLoopback
    */

    /*!
     * @brief datasetの現在の値で、LOOPBACKMODELを更新します。
     *
     * DO マスターからのみ呼び出し可能です。
     * LOOPBACKMODELの再計算には、RefreshLoopback()を呼び出します。
     *
     * @see RefreshLoopback()
     */

    void UpdateLoopback();

    /*
    Description: Recalculates the LOOPBACKMODEL using the updated values.
    Returns: true if the variables of the LOOPBACKMODEL were successfully recalculated, false if not or an
    error occurred.
    Remarks: This method can only be called on a Duplication master. When a dataset changes,
    to update the LOOPBACKMODEL on the Duplication master you need to call UpdateLoopback
    to add the new values to the model, then RefreshLoopback to recalculate the model using
    the new values. You can then use the PredictLoopbackValue, or similar to return the
    values predicted by the LOOPBACKMODEL at the specified time. Refer to UpdateLoopback
    for an example of this method in use.

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter* and
    the *loopback* DDL property in its DDL declarations.

    See Also: UpdateLoopback
    */

    /*!
     * @brief 更新された値で、LOOPBACKMODELを再計算します。
     * @return true 再計算が成功した場合 false 再計算が失敗した場合。
     *
     * DO マスターからのみ呼び出し可能です。
     * UpdateLoopback()によるLOOPBACKMODELの更新後に呼び出します。
     *
     * @see UpdateLoopback()
     */

    qBool RefreshLoopback();

    /*
    Summary: Predicts the value of a loopbacked variable based on the current LOOPBACKMODEL at the specified time.

    Parameters:
    v: The dataset extrapolation variable for which the prediction is made.  This
    value is obtained using the LOOPBACKMODEL macro.

    tPredictionTime: The time for which the prediction is made, in milliseconds. If the time is not specified, the method
    predicts the value at the current time.

    Returns: When successful, returns the predicted value of the referenced loopback variable at the specified time based on the
    current LOOPBACKMODEL.
    If called on a dataset that that does not use loopback, this method returns 0 and signals
    the error SYSTEMERROR_GEN_INVALID_OPERATION. If called on a Duplica,
    it returns 0 and signals the error SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA.

    Remarks: This method can only be called on a Duplication master. It is used to obtain what the predicted
    values of a dataset will be, as given by its LOOPBACKMODEL, at the specified time.
    For example, in the Duplicated3DObject class of the Real-time 3D Framework, we use this method
    to return the values of the LOOPBACKMODEL as follows. As we do not specify a tPredictionTime, the
    dataset values are predicted at the time given by the current loopback delay.

            Vector3D Duplicated3DObject::GetLoopbackPosition() {
                return Vector3D(m_dsPosition.PredictLoopbackValue(LOOPBACKMODEL(m_dsPosition,m_fX)),
                                m_dsPosition.PredictLoopbackValue(LOOPBACKMODEL(m_dsPosition,m_fY)),
                                m_dsPosition.PredictLoopbackValue(LOOPBACKMODEL(m_dsPosition,m_fZ)));
            }

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter* and
    the *loopback* DDL property in its DDL declarations.

    See also: LOOPBACKMODEL, UpdateLoopback, RefreshLoopback, PredictLoopbackRateOfChange, PredictLoopbackAcceleration
    */

    /*!
     * @brief ループバックモデルから、指定された時間のループバックされた値を予測します。
     *
     * m_dsPosition.PredictLoopbackValue(LOOPBACKMODEL(m_dsPosition,m_fX)）というように、LOOPBACKMODELマクロを利用します。
     *
     * @param[in] v datasetの推定値を指定します。LOOPBACKMODELマクロを使って指定します。
     * @param[in] tPredictionTime 予測する時点の時間を指定します。指定されないと、現在の時間となります。
     *
     *
     */
    qDouble PredictLoopbackValue(PHBDRVar_qDouble& v, Time tPredictionTime=SessionClock::GetTime());
    /*!
     * @brief ループバックモデルから、指定された時間のループバックされた値を予測します。
     *
     * m_dsPosition.PredictLoopbackValue(LOOPBACKMODEL(m_dsPosition,m_fX)）というように、LOOPBACKMODELマクロを利用します。
     *
     * @param[in] v datasetの推定値を指定します。LOOPBACKMODELマクロを使って指定します。
     * @param[in] tPredictionTime 予測する時点の時間を指定します。指定されないと、現在の時間となります。
     *
     *
     */
    qFloat PredictLoopbackValue(PHBDRVar_qFloat& v, Time tPredictionTime=SessionClock::GetTime());
    /*!
     * @brief ループバックモデルから、指定された時間のループバックされた値を予測します。
     *
     * m_dsPosition.PredictLoopbackValue(LOOPBACKMODEL(m_dsPosition,m_fX)）というように、LOOPBACKMODELマクロを利用します。
     *
     * @param[in] v datasetの推定値を指定します。LOOPBACKMODELマクロを使って指定します。
     * @param[in] tPredictionTime 予測する時点の時間を指定します。指定されないと、現在の時間となります。
     *
     *
     */
    qFloat PredictLoopbackValue(PHBDRVar_qInt32& v, Time tPredictionTime=SessionClock::GetTime());
    /*!
     * @brief ループバックモデルから、指定された時間のループバックされた値を予測します。
     *
     * m_dsPosition.PredictLoopbackValue(LOOPBACKMODEL(m_dsPosition,m_fX)）というように、LOOPBACKMODELマクロを利用します。
     *
     * @param[in] v datasetの推定値を指定します。LOOPBACKMODELマクロを使って指定します。
     * @param[in] tPredictionTime 予測する時点の時間を指定します。指定されないと、現在の時間となります。
     *
     *
     */
    qFloat PredictLoopbackValue(PHBDRVar_qInt16& v, Time tPredictionTime=SessionClock::GetTime());
    /*!
     * @brief ループバックモデルから、指定された時間のループバックされた値を予測します。
     *
     * m_dsPosition.PredictLoopbackValue(LOOPBACKMODEL(m_dsPosition,m_fX)）というように、LOOPBACKMODELマクロを利用します。
     *
     * @param[in] v datasetの推定値を指定します。LOOPBACKMODELマクロを使って指定します。
     * @param[in] tPredictionTime 予測する時点の時間を指定します。指定されないと、現在の時間となります。
     *
     *
     */
    qFloat PredictLoopbackValue(PHBDRVar_qInt8& v, Time tPredictionTime=SessionClock::GetTime());

    /*
    Summary: Predicts the rate of change of an extrapolated variable based on current information.

    Parameters:
    v: Dataset extrapolation variable for which the prediction is done.  This value is obtained
    using the LOOPBACKMODEL macro.

    tPredictionTime: The time for which the prediction is made, in milliseconds. If the time is not specified, the method
    predicts the value at the current time.

    Returns: When successful, returns the predicted rate of change of an extrapolated variable at the secified time
    based on current information. If called on a dataset that does not use loopback, this method returns 0 and signals the error
    SYSTEMERROR_GEN_INVALID_OPERATION. If called on a Duplica, it returns 0 and signals the error
    SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA.

    Remarks: This method predicts the rate of change, expressed in units per millisecond, of an
    extrapolated variable, based on current information, its LOOPBACKMODEL and the specified time. It can only
    be called on a Duplication master.

    For an example of how to use this method, refer to PredictLoopbackValue.

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter* and
    the *loopback* DDL property in its DDL declarations.

    See also: LOOPBACKMODEL, PredictLoopbackValue, PredictLoopbackAcceleration
    */
    qDouble PredictLoopbackRateOfChange(PHBDRVar_qDouble& v, Time tPredictionTime=SessionClock::GetTime());
    qFloat PredictLoopbackRateOfChange(PHBDRVar_qFloat& v, Time tPredictionTime=SessionClock::GetTime());
    qFloat PredictLoopbackRateOfChange(PHBDRVar_qInt32& v, Time tPredictionTime=SessionClock::GetTime());
    qFloat PredictLoopbackRateOfChange(PHBDRVar_qInt16& v, Time tPredictionTime=SessionClock::GetTime());
    qFloat PredictLoopbackRateOfChange(PHBDRVar_qInt8& v, Time tPredictionTime=SessionClock::GetTime());

    /*
    Summary: Predicts the acceleration of an extrapolated variable based on current information.

    Parameters:
    v: Dataset extrapolation variable for which the prediction is done.  This
    value is obtained using the LOOPBACKMODEL macro.
    tPredictionTime: The time for which the prediction is made, in milliseconds. If the time is not specified, the method
    predicts the value at the current time.

    Returns: The acceleration of the specified extrapolated variable at the specified time based on current information.
    If called on a dataset that does not use loopback, this method returns 0 and signals the error
    SYSTEMERROR_GEN_INVALID_OPERATION. If called on a Duplica, it returns 0 and signals the error
    SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA.

    Remarks: This method predicts the acceleration, expressed in units per millisecond^2, of an
    extrapolated variable based on current information, its LOOPBACKMODEL and the specified time. It can only be
    called on a Duplication master.

    $PRODUCTNAME$ uses a first or second order function to predict variable
    values and therefore, the acceleration is constant. However, when convergence
    occurs, at the point where the convergence curve meets the tracking
    curve a change in acceleration may occur.

    For an example of how to use this method, refer to PredictLoopbackValue.

    Note:  This method is only generated for a dataset that includes the *extrapolation_filter* and
    the *loopback* DDL property in its DDL declarations.

    See also: LOOPBACKMODEL, PredictLoopbackValue, PredictLoopbackRateOfChange
    */
    qDouble PredictLoopbackAcceleration(PHBDRVar_qDouble& v, Time tPredictionTime=SessionClock::GetTime());
    qFloat PredictLoopbackAcceleration(PHBDRVar_qFloat& v, Time tPredictionTime=SessionClock::GetTime());
    qFloat PredictLoopbackAcceleration(PHBDRVar_qInt32& v, Time tPredictionTime=SessionClock::GetTime());
    qFloat PredictLoopbackAcceleration(PHBDRVar_qInt16& v, Time tPredictionTime=SessionClock::GetTime());
    qFloat PredictLoopbackAcceleration(PHBDRVar_qInt8& v, Time tPredictionTime=SessionClock::GetTime());
#endif //FOR_DOC_PURPOSE

    // {secret}
    void InitVariables() {};

    /*
    Description: Returns whether or not the Dataset uses Loopback.
    Returns: true if the loopback DDL property was specified for this dataset, false if not.
    Remarks: To use Loopback, the loopback and extrapolation_filter dataset properties must be included in
    the DDL declarations of the dataset.
    See Also: DDL File Syntax, Dead Reckoning Overview, Loopback
    */
    qBool HasLoopback() const
    {
        return false;
    }

    // {secret}
    // The base implementation does not do anything.
    static void RegisterCounters(qUnsignedInt32 /*uiOp*/) {};

    /*!
     * @brief Reliableの送信時で利用するSubStreamIDを指定します。
     *
     * デフォルトは SubStreamIDDefine::SYSTEMで、システムと共用です。
     *
     * @param[in] subStreamID 送信するSubStreamID
     *
     * @see SubStreamIDDefine, StreamSettings::SetMaxUserReliableSubStreams()
     */
    void SetSubStreamID(SubStreamID subStreamID)
    {
        m_subStreamID = subStreamID;
    }

    /*!
     * @brief Reliableの送信時で利用するSubStreamIDを取得します。
     *
     * デフォルトは SubStreamIDDefine::SYSTEMで、システムと共用です。
     *
     * @return 送信するSubStreamID
     *
     * @see SubStreamIDDefine, StreamSettings::SetMaxUserReliableSubStreams()
     */
    SubStreamID GetSubStreamID() const
    {
        return m_subStreamID;
    }


    void SetRecvSubStreamID(SubStreamID subStreamID)
    {
        m_RecvSubStreamID = subStreamID;
    }

    /*!
     * @brief Reliableの受信時に利用されたSubStreamIDを取得します。
     *
     * デフォルトは SubStreamIDDefine::SYSTEMで、システムと共用です。
     *
     * @return 受信時のSubStreamID
     *
     * @see SubStreamIDDefine, StreamSettings::SetMaxUserReliableSubStreams()
     */
    SubStreamID GetRecvSubStreamID() const
    {
        return m_RecvSubStreamID;
    }


protected:

#ifdef NEX_DEBUG_VERSION
    DuplicatedObject* m_pDOContainer;
#endif

    SubStreamID m_subStreamID;
    SubStreamID m_RecvSubStreamID;
};



END_NAMESPACE_NEX

