﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Platform/Core/PerfCounter.h>

#ifdef NEX_PERFCOUNTER_ENABLED

BEGIN_NAMESPACE_NEX

//Group= Performance Counters

/*
Description: This enumeration lists the DataSet performance counters that are recorded.
*/
enum DatasetPerfCounterType
{
    /*
    Description: Records the frequency of update calls that
    triggered at least one update message.
    Remarks: If an update generates several messages since it has been discovered
    by several stations, this counts as just one update multicast.
    */
    PERF_DATASET_UPDATE_MULTICASTS = 0,

    /*
    Description: Records the frequency at which update messages are sent.
    Remarks: When only two stations belong to the session, the frequency of
    this counter is identical to the PERF_DATASET_UPDATE_MULTICASTS frequency.
    When two or more stations belong to the session, the frequency of this
    counter will typically be higher as some calls to DuplicatedObject::Update
    will result in a multicast update message sent to several stations.
    */
    PERF_DATASET_UPDATE_MESSAGES,

    // {Secret}
    PERF_DATASET_HIGHEST_ID = PERF_DATASET_UPDATE_MESSAGES
};

/*
Description: Contains the performance counters associated with the dataset class.
Remarks: These performance counters can be accessed by means of DataSet::GetPerformanceCounters.
A DataSetPerfCounters class is created by $PRODUCTNAME$ for each
DataSet declared in the DDL file that specifies the update_perf_counter DDL property.
The performance counters class can be accessed via the static method DataSet::GetPerformanceCounters.
If you try to access the performance counters for a dataset that did not declare the update_perf_counter
DDL property a compiler error will occur.

For example, if the following DataSet is declared in the DDL:

        dataset Speed {
            double sx;
            double sy;
        } update_perf_counter ;

Then, the following call can be made to access the
performance counters associated with the Speed dataset:

        DataSetPerfCounters *pSpeedPerfCounters=Speed::GetPerformanceCounters();

See also: DataSet::GetPerformanceCounters, Dataset DDL File Syntax
*/
class DataSetPerfCounters: public RootObject
{
    //group=Methods
public:

    /*
    Remarks: A DataSetPerfCounters object should never be directly instantiated. Instead, it is created by
    $PRODUCTNAME$ before a session is created. To access the DataSetPerfCounters object from within your
    game use the DataSet::GetPerformanceCounters method.
    */
    DataSetPerfCounters(const qChar* szDataSet);

    /*
    Remarks: A DataSetPerfCounters object should never be directly destroyed. It is destroyed by $PRODUCTNAME$
    when the session terminates.
    */
    virtual ~DataSetPerfCounters();

    // {Secret}
    void Register();
    // {Secret}
    void Unregister();

    // {Secret}
    void Update(qUnsignedInt32 uiNbSent);

    //Summary: Traces the values of the dataset performance counters to the TraceLog.
    void Trace();

    //Summary: Resets all the dataset performance counters to zero.
    void Reset();

    /*
    Summary: Returns a performance value.
    Parameters:
    eCounterID: Queried performance counter
    eValue: Requested value.
    Returns: The requested performance value if the call succeeded or 0 and SYSTEMERROR_GEN_OUT_OF_RANGE if an error occurred.
    Remarks: The following counters are available: PERF_DATASET_UPDATE_MULTICASTS and PERF_DATASET_UPDATE_MESSAGES. For
    each counter, the following values can be queried: PERF_FREQUENCY_LAST,
    PERF_FREQUENCY_AVERAGE, PERF_FREQUENCY_MIN, PERF_FREQUENCY_MAX and PERF_COUNT_TOTAL.

    See Also: DataSet::GetPerformanceCounters, TransportPerfCounters, RootTransport::GetPerfCounters
    */
    qInt GetValue(
        DatasetPerfCounterType eCounterID,
        PerfCounterValueType eValue);

private:
    void RegisterCounter(PerfCounter** ppCounter, const qChar* szName) const;
    PerfCounter* GetCounter(qInt iCounterID);

private:

    String m_strDataSet;
    PerfCounter* m_ppcPerfCounter[3];
    qBool m_bRegistered;
};

END_NAMESPACE_NEX

#endif // NEX_PERFCOUNTER_ENABLED

