﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/Station.h>
#include <OnlineCore/src/Core/SystemComponent.h>

#include <OnlineCore/src/Transport/PRUDP/PRUDPUserProtocol.h>

BEGIN_NAMESPACE_NEX

/*!
    @brief 複製オブジェクトを使用しない単純な送受信（ダイレクトトランスポート）機能を扱うクラスです。

    本機能は互換性維持のために残っています。使用は非推奨となります。
    代わりにダイレクトストリーム(@ref DirectStream)を使用してください。
 */
class DirectTransport: public SystemComponent
{
public:
    NEX_DECLARE_SYSCOMPONENT_METHODS(DirectTransport,SystemComponent);

    /*!
    @brief このコンストラクタはアプリケーション側から直接呼び出さないでください。DirectTransportクラスのインスタンスは、システムによって自動的に生成されます。
     */
    DirectTransport();

    /*!
    @brief このデストラクタはアプリケーション側から直接呼び出さないでください。DirectTransportクラスのインスタンスは、システムによって自動的に破棄されます。
     */
    virtual ~DirectTransport();

    /*!
    @brief DirectTransportのインスタンスを取得します。

    @return DirectTransportオブジェクトへのポインタが返ります。
     */
    static DirectTransport* GetInstance();

public:
    /*!
    @brief ダイレクトトランスポート機能でデータを受信した際に呼び出されるコールバック関数を登録します。

    到達保証の有無に関わらず、他ステーションから送信されたデータが受信コールバックで渡されます。<br>
    登録するコールバック関数は以下の仕様に従ってください。

    void UserReceiveCallback(StationID sourceId, qByte* buffer, qUnsignedInt32 size)<br>
    ・sourceId ： 送信元ステーションのStationID<br>
    ・buffer ： 受信したデータのポインタ<br>
    ・size ： 受信したデータのサイズ

    受信コールバック関数の処理時間が長すぎると受信が安定しなくなる可能性があります。

    @param[in] callback コールバック関数へのポインタを指定します。
     */
    void RegisterUserReceiveCallback( PRUDPUserReceiveCallback callback );

    /*!
    @brief 登録されている受信コールバック関数を登録解除します。
     */
    void UnregisterUserReceiveCallback();

    /*
    n1589:Scheduler::Dispatchを呼ぶスレッドでのみ使用することができます
    トランスポートスレッドモードではRegisterUserReceiveCallbackでセットした
    コールバックはトランスポートスレッドから呼ばれますので
    コールバック内で直接送受信APIを呼ぶことは禁止されています
    */

    /*!
    @brief 到達保証性のない通信で、宛先で指定したステーションにデータを送信します。

    StationIDは@ref Station::GetStationID で取得できます。

    パケットバッファに空きがない場合PacketBufferFullエラーを返します。
    これは一時的にパケットバッファが不足していることを示すエラーであるため、
    必要に応じてパケットバッファを増やしてください。パケットバッファの設定については、@ref PacketBufferManager を参照してください。

    @param[in] destId 宛先のステーションのStationID。
    @param[in] buffer 送信するデータのポインタ。
    @param[in] size 送信するデータのサイズ。1250以下となるようにしてください。

    @retval QRESULT_SUCCESS                               送信に成功した場合
    @retval "QRESULT_ERROR(Transport, Unknown)"           PRUDPUserProtocol::GetInstance()がNULLの場合
    @retval "QRESULT_ERROR(Core, InvalidArgument)"        destId が誤っていた場合
    @retval "QRESULT_ERROR(Transport, ConnectionFailure)" 宛先Stationとの接続が確立されていなかった場合
    @retval "QRESULT_ERROR(Transport, PacketBufferFull)"    パケットバッファ枯渇により送信できなかった場合
     */
    qResult SendUnreliable(StationID destId, qByte* buffer, qUnsignedInt32 size);

    /*!
    @brief 到達保証性のない通信で、セッションに参加している全てのステーションにデータを送信します。

    パケットバッファに空きがない場合PacketBufferFullエラーを返します。
    これは一時的にパケットバッファが不足していることを示すエラーであるため、
    必要に応じてパケットバッファを増やしてください。パケットバッファの設定については、@ref PacketBufferManager を参照してください。


    @param[in] buffer 送信するデータのポインタ。
    @param[in] size 送信するデータのサイズ。1250以下となるようにしてください。

    @retval QRESULT_SUCCESS                               送信に成功した場合
    @retval "QRESULT_ERROR(Transport, Unknown)"           PRUDPUserProtocol::GetInstance()がNULLの場合
    @retval "QRESULT_ERROR(Transport, ConnectionFailure)" データの送信に成功したStationが一台も存在しなかった場合
    @retval "QRESULT_ERROR(Transport, PacketBufferFull)"    パケットバッファ枯渇により送信できなかった場合
    */
    qResult SendUnreliableAll(qByte* buffer, qUnsignedInt32 size);

    /*!
    @brief 到達保証性のある通信で、宛先で指定したステーションにデータを送信します。

    StationIDは@ref Station::GetStationID で取得できます。

    パケットバッファに空きがない場合PacketBufferFullエラーを返します。
    これは一時的にパケットバッファが不足していることを示すエラーであるため、
    ディスパッチを行い時間をおいてから再度送信を行ってください。
    高頻度でPacketBufferFullが発生する場合は、パケットバッファを増やす対策も行ってください。
    パケットバッファの設定については、@ref PacketBufferManager を参照してください。

    @param[in] destId 宛先のステーションのStationID。
    @param[in] buffer 送信するデータのポインタ。
    @param[in] size 送信するデータのサイズ。

    @retval QRESULT_SUCCESS                               送信に成功した場合
    @retval "QRESULT_ERROR(Transport, Unknown)"           PRUDPUserProtocol::GetInstance()がNULLの場合
    @retval "QRESULT_ERROR(Core, InvalidArgument)"        destId が誤っていた場合
    @retval "QRESULT_ERROR(Transport, ConnectionFailure)" 宛先Stationとの接続が確立されていなかった場合
    @retval "QRESULT_ERROR(Transport, PacketBufferFull)"    パケットバッファ枯渇により送信できなかった場合
     */
    qResult SendReliable(StationID destId, qByte* buffer, qUnsignedInt32 size);

    /*!
    @brief 到達保証性のある通信で、セッションに参加している全てのステーションにデータを送信します。

    パケットバッファに空きがない場合PacketBufferFullエラーを返します。
    これは一時的にパケットバッファが不足していることを示すエラーであるため、
    ディスパッチを行い時間をおいてから再度送信を行ってください。
    高頻度でPacketBufferFullが発生する場合は、パケットバッファを増やす対策も行ってください。
    パケットバッファの設定については、@ref PacketBufferManager を参照してください。

    @param[in] buffer 送信するデータのポインタ。
    @param[in] size 送信するデータのサイズ。

    @retval QRESULT_SUCCESS                               送信に成功した場合
    @retval "QRESULT_ERROR(Transport, Unknown)"           PRUDPUserProtocol::GetInstance()がNULLの場合
    @retval "QRESULT_ERROR(Transport, ConnectionFailure)" データの送信に成功したStationが一台も存在しなかった場合
    @retval "QRESULT_ERROR(Transport, PacketBufferFull)"    パケットバッファ枯渇により送信できなかった場合
     */
    qResult SendReliableAll(qByte* buffer, qUnsignedInt32 size);
};

END_NAMESPACE_NEX

