﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DOOperation.h>
#include <NetZ/src/Core/DOCore/Station.h>
#include <OnlineCore/src/Core/LogicalClock.h>

BEGIN_NAMESPACE_NEX
//Group = System Operations

/*
Description: This DOOperation is performed every time a fault is detected on a station.
Remarks: When a station fails, a FaultRecoveryOperation is performed on the session master for each duplicated object
whose Duplication master resided on the failed station.

    o   The SafetyExecutive is called to determine if the object should survive the fault. If so, we continue, if not we exit.
    o   The method DuplicatedObject::OperationBegin is called,
    o   The method DuplicatedObject::ApproveFaultRecovery is called.
    o   If DuplicatedObject::ApproveFaultRecovery returns true, the session master takes control of the duplicated
    object via the following sequence of events:
        o   The ChangeMasterStationOperation is performed,
        o   The method DuplicatedObject::OperationEnd is called.
    o   If DuplicatedObject::ApproveFaultRecovery returns false the object is removed from the store via the following
    sequence of events:
        o   The RemoveFromStoreOperation is triggered,
        o   The method DuplicatedObject::OperationEnd is called,
        o   The object destructor is called.

*/

/*!
 * @brief   このDOOperationはステーションに障害が発生するたびに実行されます。
 *
 *          ステーション(@ref Station)が脱落すると、FaultRecoveryOperationが、脱落したステーションが
 *          複製マスターとなっているそれぞれの複製オブジェクトに対して実行されます。
 *
 *          以下のようなシーケンスによって実行されます。
 *
 *          ・@ref SafetyExecutive が呼び出され、この操作を行ってもよいかを決定します。
 *          　実行してもよければオブジェクトを障害から救い出し、そうでなければ終了します。<br>
 *          ・@ref DuplicatedObject::OperationBegin() が呼び出されます。<br>
 *          ・@ref DuplicatedObject::ApproveFaultRecovery() が呼び出されます。<br>
 *          ・先ほど呼び出した@ref DuplicatedObject::ApproveFaultRecovery() が true を返した場合、
 *          　セッションマスターが複製オブジェクトのコントロール権限を以下の手順で取得します。<br>
 *          　　・@ref ChangeMasterStationOperation を実行します。<br>
 *          　　・@ref DuplicatedObject::OperationEnd() が呼び出されます。<br>
 *          ・@ref DuplicatedObject::ApproveFaultRecovery() が false を返した場合、
 *          　以下のようにして複製オブジェクトが複製オブジェクトストアから取り除かれます。
 *          　　・@ref RemoveFromStoreOperation がトリガーされます。<br>
 *          　　・@ref DuplicatedObject::OperationEnd が呼び出されます。<br>
 *          　　・複製オブジェクトのデストラクタが呼び出されます。
 */
class FaultRecoveryOperation: public DOOperation
{
    //Group=Methods
public:
    //{secret}
    FaultRecoveryOperation(DuplicatedObject* pDO, DOHandle hNewMaster, LogicalClock lcNewMasterVersion);
    //{secret}
    virtual ~FaultRecoveryOperation();

    // {secret}
    DOHandle GetNewMaster() const
    {
        return m_refNewMaster.GetHandle();
    }
    // {secret}
    const LogicalClock& GetNewMasterVersion() const
    {
        return m_lcNewMasterVersion;
    }

    NEX_IMPL_OPERATION_COMMON_METHODS( FaultRecovery );

private:

#ifdef NEX_TRACE_ENABLED    // n2370
    virtual void TraceImpl(Event eEvent, qUnsignedInt64 uiTraceFlags) const;
#endif  // NEX_TRACE_ENABLED

    //{secret}
    virtual qBool CallsBackOnDataSet()
    {
        return false;
    }
    //{secret}
    virtual qBool CallsBackOnDataSet(qByte /*byIndex*/)
    {
        return false;
    }

private:
    Station::Ref m_refNewMaster;
    LogicalClock m_lcNewMasterVersion;
};

END_NAMESPACE_NEX

