﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/DORef.h>
#include <NetZ/src/Core/DOCore/ActiveDOCallContext.h>

BEGIN_NAMESPACE_NEX

// Group=Results Classes

/*
Summary: This class defines the context of a call to DuplicatedObject::FetchDuplica.
Remarks: Before a call to DuplicatedObject::FetchDuplica is made, the context of the
call must be defined via the FetchContext class. Within this class you set the station you wish to fetch a Duplica from
and the appropriate flags. You can also obtain information about the CallContext::State and
DOCallContext::Outcome of the call.

Refer to DuplicatedObject::FetchDuplica for an example of how to use a FetchContext.
*/

/*!
 * @brief   このクラスは、@ref DuplicatedObject::FetchDuplica() のためのコンテキストです。
 *
 *          @ref DuplicatedObject::FetchDuplica() を呼び出す前に、
 *          FetchContextクラスによってコンテキストを定義しなければなりません。
 *          デュプリカをとってきたいステーションをセットし、適切なフラグがセットされたこのクラスの中で
 *          この呼び出しの@ref CallContext::State と、@ref DOCallContext::Outcome の情報を含めることが出来ます。
 */
class FetchContext: public ActiveDOCallContext
{
    //Group=Methods
public:

    /*
    Remarks: If no arguments are passed to the constructor, the FetchContext instance will
    initialize the value of the hTargetStation to INVALID_DOHANDLE. By default the
    DuplicatedObject::FetchDuplica call
    will be made synchronously, /i.e./ the system will wait for the FetchDuplica call to
    return a value or an error before performing the next task. The method DOCallContext::Wait
    can be used if the FetchDuplica call is to be made asynchronously.

    Parameters:
    hTargetStation: The station from which we wish to fetch a Duplica.
    bSynchronousCall: true if the call is made synchronously, which is the default, false if asynchronously.
    */

    /*!
     * @brief     オブジェクトを初期化します。
     * @param[in] hTargetStation    デュプリカを取ってくる先のステーションを指定します。
     * @param[in] bSynchronousCall trueを指定すると同期呼び出しになり、実行が終了するまで関数が返りません。
     *                              falseを指定すると非同期呼び出しになります。
     */
    FetchContext(DOHandle hTargetStation=INVALID_DOHANDLE,
                 qBool bSynchronousCall=true);
    virtual ~FetchContext();

    /*
    Description: Returns a DOHandle to a fetched object.
    Returns: A DOHandle to a fetched object.
    Remarks: Refer to DuplicatedObject::FetchDuplica for an example of this method in use.
    */

    /*!
     * @brief     取ってくる複製オブジェクトのハンドル(@ref DOHandle)を取得します。
     * @return    取ってくる複製オブジェクトのハンドル(@ref DOHandle)が返ります。
     */
    DOHandle GetFetchedObject()
    {
        return m_refTargetObject.GetHandle();
    }

    // {secret}
    void ReleaseFetchedObject()
    {
        m_refTargetObject=INVALID_DOHANDLE;
    }

#ifdef NEX_TRACE_ENABLED    // n2370
    // {secret}
    virtual void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);
#endif  // NEX_TRACE_ENABLED

    // {secret}
    virtual Type GetType() const
    {
        return FetchCall;
    }

    // {secret}
    qBool FetchDuplica(DOHandle hFetchedObject);
    // {secret}
    qBool FetchDuplicaImpl(DOHandle hFetchedObject);
    // {secret}
    virtual void ProcessResponse(UserContext byResponseContext, State* pResultingState, Outcome* pResultingOutcome);

    // {secret}
    void DeleteOrphanOnFailure();

private:
    qBool m_bDeleteOrphanOnFailure;
};


END_NAMESPACE_NEX

