﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

//#include <NetZ/src/Core/DOCore/DOClassTemplate.h>
#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/IDGeneratorDDL.h>

BEGIN_NAMESPACE_NEX

// Group=ID Classes

// This macro is never defined.  It is used only to get around a compilation problem with DocJet.
#ifdef FOR_DOC_PURPOSE
/*
Description: Generates unique IDs for a given entity.
Remarks: This class is used to generate unique IDs that can be assigned to objects or other entities
in your game. IDs generated by an instance of this class are guaranteed to be unique however, two
IDGenerator instances may generate the same ID.

This class can be used to assign specific IDs to duplicated objects rather than having
$PRODUCTNAME$ assign the IDs automatically. However, when this class is used to generate IDs for
duplicated objects, an instance of this class needs to be created for each duplicated object class
for which IDs are to be generated.

All IDGenerator objects are guaranteed to be discovered by a Station before the Session commences on
that Station. As IDGenerator objects start with an empty ID range, immediately upon discovery the objects
request an ID range of the default size from their Duplication master.

Refer to DuplicatedObject::Create for an example of how you can create objects and assign DOID s to them.
Note that a call to DuplicatedObject::Create may block if no more IDs are available on the station
where the call is made. In order to decrease the likelyhood of this occurring, you can increase
the ID range that each station requests via SetDefaultRequestedRangeSize or SetRequestedRangeSize.
*/

/*!
 * @brief   エンティティごとに一意なIDを生成するためのクラスです。
 *
 * このクラスは、アプリケーションの中のエンティティごとに一意なIDを生成するために利用されます。
 * このクラスのインスタンスによってIDが生成され、それは一意であることが保証されますが、
 * 複数のIDGeneratorクラスのインスタンスは同一のIDを生成する可能性があります。
 *
 * このクラスは、システムが自動的に複製オブジェクトのIDを設定するために使用されます。
 * しかし、このクラスが複製オブジェクトのIDを生成するために利用されるとき、
 * このクラスはそれぞれの複製オブジェクトクラスごとに作成される必要があります
 * (それぞれのクラスごとに一意のIDを割り当てる必要があるため)。
 *
 * すべてのIDGeneratorオブジェクトはグローバルであり、ステーションがセッションに
 * 参加するために発見されることが保証されています。
 */
class IDGenerator : DuplicatedObject
{

#else

class IDGenerator : public DOCLASS(IDGenerator)
{

#endif
    // Group=Methods
public:
    IDGenerator();

public:
    virtual ~IDGenerator();

public:
    // {secret}
    enum
    {
        DefaultRequestedRangeSize = 256
    };

    /*
    Description: Generates an ID unique to this IDGenerator object valid across all stations.
    Parameters:
    uiRetID:  A pointer to a variable that will receive the generated ID upon success.
    uiTimeout: The maximum time in milliseconds that the system will wait if the ID
    generator needs to query the session master for the available IDs. The default uiTimeout
    is infinite.
    Returns: true if an ID was successfully generated, false if not or an error occurred.
    */

    /*!
     * @brief      このIDGeneratorからすべてのステーションを通じて利用可能な一意のIDを生成します。
     *
     * すべてのIDGeneratorオブジェクト(複製マスターもデュプリカも含む)は、それぞれが利用できるIDのローカルプールを持っています。
     * このプールが空になる前に(おおよそ25%のIDが使われたとき)に新しいIDのローカルプールを複製マスターに要求します。
     *
     * もしも有効なIDがローカルIDプールからすべてなくなってしまったときには、
     * ステーションは、新しいIDプールを複製マスターに要求する必要があります。
     * そのときにいくらかの時間(だいたい複製マスターとのラウンドトリップ時間と同等ぐらいの時間)この関数はブロックします。
     *
     * @param[out] uiRetID   生成に成功したときにID番号を受け取るためのポインタを指定します。
     * @param[in]  uiTimeout IDを生成するためのシステムが待つための最大待ち時間をミリ秒単位で指定します(デフォルトはタイムアウトなし)。
     * @retval     true      正確にIDが生成できた場合に返ります。
     * @retval     false     生成に失敗した場合、もしくはエラーが発生した場合に返ります。
     */
    qBool GenerateID( ID *uiRetID, qUnsignedInt32 uiTimeout=WAIT_INFINITE_TIMEOUT);

    /*
    Description: Sets the range of IDs that the generator can generate.
    Parameters:
    idFirst: The first value in the range of possible IDs to generate.
    idLast:  The last value in the range of possible IDs to generate.
    Remarks: The generator will generate IDs starting from idFirst and ending with idLast.
    If the generator reaches idLast, the GenerateID method will fail. This function
    can only be called on a duplication master and must be called before GenerateID
    is called on any station.
    Returns: true if the ID range was successfully set, false if not or an error occurred.
    The error SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA is signalled if this method is called on a Duplica.
    See Also: GenerateID
    */

    /*!
     * @brief      このIDGeneratorが生成できるIDの範囲を設定します。
     *
     * @a idFirstで指定されたIDから、@a idLastで指定した範囲のIDを
     * このIDGeneratorが生成するようになります。ここで指定した、最後の値に
     * 生成されたIDが到達すると、それ以上の@ref GenerateID 関数呼び出しは失敗します。
     *
     * この関数は、IDGeneratorの複製マスターがのみが呼び出すことが出来ます。
     * また、いずれかのステーションが@ref GenerateID 関数を呼ぶ前でなければなりません。
     *
     * @param[in]  idFirst 生成が開始されるIDを指定します。
     * @param[in]  idLast  生成が終了されるIDを指定します。
     * @retval     true    正確にIDの範囲が設定できた場合に返ります。
     * @retval     false   エラーが発生した場合に返ります。
     *                    デュプリカに対してこの関数を呼び出した場合には、SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICAがシグナルされます。
     * @see        GenerateID, DuplicatedObject::IsADuplicationMaster
     */
    qBool SetIDRange(ID idFirst, ID idLast);

    // {secret}
    qUnsignedInt32 GetRangeFirst()
    {
        return m_oCurrentLocalRange.m_uiFirst;
    }
    // {secret}
    qUnsignedInt32 GetRangeLast()
    {
        return m_oCurrentLocalRange.m_uiLast;
    }
    // {secret}
    qUnsignedInt32 GetMasterRangeFirst()
    {
        return m_oMasterRange.m_uiFirst;
    }
    // {secret}
    qUnsignedInt32 GetMasterRangeLast()
    {
        return m_oMasterRange.m_uiLast;
    }

    /*
    Description: Sets the size of the ID range requested by duplicas from the duplication master.
    Parameters:
    uiRangeSize: The new size of the range. The default uiRangeSize is 256 IDs.
    Remarks: All IDGenerator objects (Duplication master and Duplica s) have a local pool of IDs that they can use.
    Before this pool is empty, (when approximately 25% of the IDs have been used) the objects query a new pool
    of IDs from their Duplication master. This function sets the size of this pool. This method should be called
    on both the IDGenerator Duplication master and all its duplicas.

    When no more IDs are available, a station may block for a period of time, which is roughly
    equivalent to the round-trip time to the IDGenerator master station, while the station requests
    a new range. By increasing the value of the range size you can minimize the likelyhood of this
    occurring.
    */

    /*!
     * @brief      デュプリカが複製マスターに対して要求するID範囲を設定します。
     *
     * すべてのIDGeneratorオブジェクト(複製マスターもデュプリカも含む)は、それぞれが利用できるIDのローカルプールを持っています。
     * このプールが空になる前に(おおよそ25%のIDが使われたとき)に新しいIDのローカルプールを複製マスターに要求します。
     * この関数は、そのプールの大きさを設定します。
     * この関数はIDGeneratorの複製マスターもデュプリカも呼び出すことが出来ます。
     *
     * もしも有効なIDがローカルIDプールからすべてなくなってしまったときには、
     * ステーションは、新しいIDプールを複製マスターに要求する必要があります。
     * そのときにいくらかの時間(だいたい複製マスターとのラウンドトリップ時間と同等ぐらいの時間)
     * ID生成(@ref GenerateID 関数)はブロックされます。
     * ここに設定するサイズを増やすことによって、この現象が発生する回数を最小化することが出来ます。
     *
     * @param[in]  uiRangeSize ローカルIDプールの範囲を指定します。
     *                         IDGeneratorが生成されたときには、@ref SetDefaultRequestedRangeSize で指定された値でID範囲が初期化されます。
     *                         もしも@ref SetDefaultRequestedRangeSize が一度も呼び出されていなければ、その値は256となります。
     * @see        GetRequestedRangeSize, SetDefaultRequestedRangeSize
     */
    void SetRequestedRangeSize(qUnsignedInt32 uiRangeSize)
    {
        m_uiRequestRangeSize = uiRangeSize;
    }

    /*
    Description: Returns the current request range size for this IDGenerator object.
    Returns: The current request range size for this IDGenerator object.
    Remarks: This value will be equivalent to the default request range size unless the range has been
    reset via SetRequestedRangeSize.
    See Also: SetRequestedRangeSize, SetDefaultRequestedRangeSize
    */

    /*!
     * @brief      デュプリカが複製マスターに対して要求するID範囲を取得します。
     *
     * すべてのIDGeneratorオブジェクト(複製マスターもデュプリカも含む)は、それぞれが利用できるIDのローカルプールを持っています。
     * このプールが空になる前に(おおよそ25%のIDが使われたとき)に新しいIDのローカルプールを複製マスターに要求します。
     * この関数は、そのプールの大きさを取得します。
     *
     * @return     設定されたID範囲が返ります。
     * @see        SetRequestedRangeSize, SetDefaultRequestedRangeSize
     */
    size_t GetRequestedRangeSize() const
    {
        return m_uiRequestRangeSize;
    }

    /*
    Description: Sets the default request range size for this IDGenerator object.
    Parameters:
    uiRangeSize: The new size of the range. The default uiRangeSize is 256 IDs.
    Remarks: This method sets the default ID range size requested by an IDGenerator object. The range may be reset
    via SetRequestedRangeSize.
    This method must be called after the $PRODUCTNAME$ object is created and before the Session is created or joined.
    See Also: GetDefaultRequestedRangeSize, SetRequestedRangeSize
    */

    /*!
     * @brief      デュプリカが複製マスターに対して要求するID範囲の、IDGenerator生成時におけるデフォルト値を設定します。
     *
     * すべてのIDGeneratorオブジェクト(複製マスターもデュプリカも含む)は、それぞれが利用できるIDのローカルプールを持っています。
     * このプールが空になる前に(おおよそ25%のIDが使われたとき)に新しいIDのローカルプールを複製マスターに要求します。
     * この関数は、IDGeneratorが作成されたときに利用される、デフォルトのローカルIDプールサイズを設定します。
     *
     * @param[in]  uiRangeSize デュプリカが複製マスターに対して要求するID範囲の、IDGenerator生成時におけるデフォルト値を指定します。
     * @see        GetDefaultRequestedRangeSize, SetRequestedRangeSize
     */
    static void SetDefaultRequestedRangeSize(qUnsignedInt32 uiRangeSize)
    {
        s_uiDefaultRequestRangeSize = uiRangeSize;
    }

    /*
    Description: Returns the default request range size for this IDGenerator object.
    Returns: The default request range size for this IDGenerator object.
    Remarks: The default request range size is set via SetDefaultRequestedRangeSize.
    See Also: SetDefaultRequestedRangeSize
    */

    /*!
     * @brief      デュプリカが複製マスターに対して要求するID範囲の、IDGenerator生成時におけるデフォルト値を取得します。
     *
     * すべてのIDGeneratorオブジェクト(複製マスターもデュプリカも含む)は、それぞれが利用できるIDのローカルプールを持っています。
     * このプールが空になる前に(おおよそ25%のIDが使われたとき)に新しいIDのローカルプールを複製マスターに要求します。
     * この関数は、IDGeneratorが作成されたときに利用される、デフォルトのローカルIDプールサイズを取得します。
     *
     * @return     デュプリカが複製マスターに対して要求するID範囲の、IDGenerator生成時におけるデフォルト値が返ります。
     * @see        GetDefaultRequestedRangeSize, SetRequestedRangeSize
     */
    static qUnsignedInt32 GetDefaultRequestedRangeSize()
    {
        return s_uiDefaultRequestRangeSize;
    }

    //{secret}
    static DOHandle GetIDGenerator(DOClassID id)
    {
        DOHandle hResult;
        hResult.SetDOClassID(DOCLASSID(IDGenerator));
        hResult.SetDOID(DOID(id));
        return hResult;
    }

    // {secret}
    void RequestIDRangeFromMaster(qUnsignedInt32 uiNbRequestedID, ID* pidFirst, ID* pidLast);

    // {secret}
    virtual qBool ApproveEmigration(qUnsignedInt32 uiReason);

    // {secret}
    virtual qBool ApproveFaultRecovery();

    // {secret}
    virtual void  OperationEnd(DOOperation* pOperation);

    /*
    Remarks: This method traces the current ID range on both the Duplication master and the local Station.
    */

    /*!
     * @brief      現在のID範囲複製マスターと、このローカルステーションに関する情報をトレースします。
     * @param[in]  uiTraceFlags どのトレースフラグが立っているときに、内容をトレースするか指定します。
     */
#ifdef NEX_TRACE_ENABLED    // n2370
    virtual void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const;
#endif  // NEX_TRACE_ENABLED

    /*
    Description: Retrieves a new pool of IDs from the Duplication master.
    Returns: true if a new pool of IDs was successfully retrieved from the Duplication master, false if not
    or an error occurred. This method will return false if the call cannot be initiated due to another call
    to retreive IDs already being in progress.
    Remarks: The local pool of IDs may be replenished by either calling this method directly or automatically
    by the system based on the size of the pool, which is set via SetRequestedRangeSize.
    See Also: SetRequestedRangeSize
    */
    // {secret}
    qBool RequestLocalIDs();

    /*
    Description: Returns the number of IDs currently available in the local pool.
    Returns: The number of IDs currently available in the local pool.
    Remarks: The size of the local pool of IDs depends on the number of IDs retrieved by a duplica,
    which is set via SetDefaultRequestedRangeSize or SetRequestedRangeSize, and on how quickly the IDs are used up.
    See Also: SetDefaultRequestedRangeSize, SetRequestedRangeSize
    */

    /*!
     * @brief      ローカルプールに存在している利用可能なID数を取得します。
     *
     * すべてのIDGeneratorオブジェクト(複製マスターもデュプリカも含む)は、それぞれが利用できるIDのローカルプールを持っています。
     * このプールが空になる前に(おおよそ25%のIDが使われたとき)に新しいIDのローカルプールを複製マスターに要求します。
     * この関数を利用することで、今、どのぐらい利用可能なIDがローカルIDプールに存在する状態なのかを高速に取得することが出来ます。
     *
     * @return     ローカルプールに存在している利用可能なID数が返ります。
     */
    size_t GetNbLocalIDs() const;

    // {secret}
    qBool CanRequestLocalIDs() const;

private:
    //{secret}
    IDGenerator(const IDGenerator&);
    //{secret}
    IDGenerator& operator=(const IDGenerator& oIDFac); // Assignment disabled, no body.

    qUnsignedInt32 GetNextLocalID();
    qBool UseNextLocalRange();

private:

    Range m_oCurrentLocalRange;
    Range m_oNextLocalRange;

    static qUnsignedInt32 s_uiDefaultRequestRangeSize;
    qUnsignedInt32 m_uiRequestRangeSize;

    RMCContext m_ccGetIDRange;
    qReal m_dRangeIncrease;

};

END_NAMESPACE_NEX

