﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/SessionOperation.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <NetZ/src/Core/DOCore/Station.h>
#include <NetZ/src/Core/DOCore/Session.h>

BEGIN_NAMESPACE_NEX

//Group = System Operations

//n1589:memo
/*
Description: This SessionOperation is performed whenever a Station joins the Session.
Remarks: When a Station attempts to join a Session, the JoinApproval callback (Session::RegisterJoinApprovalCallback) will
be called to determine whether the request should be approved or denied. If approval (Approve) to join the Session is granted, a
JoinSessionOperation is performed, if the join is denied (Deny) then no operation is performed and a message is returned to the
joining Station confirming that the join request was denied. The JoinApproval callback and subsequent JoinSessionOperation
are called on the Connection point Station, which is the Station via which the joining Station attempts to join the Session.

When a Station attempts to join a Session then the following sequence of events occurs:
    o   The JoinApproval callback (Session::RegisterJoinApprovalCallback) is called,
    o   If the JoinApproval callback returns Approve then:
        o   The method SessionOperation::Begin is called,
        o   The Station object of the joining station is published and its Station::_State is updated to JoiningSession,
        o   The Connection point Station sends a message to the joining Station confirming that the join request was approved,
        o   The Duplication master of the new Station is migrated from the Connection point station to the joining station,
        o   The method SessionOperation::End is called.
    o   If the JoinApproval callback returns Deny then the Connection point Station sends a message to the joining Station
    confirming that the join request was denied.

Refer to Session::RegisterJoinApprovalCallback for an example of how a JoinSessionOperation can be used.
See Also: Session::RegisterJoinApprovalCallback Session::JoinSession
*/

/*!
 * @brief   このセッション操作は、ステーションがセッションに参加するときに実行されます。
 *
 * ステーション(@ref Station)がセッション(@ref Session)に参加しようとしたときに、
 * JoinApprovalコールバック(@ref Session::RegisterJoinApprovalCallback())が呼び出され、
 * その要求を受け入れるか拒否するかを決定することが出来ます。もしセッションへ受け入れるときは、
 * JoinSessionOperationが実行され、受け入れを拒否するときは、何の操作も実行されずに、
 * 参加しようとしているステーションに対してセッションへの参加が拒否されたことを通知するメッセージが送られます。
 * JoinApprovalコールバックとその後に続いて実行されるJoinSessionOperationは接続しようとしている
 * ステーションの接続点のステーションによって呼び出されます。
 *
 * ステーションがセッションに参加しようとしたとき、以下の流れでイベントが発生します。
 *
 * - JoinApprovalコールバックが実行されます。
 * - もしもコールバックがApproveを返したとき、
 *     - SessionOperation::Begin()が呼び出されます。
 *     - @ref Session オブジェクトは@ref Station を@ref DuplicatedObject::Publish() し、@ref Station::State はJoiningSessionに変化します。
 *     - 接続点のステーションは参加しようとしているステーションに対して参加リクエストは受理されたことを確認するためのメッセージを送ります
 *     - 新しい@ref Station オブジェクトの複製マスターは参加しようとしているステーションにマイグレーション(複製オブジェクトのコントロール権転移)を行います
 *     - SessionOperation::End()が呼び出されます。
 * - もしもコールバックがDenyを返したときは、接続点が参加拒否を参加しようとしているステーションに伝えます。
 */
//TODO :    SessionOperation::Begin, End関数は存在しないようですが…
//          DuplicatedObject::OperationBegin, OperationEnd？
class JoinSessionOperation: public SessionOperation
{
    //Group=Methods
public:

    //{secret}
    JoinSessionOperation(Station* pStation);
    //{secret}
    virtual ~JoinSessionOperation();

    //group= Type Definitions

    /*
    Description: This enumerates the various ApprovalState s that a JoinSessionOperation may return.
    See Also: _ApprovalState, GetApprovalState, PostponeDecision
    */

    /*!
     * @brief     GetApprovalStateの戻り値の列挙体です
     */
    enum ApprovalState
    {
        // The ApprovalState of the request to join the Session is unknown.
        //! 承認ステータスは不明な状態です
        Unknown,
        // The decision as to approve or deny the request to join the Session has been posponed, via PostponeDecision.
        //! セッションへの参加を承認するか拒否するかを@ref PostponeDecision によって保留しています。
        Pending,
        // The request to join the Session has been approved.
        //! セッションへの参加は承認されました。
        Approved,
        // The request to join the Session has been denied.
        //! セッションへの参加は拒否されました。
        Denied
    };

    NEX_IMPL_OPERATION_COMMON_METHODS(JoinSession);

    //Group=Methods

    /*
    Description: Returns the current ApprovalState of the request to join the Session.
    Returns: The current ApprovalState of the request to join the Session, which may be one of
    Approved, Denied, Pending, or Unknown.
    Remarks: The ApprovalState of the join request will be Unknown if this is the first call to the
    JoinApproval callback, and Pending if the decision was posponed via PostponeDecision.
    See Also: ApprovalState, Session::RegisterJoinApprovalCallback
    */

    /*!
     * @brief     現在のセッションに対する参加要求の承認ステータスを取得します。
     *
     * 参加要求の承認ステータスは、これをJoinApprovalコールバックではじめて呼び出したときはUnknownとなります。
     * そして、@ref PostponeDecision によって保留した場合はPendingとなります。
     *
     * @return    現在のセッションに対する参加要求の承認ステータスを返します。
     */
    JoinSessionOperation::ApprovalState GetApprovalState()
    {
        return m_eApprovalState;
    }


    // {secret}
    qResult GetJoinDeniedOutcome() const
    {
        return m_rOutcome;
    }

    /*
    Description: Approves a Session join request.
    Remarks: When this method is called within the JoinApproval callback, the ApprovalState will be updaed to Approved.
    If no JoinApproval callback is registered or the callback does not call either of Approve or Deny, then by default
    this method will be returned and therefore, the join request will be approved.

    Refer to Session::RegisterJoinApprovalCallback for an example of this method in use.
    See Also: Deny, PostponeDecision, ApprovalState, Session::RegisterJoinApprovalCallback
    */

    /*!
     * @brief     セッションへの参加要求を承認します。
     *
     * この関数がJoinApprovalの中で呼び出されたとき、ApprovalStateはApprovedに変化します。
     * JoinApprovalコールバックが登録されていないか、コールバックの中で@ref Approve も@ref Deny も呼ばなかった場合、
     * デフォルトとして承認されたこととなります。
     */
    void Approve();

    /*
    Description: Denies a Session join request.
    Parameters:
    rOutcome: An optional user defined error code for the denial. By default, this is QERROR( DOCore, JoinDenied).
    Remarks: When this method is called within the JoinApproval callback ,the ApprovalState will be updated to Denied.

    Refer to Session::RegisterJoinApprovalCallback for an example of this method in use.
    See Also: Approve, PostponeDecision, ApprovalState, Session::RegisterJoinApprovalCallback
    */

    /*!
     * @brief     セッションへの参加要求を拒否します。
     *
     * この関数がJoinApprovalの中で呼び出されたとき、ApprovalStateはDeniedに変化します。
     * JoinApprovalコールバックが登録されていないか、コールバックの中で@ref Approve も@ref Deny も呼ばなかった場合、
     * デフォルトとして承認されたこととなります。
     *
     * @param[in] rOutcome 拒否したエラーコードの値を指定します(オプション)。
     */
    void Deny(const qResult& rOutcome = qResult(QERROR( DOCore, JoinDenied)));

    /*
    Description: Postpones a Session join request decision.
    Parameters:
    tiPostponeDelay: The time interval, in milliseconds, that the join request decision will be delayed. The default
    is 0 ms.
    Remarks: When this method is called within the JoinApproval callback, the ApprovalState will be updated to Postponed.
    If you need to postpone a decision, the context used for the
    decision can be attached to the operation via Operation::SetUserData.
    See Also: Approve, Deny, ApprovalState, Session::RegisterJoinApprovalCallback
    */

    /*!
     * @brief     セッションへの参加要求の決定を保留します。
     *
     * この関数がJoinApprovalコールバックの中で呼び出されたとき、セッションへの参加は遅延されます。
     * もしも決定を保留する場合、決定のためのコンテキストは@ref Operation::SetUserData() によってアタッチすることができます。
     *
     * @param[in] tiPostponeDelay 保留するための遅延時間をミリ秒単位で指定します。
     */
    void PostponeDecision(TimeInterval tiPostponeDelay);

    // {secret}
    TimeInterval GetWaitTime() const
    {
        return m_tiWaitTime;
    }

    /*
    Description: Returns a pointer to the joining Station.
    Returns: A pointer to the joining Station.
    Remarks: Refer to Session::RegisterJoinApprovalCallback for an example of this method in use.
    See Also: Session::RegisterJoinApprovalCallback
    */

    /*!
     * @brief     参加しているステーションへのポインタを取得します。
     * @return    参加しているステーションへのポインタが返ります。
     */
    const Station* GetJoiningStation() const;

private:

#ifdef NEX_TRACE_ENABLED    // n2370
    virtual void TraceImpl(Event eEvent, qUnsignedInt64 uiTraceFlags) const;
#endif  // NEX_TRACE_ENABLED

private:
    Station::Ref m_refJoiningStation;
    ApprovalState m_eApprovalState;
    TimeInterval m_tiWaitTime;
    qResult m_rOutcome;
};

END_NAMESPACE_NEX

