﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/ContainerOperation.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Platform/Core/qList.h>

BEGIN_NAMESPACE_NEX

#define INVALID_OP_INDEX -1

//==============================================================================
// The member container base class template is intended to implement the
// MemberVector, MemberList, and MemberQueue classes). Container DO class
// members are similar to STL containers adding the logic to duplicate the
// changes by sending operations instead of the whole member content whenever
// it is possible.
//==============================================================================

//group=Member Containers

/*
Description:
$PRODUCTNAME$ allows for the association of a vector, list, or queue as a dataset member.

Remarks:
The containers used in $PRODUCTNAME$ function in a very similar manner to the containers of the
C++ standard library.  The types of containers used in $PRODUCTNAME$ are vector, list, and queue.
In $PRODUCTNAME$, the methods available to perform standard operations such as
remove, add, insert, and iterate over the dataset members of a container are found
in the classes MemberVector, MemberList, and MemberQueue. When declared in the DDL,
a container must specify that updates are sent over a reliable channel, which is the default channel,
and they cannot specify
the use of an extrapolation filter. For example, the dataset class MyContainer, which contains
three containers and is associated to the duplicated object class Avatar,
would be defined in the DDL according to DDL File Syntax as follows:

        dataset MyContainer {
            vector<real> vec;
            list<dohandle> lst;
            queue<int32> que;
        };

        doclass Avatar {
            MyContainer m_dsContainer;
        };

To perform operations on these dataset containers access the relevant duplicated object instance of the
Avatar's dataset (m_dsContainer) and use the methods of the MemberVector, MemberList, and MemberQueue classes.

See Also: DDL File Syntax, MemberList, MemberVector, MemberQueue, MemberListIterator, MemberVectorIterator
*/
template<class TDDL_Type>
class MemberContainer: public RootObject
{
    enum DuplicationMode
    {
        OPERATIONS = 0,
        CONTENT = 1
    };

    // Group=Methods
public:
    //{secret}
    typedef ContainerOperation<TDDL_Type> ContOp;
    //{secret}
    typedef typename TDDL_Type::CPPType Type;

    //{secret}
    MemberContainer()
        : m_coOperationList()
        , m_bReplayable(true)
    {
    }

    //{secret}
    virtual ~MemberContainer()
    {
    }

    /*
    Description: Forces the next update of a container to content mode.
    Remarks: Calling this method forces the next call to the method
    DuplicatedObject::Update to update the whole content of the container
    on each Duplica. The Update call sets the mode back to use operations.
    Note: This method shall not be called while operations are pending to be
    updated.
    */
    void SetContentUpdate()
    {
        SYSTEMCHECK(m_coOperationList.empty() &&
                    NEX_T("Operation mode should not be changed when operations are pending to be updated."));
        m_bReplayable = false;
    }

    //{secret}
    void ClearOperations()
    {
        m_coOperationList.clear();
    }

    //{secret}
    virtual qBool ReplayOperations() = 0;

    //{secret}
    virtual qList<ContOp>* GetOperationsList()
    {
        return &m_coOperationList;
    }

protected:
    //{secret}
    virtual void AddContentToMsg(Message *pMsg) = 0;
    //{secret}
    virtual void ExtractContentFromMsg(Message *pMsg) = 0;

    //{secret}
    virtual void AddOperationsToMsg(Message *pMsg)
    {
        qList<ContOp> olTmp = m_coOperationList;

        // 32bitと64bitでバイナリフォーマットを同一にするためsize_tをqUnsignedInt32にキャスト
        const qUnsignedInt32 uiSize = static_cast<qUnsignedInt32>(m_coOperationList.size());
        SYSTEMCHECK(uiSize <= (std::numeric_limits<qUnsignedInt32>::max)());
        *pMsg << uiSize;

        while(!m_coOperationList.empty())
        {
            m_coOperationList.back().AddSourceTo(pMsg);
            m_coOperationList.pop_back();
        }
    }

    //{secret}
    virtual void ExtractOperationsFromMsg(Message *pMsg)
    {
        ContOp coTmp;
        qUnsignedInt uiNbOp=0;
        *pMsg >> uiNbOp;
        m_coOperationList.clear();
        while(uiNbOp>0)
        {
            coTmp.ExtractFrom(pMsg);
            m_coOperationList.push_front(coTmp);
            uiNbOp--;
        }
    }

public:
    //{secret}
    virtual void AddSourceTo(Message *pMsg, qBool bDiscovery = false)
    {
        // bDiscovery indicates that the container must be transmitted in whole
        // and therefore replaying the operations is not applicable.
        const qBool bReplayable = !bDiscovery && m_bReplayable;

        if(bReplayable)
        {
            *pMsg << static_cast<qByte>(OPERATIONS);
            AddOperationsToMsg(pMsg);
        }
        else
        {
            *pMsg << static_cast<qByte>(CONTENT);
            AddContentToMsg(pMsg);
            ClearOperations();
        }

        m_bReplayable = true;
    }

    //{secret}
    virtual void ExtractFrom(Message *pMsg)
    {
        qByte bDataDDL_Type = OPERATIONS;
        *pMsg >> bDataDDL_Type;
        switch(bDataDDL_Type)
        {
        case OPERATIONS:
            ExtractOperationsFromMsg(pMsg);
            ReplayOperations();
            break;
        case CONTENT:
            ExtractContentFromMsg(pMsg);
            break;
        }
    }

    // {secret}
    qBool IsReplayable() const
    {
        return m_bReplayable;
    }

protected:
    //{secret}
    qInt AddOperation(ContOp cOp)
    {
        if (SizeOfOperations() > SizeOfContent())
        {
            m_bReplayable = false;
            ClearOperations(); // No need to keep the operations.
        }

        if (m_bReplayable)
        {
            m_coOperationList.push_front(cOp);
        }

        return 0;
    }

private:
    //{secret}
    size_t SizeOfOperations() const
    {
        return sizeof(ContOp) * m_coOperationList.size();
    }

    //{secret}
    virtual size_t SizeOfContent() const = 0;

protected:
    //{secret}
    qList<ContOp> m_coOperationList;
    //{secret}
    qBool m_bReplayable;
};

END_NAMESPACE_NEX

