﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/MemberContainer.h>
#include <NetZ/src/Core/DOCore/MemberListIterator.h>
#include <OnlineCore/src/Platform/Core/qList.h>

#ifdef WIN_FAMILY
// Warning 4505: 'function' : unreferenced local function has been removed
#pragma warning(disable : 4505)
#endif

BEGIN_NAMESPACE_NEX

//Group = Member Containers

/*
Description:  Contains methods to control the elements of a list declared in a
dataset and associated with a doclass in the DDL file.

Remarks: MemberLists are basically just standard C++ vectors optimized for $PRODUCTNAME$.
The MemberListIterator keeps track of the operations performed on the MemberList in order
to transmit updates efficiently, /i.e./ transmit just those elements that change between updates.
On the other hand, the std_list transmits all its elements each update. In most cases it is
preferable to use MemberLists however, if you need to do a lot of manipulation on a dataset
that changes infrequently you will probably find the std_list easier to use given its standard API.

A MemberList instance is associated with a doclass in the DDL file in the following way:

        dataset ListDataset{
            list<double> lst;
        };
        doclass MyDOClass{
            ListDataset m_dsListOfDoubles;
        };

The MyDOClass implementation inherits from the DDL generated DOCLASS (MyDOClass) that
associates the m_dsListOfDoubles to your class.  Thus you can access this dataset member
and perform operations on it just like any other dataset member of your doclass.  For example:

        class MyDOClass: public DOCLASS( MyDOClass) {
            public:

            void PushFrontAndUpdate(qReal dNewValue) {
                m_dsListOfDoubles.lst.Push_Front(dNewValue);
                Update();
            }
            // other user-implemented code

        }
*/
template<class TDDL_Type>
class MemberList : public MemberContainer<TDDL_Type>
{
    //Group=Methods
public:
    //{secret}
    typedef MemberListIterator<TDDL_Type> Iterator;
    // {secret}
    typedef typename TDDL_Type::CPPType Type;
    // {secret}
    typedef typename MemberContainer<TDDL_Type>::ContOp ContOp_t;

    //{secret}
    MemberList()
    {
        m_it.Init(*this);
        m_begin.Init(*this);
        m_end.Init(*this);
    }


    // {secret}
    MemberList(const MemberList<TDDL_Type>& ml)
    {
        m_it.Init(*this);
        m_begin.Init(*this);
        m_end.Init(*this);
        for(Iterator it = ml.Begin(); it != ml.End(); ++it)
        {
            Push_Back(*it);
        }
    }

    /*
    Description: Pushes an element to the front of the list.
    Parameters:
    tElem: The element to insert.
    Remarks: Calling this method invalidates MemberListIterator instances.
    */
    void Push_Front(const Type& tElem)
    {
        MemberContainer<TDDL_Type>::AddOperation(ContOp_t(ContOp_t::PUSH_FRONT_OPCODE, tElem));
        m_stdlList.push_front(tElem);
    }
    /*
    Description: Pushes an element to the back of the list.
    Parameters:
    tElem: The element to insert.
    */
    void Push_Back(const Type& tElem)
    {
        MemberContainer<TDDL_Type>::AddOperation(ContOp_t(ContOp_t::PUSH_BACK_OPCODE, tElem));
        m_stdlList.push_back(tElem);
    }

    /*
    Description: Pops an element from the front of the list.
    */
    void Pop_Front()
    {
        MemberContainer<TDDL_Type>::AddOperation(ContOp_t(ContOp_t::POP_FRONT_OPCODE));
        m_stdlList.pop_front();
    }

    /*
    Description: Pops an element from the back of the list.
    */
    void Pop_Back()
    {
        MemberContainer<TDDL_Type>::AddOperation(ContOp_t(ContOp_t::POP_BACK_OPCODE));
        m_stdlList.pop_back();
    }

    /*
    Description: Inserts an element at the current position of the iterator.
    Parameters:
    it: The iterator to use to insert the element into the MemberList.
    tElem: The element to insert.
    n: Optional parameter that specifies how many copies of the element to insert.
    Returns: An iterator that points to the position where the new element was
    inserted into the list. Or if the Boolean method is used, true if the
    element is successfully inserted into the MemberList, false if not or an error
    occurred.
    */
    Iterator& Insert(Iterator& it, const Type& tElem)
    {
        SYSTEMCHECK(it.IsValid());
        it.Refresh();
        it.SetStdIterator(m_stdlList.insert(it.GetStdIterator(), tElem));
        //it.IncPos();
        MemberContainer<TDDL_Type>::AddOperation(ContOp_t(ContOp_t::INSERT_PX_OPCODE, it.GetPos(), tElem));
        return it;
    }

    Iterator Insert(Iterator it, typename qList<Type>::size_type n, const Type& tElem)
    {
        SYSTEMCHECK(it.IsValid());
        it.Refresh();
        m_stdlList.insert(it.GetStdIterator(), n, tElem);
        MemberContainer<TDDL_Type>::AddOperation(ContOp_t(ContOp_t::INSERT_PNX_OPCODE, it.GetPos(), n, tElem));
        return it;
    }

    /*
    Description: Removes an element from specified positions in the list.
    Parameters:
    it: The iterator indicating the element to erase.
    */
    Iterator& Erase(Iterator& it)
    {
        it.Refresh();
        it.SetStdIterator(m_stdlList.erase(it.GetStdIterator()));
        MemberContainer<TDDL_Type>::AddOperation(ContOp_t(ContOp_t::ERASE_OPCODE, it.GetPos(), 0));
        it.Invalidate();
        return it;
    }

    // This method is only used for MemberListIterator::operator= implementation
    // NOTE: This is an internal method not to mix up with Assign(n, tElement)
    //{secret}
    qBool Assign(Iterator& it, const Type& tElem)
    {
        if(it.IsValid())
        {
            return false;
        }

        it->Refresh();
        MemberContainer<TDDL_Type>::AddOperation(ContOp_t((qByte)ContOp_t::ASSIGN_OPCODE, it.GetPos(), tElem));
        return true;
    }

public:
    //{secret}
    void Clear()
    {
        MemberContainer<TDDL_Type>::AddOperation(ContOp_t(ContOp_t::CLEAR_OPCODE));
        m_stdlList.clear();
    }

    /*
    Description: Returns the element at the front of the list.
    Returns: The element at the front of the list.
    */
    const Type& Front() const
    {
        return m_stdlList.front();
    }

    /*
    Description: Returns the element at the back of the list.
    Returns: The element at the back of the list.
    */
    const Type& Back() const
    {
        return m_stdlList.back();
    }

    /*
    Description: Returns whether or not the list is empty.
    Returns: true if the MemberList is empty, false if not or an error occurred.
    */
    qBool Empty()
    {
        return m_stdlList.empty();
    }

    /*
    Description: Returns the number of elements in the list.
    Returns: The number of elements in the MemberList
    */
    typename qList<Type>::size_type Size() const
    {
        return m_stdlList.size();
    }

    /*
    Description:  Sets the MemberListIterator to point to the first element in the MemberList.
    Returns:  The MemberListIterator that now points to the beginning of the MemberList.
    */
    const Iterator& Begin() const
    {
        MemberList<TDDL_Type>& oThis = *const_cast<MemberList<TDDL_Type>*>(this);
        oThis.m_begin.SetStdIterator(oThis.m_stdlList.begin());
        return m_begin;
    }

    /*
    Description:  Sets the MemberListIterator to point to the last element in the MemberList.
    Returns:  The MemberListIterator that now points to the last element in the MemberList.
    */
    Iterator& End()
    {
        m_end.SetStdIterator(m_stdlList.end());
        SYSTEMCHECK(m_end.GetStdIterator() == m_stdlList.end());
        return m_end;
    }

    const Iterator& End() const
    {
        MemberList<TDDL_Type>& oThis = *const_cast<MemberList<TDDL_Type>*>(this);
        oThis.m_end.SetStdIterator(oThis.m_stdlList.end());
        return m_end;
    }

private:
    ////////////////////////////
    // Replay operations (BEGIN)
    ////////////////////////////
    //{secret}
    qBool ReplayInsert(typename qList<Type>::size_type iPos, const Type& tElem)
    {
        m_it.Begin();
        for(typename qList<Type>::size_type i = 0; i < (iPos-1); i++)
        {
            ++m_it;
        }
        m_it.SetStdIterator(m_stdlList.insert(m_it.GetStdIterator(), tElem));
        return true;
    }

    //{secret}
    qBool ReplayInsert(typename qList<Type>::size_type iPos, typename qList<Type>::size_type n, const Type& tElem)
    {
        m_it.Begin();
        for(typename qList<Type>::size_type i = 0; i < (iPos-n); i++)
        {
            ++m_it;
        }
        m_stdlList.insert(m_it.GetStdIterator(), n, tElem);
        m_it.Invalidate();
        return true;
    }

    //{secret}
    qBool ReplayErase(typename qList<Type>::size_type iPos)
    {
        m_it.Begin();
        for(typename qList<Type>::size_type i = 0; i < iPos; i++)
        {
            ++m_it;
        }
        m_stdlList.erase(m_it.GetStdIterator());
        m_it.Invalidate();
        return true;
    }

    //{secret}
    qBool ReplayAssign(typename qList<Type>::size_type iPos, const Type& tElem)
    {
        m_it.Begin();
        typename qList<Type>::iterator stdIt;
        for(typename qList<Type>::size_type i = 0; i < iPos; i++)
        {
            ++m_it;
        }
        stdIt = m_it.GetStdIterator();
        *stdIt = tElem;
        return true;
    }
    ////////////////////////////
    // Replay operations (END)
    ////////////////////////////

protected:
    //{secret}
    virtual void AddContentToMsg(Message *pMsg)
    {
        qList<Type> listCopy = m_stdlList;

        // 32bitと64bitでバイナリフォーマットをそろえるためsize_tをqUnsignedInt32にキャスト
        const qUnsignedInt32 uiSize = static_cast<qUnsignedInt32>(listCopy.size());
        *pMsg << uiSize;

        while(!listCopy.empty())
        {
            TDDL_Type::Add(pMsg, listCopy.back());
            listCopy.pop_back();
        }
    }

    //{secret}
    virtual void ExtractContentFromMsg(Message *pMsg)
    {
        qUnsignedInt uiNbElem=0;
        Type tElem;
        m_stdlList.clear();
        *pMsg >> uiNbElem;
        while(uiNbElem>0)
        {
            TDDL_Type::Extract(pMsg, &tElem);
            m_stdlList.push_front(tElem);
            uiNbElem--;
        }
    }

public:
    //{secret}
    qBool ReplayOperations()
    {
        qBool bRet = true;
        ContOp_t *pcoOpTmp=NULL;

        while(!MemberContainer<TDDL_Type>::m_coOperationList.empty())
        {
            pcoOpTmp = &(MemberContainer<TDDL_Type>::m_coOperationList.back());

            switch(pcoOpTmp->GetOpCode())
            {
            case ContOp_t::PUSH_FRONT_OPCODE:
                m_stdlList.push_front(pcoOpTmp->GetData());
                break;
            case ContOp_t::PUSH_BACK_OPCODE:
                m_stdlList.push_back(pcoOpTmp->GetData());
                break;
            case ContOp_t::POP_FRONT_OPCODE:
                m_stdlList.pop_front();
                break;
            case ContOp_t::POP_BACK_OPCODE:
                m_stdlList.pop_back();
                break;
            case ContOp_t::INSERT_PX_OPCODE:
                bRet = ReplayInsert(pcoOpTmp->GetPos(), pcoOpTmp->GetData());
                break;
            case ContOp_t::INSERT_PNX_OPCODE:
                bRet = ReplayInsert(pcoOpTmp->GetPos(), pcoOpTmp->GetRepeat(), pcoOpTmp->GetData());
                break;
            case ContOp_t::INSERT_PFL_OPCODE:
                //TODO
                break;
            case ContOp_t::ERASE_OPCODE:
                bRet = ReplayErase(pcoOpTmp->GetPos());
                break;
            case ContOp_t::CLEAR_OPCODE:
                m_stdlList.clear();
                break;
            case ContOp_t::ASSIGN_OPCODE:
                bRet = ReplayAssign(pcoOpTmp->GetPos(), pcoOpTmp->GetData());
                break;
            default:
                SYSTEMCHECK(false && NEX_T("Invalid container operation code for MemberList."));
                return true; // Warning: This breaks the loop!
            }

            pcoOpTmp = NULL;
            MemberContainer<TDDL_Type>::m_coOperationList.pop_back();
        }
        return bRet;
    }

    //{secret}
    qList<Type>* GetStdList()
    {
        return &m_stdlList;
    }

    MemberList<TDDL_Type>& operator=(const MemberList<TDDL_Type>& mlSource)
    {
        MemberList<TDDL_Type>& ml = const_cast<MemberList&>(mlSource);
        Clear();

        for(Iterator it = ml.Begin(); it != ml.End(); ++it)
        {
            Push_Back(*it);
        }
        return *this;
    }

    virtual qBool operator==(const MemberList<TDDL_Type>& ml) const
    {
        return (this->m_stdlList == ml.m_stdlList);
    }

    virtual qBool operator!=(const MemberList<TDDL_Type>& ml) const
    {
        return (this->m_stdlList != ml.m_stdlList);
    }


#if defined(NEX_TRACE_ENABLED)
    // Warning: TraceContent assumes the template type is qReal
    //{secret}
    qBool TraceContent(qUnsignedInt64 uiTraceFlag)
    {
        QTRACE(uiTraceFlag, NEX_T("\n\nTracing list content: "));
        typename qList<Type>::iterator it = m_stdlList.begin();
        while(it != m_stdlList.end())
        {
            QTRACE(uiTraceFlag, (*it));
            ++it;
        }
        return true;
    }
#endif

private:
    size_t SizeOfContent() const
    {
        return sizeof(Type) * m_stdlList.size();
    }

private:
    qList<Type> m_stdlList;
    Iterator m_it;
    Iterator m_begin;
    Iterator m_end;
};

END_NAMESPACE_NEX

#ifdef WIN_FAMILY
#pragma warning(disable : 4505)
#endif

