﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/MethodID.h>
#include <OnlineCore/src/Platform/Core/GlobalVariables.h>
#include <OnlineCore/src/Platform/Core/AtomicValue.h>
#include <OnlineCore/src/Platform/Core/qMap.h>
#include <OnlineCore/src/Platform/Core/String.h>

BEGIN_NAMESPACE_NEX

// Group = ID Classes

/*
Description: This class generates unique identifiers for all methods defined in the DDL.
Remarks: $PRODUCTNAME$ uses this class to generate unique identifiers for each method
(Actions, RMC) defined in the DDL file.
*/

/*!
 * @brief   このクラスは、DDLに定義されたメソッドに一意な識別子を生成します。
 *
 * システムはDDL(データ定義言語)やメソッド(アクションか、RMC：リモートメソッドコール)ごとに
 * 一意な識別子を割り当てます。このクラスは、識別子を生成します。
 *
 * クラスごとのメソッドの上限は、255個です。
 */
class MethodIDGenerator: public RootObject
{
public:

    //group=Methods:

    // {secret}
    MethodIDGenerator() {}
    // {secret}
    virtual ~MethodIDGenerator() {}

    // {secret}
    static MethodID AssignID(String strMethodName, qByte classid);

    /*
    Description: Returns the MethodID of a method given its name.
    Parameters:
    strMethodName: The name of the method as a string.

    Returns: The MethodID of the method that corresponds to the given strMethodName.
    Remarks: Each method (Actions, RMC) defined in the DDL is assigned a unique ID by the
    system generated via the MethodIDGenerator class.
    See Also: GetMethodName, MethodID
    */

    /*!
     * @brief     指定された名前のメソッドのMethodIDを取得します。
     *
     * DDL(データ定義言語)ファイルに定義されているメソッド(アクションかRMC：リモートメソッドコール)は
     * システムによって@ref MethodID(一意の識別子)が割り当てられています。
     * この静的関数は、DDLに定義されている名前から、該当するメソッドのMethodIDを返します。
     *
     * @param[in] strMethodName メソッドの名前を文字列で指定します。"クラス名::メソッド名"の形式で指定します。
     * @return    @a strMethodNameで指定された名前を持つメソッドのMethodIDが返ります。
     */
    static MethodID GetID(const String &strMethodName);  //n1769: const付加。

    /*
    Description: Returns the name of a method given its MethodID.
    Parameters:
    id: The MethodID of the method.
    Returns: The name of the method that corresponds to the given id.
    Remarks: Each method (Actions, RMC) defined in the DDL is assigned a unique ID by the
    system generated via the MethodIDGenerator class.
    See Also: GetID, MethodID
    */

    /*!
     * @brief     指定されたMethodIDに対応するメソッドの名前を取得します。
     *
     * DDL(データ定義言語)ファイルに定義されているメソッド(アクションかRMC：リモートメソッドコール)は
     * システムによって@ref MethodID(一意の識別子)が割り当てられています。
     * この静的関数は指定されたMethodIDから、DDLで定義された名前を取得します。
     *
     * @param[in] id 取得したい対象のメソッドの、MethodIDを指定します。
     * @return    @a idにて指定されたメソッドの名前が返ります。
     */
    static const qChar* GetMethodName(MethodID id);

    // {secret}
    static void CleanupMethodIDTable();

private:

    static AtomicValue s_ulNextID;
};

#define _METHODID(CLASS, METHOD)    NAMESPACE_NEX MethodIDGenerator::GetID(#CLASS "::" #METHOD) //See DOClassCGMacros.h _DOC_InitMethod

END_NAMESPACE_NEX

