﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

BEGIN_NAMESPACE_NEX

// group=Miscellaneous Types and Macros

/*
    Description: Enumerates all possible networking topologies that a session may establish.
    Remarks: Enumerates the networking topologies that a session may establish (Peer2Peer or
    ClientServer).
*/
/**
  * @brief セッションが構築可能なネットワークトポロジー（P2P／クライアントサーバー）を列挙する名前空間です。
  */
namespace NetworkTopology
{

/**
  * @brief ネットワークトポロジーの列挙型です。
  */
enum _NetworkTopology
{

    //  Description: The Default Net-Z topology where direct connections are
    //  established between every pair of stations. With this topology,
    //  messages are always sent directly from one station to another.
    /**
      * @brief P2P型ネットワークトポロジーです。
      *        Net-Zにおけるデフォルトは、全ステーション間に直接的な接続を構築する、フルメッシュ型です。
      *        このトポロジーにおいては、ステーション間でやりとりされるメッセージは常に
      *        「直接」届けられます。
      */
    Peer2Peer,

    //  Description: A network topology where connections are established
    //  from each station in the session to a central one (the session master).
    //  Communications between the session master and any other station
    //  are direct while communications between any two non
    //  session master stations are routed through the session master.
    /**
      * @brief クライアントサーバー型ネットワークトポロジーです。
      *        接続は各ステーションと、中心的なステーション（セッションマスター）との間に構築されます。
      *        セッションマスターとその他のステーションでの通信は直接行われますが、
      *        非セッションマスター同士の通信は、セッションマスターを経由してルーティングされます。
      */
    ClientServer,

    //n1589:memo:this mode is for using router, only if connecting direct is failed
    /**
      * @brief P2P型とクライアントサーバー型の中間的なトポロジーです。
      *        完全なフルメッシュ型のトポロジー（P2P型）を構築できない環境に対応できます。
      *        直接接続を確立できた非セッションマスター同士は、セッションマスターを経由せずに
      *        データ転送を行い、直接接続を確立できなかった非セッションマスター同士は、
      *        セッションマスターを経由したデータ転送を行います。
      */
    Hybrid,

#ifndef FOR_DOC_PURPOSE
    // {secret}
    // Description: A network topology which uses an external process (i.e. not part
    // of the current Net-Z session) to route packets between Net-Z stations
    /**
      * @internal
      * @brief 外部ルータモード。
      *        （例えばNet-Zセッションに属さない）外部プロセスを使って、Net-Zセッションに
      *        おけるパケットをルーティングするモードです。
      */
    ExternalRouter
#endif
};

// Description: Used with NetZ::SetNetworkTopology to define the NetworkTopology that
// a Net-Z instance may establish.
/**
  * @brief Net-Zオブジェクトのインスタンスが構築するネットワークのトポロジーを決定するAPIである、
  *        @ref NetZ::SetNetworkTopology 関数によって使われる型です。
  */
typedef enum _NetworkTopology Mode;

}

END_NAMESPACE_NEX


