﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

// This file must have the name of the Dataset that it implements, PromotionReferee.h

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/PromotionRefereeDDL.h>
#include <NetZ/src/Core/DOCore/ElectionTable.h>
#include <NetZ/src/Core/DOCore/ConfirmFaultyStationTable.h>

BEGIN_NAMESPACE_NEX

//Group=Miscellaneous Classes

#ifdef FOR_DOC_PURPOSE
/*
Description: This class controls to which Station s fault-tolerant objects will migrate after a fault occurs.
Remarks: This class controls the Station to which an object migrates after a fault on the object's
Duplication master Station. The PromotionReferee decides which Station should become the new Duplication master
via affinity levels. When the Station where the Duplication master of a fault-tolerant object resides quits the
Session due to a failure, each Station that holds a Duplica of this object calls the PromotionReferee
via a ComputeAffinityCallback to indicate the affinity level and wait time of the Station for the object and
that that Station would like to become the new Duplication master. By default the Station with the highest affinity
level becomes the new Duplication master. The default affinity level for each process is as follows: server=100,
client=50, and tool=0. In addition, the Session master is given a bonus 10 points and by default unkonwn
processes are assumed to be client proceses. Therefore, objects are preferentially migrated to server processes,
rather than client processes, rather than tool processes. If two or more processes with the same affinity call the
PromotionReferee, then the first Station that called the PromotionReferee will be elected the new Duplication master.
Refer to ComputeAffinityCallback for further details as to the decision process undertaken by the PromotionReferee.
See Also: Fault Tolerance Overview
*/
/**
  @brief ステーションに障害が発生した後、フォールトトレラントなオブジェクトをどのステーションにマイグレーション
         させるかを制御するクラスです。

  本クラスは、複製オブジェクトが存在していたステーションに障害が発生した後、どのステーションに複製オブジェクトを
  マイグレーションさせるかを制御します。PromotionRefereeは、類似度レベルに応じて、どのステーションが新しい
  複製マスターになるべきかを決定します。フォールトトレラントな複製マスターが存在していたステーションが、
  障害によってセッションを離脱したとき、この複製マスターに対応するデュプリカを擁する各ステーションは
  ComputeAffinityCallbackを通してPromotionRefereeを呼び出し、このオブジェクトにおける類似レベルとステーションの
  待ち時間を明示し、どのステーションが新たな複製マスターとなるかを決定します。
  デフォルトでは、最大の類似度レベルを持つステーションが新たな複製マスターとなります。各プロセスにおける
  デフォルトの類似度レベルは、サーバープロセスが100、クライアントプロセスが50、ツールプロセスは0となります。
  また、セッションマスターはボーナスとして10ポイントが加算され、デフォルトで未知のプロセスはクライアントプロセスと
  見なされます。
  よって、オブジェクトのマイグレーション先としては、クライアントプロセスやツールプロセスよりも、サーバープロセスが
  優先されるようになります。
  もし同一の類似度レベルを持つ複数のプロセスがPromotionRefereeを呼び出した場合は、最初にPromotionRefereeを呼び出した
  ステーションが新たな複製マスターとして選出されます。
  PromotionRefereeが実行している決定プロセスの詳細については、ComputeAffinityCallbackを参照してください。
 */
class PromotionReferee: public DuplicatedObject
{

#else
class PromotionReferee: public DOCLASS(PromotionReferee)
{
#endif
public:

    /*
    Remarks: A PromotionReferee object should never be directly instantiated. Instead, it is created by $PRODUCTNAME$
    prior to session creation.
    */
    PromotionReferee();

    /*
    Remarks: A PromotionReferee object should never be directly destroyed. It is destroyed by $PRODUCTNAME$ when the
    session terminates.
    */
    virtual ~PromotionReferee();

    //group=Type Definitions

    /*
    Description: Defines a ComputeAffinityCallback.
    Parameters:
    o /hObject/ : A DOHandle to the local Duplica object whose Duplication master was located on a Station that faulted.
    o /pbyAffinity/ : A pointer to the affinity of the Station where the ComputeAffinityCallback is made.
    o /ptiWaitTime/ : A pointer to the period of time, in milliseconds, that the PromotionReferee will wait before
    declaring the station the new Duplication master.
    Remarks: The default affinity level for each process is as follows: server=100, client=50, and tool=0. In addition,
    the Session master is given a bonus 10 points and by default unkonwn processes are assumed to be client proceses.
    Therefore, objects are preferentially migrated to server processes rather than cient processes, rather than tool
    processes. If two or more processes with the same affinity call the PromotionReferee, then the first Station that
    called the PromotionReferee will be elected the new Duplication master.

    The wait time makes the PromotionReferee wait for a given period of time before declaring a given station the new
    Duplication master in case another station with a higher affinity takes longer to respond to the fault than the initial
    station.
    The default /ptiWaitTime/ is 1000 ms for a tool process (as we really want another process to take control of the object),
    500 ms for a client process, and 0 ms for a server process. Therefore, the call returns as soon as a server process
    notifies the PromotionReferee that it wants to take control of an orphaned object.

    After a fault occurs, when the PromotionReferee receives the first migration request from a given Station, that Station
    is marked as the "best choice" for the new Duplication master and the PromotionReferee waits for a period of time up to
    the wait time of that station for a request from a Station with a higher affinity. If none is received then the Station
    is confirmed as the new Duplication master and object migration proceeds. On the other hand, if a request of higher affinity
    is received then that Station is marked as the "best choice" and the PromotionReferee waits for that Station's wait time
    before confirming it the new master. This process continues until the wait time expires, whereupon the PromotionReferee
    makes its final decision and after which any further requests are ignored. Once the PromotionReferee has decided which
    Station will be the new Duplication master, it attempts to migrate the object to the new master station.
    Note: This method applies to UserDOs that are not Wellknowns.
    See Also:  RegisterAffinityCallback
    */
    /**
      @brief ComputeAffinityCallbackのtypedefです。

      デフォルトの類似度レベルは、サーバープロセスが100、クライアントプロセスが50、ツールプロセスは0となります。
      また、セッションマスターはボーナスとして10ポイントが加算され、デフォルトで未知のプロセスはクライアントプロセスと
      見なされます。
      よって、オブジェクトのマイグレーション先としては、クライアントプロセスやツールプロセスよりも、サーバープロセスが
      優先されるようになります。
      もし同一の類似度レベルを持つ複数のプロセスがPromotionRefereeを呼び出した場合は、最初にPromotionRefereeを呼び出した
      ステーションが新たな複製マスターとして選出されます。

      ウェイトタイムはPromotionRefereeを待たせることで、与えられたステーションが新複製マスターであると宣言するまでの時間を
      引き延ばします。これは、より高い類似度レベルを持っていた他のステーションが、障害への応答に遅れてしまったケースに
      備えてのものです。

      デフォルトの ptiWaitTimeは、ツールプロセスなら1000ミリ秒（オブジェクトの制御は、ツールプロセス以外に任せたいという強い意図のため）、
      クライアントプロセスならば500ミリ秒、そしてサーバープロセスならば0ミリ秒となっています。
      よって、サーバープロセスがPromotionRefereeに孤児オブジェクトの面倒を見ることを通知すれば、この関数呼び出しは返ります。

      障害発生後、PromotionRefereeが最初のマイグレーション要求をステーションから受け取った時、そのステーションは
      新たな複製マスターとして「ベストな選択」であるとマークされ、PromotionRefereeは、より高い類似度レベルのステーションからの
      リクエストに備えて、wait timeだけ待ちます。何もリクエストがなかった場合は、このステーションが新たな複製マスターとして
      承認され、オブジェクトのマイグレーションが進行します。一方、より高い類似度レベルのリクエストを受け取った場合は、
      そのステーションが「ベストな選択」であるとマークされ、そのステーションが新たな複製マスターであると承認されるまで、
      PromotionRefereeはそのステーションのwait timeだけ待ちます。このプロセスはwait time期限切れまで続行され、その結果、
      PromotionRefereeは最終の決定を下し、それ以降のリクエストは全て無視します。いったんPromotionRefereeが新たな
      複製マスターのステーションを決定したら、新しいマスターステーションへのマイグレーションが試みられます。

      注意：このメソッドは、既知オブジェクトではないユーザーDOに適用されます。

      @param[in] hObject      複製マスターのステーションに障害が発生したローカルのデュプリカへの@ref DOHandle 。
      @param[in] pbyAffinity  ComputeAffinityCallbackによって作成される、ステーションの類似度レベル
      @param[in] ptiWaitTime  PromotionRefereeが新たな複製マスターを宣誓するまでに待つことになる時間。単位はミリ秒。

      @see RegisterAffinityCallback
     */
    typedef void (*ComputeAffinityCallback)(DOHandle hObject, qByte *pbyAffinity, TimeInterval* ptiWaitTime);

    /*
    Description: Defines a ValidateAffinityCallback.
    Parameters:
    o /hObject/ : A DOHandle to the local Duplica object whose Duplication master was located on a Station that faulted.
    o /hCalleeStation/ : A DOHandle to the Station that this callback will be called on.
    o /byAffinity/ : The affinity level of the hCalleeStation. This is defined by the CalleeStation.
    Remarks:
    The default affinity level for each process is as follows: server=100, client=50, and tool=0. In addition,
    the Session master is given a bonus 10 points and by default unkonwn processes are assumed to be client proceses.
    Therefore, objects are preferentially migrated to server processes rather than cient processes, rather than tool
    processes. If two or more processes of the same type call the PromotionReferee, then if none of the processes are
    the Session master then the first Station that called the PromotionReferee will be elected the new Duplication master.
    Note: This method applies to UserDOs that are not Wellknowns.
    See Also:  RegisterAffinityValidationCallback
    */
    /**
      @brief ValidateAffinityCallbackのtypedefです。

      デフォルトの類似度レベルは、サーバープロセスが100、クライアントプロセスが50、ツールプロセスは0となります。
      また、セッションマスターはボーナスとして10ポイントが加算され、デフォルトで未知のプロセスはクライアントプロセスと
      見なされます。
      よって、オブジェクトのマイグレーション先としては、クライアントプロセスやツールプロセスよりも、サーバープロセスが
      優先されるようになります。
      もし同タイプの複数のプロセスがPromotionRefereeを呼び出した場合は、最初にPromotionRefereeを呼び出した
      ステーションが新たな複製マスターとして選出されます。

      注意：このメソッドは、既知オブジェクトではないユーザーDOに適用されます。

      @param[in] hObject         複製マスターのステーションに障害が発生したローカルのデュプリカへの@ref DOHandle 。
      @param[in] hCalleeStation  コールバックが呼び出されるステーションの@ref DOHandle 。
      @param[in] byAffinity      hCalleeStationの類似度レベル。呼び出されたステーション側で定義されます。

      @see RegisterAffinityValidationCallback
     */
    typedef qByte (*ValidateAffinityCallback)(DOHandle hObject, DOHandle hCalleeStation, qByte byAffinity);

    //group=Methods

    // Entry point, for the system.
    //{secret}
    qBool InitiateNewMasterElection(RMCContext* pCallContext, DOHandle *phResult, qByte *pbyNewMasterVersion,
                                    DOHandle hObject, DOHandle hFaultyMaster, LogicalClock lcMasterVersion);

    //{secret}
    static void ProcessLeavingStation(DOHandle hStation);

    /*
    Description: Registers a callback that notifies the PromotionReferee of the affinity level of a Station.
    Parameters:
    pfComputeAffinity: A pointer to a ComputeAffinityCallback that notifies the PromotionReferee
    of the affinity level of a Station.
    Remarks: This method is called by a given Station on the PromotionReferee. The following
    is an example of the implementation of a ComputeAffinityCallback where the Duplication master
    of orphaned objects is (virtually always) assigned to a server process. In this case objects are
    randomly assigned to a given server process to avoid one particular server taking control of all
    objects and therefore, risking an overload. To distribute server load, load balancing of course can
    be implemented across the server cluster. A non-server process will only take control of orphaned
    objects if no server process exists.

    $import PromotionReferee1 TOP\Tests\Integration\Faults\NetZ\Test.cpp$

    See Also: RegisterAffinityValidationCallback
    */
    /**
      @brief ステーションの類似度レベルを通知するコールバックを登録します。

      このメソッドは、与えられたステーションのPromotionRefereeで呼び出されます。次に示すのは、
      孤児オブジェクトの複製マスターが（事実上、常に）サーバープロセスに割り当てられる
      @ref ComputeAffinityCallback の実装例です。
      この例では、特定のサーバーが全てのオブジェクトを引き受けてしまい、処理負荷が集中してパンクしてしまう
      危険を避けるために、オブジェクトは与えられたサーバープロセスにランダムに割り当てられます。
      処理負荷を分散するためには、もちろんロードバランシングをサーバークラスタに実装することもできます。
      非サーバープロセスは、サーバープロセスが存在しないときにのみ、孤児オブジェクトの制御を引き受けます。

      @param[in] pfComputeAffinity PromotionRefereeにステーションの類似度レベルを通知する @ref ComputeAffinityCallback コールバックへのポインタ。

      @see RegisterAffinityValidationCallback
     */
    static void RegisterAffinityCallback(ComputeAffinityCallback pfComputeAffinity);
    /*
    Description: Registers a callback that validates the affinity level of a Station.
    Parameters:
    pfValidateAffinity: A pointer to a ValidateAffinityCallback that validates the affinity
    level of a Station.
    Remarks: This method is called by the PromotionReferee on a given Station. The following is an
    example of an ValidateAffinityCallback. It verifies that the affinity level claimed by a
    Station is vaild, which in this case means that a server process has an affinity between 1 and 101
    and a client process has an affinity of 0.

    $import PromotionReferee2 TOP\Tests\Integration\Faults\NetZ\Test.cpp$

    See Also: RegisterAffinityCallback
    */
    /**
      @brief ステーションの類似度レベルを検証するコールバックを登録します。

      このメソッドは、与えられたステーションのPromotionRefereeによって呼び出されます。
      以下に示すのは、@ref ValidateAffinityCallback の例です。
      この例の場合は、サーバープロセスは1から101までの類似度レベルを持ち、
      クライアントプロセスは類似度レベル0であるという条件下において、
      ステーションが主張する類似度レベルが有効であるかどうかを検証しています。

      @param[in] pfValidateAffinity ステーションの類似度レベルを有効にする @ref ValidateAffinityCallback コールバック関数へのポインタ。

      @see RegisterAffinityCallback
     */
    static void RegisterAffinityValidationCallback(ValidateAffinityCallback pfValidateAffinity);

#ifdef NEX_TRACE_ENABLED    // n2370
    // {secret}
    void TraceElectionTable(qUnsignedInt64 uiTraceFlags) const
    {
        m_oLocalElectionTable.Trace(uiTraceFlags);
    }
#endif  // NEX_TRACE_ENABLED

    // {secret}
    qBool ElectionIsInProgress();
    // {secret}
    qBool ElectionIsInProgressForTermination();

    // {secret}
    static qBool IsSubjectToCustomAffinity(DOHandle hDO);

    //本当にあるStationが未検出になった場合、Stationごとの検出のずれは、MaxSilenceTime+KeepAliveTimeout程度になるはず
    //2000は、多数決投票による遅延
    static qUnsignedInt32 GetCallTimeOut()
    {
        return Stream::GetSettings(Stream::DO).GetMaxSilenceTime()+Stream::GetSettings(Stream::DO).GetKeepAliveTimeout();
    }

    ConfirmFaultyStationTable::Entry::State GetConfirmFaultyStationTableState(DOHandle hFaultyMaster)
    {
        ConfirmFaultyStationTable::Entry* entry = m_oConfirmFaultyMasterTable.GetEntryAndCreate(hFaultyMaster);
        entry->StationDeadCheck();
        return entry->GetState();
    }

    String ConfirmFaultyStationTableToString(DOHandle hFaultyMaster)
    {
        return m_oConfirmFaultyMasterTable.ToString(hFaultyMaster);
    }

private:

    friend class _DO_PromotionReferee;
    // RMC.
    //{secret}
    void ConfirmElection(DOHandle hObject, DOHandle hFaultyMaster, DOHandle hNewMaster);
    void DeclinePromotion(DOHandle hObject, DOHandle hFaultyMaster);

    void ElectNewMaster(DOHandle hObject,
                        DOHandle hFaultyMaster, qByte byMasterVersion,
                        DOHandle hCalleeStation, qByte byAffinity, qUnsignedInt16 uiWaitTime,
                        DOHandle* phNewMaster, qByte *pbyNewMasterVersion);
    void RequestConfirmFaultyMaster(DOHandle hFaultyMaster);
    void ReplyConfirmFaultyMaster(DOHandle hFaultyMaster, qByte byResult);


    static void ComputeAffinity(DOHandle hObject, qByte *pbyAffinity, TimeInterval* ptiWaitTime);
    static qByte ValidateAffinity(DOHandle hObject, DOHandle hCalleeStation, qByte byAffinity);


private:
    // Implementation - for bootstrap.
    //{secret}
    DOHandle ElectOwnMaster(DOHandle hFaultyMaster, DOHandle hCalleeStation);

    ElectionTable m_oLocalElectionTable;

    ConfirmFaultyStationTable m_oConfirmFaultyMasterTable;

    static ComputeAffinityCallback s_pfComputeAffinity;
    static ValidateAffinityCallback s_pfValidateAffinity;
};

END_NAMESPACE_NEX

