﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DOOperation.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>

BEGIN_NAMESPACE_NEX


//Group = System Operations

/*
Description: This DOOperation is performed each time a duplicated object is deleted.

Remarks: Each Station maintains its own duplicated object store that contains a list of all the active local
Duplication master (locally published) and Duplica (locally discovered) objects. Objects are added to and removed
from the store as a result of objects being created and deleted at run time. Only objects in the store are
considered active and therefore, updated on the local Station. A Duplication master is considered active if it
exists on the local station, /i.e./ DuplicatedObject::Publish has been called and DuplicatedObject::DeleteMainRef
has not yet been called. A Duplica is considered active if its Duplication master exists and the object is relevant
on the local station, /i.e./ it is relevant in a Duplication space (DuplicationSpace::CheckRelevance returns true)
or a Hard reference to the object exists.

The RemoveFromStoreOperation behaves differently depending on whether it is a Duplication master
or a Duplica that is being deleted.

A Duplication master instance is deleted as a result of one of these two possible sequences of events:
    o   A station that holds a Duplication master instance leaves the session and the DuplicatedObject::ApproveEmigration or DuplicatedObject::ApproveFaultRecovery system callback returns false,
    o   A Duplication master instance of a duplicated object is deleted via the method DuplicatedObject::DeleteMainRef.

When an object is removed from the duplication master store the following sequence of events occurs:
    o   The SafetyExecutive is called to determine if the object may be deleted. If so, we continue, if not we exit.
    o   The method DataSet::OperationBegin is called on all datasets,
    o   The method DuplicatedObject::OperationBegin is called,
    o   The duplicas of the Duplication master object are notified of the deletion via a multicasted message which results in duplica deletion operations (see below),
    o   The reference to the object in the duplication master store is deleted,
    o   The method DataSet::OperationEnd is called on all datasets,
    o   If in the DDL the object has an adapter associated to it, then the AdapterEvent (deprecated class) that corresponds to the deletion of the object is triggered,
    o   The method DuplicatedObject::OperationEnd is called. Iterations can no longer be performed over the duplication master object.
    o   When there are no more references (hard or soft) to the object, created via the RefTemplate, the object destructor is called.

A Duplica instance may also be deleted as a result of the DuplicationSpace::CheckRelevance and DuplicationSpace::PSMatch
virtual functions returning false.

When an object is deleted from the duplica store, the following sequence of events occurs:
    o   The SafetyExecutive is called to determine if the object may be deleted. If so, we continue, if not we exit.
    o   The method DataSet::OperationBegin is called on all datasets,
    o   The method DuplicatedObject::OperationBegin is called,
    o   The reference to the object in the duplica store is deleted,
    o   The method DataSet::OperationEnd is called on all datasets,
    o   If in the DDL the object has an adapter associated to it, then the AdapterEvent (deprecated class) that corresponds to the deletion of the object is triggered,
    o   The method DuplicatedObject::OperationEnd is called. Iterations can no longer be performed over the duplica object.
    o   When there are no more references (hard or soft) to the object, created via the RefTemplate, the object destructor is called.

This operation will not be called on a DuplicatedObject when the system state is locked via Scheduler::SystemLock.

Note: Note that the Adapter extension has been deprecated and will be removed in the next release. Refer to System
Operations for information on how to interface discreet class hierarchies. Events can now be caught by a system callback
or a DO callback for a specific object.

When $PRODUCTNAME$ triggers the AdapterEvent (deprecated class), if the Adapter has not previously been deleted the
DeleteName method for the Adapter is called. The processing of the event, which is controlled via the
AdapterEventDispatcher class, could be processed directly or queued and processed when the
DispatchQueue::Dispatch method, or a similar method, is called.

See Also: DuplicatedObject::DeleteMainRef, System Operations, Duplicated Object Overview
*/

/*!
 * @brief   この操作は複製オブジェクトが削除されるたびに実行されます。
 *
 * それぞれのステーションは、複製オブジェクトストアと呼ばれる領域を持っており、
 * そこにアクティブな複製マスター(ローカルで発行済み)と、デュプリカ(ローカルで発見済み)のリストを格納しています。
 * 複製オブジェクトがアプリケーション実行中に作成されたり削除されたりするときに、その結果として、
 * 複製オブジェクトストアから追加されたり削除されたりします。
 * 複製オブジェクトストアに格納されているオブジェクトのみがアクティブであるとみなされ、
 * ローカルステーションで更新されます。
 *
 * RemoveFromStoreOperationは消される対象が複製マスター(@ref DuplicatedObject::IsADuplicationMaster())か、
 * デュプリカ(@ref DuplicatedObject::IsADuplica())であるのかで挙動が異なってきます。
 *
 * 以下の二つのうちのいずれかによって複製マスターのインスタンスが削除されます。
 *
 * @li 複製マスターを持っているステーションがセッションを離れ、かつ@ref DuplicatedObject::ApproveEmigration() か
 *     @ref DuplicatedObject::ApproveFaultRecovery() システムコールバックが false を返したとき
 * @li 複製マスターのインスタンスが@ref DuplicatedObject::DeleteMainRef() によって削除されたとき
 *
 * 上記のいずれかによって複製マスターが削除されるとき、以下の流れに従ってイベントが発生します。
 *
 * @li @ref SafetyExecutive が呼び出され、この操作を実行してもよいかを判断します。問題なければ続行し、問題があれば中断します。
 * @li @ref DataSet::OperationBegin() が、対象の複製オブジェクトのすべてのデータセットに対して呼び出されます。
 * @li @ref DuplicatedObject::OperationBegin() が呼び出されます。
 * @li セッション中に存在するデュプリカに対してマルチキャストの削除メッセージが送られます(下記参照)
 * @li 複製オブジェクトストアにある複製マスターのリストを保持する領域から、そのオブジェクトに対する参照が削除されます。
 * @li @ref DataSet::OperationEnd() が、対象の複製オブジェクトのすべてのデータセットに対して呼び出されます。
 * @li @ref DuplicatedObject::OperationEnd() が呼び出されます。
 * @li このオブジェクトに対するすべての参照(@ref RefTemplate)がなくなったとき、オブジェクトのデストラクタが呼び出されます。
 *
 * デュプリカが削除されるとき、以下の流れに従ってイベントが発生します。
 *
 * @li @ref SafetyExecutive が呼び出され、この操作を実行してもよいかを判断します。問題なければ続行し、問題があれば中断します。
 * @li @ref DataSet::OperationBegin() が、対象の複製オブジェクトのすべてのデータセットに対して呼び出されます。
 * @li @ref DuplicatedObject::OperationBegin() が呼び出されます。
 * @li 複製オブジェクトストアにある、デュプリカのリストを保持する領域から、そのオブジェクトに対する参照が削除されます。
 * @li @ref DataSet::OperationEnd() が、対象の複製オブジェクトのすべてのデータセットに対して呼び出されます。
 * @li @ref DuplicatedObject::OperationEnd() が呼び出されます。
 * @li このオブジェクトに対するすべての参照(@ref RefTemplate)がなくなったとき、オブジェクトのデストラクタが呼び出されます。
 *
 * @ref Scheduler::SystemLock() によってシステム状態がロックされているときは、
 * この操作は複製オブジェクトに対して呼び出されません。
 *
 * @see     DuplicatedObject::DeleteMainRef
 */
class RemoveFromStoreOperation: public DOOperation
{

public:
    //{secret}
    RemoveFromStoreOperation(DOHandle hOperationOrigin, DuplicatedObject* pDO, qBool bDeleted, qBool bRemoveDuplicas);
    //{secret}
    virtual ~RemoveFromStoreOperation();

    //Group=Methods

    /*
    Description: Returns whether or not the RemoveFromStoreOperation was triggered by a duplicated object deletion.
    Remarks: Returns true if the RemoveFromStoreOperation is being performed as a result of a call to
    DuplicatedObject::DeleteMainRef or the disconnection from the session of a master
    station for a non-fault-tolerant duplicated object.
    Returns: true if the RemoveFromStoreOperation is being performed as a result of a call to
    DuplicatedObject::DeleteMainRef or the disconnection from the session of a master
    station followed by a return of false for the DuplicatedObject::ApproveEmigration or
    DuplicatedObject::ApproveFaultRecovery system callbacks, false if not or an error occurred.
    */

    /*!
     * @brief   この操作が複製オブジェクトの削除をきっかけとして呼び出されたかどうかを取得します。
     * @retval  true    @ref DuplicatedObject::DeleteMainRef() が呼び出されたか、
     *                  あるいは複製オブジェクトのマスターとなっているステーションがセッションを離脱して
     *                  @ref DuplicatedObject::ApproveEmigration() か
     *                  @ref DuplicatedObject::ApproveFaultRecovery() システムコールバックにfalseを返した場合、この値が返ります。
     * @retval  false   上記以外の方法で操作が実行されるときに返ります。
     */
    qBool ObjectIsDeleted()
    {
        return m_bDeleted;
    }

    /*
    Description: Returns whether or not the RemoveFromStoreOperation was triggered by a Relevance check in
    the DuplicationSpace.
    Remarks: This method can only return true if the RemoveFromStoreOperation is being performed on a Duplica,
    since only duplicas can become irrelevant.  If this method returns true, it implies that the
    removal from the duplica store that is performed is a result of a DuplicationSpace::CheckRelevance
    call followed by the DuplicationSpace::PSMatch function returning false.
    Returns: true if the RemoveFromStoreOperation is being performed as a result of a
    DuplicationSpace::CheckRelevance call followed by the DuplicationSpace::PSMatch function returning false.
    False if not or an error occurred.
    See Also: DOOperation::GetAssociatedDO
    */
    qBool ObjectIsIrrelevant() const
    {
        return !m_bDeleted;
    }

    // {secret}
    qBool RemoveDuplicas() const
    {
        return m_bRemoveDuplicas;
    }

    NEX_IMPL_OPERATION_COMMON_METHODS( RemoveFromStore );

private:

#ifdef NEX_TRACE_ENABLED    // n2370
    virtual void TraceImpl(Event eEvent, qUnsignedInt64 uiTraceFlags) const;
#endif  // NEX_TRACE_ENABLED

    //{secret}
    virtual qBool CallsBackOnDataSet()
    {
        return true;
    }
    //{secret}
    virtual qBool CallsBackOnDataSet(qByte /*byIndex*/)
    {
        return true;
    }

private:
    //{secret}
    qBool m_bDeleted;
    //{secret}
    qBool m_bRemoveDuplicas;
};

END_NAMESPACE_NEX

