﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

#include <OnlineCore/src/Platform/Core/SystemError.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <OnlineCore/src/Platform/Core/qMap.h>

#include <OnlineCore/src/Platform/Core/qList.h>

BEGIN_NAMESPACE_NEX

class DuplicatedObject;
class DORef;

// Group=Utility Subsystem

class Store;

class Selection: public RootObject
{
    // Group=
public:
    typedef enum _Flags
    {
        OwnsReference=1,
        LocksRelevance=2
    } Flags;

    Selection(qByte byFlags=0);
    virtual ~Selection();

    void SetFlags(qByte byFlags);

    typedef qMap<DOHandle, DuplicatedObject*>::iterator iterator;
    typedef qMap<DOHandle, DuplicatedObject*>::const_iterator const_iterator;

    iterator begin()
    {
        return m_map.begin();
    }
    const_iterator begin() const
    {
        return m_map.begin();
    }

    iterator end()
    {
        return m_map.end();
    }
    const_iterator end() const
    {
        return m_map.end();
    }

    const_iterator lower_bound(DOHandle hValue) const
    {
        return m_map.lower_bound(hValue);
    }
    const_iterator find(DOHandle hValue) const
    {
        return m_map.find(hValue);
    }
    iterator find(DOHandle hValue)
    {
        return m_map.find(hValue);
    }

    size_t size() const
    {
        return m_map.size();
    }
    qBool empty() const
    {
        return m_map.empty();
    }

    qBool Contains(DOHandle hValue) const
    {
        return m_map.find(hValue)!=end();
    }

    void Clear();
    void ClearOnCorruption();

    qBool Add(DOHandle hDO);
    qBool Add(DuplicatedObject* pDO);
    qBool Remove(DOHandle hDO);
    qBool Remove(const DuplicatedObject* pDO);

    // キャッシュ関係
    // Selectionのポインタ経由でアクセスできるように、Selectionにvirtualメソッドを用意しておく
    virtual void AddCache(DuplicatedObject* pDO)
    {
        DUMMY_PARAM(pDO);    // SelectionMasterでない限りキャッシュ更新は無い
    }
    virtual void RemoveCache(DOHandle hDO)
    {
        DUMMY_PARAM(hDO);    // SelectionMasterでない限りキャッシュ更新は無い
    }
    virtual qBool IsCache() const
    {
        return false;    // キャッシュでないので常にfalse
    }
    virtual Selection* GetMasterSelection() const
    {
        SYSTEMCHECK(false);    // これは絶対呼ばないとして落としておく
        return NULL;
    }
    virtual Selection* GetCacheSelection(DOClassID docid) const
    {
        DUMMY_PARAM(docid);    // キャッシュは常に無い
        return NULL;
    }

public:

#ifdef DEBUG_REFERENCES
    static void TraceReferencesImpl(qUnsignedInt64 uiTraceFlag, DOHandle hDO);
    static qUnsignedInt32 CountReferencesImpl(DOHandle hDO);

    void NameSelection(const qChar* szName)
    {
        m_szName = szName;
    }
#else
    void NameSelection(const qChar* /*szName*/) {}
#endif

private:

    void ApplyRemovalRules(DuplicatedObject* pDO) const;
    void ApplyInsertionRules(DuplicatedObject* pDO) const;

protected:
#ifdef DEBUG_REFERENCES
    const qChar* m_szName;
#endif

    qMap<DOHandle, DuplicatedObject*> m_map;
    qByte m_byFlags;
};

/*! @internal
 特定のクラスのDO集合を持つクラス。 SelectionMasterのキャッシュの役割を担う。
 */
class SelectionCache: public Selection
{
public:
    SelectionCache(qByte byFlags=0);
    virtual ~SelectionCache();

    // キャッシュ関係
    virtual qBool IsCache() const
    {
        return true;    // キャッシュなので常にtrue
    }
    void SetMasterSelection(Selection* pMaster)
    {
        m_pMasterSelection = pMaster;    // キャッシュ元のセット
    }
    virtual Selection* GetMasterSelection() const
    {
        return m_pMasterSelection;    // キャッシュの場合、元をたどれる
    }
    virtual Selection* GetCacheSelection(DOClassID docid) const
    {
        DUMMY_PARAM(docid);    // キャッシュは絶対取れない
        SYSTEMCHECK(false);
        return NULL;
    }

protected:
    Selection* m_pMasterSelection;
};

/*! @internal
 DO集合の親玉。
 特定のクラスのDO集合をキャッシュとして利用できるようにする。
 キャッシュの実体はDOSelectionsクラスのメンバとして用意し、DOSelections::DOSelectionsでセットする。
 */
class SelectionMaster: public SelectionCache
{
public:
    SelectionMaster(qByte byFlags=0);
    virtual ~SelectionMaster();

    // キャッシュ関係
    virtual void AddCache(DuplicatedObject* pDO); // キャッシュ更新するので実装
    virtual void RemoveCache(DOHandle hDO); // キャッシュ更新するので実装
    virtual qBool IsCache() const
    {
        return false;    // キャッシュでないので常にfalse
    }
    virtual Selection* GetMasterSelection() const
    {
        SYSTEMCHECK(false);
        return NULL;
    }
    virtual Selection* GetCacheSelection(DOClassID docid) const; // キャッシュ取得できるように実装
    void AddCacheSelection(DOClassID docid, SelectionCache* pSelection); // キャッシュの追加
protected:
    qMap<DOClassID, Selection*> m_cacheSelectionMap; // クラス単位に集合を持てるようにしておく
};

END_NAMESPACE_NEX

