﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

// This file must have the name of the DOClass that it implements, Session

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <NetZ/src/Core/DOCore/DOClass.h>
#include <NetZ/src/Core/DOCore/SessionDDL.h>
#include <NetZ/src/Core/DOCore/Station.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <NetZ/src/Core/DOCore/SharedSessionDescription.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/qSet.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <NetZ/src/Core/DOCore/SessionDescription.h>

BEGIN_NAMESPACE_NEX

class SessionOperation;
class JoinSessionOperation;
class SessionDiscoveryProtocol;
class JobSyncStationList;

#ifdef NEX_ENABLE_CTR_UDS
class JobCheckVirtualNetworkDisconnected;
#endif


//group=Session Object Roles

enum
{
    //Summary: The role of the session object is undefined. See GetRole.
    //See Also: Session::GetRole
    SESSION_ROLE_UNDEFINED=0x0,
    //Summary: The session object is a Duplication master and, therefore, the Session master. See GetRole.
    //See Also: Session::GetRole
    SESSION_ROLE_MASTER=0x01,
    //Summary: The session object is a Duplica. See GetRole.
    //See Also: Session::GetRole
    SESSION_ROLE_DUPLICA=0x02
};
// This macro is never defined.  It is used only to get around a compilation problem with
// DocJet, which does not like the inheritance to the DOCLASS(Session).
#ifdef FOR_DOC_PURPOSE

//Group=Session Management
// $ifdef WII_COMMON$
// group=Net-Z API
// $endif$
// $ifdef WII_DWC$
//group=WII_DWC
// $endif$

/*
Description: Encapsulates $PRODUCTNAME$ Session operations.
Remarks: Creates or joins a Session and sets and retrieves specific information
about the session. The Session class is a singleton meaning that it has only one instance, which has a global
point of access. In other words, only one Session may be created for each $PRODUCTNAME$ instance.
Therefore, if you wish to leave a Session to join or create another one, you need to destroy the current
$PRODUCTNAME$ object and then create a new $PRODUCTNAME$ instance.

As the Session master is the Duplication master of the Session object, the Session master may be determined
by calling DuplicatedObject::IsADuplicationMaster or DuplicatedObject::GetMasterStation. The Session object
is automatically fault tolerant and therefore, the Duplication master ( /i.e./ the Session master) may migrate during
a Session if the current Session master leaves the session. If your game operates online rather than simply over a LAN,
then when a Session master migration occurs you need to notify the lobby of the change in Session master. Rendez-Vous,
Quazal's lobby solution, provides a Host Migration extension which does exactly this however, if you use any
other lobby solution you will need to implement your own remedy.

When a player joins or quits a $PRODUCTNAME$ Session, the player's corresponding Station is added or
deleted from each other player's duplicated object store via an AddToStoreOperation or RemoveFromStoreOperation.
Therefore, if you want players to know when another player joins or quits a $PRODUCTNAME$ Session then you can
register an OperationCallback and trap either the AddToStoreOperation or RemoveFromStoreOperation as appropriate.
Alternatively, you could create a duplicated object that corresponds to a player and add some suitable code
in the object's constructor or destructor. This second option is preferable if you want player related information
( /i.e./ the object's datasets) to be available to other players.
*/
/**
 * @brief   セッション操作をカプセル化するクラスです。
 *
 * セッションの作成や参加、またセッションに対する情報の設定などが出来ます。
 * セッションクラスはシングルトンであり、ひとつのインスタンスしか作成されません。
 * このインスタンスは、NetZによって作成されます。
 * したがって、セッションから抜けて新しいセッションを作成したり参加したい場合は、
 * NetZのオブジェクトを一度破棄して、別の新しいNetZのオブジェクトを作成する必要があるということです。
 *
 * この Session クラス自体も複製オブジェクトとして扱われます。
 * Sessionクラスの複製マスターがセッションマスターとなります。
 * @ref DuplicatedObject::IsADuplicationMaster を調べることで、セッションマスターかそうでないかを区別することが出来ます。
 * また、 @ref DuplicatedObject::GetMasterStation を調べると、セッションマスターのステーションを取得することが出来ます。
 * Session オブジェクトを保持しているマスターがセッションを抜けた場合、
 * 自動的にフォールトトレランス(詳細はNEXプログラミングマニュアルを参照)が発生します。
 */
// {tag:Session class}
class Session : DuplicatedObject
{

#else

class Session: public DOCLASS(Session)
{

#endif

    //group=A_Session System Callbacks
public:

    /*
    Description: Typedef used by RegisterWellKnownDOsFactory and UnregisterWellKnownDOsFactory.
    See Also: RegisterWellKnownDOsFactory, UnregisterWellKnownDOsFactory
    */
    typedef void (__CDECL *WellKnownDOsFactoryFuncPtr)( void );

    /*
    Description: Typedef used by RegisterJoinApprovalCallback.
    See Also: RegisterJoinApprovalCallback, JoinSessionOperation
    */

    /*!
        @brief セッションへの参加可否を決定するためのコールバックです。

        @see   JoinSessionOperation, RegisterJoinApprovalCallback
    */
    typedef void (*JoinSessionOperationCallback)(JoinSessionOperation* pOperation);

    //group=A_Session Methods

    /*
    Remarks: A Session object should never be directly instantiated. Instead, it is created by $PRODUCTNAME$
    when a session is created via the CreateSession method.
    */

    /*!
     * @brief   Sessionクラスのインスタンスは自動的に作成されます。ユーザーはこのコンストラクタを呼び出さないでください。
     *
     * Net-Zシステムが、@ref Session::CreateSession 関数を呼び出したときに、
     * Sessionクラスのインスタンスは自動的に作成されます。
     */
    Session();

    /*
    Remarks: A Session object should never be directly destroyed. It is destroyed by $PRODUCTNAME$ when
    the $PRODUCTNAME$ object is deleted and as a consequence the session terminated. Therefore, to leave a Session
    and join another one you need to delete the $PRODUCTNAME$ object and then create a new $PRODUCTNAME$ instance
    and either create or join a Session for that instance.
    See Also: $FACADENAME$::~$FACADENAME$
    */

    /*!
     * @brief   Sessionクラスのインスタンスは自動的に破棄されます。ユーザーはこのデストラクタを呼び出さないでください。
     *
     * Sessionクラスのインスタンスは、NetZインスタンスが破棄されるタイミングで
     * 自動的に実行されます。したがって、現在参加しているセッションを抜けて、別のセッションに参加したい場合は、
     * NetZオブジェクトのインスタンスを破棄したうえで新しいNetZオブジェクトを作成する必要があります。
     */
    virtual ~Session();

    //Description: Returns a DOHandle to a Session instance.
    //Returns: The DOHandle of the Session instance.
    /*!
     * @brief   Sessionオブジェクトの複製オブジェクトハンドルを取得します。
     * @return  Sessionオブジェクトの複製オブジェクトハンドルが返ります。
     */
    static DOHandle GetInstanceHandle();

    /*
    Description: Returns a pointer to the Session instance.
    Returns: A pointer to the Session object.
    Remarks:
    Use the reference returned by this method to control the Session object.
    For example, the name of the Session could be returned as follows:

            Session::GetInstance()->GetSessionName();

    */

    /*!
     * @brief   Sessionオブジェクトのインスタンスを取得します。
     *
     * セッションクラスはシングルトンであり、ひとつのオブジェクトのみが自動的に生成されます。
     * この関数を呼び出すことで、唯一のインスタンスを取得することが出来ます。
     *
     * @return  Sessionオブジェクトインスタンスへのポインタが返ります。
     */
    static Session* GetInstance();

    //n1589
    static qBool IsActive();

    // {secret}
    virtual void OperationBegin(DOOperation* pOperation);
    // {secret}
    virtual void OperationEnd(DOOperation* pOperation);

    /*
    Description: Returns the role of the session object.
    Returns:
    SESSION_ROLE_MASTER
    SESSION_ROLE_DUPLICA
    SESSION_ROLE_UNDEFINED
    */

    /*!
     * @brief   セッションオブジェクトに対する役割を取得します。
     * @retval SESSION_ROLE_MASTER セッションマスター(主催者)のときに返ります。
     * @retval SESSION_ROLE_DUPLICA セッションの参加者(マスターでないとき)に返ります。
     * @retval SESSION_ROLE_UNDEFINED セッションに参加できていないときなどに返ります。
     */
    static qByte GetRole();

    static qByte GetRoleWithoutLock();


    /*
    Description: Returns the session name of this Session.
    Returns: The session name of this Session.
    Remarks: The session name can be specified when a session is created via CreateSession or via
    SessionDescription::SetSessionName.
    See Also: CreateSession, SessionDescription::SetSessionName
    */

    /*!
     * @brief   このセッションの名前を取得します。
     *
     * セッションの名前は@ref CreateSession 関数を呼び出したときか、
     * SetSessionName()を呼び出したときに設定することが出来ます。
     *
     * @return  セッションの名前へのポインタが返ります。
     */
    const qChar *GetSessionName();

    // {secret}
    qBool CompleteCreation();
    // {secret}
    void CompleteJoin();

    // {secret}
    void CreateRootGroupReference(DOHandle hRootGroup);
    // {secret}
    void ReleaseRootGroupReference();

    // {secret}
    void AddStation(DOHandle hNewStation);

    //group=A_Session System Callbacks

    /*
    Description: Called by $PRODUCTNAME$ to determine whether or not a given Station can join the Session.
    Parameters:
    pfApproveOperation: A pointer to the JoinApproval callback function.
    Returns: The previously registered callback
    Remarks: When a Station attempts to join a Session $PRODUCTNAME$ will call this callback, as part of a
    JoinSessionOperation, in order to determine whether or not the Station can join the Session.
    The JoinApproval callback and subsequent JoinSessionOperation are called on the Connection point Station,
    which is the Station via which the joining Station attempts to join the Session.
    If no JoinApproval callback is registered or the callback does
    not call either of JoinSessionOperation::Approve or JoinSessionOperation::Deny, then by default the join request
    will be approved. Note that this callback can only be called if the JoinSessionOperation::ApprovalState is either
    Unknown or Pending.

    For example, say you wanted to restrict the number of players in your game to four and only let those players
    who have the secret password join, then you could implement the JoinApproval callback as follows:

            void CheckJoiningProcesses(JoinSessionOperation* pOperation) {
                // Count the number of stations, but skip the
                // tool stations (e.g. ObjectMonitor)
                qUnsignedInt uiPlayerCount=0;
                Station::SelectionIterator it;
                while (!it.EndReached()) {
                    if (it->GetProcessType()!=Station::ToolProcess) {
                        uiPlayerCount++;
                    }
                    it++;
                }

                // Approve the join if the maximum number of
                // players has not been reached and if the joining
                // player has supplied the correct password.
                if (uiPlayerCount<=4 &&
                    pOperation->GetJoiningStation()->GetIdentificationToken()==NEX_T("CorrectPassword")) {
                    pOperation->Approve();
                    return;
                }
                // Always approve the join when the station is a tool process.
                // Note that for security purposes, we might want to force
                // tool processes to also use a password, or this section of
                // the code could be disabled in release mode.
                if (pOperation->GetJoiningStation()->GetProcessType()==Station::ToolProcess) {
                    pOperation->Approve();
                    return;
                }
                pOperation->Deny();
            }

    See Also: JoinSessionOperation
    */

    /*!
     * @brief   セッションへの参加可否を決定するためのコールバックを設定します。
     *
     * セッションへ誰かが参加しようとしたときに、@a pfOperationCallbackで指定された
     * 関数がNet-Zシステム側からコールバックされます。指定した関数の中で、
     * 新規参加者がセッションへ参加してもよいか、それとも拒否するかを決定することが出来ます。
     * コールバックが設定されていない場合か、コールバックが@ref JoinSessionOperation::Approve も
     * @ref JoinSessionOperation::Deny も返さない場合は、デフォルトではその参加要求は
     * 承認されたものとなります。コールバックは、JoinSessionOperation::ApprovalStateが
     * UnknownかPendingであるときのみ呼び出されます。
     * この段階で、Station::SelectionIteratorで取得可能なStationには、セッションへ参加してきた
     * ステーションは含まれません。
     *
     * @param[in]   pfOperationCallback コールバック関数へのポインタを指定します。
     * @return  以前登録されていたコールバック関数へのポインタが返ります。
     */
    static JoinSessionOperationCallback RegisterJoinApprovalCallback(JoinSessionOperationCallback pfOperationCallback);

    // {secret}
    static void CallApproveJoinSessionCallback(JoinSessionOperation* pOperation);
    /*
    Description: Called by $PRODUCTNAME$ each time a SessionOperation begins.
    Parameters:
    pfOperationBegin: A pointer to the OperationBegin callback function.
    Remarks: The registered callback method is called at the beginning of a SessionOperation with a reference to
    the SessionOperation that just began as a parameter. It is called on the Station where the joining or
    leaving Station's Duplication master resides.
    See Also: JoinSessionOperation, LeaveSessionOperation
    */

    /*!
     * @internal
     * @brief   SessionOperationが実行されるたびに呼び出されるコールバックを登録します。
     *
     * この関数で登録したコールバックは、pOperation が
     * JoinSessionOperation の場合はセッションマスターでのみ呼ばれ、
     * LeaveSessionOperation の場合は対象が自分のStationのときのみに呼ばれます。
     *
     * すべてのステーションの参加・離脱を知りたい場合は、@ref Station::RegisterOperationBeginCallback を使用してください。
     *
     * SessionOperationが呼び出される最初に、指定されたコールバック関数が呼び出されるようになります。
     * このコールバックには、呼び出されるSessionOperationへのポインタが渡されます。
     * このコールバックは、セッションに参加している、あるいはセッションから離脱している
     * ステーションで呼び出されます。
     *
     * @param[in]   pfOperationBegin  コールバック関数へのポインタを指定します。
     * @see     JoinSessionOperation, LeaveSessionOperation
     */
    static void RegisterOperationBeginCallback(void (*pfOperationBegin)(SessionOperation* pOperation));

    // {secret}
    static void CallOperationBeginCallback(SessionOperation* pOperation);
    /*
    Description: Called by $PRODUCTNAME$ each time a SessionOperation ends.
    Parameters:
    pfOperationEnd: A pointer to the OperationEnd callback function.
    Remarks: The registered callback method is called at the end of a SessionOperation with a reference
    to the SessionOperation that just terminated as a parameter. It is called on the Station where the joining
    or leaving Station's Duplication master resides.
    See Also: JoinSessionOperation, LeaveSessionOperation
    */

    /*!
     * @internal
     * @brief   SessionOperationが終了されようとしているときに呼び出されるコールバックを登録します。
     *
     * この関数で登録したコールバックは、pOperation が
     * JoinSessionOperation の場合はセッションマスターでのみ呼ばれ、
     * LeaveSessionOperation の場合は対象が自分のStationのときのみに呼ばれます。
     *
     * すべてのステーションの参加・離脱を知りたい場合は、@ref Station::RegisterOperationEndCallback を使用してください。
     *
     * SessionOperation操作が終った後で登録されたコールバック関数が呼び出されるようになります。
     * このコールバックには、呼び出されるSessionOperationへのポインタが渡されます。
     * このコールバックは、セッションに参加している、あるいはセッションから離脱している
     * ステーションで呼び出されます。
     *
     * @param[in]   pfOperationEnd コールバック関数へのポインタを指定します。
     * @see     JoinSessionOperation, LeaveSessionOperation
     */
    static void RegisterOperationEndCallback(void (*pfOperationEnd)(SessionOperation* pOperation));

    // {secret}
    static void CallOperationEndCallback(SessionOperation* pOperation);

    // group=A_Session States

    // {secret}
    qBool SetSystemState(qByte byNewState);

    // {secret}
    qByte GetSystemState();

    /*
    Description: Sets a user-defined state.
    Parameters:
    uiNewState: A user-defined state.
    Returns: true if the user-defined state was successfully set, false if not or an error occurred.
    The error SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA will be signalled if this method is called on a Duplica.
    Remarks: The user-defined state is not interpreted by $PRODUCTNAME$. $PRODUCTNAME$
    simply replicates it to all the stations. This method must be called on the session master.  The state is a
    distributed value, /i.e./ it is set on the session master but duplicated to
    all the stations that belong to the session.
    See Also: GetUserDefinedState
    */

    /*!
     * @brief   ユーザー定義の値を設定します。
     *
     * ユーザー定義の値はNet-Zによって利用されません。
     * Net-Zはただ単純にこの値をすべての参加者に対して複製します。
     * この関数はセッションマスターのみが呼び出すことが出来ます。
     * そのため、セッションマスターによって設定、配布され、他の参加者は
     * ただそれを受け取るだけの用途に利用してください。
     *
     * @param[in]   uiNewState 新しいユーザー定義の値を指定します。
     * @retval  true    設定に成功した場合に返ります。
     * @retval  false   設定に失敗した場合に返ります。
     *                  セッションマスター以外によってこの関数が呼び出された場合、
     *                  SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICAエラーがシグナルされます。
     * @see     GetUserDefinedState, DuplicatedObject::IsADuplicationMaster
     */
    qBool SetUserDefinedState(qUnsignedInt32 uiNewState);

    /*
    Description: Returns the user-defined state.
    Returns: The user-defined state.
    Remarks: This method can be called on any station that belongs to the session.  It
    returns the user state defined on the session master via the method SetUserDefinedState.
    See Also: SetUserDefinedState
    */

    /*!
     * @brief   ユーザー定義の値を取得します。
     *
     * この関数は、セッションのすべての参加者から呼び出すことが出来ます。
     * セッションマスターが@ref SetUserDefinedState 関数を通じて呼び出した、
     * ユーザー定義の状態(ステート)を取得することが出来ます。
     *
     * @return  セッションマスターによって設定された、ユーザー定義の値が返ります。
     * @see     SetUserDefinedState
     */
    qUnsignedInt32 GetUserDefinedState();

    // group=A_Session Create/Join

    /*
    Description: Creates a $PRODUCTNAME$ Session.
    Parameters:
    szSessionName: The name of the session that should be created. This is useful when
    calling QuerySessions from another station.
    bNetworked: True (the default), if the session will be networked. False (non-networked), otherwise.
    ui16WKPortNumber: (optional) If Proprietary Reliable UDP (PRUDP) is used for transport, sets the IP
    port number on which to establish the connection. Otherwise the port specifed
    by RootTransport::SetWellKnownPortNumber is used. As a last resort, port 9103 is used. When this parameter is set,
    $PRODUCTNAME$ resets the well-known port number to this value.
    Returns:  true if the session was successfully created, false if not or an error occurred.
    The error SYSTEMERROR_DC_CLUSTER_REDEFINITION is signalled if this session has already been created,
    SYSTEMERROR_GEN_INVALID_PARAMETER if ui16WKPortNumber is invalid or unknown, and
    SYSTEMERROR_OBJDUP_WKHANDLE_NOT_INITIALIZED if a duplicated object corresponding to a
    declared well-known handle has not as yet been initialized.
    Remarks:
    The following sample code creates a session called "John's session":

    $FACADENAME$ *g_p$FACADENAME$ = qNew $FACADENAME$;

    if (!(Session::CreateSession(TEXT("John's session")))){
        //SystemError
    };

    To create a session without giving it a name:

    if(!(Session::CreateSession())){
        // Process the session creation error
        // SystemError
    }

    Note that this method will fail if the well-known port number that this session tries to use is already used
    by a previously created session. For example, this will occur if you try to create more than one $PRODUCTNAME$
    Session on a single machine without changing the default well-known port number.

    Note: As $PRODUCTNAME$ guarantees that all well-known objects are discovered before the game Session is created,
    DuplicatedObject::CreateWellKnown must be called before a Session is created to ensure that all
    well-known objects are created at the appropriate time. If not, the call to create the session will fail.

    See Also: CreateOrJoinSession, DuplicatedObject::CreateWellKnown
    */

    /*!
     * @brief   セッションを作成します。
     *
     * この関数を呼び出した場合、セッションマスターとなります。
     * @a szSessionNameとして指定した名前で、セッションの名前が決定されます。
     * この名前は、製品のデバッグ時に、セッションへの参加者が@ref QuerySessions 関数を呼び出す
     * 際にセッションを識別するために役立ちます。
     *
     * @a bNetworked はセッションがネットワーク対応かどうかを定義する Boolean です。
     * bNetwork パラメータが偽の場合、他のプレイヤーがセッションに参加することはできません。
     * また、セッションは Session::QuerySessions が呼び出されても列挙されません。
     * ネットワーク対応のコードで、シングルプレイヤーモードを実装する用途で利用できます。
     *
     * 
     * ローカル通信のセッションを作る際、UDSネットワークのマスターがセッションのマスターとなります。
     * UDSクライアントがこの関数を呼び出すと アサートエラーが発生するか、falseが返ります。
     *
     * 
     * 
     *
     * インターネット通信の場合は、本関数を呼び出すまえに @ref Network::SetP2PDataPacketSessionSignatureKey() を用いて、
     * ゲームサーバーから配布される署名鍵を設定する必要があります。
     *
     * @param[in]   szSessionName       セッションの名前を指定します(デフォルトはNULL)。
     * @param[in]   bNetworked          セッションがネットワーク対応かを設定します。通常trueを指定してください(デフォルト)。
     *
     * @retval  true    正常にセッションが作成できた場合に返ります。
     * @retval  false   何らかのエラーが発生した場合に返ります。<br>
     *                  すでにセッションが構築されている場合、SYSTEMERROR_DC_CLUSTER_REDEFINITIONがシグナルされます。<br>
     *                  定義されている既知オブジェクトが初期化されていない場合、SYSTEMERROR_OBJDUP_WKHANDLE_NOT_INITIALIZEDがシグナルされます。<br>
     *                  UDSクライアントがこの関数を呼び出すと。アサートエラーが発生するか、falseが返ります。
     *
     */
    static qBool CreateSession(const qChar *szSessionName=NULL, qBool bNetworked=true);
    // ※この関数は呼び出さないでください！
    static qBool CreateSession(const qChar *szSessionName, qUnsignedInt16 ui16WKPortNumber);

    // {secret}
    static qBool SessionMasterSignalsFaultsToAll();
    // {secret}
    static void SetMasterSignalsFaultsToAll();
    // {secret}
    static void ClearMasterSignalsFaultsToAll();
    // {secret}
    static qBool IsAllowingDynamicPortAllocation()
    {
        return s_bUseDynamicPort;
    };
    // {secret}
    static void SetAllowDynamicPortAllocation()
    {
        s_bUseDynamicPort = true;
    }
    // {secret}
    static void ClearAllowDynamicPortAllocation()
    {
        s_bUseDynamicPort = false;
    }

private:
    // {secret}
    static void CreateSessionCleanUp(qUnsignedInt32 curStep);

public:

    /*
    Description: Joins a $PRODUCTNAME$ Session.
    Parameters:
    refStationURL: A reference to the StationURL of the station via which you wish to join the session.
    szMachine: The name of a station that belongs to the session that will be joined using Proprietary
    Reliable UDP (PRUDP) as the transport.
    pSessionDescription: A description of the session that will be joined using PRUDP containing the
    session's location and other information (session name, library version number).
    lstURLs: A list of possible StationURL s through which you wish to join the session. If a StationURL is not
    accompanied by a port number, the well-known port number set by RootTransport::SetWellKnownPortNumber
    is used. This parameter is typically used to join a $PRODUCTNAME$ session via Rendez-Vous.
    pCallContext: A pointer to the CallContext of this call. If this parameter is defined, the call will
    be made asynchronously.
    lstStationURL: A list of possible StationURL s with which to join the session.
    If port numbers are not included, then $PRODUCTNAME$ uses the well-known port number set by
    RootTransport::SetWellKnownPortNumber. Typically used by Rendez-Vous to join a $PRODUCTNAME$ session.
    Returns:  true if the method has been initalized, false if not or an error occurred. The following errors can be signaled:

          o SYSTEMERROR_DC_CLUSTER_JOIN_DENIED if the station does not pass the internal $PRODUCTNAME$
            authentication,
          o SYSTEMERROR_OBJDUP_WKHANDLE_WERE_CREATED if the station attempting to join the session
            has previously created well-known objects (only the session creator can create well-known objects,
            joining stations discover them),
          o SYSTEMERROR_GEN_INVALID_PARAMETER if pSessionDescription is invalid or
            unknown,
          o SYSTEMERROR_GEN_INVALID_OPERATION if szMachine is invalid or unknown,
          o SYSTEMERROR_OBJDUP_TRANSPORT_NOT_FOUND if no transport was found for the specified URL, and
          o SYSTEMERROR_DC_CLUSTER_REDEFINITION if an attempt was made to create or join the session more than once.

    The JoinSession method will fail if the caller of this method cannot connect to all current session
    participants, as may happen if a given participant is behind an incompatible NAT device. It will also
    fail if the list of StationURL s passed pertain to more than one Rendez-Vous ConnectionID.

    In the case of failure, the asynchronous version of JoinSession sets the CallContext outcome to one of the following values:

          o QERROR(DOCore, JoinFailed): in the case of a network fault, most generally;
          o QERROR(DOCore, ConnectivityTestFailed): one of the stations cannot be reached (e.g. in NON_SUPPORT Service environment,
            this error could correspond to QOS failure);
          o QERROR(DOCore, JoinDenied): the session master denied the join request. Note: If you wish to have the
            master return another error code, you could invoke JoinSessionOperation::Deny with one that differs from QERROR(DOCore, JoinDenied).

    Note: A call to JoinSession will wait up to the default connection timeout for the Session master to respond. If no response is received
    within this time then the call will timeout and will therefore, return false. The connection timeout is defined via
    RootTransport::SetConnectEndPointTimeout and has a default value of 30 seconds.

    Remarks: This method will be called asynchronously if the pCallContext parameter is specified and synchronously if
    not. If the call is Synchronous, the system will wait for the call to return a value or an error
    before performing the next task. If Asynchronous, you may perform other tasks while waiting for the call to complete.
    The outcome of this call is given by CallContext::GetOutcome. Take a look at DOCoreErrorCode and DOCoreSuccessCode
    (in DOCoreErrors.h). Regardless of which signature you use, in the end JoinSession
    passes a StationURL, which if not stated directly is either extracted or constructed from the supplied parameters.

    When JoinSession is called, a pre-connection test is automatically performed according to the lobby used,
    /e.g./ Quazal Rendez-Vous, NON_SUPPORT Service, to ensure that the joining player can connect to all current participants
    thereby avoiding inconsistent object duplication and player connections, which commonly occur when players are
    located behind incompatible NAT devices.

    A given Station should always join a Session via the Session master's Station, or in Rendez-Vous the gathering host's station.
    For example, the following sample code joins a session on machine "BobsMachine":

    $FACADENAME$ *g_p$FACADENAME$ = qNew $FACADENAME$;

    if(!(Session::JoinSession(TEXT("BobsMachine")))){
        //SystemError
    }

    If you do not know the name of the Session you wish to join, you can use QuerySessions to find the available sessions
    and then join the one you want.

    Before the JoinSession method exits, it is guaranteed that the SessionClock is synchronized and all well-known
    duplicated objects and all
    IDGenerator core objects are discovered, and that these objects will be discovered before any user
    duplicated objects. In addition, when single thread mode is used (refer to Threading Model), it is
    guaranteed that no UserDO will be discovered within the JoinSession method. The first UserDO will only
    be discovered when you call Scheduler::Dispatch after calling JoinSession. On the other hand, in
    multi-thread mode, some UserDOs may be discovered before the JoinSession method terminates as the system
    may dispatch the creation of a UserDO between the end of JoinSession and the next call.
    Once a station has successfully established a connection with one station in the session, the joining station
    will then simultaneously try to connect to each other station in the session.

    When JoinSession is called, the SessionClock on the joining station is synchronized with that of the Session master.
    If the latency between the two stations is symmetric, then the SessionClock on the joining station will be quite accurate
    from the first synchronization pulse. On the other hand, if the latency is non-symmetric then the joining station's
    SessionClock will take a certain period of time (~20 seconds with the default settings) to synchronize. In most cases
    this will not be noticeable however, if dead reckoning is used this will lead to inaccurate dataset extrapolation
    until the SessionClock s are synchronized.

    Note: If you use a list of StationURL s to join a $PRODUCTNAME$ session via Rendez-Vous and Rendez-Vous
    has NAT traversal invoked via its NAT Traversal Service, then the $PRODUCTNAME$ session will automatically
    use NAT traversal where required.

    Note: The Session::JoinSession call may block while performing the DNS lookup on the server name provided when
    using Net-Z in single threaded mode. This is mainly due to the blocking semantic of the gethostbyname API call
    on most platforms. Creating a thread to perform the call that circumvents this behaviour may not be appropriate
    when the user has specified that threading should not be used. To avoid this problem, the user should perform the
    DNS lookup on the server's address so that JoinSession is always called using a numeric address.

    See Also: CreateOrJoinSession, QuerySessions, DuplicatedObject::CreateWellKnown, RegisterConnectivityTester, JoinSessionOperation.
    */

    /*!
     * @brief   セッションに参加します。
     *
     * このオーバーロードは同期実行されるため、セッションへの参加が完了するまで関数は返りません。
     * 非同期(ユーザースレッドモードの場合、@ref Scheduler::Dispatch のタイミングで)実行するためには、
     * @ref CallContext を指定するオーバーロードを利用してください。
     *
     * 本関数は、ローカル通信以外で利用することができません。インターネット通信での利用の際には、Assertによる停止、もしくは、失敗します。
     *
     * 
     *
     * UDSマスターがこの関数を呼び出すと アサートエラーが発生するか、falseが返ります。
     *
     * @param[in]   refStationURL 参加するセッションのステーションのStationURLオブジェクトへの参照を指定します。
     * @retval  true 正しく呼び出しが開始されたときに返ります(実際に成功したかどうかは@a pCallContextの状態を参照してください)。
     * @retval  false Join処理に失敗しました。NetZインスタンスはローカルセッションオブジェクトを保持するので、
     *          再度セッションに参加するにはNetZインスタンスを作りなおす必要があります。<br>
     *          また、インターネット通信で、NAT超えを行っている場合には、StopNATSessionおよびStartNATSessionでNAT超えをし直す必要があります。<br>
     *          UDSマスターがこの関数を呼び出すと アサートエラーが発生するか、falseが返ります。
     */
    static qBool JoinSession(const StationURL& refStationURL);

    /*!
     * @brief   セッションに参加します。
     *
     * このオーバーロードは同期実行されるため、セッションへの参加が完了するまで関数は返りません。
     * 非同期(ユーザースレッドモードの場合、@ref Scheduler::Dispatch のタイミングで)実行するためには、
     * @ref CallContext を指定するオーバーロードを利用してください。
     *
     * 
     *
     * インターネット通信の場合は、本関数を呼び出すまえに @ref Network::SetP2PDataPacketSessionSignatureKey() を用いて、
     * ゲームサーバーから配布される署名鍵を設定する必要があります。
     *
     * UDSマスターがこの関数を呼び出すと アサートエラーが発生するか、falseが返ります。
     *
     * ※この関数はデバッグ目的に用意されています。製品版ではこの機能を利用しないでください。
     *
     * @param[in]   szMachine 参加する対象のPRUDPを伝送層に利用するセッションに所属するマシンの名前を指定します。
     * @retval  true 正しく呼び出しが開始されたときに返ります(実際に成功したかどうかは@a pCallContextの状態を参照してください)。
     * @retval  false Join処理に失敗しました。NetZインスタンスはローカルセッションオブジェクトを保持するので、
     *          再度セッションに参加するにはNetZインスタンスを作りなおす必要があります。<br>
     *          また、インターネット通信で、NAT超えを行っている場合には、StopNATSessionおよびStartNATSessionでNAT超えをし直す必要があります。<br>
     *          UDSマスターがこの関数を呼び出すと アサートエラーが発生するか、falseが返ります。
     */
    static qBool JoinSession(const qChar *szMachine);

    /*!
     * @brief   セッションに参加します。
     *
     * このオーバーロードは同期実行されるため、セッションへの参加が完了するまで関数は返りません。
     * 非同期(ユーザースレッドモードの場合、@ref Scheduler::Dispatch のタイミングで)実行するためには、
     * @ref CallContext を指定するオーバーロードを利用してください。
     *
     * 
     *
     * インターネット通信の場合は、本関数を呼び出すまえに @ref Network::SetP2PDataPacketSessionSignatureKey() を用いて、
     * ゲームサーバーから配布される署名鍵を設定する必要があります。
     *
     * UDSマスターがこの関数を呼び出すと アサートエラーが発生するか、falseが返ります。
     *
     * @param[in]   pSessionDescription セッション記述子へのポインタを指定します。
     * @retval  true 正しく呼び出しが開始されたときに返ります(実際に成功したかどうかは@a pCallContextの状態を参照してください)。
     * @retval  false Join処理に失敗しました。NetZインスタンスはローカルセッションオブジェクトを保持するので、
     *          再度セッションに参加するにはNetZインスタンスを作りなおす必要があります。<br>
     *          また、インターネット通信で、NAT超えを行っている場合には、StopNATSessionおよびStartNATSessionでNAT超えをし直す必要があります。<br>
     *          UDSマスターがこの関数を呼び出すと アサートエラーが発生するか、falseが返ります。
     */
    static qBool JoinSession(SessionDescription *pSessionDescription);

    /*!
     * @brief   セッションに参加します。
     *
     * このオーバーロードは同期実行されるため、セッションへの参加が完了するまで関数は返りません。
     * 非同期(ユーザースレッドモードの場合、@ref Scheduler::Dispatch のタイミングで)実行するためには、
     * @ref CallContext を指定するオーバーロードを利用してください。
     *
     * インターネット通信の場合は多くのケースで一つのステーションに複数のURLが割り当てられます。
     * このオーバーロードを利用すると接続可能なURLを自動的に選択し、接続します。
     *
     * 
     *
     * インターネット通信の場合は、本関数を呼び出すまえに @ref Network::SetP2PDataPacketSessionSignatureKey() を用いて、
     * ゲームサーバーから配布される署名鍵を設定する必要があります。
     * MatchMakingClient::GetSessionURLs()で取得したURLリストは、加工せずにそのまま与えてください。
     *
     * UDSマスターがこの関数を呼び出すと アサートエラーが発生するか、falseが返ります。
     *
     * @param[in]   lstURLs 参加するセッションのステーションのリストへの参照を指定します。   MatchMakingClient::GetSessionURLs()で取得したURLリストを、加工せずにそのまま与えてください。
     * @retval  true 正しく呼び出しが開始されたときに返ります(実際に成功したかどうかは@a pCallContextの状態を参照してください)。
     * @retval  false Join処理に失敗しました。NetZインスタンスはローカルセッションオブジェクトを保持するので、
     *          再度セッションに参加するにはNetZインスタンスを作りなおす必要があります。<br>
     *          また、インターネット通信で、NAT超えを行っている場合には、StopNATSessionおよびStartNATSessionでNAT超えをし直す必要があります。<br>
     *          UDSマスターがこの関数を呼び出すと アサートエラーが発生するか、falseが返ります。
     */
    static qBool JoinSession(const qList<StationURL>& lstURLs);

    /*!
     * @brief   セッションに参加します。
     *
     * このオーバーロードは非同期(ユーザースレッドモードの場合、@ref Scheduler::Dispatch のタイミングで)
     * 実行されるため、この関数は即座に返ります。
     * 実行状況は@a pCallContextで指定した呼び出しコンテキストを監視してください。
     *
     * 本関数は、ローカル通信以外で利用することができません。インターネット通信での利用の際には、Assertによる停止、もしくは、失敗します。
     *
     * CallContextのタイムアウトは @ref RootTransport::SetNATTraversalTimeout() に設定した値の2倍+α(５秒程度) が適切です。
     *
     * 
     *
     *
     * UDSマスターがこの関数を呼び出すと アサートエラーが発生するか、falseが返ります。
     *
     * @param[in]   pCallContext    非同期実行を行うための呼び出しコンテキストを指定します。
     * @param[in]   refStationURL   参加するセッションのステーションのStationURLオブジェクトへの参照を指定します。
     * @retval  true 正しく呼び出しが開始されたときに返ります(実際に成功したかどうかは@a pCallContextの状態を参照してください)。
     * @retval  false Join処理に失敗しました。NetZインスタンスはローカルセッションオブジェクトを保持するので、
     *          再度セッションに参加するにはNetZインスタンスを作りなおす必要があります。<br>
     *          また、インターネット通信で、NAT超えを行っている場合には、StopNATSessionおよびStartNATSessionでNAT超えをし直す必要があります。<br>
     *          UDSマスターがこの関数を呼び出すと アサートエラーが発生するか、falseが返ります。
     */
    static qBool JoinSession(CallContext* pCallContext, const StationURL& refStationURL);

    /*!
     * @brief   セッションに参加します。
     *
     * このオーバーロードは非同期(ユーザースレッドモードの場合、@ref Scheduler::Dispatch のタイミングで)
     * 実行されるため、この関数は即座に返ります。
     * 実行状況は@a pCallContextで指定した呼び出しコンテキストを監視してください。
     *
     * CallContextのタイムアウトは @ref RootTransport::SetNATTraversalTimeout() に設定した値の2倍+α(５秒程度) が適切です。
     *
     * インターネット通信の場合は多くのケースで一つのステーションに複数のURLが割り当てられます。
     * このオーバーロードを利用すると接続可能なURLを自動的に選択し、接続します。
     *
     * 
     *
     * インターネット通信の場合は、本関数を呼び出すまえに @ref Network::SetP2PDataPacketSessionSignatureKey() を用いて、
     * ゲームサーバーから配布される署名鍵を設定する必要があります。
     * MatchMakingClient::GetSessionURLs()で取得したURLリストは、加工せずにそのまま与えてください。
     *
     * UDSマスターがこの関数を呼び出すと アサートエラーが発生するか、falseが返ります。
     *
     * @param[in]   pCallContext    非同期実行を行うための呼び出しコンテキストを指定します。
     * @param[in]   lstStationURL           参加するセッションのステーションのリストへの参照を指定します。  MatchMakingClient::GetSessionURLs()で取得したURLリストを、加工せずにそのまま与えてください。
     * @retval  true 正しく呼び出しが開始されたときに返ります(実際に成功したかどうかは@a pCallContextの状態を参照してください)。
     * @retval  false Join処理に失敗しました。NetZインスタンスはローカルセッションオブジェクトを保持するので、
     *          再度セッションに参加するにはNetZインスタンスを作りなおす必要があります。<br>
     *          また、インターネット通信で、NAT超えを行っている場合には、StopNATSessionおよびStartNATSessionでNAT超えをし直す必要があります。<br>
     *          UDSマスターがこの関数を呼び出すと アサートエラーが発生するか、falseが返ります。
     */
    static qBool JoinSession(CallContext* pCallContext, const qList<StationURL>& lstStationURL)
    {
        return JoinSessionImpl(pCallContext, lstStationURL);
    }

    // NOTE 1: using this function will cause the join to be sequential and hence will
    // raise the duration to join the session. It might be suited to increase the connection
    // timeout.
    // NOTE 2: It should not be used in client-server network topology. This flag will be
    // ignored.
    // {secret}
    static void WaitInitializedURLsToJoin();
    //{secret}
    static qBool IsWaitingInitializedURLsToJoin();

#if     NEX_ENABLE_QUAZAL_TERMINATE_SYNCTERMINATION
    // {secret}
    static void UpdateSynchronizedTerminationCondition();
    // {secret}
    static qBool IsTerminationSynchronized();
#endif  // NEX_ENABLE_QUAZAL_TERMINATE_SYNCTERMINATION

private:
    // {secret}
    static qBool JoinSessionImpl(CallContext* pCallContext, const qList<StationURL>& lstStationURL);

public:

    // group=A_Session Discovery

    /*
    $ifdef WII_DWC$
    Description: {html:<b>} Not supported in NetZ for the Wii (DWC){html:</b>} unless custom
    SessionDiscoveryProtocol is registered.
    $else$
    Description: Searches for and returns the SessionDescription of each available session found
    on the network.
    $endif$
    Parameters:
    pOutputList: A pointer to the list of SessionDescription s.
    bFilterMatching: True, the default, if the queried sessions should have the same title
    as the one calling this function.
    uiTimeout: The maximum time in milliseconds that the system will wait. The default value is 1000 ms.
    pContext: A pointer to the CallContext of this call. If this parameter is defined the call will
    be made asynchronously.
    Returns: A list of the SessionDescription s of each available Session found on the network, or 0
    if an error is signalled. If this method is called asynchronously ( /i.e./ pContext is defined),
    this method will return true if the method has been initalized, false if not or an error occurred.
    The SYSTEMERROR_DC_CLUSTER_ALREADY_INITIALIZED is signalled if this
    method is called after a session is joined or created and SYSTEMERROR_GEN_INVALID_PARAMETER
    signalled if no session descriptions are found.

    Remarks:
    $ifdef WII_DWC$
    By default, Session::QuerySessions does not work with NetZ for the Wii (DWC), unless you implement
    and register your own SessionDiscoveryProtocol (see Session::RegisterSessionDiscovery).
    The reason is that LANSessionDiscovery, which allows a peer to search sessions on its LAN
    through IP broadcasting, is normally automatically initialized upon NetZ startup but is not
    supported on the Wii (DWC) platform.
    $endif$

    QuerySession can be called from within a session but also before a session is created or joined. When
    QuerySessions is called, the method will call SessionDiscoveryProtocol::ProbeSessions,
    wait for a certain amount of time, and then call SessionDiscoveryProtocol::GatherProbeResults.
    The developer must define exactly what these two methods do. Refer to the SessionDiscoveryProtocol
    class documentation for details.

    QuerySessions will be called asynchronously if the pContext parameter is specified and synchronously if
    not. If the call is Synchronous, the system will wait for the call to return a value or an error
    before performing the next task. If Asynchronous, you may perform other tasks while waiting for the
    call to complete.

    To search for $PRODUCTNAME$ sessions on the network and print out various information from the
    session descriptions we could do the following. Refer to the SessionDescription class for more
    information regarding the information contained in the session description. A full implementation
    of QuerySessions is given in the EnumerateSessions tool.

    qList<SessionDescription> lstResult;
    if (Session::QuerySessions(&lstResult,false)==0) {
        Display(NEX_T("No session found.\n"));
    } else {
        qList<SessionDescription>::iterator it=lstResult.begin();
        while (it!=lstResult.end()) {
            Display(NEX_T("-------------------------------------------\n"));
            Display(NEX_T("Application Title : %s\n"),it->GetApplicationTitle());
            Display(NEX_T("Application Version : %x\n"),it->GetApplicationVersionNumber());
            Display(NEX_T("Session Name : %s\n"),it->GetSessionName());
            Display(NEX_T("Session ID : %x\n"),it->GetSessionID());
            Display(NEX_T("Connecting StationURL : %s\n"),it->GetStationURL());
            Display(NEX_T("SessionDiscovery type : %s\n"),it->GetSessionDiscoveryType());
            Display(NEX_T("SessionDiscovery Session ID : %s\n"),it->GetSessionDiscoverySessionID());
            Display(NEX_T("Lib. Version Number : %d.%d\n"),
                    MAJOR_VERSION(it->GetLibraryVersionNumber()),
                    MINOR_VERSION(it->GetLibraryVersionNumber()));
            Display(NEX_T("Lib. Build Number : %d.%d\n"),
                    MAJOR_BUILD(it->GetLibraryBuildNumber()),
                    MINOR_BUILD(it->GetLibraryBuildNumber()));
            // Display generic attributes.
            qChar szKey[128];
            if (it->FindFirstAttribute(szKey, 128)) {
                do {
                    qChar szValue[128];
                    it->GetAttribute(szKey, szValue, 128);
                    Display(NEX_T("\t%s = %s\n"), szKey, szValue);
                } while (it->FindNextAttribute(szKey, 128));
            }
            ++it;
        }
    }

    See Also: JoinSession, CreateOrJoinSession, SessionDescription, SessionDiscoveryProtocol::ProbeSessions,
    SessionDiscoveryProtocol::GatherProbeResults
    */
    //n1589:todo2:uiTimeout=1000
    /*!
     * @brief   LAN上に存在するセッションを検索します。
     *
     * socketを利用して接続しているとき、同一のアクセスポイントに接続している
     * 別のプロセスがセッションを開いていないかを調べます。
     * この機能を利用すると、サーバーを介したマッチング操作を省略して
     * socketを利用したP2P機能をテストすることが出来ます。
     *
     * このオーバーロードは同期実行されるため、セッションへの参加が完了するまで関数は返りません。
     * 非同期(ユーザースレッドモードの場合、@ref Scheduler::Dispatch のタイミングで)実行するためには、
     * @ref CallContext を指定するオーバーロードを利用してください。
     *
     * ※この関数はデバッグ目的に用意されています。製品版ではこの機能を利用しないでください。
     *
     * @param[out]  pOutputList     見つかったセッションに対する記述子を受け取るためのリストへのポインタを指定します。
     * @param[in]   bFilterMatching  同一のタイトルを持つセッションを列挙するかどうかを指定します(デフォルトはtrue)。
     * @param[in]   uiTimeout        セッション検索に対するタイムアウトをミリ秒単位で指定します(デフォルトは1000)。
     *
     * @retval  true    セッションの検索が正しく行えた場合に返ります。
     * @retval  false   呼び出しが実行できなかった場合に返ります。
     *                  すでにセッションに参加しているか、あるいは構築した場合はSYSTEMERROR_DC_CLUSTER_ALREADY_INITIALIZEDがシグナルされます。
     *                  セッション記述子が見つからなかった場合、SYSTEMERROR_GEN_INVALID_PARAMETERがシグナルされます。
     *
     * @see     JoinSession, CreateSession
     */
    static size_t QuerySessions(
        qList<SessionDescription>* pOutputList,
        qBool bFilterMatching = true,
        qUnsignedInt32 uiTimeout=1000
    );

    /*!
     * @brief   LAN上に存在するセッションを検索します。
     *
     * socketを利用して接続しているとき、同一のアクセスポイントに接続している
     * 別のプロセスがセッションを開いていないかを調べます。
     * この機能を利用すると、サーバーを介したマッチング操作を省略して
     * socketを利用したP2P機能をテストすることが出来ます。
     *
     * このオーバーロードは非同期(ユーザースレッドモードの場合、@ref Scheduler::Dispatch のタイミングで)
     * 実行されるため、この関数は即座に返ります。
     * 実行状況は@a pCallContextで指定した呼び出しコンテキストを監視してください。
     *
     * ※この関数はデバッグ目的に用意されています。製品版ではこの機能を利用しないでください。
     *
     * @param[in,out] pContext       実行状況をアプリケーション側から判定するための呼び出しコンテキストへのポインタを指定します。
     * @param[out]  pOutputList      見つかったセッションに対する記述子を受け取るためのリストへのポインタを指定します。
     * @param[in]   bFilterMatching  同一のタイトルを持つセッションを列挙するかどうかを指定します(デフォルトはtrue)。
     * @param[in]   uiTimeout        セッション検索に対するタイムアウトをミリ秒単位で指定します(デフォルトは1000)。
     *
     * @retval  true    呼び出しが正常に実行された場合に返ります(実際に成功したかどうかは@a pContextの状態を参照してください)。
     * @retval  false   呼び出しが実行できなかった場合に返ります。
     *                  すでにセッションに参加しているか、あるいは構築した場合はSYSTEMERROR_DC_CLUSTER_ALREADY_INITIALIZEDがシグナルされます。
     *                  セッション記述子が見つからなかった場合、SYSTEMERROR_GEN_INVALID_PARAMETERがシグナルされます。
     *
     * @see     JoinSession, CreateSession
     */
    static qBool QuerySessions(
        CallContext* pContext,
        qList<SessionDescription>* pOutputList,
        qBool bFilterMatching = true,
        qUnsignedInt32 uiTimeout=1000
    );

    //{secret}
    static qBool FindLANSession(
        const qChar *szName,
        SessionDescription *pSignature,
        qUnsignedInt32 uiTimeout = 1000
    );

    /*
    Description: Registers a SessionDiscoveryProtocol object to this Session.
    Parameters:
    pSessionDiscovery: A pointer to the SessionDiscoveryProtocol that is registered to this Session.
    bSelectAsPrimarySessionDiscovery: If true (the default), the specified pSessionDiscovery will be registered
    to the Session when it is created. If false, no Session Discovery Protocol will be registered
    to the Session. It will only be accessible on the LAN.
    Returns: true if the SessionDiscoveryProtocol was successfully registered to this Session, false if not or an
    error occurred.

    Remarks: RegisterSessionDiscovery will register the specified SessionDiscoveryProtocol to this Session.
    A given Session may only have one SessionDiscoveryProtocol registered to it however, a
    SessionDiscoveryProtocol object may be registered to more than one Session. This method must be called
    before a Session is created.
    See Also: UnregisterSessionDiscovery, SessionDiscoveryProtocol
    */
    static qBool RegisterSessionDiscovery(SessionDiscoveryProtocol* pSessionDiscovery, qBool bSelectAsPrimarySessionDiscovery=true);

    /*
    Description: Unregisters a SessionDiscoveryProtocol object from this Session.
    Parameters:
    pSessionDiscovery: A pointer to the SessionDiscoveryProtocol that will be unregistered from this Session.
    Returns: true if the SessionDiscoveryProtocol was successfully unregistered from this Session, false if not or an
    error occurred.
    Remarks: Before a SessionDiscoveryProtocol can be unregistered is must have been registered
    via RegisterSessionDiscovery.
    See Also: RegisterSessionDiscovery, SessionDiscoveryProtocol
    */
    static qBool UnregisterSessionDiscovery(SessionDiscoveryProtocol* pSessionDiscovery);

    /*
    Description: Returns a pointer to the SessionDiscoveryProtocol that is registered to this Session.
    Parameters:
    szSessionDiscoveryType: A unique string, part of the SessionDescription, describing the type of SessionDiscovery.
    Returns: A pointer to a registered SessionDiscoveryProtocol of the specified type.
    Remarks: GetSessionDiscovery will only return a pointer to a SessionDiscoveryProtocol that has
    been previously registered to this Session via RegisterSessionDiscovery.
    See Also: SessionDescription::GetSessionDiscoveryType
    */
    static SessionDiscoveryProtocol* GetSessionDiscovery(qChar* szSessionDiscoveryType);

    // group=A_Session Create/Join

    /*
    Description: Joins or creates a $PRODUCTNAME$ Session.
    Returns:  true if the session was successfully either created or joined,  false if nor or an error occurred.
    The error SYSTEMERROR_DC_CLUSTER_JOIN_DENIED is signalled if the station does not pass the internal $PRODUCTNAME$
    authentication, SYSTEMERROR_OBJDUP_WKHANDLE_WERE_CREATED if the station attempting to join the session
    has previously created well-known objects (only the session creator can create well-known objects,
    joining stations discover them), SYSTEMERROR_OBJDUP_WKHANDLE_NOT_INITIALIZED if a Session
    is created and the well-known objects declared in the DDL were not created in the user-defined
    pfCreateCallback function, and SYSTEMERROR_DC_CLUSTER_REDEFINITION if an attempt to create or join the session
    was made more than once.

    The CreateOrJoinSession method will fail if the caller of this method cannot connect to all current session
    participants, as may happen if a given participant is behind an incompatible NAT device.

    Note: A call to JoinSession will wait up to 10 seconds for the Session master to respond. If no response is received
    within this time then the call will timeout and will therefore, return false.

    Parameters:
    pfCreateCallback: A pointer to the user-defined CreateCallback.
    pfSessionSelection: A pointer to the user-defined SessionSelection function.
    Remarks: This method can be used to join an existing session or to create a session if
    no sessions are found. Essentially, this method is a wrapper to the QuerySessions, JoinSession,
    and CreateSession methods.

    When CreateOrJoinSession is called, $PRODUCTNAME$ will first search for any sessions that currently exist
    and return a list of available sessions via the QuerySessions method. The session to join is then determined
    via the user-defined pfSessionSelection function. If no function is defined then $PRODUCTNAME$ will join the
    first session in the list. If no available sessions are found, this method will create the session. If
    well-known objects have been declared in the DDL then the user must ensure that all well-known objects are
    created before the session is created via the pfCreateCallback function. This function can also be used
    to set information in the SessionDescription before the Session is created.

    When CreateOrJoinSession is called, if a Session is to be joined a pre-connection test is automatically performed according
    to the lobby used, /e.g./ Quazal Rendez-Vous, NON_SUPPORT Service, to ensure that the joining player can connect to all current
    participants thereby avoiding inconsistent object duplication and player connections, which commonly occur when players are
    located behind incompatible NAT devices.

    For example, to join an existing Session or to create a new one we could do the following:

    void InitSessionDiscoveryValues() {
    // Performs the tasks that are required before the session is
    // created, e.g. create well-known objects, initialize the
    // description of the new session to the session description.
    // In this case we do the later.
    Session::GetLocalSessionDescription()->SetAttribute(NEX_T("Cost"), NEX_T("Free"));
    Session::GetLocalSessionDescription()->SetAttribute(NEX_T("Membership"), NEX_T("Not required"));
    Session::GetLocalSessionDescription()->SetApplicationVersionNumbers(1,2,3,4);
    }

    SessionDescription* SessionSelection(qList<SessionDescription>& lstSessions) {
    // 1. Display the content of lstSession.
    // 2. Ask the user to select one of the sessions or to create one.
    // 3. Return a pointer to the selected session or NULL if the user
    //    wishes to create a new one.
    }

    // In the main loop

    // The user is presented with a list of the currently available sessions.
    // He may either join one of these sessions or create a new one.
    Session::CreateOrJoinSession(&InitSessionDiscoveryValues,&SessionSelection);

    Note: As $PRODUCTNAME$ guarantees that all well-known objects are discovered before the game Session is created,
    DuplicatedObject::CreateWellKnown must be called before a Session is created to ensure that all
    well-known objects are created at the appropriate time. If not the call to create the session will fail.

    See Also: CreateSession, JoinSession, QuerySessions, DuplicatedObject::CreateWellKnown, RegisterConnectivityTester
    */

    /*!
     * @brief   セッションへの参加を試み、失敗した場合はセッションを新規作成します。
     *
     * この関数は@ref QuerySessions 、@ref JoinSession 、および@ref CreateSession の、3関数を
     * ラップした関数です。この関数が呼び出されると、まず最初に@ref QuerySessions 関数と同様、
     * 同一アクセスポイントに接続されているプロセスが作成しているセッションを列挙します。
     * もし見つかった場合、どのセッションに参加するかを@a pfSessionSelectionコールバックによって決定します。
     * セッションが見つかり、参加するセッションが決まった場合は@ref JoinSession 関数と同様に、
     * セッションへの参加を行います。
     *
     * もし同一アクセスポイントに接続されているプロセスがセッションを開催していない場合、
     * この関数を実行したデバイスが親となって@ref CreateSession 関数と同様に、セッションを構築します。
     * セッションが構築される前に@a pfCreateCallbackで指定したコールバック関数が呼び出されます。
     *
     * ※この関数はデバッグ目的に用意されています。製品版ではこの機能を利用しないでください。
     *
     * @param[in]   pfCreateCallback    セッションが構築されるときに呼び出されるコールバック関数を指定します(デフォルトはNULL)。
     *                                  もしもDDL(データ定義言語)にて、既知オブジェクトが定義されている場合、このコールバック関数内で
     *                                  すべての既知オブジェクトを作成しておく必要があります。
     *                                  また、セッション記述子(@ref SessionDescription )の情報をセッションが構築される前に設定するためにも利用できます。
     *
     * @param[in]   pfSessionSelection  複数のセッションが見つかった場合にどのセッションに参加するかを決定するためのコールバック関数を指定します。
     *                                  このポインタがNULL(デフォルト)の場合、Net-Zは自動的に一番最初に見つかったセッションに参加します。
     *
     * @retval  true    セッションへの参加か作成に成功したときに返ります。
     * @retval  false   セッションへの参加も作成も失敗した、あるいはエラーが発生したときに返ります。
     *                  Net-Zの内部認証に失敗した場合、SYSTEMERROR_DC_CLUSTER_JOIN_DENIEDがシグナルされます。
     *                  セッションが見つかって参加しようとするときに既知オブジェクトが作成されていた場合には、
     *                  SYSTEMERROR_OBJDUP_WKHANDLE_WERE_CREATEDがシグナルされます(セッションマスターのみが既知オブジェクトを作成できます)。
     *                  セッションが見つからずに、新規作成しようとするときに、既知オブジェクトが作成されていない場合には
     *                  SYSTEMERROR_OBJDUP_WKHANDLE_NOT_INITIALIZEDがシグナルされます(セッションを構築する場合は既知オブジェクトを作成しなければなりません)。
     *                  複数回セッションへ参加しようとしたり、作成しようとしたときにはSYSTEMERROR_DC_CLUSTER_REDEFINITIONがシグナルされます。
     *
     * @see     QuerySessions, JoinSession, CreateSession
     */
    static qBool CreateOrJoinSession(void (*pfCreateCallback)()=NULL,
                                     SessionDescription* (*pfSessionSelection)(qList<SessionDescription>& lstSessions)=NULL);

    // group=A_Session System Callbacks

    /*
    Description: Registers a callback method for the creation of your Well-known duplicated object s.
    Parameters:
    pCallback: A pointer to a function that will be called back when it is time to create your Well-known duplicated object s.
    Remarks: The prototype of this function is: void __CDECL FunctionName( void );
    See Also: UnregisterWellKnownDOsFactory, DuplicatedObject::CreateWellKnown
    */
    static void RegisterWellKnownDOsFactory(WellKnownDOsFactoryFuncPtr pCallback);

    /*
    Description: Unregisters a callback method previously registered via RegisterWellKnownDOsFactory.
    Parameters:
    pCallback: A pointer to a function that was previously passed to RegisterWellKnownDOsFactory.
    See Also: RegisterWellKnownDOsFactory
    */
    static void UnregisterWellKnownDOsFactory(WellKnownDOsFactoryFuncPtr pCallback);

    // {secret}
    void InitSessionDescription(qBool bNetworked=true);
    // {secret}
    static void InitStaticSessionDescription();

    // {secret}
    void UpdateSessionDiscoverySessionDescription();

    //group=A_Session Methods

    /*
    Description: Returns the session description of the local Station.
    Returns: A pointer to the local SessionDescription.
    Remarks: The session description can only be modified by the session master via the SessionDescription
    class.

    Refer to CreateOrJoinSession for an example of this method in use.
    See Also: SessionDescription
    */

    /*!
     * @brief   ローカルステーションのセッション記述子へのポインタを取得します。
     *
     * セッション記述子は、セッションマスターのみが変更することが出来ます。
     *
     * @return  ローカルステーションのセッション記述子へのポインタが返ります。
     */
    static SessionDescription* GetLocalSessionDescription();

    /*
    Description: Sets the well-known port number used by $PRODUCTNAME$.
    Parameters:
    ui16WKPortNumber: The well-known port number that will be used. The default is port 9103.
    Remarks: By default, $PRODUCTNAME$ will use port 9103 as the default well-known port.
    If the well-known port cannot be used, the system will open a dynamic port. The default well-known
    port can be changed via this method, which must be called before the session is created or joined.
    Nevertheless, in general you should not need to change the well-know port number except where necessary
    in order to avoid a clash with another application that uses the same port number, such as if you wish
    to create more than one session on the same station.

    Note: This method is no longer valid and has been replaced by RootTransport::SetWellKnownPortNumber.
    See Also: CreateSession, JoinSession
    */
    // ※この関数は呼び出さないでください！
    static void SetPRUDPWKPortNumber(qUnsignedInt16 ui16WKPortNumber);

    // {secret}
    static qBool JoinIsAllowed();

    // {secret}
    void SetJoinDenialDelayOnPromotion(TimeInterval tiDelay)
    {
        s_tiJoinDenialDelayOnPromotion = tiDelay;
    }

    // {secret}
    qBool SynchronizeTermination(DOHandle hLeavingStation);
    // {secret}
    void RetrieveURLs(DOHandle hStation, qList<StationURL>* plstURLs);
    /*!
     * @brief   ネットワークから切断されたときに、フォルト処理なく強制的にセッションから離脱する関数です。
     *
     * この関数は以下の条件でのみ使用可能です。
     *
     * @li 自分以外のStationがセッションに残っているが、明らかにネットワークから切断されているとき
     * @li Stationが自分しかおらず一人ぼっちのとき
     *
     * この関数を使用するとフォルト処理が実行されません。
     * フォルト処理が省かれることにより、ステーション間で共有すべき情報に
     * 不整合が発生して、P2P セッションの継続が困難になります。
     *
     * 親と子を含めたP2P セッション全体を強制終了させたい場合は、
     * @ref GameOver() を使用してください。
     *
     * フォルト処理が必要ない場合にこの関数を使用すれば、
     * JobTerminateDOCoreが素早く完了できる
     * （JobTerminateDOCore::SyncTerminationで時間がかからない）
     * という利点があります。
     *
     * この関数を使用後はホストマイグレーションが出来なくなります。
     *
     * NetZ::SetTerminateImmediately()にtrueが自動設定されます。
     *
     *
     * @see     GameOver(), NetZ::SetTerminateImmediately()
     */
    void DestroySession();


    void DestroySessionWithMasterCheckFromRemote();
    void DestroySessionFromRemote();
    void DestroySessionWithoutLock();

    /*
    Description: Indicates that the Net-Z session is terminating.
    Remarks: Set when the master calls the RMC GameOver() to terminate the session on all
    stations. In a ClientServer topology, if the master encounters a fault the game
    session becomes invalid. When this happens and the system detects the server's fault,
    it sets GameIsOver to True to let the game know that the session is no longer valid.

    The game can use this flag, for example, to make the following tests at each
    iteration:

         if (finished_game) {
             wait_until_master_says_GameOver;
             delete NetZ;
         }

         if (done_playing_& ! CheckGameIsOver()) {
             delete NetZ;
         }

         if (didnt_know_game_finished_but_master_says_GameOver ) {
             delete NetZ;
         }

    See Also: GameOver
    */

    /*!
     * @brief   Net-Zのセッションが終了中であるかどうかを取得します。
     *
     * セッションマスターがセッションを終了させるために
     * @ref GameOver() 関数によってすべての参加者にRMC(リモートメソッドコール)を
     * 送信したときに設定されます。
     * また、  Station::KickStation() や、 Station::KickMe() により、
     * キックされたときに設定されます。
     *
     * trueの場合には、Terminate処理を実行してNetZを終了してください。
     *
     * クライアントサーバー型のネットワークトポロジーでは、セッションマスターに
     * 障害が発生したときもセッションが無効になります。システムがサーバー障害を
     * 検知したときに、フラグがtrueにセットされ、セッションがもう存在しないことをゲームに伝えます。
     *
     * @retval  true  セッションが終了中であるときに返ります。
     * @retval  false セッションが存在しているときに返ります。
     *
     * @see NetZ::SetTerminateImmediately(), GameOver(), Station::KickStation(), Station::KickMe()
     */
    qBool CheckGameIsOver();

    /*
    Description: Called by the Session Master to set a fault on all stations and set the GameIsOver flag
    to true.
    Remarks: Called by the Session Master to terminate a game session.
    This may be needed in an instance where, for example, all stations decide to leave a game at about the same
    time and, while they are attempting to migrate their objects, provoke a general deadlock such that one or
    many stations may corrupt their instance of Net-Z.

    If, for whatever reason, a Master determines that its game session should shut down, it can call this RMC
    to set a fault and the GameIsOver flag on all stations participating in the session before terminating its
    own instance of Net-Z as follows:

            if (master & finished_game) {
                Session::GameOver();
                delete NetZ;
            }

    See Also: GameIsOver()
    */

    /*!
    * @brief    すべてのセッション参加者にこのセッションが終了したことを通知します。
    *
    * この関数はセッションマスターのみが呼び出すことが出来ます。
    * セッション参加者は自分がマスターだと思っているマスターからの呼び出しに応じて、ゲームを終了します。
    *
    * マスターがセッションをシャットダウンすべきと決定したら、
    * このRMCを呼び出し、Net-Zのインスタンスを終了する前に
    * すべてのセッション参加者のゲーム終了フラグをオンにします。
    * 子は、 CheckGameIsOver() を実行して、 GameOver() が呼ばれたかチェックを行い、Terminate処理を実行してNetZを終了してください。
    * GameOver() を使用する場合、マイグレーションは行われません。

    * 多数台接続時に同時に切断が行われるとマイグレーション処理が長くなり、NetZオブジェクトの終了処理がタイムアウトする可能性が高くなります。
    * タイムアウトを回避するためには、セッションマスターがNetZオブジェクトの終了処理を実行する直前に
    * Session::GameOver()を呼び出し、全ステーションに対してセッションの終了を通知する方法があります。
    *
    * セッションマスター、セッション参加者ともに NetZ::SetTerminateImmediately() へtrueが自動設定されます。
    *
    * @retval   true    すべてのセッション参加者にセッションの終了を通知したときに返ります。
    * @retval   false   セッションへの参照が無効なときに返ります。
    * また、セッションマスター以外がこの関数を呼び出したときにも返ります。
    *
    * @see NetZ::SetTerminateImmediately(), CheckGameIsOver()
    */
    qBool GameOver();


    /*!
     * @internal
     * @brief   セッション内のステーションの状態変化（セッションへの参加・離脱）時に呼び出されるコールバック関数を登録します。
     *
     * StationEvent::Participated よりも前にそのStationからのパケットが届くことがある問題があるので、注意してください。
     *
     * @param[in] callback コールバック関数へのポインタを指定します。
     */
    void RegisterStationEventCallback( StationEventCallback callback );

    /*!
     * @internal
    @brief 登録されているステーション状態変化コールバック関数を登録解除します。
     */
    void UnregisterStationEventCallback();


    /*!
     * @brief ステーション不一致を検出するためセッションマスターからステーション数を通知する間隔を設定します。
     *
     * @param time 送信間隔をmsec単位で指定します。
     *
     * @see GetSyncStationListInterval
     * @see SetDetectStationInconsitencyThresholdCount,GetDetectStationInconsitencyThresholdCount
     */
    static void SetSyncStationListInterval(TimeInterval time)
    {
        s_tellStationNumberInterval = time;
    }

    /*!
     * @brief ステーション不一致を検出するためセッションマスターからステーション数を通知する間隔取得します。
     *
     * @return 送信間隔(単位はmsec)を返します。デフォルト値は、5000msecです。
     *
     * @see SetSyncStationListInterval
     * @see SetDetectStationInconsitencyThresholdCount,GetDetectStationInconsitencyThresholdCount
     */
    static TimeInterval GetSyncStationListInterval()
    {
        return s_tellStationNumberInterval;
    }


    /*!
     * @brief ステーション不一致を検出するチェック回数の閾値を指定します。
     *
     * 子において、セッションマスターから通知されたステーション数とローカルのステーション数が指定回数連続して
     * 等しくない場合に、GetFatalErrorとしてQERROR(DOCore, StationInconsistency)で
     * 取得可能なFatalエラーが発生します。
     * 指定する回数とGetSetSyncStationListIntervalの値を掛け合わせた値が、
     * RootTransport::GetNATTraversalTimeoutの値より5000から10000[msec]大きくなるように指定してください。
     *
     *
     * @param count ステーション不一致検出閾値
     *
     * @see GetDetectStationInconsitencyThresholdCount
     * @see SetSyncStationListInterval, GetSyncStationListInterval
     * @see RootTransport::SetNATTraversalTimeout, RootTransport::GetNATTraversalTimeout
     * @see GetFatalError
     */
    static void SetDetectStationInconsitencyThresholdCount(size_t count)
    {
        s_detectStationInconsitencyThresholdCount = count;
    }

    /*!
     * @brief ステーション不一致を検出するチェック回数の閾値を取得します。
     *
     * @return ステーション不一致検出閾値
     *
     * @see SetDetectStationInconsitencyThresholdCount
     * @see SetSyncStationListInterval, GetSyncStationListInterval,
     * @see RootTransport::SetNATTraversalTimeout
     */
    static size_t GetDetectStationInconsitencyThresholdCount()
    {
        return s_detectStationInconsitencyThresholdCount;
    }

    // Remarks: This method traces whether or not the Session uses the well-known port.
    virtual void Trace(qUnsignedInt64 uiTraceFlags) const;

    qInt CountDirectConnectedStations() const;
    qInt CountRoutingConnectedStations() const;

#if NEX_ENABLE_HYBRID_DETECT_LIMITATION
    /*!
     * @brief   セッション全体に存在するマスター経由通信の数を取得します。
     *
     * @ref RootTransport::GetRoutingHistoryTTL() [ms]以内に通信の発生したマスター経由通信の数を返します。
     * 送信、受信をそれぞれカウントします。つまり、あるステーションが別の1台のステーションとマスター経由通信していた場合、この関数は2を返します。
     * 本関数を使用する前にはあらかじめ@ref RootTransport::SetRoutingHistoryUpdateSpan で0より大きい更新間隔を設定する必要があります。
     * この関数はセッションマスターのみ呼び出せます。
     * @return  セッション全体に存在するマスター経由通信の数を返します。
     */
    size_t GetNumOfRoutingConnections();
#endif
    void SyncStationList(qVector<qUnsignedInt32> &stationHandleValues); //from RMC

    void SendSynStationList(); //from JobSynStationListr::Execute

    void DisableJobSyncStationList();

private:
    static Time s_tPromotionTime;
    static TimeInterval s_tiJoinDenialDelayOnPromotion;
    static qBool s_bUseDynamicPort;

    DORef m_refMulticastRoot;

    // Synchronization of the termination is done using the
    // following variables...
    //{secret}
    qList <DOHandle> m_lstLeavingStations;
    DOHandle m_dohLeavingRemoteSessionMaster;
    DOHandle m_dohLeavingLocalSessionMaster;

    qBool m_bGameIsOver;

    static qBool s_bMasterSignalsFaultsToAll;
    static qBool s_bWaitInitializedURLsToJoin;

    // Callbacks.
    //{secret}
    static JoinSessionOperationCallback s_pfApproveOperation;
    static void (*s_pfOperationBegin)(SessionOperation* pOperation);
    static void (*s_pfOperationEnd)(SessionOperation* pOperation);

    JobSyncStationList* m_pJobSynStationList;
    size_t m_detectInconsitencyCount;
    size_t m_detectNeedDisconnectInconsitencyCount;
    size_t m_prevLocalStations;
    size_t m_prevExpectStations;
    DOHandle m_prevMasterStation;


    static void CheckDetectStationInconsitencyThreshold();

    void EnableJobSyncStationList();
    void StationNumberCheck(size_t expectStations, size_t actualStations);
    void ResetTellStationCounts()
    {
        m_detectInconsitencyCount = 0;
        m_detectNeedDisconnectInconsitencyCount = 0;
        m_prevLocalStations = 0;
        m_prevExpectStations = 0;
        m_prevMasterStation = INVALID_DOHANDLE;
    }

    static qBool InitPrameters();


#if     NEX_ENABLE_QUAZAL_TERMINATE_SYNCTERMINATION
    // {secret}
    static PseudoGlobalVariable<qBool> s_bSynchronizedTerminationCondition;
    // {secret}
    static PseudoGlobalVariable<qBool> s_bNeedToUpdateSynchronizedTerminationCondition;
#endif  // NEX_ENABLE_QUAZAL_TERMINATE_SYNCTERMINATION


    static TimeInterval s_tellStationNumberInterval;
    static size_t s_detectStationInconsitencyThresholdCount;

#ifdef NEX_ENABLE_CTR_UDS
    void RegisterJobCheckVirtualNetworkDisconnected();
    void UnRegisterJobCheckVirtualNetworkDisconnected();
    JobCheckVirtualNetworkDisconnected* m_pJobCheckVirtualNetworkDisconnected;
#endif
};

#ifdef NEX_ENABLE_CTR_UDS
class JobCheckVirtualNetworkDisconnected : public Job
{
public:
    JobCheckVirtualNetworkDisconnected();
    virtual ~JobCheckVirtualNetworkDisconnected() {}
    virtual void Execute();
};
#endif

END_NAMESPACE_NEX

