﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

// This file must have the name of the DOClass that it implements, Station

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

BEGIN_NAMESPACE_NEX
class Message;
class EndPoint;
class StationOperation;
class StationDetectionAlgorithm;
END_NAMESPACE_NEX
#include <NetZ/src/Core/DOCore/DOClass.h>
#include <NetZ/src/Core/DOCore/StationDDL.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <NetZ/src/Core/DOCore/UpdateContextMap.h>
#include <NetZ/src/Core/DOCore/MessageBundle.h>
#include <NetZ/src/Core/DOCore/DistanceComputationCache.h>
#include <OnlineCore/src/Transport/Interface/EndPointInfoInterface.h>
#include <OnlineCore/src/Transport/Interface/EndPointInfo.h>


#define NEX_STATION_PLAYER_NAME_LEN 16  //!< Station::SetLocalPlayerName() で指定できるプレーヤー名の最大文字列長です。

/*!
 * @brief ステーションの現在の状態を示す列挙体です。
 */
BEGIN_NAMESPACE_NEX

namespace StationEvent
{
typedef enum
{
    None,
    Participated,
    Left
} Type;
}

//n2788:
/*!
 @internal
 @brief ステーションの状態に変化があったことを通知する関数ポインタ型の定義です。

これは、@ref Session::RegisterStationEventCallback 関数に設定されます。

@li hStation ： 状態が変化したステーションのDOHandle
@li oEvent ： 変化後のステーションの状態

StationEvent::Participated よりも前にそのStationからのパケットが届くことがある問題があるので、注意してください。
*/
typedef void (*StationEventCallback)(DOHandle hStation, StationEvent::Type oEvent);

enum
{
    USER_STATION_MESSAGE=0
};

//Group=Miscellaneous Types and Macros

/*
Description: The round trip time for a message is unknown.
Remarks: The round trip time (RTT) for a message between two stations is unknown if the stations
are not connected directly.
See Also: Station::GetRTT, Station::GetRTTAverage
*/


class Observer;
class PRUDPEndPoint;

//Group=Session Management

// This macro is never defined.  It is used only to get around a compilation problem with
// DocJet, which does not like the inheritance to the DOCLASS(Station).
#ifdef FOR_DOC_PURPOSE
#include <NetZ/src/Core/DOCore/SelectionIteratorTemplate.h>
/*
Description: Retrieves specific information about a Station.
Remarks: This class is used to retrieve specific information about a Station and to set
the observer used when distance-based dead reckoning is invoked.
*/

/*!
 * @brief ステーション(セッションに参加しているマシン)の情報をあらわすクラスです。
 *
 * このクラスを経由してセッションに参加しているそれぞれのマシンを管理することが出来ます。
 * Stationクラス自体も複製オブジェクトとして管理されています。
 */
class Station : DuplicatedObject
{
public:
    /*!
     * @brief Stationオブジェクトを列挙するためのイテレータです。
     */
    typedef SelectionIteratorTemplate<Station> SelectionIterator;
#else
class Station: public DOCLASS(Station)
{
#endif
public:
    //n1589:moved static methods to here
    /*!
     * @brief
       無効なRTTを表します。
     */
    static const TimeInterval STATION_UNKNOWN_RTT=-1;

    /*
    Description: The input or output bandwidth for this Station is unknown.
    Remarks: The input and output bandwidth for a station's connection must be defined
    via Station::SetBandwidthInfo.
    See Also: Station::SetBandwidthInfo, Station::GetInputBandwidth, Station::GetOutputBandwidth
    */
    static const qUnsignedInt STATION_UNKNOWN_BANDWIDTH=0;
    /*
    Description: The input or output latency for this Station is unknown.
    Remarks: The input and output latency due to a station's connection must be defined
    via Station::SetLatencyInfo.
    See Also: Station::SetLatencyInfo, Station::GetInputLatency, Station::GetOutputLatency
    */
    static const qUnsignedInt STATION_UNKNOWN_LATENCY =0;

    // {secret}
    static void SetLocalStation(DOHandle hStation);

    //Group=Methods

    //Description: Returns the DOHandle of the local station.
    //Returns: The DOHandle of the local station.
    /*!
     * @brief 現在実行しているマシンを表すステーションの複製オブジェクトハンドルを取得します。
     * @return 現在実行しているマシンを表すステーションの複製オブジェクトハンドルが返ります。
     * @retval INVALID_DOHANDLE セッションに参加していないなどの理由で、実行しているマシンのステーションが存在しないときなどに返ります。
     */
    // n2967 高速化のためStation.cppから移動してinline化
    inline static DOHandle GetLocalStation()
    {
        return s_hLocalStation.GetValue(PseudoSingleton::GetCurrentContext());
    }

    //Description: Returns a pointer to the instance of the local
    //Station.
    //Returns: A pointer to the local station object, NULL if the
    //object does not exist.
    /*!
     * @brief 現在実行しているマシンを表すステーションの複製オブジェクトへのポインタを取得します。
     * @return 現在実行しているマシンを表すステーションの複製オブジェクトへのポインタが返ります。
     * @retval NULL セッションに参加していないなどの理由で、実行しているマシンのステーションが存在しないときなどに返ります。
     */
    // n2967 高速化のためStation.cppから移動してinline化
    inline static Station* GetLocalInstance()
    {
        // キャッシュから取り出す。
        Station* pStation = s_hLocalStationCache.GetValue(PseudoSingleton::GetCurrentContext());
        if (!pStation)
        {
            // キャッシュが無い場合はDOHandleから取り出す。
            Station::Ref refStation(GetLocalStation());
            pStation = *refStation;

            // キャッシュを更新する。
            s_hLocalStationCache.SetValue(PseudoSingleton::GetCurrentContext(), pStation);
        }
        return pStation;
    }

    static void GetStationHandles(Role rSelectionType, qVector<DOHandle> &handles);
    static void GetStationHandles(qVector<DOHandle> &handles);

    /*!
     * @brief StationIDから Station の DOHandle に変換します。
     * @param[in] stationId  変換したい StationID を指定します。
     * @return 変換した結果の DOHandle が返ります。
     */
    static DOHandle ConvertStationIDToDOHandle(StationID stationId);
    /*!
     * @brief StationIDから Station の DOHandle に変換します。 本関数の使用は非推奨です。@ref ConvertStationIDToDOHandle() を使用してください。
     * @param[in] stationId  変換したい StationID を指定します。
     * @return 変換した結果の DOHandle が返ります。
     * @see ConvertStationIDToDOHandle
     */
    static DOHandle ConvertConnectionIDToDOHandle(StationID stationId);

    // {secret}
    static DOHandle ConvertIDToDOHandle(StationID idStation);
    // {secret}
    static DOHandle ConvertDOIDToDOHandle(DOID idStation);

    /*!
     * @brief 任意の Station の DOHandle を指定して、 P2P セッション上でユニークな ID（StationID）に変換します。 本関数の使用は非推奨です。 @ref ConvertDOHandleToStationID() を使用してください。
     * @param[in] hStation  変換したい DOHandle を指定します。
     * @return 変換した結果の StationID が返ります。
     * @see ConvertDOHandleToStationID
     */
    static StationID ConvertDOHandleToID(DOHandle hStation);

    /*!
     * @brief 任意の Station の DOHandle を指定して、 P2P セッション上でユニークな ID（StationID）に変換します。
     * @param[in] hStation  変換したい DOHandle を指定します。
     * @return 変換した結果の StationID が返ります。
     */
    static StationID ConvertDOHandleToStationID(DOHandle hStation);

    /*!
     * @brief 任意の Station の DOHandle を指定して、 DOIDに変換します。
     * @param[in] hStation  変換したい DOHandle を指定します。
     * @return 変換した結果の DOID が返ります。
     */
    static DOID ConvertDOHandleToDOID(DOHandle hStation);

    /*
    Description: Sets the process details for this Station.
    Parameters:
    uiType: The ProcessType of this Station, which may be one of UnknownProcess, ClientProcess, ServerProcess, or ToolProcess.
    strProcessName: The name of this process. This value is user-defined.
    uiProductVersion: The product version number of this process. This value is user-defined.
    Remarks: This method is used to define the type of process that a Station is and to define its name and version number.
    This information may be used to restrict the activities that given types of processes can perform. For example, $PRODUCTNAME$
    does not permit ToolProcesses to become a session master.

    Note that this method should only be called after the $PRODUCTNAME$ object has been created and before a Session is created or joined.
    See Also: ProcessType, GetProcessType, GetProcessName, GetProductVersion
    */
    static void SetLocalProcessInfo(qUnsignedInt32 uiType, const String& strProcessName, qUnsignedInt32 uiProductVersion=0);

    /*
    Description: Sets the identification token details for this Station.
    Parameters:
    strIdentificationToken: A user-defined string that is associated with this Station.
    Remarks: An identification token is a user-defined string that is associated with a given Station. One way
    to use an identification token is to use it to pass a password or other identifying tag when a Station attempts to
    join a Session in order to determine whether or not the Station has the right to join the Session. In this case the
    information would be passed to the Connection point Station so that that Station can make a decision about the join
    request. The identification token is set on the joining Station object and is then accessable via GetIdentificationToken.
    Refer to Session::RegisterJoinApprovalCallback for an example of how an identification token could be used.

    Note that this method should only be called after the $PRODUCTNAME$ object has been created and before a Session is created or joined.
    See Also: GetIdentificationToken
    */

    /*!
     * @brief   ローカルステーションの識別トークンを設定します。
     * この関数で設定した値は他のステーションから @ref GetIdentificationToken で参照する事ができます。
     * この関数は Net-Z インスタンスを作成した後に呼び出してください。
     * @param   strIdentificationToken  ステーションに紐づいたユーザー定義の文字列
     */
    static void SetLocalIdentificationToken(const String& strIdentificationToken);

    /*!
     * @brief   ローカルステーションのプレーヤー名と言語コードを設定します。
     *
     * ローカルステーションのプレーヤー名と言語コードを設定し、自動的にクライアントの プリンシパル ID を設定します。
     *
     * この関数で設定した値は他のステーションから @ref GetPlayerName() , @ref GetLanguageCode() , @ref GetPrincipalID() で参照することが出来ます。
     * 名前の長さは16文字(NULL終端を含めない)以内である必要があります。
     *
     * この関数は Net-Z インスタンスを作成した後に呼び出してください。
     * P2P セッション毎(NetZ オブジェクトを作り直す毎に)に本関数を用いてプレーヤー名を指定する必要があります。
     *
     * 
     * languageCode については、アプリケーションで別リージョンの言語を判定する場合に指定してください。本体の言語設定を使用したい場合には、
     * nn::cfg::CTR::GetLanguage() の値が利用可能です。
     * 
     * 
     *
     * @param    playerName    他の P2P セッション参加者から参照される名前
     * @param    languageCode    名前の言語コード
     */
    static void SetLocalPlayerName(const String& playerName, qUnsignedInt8 languageCode);

    /*!
     * @brief ローカルステーションのアプリケーションバージョンを設定します。
     * アプリケーション独自の任意の数値を設定できます。
     * この関数で設定した値は他のステーションから @ref GetApplicationVersion で参照する事ができます。
     * この関数は Net-Z インスタンスを作成した後に呼び出してください。
     * @param   applicationVersion  アプリケーションバージョンを示す数値
     */
    static void SetLocalApplicationVersion(qUnsignedInt32 applicationVersion);


    /*
    Description: Returns a DOHandle to the observer on the local station.
    Returns: A DOHandle to the object on the local station that is the registered observer, or
    INVALID_DOHANDLE if there is no registered observer.
    See Also: RegisterObserver, UnregisterObserver, GetObserver, Dead Reckoning Overview
    */

    /*!
     * @brief   ローカルステーションのオブザーバーを取得します。推測航法で利用します。
     *
     * ローカルステーションに対して Station::GetObserver() で得られるものと同等です。
     *
     * 推測航法についてはプログラミングマニュアルを参照してください。
     *
     * @see Station::GetObserver, Station::RegisterObserver
     */
    static DOHandle GetLocalObserver();

    /*
    Description: Registers the specified object as the observer on this Station.
    Parameters:
    idObserver: The DOHandle of the object that you want to register as the observer on this Station.
    Returns: true if the observer was successfully registered, false if not or an error occurred.
    The error SYSTEMERROR_GEN_INVALID_OPERATION is signalled if an observer is already registered.
    Remarks: Either a Duplication master or a Duplica may be selected as the observer for dead reckoning
    calculations. Refer to the Dead Reckoning Overview for details as to the calculations performed.
    If no observer is registered and distance-based dead reckoning is invoked, then the default
    distance defined by ErrorToleranceFunction::SetDefaultDistance is used to calculate the error tolerance
    used in the dead reckoning algorithm.

    As an example, the following code creates two instances of spheres and defines the user-controlled sphere on
    the local station as the observer for that station:

           // Create and publish UserSphere
           Sphere *pUserSphere = Sphere::Create();
           pUserSphere-&gt;Publish();
           // Create and publish AISphere
           Sphere* pAISphere = Sphere::Create();
           pAISphere-&gt;Publish();
           // Register UserSphere as the observer on the local station
           Station::RegisterObserver(pUserSphere-&gt;GetHandle());

    Although only one object may be registered as an observer for a given Station at any time, this
    does not restrict how many individual observer avatars can be set on a Station. To
    implement multiple observers you simply register an object that contains a list of observer
    avatars. For example, say that you have a split-screen mode with your game. Then when in split-screen
    mode, rather than registering a player's avatar as the observer you would register a "SplitScreen"
    object, which contains an "Observers" dataset that is a list of all the avatars
    that you wish to be considered as observers. Then in the ComputeDistance callback for the SplitScreen
    object you would calculate the distance between a specific object and each of the observers, returning
    the minimum value. This ensures that as long as one of the observers is close enough to the object
    for it to require an update, an update will be sent.

    Note: As the distance between objects is calculated on a remote Station to that where the observer
    object is registered, the idObserver object must be published before it can be registered.
       See Also: GetObserver, UnregisterObserver, ErrorToleranceFunction::SetDefaultDistance,
       Dead Reckoning, Dead Reckoning Overview
           */

    /*!
     * @brief   複製オブジェクトをステーションのオブザーバに登録します。推測航法で利用します。
     *
     * DuplicatedObject::ComputeDistance() の引数として渡される複製オブジェクトを登録できます。
     *
     * 推測航法についてはプログラミングマニュアルを参照してください。
     *
     * @param[in]   idObserver  オブザーバーとして登録する複製オブジェクトの DOHandle を指定します。
     *
     * @see Station::UnregisterObserver, DuplicatedObject::ComputeDistance
     */
    static qBool RegisterObserver(DOHandle idObserver);

    /*
    Description: Unregisters the current observer.
    Returns: true if the observer was successfully unregistered, false if not or an error occurred.
    The error SYSTEMERROR_GEN_INVALID_OPERATION is signalled if no observer is currently registered.
    See Also: GetObserver, RegisterObserver, Dead Reckoning Overview
    */

    /*!
     * @brief  ステーションのオブザーバ登録を取り消します。推測航法で利用します。
     *
     * 推測航法についてはプログラミングマニュアルを参照してください。
     *
     * @see Station::RegisterObserver
     */
    static qBool UnregisterObserver();

    /*
    Description: Sends all messages stored in all message bundles.
    Remarks: When this method is called all messages in all message bundles on the local station will be sent. Once
    all bundles are sent all messages at the transport level are then also flushed.
    See Also: FlushBundle, BundlingPolicy
    */

    /*!
     * @brief   すべてのメッセージバンドルに登録されているメッセージを送信します。
     *
     * メッセージバンドルとは、送信されるメッセージを一時的にプールして送信を待機させ、
     * ある程度まとまった間隔でメッセージを送信する機能のことです。
     * これによって帯域幅を節約することが出来ます。
     * 詳しくはNEXプログラミングマニュアルをご覧ください。
     *
     * メッセージバンドル機能が有効である場合、送信されようとしたメッセージは一時的にプールされ、
     * 即座に送信されることはありません。この関数を呼びだしたときに、プールされたすべてのメッセージを
     * フラッシュし、すべてのステーションに対するすべてのメッセージを送信します。
     *
     * @see     FlushBundle, BundlingPolicy
     */
    static void FlushAllBundles(qBool bFlushTransportBundles = true);

    // FlushAllBundles()のSystemLockなし版。BundlingPolicy::FlushJob::Execute()向け。
    static void FlushAllBundlesImpl(qBool bFlushTransportBundles);

    // {secret}
    static qBool SignalCompleteFault(qBool bProcessNow = true);
    // {secret}
    static void InitiateFaultProcessingForStation(DOHandle hFaultyStation, qUnsignedInt32 uiFault);
    // {secret}
    static size_t GetNbLocalStations();

    static size_t GetNbDuplicaStations();

    /*
    Remarks: A Station object should never be directly instantiated. Instead, it is created by $PRODUCTNAME$
    when a station joins the session.
    */

    /*!
     * @brief このコンストラクタはアプリケーション側から直接呼び出さないでください。
     *        セッションに参加したタイミングに、Net-Zによって自動的に作成されます。
     */
    Station();

    /*
    Remarks: A Station object should never be directly destroyed. It is destroyed by $PRODUCTNAME$ when
    the associated station leaves the session.
    */

    /*!
     * @brief このデストラクタはアプリケーション側から直接呼び出さないでください。
     *        セッションを離脱したときなどにNet-Zによって自動的に破棄されます。
     */
    virtual ~Station();

    //Group=Type Definitions

    //Description: Enumerates various Station states.
    //See Also: State
    /*!
     * @brief ステーションの現在の状態を示す列挙体です。
     */
    enum State
    {
        // The State of the Station is unknown.
        //! 現在の状態が不明であることを示します。
        Unknown = 0,

        // The Station is in the process of joining the session.
        //! ステーションがセッションに対する参加中処理を実行中であることを示します。
        JoiningSession = 1,

        // The Station is in the process of creating the session.
        //! ステーションが現在セッションを作成している最中であることを示します。
        CreatingSession = 2,

        // The Station is currently participating in the session.
        //! ステーションがセッションに参加している状態であることを示します。
        Participating = 3,

        // The Station is in the process of leaving the session.
        //! ステーションがセッションに対する離脱処理を実行中であることを示します。
        Leaving = 4,

        // The Station is in the process of leaving the session due to a fault.
        //! ステーションに(タイムアウトなどの)障害が発生したために、セッションからの離脱処理を実行中であることを示します。
        LeavingOnFault = 5
    };

    // {last three bits are reserved}

    // Description: Enumerates the process types that a Station may be.
    /*!
     * @brief このステーションのプロセスの種類を表す列挙体です。
     */
    enum ProcessType
    {
        // The Station process type is unknown. This is the default process type.
        //! このステーションのプロセスは不明です。これはデフォルトのプロセス種類になります。
        UnknownProcess  =0,

        // The Station is a client process.
        //! このステーションはクライアントプロセスです。
        ClientProcess   =1,

        // The Station is a server process.
        //! このステーションはサーバープロセスです。
        ServerProcess   =2,

        // The Station is a tool process. An example of a tool process is the $PRODUCTNAME$ Object
        // Monitor tool. A tool process cannot become a session master.
        //! オブジェクトモニタ(セッションに存在する複製オブジェクトの監視ツール)などに代表される、付属ツールのプロセスです。ツールプロセスの場合、セッションのマスターになることは出来ません。
        ToolProcess     =4
    };

    // {secret}
    enum ConnectionState
    {
        Disconnected,
        Connecting,
        Connected,
        Disconnecting
    };

    //Group=Methods

    /*
    Description: Returns the current State of the station.
    Returns: The current State of the station, which is one of: Unknown, JoiningSession,
    CreatingSession, Participating, Leaving, or LeavingOnFault.
    See Also: State
    */

    /*!
     * @brief ステーションの現在のステータスを取得します。
     * @return ステーションの現在のステータスが返ります。 @ref State 列挙体で定義される値のうちのいずれかになります。
     */
    State GetState() const
    {
        return (State) m_dsStationState.Get();
    }

    // {secret}
    void SetState(State uNewState);

    /*
    Description: Returns whether or not a Station is a peer.
    Returns: true if the Station is a peer station, false if it is the local station.

    $ifdef NETZ$
    Remarks: In Net-Z, all remote stations in the Session are peer stations to a given Station.
    $endif$

    $ifdef ETERNA$
    Remarks: The peers of a given station depend on the implementation of the distributed
    broadcast. If a distributed broadcast is implemented the peers of a given station will
    consist of: the stations where the duplication masters of the duplicas on the local
    station reside, the stations where the duplicas of the duplication masters of the
    local station reside, and the station where the Duplication master of the
    RootMulticastGroup object resides. In addition, if the local station contains a
    Duplication master of a MulticastGroup, its peers will also include the stations of
    all of the MulticastGroup members. In contrast, if a distributed broadcast is
    disabled, the peers will consist of all the stations in the session.
    $endif$

    See Also: IsLocal
    */

    /*!
     * @brief   このステーションがセッションに参加しているリモート接続相手であるかどうかを取得します。
     *
     * @ref IsLocal で返る値の逆が返ります。
     *
     * @retval true セッションに参加しているリモート接続相手であるときに返ります。
     * @retval false セッションに参加しているリモート接続相手でないときに返ります。
     */
    qBool IsAPeer()const;

    /*
    Description: Returns whether or not a Station is the local station.
    Returns: true if the Station is the local station, false if it is a peer station.
    Remarks: The local station is typically the Duplication master of the Station
    but this is not always the case.
    See Also: IsAPeer
    */

    /*!
     * @brief   ステーションのインスタンスが、このマシン自身を表すかどうかを取得します。
     *
     * @ref IsAPeer で返る値の逆が返ります。
     *
     * @retval true このステーションはこのマシン自身であるときに返ります。
     * @retval false このステーションはこのマシンでないときに返ります。
     */
    qBool IsLocal()const;

    qBool IsNotLeavingOrLeft() const;

    // {secret}
    void ReleaseOwnReference();
    // {secret}
    void AcquireOwnReference();
    // {secret}
    qBool OwnsReferenceToItself();

    /*
    Description: Returns the type of this process.
    Returns: The type of this process as a qUnsignedInt32. The ProcessType may be one of UnknownProcess,
    ClientProcess, ServerProcess, or ToolProcess and is set via SetLocalProcessInfo.
    Remarks: Refer to DataSet::UpdateIsRequired for an example of this method in use.
    See Also: ProcessType, SetLocalProcessInfo, GetProcessName, GetProductVersion
    */

    /*!
     * @brief このステーションのプロセス種別を取得します。
     * @return このステーションを制御しているプロセスの種類が返ります。 @ref ProcessType 列挙体の値のうちいずれかが返ります。
     */
    qUnsignedInt32 GetProcessType() const;

    /*
    Description: Returns the name of this process.
    Returns: The name of this process as a String. The ProcessName is user-defined and is set via
    SetLocalProcessInfo.
    See Also: ProcessType, SetLocalProcessInfo, GetProcessType, GetProductVersion
    */
    const String& GetProcessName() const;

    /*
    Description: Returns the product version number of this process.
    Returns: The product version number of this process as a qUnsignedInt32. The ProductVersion is user-defined
    and is set via SetLocalProcessInfo.
    See Also: ProcessType, SetLocalProcessInfo, GetProcessType, GetProcessName
    */
    qUnsignedInt32 GetProductVersion() const;

    /*
    Description: Returns the identification token of this process.
    Returns: The identification token of this process. An identification token is a user-defined string that is
    associated to a given Station.
    Remarks: Refer to Session::RegisterJoinApprovalCallback for an example of this method in use.
    See Also: SetLocalIdentificationToken
    */

    /*!
     * @brief このステーションの識別トークンを取得します。
     * @return このステーションの識別トークンが返ります。
     */
    const String& GetIdentificationToken() const;

    /*!
     * @brief このステーションのプレーヤー名を取得します。
     * @return このステーションのプレーヤー名が返ります。
     */
    const String& GetPlayerName() const;

    /*!
     * @brief このステーションのプレーヤー名の言語コードを取得します。
     * @return このステーションのプレーヤー名の言語コードが返ります。
     */
    qUnsignedInt8 GetLanguageCode() const;

#ifndef NEX_USE_PRINCIPALID_64BIT
    /*!
     * @brief このステーションのプリンシパル ID を取得します。
     * @return このステーションのプリンシパル ID が返ります。
     */
    PrincipalID GetPrincipalID() const;
#endif

    /*!
     * @brief このステーションのアプリケーションバージョンを取得します。
     * @return このステーションのアプリケーションバージョンが返ります。
     */
    qUnsignedInt32 GetApplicationVersion() const;

    // {secret}
    qResult Send(Message* pMsg, qUnsignedInt32 uiFlags);

    //n1589
    void SetLinkObject(void* object)
    {
        m_pLinkObject = object;
    }

    void* GetLinkObject()
    {
        return m_pLinkObject;
    }

    // {secret}
    virtual qBool ApproveEmigration(qUnsignedInt32 uiReason);
    // {secret}
    virtual qBool ApproveFaultRecovery();

    // {secret}
    virtual qBool ValidOperation(DOOperation* pOperation);

    // n2370
    // {secret}
    virtual void OperationBegin(DOOperation* pOperation);

    // {secret}
    virtual void OperationEnd(DOOperation* pOperation);
    // {secret}
    void ReleaseSystemReferences();

    // {secret}
    void LeaveSessionNormally();
    // {secret}
    void LeaveSessionOnFault();

    //Group=Connection Information

    /*
    Description: Returns whether or not this Station is connected to the target Station.
    Returns: true if this Station is connected to the target Station, false if not or an error occurred.
    */

    /*!
     * @brief 実行しているマシンとこのステーションが接続を持っているかどうかを取得します。
     * @retval true 実行しているマシンはこのステーションとの接続を持っています。
     * @retval false 接続を持っていない、あるいは何かしらエラーが発生したときに返ります。
     */
    qBool IsConnected() const;

    //n1589
    qBool IsConnectedWithoutLock() const;

    // {secret}
    qBool IsFaulty() const;

    // {secret}
    qBool DiscoversGlobalObject(DuplicatedObject* pDO);

    // {secret}
    // Called in ProcessJoinRequest
    void SetConnection(EndPoint* pEndPoint);

    // {secret}
    UpdateContextMap& UpdateContextes() const
    {
        return m_mapUpdateContext;
    }

    /*
    Description: Returns the currently registered observer on this Station.
    Parameters:
    Returns: A DOHandle to the object on the given station that is the registered observer, or
    INVALID_DOHANDLE if there is no registered observer.
    Remarks: If an observer, which is used in dead reckoning calculations, is registered then
    this method returns which object is the observer.
    See Also: RegisterObserver, UnregisterObserver, GetLocalObserver, Dead Reckoning Overview
    */

    /*!
     * @brief   ステーションのオブザーバーを取得します。推測航法で利用します。
     *
     * 推測航法についてはプログラミングマニュアルを参照してください。
     *
     * @see Station::RegisterObserver
     */
    DOHandle GetObserver()const ;

    //{secret}
    qReal ComputeDistanceTo(DOHandle hDO);

    //Group=Connection Information

    /*
    Description: Returns the current round trip time for a message between this Station and the local station.
    Returns: The current round trip time (RTT) in milliseconds or STATION_UNKNOWN_RTT if this Station
    is not directly connected to the local station. The default initial value is 250 ms.
    Remarks: The round trip time is the time taken for a message to reach another Station
    and the acknowledgement to be received by the sender. The RTT can only be calculated
    if the two stations are directly connected to each other. The maximum time between messages that
    are used to calculate the RTT is defined by the PRUDP_KEEPALIVETIMEOUT.
    See Also: GetRTTAverage
    */

    /*!
     * @brief   ローカルステーションと相手ステーションとの間のRTTの現在値を取得します。
     *
     * ローカルステーションから、相手ステーションに向けてメッセージを送り、
     * 相手からそのメッセージに対する確認が帰ってくるまでの時間のことを
     * RTT(Round Trip Time)と呼びます。この機能は、相手のステーションまで直接接続されているときのみ利用できます。
     *
     * @return  相手ステーションまでのRTTの現在の値が返ります。
     * @retval  STATION_UNKNOWN_RTT 接続が直接行われていないときに返ります。
     * @see     GetRTTAverage
     */
    TimeInterval GetRTT();

    /*
    Description: Returns the average round trip time for a message between this Station and the local station.
    Returns: The average round trip time (RTT) in milliseconds or STATION_UNKNOWN_RTT if this Station is not
    directly connected to the local station. The default initial value is 250 ms.
    Remarks: The average round trip time is the average time taken for a message to reach another
    station and the acknowledgement to be received by the sender. The RTT can only be calculated
    if two stations are directly connected to each other. The maximum time between messages that are
    used to calculate the RTT is defined by the PRUDP_KEEPALIVETIMEOUT.

    The average round trip time is calculated using smoothed averages based on V. Jacobson's well-known algorithms.
    See Also: GetRTT
    */

    /*!
     * @brief   ローカルステーションと相手ステーションとの間のRTTの平均値を取得します。
     *
     * ローカルステーションから、相手ステーションに向けてメッセージを送り、
     * 相手からそのメッセージに対する確認が帰ってくるまでの時間のことを
     * RTT(Round Trip Time)と呼びます。この機能は、相手のステーションまで直接接続されているときのみ利用できます。
     *
     * @return  相手ステーションまでのRTTの平均の値が返ります。
     * @retval  STATION_UNKNOWN_RTT 接続が直接行われていないときに返ります。
     * @see     GetRTT
     */
    TimeInterval GetRTTAverage();

    /*
    Description: Sets the input and output latency induced by this station's connection.
    Parameters:
    tiInputLatency: The input latency in milliseconds of this station's connection.
    tiOutputLatency: The output latency in milliseconds of this station's connection.
    Returns: true if the station's latency was successfully set, false if not or an error occurred.
    The error SYSTEMERROR_GEN_INVALID_OPERATION is signalled if this method is called on a Duplica.
    Remarks: SetLatencyInfo can only be called on a Duplication master. The method is used to
    account for asymmetric connection speeds between stations. $PRODUCTNAME$ will use this
    user-supplied information to improve time synchronization between stations. Refer to
    SessionClock::AdjustForStationLatency for more information on how the input and output
    latency affects time synchronization.
    See Also: GetInputLatency, GetOutputLatency, SessionClock::AdjustForStationLatency
    */
    qBool SetLatencyInfo(TimeInterval tiInputLatency, TimeInterval tiOutputLatency);

    /*
    Description: Returns the input latency of this station's connection.
    Returns: The input latency in milliseconds or STATION_UNKNOWN_LATENCY if there is no
    information known about the station's connection.
    Remarks: The input latency of this station's connection is set via SetLatencyInfo.
    See Also: SetLatencyInfo, GetOutputLatency
    */
    TimeInterval GetInputLatency();

    /*
    Description: Returns the output latency of this station's connection.
    Returns: The output latency in milliseconds or STATION_UNKNOWN_LATENCY if there is no
    information known about the station's connection.
    Remarks: The output latency of this station's connection is set via SetLatencyInfo.
    See Also: SetLatencyInfo, GetInputLatency
    */
    TimeInterval GetOutputLatency();

    /*
    Description: Sets the input and output bandwidth of this station's connection.
    Parameters:
    uiInputBandwidth: The input bandwidth in bits per second of this station's connection.
    uiOutputBandwidth: The output bandwidth in bits per second of this station's connection.
    Returns: true if the station's bandwidth was successfully set, false if not or an error occurred.
    Remarks: This method can only be called on a Duplication master. SYSTEMERROR_GEN_INVALID_OPERATION
    is signalled if it is called on a Duplica.
    See Also: GetInputBandwidth, GetOutputBandwidth
    */
    qBool SetBandwidthInfo(qUnsignedInt uiInputBandwidth, qUnsignedInt uiOutputBandwidth);

    /*
    Description: Returns the input bandwidth of this station's connection.
    Returns: The input bandwidth in bits per second or STATION_UNKNOWN_BANDWIDTH if there is no
    information known about the station's connection.
    Remarks: The input bandwidth of this station's connection is set via SetBandwidthInfo.
    See Also: SetBandwidthInfo, GetOutputBandwidth
    */
    TimeInterval GetInputBandwidth();

    /*
    Description: Returns the output bandwidth of this Station.
    Returns: The output bandwidth in bits per second or STATION_UNKNOWN_BANDWIDTH if there is no
    information known about the station's connection.
    Remarks: The output bandwidth of this station's connection is set via SetBandwidthInfo.
    See Also: SetBandwidthInfo, GetInputBandwidth
    */
    TimeInterval GetOutputBandwidth();

    // {secret}
    void SetAtEOS();
    // {secret}
    void ClearAtEOS();
    // {secret}
    qBool AtEOS();

    // {secret}
    qUnsignedInt32 GetMachineUniqueID() const;

    // {secret}
    qBool TestAndSetFaultFlag();
    // {secret}
    qBool Faulty();

#ifdef NEX_TRACE_ENABLED    // n2370
    // {secret}
    virtual void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const;

    //n1589
    void TraceID(qUnsignedInt64 uiTraceFlags) const;

#endif  // NEX_TRACE_ENABLED

    // {secret}
    void InitLocalStationInfo();

    /*
     * @internal
     * @brief   ステーションの @ref PRUDPEndPoint を取得します。
     *
     * ステーションが保持している @ref PRUDPEndPoint のポインタを取得します。
     *
     * @return  ステーションの @ref PRUDPEndPoint へのポインタ
     *
     * @see     PRUDPEndPoint, Scheduler::SystemLock
     */
    PRUDPEndPoint* GetPRUDPEndPoint()const
    {
        return reinterpret_cast<PRUDPEndPoint*>(GetEndPoint());
    }

    EndPoint* GetEndPoint()const;


    /*!
     * @brief   ステーションの @ref EndPointInfoInterface を取得します。
     *
     * ステーションが保持している @ref EndPointInfoInterface のポインタを取得します。
     * EndPointInfoInterface を介して、 EndPointの統計情報である EndPointInfo の取得・操作が可能です。
     *
     * EndPointInfoInterface の取得の際には、  Scheduler::SystemLock()が必要です。
     *
     * @return  ステーションの @ref EndPointInfoInterface へのポインタ。
     *          ローカルステーションの場合などEndPointが存在しない場合には、NULL。
     *
     * @see     EndPointInfoInterface, EndPointInfo, Scheduler::SystemLock()
     */
    EndPointInfoInterface* GetEndPointInfoInterface() const;


    // {secret}
    MessageBundle *GetSharedMessageBundle()
    {
        return &m_oSharedMessageBundle;
    }
    // {secret}
    MessageBundle *GetUnreliableMessageBundle()
    {
        return &m_oUnreliableMessageBundle;
    }

#if NEX_ENABLE_HYBRID
    static void SessionMasterDisconnected();
#endif

    //Group=Methods

    /*
    Description: Returns a list of valid URLs for this Station.
    Parameters:
    plstURLs: A pointer to the list of valid StationURL s for this Station.
    Returns: A list of valid StationURL s for this Station.
    */
    /**
     * @brief   このステーションの、有効な StationURL のリストを取得します。
     *
     * ステーションのURLは、外回線に出るときのグローバルなものや、
     * LAN内などの閉じた環境でのURLなど、複数存在します。
     * 本関数はそれを一度に複数取得できます。
     *
     * 先頭要素は基本的に外回線にでるときのグローバルなURLが格納されています。
     *
     * 1つのURLだけ取得したい場合は @ref GetStationURL の使用を推奨します。
     *
     * @param[out] plstURLs  このステーションの有効なURLのリストを格納するためのqList型のポインタを指定します。
     * @see GetStationURL
     */
    void GetStationURLs(qList<StationURL> *plstURLs)const;

    /*!
     * @brief このステーションのURLを文字列にしたものを取得します。
     *
     * ステーションのURLは、外回線に出るときのグローバルなものや、
     * LAN内などの閉じた環境でのURLなど、複数存在します。
     * 本関数はそれを1つだけ取得できます。
     *
     * 先頭要素は基本的に外回線にでるときのグローバルなURLが格納されています。
     * つまり、 GetStationURL(0) でグローバルなURLの文字列を取得することができます。
     *
     * 1～4以外の引数を指定した場合は常に空文字列が返ります。
     *
     * 一度に複数のURLを取得したい場合は @ref GetStationURLs を使用してください。
     *
     * @param[in]   iIndex  取得したいURLリストのインデックス
     * @return  URLを表す文字列。
     * @see     GetStationURLs
     */
    const qChar* GetStationURL(qInt iIndex)const;

    /*
    Description: Sends all messages stored in a given message bundle.
    Parameters:
    bFlushTransportBundles: If true (the default), the transport message bundles will be flushed
    once the message bundle is flushed.
    Remarks: When this method is called all messages in the message bundle on the local station associated
    with this Station will be sent. If bFlushTransportBundles is set to true, transport bundles will be sent
    after the message bundle has been sent. Note that transport bundles need to be flushed periodically so if
    you call this method you must set the bFlushTransportBundles parameter to true at regular intervals.
    See Also: FlushAllBundles, BundlingPolicy
    */
    /**
     * @brief   現在このステーションにメッセージバンドルとしてためられているをメッセージをすべて送信します。
     *
     * メッセージバンドルとは、送信されるメッセージを一時的にプールして送信を待機させ、
     * ある程度まとまった間隔でメッセージを送信する機能のことです。
     * これによって帯域幅を節約することが出来ます。
     * 詳しくはNEXプログラミングマニュアルをご覧ください。
     *
     * 伝送メッセージバンドルは定期的にフラッシュされる必要がありますので、
     * 通常の間隔でこの関数を呼び出すときは @a bFlushTransportBundles をtrueにしなければなりません。
     *
     * @param[in]   bFlushTransportBundles  これが true のとき、メッセージバンドルのあとで伝送メッセージバンドルもフラッシュされます(デフォルトは true)。
     * @see     FlushAllBundles, BundlingPolicy
     * @return この関数の実行結果が返されます。
     */
    qResult FlushBundle(qBool bFlushTransportBundles = true);

    /*!
     * @brief 指定されたステーションを強制切断します。デバッグ用機能です。
     *
     * ステーション不一致の検証用に利用してください。
     *
     * @param[in] bProcessNow  true:他のステーションと終了処理なしにステーションを切断します。
     *                         false:他のステーションとの終了処理を行ってステーションを切断します。
     *
     * @retval true 強制切断が成功した場合に返ります。
     * @retval false すでに接続がなかった場合など、強制切断が成功しなかった場合に返ります。
     *
     */
    qBool SignalFault(qBool bProcessNow = true);

    // {secret}
    void SignalAsFaulty(qUnsignedInt32 uiFaultEvent);

    static void SignalAsFaultyAllStations();

    // {secret}
    void InitIdentification(StationIdentification* pIdentification);

    //Description: Returns the ID of the Station.
    //Returns: The StationID of the Station.
    /**
     * @brief セッションの中でステーション(マシン)を特定するためのIDを取得します。
     *        StationIDは、マスター交代後の新規ステーションにアサインされる値が飛ぶ可能性があり、連続値である保証はありません。P2P参加ステーション内でユニークであることは保証されます。
     * @return このステーションのIDが返ります。
     */
    StationID GetStationID() const;

    //n1589
    /**
     * @brief 通信時にステーション（マシン）を指定するためのIDを取得します。 この関数の使用は非推奨です。@ref GetStationID() を使用してください。
     * @return このステーションのStationIDが返ります。
     * @see GetStationID
     */
    StationID GetConnectionID() const;

    //n1589
    qBool ConnectionIsPossible() const;

#if NEX_ENABLE_HYBRID
    //n2781
    qBool HasRoutingConnection(const InetAddress *router = NULL) const;
#endif
#if NEX_ENABLE_HYBRID_DETECT_LIMITATION
    /*!
     * @brief   このステーションが持つマスター経由通信の数を取得します。
     *
     * @ref RootTransport::GetRoutingHistoryTTL() [ms]以内に通信の発生したマスター経由通信の数を返します。
     * 送信、受信をそれぞれカウントします。つまり、このステーションが別の1台のステーションとマスター経由通信していた場合、この関数は2を返します。
     * 本関数を使用する前にはあらかじめ@ref RootTransport::SetRoutingHistoryUpdateSpan で0より大きい更新間隔を設定する必要があります。
     * この関数はセッションマスターのみ呼び出せます。
     * @return  このステーションが持つマスター経由通信の数を返します。
     */
    size_t GetNumOfRoutingConnections() const;
#endif

    /**
     * @brief   指定されたステーションをセッションから離脱させます。
     * 指定されたステーションは、周りのステーションからフォルト扱いで切断されて、  Session::CheckGameIsOver() がtrueとなります。
     *
     * 本メソッドは通常セッションマスター( Session::GetRole()で、 SESSION_ROLE_MASTERが返ってくるステーション)で実行します。
     * また、ローカルステーションを指定することができません。ローカルステーションを離脱させるときには、 KickMe()を用いてください。
     *
     * ロス率が高いときには、指定されたステーションを離脱する信号が正しく到着しない場合があります。
     * その場合は、指定されたステーションは、周りステーションから切断されて孤立します。
     *
     * @retval  true    離脱する信号の送信に成功したときに返ります。
     * @retval  false   キックする相手がいなかった時やローカルステーションを指定した時など、離脱する信号の送信に失敗したときに返ります。
     *
     *
     * @see Session::CheckGameIsOver(), KickMe()
     */
    qBool KickStation();

    /**
     * @brief   ローカルステーションをセッションから離脱させます。
     * ローカルステーションは、周りのステーションからフォルト扱いで切断されて、  Session::CheckGameIsOver() がtrueとなります。
     * スレッドセーフモードのときはスレッドセーフです。
     *
     * @retval  true    離脱する信号の送信に成功したときに返ります。
     * @retval  false   離脱する信号の送信に失敗したときに返ります。
     *
     * @see Session::CheckGameIsOver(), KickStation()
     */
    static qBool KickMe();

    //n2788
    // {secret}
    // static qBool IsFatalReliableSendBufferFull(Station::Ref &refStation);

#if NEX_ENABLE_HYBRID_DETECT_LIMITATION
    void SetRoutingLimit(qBool b);
    qBool GetRoutingLimit()
    {
        return m_bRoutingLimit;
    }
#endif
    qUnsignedInt32 GetNetZVersion() const;

    /**
     * @brief   StationOperationが実行されるたびに呼び出されるコールバックを登録します。
     *
     * @ref StationOperation が実行されるたびに呼び出されるコールバックを登録し、
     * ステーションがセッションに参加した際の開始タイミング、セッションから離脱した際の開始タイミングを知ることができます。
     *
     * @param[in]   pfOperationBegin    コールバック関数へのポインタを指定します。
     * @see     StationOperation, JoinStationOperation, LeaveStationOperation
     */
    static void RegisterOperationBeginCallback(void (*pfOperationBegin)(StationOperation* pOperation));

    /**
     * @brief   StationOperationが実行されるたびに呼び出されるコールバックを登録します。
     *
     * @ref StationOperation が実行されるたびに呼び出されるコールバックを登録し、
     * ステーションがセッションに参加した際の完了タイミング、セッションから離脱した際の完了タイミングを知ることができます。
     *
     * @param[in]   pfOperationEnd    コールバック関数へのポインタを指定します。
     * @see     StationOperation, JoinStationOperation, LeaveStationOperation
     */
    static void RegisterOperationEndCallback(void (*pfOperationEnd)(StationOperation* pOperation));

    static qBool GetAutoAddRecentPlayRecord();

    /**
     * @internal
     * @brief    「いっしょに遊んだ記録」への自動登録を行うかどうかを指定します
     *
     * 自動的に「いっしょに遊んだ記録」への自動登録を行いたくない場合、本関数を使用して自動登録を抑制してください。
     * 自動登録を抑制すると、 @ref Session::CreateSession() , @ref Session::JoinSession() で @ref SetLocalPlayerName() で
     * 名前が登録されているかのチェックを行いません。
     *
     * NEXのデフォルトはtrue(自動登録を行う)です。
     *
     * @param[in]  flag 「いっしょに遊んだ記録」への自動登録を行うかどうか。trueの場合、自動登録を行います。falseの場合、自動登録を行いません。
     */
    static void SetAutoAddRecentPlayRecord(qBool flag);

    /*!
     * @brief セッションへの参加が承認されたステーションの数を返します。
     * セッションの参加の承認・拒否は、@ref Session::RegisterJoinApprovalCallback に登録したコールバックで行うことができます。
     * セッションマスターのステーションは常に承認済みとしてカウントされます。
     *
     * @return セッションへの参加が承認されたステーションの数が返ります。
     *
     * @see Session::RegisterJoinApprovalCallback
     */
    static size_t GetApprovedStationNum();

    /*! @internal
     * @brief セッションへの参加が承認されているかを返します。
     * Publishであれば承認済みとみなします。
     *
     * @return セッションへの参加が承認されていればtrue、されていなければfalseが返ります。
     */
    qBool IsApproved() const
    {
        return IsPublished() || m_isApproved;
    }

    /*! @internal
     * @brief セッションへの参加が承認された状態にします。
     *
     */
    void SetApproved()
    {
        m_isApproved = true;
    }


    //n1589:moved private methods to here
private:
    qResult SendImpl(Message *pMessage, qUnsignedInt32 uiFlags);
    qResult QueueLocalMessage(Message* pMsg);
    qResult SendRemoteMessage(Message* pMsg, qUnsignedInt32 uiFlags);

    qResult FlushBundleImpl(qBool bFlushTransportBundles);

    // {secret}
    EndPoint* ReleaseConnection();

    // {secret}
    qBool RegisterObserverImpl(DOHandle idObserver);

    virtual void TestInvariants();

private:
    friend class JobDisconnectStation;
    friend class JobConnectStation;

    // Local station.
    // {secret}
    static PseudoGlobalVariable<DOHandle> s_hLocalStation;
    static PseudoGlobalVariable<Station*> s_hLocalStationCache;

    // {secret}
    EndPoint* m_poEndPoint;

    // Info used to implement different functionnality of the system.
    // {secret}
    mutable UpdateContextMap m_mapUpdateContext;

    DistanceComputationCache m_oDistanceComputation;

    qBool m_bAtEOS;
    AtomicValue m_bFaulty;

    // This message bundle either for reliable messages (when not
    // mixing reliability) or for both type of messages (when it is
    // mixed)
    MessageBundle  m_oSharedMessageBundle;
    // This message bundle is used for unreliable messages, when we
    // do not mix reliability.
    MessageBundle  m_oUnreliableMessageBundle;

    qUnsignedInt32 m_uiLastConnectionAttempt;

    //n1589
    void* m_pLinkObject;

#if NEX_ENABLE_HYBRID_DETECT_LIMITATION
    qBool m_bRoutingLimit;
#endif
    qBool m_isApproved; // Sessionへの参加が承認されているかどうか
    // 最近遊んだ人リストへの自動追加設定 Cafeでのみ有効に働く
    static qBool s_bAutoAddRecentPlayRecord;

    static void (*s_pfOperationBegin)(StationOperation* pOperation);
    static void (*s_pfOperationEnd)(StationOperation* pOperation);
    StationDetectionAlgorithm* m_pStationDetectionAlgorithm;

public:
    static void CallOperationBeginCallback(StationOperation* pOperation);
    static void CallOperationEndCallback(StationOperation* pOperation);
};

// For the TraceOutput.
// {secret}
extern qUnsignedInt32 GetLocalStationHandle();

END_NAMESPACE_NEX

