﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupImpl.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/Time.h>

BEGIN_NAMESPACE_NEX

class DuplicatedObject;
class Station;

//Group=Miscellaneous Classes

/*
Description: This class is the root class of all station filters.
Remarks: If you declared a custom dataset property that specifies the "updatefilter" property qualifier,
then you need to implement your filter class via either the GlobalUpdateFilter or the StationUpdateFilter
class, which respectively filter dataset updates either globally or on a per station basis. A global
update filter returns whether or not a dataset should be updated whereas, a station update filter returns
whether or not a dataset should be updated on a specific station.

When DuplicatedObject::Update is called, GlobalUpdateFilter::UpdateRequired is called on each
GlobalUpdateFilter. If this method returns false, then the method returns and no update is sent.
If this method returns true, then StationUpdateFilter::UpdateRequired is called on each StationUpdateFilter. If
this method returns true then StationUpdateFilter::UpdateSent is called with the updated Station as a parameter.
Finally, as long as at least one update was sent, GlobalUpdateFilter::UpdateSent is called. Note that when a
Station discovers an object, only the station filters and not the global filters are called.

As an example, if in the DDL we declare the following dataset property:

        dsproperty station_filter: updatefilter;

Then the property could be implemented as follows:

        // This filter updates datasets according to the response returned by
        // the UpdateIsRequired method. This filter is implemented as a template
        // and as such may be used for any dataset.
        template<class DS>
        class StationFilter: public StationUpdateFilter {
            public:
                StationFilter() {};
                ~StationFilter() {};

                virtual qBool UpdateRequired(DuplicatedObject* pDO,
                                            void* pDS, qByte byIndex,
                                            Time tTimestamp,
                                            Station* pTargetStation) {
                    // Some params are dummy - we don't need them here.
                    DUMMY_PARAM(pDO);
                    DUMMY_PARAM(byIndex);
                    DUMMY_PARAM(tTimestamp);

                    return static_cast<DS*>(pDS)->UpdateIsRequired(pDO,pTargetStation);
                }
        };

        // Define the updatefilter macro
        #define _PR_station_filter_filterdecl(DS)   StationFilter< DS >

See Also: GlobalUpdateFilter
*/
class StationUpdateFilter: public RootObject
{
    // group=Methods
public:
    StationUpdateFilter()
    {
        m_uiUpdateLoopNumber=0;
    }
    virtual ~StationUpdateFilter() {}

    // {secret}
    void SetUpdateLoopNumber(qUnsignedInt32 uiUpdateLoopNumber)
    {
        m_uiUpdateLoopNumber=uiUpdateLoopNumber;
    }
    // {secret}
    qUnsignedInt32 GetUpdateLoopNumber() const
    {
        return m_uiUpdateLoopNumber;
    }

    /*
    Description: Returns whether or not the specified dataset is required to be updated.
    Parameters:
    pDO: A pointer to the DuplicatedObject to which the DataSet pDS is associated.
    pDS: A pointer to the DataSet that will be verified whether or not it needs to be updated.
    tUpdateTime: The time at which the update is valid. This is equal to the time that is
    passed when DuplicatedObject::Update is called.
    pTargetStation: A pointer to the remote Station where the Duplica we wish to verify the update for resides.
    Returns: true if the DataSet pDS needs to be updated on the pTargetStation, false if not or an error occurred.
    See Also: UpdateSent
    */
    virtual qBool UpdateRequired(DuplicatedObject* pDO,
                                 void* pDS,
                                 Time tUpdateTime,
                                 Station* pTargetStation)=0;

    // Never called for now.

    /*
    Description: This method is called if StationUpdateFilter::UpdateRequired returns true.
    Parameters:
    pDO: A pointer to the DuplicatedObject to which the DataSet pDS is associated.
    pDS: A pointer to the DataSet that was updated.
    tTimestamp: The time at which the update was valid. This is equal to the time that is
    passed when DuplicatedObject::Update is called.
    pTargetStation: A pointer to the remote Station where the updated Duplica resides.
    Remarks: It is optional whether or not you implement this method.
    See Also: UpdateRequired
    */
    virtual void UpdateSent(DuplicatedObject* pDO,
                            void* pDS,
                            Time tTimestamp,
                            Station* pTargetStation)
    {
        // All params are dummy - we don't need them here.
        DUMMY_PARAM(pDO);
        DUMMY_PARAM(pDS);
        DUMMY_PARAM(tTimestamp);
        DUMMY_PARAM(pTargetStation);
    };
private:
    qUnsignedInt32 m_uiUpdateLoopNumber;
};

END_NAMESPACE_NEX

