﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/Facade/FacadeHeaderBegin.h>

#include <NetZ/src/Core/Facade/NetZDecl.h>
#include <NetZ/src/Core/Facade/Exports.h>
#include <NetZ/src/Core/Facade/ProductFacade.h>
#include <NetZ/src/Core/DOCore/NetworkTopology.h>

BEGIN_NAMESPACE_NEX
/*
{group: Session Management}
{noAutoLink}
Description: This class encapsulates the run time of the NetZ system.
Remarks: NetZ is the class that encapsulates the run time of the NetZ system. It must be
instantiated to use NetZ. Upon destruction of this class NetZ will cease to function.
*/

/*!
 * @brief   NetZのランタイム機能をカプセル化するクラスです。
 *
 * NetZを使用するためにはインスタンスを生成しなければなりません。
 * インスタンスを破棄するとNetZ機能は終了します。
 *
 * ユーザーDO を使用する(ddlファイルを作成する必要がある場合)場合、DDLコンパイラによって生成される InitDOClasses() を
 * NetZ::SetDOClassInitializationCallback() でセットしてください。 ユーザーDO を使用しない場合は特別な処理は不要です。
 * ホストマイグレーション拡張機能を使用する場合は、ユーザーDOを使用しない場合でも InitDOClasses() の設定が必要です。
 *
 * NetZインスタンスはローカルセッションオブジェクトを保持するので、
 * セッションごとにNetZインスタンスを作りなおす必要があります。
 * @ref Session::CreateSession 、@ref Session::JoinSession の処理に失敗した場合でも、
 * 再度セッションに参加するにはNetZインスタンスを作りなおす必要があるので注意してください。
 */
class NetZ: public ProductFacade
{
public:
    //group=Methods

    /*
    Remarks: Only one instance of NetZ is allowed. Once this instance is
    destroyed, the currently running session (if any) is terminated and the run time stops.
    Returns: The created NetZ object.
    SYSTEMERROR_GEN_MULTIPLE_INSTANCES is signaled if more than one NetZ object is created.
    */

    /*!
     * @brief   NetZクラスのインスタンスの存在はただ一つのみ許されます。
     *
     * このインスタンスが破棄されれば、現在作成されているセッションは終了され、
     * ランタイム機能は停止します。
     * @retval  作成されたNetZオブジェクトを返します。
     *          2つ以上のNetZオブジェクトを作成するとSYSTEMERROR_GEN_MULTIPLE_INSTANCESが
     *          シグナルされます。
     */
    NetZ();

    /*
    Remarks: When the NetZ instance is destroyed, it cleanly shuts down all NetZ related instances.

    All the duplicated objects (Duplication master and Duplica) stored on the local station are destroyed, as long as no references to the objects still exist, /e.g./ those created via the RefTemplate or DuplicatedObject::Ref.

    This includes the local Session object, which is deleted by NetZ during shut down thereby terminating the currently running Session.

    Therefore, in order to join another Session you need to create a new NetZ instance once the current NetZ object has been deleted.

    NetZ destruction is performed in a Synchronous manner and in single thread mode, even if you have previously called UseThreads.

    Therefore, this method will block while NetZ is shut down and the objects destroyed.

    If you do not wish the destructor to block, you can call Terminate before the NetZ destructor.

    In this case, Terminate causes all NetZ related instances to be cleanly shut down however, the objects are not actually destroyed until the NetZ destructor is called.

    Note: All references to all duplicated objects must be released before the NetZ object destructor is called.
    If not,  NetZ cannot call the destructor of the referenced objects.
    See Also: Duplication master, Duplica, Terminate
    */

    /*!
     * @brief   NetZインスタンスが破棄されたとき、全てのNetZ関連のインスタンスはシャットダウンされます。
     *
     * デストラクタを呼び出す前に、必ず @ref Terminate を実行して
     * P2P セッションの終了を行なってください。
     * @ref Terminate を呼び出さずに、デストラクタが呼ばれた場合、内部で@ref Terminate を呼び出します。
     * @ref Terminate も参照してください。
     *
     * @see     Terminate
     */
    virtual ~NetZ();

    /*
    Description: Ends, but does not destroy, this NetZ session.
    Parameters:
    pContext: A pointer to the CallContext for this call.
    Returns: true if the NetZ session terminated successfully, false if not or an error occurred.
    Remarks: When this method is called all NetZ related instances are cleanly shut down however,
    the objects are not actually destroyed until the NetZ destructor is called. As this method
    is called in an Asynchronous manner, it enables processing to continue while a NetZ session
    is in the process of shutting down.

    Whether you call the NetZ destructor, or Terminate and then the NetZ destructor to end a NetZ
    Session, the termination of processes and subsequent destruction of objects is still performed
    in the same manner. The only difference is that with the latter option, the termination part of
    the operation is called asynchronously.
    Example:
                CallContext oContext;

                NetZ::GetInstance()->Terminate(&oContext);
                // Directly wait, or continue
                // running the game loop until the call context is
                // completed.
                oContext.Wait();

                // Now, when the destructor is called, the method
                // will return instantaneously
                delete pNetZ;

    See Also: ~NetZ
    */

    /*!
     * @brief   オブジェクトは破棄せずに P2P セッションの終了処理を行います。
     *
     * NetZのデストラクタを呼ぶ前に、本関数を必ず実行してください。
     * 本関数は通信状況によっては完了までに遅くとも5秒程度かかることがあります。
     * 処理時間の要因については、プログラミングマニュアルを参照してください。
     *
     * 終了処理によって、すべての複製オブジェクトが破棄されます。
     * アプリケーションによる複製オブジェクトへの参照（例えば、RefTemplateまたはDuplicatedObject::Refによって
     * 作成されたもの）が残っていても、強制的に破棄されます。
     * そのため、本関数を呼び出す前に、必ずアプリケーションで保持しているずすべての複製オブジェクトへの参照を開放してください。
     *
     * この複製オブジェクトにはローカルセッションオブジェクトが含まれ、
     * そして、それは現在のセッションのシャットダウン中に
     * NetZによって削除されます。
     * したがって、他のセッションに参加するためには、一旦現在のNetZオブジェクトを
     * 削除して、新たなNetZを作る必要があります。
     * @ref Session::CreateSession 、@ref Session::JoinSession の処理に失敗
     * した場合にもローカルセッションオブジェクトが残っている可能性があるため、
     * NetZを作りなおす必要があります。
     *
     * この関数が非同期に呼ばれた場合、 P2P セッションがシャットダウンプロセスにいる間も
     * 処理を継続できます。
     *
     * @param   pContext    呼び出しの@ref CallContext へのポインタ
     * @retval  true    P2P セッションが正常に強制終了された場合に返します。
     * @retval  false   P2P セッションが異常に強制終了された場合、またはエラーが起こった場合に返します。
     */
    virtual qBool Terminate(CallContext* pContext)
    {
        return ProductFacade::Terminate(pContext);
    }

    //{secret}
    virtual qResult Terminate()
    {
        return ProductFacade::Terminate();
    }

    /*
    Description: Returns a pointer to the NetZ instance.
    Returns: A pointer to the NetZ instance.
    Remarks: Use the reference returned by this method to control the NetZ object.

    Refer to Terminate for an example of this method in use.
    */

    /*!
     * @brief   NetZインスタンスへのポインタを返します。
     *
     * NetZオブジェクトを操作するためには、この関数が返す参照を使用して下さい。
     * @return  NetZインスタンスへのポインタ。
     */
    static NetZ* GetInstance()
    {
        return static_cast<NetZ*>(ProductFacade::GetInstance());
    }

    /*
    Description:  Returns whether or not the NetZ instance is valid.
    Returns: true if the NetZ instance is valid, false if not or an error occurred.
    Remarks: The NetZ instance will not be valid if more than one is created.
    */

    /*!
     * @brief   NetZインスタンスが正常かどうかを返します。
     *
     * NetZインスタンスが2つ以上作成されると異常になります。
     *
     * @retval  true    NetZインスタンスが正常な場合に返ります。
     * @retval  false   NetZインスタンスが異常な場合、またはエラーが起こった場合に返します。
     */
    virtual qBool IsValid() const
    {
        return ProductFacade::IsValid();
    }

    /*!
     * @brief   NetZ内部で発生しているフェイタルエラーを取得します。
     *
     * NetZインスタンス作成後、毎フレームに1度の頻度で実行し、
     * フェイタルエラーが発生していないかどうかをチェックしてください。
     * @ref NetZ::Terminate 中は本関数でエラーチェックを行う必要はありません。
     *
     * 何らかのフェイタルエラーが発生している場合には、失敗を示す @ref qResult が返ります。
     *
     * 本関数の返り値のハンドリング方法は以下のとおりです。
     *
     * @li フェイタルエラーが発生しているかどうかは、本関数の返り値を @ref QFAILED を使ってチェックするか、false と比較してください。
     * @li 本関数が返す @ref qResult が失敗を示す場合は何らかのフェイタルエラーが発生しているので、エラーを保存して速やかに @ref NetZ を破棄し、
     * @ref ErrorCodeConverter::ConvertToNetworkErrorCode で取得できるエラーコードを画面に表示してください。
     * @li 本関数が返す @ref qResult が成功を示す場合は、アプリケーションで特別なハンドリングを行う必要はありません。
     *
     * 本関数で取得できるqResultは以下のとおりです。
     *
     * @li QSUCCESS(Core, Success) (エラーが発生していないときに返る成功を示す値)
     * @li QERROR(DOCore, InvalidState)
     * @li QERROR(DOCore, FaultRecoveryFatal)
     * @li QERROR(DOCore, FaultRecoveryJobProcessFailed)
     * @li QERROR(Transport, ReliableSendBufferFullFatal)
     * @li QERROR(Transport, NoBuffer)
     * @li QERROR(DOCore, StationInconsistency)
     *
     * @return  内部で発生したフェイタルエラー
     * @see     ErrorCodeConverter::ConvertToNetworkErrorCode, QFAILED, DOCoreErrorCode::ErrorCode
     *
     */
    virtual qResult GetFatalError()const
    {
        return ProductFacade::GetFatalError();
    }

    /*!
     * @brief   @ref NetZ::Terminate の終了モードを設定します。
     *
     * ホストがホストの権限を他のステーションへ移譲するときや、DOのマスターを他のステーションへ移譲するときに、
     * @ref NetZ::Terminate の処理時間が5秒程度かかることがあります。
     *
     * 強制終了モードへの変更により、1秒程度で処理を終えることができます。
     * ただし、通常の切断処理が行われないため、通信相手にはキープアライブのタイムアウトが発生してこちらのステーションがネットワークから切断されたように見える場合があります。
     * そのため、通常の終了シーケンスで設定することは控えてください。
     *
     * 以下の条件で強制終了モードに自動設定されます。
     *
     * @li ホストが@ref Session::GameOver() を呼び出して、P2P全体のセッションを終了した場合
     * @li ホストの@ref Session::GameOver() の呼び出しで、ホスト以外のステーションで@ref Session::CheckGameIsOver() がtrueを返す場合
     * @li 明らかにネットワークから切断されている、もしくは自分以外のステーションが無く孤立しているため、@ref Session::DestroySession() を呼び出した場合
     *
     * 初期値は通常終了モードです。
     * 本設定は、NetZインスタンス毎に保持されます。
     *
     * @param[in] immediately trueを指定すると強制終了モードになります。falseを指定すると通常終了モードになります。
     *
     * @see Core::SetTerminateImmediately(), Session::GameOver(), Session::DestroySession()
     */
    virtual void SetTerminateImmediately(qBool immediately)
    {
        ProductFacade::SetTerminateImmediately(immediately);
    }

    /*!
     * @brief   @ref NetZ::Terminate の終了モードを取得します。
     *
     * @return @ref NetZ::Terminate の終了モードが強制終了モードであればtrue、通常終了モードであればfalseが返ります。
     */
    virtual qBool GetTerminateImmediately()
    {
        return ProductFacade::GetTerminateImmediately();
    }

    // {secret}
    /*
    Allows you to manually appoint the router for ClientServer sessions.
    Remarks: Applicable only when the selected mode is ClientServer. Manually appoints the
    router for Net-Z sessions by specifying the router's address. By default, if no router is defined,
    the session master (the station that created the session) is the router.
    Parameters:
    szMachine: The URL of the router
    ui16Port: The Net-Z port on the router.
    */
    void SetRouterAddress(const qChar *szMachine, const qUnsignedInt16 ui16Port);

    /*
    Description: Defines the type of network topology that Net-Z will establish.
    Remarks: Use this method before creating the NetZ facade object and Net-Z will establish the network
    of your choice. The default network topology is (the traditional) Peer2Peer where Net-Z establishes direct
    communications between all stations in a game session. Using this command, you can manually instruct
    Net-Z to establish a ClientServer network.
    In a ClientServer network, the session master acts as a server relaying game messages for all
    participants.
    Parameters:
    eSelectedTopology:  The networking topology that a Net-Z session will establish. This parameter
    must be Peer2Peer or ClientServer or Hybrid.
    */

    /*!
     * @brief   NetZが確立するネットワークトポロジーのタイプを定義します。
     *
     * NetZファサードオブジェクトの作成前にこの関数を使用すれば、
     * 選択したタイプのトポロジーを使用したNetZネットワークトポロジーが確立されます。
     *
     * デフォルトのネットワークトポロジーは、すべてのステーションがセッションに
     * 参加すると相互に直接接続する典型的な P2P ネットワークです。
     * このコマンドを使用すれば、NetZにサーバークライアントネットワークを確立するように
     * 指定することが出来ます。サーバークライアントネットワークでは、セッションマスターは
     * すべての参加者にゲームメッセージを送るサーバーになります。
     * ハイブリッドネットワークでは、Peer2Peerネットワークをベースに、
     * 相互に直接接続できないステーション間の通信をセッションマスター経由で通信します。
     *
     * @param   eSelectedTopology   P2P セッションが確立するネットワークトポロジーです。
     *                              この引数は Peer2Peer、ClientServer、Hybridのいずれかです。
     */
    static void SetNetworkTopology(NetworkTopology::Mode eSelectedTopology);


    /*!
     @brief DO クラス初期化関数を登録します。

     NetZオブジェクトのコンストラクト時にセットしたDO クラス初期化関数が呼び出されます。
     ユーザーDO 機能を使用する場合は、DDLコンパイラによって生成された InitDOClasses() をセットしてください。
     ユーザーDO 機能を使用しない場合は、本関数を呼び出す必要はありません。

     @param fInitDOClasses DO クラス初期化関数
     */
    static void SetDOClassInitializationCallback(InitDOClassesFunction fInitDOClasses);

private:
    // no copying and copy assignment allowed

    NetZ (const NetZ&);
    NetZ& operator= (const NetZ&);

    static InitDOClassesFunction s_fInitDOClasses;

};


END_NAMESPACE_NEX

#include <NetZ/src/Core/Facade/FacadeHeaderEnd.h>

