﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Core/OperationManager.h>

BEGIN_NAMESPACE_NEX

//Group=Host Migration

/*
Description: This class implements a callback on a specific session that is called by Net-Z
when the session master migrates.
Remarks: The role of a Net-Z session master (the station that hosts a Net-Z session) is fault-tolerant
and, therefore, the role of session master may migrate if the current session master leaves the
session. When a migration occurs, if you have registered the HostMigrationExtension with Net-Z, Net-Z
will call this HostMigrationCallback to notify the Rendez-Vous servers of the change and ensure that
the Rendez-Vous database is kept up-to-date. In particular, this will ensure that the PrincipalID of
the Rendez-Vous Gathering host (the Net-Z session master), stored in the Gatherings database table.

When partial faults occur in a Net-Z game session as the session master automatically migrates, in some situations
two or more players in the same Gathering game session may believe they should be the session master. Therefore,
to avoid the Gathering host being incorrectly updated on the $PRODUCTNAME$ database $PRODUCTNAME$ implements
a voting scheme. In essence, $PRODUCTNAME$ verifies with all the gathering's participants who they think should
be the Gathering host and determines the host according to the votes received. Therefore, unless the majority
of participants think the session master has migrated, the $PRODUCTNAME$ database will not be altered. This prevents
players forcing a gathering host migration on an unsuspecting session host, as could occur when players intentionally
or otherwise momentarily loose their network connection.
The only situation where $PRODUCTNAME$ is unable to accurately determine the Gathering host is for 2-player sessions.
Therefore, for 2-player games or when there are only two players left in the session, when both players believe they are
the session host we recommend that you end the match and return both players to the lobby.
*/

/*!
 @brief    Net-Z のホストマイグレーション実行時のコールバック関数

 Net-Z のホストマイグレーション実行時のコールバック関数です。Net-Z のホストマイグレーションが行われると、
 自動的に新しいホストがゲームサーバーに登録されます。

 NetZ のホストマイグレーションは自動的に行われますが、同時に 2 人以上のプレーヤーがセッションホストであるかのように
 振舞うことがあります。このような場合にゲームサーバーに間違ったホストが登録されてしまわないように、NEX では
 他のステーションからの投票があった新しいホストがゲームサーバーに登録されます。

 ただし、セッション内に 2人しかプレーヤーがおらず、タイムアウトなどが発生してお互いのステーションを異常状態で見失った場合は
 どちらがホストであるのかを投票では決定することができません。
 両者がどちらも自分がホストであると認識している場合（すなわち孤立状態に陥っている場合）はセッションを終了する
 か、新しくマッチメイクセッションを登録することを推奨します。詳細は、 @ref HostMigrationCallback::SessionHostIsOrphan() の
 API リファレンスを参照してください。
 */
class HostMigrationCallback: public OperationCallback
{
public:
    //group=Methods

    /*
    Parameters:
    pClient: A pointer to the MatchMakingClient that will call this HostMigrationCallback.
    iPriority: The priority allocated to this HostMigrationCallback. This value must be between 513 and 1023
    and the default value is 600. Unless you implement several HostMigrationCallback s you should not need
    to change this parameter.
    Remarks: A HostMigrationCallback can only be created after the MatchMakingClient object is created, which must remain
    valid for the lifetime of the callback. In addition, the HostMigrationCallback must be created before any call to the
    Net-Z Session::CreateSession method.

    The priority of a HostMigrationCallback, as given by the iPriority parameter, refers to the call
    priority as defined in the Net-Z OperationManager and OperationCallback classes. Refer to your Net-Z
    documentation for further details.
    */

    /*!
     @brief    ホストマイグレーション発生時に呼び出されるコールバック関数の定義です

     @param[in]    pMatchMakingClientClient    コンストラクタで指定した @ref MatchmakeExtensionClient のポインタが渡されます
     @param[in]    gatheringID                 Register() で指定した @ref GatheringID が渡されます
     */
    typedef qBool (*UpdateSessionHostCB)(void* pMatchMakingClientClient, qUnsignedInt32 gatheringID);

    /*!
     @brief    コンストラクタです。

     @param[in]    pMatchMakingClientClient    有効な( Bind された) @ref MatchmakeExtensionClient
     @param[in]    updateSessionHostCB  ホストマイグレーション発生時に呼び出されるコールバック関数。通常 @ref MatchMakingClient::StaticUpdateSessionHost
                                        もしくは @ref MatchMakingClient::StaticUpdateSessionHostWithOwner を指定します。
     @param[in]    iPriority    優先度。デフォルト値は 600 です。
     */
    HostMigrationCallback(void* pMatchMakingClientClient, UpdateSessionHostCB updateSessionHostCB , qInt32 iPriority=600);

    /*
    Remarks: A HostMigrationCallback should be deleted before the NetZ object and only once all Gathering s
    have been unregistered via Unregister.
    */

    /*!
     @brief    デストラクタです。
     */
    virtual ~HostMigrationCallback();

    /*
    Description: Registers the specified Gathering to this HostMigrationCallback.
    Parameters:
    idGathering: The GatheringID of the Gathering that will be registered to this HostMigrationCallback.
    Remarks: Register should be called by each Principal that participates in the Gathering. The HostMigrationExtension
    package uses Net-Z. So you must either create or join a session before calling this method. That is,
    call this method after the NetZ object is created and after the Principal has called MatchMakingClient::Participate.

    Once registered, a Gathering may be unregistered from a HostMigrationCallback via Unregister.
    Example:
    $import HostMigrationCallbackRegister1 TOP\Tests\Services\Framework\ClientContext.h$
    $import HostMigrationCallbackRegister2 TOP\Tests\Services\Lobby\TestNetZIntegration.cpp$

        // Participate in the gathering
        {
            ProtocolCallContext oContext;
            qBool bCallResult;
            pPlayerA->GetMMClient()->Participate(&oContext, hGA->GetID(), &bCallResult);
            oContext.Wait();
            bCallResult;
        }

        // Register the gathering to the callback
        if (bUseHostMigration) {
            pPlayerA->GetHostMigrationCallback()->Register(hGA->GetID());
            // Check that the player is not an orphan.
            pPlayerA->GetHostMigrationCallback()->SessionHostIsOrphan() == false);
        }

    See Also: Unregister
    */

    /*!
     @brief    ホストマイグレーションの対象を登録します

     P2P セッションの生成・参加後( @ref Session::CreateSession 関数、@ref Session::JoinSession 関数の非同期処理完了後)
     に行われる必要があります。

     @param[in]    idGathering    登録するギャザリングの ID
     */
    void Register(qUnsignedInt32 idGathering);

    /*
    Description: Unregisters the specified Gathering from this HostMigrationCallback.
    Parameters:
    idGathering: The GatheringID of the Gathering that will be unregistered from this HostMigrationCallback.
    Remarks: A Gathering should be unregistered from the associated HostMigrationCallback via this method as soon as
    a given Principal no longer participates in the Gathering, /i.e./ the Principal calls MatchMakingClient::CancelParticipation,
    MatchMakingClient::EndParticipation, or the Gathering ends.

    In order to unregister a Gathering from a HostMigrationCallback, it must have previously been registered via Register.
    This method should be called before the NetZ object is destroyed.
    See Also: Register
    */

    /*!
     @brief    ホストマイグレーションの対象から削除します

     ホストマイグレーションの対象から削除します。
     指定したギャザリング ID が Register() されていない場合は何も行いません。
     @ref NetZ::Terminate で終了処理をする前に実行してください。

     @param[in]    idGathering    削除するギャザリングの ID
     */
    void Unregister(qUnsignedInt32 idGathering);

    /*
    Description: Returns whether or not the Gathering session host is orphaned.
    Returns: true if the Gathering session host is orphaned, false if it is not or an error occurred.
    Remarks: A Gathering session host is considered orphaned if the following constraints are all
    observed:
        o the station was promoted to the session master by Net-Z,
        o no other stations confirmed to Rendez-Vous that the station is the new session master, and
        o the session master is the only station in the session.

    A Gathering host may become orphaned when certain fault sequences occur between the participants in a
    Net-Z gathering. This results in a given station faulting and therefore, becoming the session host of its own
    session, in which it is the only participant. Therefore, we have the situation where there exists two Net-Z
    sessions, one containing one participant and the other containing all remaining participants, both with the
    same GatheringID. When the newly promoted session master updates the gathering host on the Rendez-Vous database
    to itself, major problems entail as the gathering participants cannot connect to the new master. Therefore,
    SessionHostIsOrphan should be called before a gathering host is updated to ensure that orphaned session masters
    do not update the Rendez-Vous database.

    SessionHostIsOrphan should be called on every Net-Z ChangeMasterStation operation and generally be polled every frame
    in order to detect when a given station has been isolated in its own session.

    Refer to Register for an example of this method in use.
    See Also:Register
    */

    /*!
     @brief    自分が孤立したセッションホストであるかを判定します

     自分が孤立したセッションホストであるかを判定します。孤立状態に陥いっても P2P セッションを継続することは
     可能ですが、他のクライアントがこの P2P セッションに参加することはできなくなります。
     この問題を避けるためには、今まで入っていたマッチメイクセッションから退出し、新しくマッチメイクセッションを
     登録する必要があります。そのとき、既存の @ref HostMigrationCallback のインスタンスを破棄し、
     新しく @ref HostMigrationCallback のインスタンスを生成して新しいマッチメイクセッションのIDを登録しなおしてください。
     @ref SessionHostIsOrphan() が true を返したときに@ref HostMigrationCallback のインスタンスを再利用することはできません。

     下記の条件を全て満たしたとき、孤立したセッションホストであると判定します。

     @li    ・Net-Z のセッションホストに昇格した<br>
     @li    ・他の Station から自分がセッションホストであると承認されていない<br>
     @li    ・最後に抜けたステーションのうち、離脱原因が正常ではなく、タイムアウトなど障害によるものがひとつでもあった(　Station::LeavingOnFault) <br>
     @li    ・セッション内に唯一の Station である<br>

     ChangeMasterStation が実行されたときには孤立状態に移行していないかを確認するようにしてください。
     フレーム毎にポーリングしても構いません。

     @retval    true    孤立している。
     @retval    false    孤立していない。
     */
    qBool SessionHostIsOrphan() const;

protected:
    //{secret}
    void UpdateGatheringHost();

    friend class HostMigrationNotifier;

private:
    virtual void CallMethod(Operation* pOperation);
    void TryUpdateHost();

private:
    void* m_pClient;
    UpdateSessionHostCB m_UpdateSessionHostCB;
    qList<qUnsignedInt32> m_lstGatherings;

    qBool m_bLocalObservation;
    qBool m_bRemoteObservation;
    qBool m_bHostMigrationDone;
    qBool m_bSessionIsOver;
};

END_NAMESPACE_NEX

