﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DatasetCGMacros.h>
#include <NetZ/src/Core/DOCore/StationUpdateFilter.h>
#include <NetZ/src/Core/DOCore/GlobalUpdateFilter.h>
#include <OnlineCore/src/Core/Holder.h>

BEGIN_NAMESPACE_NEX

class DuplicatedObject;
class Station;

//Group=Custom Dead Reckoning

/*
Description: (Template class) Implements a scheme which updates Duplicas in an identical manner.
Remarks: When this policy is used, the Duplication master locally instantiates only one instance
of the dead reckoning model class, which is used to determine if any of its duplicas need to be
updated. Therefore, as the same model instance is used for each Duplica, if the model is no longer
accurate enough and needs to be updated a message is sent to all duplicas.

Aa a SharedModelPolicy is associated to the Duplication master object, its lifetime is the same as the
duplicated object. Therefore, if you need to perform any specific operations on the model upon its
initialization or deletion you should do so within the AddToStoreOperation and RemoveFromStoreOperation
respectively.

This is one manner in which a dead reckoning model can be applied to the duplicas. You may also use
the PerStationModelPolicy or implement your own model policy. Refer to Custom DR Implementation for
details on custom implementations.
See Also: PerStationModelPolicy, the Marbles sample
*/
template<class MODEL>
class SharedModelPolicy: public RootObject
{
public:
    //Group=Methods

    SharedModelPolicy() {};
    ~SharedModelPolicy() {};

#ifdef FOR_DOC_PURPOSE
    /*
    Description: Returns a Duplication master's local instance of one of its Duplica's dead reckoning models.
    Parameters:
    pTargetStation: A pointer to the remote Station for which we wish to retrieve the dead reckoning model.

    Note: As this model policy is the same for all Duplicas, we do not specify a
    target station because, regardless of which station we choose, the result will always be the same.

    Returns: A pointer to the Duplication master's local instance of the dead reckoning model, that you defined as described
    in the MyOwnModel class, which is associated to the Duplica that resides on /pTargetStation/, or a NULL pointer
    if no model is found.
    Remarks: This method is called when DuplicatedObject::Update is called and an update is required, in order to
    access a Duplication master's local instance of a given Duplica's dead reckoning model. If this method returns a NULL
    pointer then an update will not be sent to the remote Station.

    If you wish to indicate continuity breaks, one way to implement them is to modify the implementation of this
    method so that when the method is called and a continuity break is indicated, then the model's behaviour changes.
    For example, you could force updates to be sent (IsAccurateEnough returns false) for the next three Update calls.
    Alternatively, you could implement continuity breaks as part of the dataset. Another way to use this method is to
    set the error threshold between the Duplication master's dataset values and its Duplica's via this method's implementation.
    See Also: GetLocalDeadReckoningModel, the Marbles sample.
    */
    MODEL* GetRemoteDeadReckoningModel(const Station* pTargetStation)
    {
        return &m_oSharedModel;
    }
#else
    MODEL* GetRemoteDeadReckoningModel(const Station* /*pTargetStation*/)
    {
        return &m_oSharedModel;
    }
#endif
    /*
    Description: Returns a Duplica's local instance of the dead reckoning model.
    Returns: A pointer to the Duplica's local instance of the dead reckoning model that you defined as described
    in the MyOwnModel class.
    Remarks: This method is called when DuplicatedObject::Refresh is called and an update has been received,
    in order to access a given Duplica's local instance of the dead reckoning model. If you wish to implement
    local corrections, you could do so by modifying the implementation of this method so that when the method is called
    corrections are made to the duplica's local model to ensure the object does not pass through walls, floors, and the like.
    See Also: GetRemoteDeadReckoningModel
    */
    MODEL* GetLocalDeadReckoningModel()
    {
        return &m_oSharedModel;
    }


    // This method is not implemented for a SharedModelPolicy bacause the Duplication master only instantiates one MODEL.
    //{secret}
    void ClearRemoteDeadReckoningModel(const Station* /*pTargetStation*/) {};

    /*
    Description: Returns the number of dead reckoning model class instances that are currently
    instantiated by the Duplication master.
    Returns: The number of instances of the given dead reckoning model class that are
    currently instantiated by the Duplication master.
    Remarks: If a SharedModelPolicy is used then the number of models will be equal to one as the same
    model is used for all duplicas.
    If a PerStationModelPolicy is used then the number of models will be equal to the number of duplicas as
    one model is instantiated for each Duplica.
    */
    qUnsignedInt32 GetNbModels() const
    {
        return 1;
    }

private:
    // We keep one MODEL for all duplicas
    //{secret}
    MODEL m_oSharedModel;
};

END_NAMESPACE_NEX


