﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

// 以下、コールバックが関数ポインタかどうかを判断するためのユーティリティを定義する。
// コールバック関数かどうかを判断する機能はDeleteIfNotFunctionPointer()で使用している
// C++11ではstd::is_functionとstd::is_pointerを用いて判定できるが、C++03に対応するため自前で定義する。

// C++03用にstd::enable_ifと同等のものを定義する
template < bool Cond, class T = void >
struct EnableIfImpl
{
    typedef T type;
};

template < class T >
struct EnableIfImpl< false, T > {};

template < bool Cond, class T = void >
struct EnableIf : public EnableIfImpl< Cond, T > {};

template <class T>
struct RemovePointer;

// C++03用にstd::remove_pointerと同等のものを定義する
template <class T>
struct RemovePointer<T*>
{
    typedef T type;
};

// TFunctionToDeleteがCallbackHolderが受け取れる関数かどうかを判定する。
// 汎用のものを実装するのは大変なので、void(void) もしくは void(TCallbackParam)
// かどうかだけ判断できる。
template<typename TFunctionToDelete>
struct IsCallbackFunction
{
    static const qBool value = false;
};

template<typename TCallbackParam>
struct IsCallbackFunction<void(TCallbackParam)>
{
    static const qBool value = true;
};

template<>
struct IsCallbackFunction<void(void)>
{
    static const qBool value = true;
};


// 以下任意のvoidを引数に取る関数に対して指定した引数を追加するユーティリティを定義する
/*!
@internal
@brief AddArgumentAdapter() のためのヘルパークラス
*/
template<typename TFunction, typename TAdded>
struct AddArgumentAdapterHelper : public RootObject
{
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    AddArgumentAdapterHelper(TFunction&& function) NEX_NOEXCEPT
        : m_function(std::forward<TFunction>(function))
    {}
#else
    AddArgumentAdapterHelper(const TFunction& function)
        : m_function(function)
    {}
#endif
    template<typename U>
    void CallFunction(U& function)
    {
        function();
    }
    template<typename U>
    void CallFunction(U* function)
    {
        (*function)();
    }
    void operator()(TAdded)
    {
        CallFunction(m_function);
    }
    TFunction m_function;
};

// 参照が渡された時の特殊化。
// 参照が渡された時に参照を保持するのではなくコピーするため特殊化する。
template<typename TFunction, typename TAdded>
struct AddArgumentAdapterHelper<TFunction&, TAdded> : public RootObject
{
    AddArgumentAdapterHelper(const TFunction& function)
        : m_function(function)
    {}

    template<typename U>
    void CallFunction(U& function)
    {
        function();
    }
    template<typename U>
    void CallFunction(U* function)
    {
        // ポインタの参照が渡されることがあるので
        // ポインタの場合はデリファレンス
        (*function)();
    }
    void operator()(TAdded)
    {
        CallFunction(m_function);
    }
    TFunction m_function;
};

/*!
@internal
@brief    引数を持たない関数を受け取り、TAdded を引数に取る関数に変換する関数アダプタです。

引数を取らない関数ポインタや関数オブジェクトなどを function に指定すると TAdded を引数に取る関数オブジェクトを返します。
CallContext::RegisterCompletionCallbackGeneric() に引数を一つも取らない関数を渡したい場合に使用します。

@param function 引数を一つも取らない関数ポインタや関数オブジェクト。
*/
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
template<typename TFunction, typename TAdded>
AddArgumentAdapterHelper<TFunction, TAdded> AddArgumentAdapter(TFunction&& function)
{
    return std::forward<AddArgumentAdapterHelper<TFunction, TAdded> >(
        AddArgumentAdapterHelper<TFunction, TAdded>(std::forward<TFunction>(function)));
}
#else
template<typename TFunction, typename TAdded>
AddArgumentAdapterHelper<TFunction, TAdded> AddArgumentAdapter(const TFunction& function)
{
    return AddArgumentAdapterHelper<TFunction, TAdded>(function);
}
#endif

/*!
@internal
@brief AddArgumentIfVoidAdapter() のためのヘルパークラス
*/
// 基本ケース。すでに引数を持つ場合の定義。変換せずにそのまま関数を返す。
template<typename TFunction, typename TAdded, typename TParam>
struct AddArgumentIfVoidAdapterHelper
{
    typedef TFunction ReturnType;
    template<typename TFunctionReal>
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    static ReturnType f(TFunctionReal&& function)
    {
        return std::forward<ReturnType>(function);
    }
#else
    static ReturnType f(const TFunctionReal& function)
    {
        return function;
    }
#endif
};

// 引数がvoidのケース。引数をTAddにした関数オブジェクトを返す。
template<typename TFunction, typename TAdded>
struct AddArgumentIfVoidAdapterHelper<TFunction, TAdded, void>
{
    typedef AddArgumentAdapterHelper<TFunction, TAdded> ReturnType;
    template<typename TFunctionReal>
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    static ReturnType f(TFunctionReal&& function)
    {
        return std::forward<ReturnType>(AddArgumentAdapter<TAdded>(std::forward<TFunction>(function)));
    }
#else
    static ReturnType f(const TFunctionReal& function)
    {
        return AddArgumentAdapter<TAdded>(function);
    }
#endif
};

/*!
@internal
@brief    TParam が void の場合に TFunction を TAdded を引数に取る関数オブジェクトに変換して返す関数アダプタです。

CallContext::RegisterCompletionCallbackGeneric() に引数を一つも取らない関数を渡せるようにするために使用します。

@param function 引数を一つも取らないもしくは一つとる関数ポインタや関数オブジェクト。
@return TAdded を引数に持つ関数
*/
template<typename TFunction, typename TAdded, typename TParam>
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
typename AddArgumentIfVoidAdapterHelper<TFunction, TAdded, TParam>::ReturnType AddArgumentIfVoidAdapter(TFunction&& function)
{
    return std::forward<typename AddArgumentIfVoidAdapterHelper<TFunction, TAdded, TParam>::ReturnType>(
        AddArgumentIfVoidAdapterHelper<TFunction, TAdded, TParam>::f(std::forward<TFunction>(function)));
}
#else
typename AddArgumentIfVoidAdapterHelper<TFunction, TAdded, TParam>::ReturnType AddArgumentIfVoidAdapter(const TFunction& function)
{
    return AddArgumentIfVoidAdapterHelper<TFunction, TAdded, TParam>::f(function);
}
#endif


// コールバックを保持するためクラスのベースクラス。
// 様々な種類のCallbackHolderテンプレートクラスをm_pCompletionCallbacksに保持するための抽象クラス。
class CallbackHolderBase : public RootObject
{
public:
    CallbackHolderBase(qBool deleteCallbackAfterCall)
        : m_deleteCallbackAfterCall(deleteCallbackAfterCall)
    {
    }
    virtual ~CallbackHolderBase()
    {
    }
    virtual void Call(void*) = 0;
private:
    // 代入演算子は使わない
    CallbackHolderBase& operator=(const CallbackHolderBase&);
protected:
    const qBool m_deleteCallbackAfterCall;
};


// 以下、コールバックを保持するユーティリティを定義する。
// 実際にはTCallbackにはRegisterCompletionCallbackGenericで渡されたコールバック、
// TCallbackParamにはCallContextもしくはそのサブクラスが指定される。
// TCallbackに値が渡された場合はそれをコピーする。
// (呼び出し側で指定した関数オブジェクトをすぐ破棄出来るようにするため、参照を保持してはいけない)
template<typename TCallback, typename TCallbackParam>
class CallbackHolderImpl : public CallbackHolderBase
{
    typedef TCallbackParam CallbackParamType;
public:
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    CallbackHolderImpl(TCallback&& callback, qBool deleteCallbackAfterCall)
        : CallbackHolderBase(deleteCallbackAfterCall)
        , m_callback(std::forward<TCallback>(callback))
    {}
#endif
    CallbackHolderImpl(const TCallback& callback, qBool deleteCallbackAfterCall)
        : CallbackHolderBase(deleteCallbackAfterCall)
        , m_callback(callback)
    {}
    virtual ~CallbackHolderImpl(){}
    virtual void Call(void* cc)
    {
        m_callback(static_cast<TCallbackParam>(cc));

        // TCallbackがポインタではない場合は誤ってdeleteCallbackAfterCallを指定しているので警告を出す。
        if (m_deleteCallbackAfterCall)
        {
            QLOG(EventLog::Warning, NEX_T("You specified to delete callback object that is non pointer type after callback.")
                NEX_T(" Please check CallContext::RegisterCompletionCallbackGeneric() call."));
        }
    }
    TCallback m_callback;
};

// TCallbackParamがvoidの時の特殊化
// TCallbackParamがvoidの時はコールバックするときの引数が空になる。
template<typename TCallback>
class CallbackHolderImpl<TCallback, void> : public CallbackHolderBase
{
    typedef void CallbackParamType;
public:
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    CallbackHolderImpl(TCallback&& callback, qBool deleteCallbackAfterCall)
        : CallbackHolderBase(deleteCallbackAfterCall)
        , m_callback(std::forward<TCallback>(callback))
    {}
#endif
    CallbackHolderImpl(const TCallback& callback, qBool deleteCallbackAfterCall)
        : CallbackHolderBase(deleteCallbackAfterCall)
        , m_callback(callback)
    {}
    virtual ~CallbackHolderImpl() {}
    virtual void Call(void*)
    {
        m_callback();

        // TCallbackがポインタではない場合は誤ってdeleteCallbackAfterCallを指定しているので警告を出す。
        if (m_deleteCallbackAfterCall)
        {
            QLOG(EventLog::Warning, NEX_T("You specified to delete callback object that is non pointer type after callback.")
                NEX_T(" Please check CallContext::RegisterCompletionCallbackGeneric() call."));
        }
    }
    TCallback m_callback;
};

// TCallbackにポインタが指定された場合の特殊化
template<typename TCallback, typename TCallbackParam>
class CallbackHolderImpl<TCallback*, TCallbackParam> : public CallbackHolderBase
{
    typedef TCallbackParam CallbackParamType;
public:
    CallbackHolderImpl(TCallback* callback, qBool deleteCallbackAfterCall)
        : CallbackHolderBase(deleteCallbackAfterCall)
        , m_callback(callback)
    {}
    virtual ~CallbackHolderImpl(){}
    virtual void Call(void* cc)
    {
        // TCallbackがポインタの場合はデリファレンスする必要がある。(このために特殊化している)
        (*m_callback)(static_cast<TCallbackParam>(cc));

        // TCallbackがポインタの場合はコールバックオブジェクトを削除する。
        if (m_deleteCallbackAfterCall)
        {
            // 関数ポインタをdeleteするとコンパイルエラーになるため、
            // m_callbackが関数ポインタ以外(要するに関数オブジェクトの場合)のみdeleteする。
            DeleteIfNotFunctionPointer(m_callback);
        }
    }

    template<typename TFunctionToDelete>
    typename EnableIf<!IsCallbackFunction<typename RemovePointer<TFunctionToDelete>::type>::value>::type
        DeleteIfNotFunctionPointer(TFunctionToDelete func)
    {
        // TFunctionToDeleteが関数ポインタではないケース
        qDelete func;
    }
    template<typename TFunctionToDelete>
    typename EnableIf<IsCallbackFunction<typename RemovePointer<TFunctionToDelete>::type>::value>::type
        DeleteIfNotFunctionPointer(TFunctionToDelete)
    {
        // TFunctionToDeleteが関数ポインタのケース
        // deleteするとコンパイルエラーになるのでしない。
        QLOG(EventLog::Warning, NEX_T("You specified to delete function pointer after callback.")
            NEX_T(" Please check CallContext::RegisterCompletionCallbackGeneric() call."));
    }
    TCallback* m_callback;
};

// TCallbackにポインタが指定された場合かつTCallbackParamがvoidの時の特殊化
// TCallbackParamがvoidの時はコールバックするときの引数が空になる。
template<typename TCallback>
class CallbackHolderImpl<TCallback*, void> : public CallbackHolderBase
{
    typedef void CallbackParamType;
public:
    CallbackHolderImpl(TCallback* callback, qBool deleteCallbackAfterCall)
        : CallbackHolderBase(deleteCallbackAfterCall)
        , m_callback(callback)
    {}
    virtual ~CallbackHolderImpl(){}
    virtual void Call(void*)
    {
        // TCallbackがポインタの場合はデリファレンスする必要がある。(このために特殊化している)
        (*m_callback)();

        // TCallbackがポインタの場合はコールバックオブジェクトを削除する。
        if (m_deleteCallbackAfterCall)
        {
            // 関数ポインタをdeleteするとコンパイルエラーになるため、
            // m_callbackが関数ポインタ以外(要するに関数オブジェクトの場合)のみdeleteする。
            DeleteIfNotFunctionPointer(m_callback);
        }
    }

    template<typename TFunctionToDelete>
    typename EnableIf<!IsCallbackFunction<typename RemovePointer<TFunctionToDelete>::type>::value>::type
        DeleteIfNotFunctionPointer(TFunctionToDelete func)
    {
        // TFunctionToDeleteが関数ポインタではないケース
        qDelete func;
    }
    template<typename TFunctionToDelete>
    typename EnableIf<IsCallbackFunction<typename RemovePointer<TFunctionToDelete>::type>::value>::type
        DeleteIfNotFunctionPointer(TFunctionToDelete)
    {
        // TFunctionToDeleteが関数ポインタのケース
        // deleteするとコンパイルエラーになるのでしない。
        QLOG(EventLog::Warning, NEX_T("You specified to delete function pointer after callback.")
            NEX_T(" Please check CallContext::RegisterCompletionCallbackGeneric() call."));
    }
    TCallback* m_callback;
};

#ifdef NEX_MOVE_SEMANTICS_SUPPORT
// TCallbackにポインタの参照が渡された場合にCallbackHolderImplでポインタ型と見なされず、コンパイルエラーになるのを防ぐため、
// remove_referenceで参照を外す
// TCallbackがポインタの参照となるのは左辺値参照がTCallbackに渡されてユニバーサル参照によりTCallback&&→TCallback&と見なされた場合のみ。
// そのためNEX_MOVE_SEMANTICS_SUPPORTのときだけ参照を外せば良い。
template<typename TCallback, typename TCallbackParam>
using CallbackHolder = CallbackHolderImpl<typename std::remove_reference<TCallback>::type, TCallbackParam>;
#else
template<typename TCallback, typename TCallbackParam>
class CallbackHolder: public CallbackHolderImpl<TCallback, TCallbackParam>
{
public:
    CallbackHolder()
        : CallbackHolderImpl<TCallback, TCallbackParam>(false)
    {}
    CallbackHolder(const TCallback& callback, qBool deleteCallbackAfterCall)
        : CallbackHolderImpl<TCallback, TCallbackParam>(callback, deleteCallbackAfterCall)
    {}
};
#endif

END_NAMESPACE_NEX
