﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/Result.h>
#include <OnlineCore/src/Platform/Core/ScopedCS.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>

BEGIN_NAMESPACE_NEX

/** !private
 *  @brief 最初にセットされた @ref qResult だけを保持するスレッドセーフな管理クラスです。
 *
 *  @ref SetIfFirst は、一番最初に発生したエラーのみ保持されます。
 *  エラーを解除するAPIは用意していないので、リセットしたい場合はインスタンスを破棄して
 *  作りなおす必要があります。
 **/
class FirstErrorHolder : public RootObject
{
public:
    FirstErrorHolder() :
        m_FirstError(true)
    {}

    /** !private
     * @brief 最初に発生したエラーである場合、中で保持されます
     *
     * 必ずErrorの @ref qResult をセットしてください。
     * Successな @ref qResult をセットした場合はSYSTEMCHECKで停止します。
     **/
    void SetIfFirst(qResult error)
    {
        ScopedCS lock(m_Lock);
        if ( m_FirstError == true ) // 一度もエラーが発生していないときだけセット
        {
            SYSTEMCHECK(error==false);
            m_FirstError = error;
        }
    }

    qResult Get() const
    {
        ScopedCS lock(m_Lock);
        return m_FirstError;
    }

private:
    mutable CriticalSection m_Lock;
    qResult m_FirstError;
};


END_NAMESPACE_NEX


