﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/DebugString.h>
#include <OnlineCore/src/Core/InstantiationContext.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/UserContext.h>
#include <OnlineCore/src/Platform/Core/ProfilingUnit.h>
#include <OnlineCore/src/Platform/Core/qChain.h>


//n1769: Jobのコンストラクタに渡した文字列（Jobの解説）を保持する機能を無効に
//       したいときに、このマクロを有効にして下さい。
//#define DISABLE_JOB_DESCRIPTION

BEGIN_NAMESPACE_NEX

class CallbackRoot;

/*!
    @brief Scheduler に登録可能な非同期処理を行うクラスです。本クラスを直接使用しないで下さい。
*/
class Job: public RefCountedObject
{
    // Group=
public:
    enum State
    {
        Initial,
        Waiting,
        Suspended,
        Ready,
        Running,
        Complete
    };
    enum Priority {Low, Medium, High};

    typedef enum
    {
        Core = 0,
        NetZ,
        RV,
        Test
    } JobType;

    Job(JobType type, const DebugString& strJobDescription);
    virtual ~Job();

    qBool UsesGlobalLock() const
    {
        return m_bUsesGlobalLock;
    }
    void SetUseGlobalLockFlag()
    {
        m_bUsesGlobalLock = true;
    }

    qBool ShouldTraceState() const
    {
        return m_bTraceState;
    }

	// TRACE_SCHEDULERでトレースした際にこのJobをトレースするかどうかを設定する。
	// 頻繁に実行されるJobではfalseに設定しておく。
	// 初期値はtrue。
    void SetTraceStateFlag(qBool enable)
    {
        m_bTraceState = enable;
    }

    // This is the access point, from which Execute will be called.
    void PerformExecution(const Time& tDeadline, Time currentTime);

    // This is a method that can be overriden by subclasses, if
    // there is a need to add code prior or after the execution of
    // a job (e.g. profiling, traces, etc).
    virtual void DecoratedExecute()
    {
        Execute();
    }
    virtual void Execute()=0;
    // Called by scheduler, when a job is suspended.
    virtual void TestSuspendedJobState() {};

    //  Valid if activity tracing is enabled
    virtual void AddActivity(const qChar* szSupplement=NULL);

    State GetState() const
    {
        return m_eState;
    }

    virtual String GetTraceInfo() const;
    const qChar* GetStateString(State eState) const;
#ifdef NEX_JOB_TRACE_INFO_ENABLED
    const qChar* GetDescription() const;
    virtual void Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS);
#else
    const qChar* GetDescription() const
    {
        return NEX_T("");
    }
    void Trace(qUnsignedInt64 /*uiTraceFlags*/) {}
#endif

    TimeInterval GetWaitTime() const
    {
        return m_tiWaitTime;
    }
    void SetWaitTime(TimeInterval tiWaitTime)
    {
        m_tiWaitTime=tiWaitTime;
    }

    // Default behavior for a job: complete after the execution.
    virtual void SetDefaultPostExecutionState()
    {
        SetState(Complete);
    }

    virtual qBool SkipWaitDelayAtTermination()
    {
        return false;
    }

    qUnsignedInt32 GetNbExecutions() const
    {
        return m_uiNbExecutions;
    }

#if     NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB
    virtual void CancelJob();
#endif  // NEX_ENABLE_QUAZAL_TERMINATE_CANCELJOB
    // NetZ::Terminate中に呼ばれる。 呼ばれたら速やかにJobを終了させる。
    // 必要なJobにのみ実装する
    virtual void ProcessInTermination() {}

    qBool WaitForCompletion(qUnsignedInt32 uiTimeout=WAIT_INFINITE_TIMEOUT);

    //{secret}
    typedef void (*CompletionCallback)(Job*, const UserContext*);

    void RegisterCompletionCallback(CompletionCallback pfCompletionCallback, const UserContext& oContext);
    void RegisterCompletionCallback(CallbackRoot* pCallback);

    CompletionCallback GetCompletionCallback() const
    {
        return m_pfCompletionCallback;
    }

    void Resume()
    {
        SYSTEMCHECK(GetState()==Job::Suspended);
        SetState(Job::Ready);
    }

    void SetToWaiting(TimeInterval tiWaitTime)
    {
        SetState(Job::Waiting);
        m_tiWaitTime=tiWaitTime;
    }

    void SetToSuspended()
    {
        SetState(Job::Suspended);
    }

    void SetToReady()
    {
        SetState(Job::Ready);
    }

    void SetToRunning()
    {
        SetState(Job::Running);
    }

    void SetToComplete();

    void SetToCancel()
    {
        SetState(Job::Complete);
    }

    void SetToInitial()
    {
        SetState(Job::Initial);
    }

    virtual void ForceTerminate() {}

    //n1589
#ifdef NEX_TRACE_ENABLED
    LargeTimeInterval GetElapsedTime();
#endif

    Time GetDeadline() const
    {
        return m_tDeadline;
    }

    // 直近でExecute()された時刻を返す。
    Time GetExecutedTime() const
    {
        return m_tExecutedTime;
    }

    qBool DeadlineExpired() const
    {
        return Time::ConvertDeadlineToTimeout(m_tDeadline)==0;
    }

    //n1589:memo:JobPeriodicUpdate::JobPeriodicUpdateのみでtrue
    qBool IsSpecialJob() const
    {
        return m_bIsSpecialJob;
    }

    JobType GetJobType() const
    {
        return m_jobType;
    }

public:
    // public for the time of the merge.  Should be made back
    // private after.
private:

    void SetState(State eNewState);

    qBool ValidateTransition(State eInitState, State eFinalState);

    // Link interface
public:
    CHAINABLE(Job);

protected:
    qBool m_bIsSpecialJob;

private:
    CompletionCallback m_pfCompletionCallback;
    UserContext m_oContext;

#ifndef DISABLE_JOB_DESCRIPTION   //n1769
    JobType m_jobType;
    DebugString m_strJobDescription;
#endif

    State m_eState;
    TimeInterval m_tiWaitTime;
    qUnsignedInt32 m_uiNbExecutions;            //!< How many times was this job executed?

    Time m_tDeadline;
    Time m_tExecutedTime;   // ジョブがExecuteされた時刻
    qBool m_bUsesGlobalLock;
    qBool m_bTraceState;

    //n1589
#ifdef NEX_TRACE_ENABLED
    Time m_tiStartTime;
#endif
};

END_NAMESPACE_NEX

