﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Core/LiteBuffer.h>
#include <OnlineCore/src/Core/ByteOrder.h>

BEGIN_NAMESPACE_NEX

/*
 * @brief 静的領域を持ったByteStreamクラス
 *
 * 内部でnew,deleteしません
 * また、自動拡張しません
 *
 */

#define NEX_SIMPLE_TYPE_BYTE_STREAMING(TYPE)                \
        qBool Extract(TYPE *pValue) {return ExtractPrimitive(pValue);} \
        qBool Append(const TYPE &refValue) {return AppendPrimitive(refValue);} \
        qBool Extract(TYPE &refValue) {return ExtractPrimitive(&refValue);} \
        qBool Append(const TYPE *refValue) {return AppendPrimitive(*refValue);} \
        LiteByteStream& operator<<(const TYPE &refValue) {Append(&refValue); return *this;} \
        LiteByteStream& operator>>(TYPE &refValue) {Extract(&refValue); return *this;}

/*
Description: Implements streaming to and from a ByteStream for a custom data type.
Remarks: This class is used to implement streaming to and from a ByteStream for custom (user-defined) data types.
See Also: DDL File Syntax
 */
template <qInt SIZE> class LiteByteStream: public RootObject
{
public:
    LiteByteStream()
    {
        Init();
    }

    virtual ~LiteByteStream() {}

    void Clear()
    {
        m_Buffer.Clear();
        Init();
    }
    size_t GetLength() const
    {
        return m_Buffer.GetContentSize();
    }
    qBool SetLength(size_t uiLength)
    {
        if (uiLength > m_Buffer.GetMaxSize())
        {
            return false;
        }
        else
        {
            m_Buffer.SetContentSize(uiLength);
            return true;
        }
    }
    size_t GetMaxSize() const
    {
        return m_Buffer.GetMaxSize();
    }

    size_t GetPosition() const
    {
        return m_uiPosition;
    }

    qBool SetPosition(size_t uiPosition)
    {
        if (uiPosition >= GetLength())
        {
            return false;
        }
        else
        {
            m_uiPosition = uiPosition;
            return true;
        }
    }

    qBool AddPosition(size_t addPos)
    {
        return SetPosition(GetPosition() + addPos);
    }

    LiteBuffer<SIZE> *GetBuffer()
    {
        return &m_Buffer;
    }
    const LiteBuffer<SIZE> *GetBuffer() const
    {
        return &m_Buffer;
    }

    qBool ErrorHasOccured() const
    {
        return m_bErrorHasOccured;
    }
    void ClearErrorFlag()
    {
        m_bErrorHasOccured=false;
    }
    void SetErrorFlag()
    {
        m_bErrorHasOccured=true;
    }

    //  Similar to CanExtract, but logs an error if a request of uiNbBytes bytes
    //  overruns the buffer currently.
    //  Also, sets the error flag if it is the case.
    qBool ValidateBufferLimit(size_t uiNbBytes)
    {
        if (m_Buffer.GetContentSizeValue() - GetPosition() >= uiNbBytes)
        {
            return true;
        }
        else
        {
            QLOG(EventLog::Error, NEX_T("ByteStream limit is exceeded by request of ") << uiNbBytes
                 << NEX_T(" bytes. Length:") << m_Buffer.GetContentSizeValue()
                 << NEX_T(" Position: ") << GetPosition());

            SetErrorFlag();

            return false;
        }
    }

    qBool Append(const qByte* pBuffer, size_t uiNbBytes)
    {
        return m_Buffer.AppendData(pBuffer,uiNbBytes);
    }

    qBool Extract(qByte* pBuffer, size_t uiNbBytes)
    {
        qBool bResult = true;
        qUnsignedInt uiNbBytesToExtract = 0;

        if (! ValidateBufferLimit(uiNbBytes))
        {
            uiNbBytesToExtract = GetLength()-GetPosition();
            bResult = false;
        }
        else
        {
            uiNbBytesToExtract = uiNbBytes;
        }

        if (uiNbBytesToExtract > 0)
        {
            m_Buffer.CopyContent(pBuffer,uiNbBytesToExtract,m_uiPosition);
            m_uiPosition+=uiNbBytesToExtract;
        }
        return bResult;
    }

private:
    //n2967
    template<typename T> qBool ExtractPrimitive(T* ptr)
    {
        if (! ValidateBufferLimit(sizeof(T)))
        {
            *ptr = (T) 0;
            return false;
        }

        NexNetworkOrderToHostOrder<T>(m_Buffer.GetContentPtr()+m_uiPosition, ptr);
        m_uiPosition+=sizeof(T);
        return true;
    }

    template<typename T> qBool AppendPrimitive(const T v)
    {
        qByte temp[sizeof(qUnsignedInt64)];
        HostOrderToNexNetworkOrder<T>(v, temp);
        return m_Buffer.AppendData(temp, sizeof(T));
    }


    qBool ExtractPrimitive_qBool(qBool* ptr)
    {
        if (! ValidateBufferLimit(sizeof(qInt8)))
        {
            *ptr = false;
            return false;
        }

        *ptr = *(m_Buffer.GetContentPtr()+m_uiPosition)?true:false;
        m_uiPosition+=sizeof(qInt8);
        return true;
    }

    qBool AppendPrimitive_qBool(const qBool boolVal)
    {
        qInt8 encodedBool = boolVal?1:0;
        m_Buffer.AppendData(&encodedBool, sizeof(qInt8));
        return true;
    }
public:

    NEX_SIMPLE_TYPE_BYTE_STREAMING(qChar8);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qInt8);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qByte);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qInt16);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qUnsignedInt16);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qInt32);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qUnsignedInt32);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qFloat);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qDouble);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qInt64);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qUnsignedInt64);
#if !defined(NEX_LONG_IS_UINT64_T)
    NEX_SIMPLE_TYPE_BYTE_STREAMING(long);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(unsigned long);
#endif
    
    qBool Extract(qBool *pValue)
    {
        return ExtractPrimitive_qBool(pValue);
    }
    qBool Append(const qBool &refValue)
    {
        return AppendPrimitive_qBool(refValue);
    }
    qBool Extract(qBool &refValue)
    {
        return ExtractPrimitive_qBool(&refValue);
    }
    qBool Append(const qBool *refValue)
    {
        return AppendPrimitive(*refValue);
    }
    LiteByteStream& operator<<(const qBool &refValue)
    {
        Append(&refValue);
        return *this;
    }
    LiteByteStream& operator>>(qBool &refValue)
    {
        Extract(&refValue);
        return *this;
    }



#if defined (NEX_QCHAR16_IS_NOT_UINT16_T)
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qChar16);
#endif

    template<qInt TARGET_SIZE> qBool operator==(const LiteByteStream<TARGET_SIZE>& oStream) const
    {
        return m_Buffer == oStream.m_Buffer &&
               GetMaxSize() == oStream.GetMaxSize();
    }

    static void TraceBytes(qUnsignedInt64 uiTraceFlags, qByte* pBuffer, size_t uiSize)
    {
        (void)uiTraceFlags;
        qUnsignedInt32 uiPos=0;
        StringStream ssLine;
        ssLine << hex;
        while (uiPos<uiSize)
        {
            if ((uiPos > 0) && (((uiPos+1) % 8) == 0))
            {
                ssLine << pBuffer[uiPos] << NEX_T("\t");
                QTRACE(uiTraceFlags,ssLine);
                ssLine.Clear();
            }
            else
            {
                ssLine << pBuffer[uiPos] << NEX_T("\t");
            }
            uiPos++;
        }
        if ((uiPos > 0) && (((uiPos) % 16) != 0))
        {
            QTRACE(uiTraceFlags,ssLine);
        }
    }

    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, qBool bTraceBytes=false)
    {
        if (GetLength()==0)
        {
            QTRACE(uiTraceFlags,NEX_T("LiteByteStream: Empty."));
        }
        else
        {
            QTRACE(uiTraceFlags,NEX_T("LiteByteStream: MaxSize=") << GetMaxSize()
                   << NEX_T(" Length=") << GetLength()
                   << NEX_T(" Position=") << m_uiPosition
                   << NEX_T(" Error? ") << ErrorHasOccured());
        }

        if (bTraceBytes)
        {
            TraceBytes(uiTraceFlags, m_Buffer.GetContentPtr(), GetLength());
        }
    }

private:
    void Init()
    {
        m_uiPosition = 0;
        m_bErrorHasOccured = false;
    }

    qBool CanExtract(size_t uiBytesToExtract) const
    {
        return GetPosition()-GetMaxSize() >= uiBytesToExtract;
    }

protected:
    size_t m_uiPosition;
    LiteBuffer<SIZE> m_Buffer;
    qBool m_bErrorHasOccured;
};

#undef NEX_SIMPLE_TYPE_BYTE_STREAMING

END_NAMESPACE_NEX

