﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

class Buffer;

/*!
 @brief バッファの参照を管理するクラスです。

 バッファへのポインタを持ち、RefBufferオブジェクト間で共有します。<br>
 RefBuffer間のコピーで、バッファの参照カウンタをインクリメントします。バッファそのもののコピーは起きません。<br>
 RefBufferのデストラクタで、バッファの参照カウンタをデクリメントします。<br>
 RefBufferクラスを使用することで、バッファの参照カウンタの操作が不要となります。
 */
class RefBuffer: public RootObject
{
public:
    /*!
    @brief コンストラクタです。

    メンバの初期化のみを行います。バッファのポインタはNULLで初期化されます。
    */
    RefBuffer();
    /*!
    @brief コンストラクタです。

    引数のRefBufferオブジェクトとバッファを共有します。<br>
    参照するバッファの参照カウンタをインクリメントします。
    @param[in] refBuffer バッファを共有するRefBufferオブジェクトを指定します。
    */
    RefBuffer(const RefBuffer& refBuffer);

    /*!
    @brief デストラクタです。

    保持しているバッファの参照カウンタをデクリメントし、参照カウンタが0となる場合バッファを解放します。
    */
    ~RefBuffer();

    /*!
    @brief RefBufferで保持しているバッファへのポインタを取得します。
    @return RefBufferで保持しているバッファへのポインタを返します。
    */
    qByte* Get()
    {
        return m_pBufferPayload;
    }
    /*!
    @brief RefBufferで保持しているバッファへのポインタを取得します。
    @return RefBufferで保持しているバッファへのポインタ
    */
    const qByte* Get() const
    {
        return m_pBufferPayload;
    }

    /*!
    @brief RefBufferで保持しているバッファのサイズを取得します。
    @return RefBufferで保持しているバッファのサイズを返します。
    */
    size_t GetSize() const
    {
        return m_bufferPayloadSize;
    }

public: // operatorオーバーロード
    /*!
    @brief 代入するRefBufferオブジェクトとバッファを共有します。バッファの参照カウンタをインクリメントします。

    代入前に参照していたバッファがあった場合は、参照カウンタをデクリメントします。

    @param[in] refBuffer RefBufferオブジェクトを指定します。
    @return 自オブジェクトを返します。
    */
    RefBuffer& operator=(const RefBuffer& refBuffer);
    /*
    @brief 代入するBufferオブジェクトを管理します。Bufferオブジェクトの参照カウンタをインクリメントします。
    代入したBufferオブジェクトはRefBufferオブジェクトで管理するため、個別にqDeleteで解放しないでください。
    @param[in] buffer Bufferオブジェクトを指定します。
    @return 自オブジェクトを返します。
    */
    RefBuffer& operator=(Buffer& buffer);

    /*!
    @brief RefBufferで保持しているバッファの特定バイトを参照します。
    @param[in] pos バイト位置を指定します。
    @return RefBufferで保持しているバッファの指定バイトの参照を返します。
    */
    qByte& operator[](ptrdiff_t pos)
    {
        return *(m_pBufferPayload+pos);
    }
    /*!
    @brief RefBufferで保持しているバッファの特定バイトを参照します。
    @param[in] pos バイト位置を指定します。
    @return RefBufferで保持しているバッファの指定バイトの参照を返します。
    */
    const qByte& operator[](ptrdiff_t pos) const
    {
        return *(m_pBufferPayload+pos);
    }

private:
    /*
    @brief メンバの初期化を行います。
    */
    void Initialize();
    /*
    @brief 有効なBufferオブジェクトを保持している場合、管理から解放します。
    */
    void ReleaseBuffer();
private:
    // m_pBuffer->GetContentPtr()のキャッシュ
    qByte* m_pBufferPayload;
    // m_pBuffer->GetContentSize()のキャッシュ
    size_t m_bufferPayloadSize;
    // Bufferオブジェクトへのポインタ
    Buffer* m_pBuffer;
};

END_NAMESPACE_NEX

