﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#pragma once

#include <functional>
#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Platform/Core/AtomicValue.h>

BEGIN_NAMESPACE_NEX

/*!
 !private
 @brief 参照カウントと弱参照カウントを管理するクラスです。
 */
class SharedCounterBase: public RootObject
{
public:
    SharedCounterBase()
        : m_sharedCount(1)
        , m_weakCount(1)
    {
    }

    virtual ~SharedCounterBase()
    {
    }

    // 参照カウンタをインクリメントする。
    bool AcquireSharedRef()
    {
        // TODO: WeakPtrをマルチスレッドで利用するならここの0との比較からインクリメント処理までをアトミックに行うように修正する必要がある。
        if (m_sharedCount == 0)
        {
            // WeakPtr::lock()からSharedPtrを生成しようとしたときに参照カウントが0になっている場合がある。
            return false;
        }
        ++m_sharedCount;
        return true;
    }

    // 参照カウンタをデクリメントする。カウントが0の時に管理しているポインタを削除し、弱参照数をデクリメントする。
    void ReleaseSharedRef()
    {
        if (--m_sharedCount == 0)
        {
            DeletePointer();
            ReleaseWeakRef();
        }
    }

    // 弱参照カウンタをインクリメントする。
    void AcquireWeakRef()
    {
        ++m_weakCount;
    }

    // 弱参照カウンタをデクリメントする。カウントが0ならShared, Weak共に0ということになるので自身を削除する。
    void ReleaseWeakRef()
    {
        if (--m_weakCount == 0)
        {
            qDelete this;
        }
    }

    // 現在の参照カウントを返す。
    qInt32 GetCount() const
    {
        return m_sharedCount.Get();
    }

protected:
    // 管理しているポインタを削除する。
    virtual void DeletePointer() = 0;

    AtomicValue m_sharedCount;  // 参照カウント
    AtomicValue m_weakCount;    // 弱参照カウント
};

/*!
 !private
 @brief 参照カウンタによりポインタを管理するクラス
 SharedPtrとWeakPtrでこのオブジェクトを共有する。
 */
template<typename T>
class SharedCounterInterface : public SharedCounterBase
{
public:
    SharedCounterInterface(T* pointer)
        : SharedCounterBase()
        , m_pointer(pointer)
    {
    }

    virtual ~SharedCounterInterface()
    {
    }

    T* GetPointer() const
    {
        return m_pointer;
    }

protected:
    T* m_pointer;
};

template<typename T, typename U, typename TDeleter>
class SharedCounter : public SharedCounterInterface<T>
{
public:
    SharedCounter(T* pointer, TDeleter deleter)
        : SharedCounterInterface<T>(pointer)
        , m_deleter(deleter)
    {
    }

    virtual ~SharedCounter()
    {
    }

private:
    virtual void DeletePointer()
    {
        // SharedPtrのクラステンプレート型(T)ではなく、
        // 実際にSharedPtrのコンストラクタに渡された型(U)でdeleteする。(動的削除子の対応)
        m_deleter(static_cast<U*>(SharedCounterInterface<T>::m_pointer));
    };
    TDeleter m_deleter;
};



/*!
 !private
 @brief SharedPtrのデフォルトのdeleterです。
 */
template<typename T>
struct SharedPtrDefaultDeleter : public RootObject
{
    typedef T element_type;
    void operator()(T* pointer)
    {
#ifdef __clang__
#pragma clang diagnostic push
// SharedPtrは動的削除子に対応しているので、非virtualなデストラクタを持つクラスに対してdeleteしても安全。
// そのためその警告を抑制する。
#pragma clang diagnostic ignored "-Wdelete-non-virtual-dtor"
#endif
        qDelete pointer;
#ifdef __clang__
#pragma clang diagnostic pop
#endif
    }
};

/*!
 !private
 @brief SharedPtr向けの何もしないdeleterです。
 */
struct SharedPtrNullDeleter : public RootObject
{
    typedef void element_type;
    template<typename T>
    void operator()(T*)
    {
    }
};

// operator*の返値の型を定義
template<typename T>
struct DereferenceDefine
{
    typedef T& type;
};

// voidの場合はvoid&ではなくvoidに特殊化する(void&という表記は不正なので)
template<>
struct DereferenceDefine<void>
{
    typedef void type;
};

template<typename T>
class WeakPtr;

/*!
 !private
 @brief 参照カウンタ方式のスマートポインタです。std::shared_ptrの簡易版です。

 関数名はstd:shared_ptr準拠です。SharedPtrで利用できる関数はstd::shared_ptrでも利用できます。
 動的削除子には対応しています。
 テンプレート型Tにvoidを指定することで任意のポインタを保持でき、適切にdeleteされます。
 ただし、継承関係があってもSharedPtr<A>のオブジェクトをSharedPtr<B>やSharedPtr<void>に代入できません。
 (↑はstd::shared_ptrでは対応しているがSharedPtrでは非対応)
 デフォルトのdeleter(TDeleter)はqDeleteを使って削除します。
 make_shared/allocate_sharedには対応していません。
 コピーコンストラクタとデフォルトコンストラクタを除くコンストラクタおよびreset()でメモリを動的確保します。
 */
template<typename T>
class SharedPtr : public RootObject
{
    friend class WeakPtr<T>;
public:
    // ポインタを参照しない空のSharedPtrを作成する。
    // メモリの動的確保をせず、deleterも呼ばれない。
    SharedPtr()
        : m_pCounter(NULL)
    {
    }
    template<typename U>
    explicit SharedPtr(U* p)
        : m_pCounter(qNew SharedCounter<T, U, SharedPtrDefaultDeleter<U> >(p, SharedPtrDefaultDeleter<U>()))
    {
        // NULLを管理することは通常ないのでSYSTEMCHECKを入れておく。
        // NULLを明示的に指定すると内部でメモリアロケートが発生するので、デフォルトコンストラクタを使った方がメモリアロケートが起きず効率が良い。
        SYSTEMCHECK(p != NULL && "You initialized SharedPtr by NULL pointer."
                                 " It seems to be bug because managing NULL pointer is no use generally."
                                 " Please use empty SharedPtr generated by default constructer.");
    }

    template<typename U, typename TDeleter>
    SharedPtr(U* p, TDeleter deleter)
        : m_pCounter(qNew SharedCounter<T, U, TDeleter>(p, deleter))
    {
        // NULLを管理することは通常ないのでSYSTEMCHECKを入れておく。
        // NULLを明示的に指定すると内部でメモリアロケートが発生するので、デフォルトコンストラクタを使った方がメモリアロケートが起きず効率が良い。
        SYSTEMCHECK(p != NULL && "You initialized SharedPtr by NULL pointer."
                                 " It seems to be bug because managing NULL pointer is no use generally."
                                 " Please use empty SharedPtr generated by default constructer.");
    }

    // WeakPtrからSharedPtrを生成する
    SharedPtr(const WeakPtr<T>& wp)
        : m_pCounter(wp.m_pCounter)
    {
        if (m_pCounter)
        {
            if (!m_pCounter->AcquireSharedRef())
            {
                // もしAcquireSharedRef()できなかった場合(=SharedPtrの参照数が0ですでに目的のポインタが解放されている場合)
                // 空のSharedPtrを作る
                m_pCounter = NULL;
            }
        }
    }

    SharedPtr(const SharedPtr& rhs)
        : m_pCounter(NULL)
    {
        *this = rhs;
    };
    
    virtual ~SharedPtr()
    {
        if (m_pCounter)
        {
            m_pCounter->ReleaseSharedRef();
        }
    }

    SharedPtr& operator=(const SharedPtr& rhs)
    {
        if (m_pCounter == rhs.m_pCounter)
        {
            // 自己代入もしくはすでに同じポインタを参照しているならなにもしない
            return *this;
        }

        // 自身で管理しているカウンタをデクリメントしてから新しいカウンタをインクリメントする
        if (m_pCounter)
        {
            m_pCounter->ReleaseSharedRef();
        }
        m_pCounter = rhs.m_pCounter;
        if (m_pCounter)
        {
            m_pCounter->AcquireSharedRef();
        }
        return *this;
    }

    // 本来返値の型はT&にしたいところだが、Tがvoidの場合void&という型が不正になってしまうので、
    // DereferenceDefineを経由してvoidを特殊化する。
    // これによりTがvoidの場合は返値がvoidになる。
    typename DereferenceDefine<T>::type operator*() const
    {
        SYSTEMCHECK(m_pCounter != NULL);
        return *m_pCounter->GetPointer();
    }

    T* operator->() const
    {
        SYSTEMCHECK(m_pCounter != NULL);
        return m_pCounter->GetPointer();
    }

    T* operator[](std::ptrdiff_t i) const
    {
        SYSTEMCHECK(m_pCounter != NULL);
        return m_pCounter->GetPointer()[i];
    }

    T* get() const
    {
        if (!m_pCounter)
        {
            return NULL;
        }
        return m_pCounter->GetPointer();
    }

    bool unique() const
    {
        if (!m_pCounter)
        {
            return false;
        }
        return m_pCounter->GetCount() == 1;
    }
    
    qInt32 use_count() const
    {
        if (!m_pCounter)
        {
            return 0;
        }
        return m_pCounter->GetCount();
    }

    // ポインタを参照しないようにする。空のSharedPtrにする。
    void reset()
    {
        if (m_pCounter)
        {
            m_pCounter->ReleaseSharedRef();
        }
        m_pCounter = NULL;
    }
    
    // 他のポインタを参照する。
    template<typename U>
    void reset(U* p)
    {
        if (m_pCounter)
        {
            m_pCounter->ReleaseSharedRef();
        }
        m_pCounter = qNew SharedCounter<T, U, SharedPtrDefaultDeleter<U> >(p, SharedPtrDefaultDeleter<U>());
    }

    // 他のポインタを参照する。
    template<typename U, typename TDeleter>
    void reset(U* p, TDeleter deleter=TDeleter())
    {
        if (m_pCounter)
        {
            m_pCounter->ReleaseSharedRef();
        }
        m_pCounter = qNew SharedCounter<T, U, TDeleter>(p, deleter);
    }

    void swap(SharedPtr& rhs)
    {
        std::swap(m_pCounter, rhs.m_pCounter);
    }

    bool owner_before(const SharedPtr& rhs) const
    {
        return get() < rhs.get();
    }

    // Safe bool idiom
    // 意図しないboolへの変換を禁止した安全なboolへの変換
    // https://ja.wikibooks.org/wiki/More_C%2B%2B_Idioms/%E5%AE%89%E5%85%A8%E3%81%AA_bool%28Safe_bool%29
    // NULL以外のポインタへの参照をしていればboolへのキャストでtrueを返す。
    typedef void (SharedPtr::*bool_type)() const;
    void this_type_does_not_support_comparisons() const {}
    operator bool_type() const
    {
        return get() ? &SharedPtr::this_type_does_not_support_comparisons : 0;
    }
private:
    SharedCounterInterface<T>* m_pCounter;
};

template<class T, class U>
inline bool operator==(const SharedPtr<T>& lhs, const SharedPtr<U>& rhs)
{
    return lhs.get() == rhs.get();
}

template<class T, class U>
inline bool operator!=(const SharedPtr<T>& lhs, const SharedPtr<U>& rhs)
{
    return lhs.get() != rhs.get();
}

template<class T, class U>
inline bool operator<(const SharedPtr<T>& lhs, const SharedPtr<U>& rhs)
{
    return lhs.owner_before(rhs);
}

template<class T, class U>
inline bool swap(const SharedPtr<T>& lhs, const SharedPtr<U>& rhs)
{
    return lhs.swap(rhs);
}

// std::weak_ptrの簡易版です。現在は必要なアトミック操作が実装されていないため、マルチスレッドで利用できません。
template<typename T>
class WeakPtr : public RootObject
{
    friend class SharedPtr<T>;
public:
    WeakPtr()
        : m_pCounter(NULL)
    {
    }

    WeakPtr(const SharedPtr<T>& sp)
        : m_pCounter(sp.m_pCounter)
    {
        if (m_pCounter)
        {
            m_pCounter->AcquireWeakRef();
        }
    }

    WeakPtr(const WeakPtr& wp)
        : m_pCounter(wp.m_pCounter)
    {
        if (m_pCounter)
        {
            m_pCounter->AcquireWeakRef();
        }
    }

    ~WeakPtr()
    {
        if (m_pCounter)
        {
            m_pCounter->ReleaseWeakRef();
        }
    }

    WeakPtr& operator=(const WeakPtr& rhs)
    {
        if (m_pCounter == rhs.m_pCounter)
        {
            // 自己代入もしくはすでに同じポインタを参照しているならなにもしない
            return *this;
        }

        // 自身で管理しているカウンタをデクリメントしてから新しいカウンタをインクリメントする
        if (m_pCounter)
        {
            m_pCounter->ReleaseWeakRef();
        }
        m_pCounter = rhs.m_pCounter;
        if (m_pCounter)
        {
            m_pCounter->AcquireWeakRef();
        }
        return *this;
    }
    
    WeakPtr& operator=(const SharedPtr<T>& rhs)
    {
        if (m_pCounter == rhs.m_pCounter)
        {
            // すでに同じポインタを参照しているならなにもしない
            return *this;
        }

        // 自身で管理しているカウンタをデクリメントしてから新しいカウンタをインクリメントする
        if (m_pCounter)
        {
            m_pCounter->ReleaseWeakRef();
        }
        m_pCounter = rhs.m_pCounter;
        if (m_pCounter)
        {
            m_pCounter->AcquireWeakRef();
        }
        return *this;
    }

    SharedPtr<T> lock() const
    {
        SYSTEMCHECK(false && "Not implemented. SharedCounterBase::AcquireSharedRef() will cause race condition in multi threading environment.");
        return SharedPtr<T>(*this);
    }

    qInt32 use_count() const
    {
        if (!m_pCounter)
        {
            return 0;
        }
        return m_pCounter->GetCount();
    }
    
    void reset()
    {
        if (m_pCounter)
        {
            m_pCounter->ReleaseWeakRef();
        }
        m_pCounter = NULL;
    }
    
    bool expired() const
    {
        if (!m_pCounter)
        {
            return true;
        }
        return use_count() == 0;
    }
    
    bool owner_before(const WeakPtr& rhs) const
    {
        return lock().get() < rhs.lock().get();
    }
    
    bool owner_before(const SharedPtr<T>& rhs) const
    {
        return lock().get() < rhs.get();
    }

private:
    SharedCounterInterface<T>* m_pCounter;
};

template<class T, class U>
inline bool operator<(const WeakPtr<T>& lhs, const WeakPtr<U>& rhs)
{
    return lhs.owner_before(rhs);
}

template<class T, class U>
inline bool swap(const WeakPtr<T>& lhs, const WeakPtr<U>& rhs)
{
    return lhs.swap(rhs);
}

END_NAMESPACE_NEX
