﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Core/StepSequenceJob.h>

BEGIN_NAMESPACE_NEX

/*!
 *  @brief 一つのステップを持つジョブを実行するためのクラスです。
 *
 */
class SingleStepJob : public StepSequenceJob
{
public:
    /*!
     *  @brief コンストラクタです。
     *
     */
    SingleStepJob()
        : StepSequenceJob(Job::Core, NEX_T("SingleStepJob"))
        , m_AssociatedCallID(0)
    {}
    /*!
     *  @brief デストラクタです。
     *
     */
    virtual ~SingleStepJob(){}

    /*!
     *  @brief ジョブを開始します。
     *
     *  Scheduler にジョブを登録し、処理を開始します。
     *  登録されたジョブは、 DELAY_TIME_MSEC ミリ秒毎に SingleStepJob::OnStep() により実行状況がチェックされます。
     *
     *  @param[in,out] pContext ジョブの実行状況を管理するための情報です。
     *  @param[in] deleteOnCompletion ジョブが完了した時点で自身をdeleteするかどうか。
     *
     *  @retval     true 非同期処理の開始に成功。
     *  @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
     *                    これは他の非同期処理で使用中であることを示します。
     *                    プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool Start(CallContext* pContext, qBool deleteOnCompletion=false);

    /*!
     *  @brief ジョブの進行状況をチェックする関数です。オーバーライドしてジョブが完了した場合に true を返してください。
     *
     *  false 返すと SingleStepJob::DELAY_TIME_MSEC 後に再度本関数が呼び出されます。
     *  処理が完了して返値に true を指定するときにジョブの処理結果を result に格納してください。
     *
     *  @param[out] result 処理結果
     *
     *  @return 処理が完了したら true, 処理中の場合は false
     */
    virtual qBool OnStep(qResult* result) = 0;

    /*!
     *  @brief ジョブがキャンセルされた場合にコールバックされます。
     *
     *  キャンセルされたときに特別な処理が必要な場合はオーバーライドしてください。
     */
    virtual void OnCancelled()
    {
    }

    static const qUnsignedInt32 DELAY_TIME_MSEC = 16; //!< SingleStepJob::OnStep() が実行される間隔(ミリ秒)です。
private:
    void SingleStep();

    void CompleteJob(const qResult& rResult);

    qUnsignedInt32 m_AssociatedCallID;
};

END_NAMESPACE_NEX

