﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

// This class is based on the classes distributed with the book
// Practical Statecharts in C/C++, from Miro Samek.  The original code
// was refactored to fit more naturally into the system, but the
// algorithm and semantic were kept intact.

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Platform/Core/VirtualRootObject.h>

BEGIN_NAMESPACE_NEX

// This is based on Samek.
// Adds a validate step - were we can validate if the signal is valid
// or not.  Requires to validate DOOperations.
// Implemented as delegation to the TARGET.  To reduce ...
// Complex SIGNAL.

// TODO: Attach data to QEvent (derive a template from it?)
// ... or define two parts: the 'signal' and the data - as a template
// argument?

// For the QState casts to work properly, it is mandatory that the base
// root object has a virtual method.  This is why we derive from
// VirtualRootObject rather than RootObject.  If we don't do it,
// DuplicatedObject.cpp won't compile properly.

#define MAXIMUM_STATE_NESTING       8

#ifdef VMG_ENABLED
#pragma pack(push,16)
#endif

//n1589
#define NEX_DISABLE_DYNAMIC_STATE_TRANSITION 1

class StateMachine: public VirtualRootObject
{
public:
    // Some type definitions.

    // This one HAS to be a unsigned short, or else some of the bit
    // operations won't be working properly, I believe.
    // TODO: This might break as we port - I'm not sure.
    typedef unsigned short Signal;

    class QEvent: public RootObject
    {
    public:
        QEvent(): m_bRepeatEvent(false) {}
        virtual ~QEvent() {}

        virtual Signal GetSignal() const=0;
        qBool IsBuiltIn() const
        {
            return GetSignal()<FirstUserSignal;
        }

        // For convenience.
        void SetRepeatFlag() const
        {
            m_bRepeatEvent=true;
        }
        void ClearRepeatFlag() const
        {
            m_bRepeatEvent=false;
        }
        qBool IsRepeated() const
        {
            return m_bRepeatEvent;
        }

    private:
        mutable qBool m_bRepeatEvent;
    };

    // A QSimpleEvent is a QEvent that has nothing more than a
    // Signal.
    class QSimpleEvent: public QEvent
    {
    public:
        QSimpleEvent(Signal uiSignal): m_uiSignal(uiSignal) {}
        virtual ~QSimpleEvent() {}
        virtual Signal GetSignal() const
        {
            return m_uiSignal;
        }

    private:
        Signal m_uiSignal;
    };

    // We can't declare a State corresponding to the real method
    // signature - as it is recursive (methods returns a pointer to
    // a method that returns a pointer...)
    // The following two declarations are used to fake a
    // declaration.  It will need to be casted -
    typedef void (StateMachine::*QPseudoState)(const QEvent&);          // pseudostate
    typedef QPseudoState (StateMachine::*QState)(const QEvent&);              // state

    void InitialTransition();              // execute initial transition
    void DispatchEvent(const QEvent& e);
    qBool IsInState(QState state);

    typedef enum _BuiltInSignals
    {
        EmptySignal = 0,
        InitSignal = 1,
        EntrySignal = 2,
        ExitSignal = 3,
        FirstUserSignal = 4
    } BuiltInSignals;

protected:
    struct TransitionPath                                  // protected inner class Tran
    {
        TransitionPath();

        QState myChain[MAXIMUM_STATE_NESTING];
        // NOTE: This really must be a unsigned short (16 bits - if
        // not, the code will not work, I believe)
        unsigned short myActions;           // action mask (2-bits for action)
    };

    StateMachine(QPseudoState initial);
    virtual ~StateMachine();

    QPseudoState TopState(const QEvent&);

    QState GetCurrentState() const
    {
        return m_pfCurrentState;
    }

#if NEX_DISABLE_DYNAMIC_STATE_TRANSITION
#else
    // DynamicStateTransition are transitions were we we may go to
    // different states.
    void DynamicStateTransition(QState target);
#endif

    // StaticStateTransitions are transitions were we always go to
    // the same state.  In this case, we compute the path from the
    // current state to the target state only once - and it is
    // always valid.
    void StaticStateTransition(TransitionPath *t, QState target);

    // n2370:memo: don't call this function directly. use Q_INIT macro.
    void InitializeState(QState pfNewState)
    {
        m_pfCurrentState = pfNewState;
    }

#define Q_INIT(target_) InitializeState((QState)(target_))
#define Q_TRAN(target_) {static TransitionPath t_; StaticStateTransition(&t_, (QState)(target_));}

    // n2370:memo: this macro seems not to be used in current Quazal project.
#define Q_TRAN_DYN(target_) DynamicStateTransition((QState)(target_))

private:
    void TransitionPathSetup(TransitionPath *t, QState target);
    QState Trigger(QState pfState, unsigned short uiStandardEvent);
    QState m_pfCurrentState;
    /* This variable is used during a transition */
    QState m_pfSourceState;
};

#ifdef VMG_ENABLED
#pragma pack(pop)
#endif

// state-handler return type
typedef StateMachine::QPseudoState QSTATE;

END_NAMESPACE_NEX

