﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>


BEGIN_NAMESPACE_NEX

#define NEX_DECLARE_SYSCOMPONENT_METHODS(CLASS,PARENTCLASS)     \
    virtual void EnforceDeclareSysComponentMacro() {}   \
    static const qChar* type() {return NEX_T(#CLASS);}     \
    virtual const qChar* GetType() const {return type();}       \
    virtual qBool IsAKindOf(const qChar* szType) const {return (nex_tcscmp(type(),szType)==0) || PARENTCLASS::IsAKindOf(szType);}

class SystemComponent: public RefCountedObject
{
public:
    SystemComponent(const String& strName);
    virtual ~SystemComponent();

    void SetName(const String& strName);
    const String& GetName() const
    {
        return m_strName;
    }

    /*
     * Methods to build the SystemComponents hierarchy
     */

    static const qChar* type()
    {
        return NEX_T("SystemComponent");
    }
    virtual const qChar* GetType() const
    {
        return type();
    }
    virtual qBool IsAKindOf(const qChar* szType) const
    {
        return (nex_tcscmp(type(),szType)==0);
    }
    virtual void EnforceDeclareSysComponentMacro() =0;

    /*
     *
     * Methods related to the state of a component
     *
     *
     */

    typedef enum _State
    {
        Uninitialized       = 1<<0,
        Initializing        = 1<<1,
        Ready               = 1<<2,
        ReadyInUse          = 1<<3,
        Terminating         = 1<<4,
        TerminatingInUse    = 1<<5,
        Terminated          = 1<<6,
        Faulty              = 1<<7,
        Unknown             = 1<<8,
        HighestState        = 1<<8
    } State;

    static const qChar* GetStateString(State eValue);

    /*
     *
     * State access
     *
     */

    State GetState() const
    {
        return m_eState;
    }
    qUnsignedInt32 GetUseCount() const
    {
        return m_uiInUseCount;
    }

    /*
     *
     * Various methods
     *
     */

#ifdef NEX_TRACE_ENABLED    // n2370
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, qBool bCallDerivedClass=true) const;
#else
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, qBool bCallDerivedClass=true) const
    {
        (void)uiTraceFlags;
        (void)bCallDerivedClass;
    }
#endif  // NEX_TRACE_ENABLED

    State Initialize();
    State Terminate();
    void Failed()
    {
        SetState(Faulty);
    }

    qBool IsUninitialized() const
    {
        return GetState()==Uninitialized;
    }
    qBool IsInitializing() const
    {
        return GetState()==Initializing;
    }
    qBool IsReady() const
    {
        return GetState()==Ready || GetState()==ReadyInUse;
    }
    qBool IsTerminating() const
    {
        return GetState()==Terminating || GetState()==TerminatingInUse;
    }
    qBool IsTerminated() const
    {
        return GetState()==Terminated;
    }
    qBool IsFaulty() const
    {
        return GetState()==Faulty;
    }

    /*
     *
     * Methods related to the use of a component
     *
     *
     */

    class Use: public RootObject
    {
    public:
        Use(SystemComponent* pComponent, const qChar* szDescription=NULL);
        virtual ~Use();

        qBool IsAllowed()
        {
            return m_bIsAllowed;
        }

#if NEX_ENABLE_IMPROVE_DO_CORRUPT
        void SetAllowed(qBool o)
        {
            m_bIsAllowed = o;
        }
#endif
    private:
        /* Invalid */
        Use& operator=(const Use&);
        Use(const Use&);

    private:
        SystemComponent* m_pComponent;
        const qChar* m_szDescription;
        qBool m_bIsAllowed;
    };


    /*
     *
     * Low level state manipulation methods.  All state change goes
     * via one of these methods
     *
     */

public:

    qBool SetState(State eNewState, qBool bForced=false);

protected:
    friend class Use;

    qBool IncrementInUseCount(const qChar* szUsageDescription=NULL);
    void DecrementInUseCount(const qChar* szUsageDescription=NULL);


    /*
     *
     * Interface to derived classes.
     *
     *
     */

protected:

#ifdef NEX_TRACE_ENABLED    // n2370
    virtual void TraceImpl(qUnsignedInt64 /*uiFlags*/) const {}
#endif  // NEX_TRACE_ENABLED

    virtual void StateTransition(State /*eFinal*/) {}

    // Called after state transition from Uninitialized to
    // Initializing.  If it returns true, the state transition to Ready
    // occurs automatically.  If not, it becomes the
    // responsability of the DoWork/TestState methods to do it.
    virtual void OnInitialize()
    {
        TestState();
    }
    virtual void OnTerminate()
    {
        TestState();
    }
    virtual qBool BeginInitialization()
    {
        return true;
    }
    virtual qBool EndInitialization()
    {
        return true;
    }
    virtual qBool BeginTermination()
    {
        return true;
    }
    virtual qBool EndTermination()
    {
        return true;
    }

    virtual qBool ValidTransition(State eFinal);
    virtual qBool UseIsAllowed();

public:

    virtual State TestState()
    {
        return GetState();
    }
    virtual void DoWork() {}
    qResult WaitForTerminatedState(qUnsignedInt32 uiTimeout=WAIT_INFINITE_TIMEOUT);
    qResult WaitForReadyState(qUnsignedInt32 uiTimeout=WAIT_INFINITE_TIMEOUT);

    void SetParent(SystemComponent* pParent);
    SystemComponent* GetParent() const
    {
        return m_pParent;
    }

private:
    String m_strName;
    State m_eState;
    qUnsignedInt32 m_uiInUseCount;

    SystemComponent* m_pParent;
};

END_NAMESPACE_NEX

