﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>

#if NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION
#else
#include <OnlineCore/src/Platform/Core/String.h>
#endif  // NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION

#include <OnlineCore/src/Platform/Core/qChain.h>
#include <OnlineCore/src/Platform/Core/qList.h>

BEGIN_NAMESPACE_NEX

// group=SystemSettings

// Document the individual system settings in the .cpp files where they are
// created. See the ConnectionEncoding system setting documentation in the Core.h
// file for an example of how to document them. The SystemSettingsTemplate.html
// file $import s them into an html file as a group. The template and generated file
// are located in the UserDocs\RVOperations directory.

/*
Description: This class encapsulates one system setting data object.
Remarks: This class encapsulates a system setting data object and
provides a number of methods which allow you to access and change
its value.
*/
class SystemSetting: public RootObject
{
public:
    /* Description:
    Encapsulates one system setting data object.
    Parameters:
    strName: A reference to the system setting name.
    uiValue: An initial value for the system setting.
    */
#if NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION
    SystemSetting(const qChar* strName, qUnsignedInt32 uiValue);
#else  // NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION
    SystemSetting(const String&, qUnsignedInt32 uiValue);
#endif // NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION

    /* Description: Destroys the system setting
    */
    virtual ~SystemSetting();

    /* Description:
    Returns a reference to the name of the system setting.
    Parameters:
    m_strName: A reference to the system setting name.
    */
#if NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION
    const qChar* GetName() const
    {
        return m_strName;
    }
#else  // NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION
    const String& GetName() const
    {
        return m_strName;
    }
#endif // NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION

    /* Description:
    Returns the current value of the system setting.
    */
    qUnsignedInt32 GetValue() const
    {
        return m_uiValue;
    }

    /* Description:
    Check if the given system setting is mutable.
    */
    qBool CheckIfMutable() const;

    /* Description:
    Sets the value of the system setting.  This will return false
    if the SystemSetting is immutable.
    Parameters:
    uiValue: The value to set.
    */
    qBool SetValue(qUnsignedInt32 uiValue);

    // Description: Ensures that attempts to SetValue will return an error.
    // Remarks: If there is a point in the code beyond which changing the system
    // setting no longer has an impact (that is, it has been read
    // and will not be read again), then it is a good idea to set
    // it immutable, in which case, attempts to use SetValue will return false.
    void SetImmutable()
    {
        m_bIsImmutable = true;
    }

    // are the following secret??

    // {secret}
#if NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION
    static SystemSetting* Find(const qChar* strName);
#else  // NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION
    static SystemSetting* Find(const String& strName);
#endif // NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION

    // {secret}
    operator qUnsignedInt32() const
    {
        return m_uiValue;
    }

    // Link interface
    // {secret}
    CHAINABLE(SystemSetting);

    // {secret}
    static qChain<SystemSetting*>& GetList();

    // {secret}
    //Returns a list containing the names of all the SystemSettings in the list
#if NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION
    static qList<const qChar*> GetAllNames();
#else  // NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION
    static qList<String> GetAllNames();
#endif // NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION

private:
#if NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION
    const qChar* m_strName;
#else  // NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION
    String m_strName;
#endif // NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION

    qUnsignedInt32 m_uiValue;
    qBool m_bIsImmutable;
};

END_NAMESPACE_NEX

