﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include "STLExtDDF.h"

#ifdef WIN_FAMILY
#pragma warning(push, 1) /* STL compiles with level 4 warnings */
#endif

#include <bitset>

#include <string>
#include <OnlineCore/src/Platform/Core/qMap.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Transport/Interface/Message.h>

#ifdef WIN_FAMILY
#pragma warning( pop ) /* restore previous warning */
#endif

BEGIN_NAMESPACE_NEX

// The following three classes are used to integrate the bitset types (if
// the platform offers bitset support).

// The name of the class must be of the form DDLTYPE(TypeName) where
// TypeName is the type declared in the DDL.

class OSDK_API DDLTYPE(std_bitset8): public DDLType
{
public:
    // Whenever a std_bitset8 type is found in the DDL, $PRODUCTNAME$ will
    // generate a variable of type std_bitset8Type::Type. This typedef
    // maps the DDL type std_bitset8 to the C++ type std::bitset<8>.
    typedef std::bitset<8> CPPType;

    // This method is called by $PRODUCTNAME$ to add a variable of
    // type std_bitset8 to a message.
    static void Add(Message* pMessageToSend, const std::bitset<8>& oBitset);
    // This method is called by $PRODUCTNAME$ to extract a variable of
    // type std_bitset8 from a message.
    static void Extract(Message* pReceivedMessage, std::bitset<8>* pBitset);
};

class OSDK_API DDLTYPE(std_bitset16): public DDLType
{
public:
    // Maps the DDL type std_bitset16 to the C++ type std::bitset<16>.
    typedef std::bitset<16> CPPType;

    // Adds a variable of type std_bitset16 to a message.
    static void Add(Message* pMessageToSend, const std::bitset<16>& oBitset);
    // Extracts a variable of type std_bitset16 from a message.
    static void Extract(Message* pReceivedMessage, std::bitset<16>* pBitset);
};

class OSDK_API DDLTYPE(std_bitset32): public DDLType
{
public:
    // Maps the DDL type std_bitset32 to the C++ type std::bitset<32>.
    typedef std::bitset<32> CPPType;

    // Adds a variable of type std_bitset32 to a message.
    static void Add(Message* pMessageToSend, const std::bitset<32>& oBitset);
    // Extracts a variable of type std_bitset32 from a message.
    static void Extract(Message* pReceivedMessage, std::bitset<32>* pBitset);
};

// The following class is used to integrate the std_map type.
template < class DDLKey, class DDLValue >
class DDLTYPE(std_map) : public DDLType
{
public:
    typedef qMap<typename DDLKey::CPPType, typename DDLValue::CPPType> CPPType;

    static void Add(Message* pMessageToSend, const CPPType &uMap)
    {
        (*pMessageToSend) << (qUnsignedInt32)uMap.size();

        typename CPPType::const_iterator i;
        typename CPPType::const_iterator end = uMap.end();
        for( i = uMap.begin(); i != end; i++ )
        {
            DDLKey::Add( pMessageToSend, (*i).first );
            DDLValue::Add( pMessageToSend, (*i).second );
        }

    };
    static void Extract(Message *pReceivedMessage, CPPType *pUMap )
    {
        pUMap->clear();

        SYSTEMCHECK(pReceivedMessage != NULL);
        qUnsignedInt32 uiSize = 0;
        (*pReceivedMessage) >> uiSize;

        qUnsignedInt32 i;
        for(i = 0; i < uiSize; i++)
        {
            typename DDLKey::CPPType k;
            typename DDLValue::CPPType v;
            DDLKey::Extract( pReceivedMessage, &k );
            DDLValue::Extract( pReceivedMessage, &v );
            (*pUMap)[k] = v;
        }
    };

};

// The following class is used to integrate the std_vector type.
template < class DDLTemplateType >
class DDLTYPE(std_vector) : public DDLType
{
public:
    typedef qVector<typename DDLTemplateType::CPPType> CPPType;

    static void Add(Message* pMessageToSend, const CPPType &stdVector)
    {
        // 32bitと64bitでバイナリフォーマットを同一にするためsize_tからqUnsignedInt32にキャスト
        qUnsignedInt32 size = static_cast<qUnsignedInt32>(stdVector.size());
        (*pMessageToSend) << size;

        qUnsignedInt32 i;
        for( i = 0; i < size; i++ )
        {
            DDLTemplateType::Add( pMessageToSend, stdVector[i] );
        }
    };
    static void Extract(Message *pReceivedMessage, CPPType *pstdVector )
    {
        pstdVector->clear();

        SYSTEMCHECK(pReceivedMessage != NULL);

        qUnsignedInt32 uiSize = 0;
        (*pReceivedMessage) >> uiSize;
        pstdVector->reserve( uiSize );
        qUnsignedInt32 i;
        for(i = 0; i < uiSize; i++)
        {
            typename DDLTemplateType::CPPType v;
            DDLTemplateType::Extract( pReceivedMessage, &v );
            pstdVector->push_back( v );
        }
    };

};

// The following class is used to integrate the std_list type.
template < class DDLTemplateType >
class DDLTYPE(std_list) : public DDLType
{
public:
    typedef qList<typename DDLTemplateType::CPPType> CPPType;

    static void Add(Message* pMessageToSend, const CPPType &stdList)
    {
        qUnsignedInt32 size = (qUnsignedInt32)stdList.size();
        (*pMessageToSend) << size;

        typename CPPType::const_iterator i;
        typename CPPType::const_iterator end = stdList.end();
        for( i = stdList.begin(); i != end; i++ )
        {
            DDLTemplateType::Add( pMessageToSend, *i );
        }
    };
    static void Extract(Message *pReceivedMessage, CPPType *pstdList )
    {
        pstdList->clear();

        SYSTEMCHECK(pReceivedMessage != NULL);

        qUnsignedInt32 uiSize = 0;

        (*pReceivedMessage) >> uiSize;

        qUnsignedInt32 i;
        for(i = 0; i < uiSize; i++)
        {
            typename DDLTemplateType::CPPType v;
            DDLTemplateType::Extract( pReceivedMessage, &v );
            pstdList->push_back( v );
        }
    };
};

// The following class is used to integrate the std_string types. Integration
// of the three string instances of the string template exposed in the DDL is
// easily achieved via a template, with the three required typedefs to the
// appropriate template instance defined below.
template<class C> class StdStringGenericDDLType: public DDLType
{
public:
    typedef std::basic_string<C> CPPType;

    static void Add(Message* pMessageToSend, const std::basic_string<C>& oString)
    {
        qUnsignedInt16 uiLength=(qUnsignedInt16) oString.length();
        *pMessageToSend << uiLength;
        pMessageToSend->Append(oString.c_str(), uiLength);
    }
    static void Extract(Message* pReceivedMessage, std::basic_string<C>* pString)
    {
        SYSTEMCHECK(pReceivedMessage!=NULL);

        qUnsignedInt16 uiLength = 0;
        *pReceivedMessage >> uiLength;

        C *szNewString=qSpecialNewArray(C, uiLength+1);
        pReceivedMessage->Extract(szNewString, uiLength);
        szNewString[uiLength]=0;
        *pString = szNewString;
        qSpecialDeleteArray(szNewString);
    }
    static void ReadVariable(String* pstrValue, const CPPType& oString)
    {
        *pstrValue=oString.c_str();
    }
};

// This typedef is used to integrate the DDL type std_string. To integrate
// the std_string type, the class std_stringGenericType<qChar> is used.
typedef StdStringGenericDDLType<qChar> DDLTYPE(std_string);

typedef StdStringGenericDDLType<qChar16> DDLTYPE(std_char16string);

typedef StdStringGenericDDLType<qChar8> DDLTYPE(std_char8string);

END_NAMESPACE_NEX

