﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

class HttpConnection;

/*!
    @brief      データの送受信に使用するイベントリスナーです。

        本クラスは RootObject から継承されません。本クラスを継承したクラスで必要に応じて RootObject を継承してください。
*/
class HttpEventListener
{
public:
    /*!
        @brief      コンストラクタです。
    */
    HttpEventListener() {}

    /*!
        @brief      デストラクタです。
    */
    virtual ~HttpEventListener() {}

    /*!
        @brief      HTTP リクエストの準備をします

            HTTP リクエストを発行する前に呼びだされます。 connection に対してリクエストヘッダの設定などを行ってください。

        @param[in]  connection HTTP 設定を行うコネクションハンドル。
    */
    virtual void PrepareRequest(HttpConnection* connection)
    {
        (void)connection;
    }

    /*!
        @brief      POST するデータを分割して登録します

            HTTP メソッドに POST を指定し、 HttpConnection::UseChunkedPost() を呼び出した場合にこの関数が呼び出されます。
            本関数内で送信データを登録してください。

        @param[in]  connection 対象となるHTTPコネクションハンドル。
        @param[out] buffer アップロードするデータをここに書き込みます。
        @param[in]  bufferSize buffer に書き込める最大サイズ。
        @param[out] completed アップロードが完了したら true を代入、完了していなければfalseを代入もしくは未設定にしてください。true にすると、以後本関数は呼び出されません。

        @return buffer に書き込んだサイズ。0以上 bufferSize 以下の値になるようにしてください。0を返し続けることによりアップロード処理を一時中断できます。
    */
    virtual size_t PostChunkedBuffer(HttpConnection* connection, qUnsignedInt8* buffer, size_t bufferSize, qBool* completed)
    {
        (void)connection;
        (void)buffer;
        (void)bufferSize;

        QLOG(EventLog::Warning, NEX_T("HttpEventListener: PostChunkedBuffer is not handled. size: ") << bufferSize);
        SYSTEMCHECK(false);
        *completed = true;
        return 0;
    }

    /*!
        @brief      HTTP レスポンスヘッダの取得を完了したときに呼び出されます。

            本関数内で HTTP ステータスコードやレスポンスヘッダを取得してください。

        @param[in]  connection 対象となるHTTPコネクションハンドル。
        @param[in]  statusCode HTTP ステータスコード。
    */
    virtual void ProcessResponseHeader(HttpConnection* connection, qInt statusCode)
    {
        (void)connection;
        (void)statusCode;
    }

    /*!
        @brief      HTTP レスポンスボディを取得したときに呼び出されます。

            データがダウンロードされると、非同期処理が完了するまで本関数が複数回呼ばれます。
            ダウンロードの非同期処理が途中で失敗、キャンセルされた場合、受け取ったデータは完全ではありませんので注意してください。

        @param[in]  buffer ダウンロードされたデータ。
        @param[in]  bufferSize buffer のサイズ。

        @return データの受信処理が正常に完了したときは true を、完了できなかったときは false を返してください。
                true を返した場合はダウンロード処理が継続されます。false を返した場合は次回の本関数呼び出しで再度同じデータが渡されます。
                つまり、false を返し続けることによりダウンロード処理を一時中断できます。
    */
    virtual qBool ProcessResponse(const qUnsignedInt8* buffer, size_t bufferSize)
    {
        (void)buffer;
        (void)bufferSize;

        QLOG(EventLog::Warning, NEX_T("HttpEventListener: ProcessResponse is not handled. size: ") << bufferSize);
        SYSTEMCHECK(false);
        return true;
    }
};

/*!
    @brief      HTTPセッションの開始と終了を通知するイベントリスナーです。

        本クラスは RootObject から継承されません。本クラスを継承したクラスで必要に応じて RootObject を継承してください。
*/
class HttpSessionEventListener
{
public:
    /*!
        @brief      コンストラクタです。
    */
    HttpSessionEventListener() {}

    /*!
        @brief      デストラクタです。
    */
    virtual ~HttpSessionEventListener() {}

    /*!
        @brief      HTTPセッションの開始直前に通知されます。
    */
    virtual void Begin() {};

    /*!
        @brief      HTTPセッションの終了直後に通知されます。
    */
    virtual void End() {};
};

END_NAMESPACE_NEX

