﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/MutexPrimitive.h>

#if defined(CAFE_TARGET)
#include <cafe/os.h>
#endif
#if defined(NEX_C11_ATOMIC_SUPPORT)
#if defined(__GNUC__) && __GNUC__ == 4 && __GNUC_MINOR__ == 4
// GCC4.4ではインクルードファイル名が通常とは違う
#include <cstdatomic>
#else
#include <atomic>
#endif
#endif
BEGIN_NAMESPACE_NEX

class AtomicValue: public RootObject
{
public:
    inline AtomicValue();
    inline AtomicValue(qInt32 uiValue);
    inline AtomicValue(const AtomicValue &v);
    ~AtomicValue() {}

    inline AtomicValue &operator=(const AtomicValue &v );

    inline qInt32 Get() const;
    inline qInt32 TestAndSet()
    {
        return GetAndSet(1);
    }
    inline qInt32 operator=(qInt32 uiValue)
    {
        Set(uiValue);
        return uiValue;
    }
    inline operator qInt32() const
    {
        return Get();
    }


    inline void Set(qInt32 uiValue);
    inline qInt32 GetAndSet(qInt32 uiValue);
    // Prefix form...
    inline qInt32 operator++();
    // Postfix form
    inline qInt32 operator++(qInt iDummy);
    // Prefix form...
    inline qInt32 operator--();
    // Postfix form
    inline qInt32 operator--(qInt iDummy);

    inline qInt32 GetAndInc(qInt32 uiValue);
    inline qInt32 IncAndGet(qInt32 uiValue);

private:
#if defined(NEX_C11_ATOMIC_SUPPORT)
    std::atomic_int m_value;
#elif defined(CAFE_TARGET)
    volatile OSAtomicVar m_value;
#elif defined(CTR_TARGET)
    volatile qInt32 m_value;
#else
    qInt32 m_value;
    mutable MutexPrimitive m_mp;
#endif
};
#if defined(NEX_C11_ATOMIC_SUPPORT)
AtomicValue::AtomicValue() { m_value = 0; }
AtomicValue::AtomicValue(qInt32 uiValue) { m_value=uiValue; }
AtomicValue::AtomicValue(const AtomicValue &v)
    : RootObject(v)
{
    m_value = v.Get();
}

qInt32 AtomicValue::Get() const
{
    return m_value.load();
}

AtomicValue &AtomicValue::operator=(const AtomicValue &v )
{
    m_value.store(v.Get());
    return *this;
}


void AtomicValue::Set(qInt32 uiValue)
{
    m_value.store(uiValue);
}

qInt32 AtomicValue::GetAndSet(qInt32 uiValue)
{
    return m_value.exchange(uiValue);
}

qInt32 AtomicValue::GetAndInc(qInt32 add_val)
{
    qInt32 new_val;
    qInt32 orig_val = m_value.load();

    while(true)
    {
        new_val = orig_val + add_val;
        if (m_value.compare_exchange_weak(orig_val, new_val))
        {
            break;
        }
    }

    return orig_val;
}

qInt32 AtomicValue::IncAndGet(qInt32 add_val)
{
    qInt32 new_val;
    qInt32 orig_val = m_value.load();

    while(true)
    {
        new_val = orig_val + add_val;
        if (m_value.compare_exchange_weak(orig_val, new_val))
        {
            break;
        }
    }

    return new_val;
}

qInt32 AtomicValue::operator++()
{
    return IncAndGet(1);
}

qInt32 AtomicValue::operator++(qInt iDummy)
{
    DUMMY_PARAM(iDummy);
    return GetAndInc(1);
}

qInt32 AtomicValue::operator--()
{
    return IncAndGet(-1);
}

qInt32 AtomicValue::operator--(qInt iDummy)
{
    DUMMY_PARAM(iDummy);
    return GetAndInc(-1);
}
#elif defined(CAFE_TARGET)

AtomicValue::AtomicValue()
{
    m_value.u.s32 = 0;
}
AtomicValue::AtomicValue(qInt32 uiValue)
{
    m_value.u.s32 = uiValue;
}
AtomicValue::AtomicValue(const AtomicValue &v) : RootObject(v)
{
    m_value.u.s32 = v.Get();
}

AtomicValue &AtomicValue::operator=(const AtomicValue &v )
{
    m_value.u.s32 = v.Get();
    return *this;
}

qInt32 AtomicValue::Get() const
{
    return m_value.u.s32;
}

void AtomicValue::Set(qInt32 uiValue)
{
    m_value.u.s32 = uiValue;
}
qInt32 AtomicValue::GetAndSet(qInt32 uiValue)
{
    return OSSwapAtomic(&m_value, uiValue);
}

qInt32 AtomicValue::GetAndInc(qInt32 uiValue)
{
    return OSAddAtomic(&m_value, uiValue);
}

qInt32 AtomicValue::IncAndGet(qInt32 uiValue)
{
    //from OSAddAtomic
    s32 orig_val;
    s32 new_val;

    while(1)
    {
        orig_val = (s32)__LWARX((u32*)&m_value, 0);
        new_val = orig_val + uiValue;
        __DCBST(0, (u32)&m_value);
        if(__STWCX((u32*)&m_value, 0, new_val))
        {
            break;
        }
    }

    return new_val;
}

// Prefix form...
qInt32 AtomicValue::operator++()
{
    return IncAndGet(1);
}

// Postfix form
qInt32 AtomicValue::operator++(qInt iDummy)
{
    return OSIncAtomic(&m_value);
}
// Prefix form...
qInt32 AtomicValue::operator--()
{
    return IncAndGet(-1);
}
// Postfix form
qInt32 AtomicValue::operator--(qInt iDummy)
{
    return OSDecAtomic(&m_value);
}

#elif defined(CTR_TARGET)

AtomicValue::AtomicValue() : m_value(0) {}
AtomicValue::AtomicValue(qInt32 uiValue) : m_value(uiValue) {}
AtomicValue::AtomicValue(const AtomicValue &v)
    : RootObject(v),
      m_value(v.Get())
{}

qInt32 AtomicValue::Get() const
{
    return m_value;
}

AtomicValue &AtomicValue::operator=(const AtomicValue &v )
{
    m_value = v.Get();
    return *this;
}


void AtomicValue::Set(qInt32 uiValue)
{
    m_value = uiValue;
}

qInt32 AtomicValue::GetAndSet(qInt32 uiValue)
{
    register qInt32 orig_val;
    register qInt32 set_val = uiValue;

    while(true)
    {
        orig_val = static_cast<qInt32>(__ldrex(&m_value));
        if (__strex(set_val, &m_value) == 0)
        {
            break;
        }
    }


    return orig_val;
}

qInt32 AtomicValue::GetAndInc(qInt32 uiValue)
{
    register qInt32 orig_val;
    register qInt32 new_val;
    register qInt32 add_val = uiValue;

    while(true)
    {
        orig_val = static_cast<qInt32>(__ldrex(&m_value));
        new_val = orig_val + add_val;
        if (__strex(new_val, &m_value) == 0)
        {
            break;
        }
    }

    return orig_val;
}

qInt32 AtomicValue::IncAndGet(qInt32 uiValue)
{
    register qInt32 orig_val;
    register qInt32 new_val;
    register qInt32 add_val = uiValue;

    while(true)
    {
        orig_val = static_cast<qInt32>(__ldrex(&m_value));
        new_val = orig_val + add_val;
        if (__strex(new_val, &m_value) == 0)
        {
            break;
        }
    }

    return new_val;
}

qInt32 AtomicValue::operator++()
{
    return IncAndGet(1);
}

qInt32 AtomicValue::operator++(qInt iDummy)
{
    DUMMY_PARAM(iDummy);
    return GetAndInc(1);
}

qInt32 AtomicValue::operator--()
{
    return IncAndGet(-1);
}

qInt32 AtomicValue::operator--(qInt iDummy)
{
    DUMMY_PARAM(iDummy);
    return GetAndInc(-1);
}

#else

AtomicValue::AtomicValue() : m_value(0) {}
AtomicValue::AtomicValue(qInt32 uiValue) : m_value(uiValue) {}
AtomicValue::AtomicValue(const AtomicValue &v)
    : RootObject(v),
      m_value(v.Get())
{}

qInt32 AtomicValue::Get() const
{
    return m_value;
}

AtomicValue &AtomicValue::operator=(const AtomicValue &v )
{
    m_value = v.Get();
    return *this;
}

void AtomicValue::Set(qInt32 uiValue)
{
    m_mp.ForceEnter();
    m_value=uiValue;
    m_mp.ForceLeave();
}
qInt32 AtomicValue::GetAndSet(qInt32 uiValue)
{
    m_mp.ForceEnter();
    qInt32 uiResult=m_value;
    m_value=uiValue;
    m_mp.ForceLeave();
    return uiResult;
}
// Prefix form...
qInt32 AtomicValue::operator++()
{
    m_mp.ForceEnter();
    qInt32 uiResult=++m_value;
    m_mp.ForceLeave();
    return uiResult;
}
// Postfix form
qInt32 AtomicValue::operator++(qInt iDummy)
{
    DUMMY_PARAM(iDummy);
    m_mp.ForceEnter();
    qInt32 uiResult=m_value++;
    m_mp.ForceLeave();
    return uiResult;
}
// Prefix form...
qInt32 AtomicValue::operator--()
{
    m_mp.ForceEnter();
    qInt32 uiResult=--m_value;
    m_mp.ForceLeave();
    return uiResult;
}
// Postfix form
qInt32 AtomicValue::operator--(qInt iDummy)
{
    DUMMY_PARAM(iDummy);
    m_mp.ForceEnter();
    qInt32 uiResult=m_value--;
    m_mp.ForceLeave();
    return uiResult;
}

qInt32 AtomicValue::GetAndInc(qInt32 uiValue)
{
    m_mp.ForceEnter();
    qInt32 uiResult=m_value;
    m_value+=uiValue;
    m_mp.ForceLeave();
    return uiResult;
}

qInt32 AtomicValue::IncAndGet(qInt32 uiValue)
{
    m_mp.ForceEnter();
    m_value+=uiValue;
    qInt32 uiResult=m_value;
    m_mp.ForceLeave();
    return uiResult;
}
#endif


END_NAMESPACE_NEX


