﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/LogDevice.h>

#ifdef CTR_TARGET

#include <nn/os.h>

BEGIN_NAMESPACE_NEX

//group=Log Devices

/*!
  @internal
  @brief    このクラスは、デバッガソフトウエアとの通信を最小に抑えて通信するためのログデバイスクラスです。
            NN_LOGを大量にコールするとかなりパフォーマンスが落ち込むので、それを回避するために用意されました。
            使用するためにはキャッシュ用のバッファを指定する必要があります。

  @note     Initializeを呼ばないとLogDeviceConsoleと同じ挙動になります。

  @todo     いろいろ問題が発生したのでスケジューラーは利用していません。ネイティブスレッドでやっています。
*/
class CachedLogDevice : public LogDevice
{
public:
    CachedLogDevice();
    CachedLogDevice( char *pCacheBuffer, qInt nBufferSize );
    virtual ~CachedLogDevice();

    void Initialize( char *pCacheBuffer, qInt nBufferSize );

    virtual void Output(const qChar *pLog);

    void SetCacheConut( qInt cacheCount )
    {
        nn::os::CriticalSection::ScopedLock lock( m_csBufferLocker );
        m_nCacheCount = cacheCount;
    }
    void SetSleepMilliSec( qInt sleep )
    {
        nn::os::CriticalSection::ScopedLock lock( m_csBufferLocker );
        m_nWatcherSleep = sleep;
    }

private:
    static void WatchThread( CachedLogDevice* pCachedLogDevice );

    enum
    {
        // デフォルト キャッシュ回数
        CACHEDLOG_DEFAULT_COUNT = 8,

        // デフォルト 監視スレッド スリープ(ミリ秒)
        CACHEDLOG_DEFAULT_SLEEP = 100,

        // 監視スレッドのスタックサイズ
        CACHEDLOG_WATCHER_STACK = 4 * 1024
    };

private:
    //! ログを溜め込むためのキャッシュへのポインタ(デバッガへの出力がchar*しか受け付けないのでqCharは使わない)
    char* m_pCacheBuffer;

    //! ログを溜め込むキャッシュのサイズ(バイト)
    qInt m_nBufferSize;

    //! ログを何回受け取ったらデバッガに吐き出すか。デフォルト8回。
    qInt m_nCacheCount;

    //! ログバッファの中にどのぐらいデータが書き込まれているのか。
    qInt m_nFilledSize;

    //! 現在ログ出力命令が何回行われたか
    qInt m_nCalledCount;

    //! スリープする時間(ミリ秒)
    qInt m_nWatcherSleep;

    //! ログキャッシュ監視用 Horizonネイティブスレッド(メインスレッドよりも優先度は低い)
    nn::os::Thread m_threadWatcher;

    //! キャッシュへ書き込むための Horizonネイティブクリティカルセクション
    nn::os::CriticalSection m_csBufferLocker;

    //! オブジェクトが破棄されようとしている最中かどうかを示すフラグ
    bool m_bIsFinalizing;
};

END_NAMESPACE_NEX

#endif  // CTR_TARGET
