﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/LockMasks.h>
#include <OnlineCore/src/Platform/Core/ForcedCriticalSection.h>
#include <OnlineCore/src/Platform/Core/AtomicValue.h>

BEGIN_NAMESPACE_NEX

/*!
 * @internal
 * @brief スレッド間の同期を取るクラスです
 *
 * 各種処理系のスレッド同期機構の違いを隠ぺいしています。
 *
 */
class Condition: public RootObject
{

    class Attribute;

public:
    /*!
     * @internal
     * @brief コンストラクタです。
     *
     * @param isAll Notifiy()で、ひとつのスレッドを再開されるか、すべてのスレッドを再開させるか設定します。
     *
     */
    Condition(qBool isAll = false);

    /*!
     * @internal
     * @brief デストラクタです。
     *
     * 非virtualです。
     *
     */
    ~Condition();

    /*!
     * @internal
     * @brief 待機しているスレッドの実行を再開させます
     * コンストラクタで、isAll = falseのときは、
     * 複数のスレッドが待機している場合には、どれが選択されるかわかりません
     * また、ひとつも待機していない場合には、次回待機時に即座に再開させるかは、処理系依存です。
     * コンストラクタで、isAll = trueのときは、すべての待機スレッドが解除されます。
     * 
     *
     */
    void Notify();


    /*!
     * @internal
     * @brief 待機しているスレッドをすべて再開させます
     *
     * 待機しているスレッドをすべて再開させます。
     * 処理系によっては、Notifyとの競合時に問題となるため、pthread_cond_broadcastと同様ではありません。
     *
     */
    void CancelAll();

    /*!
     * @internal
     * @brief スレッドを待機させます
     *
     * NonReentrantableForcedCriticalSectionのアンロックと同時に待機します。待機が完了後、NonReentrantableForcedCriticalSectionのロックを取得します。
     * Cafeでは、uiTimeoutは、WAIT_INFINITE_TIMEOUT以外設定できません。
     * ForcedCriticalSectionは、待機させるスレッド間で共有させる必要があります。
     *
     */
    qBool WaitForEvent(NonReentrantableForcedCriticalSection *cs, qUnsignedInt32 uiTimeout = WAIT_INFINITE_TIMEOUT);

    qInt GetRefCount()
    {
        return m_refCount;
    }

private:

    void ResetEvent();

    qInt IncRefCount()
    {
        return ++m_refCount;
    }

    qInt DecRefCount()
    {
        return --m_refCount;
    }

    Attribute*  m_pAttribute;
    AtomicValue m_refCount;
    qBool       m_isAll;
};

/*!
 * @internal
 * @brief スレッド間の同期を取るクラスです
 *
 * エミュレートによりConditionと同様な動作をします。ただし、CPUを多く使います。
 * 以下のようなケースでこちらを使用してください。
 *
 * ・CafeでCondition::WaitForEvent()にタイムアウトを指定したいとき。(SDKにAPIがないため実装されていない)
 *
 */
class EmulatedCondition: public RootObject
{

    class Attribute;

public:
    /*!
     * @internal
     * @brief コンストラクタです。
     *
     * @param isAll Notifiy()で、ひとつのスレッドを再開されるか、すべてのスレッドを再開させるか設定します。
     *
     */
    EmulatedCondition(qBool isAll = false);

    /*!
     * @internal
     * @brief デストラクタです。
     *
     * 非virtualです。
     *
     */
    ~EmulatedCondition();

    /*!
     * @internal
     * @brief 待機しているスレッドの実行を再開させます
     * コンストラクタで、isAll = falseのときは、
     * 複数のスレッドが待機している場合には、どれが選択されるかわかりません
     * また、ひとつも待機していない場合には、次回待機時に即座に再開されます。
     *
     * コンストラクタで、isAll = trueのときは、すべての待機スレッドが解除されます。
     *
     */
    void Notify();


    /*!
     * @internal
     * @brief 待機しているスレッドをすべて再開させます
     *
     * 待機しているスレッドをすべて再開させます。
     *
     */
    void CancelAll();

    /*!
     * @internal
     * @brief スレッドを待機させます
     *
     * NonReentrantableForcedCriticalSectionのアンロックと同時に待機します。待機が完了後、NonReentrantableForcedCriticalSectionのロックを取得します。
     * ForcedCriticalSectionは、待機させるスレッド間で共有させる必要があります。
     *
     */
    qBool WaitForEvent(NonReentrantableForcedCriticalSection *cs, qUnsignedInt32 uiTimeout = WAIT_INFINITE_TIMEOUT);

    qInt GetRefCount()
    {
        return m_refCount;
    }

private:

    void ResetEvent();

    qInt IncRefCount()
    {
        return ++m_refCount;
    }

    qInt DecRefCount()
    {
        return --m_refCount;
    }

    Attribute*  m_pAttribute;
    AtomicValue m_refCount;
    qBool       m_isAll;
};

END_NAMESPACE_NEX


