﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

// <desc: Class that abstracts the notion of locks for the system>

#pragma once


#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/ObjectThread.h>
#include <OnlineCore/src/Platform/Core/LockMasks.h>
#include <OnlineCore/src/Platform/Core/MutexPrimitive.h>
#include <OnlineCore/src/Platform/Core/LockChecker.h>
#include <OnlineCore/src/Platform/Core/AtomicValue.h>
#include <OnlineCore/src/Platform/Core/NonCopyable.h>

BEGIN_NAMESPACE_NEX

// Group=Miscellaneous Classes

/*
Description: Creates and accesses critical sections.
Remarks: If you use multiple threads in your game then you will most likely need to protect certain
sections of code, by ensuring that they are only accessed by one thread at a time, so that the code
is executed correctly. One of the easiest ways is via the CriticalSection class. This class guarantees
that between the calls to Enter and Leave no state changes will occur in the system.

$ifdef RENDEZVOUS$
Critical sections are used internally by Rendez-Vous to ensure that all system callbacks are only called
in one thread at any given time and that no state changes occur during execution of the callback.
$endif$
$ifdef NETZ$
Critical sections are used internally by NetZ to ensure that all system callbacks, in particular
OperationBegin and OperationEnd, are only called in one thread at any given time and that no state changes
occur during execution of the callback.
$endif$

To avoid accidentally permanently locking a CriticalSection you can use the ScopedCS class. Refer to that
class for more details.
See Also: ScopedCS
*/

/*!
@brief    クリティカルセクションクラスです。

このクラスは、 @ref Enter() および @ref Leave() の呼び出し区間で、一つのスレッドのみが処理を行うことを保証します。
誤って CriticalSection を永久的にロックすることを避けるため、 @ref ScopedCS クラスを使用することができます。

スレッドセーフなスレッドモードを使用している場合のみ CriticalSection が有効になります。
スレッドセーフでないスレッドモードを使用している場合は CriticalSection::Enter() は何も行いません。

Scheduler::SystemLock() で取得した CriticalSection を使用すると NEX の内部処理を一時停止でき、
その間システムの変更が発生しないことを保証できます。

@see     ScopedCS
*/
class CriticalSection: public RootObject
{
    class Attributes;

    // Group=Methods
public:

#ifdef FOR_DOC_PURPOSE
    /*
    Description: Creates a CriticalSection object.
    Remarks: Assigning a critical section alters neither object.
    */

    /*!
     @brief    コンストラクタです。
     */
    CriticalSection();
#endif
    // {secret}
    CriticalSection(qUnsignedInt32 uiLockGroup = LockMasks::Default, qBool reentrantable=true);
    /*!
     @brief    デストラクタです。
     */
    virtual ~CriticalSection();

    // Copy constructing a critical section creates a new object.
    CriticalSection(const CriticalSection &cs);

    /*
    Description:  Assigns a reference to the current critical section.
    Returns: A reference to the current critical section.
    Remarks: Assigning a critical section alters neither object.
    */
    CriticalSection& operator=(const CriticalSection &cs);

    /*
    Description: Enters a critical section.
    Remarks: Between the calls to Enter and Leave, it is guaranteed that no state changes will occur.
    The thread that calls this method owns the critical section until it calls Leave. Any other thread
    that tries to Enter this CriticalSection will block until the current thread has left the critical
    section.
    See Also: ScopedCS::ScopedCS
    */

    /*!
    @brief    クリティカルセクションに入ります。

    このメソッドを呼び出すスレッドは、 @ref Leave() を呼び出すまでクリティカルセクションを所有します。
    すでに他のスレッドがクリティカルセクションに入っている場合はそのスレッドがクリティカルセクションを出るまで
    現在のスレッドがブロックします。

    @see     ScopedCS::ScopedCS
    */
    virtual void Enter()
    {
        if(!MutexPrimitive::IsANoOp())
        {
            EnterImpl();
        }
        ProcessAfterPrimitiveEnter();
    }

    /*!
    @brief    クリティカルセクションに入るのを試みます。

    他のスレッドがすでにクリティカルセクションを所有していない場合にクリティカルセクションに入ります。
    このメソッドを呼び出すスレッドは、 @ref Leave() を呼び出すまでクリティカルセクションを所有します。
    すでに他のスレッドがクリティカルセクションに入っている場合は
    本関数は即座に false を返し、クリティカルセクションに入りません。
    スレッドセーフでないスレッドモードを使用している場合は常に true を返します。

    @retval true クリティカルセクションへ入れた。
    @retval false クリティカルセクションへ入れなかった。
    */
    virtual qBool TryEnter()
    {
        if(!MutexPrimitive::IsANoOp())
        {
            if (!TryEnterImpl())
            {
                return false;
            }
        }
        ProcessAfterPrimitiveEnter();
        return true;
    }

    /*
    Description: Leaves a critical section.
    Remarks: Between the calls to Enter and Leave, it is guaranteed that no state changes will occur.
    See Also: ScopedCS::~ScopedCS
    */

    /*!
    @brief    クリティカルセクションを出ます。

     @ref Enter()  および Leave() への呼び出し間で、状態変更が発生しないことが保証されています。

    @see     ScopedCS::~ScopedCS
    */
    virtual void Leave()
    {
        ProcessBeforePrimitiveLeave();
        if(!MutexPrimitive::IsANoOp())
        {
            LeaveImpl();
        }
    }

    // ForcedCriticalSectionならtrueを返す。
    virtual qBool IsForce() const { return false; }

    // 再入可能なクリティカルセクションかどうかを返す。
    // 以下のように返される。
    // CriticalSection:                       true
    // ForcedCriticalSection:                 true
    // NonReentrantableForcedCriticalSection: false
    virtual qBool IsReentrantable() const { return true; }

    // 強制ロック,アンロック
    void ForceEnter()
    {
        m_oMutex.ForceEnter();
        m_oLockChecker.EndEnter();
        ProcessAfterPrimitiveEnter();
    }

    qBool ForceTryEnter()
    {
        if (!m_oMutex.ForceTryEnter())
        {
            return false;
        }
        m_oLockChecker.EndEnter();
        ProcessAfterPrimitiveEnter();
        return true;
    }

    void ForceLeave()
    {
        ProcessBeforePrimitiveLeave();
        m_oLockChecker.BeginLeave();
        m_oMutex.ForceLeave();
    }

    qBool HasOwnership() const;
    qBool HasNotOwnership() const;
    qBool ValidateOwnership(qBool bCurrentThread) const;
#ifdef NEX_LOCK_DEBUG_ENABLED
    void Trace(qUnsignedInt64 uiTraceFlags);
#endif

#ifdef NEX_LOCK_DEBUG_ENABLED
    //{secret}
    ThreadID GetThreadOwner() const
    {
        return m_iThreadOwner;
    }
    qInt32 GetCount() const
    {
        return m_iCounter;
    }
#endif

    /*!
    @brief    スレッド競合をチェックする機能を無効化/有効化します。

        Debug 及び Develop ビルドでは NEX のスレッドモードがスレッドアンセーフとして設定されている場合に、
        アプリケーションが NEX の API を並列に呼び出していないかをチェックしています。
        この関数ではその機能を一時的に無効化/有効化します。
        この機能を無効化するのは非常に危険です。テスト目的でのみ機能を無効化できます。

    @param[in] enable true を指定すると機能を有効化し、false を指定すると無効化します。
    */
    static void SetEnableDetectRaceCondition(qBool enable)
    {
#ifdef NEX_LOCK_DEBUG_ENABLED
        QLOG(EventLog::Warning, NEX_T("**********************************************************************************************"));
        QLOG(EventLog::Warning, NEX_T("CriticalSection::SetEnableDetectRaceCondition(") << (enable ? NEX_T("true") : NEX_T("false")) << NEX_T(") was called."));
        if (!enable)
        {
            QLOG(EventLog::Warning, NEX_T("Race condition detector is DISABLED. You are in dangerous state."));
            QLOG(EventLog::Warning, NEX_T("You can use this function for test only."));
        }
        QLOG(EventLog::Warning, NEX_T("**********************************************************************************************"));
        s_enableDetectRaceCondition = enable;
#else
        DUMMY_PARAM(enable);
#endif
    }

    /*!
    @brief    スレッド競合をチェックする機能が有効かどうかを取得します。

    @return スレッド競合をチェックする機能が有効かどうか。
    */
    static qBool GetEnableDetectRaceCondition()
    {
#ifdef NEX_LOCK_DEBUG_ENABLED
        return s_enableDetectRaceCondition;
#else
        return false;
#endif
    }

    class ScopedEnableDetectRaceCondition : public RootObject, public NonCopyable
    {
#ifdef NEX_LOCK_DEBUG_ENABLED
    public:
        ScopedEnableDetectRaceCondition(qBool enable)
            : m_original(GetEnableDetectRaceCondition())
        {
            SetEnableDetectRaceCondition(enable);
        }
        ~ScopedEnableDetectRaceCondition()
        {
            SetEnableDetectRaceCondition(m_original);
        }
    private:
        const qBool m_original;
#else
    public:
        ScopedEnableDetectRaceCondition(qBool enable)
        {
            DUMMY_PARAM(enable);
        }
        ~ScopedEnableDetectRaceCondition()
        {
        }
#endif
    };

#ifdef NEX_LOCK_RESTRICTIONS_ENABLED
    //{secret}
    qUnsignedInt32 GetCurrentLockRecursion() const
    {
        return m_oLockChecker.GetCurrentLockRecursion();
    }
    //{secret}
    qUnsignedInt32 GetMaximumLockRecursion() const
    {
        return m_oLockChecker.GetMaximumLockRecursion();
    }
    //{secret}
    void SetMaximumLockRecursion(qUnsignedInt32 uiMaximumLockRecusion)
    {
        m_oLockChecker.SetMaximumLockRecursion(uiMaximumLockRecusion);
    }
#endif

    // Condition向けにMutexPrimitiveへのアクセスを提供するクラス
    // SetThreadOwner()などを忘れずに行える。
    class SpecialPrimitiveAccessForCondition : public RootObject
    {
    public:
        SpecialPrimitiveAccessForCondition(CriticalSection* cs)
            : m_pCs(cs)
        {
            m_pCs->ProcessBeforePrimitiveLeave();
        }
        ~SpecialPrimitiveAccessForCondition()
        {
            m_pCs->ProcessAfterPrimitiveEnter();
        }
        MutexPrimitive& GetMutexPrimitive()
        {
            return m_pCs->m_oMutex;
        }
    private:
        CriticalSection* m_pCs;
    };
private:
    // MutexPrimitiveのEnterをした後に行うべき処理
    void ProcessAfterPrimitiveEnter()
    {
#ifdef NEX_LOCK_DEBUG_ENABLED
        SetThreadOwner();
#endif
    }
    // MutexPrimitiveのLeaveをする前に行うべき処理
    void ProcessBeforePrimitiveLeave()
    {
#ifdef NEX_LOCK_DEBUG_ENABLED
        ClearThreadOwner();
#endif
    }
    void EnterImpl()
    {
        m_oMutex.Enter();
        m_oLockChecker.EndEnter();
    }

    qBool TryEnterImpl()
    {
        if (!m_oMutex.TryEnter())
        {
            return false;
        }
        m_oLockChecker.EndEnter();
        return true;
    }

    void LeaveImpl()
    {
        m_oLockChecker.BeginLeave();
        m_oMutex.Leave();
    }

#ifdef NEX_LOCK_DEBUG_ENABLED
    void SetThreadOwner();
    void ClearThreadOwner();
#endif

private:
    void AddWaitingThread(qInt32 threadid);
    qInt32 GetWaitingThread();

private:
    MutexPrimitive          m_oMutex;

#ifdef NEX_LOCK_DEBUG_ENABLED
    volatile ThreadID    m_iThreadOwner;
    AtomicValue    m_iCounter;
#endif

    LockChecker m_oLockChecker;
#ifdef NEX_LOCK_DEBUG_ENABLED
    static qBool s_enableDetectRaceCondition;
#endif

};

END_NAMESPACE_NEX

