﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDeclInternal.h>
#include <OnlineCore/src/Platform/Core/TimeDefs.h>
#include <OnlineCore/src/Platform/Core/StringStream.h>
#ifdef CTR_TARGET
#include <nn/fnd.h>
#endif
#ifdef CAFE_TARGET
#include <cafe/os.h>
#endif
#ifdef CAFEIOP_TARGET
#include <cafe/os/OSTime.h> // for OSCalendarTime
#endif
#ifdef NEX_SIGLO_FAMILY
#include <nn/time/time_PosixTime.h>
#include <nn/time/time_CalendarTime.h>
#endif


BEGIN_NAMESPACE_NEX

// Group = Time Management

/*
Description: This class enables a date and time to be specified.
Remarks: The valid format of the date and time are respectively 'YYYY-MM-DD HH:MM:SS'.
*/

/*!
@brief    日付および時刻の指定を保持するクラスです。

日付および時間の有効な形式は、それぞれ'YYYY-MM-DD HH:MM:SS'です。

*/
class DateTime: public RootObject
{
public:
    //group=Methods

    /*
    Parameters:
    uiYear: The year of this DateTime object. The valid range is 1-9999 although
    this may differ on various platforms.
    uiMonth: The month of this DateTime object. The valid range is 01-12.
    uiDay: The day of this DateTime object. The valid range is 01-31.
    uiHour: The hour of this DateTime object. The valid range is 00-23.
    uiMinute: The minute of this DateTime object. The valid range is 00-59.
    uiSecond: The second of this DateTime object. The valid range is 00-59.
    */

    /*!
     @brief    コンストラクタです。

     DateTime::Never がセットされます。
     */
    DateTime();
    /*!
     @brief    コピーコンストラクタです。
     */
    DateTime(const DateTime& tTime);
    /*!
     @brief    コンストラクタです。
     @param[in]    uiYear    この @ref DateTime オブジェクトの年。有効範囲は1900から9999です。
     @param[in]    uiMonth    この @ref DateTime オブジェクトの月。有効範囲は01から12です。
     @param[in]    uiDay    この @ref DateTime オブジェクトの日。有効範囲は01から31です。
     @param[in]    uiHour    この @ref DateTime オブジェクトの時。有効範囲は00から23です。
     @param[in]    uiMinute    この @ref DateTime オブジェクトの分。有効範囲は00から59です。
     @param[in]    uiSecond    この @ref DateTime オブジェクト秒。有効範囲は00から59です。
     */
    DateTime(qUnsignedInt16 uiYear, qUnsignedInt8 uiMonth, qUnsignedInt8 uiDay,
             qUnsignedInt8 uiHour, qUnsignedInt8 uiMinute, qUnsignedInt8 uiSecond);

#ifdef CTR_TARGET
    /*!
     @brief    コンストラクタです。
     @param[in]    dateTime    nn::fnd::DateTime 型の日時オブジェクト。ミリ秒以下の時刻は切り捨てられます。
     */
    DateTime(const nn::fnd::DateTime& dateTime);
#endif
#if defined(CAFE_TARGET) || defined(CAFEIOP_TARGET)
    /*!
     @brief    コンストラクタです。
     @param[in]    dateTime    OSCalendarTime 型の日時オブジェクト。ミリ秒以下の時刻は切り捨てられます。
     */
    DateTime(const OSCalendarTime& dateTime);
#endif
#ifdef NEX_SIGLO_FAMILY
    /*!
     @brief    コンストラクタです。
     @param[in]    dateTime    nn::time::PosixTime 型の日時オブジェクト。
     */
    DateTime(const nn::time::PosixTime& dateTime);

    /*!
     @brief    コンストラクタです。
     @param[in]    dateTime    nn::time::CalendarTime 型の日時オブジェクト。
     */
    DateTime(const nn::time::CalendarTime& dateTime);
#endif

    // {Secret}
    DateTime(qUnsignedInt64 uiTime):m_ui64Value(uiTime) {}

    /*! @brief    代入演算子です。 */
    DateTime& operator=(const DateTime& tTime);

    operator qUnsignedInt64();
    operator qUnsignedInt64() const;

    /*! @brief    比較演算子です。 */
    qBool operator==(const DateTime& tTime) const;
    /*! @brief    比較演算子です。 */
    qBool operator!=(const DateTime& tTime) const;
    /*! @brief    比較演算子です。 */
    qBool operator<(const DateTime& tTime) const;
    /*! @brief    比較演算子です。 */
    qBool operator>(const DateTime& tTime) const;
    /*! @brief    比較演算子です。 */
    qBool operator<=(const DateTime& tTime) const;
    /*! @brief    比較演算子です。 */
    qBool operator>=(const DateTime& tTime) const;

    LargeTimeInterval operator-(const DateTime& tTime) const;

    /*
    Description: Returns the year specified by this DateTime object.
    Returns: The year specified by this DateTime object.
    */

    /*!
    @brief    この @ref DateTime オブジェクトによって指定された年を返します。

    @return    この @ref DateTime オブジェクトによって指定された年。
    */
    qUnsignedInt16 GetYear() const;

    /*
    Description: Returns the month specified by this DateTime object.
    Returns: The month specified by this DateTime object.
    */

    /*!
    @brief    この @ref DateTime オブジェクトによって指定された月を返します。

    @return    この @ref DateTime オブジェクトによって指定された月。
    */
    qUnsignedInt8 GetMonth() const;

    /*
    Description: Returns the day specified by this DateTime object.
    Returns: The day specified by this DateTime object.
    */

    /*!
    @brief    この @ref DateTime オブジェクトによって指定された日付を返します。

    @return    この @ref DateTime オブジェクトによって指定された日。
    */
    qUnsignedInt8 GetDay() const;

    /*
    Description: Returns the hour specified by this DateTime object.
    Returns: The hour specified by this DateTime object.
    */

    /*!
    @brief    この @ref DateTime オブジェクトによって指定された時(hour)を返します。

    @return    この @ref DateTime オブジェクトによって指定された時。
    */
    qUnsignedInt8 GetHour() const;

    /*
    Description: Returns the minute specified by this DateTime object.
    Returns: The minute specified by this DateTime object.
    */

    /*!
    @brief    この @ref DateTime オブジェクトによって指定された分を返します。

    @return    この @ref DateTime オブジェクトによって指定された分。
    */
    qUnsignedInt8 GetMinute() const;

    /*
    Description: Returns the seconds specified by this DateTime object.
    Returns: The seconds specified by this DateTime object.
    */

    /*!
    @brief    この @ref DateTime オブジェクトによって指定された秒を返します。

    @return    この @ref DateTime オブジェクトによって指定された秒。
    */
    qUnsignedInt8 GetSecond() const;

    // Description: Traces the content of this DateTime object.
    /*!
     @brief    オブジェクトの内容をトレースします
     @param[in]    uiTraceFlags     トレースフラグ
     */
    void Trace(qUnsignedInt64 uiTraceFlags);

    // Description: Returns whether or not the DateTime instance is valid.
    // Remarks: This method will return false if you pass a date or time of the wrong type.
    /*!
     @brief    セットされた時刻が有効であるかを返します。

     月ごとの日数やうるう年の判定を行いません。有効範囲内にあるかどうかのみチェックします。

     @retval true   有効な時刻である
     @retval false  無効な時刻である

     @see    DateTime::DateTime
     */
    qBool IsValid() const;

    /*!
     @brief    セットされた時刻が DateTime::Never と同等かを返します。

     @retval true   DateTime::Never と同等である
     @retval false   DateTime::Never と異なる

     @see    DateTime::Never
     */
    qBool IsNever() const;

    /*
    Description: Returns the current time specified by the system where this method is called.
    Parameters:
    tDateTime: The DateTime object that describes the system time.
    Returns: The current time specified by the system (Station) where this method is called.
    */

    /*!
     @brief    デバイスが持つ現在時刻を取得します。取得される時刻のタイムゾーンはプラットフォームに依存します。
     

     @param[out]    tDateTime    システム時刻
     */
    static void GetSystemTime(DateTime &tDateTime);

#ifdef CTR_TARGET
    /*!
     @brief    セットされた時刻を nn::fnd::DateTime 型の日付オブジェクトに変換して返します。

     nn::fnd::DateTime で扱うことのできる範囲を超えている場合、nn::fnd::DateTime::MIN_DATETIME を返します。

     @return nn::fnd::DateTime 型の時刻
     */
    nn::fnd::DateTime ToCTRDateTime() const;
#endif
#ifdef CAFE_TARGET
    /*!
     @brief    セットされた時刻を OSCalendarTime 型の日付オブジェクトに変換して返します。

     DateTime::Never がセットされている場合、OSCalendarTime の基準時刻 (2000年1月1日) を返します。

     @return OSCalendarTime 型の時刻
     */
    OSCalendarTime ToCafeDateTime() const;
#endif
#ifdef NEX_SIGLO_FAMILY
    /*!
     @brief    セットされた時刻を nn::time::PosixTime 型の日付オブジェクトに変換して返します。

     セットされている値が UTC 時刻とみなして、ローカル時刻への変換は行いません。
     DateTime::Never がセットされている場合、nn::time::PosixTime の基準時刻 (1970年1月1日 00時00分00秒) を返します。

     @return nn::time::PosixTime 型の時刻
     */
    nn::time::PosixTime ToPosixTime() const;

    /*!
    @brief    セットされた時刻を nn::time::CalendarTime 型の日付オブジェクトに変換して返します。

    DateTime::Never がセットされている場合、nn::time::CalendarTime の各値がすべて 0 にセットされます。
    @return nn::time::CalendarTime 型の時刻
    */
   nn::time::CalendarTime ToCalendarTime() const;
#endif

#if defined(NEX_GET_UTC_SYSTEM_TIME_SUPPORT)
    /*!
    @brief    システム時刻をUTCで取得します。
    

    @param[out]    tDateTime DateTime 型の日時オブジェクト。秒より細かい精度の時刻は切り捨てられます。
    */
    static void GetUTCSystemTime(DateTime &tDateTime);
#if (defined(WIN_FAMILY) && !defined(NEX_SIGLO_FAMILY)) || defined(UNIX_FAMILY)
    /*!
    @brief    システム時刻をUTCで取得します。

    @param[out]    tDateTime DateTime 型の日時オブジェクト。秒より細かい精度の時刻は microSecond に格納されます
    @param[out]    microSecond [0, 999999] の範囲でマイクロ秒が格納されます。
    */
    static void GetUTCSystemTime(DateTime& tDateTime, qUnsignedInt32& microSecond);

    /*!
    @brief    2000-01-01 00:00:00.000000 のフォーマットで UTC 時刻を ss に追記します。

    @param[in,out]    ss 結果を受け取る StringStream を指定する
    @param[in]    oDateTime DateTime 型の日時オブジェクトを指定する
    @param[in]    microSecond マイクロ秒を指定する
    */
    static void OutputHighResolutionDateTimeInUtc(StringStream& ss, DateTime& oDateTime, qUnsignedInt32 microSecond);
#endif // #if (defined(WIN_FAMILY) && !defined(NEX_SIGLO_FAMILY)) || defined(UNIX_FAMILY)
#endif // #if defined(NEX_GET_UTC_SYSTEM_TIME_SUPPORT)

#if defined(NEX_GET_LOCAL_SYSTEM_TIME_SUPPORT)
    /*!
    @brief    システム時刻をローカル時刻で取得します。
    

    @param[out]    tDateTime DateTime 型の日時オブジェクト。ミリ秒以下の時刻は切り捨てられます。
    */
    static void GetLocalSystemTime(DateTime &tDateTime);
#endif

    /*
    Description: An effectively infinite constant.
    Remarks: This constant is primarily used to set expiration times that never expire. In Python,
    this value is passed as "None".
    */

    /*!
    @brief    事実上無限の定数。

    この定数は、主に期限のきれることのない期限時間を設定するのに使用されます。
    */
    static const DateTime& Never;

    /*!
    @brief    常に未来をさす時刻。

    この定数は、9999/12/31 23:59:59を示します。
    */
    static const DateTime& Future;
    

#if !defined(NEX_SIGLO_FAMILY)
    /*!
     * @brief 2000/1/1 00:00:00 AM ( 独自のEpochTime ) を基準とした経過秒数を返します。
     * 
     * @return 経過秒数
     */
#endif
    qInt64 ToEpochTime() const;
    
    /*!
     * @brief 1970/1/1 00:00:00 AM ( UNIXのEpochTime ) を基準とした経過秒数を返します。
     * 
     * @return 経過秒数
     */
    qInt64 ToUnixEpochTime() const;
    
#if !defined(NEX_SIGLO_FAMILY)
    /*!
     * @brief 2000/1/1 00:00:00 AM ( 独自のEpochTime ) を基準とした経過秒数の値を持つ DateTime を生成します。
     * 
     * @param[in] epochTime 2000/1/1 00:00:00 AM ( 独自のEpochTime ) からの経過秒数
     * @return nex::DateTime 型の時刻
     */
#endif
    static DateTime FromEpochTime(qInt64 epochTime);
    
    /*!
     * @brief 1970/1/1 00:00:00 AM ( UNIXのEpochTime ) を基準とした経過秒数の値を持つ DateTime を生成します。
     * 
     * @param[in] epochTime 1970/1/1 00:00:00 AM ( UNIXのEpochTime ) からの経過秒数
     * @return nex::DateTime 型の時刻
     */
    static DateTime FromUnixEpochTime(qInt64 epochTime);


private:
    /* Fields are packed into an unsigned integer 64-bits wide

      year     10 bits, 1-9999
      month     4 bits, 1-12
      day       5 bits, 1-31
      hour      5 bits, 0-23
      minute    6 bits, 0-59
      second    6 bits, 0-59
    */
    // {secret}
    qUnsignedInt64 m_ui64Value;

    qInt32 IsLeapYear(qInt32 year) const;
    qInt32 DateToDays(qInt32 year, qInt32 month, qInt32 day) const;
    
    /*!
     * @brief 2000/1/1を0とした経過日数を日付に変換します。
     * 
     * @param[in] pastDays 2000/1/1からの経過日数
     */
    void DaysToDate(qInt32 pastDays);
    
    /*!
     * @internal
     * @brief 2000/1/1 00:00:00 AM ( 独自のEpochTime ) にカスタムの日数を加えた時刻を基準とした経過秒数の値を持つDateTimeを生成します。
     * 
     * @param[in] epochTime 経過秒数
     * @param[in] pastDaysOffset 基準となる時刻を決定するための独自のEpochTimeからの差分(日数)
     */
    static DateTime FromCustomEpochTime(qInt64 epochTime, qInt32 pastDaysOffset = 0);
};

//
// {secret}
extern StringStream& operator<<(StringStream& oStr, const DateTime& oDateTime);

END_NAMESPACE_NEX


