﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

// For NEX Developer See jwiki:MemoryAllocation for an overview of memory allocation in
// the system.

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#if defined(GCC_COMPILER) || defined(SN_COMPILER)
#include <OnlineCore/src/Platform/Core/MemoryManager.h>
#endif

#if defined(WII_TARGET) || defined(CAFE_TARGET) || defined(CTR_TARGET)

// Where ptrdiff_t is defined.

#include <stddef.h>
#endif
#include <cstddef>

BEGIN_NAMESPACE_NEX
#ifdef NEX_ALLOCATOR_SUPPORT

template<typename T> class MemAllocator
{
public:
    typedef T value_type;
    typedef size_t size_type;
    typedef ptrdiff_t difference_type;

    typedef T* pointer;
    typedef const T* const_pointer;

    typedef T& reference;
    typedef const T& const_reference;

#ifdef NEX_STL_REBIND_SUPPORT
    MemAllocator()      // construct default MemAllocator (do nothing)
    {
    }

    MemAllocator(const MemAllocator<T>&)    // construct by copying (do nothing)
    {
    }

    template<class U>
    MemAllocator(const MemAllocator<U>&)
    {
        // construct from a related MemAllocator (do nothing)
    }

    template<class U>
    MemAllocator<U>& operator=(const MemAllocator<U>&)
    {
        // assign from a related MemAllocator (do nothing)
        return (*this);
    }

#endif
    pointer address (reference __x) const
    {
        return &__x;
    }

    const_pointer address (const_reference __x) const
    {
        return &__x;
    }

    bool operator==(const MemAllocator<T> &) const
    {
        return true;
    }

    bool operator!=(const MemAllocator<T> &) const
    {
        return false;
    }

    pointer allocate(size_type iNbObjects, const void* pLocalityHint=0)
    {
        DUMMY_PARAM(pLocalityHint);
        return static_cast<pointer>(MemoryManager::Allocate(iNbObjects * sizeof(T), MemoryManager::AllocatorClass));
    }

    void deallocate(void *pPtr, size_type iNbObjects)
    {
        DUMMY_PARAM(iNbObjects);
        MemoryManager::Free(pPtr, MemoryManager::AllocatorClass);
    }

    void construct(pointer p, const T& val)
    {
        ::new ((void*)p) T(val);
    }
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    void construct(pointer p, T&& value)
    {
        ::new ((void*)p) T(std::move(value));
    }
#ifdef NEX_VARIADIC_TEMPLATES_SUPPORT
    template<typename Up, typename... Args>
    void construct(Up* p, Args&&... args)
    {
        ::new ((void*)p) Up{ std::forward<Args>(args)... };
    }
#endif
#endif

    void destroy(pointer p)
    {
        // The ptr can actually be dummy, if the type T is not a
        // class and has no destructor.
        DUMMY_PARAM(p);
        p->~T();
    }

    size_t max_size() const
    {
        size_t stCount = (size_t)(-1) / sizeof (T);
        return (0 < stCount ? stCount : 1);
    }

#ifdef NEX_STL_REBIND_SUPPORT
    template<class U>
    struct rebind
    {
        typedef MemAllocator<U> other;
    };
#else
    // On Win32 platform, _Charalloc is called instead of using the
    // rebind type.
    char *_Charalloc(size_type iSize)
    {
        return (char*) MemoryManager::Allocate(iSize, MemoryManager::AllocatorClass);
    }
#endif
};

#endif
END_NAMESPACE_NEX


#ifdef NEX_ALLOCATOR_SUPPORT
#if defined(STL_PORT) && !defined(NEX_STL_REBIND_SUPPORT)

// On platforms which do not support use of member template classes (i.e. MemAllocator<T>::rebind<U>)
// STL port requires that the functions __stl_alloc_rebind and __stl_alloc_create be defined.
namespace std
{

template <class _Tp1, class _Tp2>
inline NAMESPACE_NEX MemAllocator<_Tp2>&
__stl_alloc_rebind(NAMESPACE_NEX MemAllocator<_Tp1>& __a, const _Tp2*)
{
    return (NAMESPACE_NEX MemAllocator<_Tp2>&)(__a);
}

template <class _Tp1, class _Tp2>
inline NAMESPACE_NEX MemAllocator<_Tp2>
__stl_alloc_create(const NAMESPACE_NEX MemAllocator<_Tp1>& __a, const _Tp2*)
{
    return NAMESPACE_NEX MemAllocator<_Tp2>(__a);
}

} // end namespace std

#endif
#endif


