﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <stdarg.h> // for va_list
#include <OnlineCore/src/Platform/Core/RootObject.h>
#include <OnlineCore/src/Platform/Core/ConnectionID.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>


BEGIN_NAMESPACE_NEX

//group=Data Logging

/*
Description: Sets the output format for logged data.
Remarks: This class determines how and what data logged via a Log object is outputted
to a LogDevice. Sub class this class to provide your own custom output format.
See Also: Log, LogDevice
*/

/*!
 @brief ログ出力のフォーマットを扱う管理クラスです。

 日時やセッションタイムなどの情報をログに追加することができます。

 ShowLocalStationHandle で設定できるローカルステーションのハンドル情報のログ出力のみ、
 @ref NetZ を生成した際に自動で有効に設定されます。

 @ref EventLog::GetOutputFormat 、@ref TraceLog::GetOutputFormat で本クラスへのポインタを取得することができます。
 */
class OutputFormat : public RootObject
{
public:
    //group=Methods

    // Remarks: An OutputFormat object should never be directly instantiated. Instead, it is created by $PRODUCTNAME$ when
    // a TraceLog object is created. To access the OutputFormat object from within your game use the TraceLog::GetInstance method.
    OutputFormat();

    // Remarks: An OutputFormat object should never be directly destroyed. It is destroyed by $PRODUCTNAME$ when the TraceLog
    // object is deleted.
    virtual ~OutputFormat() {}

    // Format control callbacks.
    //{secret}
    virtual void StartString( qChar *szString, qUnsignedInt32 uiMaxChars );
    //{secret}
    virtual void StartPrefixes( qChar *szString, qUnsignedInt32 uiMaxChars );
    //{secret}
    virtual void AddPrefixes( qChar *szString, qUnsignedInt32 uiMaxChars );
    //{secret}
    virtual void EndPrefixes( qChar *szString, qUnsignedInt32 uiMaxChars );
    //{secret}
    virtual void AddIndent( qChar *szString, qUnsignedInt32 uiMaxChars );
#ifndef SWIG
    // 64bit環境のswigにva_listの引数をうまくコード生成できないバグがあるのでswigでは除外する
    //{secret}
    virtual qInt AddMessage( qChar *szString, qUnsignedInt32 uiMaxChars, const qChar *szFormat, va_list argp );
#endif
    //{secret}
    virtual void EndString( qChar *szString, qUnsignedInt32 uiMaxChars );


    // Format flags

    /*
    Description: Enables or disables trace numbering.
    Parameters:
    bDoIt:  Optional Boolean parameter that sets whether or not the traces will be numbered.
    Remarks: By default, traces are not numbered. If the bDoIt parameter of this method is true,
    traces will be numbered, if it is false then traces will not be numbered.
    */
    void EnableNumberTraces(qBool bDoIt=true);

    /*
    Description: Shows the process ID with each trace output or not.
    Parameters:
    bDoIt:  Optional Boolean parameter that sets whether or not the process ID should be printed
    along with each trace.
    Remarks: By default, the ID of the individual process running on a given station that generates the trace
    is not included with each trace. The process ID's are platform specific. If the bDoIt parameter of this method is
    true, the process ID that generated the trace message will be included. If the bDoIt parameter is false, the traces
    will not include the process ID.
    */
    void ShowProcessID(qBool bDoIt=true);

    /*
    Description: Shows the thread number with each trace output or not.
    Parameters:
    bDoIt: Optional Boolean parameter that sets whether or not the thread ID should be printed
    along with each trace.
    Remarks: By default, the thread ID is not included with each trace. If the bDoIt parameter of
    this method is true, the thread ID that generated the trace message will be included. If the
    bDoIt parameter is false, the traces will not include the thread ID.
    */
    void ShowThreadID(qBool bDoIt=true);

    /*
    Description: Shows the local time with each trace output or not.
    Parameters:
    bDoIt:  Optional Boolean parameter that sets whether or not the local time should be printed
    along with each trace.
    Remarks: By default, the local time is not included with each trace. If the bDoIt parameter
    of this method is true, the local time of the trace message will be included.  If the bDoIt
    parameter is false, the traces will not include the time of the trace. The time given is the
    time since initialization.
    */

    /*!
     @brief ローカルタイムをログに出力するかどうかを設定します

     true をセットすることで、NEXライブラリを初期化してから現在までの時間をログ出力に付加します。

     @param[in] bDoIt    ローカルタイムをログに出力するかどうか
     */
    void ShowLocalTime(qBool bDoIt=true);

    /*
    Description: Shows the current date and time with each trace output or not.
    Parameters:
    bDoIt:  Optional Boolean parameter that sets whether or not the current date and time should be
    printed along with each trace.
    Remarks: The date and time traced are that currently specified by the system and observe the
    following output format: "Da- YYYY-MM-DD HH:MM:SS".
    By default, the current date and time are not included with each trace. If the bDoIt parameter
    of this method is true, the date and time of the trace message will be included.  If the bDoIt
    parameter is false, the traces will not include the date nor time of the trace.
    See Also: DateTime
    */

    /*!
     @brief 日時をログに出力するかどうかを設定します

     true をセットすることで、@ref DateTime::GetSystemTime で取得できる値をログ出力に付加します。

     @param[in] bDoIt    日時をログに出力するかどうか
     */
    void ShowDateTime(qBool bDoIt=true);
    /*
    Description: Shows the session time with each trace output or not.
    Parameters:
    bDoIt:  Optional Boolean parameter that sets whether or not the session time should be printed
    along with each trace.
    Remarks: By default, the session time is not included with each trace. If the bDoIt parameter
    of this method is true, the local time of the trace message will be included.  If the bDoIt
    parameter is false, the traces will not include the time of the trace. The time given is the
    time since initialization.
    */

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     @brief セッションタイムをログに出力するかどうかを設定します

     true をセットすることで、@ref SessionClock::GetTime をログ出力に付加します。

     @param[in] bDoIt    セッションタイムをログに出力するかどうか
     */
#endif
    void ShowSessionTime(qBool bDoIt=true);

    /*
    Description: Shows the thread name along with each trace output or not.
    Parameters:
    bDoIt:  Optional Boolean parameter that sets whether or not the thread
    name should be printed along with each trace.
    Remarks: Each time a new thread is created by $PRODUCTNAME$, the thread is given a name according to
    its functionality.  The name "User Thread" is given to threads created due to user application code.
    By default, the thread name is not included with each trace. If the bDoIt parameter of this method is true,
    the thread name will be included. If the bDoIt parameter is false, the traces will not include
    the thread name of the trace.
    */
    void ShowSystemThreadName(qBool bDoIt=true);

    /*
    Description: Shows the Station DOHandle along with each trace output or not.
    Parameters:
    bDoIt:  Optional Boolean parameter that sets whether or not the local Station DOHandle should be printed
    along with each trace.
    Remarks: By default, the local Station DOHandle is not included with each trace. If the bDoIt parameter
    of this method is true, the Station DOHandle will be included with each trace. If the bDoIt parameter
    is false, the traces will not include the local Station DOHandle.
    See Also: DOHandle
    */

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     @brief 自分のステーションハンドル情報をログに出力するかどうかを設定します（NetZを生成した際に自動で有効に設定）

     true をセットすることで、@ref Station::GetLocalStation をログ出力に付加します。

     @ref NetZ を生成した際に自動で有効に設定されます。

     @param[in] bDoIt    自分のステーションハンドル情報をログに出力するかどうか
     */
#endif
    void ShowLocalStationHandle(qBool bDoIt=true);

    /*
    $ifdef RENDEZVOUS$
    Description: Shows the ConnectionID with each trace output or not.
    Parameters:
    bDoIt: Optional Boolean parameter that sets whether or not the ConnectionID should be printed
    along with each trace.
    Remarks: By default, the ConnectionID of the process that generated the trace is not included with each trace.
    If the bDoIt parameter of this method is true, the ConnectionID will be included with each trace, if it is false
    traces will not include the ConnectionID.
    See Also: ConnectionID
    $else$
    {secret}
    $endif$
    */
    void ShowCID(qBool bDoIt=true);

    /*
    $ifdef RENDEZVOUS$
    Description: Shows the PrincipalID with each trace output or not.
    Parameters:
    bDoIt:  Optional Boolean parameter that sets whether or not the PrincipalID should be printed
    along with each trace.
    Remarks: By default, the PrincipalID of the process that generated the trace is not included with each trace.
    If the bDoIt parameter of this method is true, the PrincipalID will be included with each trace, if it is false
    traces will not include the PrincipalID.
    See Also: PrincipalID
    $else$
    {secret}
    $endif$
    */
    void ShowPID(qBool bDoIt=true);

    /*
    Description: Adds a prefix to all traces.
    Parameters:
    szPrefix: The prefix to be added to all traces.
    Remarks: The szPrefix string has no restrictions on size and will be added to all traces. Prefixes
    are useful for easy identification of traces that originate from a
    particular part of $PRODUCTNAME$.  By default, there will be no
    prefix.
    */
    void AddPrefix(const qChar *szPrefix);

    //{secret}
    void ShowCurrentContext(qBool bDoIt=true);

    // {secret}
    static void RegisterStationHandleResolver(qUnsignedInt32 (*pfStationHandleResolver)())
    {
        s_pfStationHandleResolver=pfStationHandleResolver;
    }

    // {secret}
    static void RegisterCIDResolver(qUnsignedInt32 (*pfCIDResolver)())
    {
        s_pfCIDResolver=pfCIDResolver;
    }

    // {secret}
    static void RegisterPIDResolver(PrincipalID (*pfPIDResolver)())
    {
        s_pfPIDResolver=pfPIDResolver;
    }

    //{secret}
    static void RegisterThreadNameResolver(const qChar* (*pfThreadNameResolver)())
    {
        s_pfThreadNameResolver=pfThreadNameResolver;
    }

    //{secret}
    static void RegisterSessionTimeResolver(qUnsignedInt32 (*pfSessionTimeResolver)())
    {
        s_pfSessionTimeResolver=pfSessionTimeResolver;
    }

    //{secret}
    static void RegisterCurrentContextResolver(qUnsignedInt (*pfCurrentContextResolver)())
    {
        s_pfCurrentContextResolver=pfCurrentContextResolver;
    }

    //{secret}
    typedef PrincipalID (*FuncPIDResolver)();
    // {secret}
    static FuncPIDResolver GetPIDResolver()
    {
        return s_pfPIDResolver;
    }

    //{secret}
    typedef ConnectionID (*FuncCIDResolver)();
    //{secret}
    static FuncCIDResolver GetCIDResolver()
    {
        return s_pfCIDResolver;
    }

    /*
    Description: Increases the trace output indentation.
    Parameters:
    uiIncrease: The number of extra indentation spaces that will be added to the trace.
    See Also: DecreaseIndent
    */
    void IncreaseIndent(qUnsignedInt uiIncrease=2);

    /*
    Description: Decreases the trace output indentation.
    Parameters:
    uiDecrease: The number of indentation spaces that will be removed from the trace.
    See Also: IncreaseIndent
    */
    void DecreaseIndent(qUnsignedInt uiDecrease=2);
private:
    //{secret}
    static void AppendToString(qChar *szString, const qChar *szAppend, qUnsignedInt32 uiSize);
    //{secret}
    static qInt AddMessageImpl( qChar *szString, qUnsignedInt32 uiMaxChars, const qChar *szFormat, va_list argp );
    //{secret}
    static void PreparePrefix( qChar *szString, qUnsignedInt32 uiMaxChars, const qChar *szFormat, ... );

private:
    qUnsignedInt32 m_uiIndent;

    qUnsignedInt32 m_uiNumberTrace;
    qBool m_bShowThreadID;
    qBool m_bShowProcessID;
    qBool m_bShowLocalTime;
    qBool m_bShowDateTime;
    qBool m_bShowSessionTime;
    qBool m_bShowSystemThreadName;
    qBool m_bShowLocalStationHandle;
    qBool m_bShowCurrentContext;
    qBool m_bShowCID;
    qBool m_bShowPID;
#ifdef CTR_TARGET
    qByte padding1[2];  // to avoid compiler warning.
#endif
    const qChar* m_szPrefix;

    qUnsignedInt32 m_uiInitTime;

    static ConnectionID (*s_pfCIDResolver)();
    static PrincipalID (*s_pfPIDResolver)();
    static qUnsignedInt32 (*s_pfStationHandleResolver)();
    static const qChar* (*s_pfThreadNameResolver)();
    static qUnsignedInt  (*s_pfCurrentContextResolver)();
    static qUnsignedInt32 (*s_pfSessionTimeResolver)();

};


END_NAMESPACE_NEX

