﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Platform/Core/MemoryManager.h>
#include <OnlineCore/src/Core/PseudoSingleton.h>

#if defined(NEX_CLIENT_PBPOOL) || defined(FOR_DOC_PURPOSE) // クライアントのみ


BEGIN_NAMESPACE_NEX

class PacketBufferMultiUnitHeap;

namespace
{
// デフォルトのパケットバッファプールは300 kbyte確保です。
const size_t DEFAULT_PBPOOL_SIZE=300*1024;
// GetSimpleUnitHeapParamでの最小プールサイズ
const size_t MINIMUM_SIMPLE_UNIT_HEAP_SIZE=64*1024;
}

/*!
 @brief    P2P 通信向けのパケットバッファのメモリ管理クラスです。

 NEXのメモリ確保のうち、 P2P 通信で使用するパケットバッファのメモリ管理はパケットバッファマネージャーによって行われます。

 パケットバッファマネージャーは、 P2P 通信開始前に専用のメモリプール(パケットバッファプール)を一括で確保し、
 パケットバッファのメモリ確保要求に応じてパケットバッファプールからメモリ割り当てを行う機能です。

 パケットバッファプール自体のメモリ確保は他のNEXオブジェクトと同じ方法で確保されます。

 パケットバッファプールはあらかじめ制限されたサイズで確保されるため、
 大量のパケット送受信が発生した際のメモリ消費によるNEX全体のメモリ枯渇を防ぐことができます。

 以下で使用されるパケットバッファについてはパケットバッファマネージャーでは管理せず、他のNEXオブジェクトと同じメモリ管理が行われます。

 @li サーバーとの通信
 @li 複製オブジェクトのうちCoreDOによる通信
 @li パケットバッファマネージャーで確保可能なメモリブロックサイズの最大(1364byte)を超えるデータの送受信

 パケットバッファプールの確保は P2P 通信開始前に確保され、 P2P 通信停止で解放されます。
  具体的には、 ConnectivityManager::StartNATSession()、NetZ::NetZ()、 VSocket::Open()の呼び出しを契機に確保され、
 ConnectivityManager::StopNATSession()、 NetZ::~NetZ()、 VSocket::Close()の呼び出し契機で解放されます。
 

 パケットバッファプールは、複数のユニットヒープを使用して管理され、パケットのペイロード以外にペイロードに付随するパケット管理に使用するオブジェクトの確保に使用されます。
 このうち、ペイロード用のユニットヒープについて、ユニットサイズやメモリブロック数をアプリケーション側でカスタマイズすることができます。アプリケーションの通信データサイズにあわせてカスタマイズしてください。

 パケットバッファの確保は、ユニットサイズが小さいユニットヒープから順にメモリ確保を試み、最初に確保可能となったユニットヒープが使用されます。
 最大サイズのユニットヒープでも確保できない場合は、パケットバッファプールが枯渇した状態となります。
 */
class PacketBufferManager: public PseudoSingleton
{
public:
    /*!
     * @brief   パケットバッファマネージャー内部で使用するユニットヒープの構成情報です。
     *
     * ユニットサイズに指定可能な値は、4の倍数で1364以下の値です。
     * NEXの情報付加を考慮し、ユニットサイズにはアプリケーションの送受信サイズに80以上の値を足した数値を使用してください。
     *
     * @see GetUnitHeapParam(), SetUnitHeapParam()
     */
    struct UnitHeapParam : public RootObject
    {
        size_t unitSize;  //!< ユニットヒープのユニットサイズ(4の倍数かつ1364以下)
        size_t unitNum;   //!< ユニットサイズで確保可能なメモリブロックの数

        bool operator<(const UnitHeapParam& param) const
        {
            return unitSize < param.unitSize;
        }
        bool operator==(const UnitHeapParam& param) const
        {
            return unitSize == param.unitSize;
        }
    };
    /*!
     * @brief   パケットバッファマネージャー内部で使用するユニットヒープのメモリ確保情報です。
     *
     * NEX内部ではバッファ確保前にあらかじめ必要数分の空きをチェックし、不足する場合にメモリ枯渇発生回数としてカウントする場合があります。
     * そのため、実際の確保数のピーク値が最大数に達しない状態でメモリ枯渇発生回数が増加することがあります。
     *
     * 最もユニットサイズの大きいユニットヒープのメモリ枯渇発生回数が増加する場合は、メモリブロック数の最大数を上げるようにユニットヒープ構成を設定してください。
     *
     * @see GetUnitHeapDebugInfo(), ResetUnitHeapDebugInfo()
     */
    struct UnitHeapDebugInfo : public RootObject
    {
        size_t peak;     //!< 確保数のピーク値
        size_t use;      //!< 現在の確保数
        size_t max;      //!< 確保可能な最大数
        size_t unitSize; //!< ユニットヒープのユニットサイズ
        size_t realUnitSize; //!< 実際に確保するメモリブロックのサイズ
        size_t alloc; //!< 確保回数
        size_t free;  //!< 解放回数
        size_t outOfMemory;  //!< メモリ枯渇発生回数
    };

public:
    /*!
     @brief 渡されたペイロード用のユニットヒープ構成情報をもとに、必要とされるパケットバッファプールのサイズを返します。

     返されるプールサイズは、渡されたペイロード用ユニットヒープ構成情報をもとに確保されるペイロード用ユニットヒープと
     NEX内部で自動確保されるパケット管理オブジェクト用ユニットヒープの合計になります。

     渡されたユニットヒープ構成情報が以下の条件の場合、不正とみなし0を返します。

     @li ユニットサイズが0または1364超
     @li ユニットサイズが4の倍数でない
     @li メモリブロック数が0
     @li 配列サイズが0
     @li ユニットサイズが1364の要素が無い

     @param[in] param ペイロード用のユニットヒープの構成情報を指定します。
     @return メモリプールのサイズを返します。ユニットヒープ構成情報が不正の場合0を返します。
     */
    static size_t GetRequiredMemorySize(const qVector<UnitHeapParam>& param);
    static size_t GetRequiredMemorySizeImpl(const qVector<UnitHeapParam>& param);
    /*!
     @brief パケットバッファプールのサイズと平均送信データサイズを元に、ユニットヒープの簡易構成を算出して返します。

     簡易構成は、ペイロード用とパケット管理オブジェクト用のユニットヒープのメモリ合計が指定したパケットバッファプールのサイズに収まるように算出されます。
     また、ペイロード用のユニットヒープで使用するメモリのうち、半分が平均送信データサイズ、残り半分が最大サイズ(1364)のユニットヒープに割り当てられます。
     なお、ユニットサイズはNEX内部で付与されるヘッダなどを考慮して平均送信データ送信サイズを80増やして4の倍数に補正された値となります。

     パケットバッファプールで確保されるのは1364byte以下のパケットバッファのみのため、1364byteを超える送信データを除外した平均送信データサイズを与えてください。

     この関数で得られた構成をもとに、アプリケーションに合わせたチューニングを行ってください。
     プールサイズは64kbyte以上を指定する必要があります。

     @param[in] poolSize パケットバッファプールのサイズを指定します。
     @param[in] avgSendSize アプリケーションで送信するデータの平均サイズ。
     @param[out] param ペイロード用のユニットヒープの構成情報が返ります。
     @retval true  ユニットヒープ構成の算出成功
     @retval false  平均送信データサイズが0または1364を超える、プールサイズが小さすぎる
     */
    static qBool GetSimpleUnitHeapParam(size_t poolSize, size_t avgSendSize, qVector<UnitHeapParam>& param);
    /*!
     @brief    パケットバッファマネージャーで使用するペイロード用のユニットヒープの構成情報を設定します。

     すでにパケットバッファマネージャーが動作している場合は、設定できません。
     動作中かどうかは、@ref IsActive() で分かります。

     以下の条件の場合、不正情報としてアサートエラーになります。

     @li ユニットサイズが0または1364超
     @li ユニットサイズが4の倍数でない
     @li メモリブロック数が0
     @li 配列サイズが0
     @li ユニットサイズが1364の要素が無い

     設定した値はNEX終了時まで保持されます。

     @param[in] param ペイロード用のユニットヒープの構成情報を指定します。
     @retval true 設定成功
     @retval false 設定失敗
     @see IsActive(), GetUnitHeapParam()
     */
    static qBool SetUnitHeapParam(const qVector<UnitHeapParam>& param);
    /*!
     @brief    パケットバッファマネージャーで使用するペイロード用のユニットヒープの構成情報を取得します。
               @ref SetUnitHeapParam() で設定を行っていない場合は、デフォルトで使用される構成情報を返します。

     デフォルトのパケットバッファプールは、ペイロード用に1364byteのユニットサイズのユニットヒープが使用され、
     パケット管理オブジェクト用のユニットヒープを含めて300kbyteの領域を使用します。
     @param[out] param ペイロード用の各ユニットヒープの構成情報が返ります。
     @see SetUnitHeapParam()
     */
    static void GetUnitHeapParam(qVector<UnitHeapParam>& param);

    /*!
     @brief    パケットバッファマネージャーで使用しているペイロード用のユニットヒープのメモリ確保情報を取得します。
               パケットバッファマネージャーが動作中の場合のみ正しい値が取得できます。
     @param[out] debugInfo ペイロード用の各ユニットヒープのメモリ確保情報が返ります。
     @see ResetUnitHeapDebugInfo()
     */
    static void GetUnitHeapDebugInfo(qVector<UnitHeapDebugInfo>& debugInfo);
    /*! @internal
     @brief    パケット管理オブジェクト用を含んだすべてのユニットヒープのメモリ確保情報を取得します。
               パケットバッファマネージャーが動作中の場合のみ正しい値が取得できます。
               デバッグ用途。
     @param[out] debugInfo 各ユニットヒープのメモリ確保情報が返ります。
     @see ResetUnitHeapDebugInfo()
     */
    static void GetAllUnitHeapDebugInfo(qVector<UnitHeapDebugInfo>& debugInfo);
    /*!
     @brief    パケットバッファマネージャーで使用しているペイロード用のユニットヒープのメモリ確保情報をリセットします。
     @see GetUnitHeapDebugInfo()
     */
    static void ResetUnitHeapDebugInfo();

    /*!
     @brief    パケットバッファマネージャーが動作しているかを返します。動作中はユニットヒープ構成を変更できません。
     @retval true 動作中
     @retval false 動作していない
     */
    static qBool IsActive();

    /*!
     @brief    パケットバッファプールを使用するか設定します。デフォルトはtrueで、使用します。
               falseに設定した場合、デフォルトのメモリマネージャーによってパケットバッファは確保されます。
               パケットバッファマネージャーが動作している時は変更できません。
               設定した値は NEX の終了処理で破棄されます。
     @param[in] bEnable true:使用する、false:使用しない
     @retval true 設定成功
     @retval false パケットバッファマネージャーが動作中のため設定失敗
     @see IsEnablePacketBufferPool()
     */
    static qBool EnablePacketBufferPool(qBool bEnable);
    /*!
     @brief    パケットバッファプールを使用するか返します。デフォルトはtrueで、使用します。
     @retval true 使用する
     @retval false 使用しない
     @see EnablePacketBufferPool()
     */
    static qBool IsEnablePacketBufferPool();

public:
    /*! @internal
     * @brief   コンストラクタ。NEX内部からのみ呼び出し可能です。
     */
    PacketBufferManager();
    virtual ~PacketBufferManager();

    /*! @internal
     @brief    @ref PacketBufferManager のインスタンスを取得します。

     @return @ref PacketBufferManager オブジェクトのポインタ
     */
    static PacketBufferManager* GetInstance()
    {
        return static_cast<PacketBufferManager*>(InstanceControl::FindInstance(IC_TYPE_PACKET_BUFFER_MANAGER,
            PseudoSingleton::GetCurrentContext()));
    }

    /*! @internal
     @brief    パケットバッファマネージャーが動作しているかを返します。動作中はユニットヒープ構成を変更できません。
     @retval true 動作中
     @retval false 動作していない
     */
    qBool IsActiveNonStatic();

    /*! @internal
     @brief ユニットヒープ構成情報のチェック。
            以下の場合falseではじく。
            ユニットサイズが0、メモリブロック数が0、ユニットサイズが1364超、配列サイズが0、ユニットサイズが1364の要素が無い
     */
    static qBool CheckUnitHeapParam(const qVector<UnitHeapParam>& param);
    /*! @internal
     @brief パケットバッファ用に確保されるパケットペイロード以外の管理オブジェクト用のユニットヒープ構成を取得します。
            管理オブジェクト用ユニットヒープはInitialize()時に自動で追加されます。
            管理オブジェクト用ユニットヒープはGetRequiredMemorySize()で取得するサイズに含まれます。

     @param[out] paramOutput 管理オブジェクト用のユニットヒープ構成情報を返します。
     @param[in] paramInput パケットペイロードのユニットヒープ構成情報。
     */
    static void GetPacketManagementUnitHeapParam(qVector<UnitHeapParam>& paramOutput, const qVector<UnitHeapParam>& paramInput);

    /*! @internal
    @brief ユニットヒープ構成の生の値を直接取得します。

    @return ユニットヒープ構成。
    */
    static qVector<UnitHeapParam>& GetUnitHeapParamVariable();

    /*! @internal
    @brief ユニットヒープ構成をクリアします。
    */
    static void ClearUnitHeapParamVariable();

    /*! @internal
     @brief メモリプールのサイズとユニットサイズからユニットヒープで確保可能なメモリブロック数を返します。
            実際に確保されるメモリブロック数はユニットサイズ+8です。
            返されるブロック数はメモリサイズ/(ユニットサイズ+8)となります。

     @param[in] memorySize ユニットヒープで確保するメモリプールのサイズを指定します。
     @param[in] unitSize ユニットヒープで確保するブロックのサイズを指定します。
     @return メモリブロック数を返します。ユニットサイズが4の倍数でない場合0が返ります。
     */
    size_t GetUnitNum(size_t memorySize, size_t unitSize);
    /*! @internal
     @brief    NEXライブラリ内部で使用されるパケットバッファ用のメモリ確保関数を設定します。

     @param[in]    fcnMalloc  ユーザー定義のメモリ確保関数。
     @param[in]    fcnManagementMalloc  ユーザー定義のメモリ確保関数。管理オブジェクト用。
     */
    void SetPacketBufferMemoryFunctions(MemoryManager::MallocFunction fcnMalloc, MemoryManager::MallocFunction fcnManagementMalloc)
    {
        m_fcnPacketBufferMalloc = fcnMalloc;
        m_fcnPacketManagementObjectMalloc = fcnManagementMalloc;
    }

    /*! @internal
     @brief    パケットバッファマネージャーで使用しているマルチユニットヒープのメモリ確保状況を表示する。
     */
    void Dump(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);
    /*! @internal
     @brief    パケットバッファ用のメモリプールからメモリ割り当てがされていないかを返します。
     @return   true:割り当てはありません、false:割り当てられています。
     */
    qBool IsClean();

    /*! @internal
     @brief    パケットバッファマネージャー初期化。参照カウントをUPします。UP前参照カウント0の時、プール確保します。
     */
    void Initialize();

    /*! @internal
     @brief    パケットバッファマネージャー終了。参照カウントをDOWNします。プールの解放は行われません。
     */
    void Finalize();

    /*! @internal
     @brief    プールの解放を行います。PacketBufferManagerのデストラクタから自動的に呼び出されます。直接呼び出す場合は参照カウントが0の場合のみ解放を行えます。
     */
    void Destroy();

    /*! @internal
     @brief    最大のユニットサイズ。未初期化時は0。
     */
    size_t GetMaxUnitSize();
    /*! @internal
     @brief    requiredのサイズの確保に必要な最適なユニットサイズ。
     */
    size_t GetProperUnitSize(size_t required);
    /*! @internal
     @brief    requiredSizeのサイズのペイロードのパケットをrequiredNum個確保できるか。
               パケットバッファマネージャーが動作してない時は、通常プールのみが使用されるので常にOK。
     */
    qBool IsAllocatable(size_t requiredSize, size_t requiredNum);

    /*! @internal
     @brief    受信処理で必要な空き数。 実際にはRMC処理に必要な数で制限される。RMC処理ではレスポンス送信分が必要となるので、その個数考慮する必要がある。
     */
    void SetRequiredPacketSpace(size_t space)
    {
        m_requiredPacketSpace = space;
    }
    size_t GetRequiredPacketSpace()
    {
        return m_requiredPacketSpace;
    }
    /*! @internal
     @brief    パケット送信に必要な空きパケット数。 GetRequiredPacketSpace()より4パケットは低い値にする。
     */
    void SetRequiredSendPacketSpace(size_t space)
    {
        m_requiredSendPacketSpace = space;
    }
    size_t GetRequiredSendPacketSpace()
    {
        return m_requiredSendPacketSpace;
    }
    /*! @internal
     @brief 参照カウントの取得。
     */
    size_t GetRefCount()
    {
        return m_refCount;
    }
    /*! @internal
     @brief メモリ確保。 Buffer::AllocateDataBuffer()で呼び出します。 ペイロード用です。
     */
    void *Allocate(size_t stSize);
    /*! @internal
     @brief メモリ確保。 ペイロード以外のパケット管理オブジェクトの確保。
     */
    void *AllocateManagementObject(size_t stSize);
    /*! @internal
     @brief 指定した確率でメモリ確保失敗する。デバッグ用途。NEX_DEBUG_VERSIONでのみ有効。
     */
    void SetAllocateFailProbability(qReal prob)
    {
        m_allocateFailProbability = prob;
    }

private:
    static void* PacketBufferAllocate(size_t size);
    static void* PacketManagementObjectAllocate(size_t size);
    static void PacketBufferFree(void* ptr);
    static void PacketManagementObjectFree(void* ptr);
    PacketBufferMultiUnitHeap* GetHeapForPayload() { return m_pMultiUnitHeap; }
    PacketBufferMultiUnitHeap* GetHeapForManagement() { return m_pMultiUnitHeapMng; }

    static const size_t DEFAULT_REQUIRED_PACKET_SPACE = 8;
    static const size_t DEFAULT_REQUIRED_SEND_PACKET_SPACE = 4;

    static CriticalSection s_csSettingLock;
    static qVector<UnitHeapParam>* s_pUnitHeapParams;

    MemoryManager::MallocFunction m_fcnPacketBufferMalloc;
    MemoryManager::MallocFunction m_fcnPacketManagementObjectMalloc;
    void* m_pPacketBuffer;
    size_t m_packetBufferSize;
    size_t m_refCount;
    size_t m_requiredPacketSpace;
    size_t m_requiredSendPacketSpace;
    qReal m_allocateFailProbability;
    qBool m_isActive;

    PacketBufferMultiUnitHeap* m_pMultiUnitHeap; // ペイロード用のマルチユニットヒープ
    PacketBufferMultiUnitHeap* m_pMultiUnitHeapMng; // パケット管理オブジェクト用のマルチユニットヒープ
};

END_NAMESPACE_NEX

#endif // NEX_CLIENT_PBPOOL

