﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

#ifdef NEX_PERFCOUNTER_ENABLED

#include <limits>
#include <OnlineCore/src/Platform/Core/SystemClock.h>
#include <OnlineCore/src/Platform/Core/qChain.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/PerfCounterNames.h>

BEGIN_NAMESPACE_NEX

//Group=Counter Headings

/*
Description: This enumeration contains the headings for the SandboxProbe Performance counters.
*/
enum PerfCounterValueType
{
    /*
    Description: The frequency of the counter observed during the last second.
    */
    PERF_FREQUENCY_LAST = 1,

    /*
    Description: The average frequency of the counter observed since the
    creation of the counter or since the last time it was reset.
    */
    PERF_FREQUENCY_AVERAGE,

    /*
    Description: The minimum frequency of the counter observed since the
    creation of the counter or since the last time it was reset.
    */
    PERF_FREQUENCY_MIN,

    /*
    Description: The maximum frequency of the counter observed since the
    creation of the counter or since the last time it was reset.
    */
    PERF_FREQUENCY_MAX,

    /*
    Description: The total of the counter since the creation of the counter
    or since the last time it was reset.
    */
    PERF_COUNT_TOTAL
};

// {Secret}
//class used to keep track of the stress on an object (mesured in the number of events it had to deal with in a second)
//Useful for sandbox nodes
class PerfCounter: public RootObject
{

public:
    PerfCounter(const qChar *szName=NEX_T("<unknown>"), Time tNow = SystemClock::GetTime(), qBool bGlobalVariable=false, qBool bLogAlways=false, const qChar* description = NEX_T(""));
    virtual ~PerfCounter();

    // Resets all the stastics to 0 and all internal data and sets the current time to be tNow.
    void Reset(Time tNow = Time::GetTime());
    //Records that at the current time (defined by tNow), uiValue events have been recorded since Inc(...) function was last called
    void Inc(qUnsignedInt32 uiValue = 1, Time tNow = SystemClock::GetTime())
    {
        Update(tNow, static_cast<qInt>(uiValue));
    }
    void Dec(qUnsignedInt32 uiValue = 1, Time tNow = SystemClock::GetTime())
    {
        Update(tNow, 0 - static_cast<qInt>(uiValue));
    }

#ifdef NEX_64BIT_ARCH
    // 64bit環境でキャストなしでsize_tを受け取れるようにするためオーバーロードする
    void Inc(size_t uiValue,Time tNow = SystemClock::GetTime())
    {
        SYSTEMCHECK(uiValue <= (std::numeric_limits<qUnsignedInt32>::max)());
        Inc(static_cast<qUnsignedInt32>(uiValue), tNow);
    }
    void Dec(size_t uiValue,Time tNow = SystemClock::GetTime())
    {
        SYSTEMCHECK(uiValue <= (std::numeric_limits<qUnsignedInt32>::max)());
        Dec(static_cast<qUnsignedInt32>(uiValue), tNow);
    }

    // qIntでuiValueを渡されるとsize_tとqUnsignedInt32のオーバーロードを解決できないためqIntのオーバーロードも定義する
    void Inc(qInt uiValue,Time tNow = SystemClock::GetTime())
    {
        SYSTEMCHECK(uiValue >= 0);
        Inc(static_cast<qUnsignedInt32>(uiValue), tNow);
    }
    void Dec(qInt uiValue,Time tNow = SystemClock::GetTime())
    {
        SYSTEMCHECK(uiValue >= 0);
        Dec(static_cast<qUnsignedInt32>(uiValue), tNow);
    }
#endif

    //Logs the information about the PerfCounter object

    void Trace(qUnsignedInt64 uiTraceFlags, Time tNow = SystemClock::GetTime());

    //Logs the information about the PerfCounter object (statistics and name)
    void Log();
    qBool IsLogAlways() const;
    void  SetLogAlways();
    void  ClearLogAlways();
    static void LogAll(qBool bSkipUnused = true);
    //Calls the Reset(...) function on all the global PerfCounter objects
    static void ResetAll();

    //Gets the ID of the PerfCounter object
    qUnsignedInt32 GetID() const;

    //Returns the average number of events per second in the last second
    qFloat GetAvg(Time tNow = SystemClock::GetTime());
    //Returns the total number of events that occured in the last second
    qUnsignedInt32 GetTotal(Time tNow = SystemClock::GetTime());
    //Returns the minimum frequency (in events per second) recorded in the last second
    qUnsignedInt32 GetMin(Time tNow = SystemClock::GetTime());
    //Returns the maximum frequency (in events per second) recorded in the last second
    qUnsignedInt32 GetMax(Time tNow = SystemClock::GetTime());
    //Returns the frequency recorded in the latest update (interval lasting a fraction of a second)
    qUnsignedInt32 GetLast(Time tNow = SystemClock::GetTime());

    static qBool ValidValueRange(qInt iValue);

    //Fetches one of the statistics PerfCounter kept track of (which one depends on eValue)
    qUnsignedInt32 GetValue(PerfCounterValueType eValue);

    typedef qChain<PerfCounter*> List;
    //Gets a list containing all global PerfCounter objects
    static List& GetList();

    //Gets the name of the PerfCounter object
    const qChar* GetName() const;
    //Changes the name of the PerfCounter object to be szName
    void SetName(const qChar* szName);
    void SetDescription(const qChar* /*description*/) {};
    const qChar* GetDescription() const
    {
        return GetName();
    }

    // {secret}
    //Returns a list containing the ID of all the PerfCounters in the list
    static qList<qUnsignedInt32> GetAllIDs();

    // {secret}
    // Finds a PerfCounter named strName in the list and return it.  This is not meant to be efficient, and will iterate accross the entire
    //list of watermarks.
    static PerfCounter* Find(const String& strName);

    /**
        \brief Finds all PerfCounter with the named strName.
        \returns A list containing all PerfCounters named strName.
        \note This is not meant to be efficient, and will iterate across the entire list of PerfCounters.
    */
    static const qList<PerfCounter*> FindByName(const String& strName);

    // {secret}
    // Finds a PerfCounter with ID uiID in the list and return it.  This is not meant to be efficient, and will iterate accross the entire
    //list of PerfCounters.
    static PerfCounter* FindByID(const qUnsignedInt32 uiID);

    static qUnsignedInt32 GetNextID();

public:
    CHAINABLE(PerfCounter);
    //Returns the synchronization variable (s_oCS) that indicates if another PerfCounter object is doing operations on the
    //list of global PerfCounter objects
    static CriticalSection& GetCriticalSection()
    {
        return s_oCS;
    }

private:
    void Update(Time tNow, qInt iValue=0);

    //Unique ID for this ProfilingUnit object
    qUnsignedInt32 m_uiID;

    //First valid ID to assign to the next ProfilingUnit object that will be created
    static qUnsignedInt32 s_uiNextFreeID;

protected:
    static const qUnsignedInt32 NUM_SAMPLE = 10;
    //Name of the PerfCounter object
    String m_strName;
    qUnsignedInt32      m_uiCurrent[NUM_SAMPLE];
    //Stores the minimum frequency (in events per second) recorded in the last second
    qUnsignedInt32      m_uiMin;
    //Stores the maximum frequency (in events per second) recorded in the last second
    qUnsignedInt32      m_uiMax;
    qUnsignedInt32      m_uiLast;
    //Stores the frequency recorded in the latest update (interval lasting a fraction of a second)
    qUnsignedInt32      m_uiTotal;
    Time                m_tLastTime;
    Time                m_tStarted;
    qUnsignedInt32      m_uiLastSampleIndex;
    qBool               m_bLogAlways;

    //Variable initialized by an argument to the constructor that defines whether this PerfCounter object is in the list of PerfCounter objcts or not
    qBool m_bGlobalVariable;
    //Synchronization variable used to synchronize access to list of global PerfCounter objects amongst all the PerfCounter objects
    static ForcedCriticalSection s_oCS;
};

END_NAMESPACE_NEX

#endif // NEX_PERFCOUNTER_ENABLED

