﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

// Group=Platform Subsystem

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/HighResolutionClock.h>
#include <OnlineCore/src/Platform/Core/HighResolutionChrono.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/qChain.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/String.h>


BEGIN_NAMESPACE_NEX

//Class used to keep track of statistics (total time, average time, max time, min time) on the time by an event which is repeated multiple times
//Useful to gather stats on the efficienty of a function calls
/*!
    @brief コードの実行に要した時間を測定するクラスです。

    特定のコードブロックの実行に要した総合時間、平均時間、最大時間、最小時間などを得ることができます。
    例えば関数foo()の実行時間を測定したい場合は、関数foo()の前後に以下のようなコードを追加します。

    @code
        {

            static NAMESPACE_NEX ProfilingUnit s_pu(NEX_T("foo_func"));

            NAMESPACE_NEX ProfilingUnit::Scope scoped(s_pu);

            foo();

        }
    @endcode

    このコードを何度か実行した後で、s_puに対して@ref GetTotalTime() などのメンバ関数を呼び出すと、
    s_puインスタンスに記録された測定情報を取得できます。
 */
class ProfilingUnit: public RootObject
{
public:

    CHAINABLE(ProfilingUnit);

    /*!
    @brief ProfilingUnitに測定情報を書き込むためのクラスです。

    コンストラクタ、デストラクタ呼び出しのタイミングで測定が行われます。
     */
    class Scope: public RootObject
    {
        // Group=
    public:
        Scope(ProfilingUnit &pPU)
        {
            m_pPU=&pPU;
            m_bRecorded = false;
        }

        virtual ~Scope()
        {
            if (!m_bRecorded)
            {
                Record();
            }
        }

        qBool Record()
        {
            if (m_bRecorded)
            {
                return false;   // Record only once.
            }
            m_pPU->IncTime(m_oChronoTime.GetElapsed(true));
            m_bRecorded = true;
            return true;
        }

        void Reset()
        {
            m_bRecorded = false;
            m_oChronoTime.GetElapsed(true);
        }

    private:
        ProfilingUnit   *m_pPU;
        HighResolutionChrono   m_oChronoTime;
        qBool m_bRecorded;
    };

    // Group=
public:
    ProfilingUnit(const qChar *szName=NEX_T("<unknown>"), qUnsignedInt32 uiLoggingFrequency = 0, const qChar* description = NEX_T(""));
    ProfilingUnit(const ProfilingUnit& rhs);
    virtual ~ProfilingUnit();

    //n1769
    /*!
    @brief （アプリケーションにとって興味のある）何らかの量を加算します。（例：送受信バッファのサイズ等）

    @param[in] amount 加算対象。

    @see GetAmount
    */
    void           IncAmount(qUnsignedInt32 amount)
    {
        m_ui32Amount += amount;
    }

    //n1769
    /*!
    @brief （@ref IncAmount() で加算した）何らかの量を取得します。

    @see IncAmount

    @return  加算対象の和。
    */
    qUnsignedInt32 GetAmount(void) const
    {
        return m_ui32Amount;
    }

    //n1769
    /*!
    @brief ProfilingUnitのソートに用いる変数を設定します。

    @param[in] order ソート番号。
    */
    void   SetOrder(qInt32 order)
    {
        m_i32Order = order;
    }

    //n1769
    /*!
    @brief ProfilingUnitのソートに用いる変数を取得します。

    @return ソート番号。
    */
    qInt32 GetOrder(void) const
    {
        return m_i32Order;
    }

    //n1769
    /*!
    @brief プロファイリングを有効にします。
    */
    static void Enable(void);

    /*!
    @brief プロファイリングを無効にします。
    */
    static void Disable(void);

    /*!
    @brief プロファイリングが有効かどうかを返します。

    @return プロファイリングが有効であればtrue、そうでなければfalseが返ります。
    */
    static bool IsEnabled(void);

    //Adds another function call which took oTimeIncrement to the data
    void IncTime(HighResolutionTime oTimeIncrement);
    void IncTime(HighResolutionTime oTimeIncrement, const qChar* pName);
    //Gets the total time (sum) of all the event times that were declared in t he ProfilingUnit object
    HighResolutionTime GetTotalTime() const;  //!< 実行に要した総合時間を得ます。
    //Gets the time taken by the quickest event
    HighResolutionTime GetMinTime() const;    //!< 実行に要した最小時間を得ます。
    //Gets the time taken by the slowest event
    HighResolutionTime GetMaxTime() const;    //!< 実行に要した最大時間を得ます。
    //Gets the time taken by the last event
    HighResolutionTime GetLastTime() const;   //!< 最新の所要時間を得ます。
    //Gets the average time for all events recorded
    qDouble GetAvgTime() const;               //!< 実行に要した平均時間を得ます。
    //Gets the variance time for all events recorded
    qDouble GetVarTime() const;               //!< 分散を得ます。
    //Gets the number of events recorded
    qUnsignedInt32 GetNbOccurences() const;   //!< 実行回数を得ます。

    void EnableLogging(qBool bAutoLogEnabled);
    void SetLoggingFrequency(qUnsignedInt32 uiLoggingFrequency);

    //Logs information about the ProfilingUnit object (its name, statistics, time elasped since the object was last reset)
    void Log();

    //Gets the name of the ProfilingUnit object
    const qChar* GetName() const;              //!< ProfilingUnitオブジェクトに設定した名前を取得します。

    //Gets the ID of the ProfilingUnit object
    qUnsignedInt32 GetID() const;              //!< ProfilingUnitオブジェクトのIDを取得します。

    //Sets the name of the ProfilingUnit object to be szName
    void SetName(const qChar* szName);         //!< ProfilingUnitオブジェクトに名前を設定します。

    void SetDescription(const qChar* /*description*/) {};
    const qChar* GetDescription() const
    {
        return GetName();
    }

    typedef qChain<ProfilingUnit*>  List;

    //Returns the list of all ProfilingUnit objects
    static List& GetList();

    //Returns class critical section
    static NAMESPACE_NEX CriticalSection& CS();

    //Resets all the statistics for the ProfilingUnit object and sets the recorded time for the reset to be tNow
    void Reset(Time tNow = Time::GetTime());
    //Calls the Reset(...) function on all the ProfilingUnit objects in the list
    static void ResetAll();

    // {secret}
    //Returns a list containing the ID of all the ProfilingUnits in the list
    static qList<qUnsignedInt32> GetAllIDs();

    // {secret}
    // Finds a ProfilingUnit with ID uiID in the list and return it.  This is not meant to be efficient, and will iterate accross the entire
    //list of ProfilingUnits.
    static ProfilingUnit* FindByID(const qUnsignedInt32 uiID);

    /**
        \brief Finds all ProfilingUnit with name name in the list.
        \param[in] name The ProfilingUnit name.
        \returns A list of ProfilingUnits with name name.
        \note This is not meant to be efficient, and will iterate across the entire list of ProfilingUnits.
    */
    static const qList<ProfilingUnit*> FindByName(const String& name);

    // {secret}
    //  Print logging of all PUs whose name is in given list
    static void Report(const qList<String>& oListPUs);

    // {secret}
    // n2370
    const qChar* GetMaxTimeName() const
    {
        return m_strMaxTimeName.CStr();
    }
    const qChar* GetMinTimeName() const
    {
        return m_strMinTimeName.CStr();
    }

private:
    //Stores the sum of squares
    HighResolutionTime  m_oSumOfSquaredTime;
    //Stores the total time (sum) of all the event times that were declared in t he ProfilingUnit object
    HighResolutionTime  m_oTotalTime;
    //Stores the time taken by the quickest event
    HighResolutionTime  m_oMinTime;
    //Stores the time taken by the slowest event
    HighResolutionTime  m_oMaxTime;
    //Stores the time taken by the last event
    HighResolutionTime  m_oLastTime;
    Time                m_tiStartTime;
    //Stores the number of occurences
    qUnsignedInt32      m_uiNbOccurence;
    qUnsignedInt32      m_uiLoggingFrequency;
    String              m_strName;
    qBool               m_bAutoLogEnabled;
    qInt16              m_i16LastTracePeriod;

    //Unique ID for this ProfilingUnit object
    qUnsignedInt32 m_uiID;

    //n1769
    // IncAmount() increments this variable.
    qUnsignedInt32      m_ui32Amount;

    //n1769
    // Used for ordering of ProfilingUnit instance.
    qInt32              m_i32Order;

    // n2370 (もっとも時間がかかったもの、かからなかったものをマーク)
    String              m_strMaxTimeName;
    String              m_strMinTimeName;

    //First valid ID to assign to the next ProfilingUnit object that will be created
    static qUnsignedInt32 s_uiNextFreeID;
    //List of all ProfilingUnit objects
    static List                 s_lstProfilingUnits;
    static NAMESPACE_NEX CriticalSection      s_cs;

    //n1769
    static qBool s_bEnableProfiling;
};


// For use via SWIG, since inner classes are not supported.

class ProfilingScope: public ProfilingUnit::Scope
{
    // Group=
public:
    ProfilingScope(ProfilingUnit &pPU): ProfilingUnit::Scope(pPU) {}
    virtual ~ProfilingScope() {}

    qBool Record()
    {
        return ProfilingUnit::Scope::Record();
    }
    void Reset()
    {
        ProfilingUnit::Scope::Reset();
    }
};

END_NAMESPACE_NEX

