﻿/*---------------------------------------------------------------------------*
Copyright (C)2011-2018 Nintendo  All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo and/or its licensors and are
protected by national and international copyright laws.
They may not be disclosed to third parties or copied or duplicated
in any form, in whole or in part, without the prior written consent
of Nintendo.
The content herein is highly confidential and should be handled
accordingly.
*---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/MemoryManager.h>

// ライセンスの都合上ライブラリでのRapidJSONの使用が禁止されている場合はエラーにする。
// COMPILING_Q_PLATFORMはライブラリなどをビルドしている時に定義される。サンプルデモなどでは定義されない。
#if !defined(NEX_CAN_USE_RAPIDJSON_IN_LIBRARY) && defined(COMPILING_Q_PLATFORM)
#error RapidJSON can not be used on this platform.
#endif

// アプリが使うrapidjsonと競合しないように、NEXが使うrapidjsonのネームスペースを
// nn::nex::rapidjsonに置き換える。

// アプリによってRAPIDJSON_NAMESPACEがすでに定義されている場合はいったん待避する。
#ifdef RAPIDJSON_NAMESPACE
#define NEX_ORIGINAL_RAPIDJSON_NAMESPACE RAPIDJSON_NAMESPACE
#endif

#ifdef RAPIDJSON_NAMESPACE_BEGIN
#define NEX_ORIGINAL_RAPIDJSON_NAMESPACE_BEGIN RAPIDJSON_NAMESPACE_BEGIN
#endif

#ifdef RAPIDJSON_NAMESPACE_END
#define NEX_ORIGINAL_RAPIDJSON_NAMESPACE_END RAPIDJSON_NAMESPACE_END
#endif

// 新しいネームスペースを設定
#define RAPIDJSON_NAMESPACE NAMESPACE_NEX rapidjson
#define RAPIDJSON_NAMESPACE_BEGIN BEGIN_NAMESPACE_NEX namespace rapidjson {
#define RAPIDJSON_NAMESPACE_END   } END_NAMESPACE_NEX

// GCC互換コンパイラ以外ではエンディアンを設定する必要がある。
#ifndef RAPIDJSON_ENDIAN
#define NEX_RAPIDJSON_ENDIAN_DEFINED
// 0がRAPIDJSON_LITTLEENDIAN、1がRAPIDJSON_BIGENDIAN
#if NN_NEX_BYTE_ORDER == NN_NEX_BIG_ENDIAN
#define RAPIDJSON_ENDIAN 1
#else
#define RAPIDJSON_ENDIAN 0
#endif
#endif

// rapidjsonヘッダインクルード時に出る警告の抑止
// CTRでの警告抑止
#if defined(__ARMCC_VERSION)
#if __ARMCC_VERSION >= 5000000
#pragma push
#endif
#pragma diag_suppress 111,261,1303,2532,2803
#endif

// Cafeでの警告抑止
#ifdef CAFE_TARGET
#pragma ghs nowarning all
#endif

#include <rapidjson/document.h>
#include <rapidjson/writer.h>
#include <rapidjson/stringbuffer.h>
#include <rapidjson/allocators.h>
#include <rapidjson/error/en.h>

// 抑止した警告を元に戻す
#ifdef CAFE_TARGET
#pragma ghs endnowarning
#endif

#if defined(__ARMCC_VERSION)
#if __ARMCC_VERSION >= 5000000
#pragma pop
#endif
#endif

// アプリに影響しないように念のためエンディアン定義をundefする
#ifdef NEX_RAPIDJSON_ENDIAN_DEFINED
#undef RAPIDJSON_ENDIAN
#endif

// 待避した設定を元に戻す。
#undef RAPIDJSON_NAMESPACE
#ifdef NEX_ORIGINAL_RAPIDJSON_NAMESPACE
#define RAPIDJSON_NAMESPACE NEX_ORIGINAL_RAPIDJSON_NAMESPACE
#endif

#undef RAPIDJSON_NAMESPACE_BEGIN
#ifdef NEX_ORIGINAL_RAPIDJSON_NAMESPACE_BEGIN
#define RAPIDJSON_NAMESPACE_BEGIN NEX_ORIGINAL_RAPIDJSON_NAMESPACE_BEGIN
#endif

#undef RAPIDJSON_NAMESPACE_END
#ifdef NEX_ORIGINAL_RAPIDJSON_NAMESPACE_END
#define RAPIDJSON_NAMESPACE_END NEX_ORIGINAL_RAPIDJSON_NAMESPACE_END
#endif

#include <OnlineCore/src/Platform/Core/MemoryManager.h>

BEGIN_NAMESPACE_NEX

// RapidJson向けの標準アロケータ。mallocではなくMemoryManagerからメモリを確保する。
class RapidJsonNexAllocator
{
public:
    // RapidJson内でアロケータがnewされる為、アロケータ自体のnew/deleteも置き換える必要がある。
    static void *operator new(size_t size)
    {
        return MemoryManager::Allocate(size, MemoryManager::DirectCall);
    }

    void operator delete(void *p)
    {
        return Free(p);
    }

    static const bool kNeedFree = true;
    void* Malloc(size_t size)
    {
        if (size) //  behavior of malloc(0) is implementation defined.
        {
            return MemoryManager::Allocate(size, MemoryManager::DirectCall);
        }
        // rapidjson::CrtAllocatorがsize0のときにNULLを返しているのでそれを真似する
        return NULL; // standardize to returning NULL.
    }
    void* Realloc(void* originalPtr, size_t originalSize, size_t newSize)
    {
        if (newSize == 0)
        {
            Free(originalPtr);
            return NULL;
        }
        void* newPtr = Malloc(newSize);
        memcpy(newPtr, originalPtr, (std::min)(originalSize, newSize));
        Free(originalPtr);
        return newPtr;
    }
    static void Free(void *ptr)
    {
        MemoryManager::Free(ptr);
    }
};

// RapidJsonNexAllocatorをベースとしたRapidJson向けのメモリプールアロケータ。
typedef rapidjson::MemoryPoolAllocator<RapidJsonNexAllocator> RapidJsonNexMemoryPoolAllocator;

// NEX向けにrapidjson::StringBufferと同じようなものを定義する。
// rapidjson::StringBufferはcharに限定されているので、qCharを保持できるようにする。
#ifdef NEX_UNICODE_VERSION
typedef rapidjson::GenericStringBuffer<rapidjson::UTF16<qChar>, RapidJsonNexAllocator > RapidJsonNexStringBuffer;
#else
typedef rapidjson::GenericStringBuffer<rapidjson::UTF8<qChar>, RapidJsonNexAllocator > RapidJsonNexStringBuffer;
#endif

// NEX用のアロケータを使ったDocumentとValue。
// 今のところ入力がqByteのデシリアライズでしか使わないのでUTF8のみ定義する。
typedef rapidjson::GenericDocument<rapidjson::UTF8<>, RapidJsonNexMemoryPoolAllocator, RapidJsonNexAllocator> RapidJsonUtf8NexDocument;
typedef rapidjson::GenericValue<rapidjson::UTF8<>, RapidJsonNexMemoryPoolAllocator> RapidJsonUtf8NexValue;

// NEX用のアロケータを使ったWriter。
typedef rapidjson::Writer<RapidJsonNexStringBuffer, rapidjson::UTF8<>, rapidjson::UTF8<>, RapidJsonNexAllocator> RapidJsonUtf8NexWriter;

END_NAMESPACE_NEX
