﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/StringInter.h>
#include <OnlineCore/src/Platform/Core/StringStream.h>
#include <OnlineCore/src/Platform/Core/StringEncoding.h>

BEGIN_NAMESPACE_NEX

// Stringの内部バッファの文字列長(LENGTH)とNULL終端を含むサイズ(SIZE)
static const size_t STRING_INTERNAL_BUFFER_LENGTH = 15;
static const size_t STRING_INTERNAL_BUFFER_SIZE = STRING_INTERNAL_BUFFER_LENGTH + 1;

/*!
@brief    NEX で使用されるコア文字列型。

このクラスは基本的な文字列の機能を定義します。


CTR-NEX では UTF-16LE に対応しています。ただし、サロゲートペアには非対応です。


*/
class String : public RootObject
{
public:

/*!
   @brief 任意の文字列リテラルを格納するクラス。String::GetLiteralStringAs() により生成されます。
 */
template<typename CharT>
class LiteralStringHolder : public RootObject
{
public:
    /*!
       @brief コンストラクタです。内部に文字列を保持しないオブジェクトを生成します。CStr() は NULL を返します。
     */
    LiteralStringHolder()
        : m_pStr(NULL)
    {
    }

    /*!
       @brief コンストラクタです。src が保持する文字列をコピーしたオブジェクトを生成します。
     */
    explicit LiteralStringHolder(const String& src)
        : m_pStr(NULL)
    {
        src.CreateCopy(&m_pStr);
    }

    /*!
    @brief    コピーコンストラクタです。

    @param[in]    rhs    LiteralStringHolder オブジェクト。
    */
    LiteralStringHolder(const LiteralStringHolder& rhs)
        : m_pStr(NULL)
    {
        *this = rhs;
    }

#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    /*!
    @brief    ムーブコンストラクタです。

    @param[in]    rhs    LiteralStringHolder オブジェクト。
    */
    LiteralStringHolder(LiteralStringHolder&& rhs) NEX_NOEXCEPT
        : m_pStr(NULL)
    {
        *this = std::move(rhs);
    }

    /*!
    @brief    ムーブ代入演算子です。

    @param[in]    rhs    LiteralStringHolder オブジェクト。
    */
    LiteralStringHolder& operator=(LiteralStringHolder&& rhs) NEX_NOEXCEPT
    {
        if (this == &rhs)
        {
            return *this;
        }
        std::swap(m_pStr, rhs.m_pStr);
        return *this;
    }
#endif
    /*!
       @brief デストラクタです。
     */
    ~LiteralStringHolder()
    {
        String::ReleaseCopy(m_pStr);
    }

    /*!
    @brief    代入演算子です。

    @param[in]    rhs    LiteralStringHolder オブジェクト。
    */
    LiteralStringHolder& operator=(const LiteralStringHolder& rhs)
    {
        if (this == &rhs)
        {
            return *this;
        }
        CopyStringWithAlloc(&m_pStr, rhs.m_pStr);
        return *this;
    }

    /*!
       @brief NULL 終端された文字列を取得します。取得した文字列はこのオブジェクトを破棄するまで有効です。
     */
    const CharT* CStr() const
    {
        return m_pStr;
    }
private:
    CharT* m_pStr;
};

    // 初期化時にメモリ確保を伴わないようにするタグ
    struct NoAllocTag{};

    /*
    Parameters:
    szString: The string as a qChar8 or qChar16.
    */

    /*!
    @brief    コンストラクタです。

    空文字列がセットされます。動的なメモリ確保は伴いません。
    */
    String()
        : m_szContent(NULL)
    {
        InitializeContent();
    }

    /*!
    @brief    コンストラクタです。

    @param[in]    szString    qChar8 型のANSI形式の文字列。内部で各文字が qWideChar 型に変換されます。
    */
    String(const qChar8* szString)
        : m_szContent(NULL)
    {
        InitializeContent();
        Assign(szString);
    }
    /*!
    @brief    コンストラクタです。

    @param[in]    szString    qWideChar 型の文字列。
    */
    String(const qWideChar* szString)
        : m_szContent(NULL)
    {
        InitializeContent();
        Assign(szString);
    }
#ifdef NEX_QCHAR16_IS_NOT_WCHAR_T
    /*!
    @brief    コンストラクタです。

    @param[in]    szString    qChar16 型の文字列。
    */
    String(const qChar16* szString)
        : m_szContent(NULL)
    {
        InitializeContent();
        Assign(szString);
    }
#endif  // #ifdef NEX_QCHAR16_IS_NOT_WCHAR_T

    /*!
    @brief    コピーコンストラクタです。

    @param[in]    oString    String オブジェクト。
    */
    String(const String& oString)
        : m_szContent(NULL)
    {
        InitializeContent();
        Assign(oString.m_szContent);
    }

    /*
    @brief    コンストラクタです。

    @deprecated これは互換性のために残されている古い関数です。デフォルトコンストラクタと同じ動作をします。デフォルトコンストラクタを使用してください。
    @param[in]    dummy    オーバーロードを解決するためのダミー引数です。String::NoAllocTag() を渡してください。
    */
    String(NoAllocTag dummy)
        : m_szContent(NULL)
    {
        DUMMY_PARAM(dummy);
        InitializeContent();
    }

#ifdef NEX_MOVE_SEMANTICS_SUPPORT

    /*!
    @brief    ムーブコンストラクタです。

    @param[in]    rhs    String オブジェクト。
    */
    String(String&& rhs) NEX_NOEXCEPT
        : m_szContent(NULL)
    {
        InitializeContent();
        *this = std::move(rhs);
    }

    /*!
    @brief    ムーブ代入演算子です。

    @param[in]    rhs    String オブジェクト。
    */
    String& operator=(String&& rhs) NEX_NOEXCEPT
    {
        if (this == &rhs)
        {
            return *this;
        }
        Swap(rhs);
        return *this;
    }
#endif

    /*!
    @brief    デストラクタです。
    */
    virtual ~String()
    {
        DeleteContent();
    }

    /*!
    @brief    代入演算子です。

    @param[in]    szString    qChar8 型の文字列。内部で qWideChar 型に変換されます。
    */
    String& operator=(const qChar8* szString);
    /*!
    @brief    代入演算子です。

    @param[in]    szString    qWideChar 型の文字列。
    */
    String& operator=(const qWideChar* szString);
    /*!
    @brief    代入演算子です。

    @param[in]    oString    String オブジェクト。
    */
    String& operator=(const String& oString);
#if defined(NEX_QCHAR16_IS_NOT_WCHAR_T)
    /*!
    @brief    代入演算子です。

    @param[in]    szString    qChar16 型の文字列。
    */
    String& operator=(const qChar16* szString);
#endif
    /*!
    @brief    複合代入演算子です。文字列を結合します。

    @param[in]    strString    String オブジェクト。
    */
    String& operator+=(const String& strString);

    /*!
    @brief    比較演算子です。

    @param[in]    oString    比較する String オブジェクト。
    */
    qBool operator==(const String& oString) const
    {
        return IsEqual(this->m_szContent,oString.m_szContent);
    }

    /*!
    @brief    比較演算子です。

    @param[in]    szString   比較する String オブジェクト。
    */
    qBool operator==(const qChar* szString) const
    {
        return IsEqual(this->m_szContent,szString);
    }

    /*!
    @brief    比較演算子です。

    @param[in]    oString    比較する String オブジェクト。
    */
    qBool operator!=(const String& oString) const
    {
        return !IsEqual(this->m_szContent,oString.m_szContent);
    }

    /*!
    @brief    比較演算子です。

    @param[in]    szString   比較する String オブジェクト。
    */
    qBool operator!=(const qChar* szString) const
    {
        return !IsEqual(this->m_szContent,szString);
    }

    /*!
    @brief    比較演算子です。

    @param[in]    oString    比較する String オブジェクト。
    */
    qBool operator<(const String& oString) const;

    /*!
    @brief    比較演算子です。

    @param[in]    oString    比較する String オブジェクト。
    */
    qBool operator<=(const String& oString) const
    {
        return (*this)<oString || (*this)==oString;
    }

    /*!
    @brief    比較演算子です。

    @param[in]    oString    比較する String オブジェクト。
    */
    qBool operator>(const String& oString) const
    {
        return !((*this)<=oString);
    }

    /*!
    @brief    比較演算子です。

    @param[in]    oString    比較する String オブジェクト。
    */
    qBool operator>=(const String& oString) const
    {
        return !((*this)<oString);
    }

    /*
    Description: Truncates the string to the specified number of characters.
    Parameters:
    iLength: The new length of the string in number of characters.
    */

    /*!
     @brief    指定した文字列長以降を切り捨てた String オブジェクトを作成します。

     @param[in]    uiLength    文字列長
     @return     指定した文字列長以降を切り捨てた String オブジェクト
     */
    String Truncate(size_t uiLength) const;

    /*
    Description: Returns a pointer to a constant C-style version of the string.
    Returns: A pointer to a constant C-style version of the string.
    */

    /*!
     @brief    文字列を C 形式のポインタで返します。
     @return     文字列の C 形式のポインタ
     */
    const qChar* CStr() const
    {
        return m_szContent;
    }

    /*
    Description: Returns a pointer to the underlying string buffer.
    Returns: A pointer to the underlying string buffer to enable direct access to it.
    */

    /*!
     @brief    オブジェクト内部の文字列バッファを返します。
     @return     文字列バッファ
     */
    qChar* GetBufferPtr()
    {
        return m_szContent;
    }
    /*!
     @brief    オブジェクト内部の文字列バッファを返します（const版）。
     @return     文字列バッファ
     */
    const qChar* GetBufferPtr() const
    {
        return m_szContent;
    }

    /*
     @internal
     @brief    オブジェクト内部の文字列バッファをセットします。バッファの所有権はこのオブジェクトに移ります。NULL 終端文字列を指すバッファを指定してください。
     @param[in]    bufferPtr    文字列バッファ。SpecialNewArray() で確保したバッファを指定してください。
     */
    void SetBufferPtr(qChar* bufferPtr);

    /*!
     @brief    オブジェクト内部の情報を入れ替えます。
     @param[in]    rhs    入れ替える対象のオブジェクト。
    */
    void Swap(String& rhs) NEX_NOEXCEPT
    {
        qBool imUsedInternalBuffer = IsUsingInternalBuffer();
        qBool rhsUsedInternalBuffer = rhs.IsUsingInternalBuffer();
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
        std::swap(m_internalBuffer, rhs.m_internalBuffer);
#else
        qChar tempBuffer[STRING_INTERNAL_BUFFER_SIZE];
        memcpy(tempBuffer, m_internalBuffer, sizeof(m_internalBuffer));
        memcpy(m_internalBuffer, rhs.m_internalBuffer, sizeof(m_internalBuffer));
        memcpy(rhs.m_internalBuffer, tempBuffer, sizeof(m_internalBuffer));
#endif
        std::swap(m_szContent, rhs.m_szContent);
        std::swap(m_contentCapacitySize, rhs.m_contentCapacitySize);

        // もし内部バッファを使っていたらお互い別のオブジェクトの内部バッファを指すようになってしまうので修正する。
        if (imUsedInternalBuffer)
        {
            // もし自分が内部バッファを使っていたのなら入れ替えた相手も内部バッファを指すようにする。
            rhs.m_szContent = rhs.m_internalBuffer;
        }
        if (rhsUsedInternalBuffer)
        {
            // もし相手が内部バッファを使っていたのなら入れ替えた自分も内部バッファを指すようにする。
            m_szContent = m_internalBuffer;
        }
    }

    /*
    Description: Preallocates memory for the string.
    Parameters:
    iLength: The new length of the string buffer in number of characters.
    Remarks: This function deletes the current string regardless of its size. It guarantees
    that the buffer returned by GetBufferPtr can hold at least iLength qChar(s).
    See Also: GetBufferPtr
    */

    /*!
     @brief    指定した文字列長分の文字列を格納できるメモリを事前を確保します。

     内部の文字列はそのまま保持されます。もし length 分の文字列を確保可能なバッファをすでに持っている場合は何も行いません。

     @param[in]    length     文字列長。(終端含まない)
     @see String::Capacity()
     */
    void Reserve(size_t length);

    /*!
     @brief    メモリの再割り当てなしで保持可能な文字列長を取得します。

     @return    メモリの再割り当てなしで保持可能な文字列長。(終端含まない)
     @see String::Reserve()
     */
    size_t Capacity() const
    {
        return m_contentCapacitySize - 1;
    }

    /*!
     @brief    内部バッファを現在の文字列長に切り詰めます。
     */
    void ShrinkToFit()
    {
        size_t currentLength = GetLength();
        if (m_contentCapacitySize > currentLength + 1)
        {
            String tempString = Truncate(currentLength);
            this->Swap(tempString);
        }
    }

    /*!
     @internal
     @brief   事前に確保したバッファに文字列をコピーします。

       String::Reserve() によって事前に確保されたバッファに strSrc を NULL 終端を含めコピーします。
       NULL 終端を含めた strSrc のサイズより大きいサイズを事前に String::Reserve() で確保する必要があります。
       代入演算子を使った場合はバッファを自動拡張するのに対し、こちらはサイズが足りない場合に SYSTEMCHECK により停止します。

     @param[in]    strSrc     セットする NULL 終端文字列
     @see String::Reserve()
     */
    void SetStringToPreReservedBuffer(const qChar* strSrc);

#ifdef NEX_UNICODE_VERSION
    /*!
     @internal
     @brief   事前に確保したバッファに文字列をコピーします。

       String::Reserve() によって事前に確保されたバッファに strSrc を NULL 終端を含めコピーします。
       NULL 終端を含めた strSrc のサイズより大きいサイズを事前に String::Reserve() で確保する必要があります。
       代入演算子を使った場合はバッファを自動拡張するのに対し、こちらはサイズが足りない場合に SYSTEMCHECK により停止します。

     @param[in]    strSrc     セットする NULL 終端文字列
     @see String::Reserve()
     */
    void SetStringToPreReservedBuffer(const qChar8* strSrc);
#endif

    /*
    Description: Copies the string into a newly allocated buffer.
    Parameters:
    szCopy: A pointer to a pointer containing the address of the newly allocated buffer.
    Remarks: This function allocates the memory pointed to by szCopy. ReleaseCopy must be called
    after CreateCopy is called. If the string is Unicode, this function
    will convert the string to Ansi before copying it.
    See Also: ReleaseCopy
    */

    /*!
     @brief    文字列のコピーを作成します。

     引数で与えられたポインタに文字列長分のメモリを確保し、コピーします。
     内部で ANSI 形式に変換するため、元の文字列が ANSI 形式に変換できる必要があります。

     本関数で生成した文字列は、 @ref ReleaseCopy() で開放してください。

     @param[out]    szCopy     qChar8 の C 形式文字列へのポインタ
     */
    void CreateCopy(qChar8 **szCopy) const ;

    /*
    Description: Releases a copy of a string buffer copied by the CreateCopy function.
    Parameters:
    szCopy: A pointer to the buffer that was allocated by CreateCopy
    Remarks: This function must be called after CreateCopy has been called. You should
    not delete the buffer.
    See Also: CreateCopy
    */

    /*!
     @brief    文字列を解放します。

     @ref CreateCopy() でコピーした文字列に対して使用してください。

     @param[in,out] szCopy     qChar8 の C 形式文字列
     */
    static void ReleaseCopy(qChar8 *szCopy);

    // CopyString() は qChar8<=>qChar16, qChar8<=>qWideChar の変換が入る場合に失敗する可能性がある。
    // そのため、内部で保持している文字列の形式によっては失敗する。つまりNEX_UNICODE_VERSIONではqChar8を引数に取るものが、
    // 非NEX_UNICODE_VERSIONではqChar16, qWideCharを引数に取るものが返値として0を返す可能性がある。
    /*!
     @brief    文字列のコピーを作成します。

     引数で与えられたポインタに文字列をコピーします。あらかじめメモリが確保されている必要があります。
     内部で ANSI 形式に変換するため、元の文字列が ANSI 形式に変換できる必要があります。
     size が 0 以外の場合は、必ず \\0 で終端されます。

     @param[out]    pCopy  qChar8 の C 形式文字列領域へのポインタ。
     @param[in]     size   文字列領域のサイズ ( 終端の \\0 を含む ) 。*pCopy の型換算の長さ(バッファサイズ / 型サイズ)です。)
     @return        コピーした文字列領域のサイズ。 *pCopy の型換算の長さ(バッファサイズ / 型サイズ)です。  終端の \\0 を含みます。
                    変換失敗時には0となり、 pCopy 先頭に \\0 が入ります。
     */
    size_t CopyString(qChar8 *pCopy, size_t size) const;

    /*!
     @brief    文字列のコピーを作成します。

     引数で与えられたポインタに文字列長分のメモリを確保し、コピーします。
     

     本関数で生成した文字列は、 @ref ReleaseCopy() で開放してください。

     @param[out]    szCopy     qWideChar の C 形式文字列へのポインタ
     */
    void CreateCopy(qWideChar **szCopy) const;
    /*!
     @brief    文字列を解放します。

     @ref CreateCopy() でコピーした文字列に対して使用してください。

     @param[in,out] szCopy     qWideChar の C 形式文字列
     */
    static void ReleaseCopy(qWideChar *szCopy);

    /*!
     @brief    文字列のコピーを作成します。

     引数で与えられたポインタに文字列をコピーします。あらかじめメモリが確保されている必要があります。
     
     size が 0 以外の場合は、必ず \\0 で終端されます。

     @param[out]    pCopy  qWideChar の C 形式文字列領域へのポインタ。
     @param[in]     size   文字列領域のサイズ ( 終端の \\0 を含む ) 。*pCopy の型換算の長さ(バッファサイズ / 型サイズ)です。)
     @return        コピーした文字列領域のサイズ。 *pCopy の型換算の長さ(バッファサイズ / 型サイズ)です。  終端の \\0 を含みます。
                    
     */
    size_t CopyString(qWideChar *pCopy, size_t size) const;

#if defined(NEX_QCHAR16_IS_NOT_WCHAR_T)
    /*!
     @brief    文字列のコピーを作成します。

     引数で与えられたポインタに文字列長分のメモリを確保し、コピーします。
     

     本関数で生成した文字列は、 @ref ReleaseCopy() で開放してください。

     @param[out]    szCopy     qChar16 の C 形式文字列へのポインタ
     */
    void CreateCopy(qChar16 **szCopy) const;
    /*!
     @brief    文字列を解放します。

     @ref CreateCopy() でコピーした文字列に対して使用してください。

     @param[in,out] szCopy     qChar16 の C 形式文字列
     */
    static void ReleaseCopy(qChar16 *szCopy);

    /*!
     @brief    文字列のコピーを作成します。

     引数で与えられたポインタに文字列をコピーします。あらかじめメモリが確保されている必要があります。
     
     size が 0 以外の場合は、必ず \\0 で終端されます。

     @param[out]    pCopy  qChar16 の C 形式文字列領域へのポインタ。
     @param[in]     size   文字列領域のサイズ ( 終端の \\0 を含む ) 。*pCopy の型換算の長さ(バッファサイズ / 型サイズ)です。)
     @return        コピーした文字列領域のサイズ。 *pCopy の型換算の長さ(バッファサイズ / 型サイズ)です。  終端の \\0 を含みます。
                    
     */
    size_t CopyString(qChar16 *pCopy, size_t size) const;
#endif // end of NEX_QCHAR16_IS_NOT_WCHAR_T
    /*
    Description: Returns the length of the string in number of characters.
    Returns: The length of the string in number of characters.
    */

    /*!
     @brief    テンプレート引数で指定した型で文字列を取得します。

     本関数呼び出し後に String::LiteralStringHolder<CharT>::CStr() を呼び出すことで、任意の文字列型で内部値を取得できます。
     String::CStr() は qChar 型を返しますが、それ以外の型で文字列を取得したい場合に使用してください。
     例えば、 obj.GetLiteralStringAs<qChar8>.CStr() と呼び出すことで const qChar8* を受け取る関数の引数に文字列を渡すことが出来ます。
     テンプレート引数には qChar8、qChar16、qWideChar などの文字列型を指定できます。

     @return        テンプレート引数で指定した型の文字列を格納したオブジェクト。
                    その後 String::LiteralStringHolder<CharT>::CStr() を呼び出して文字列型を取得してください。
     */
    template<typename CharT>
    LiteralStringHolder<CharT> GetLiteralStringAs() const
    {
        return LiteralStringHolder<CharT>(*this);
    }

    /*!
     @brief    文字列長を返します。NULL 終端は含みません。 

     @return    NULL 終端を含まない文字列長

     
     */
    size_t GetLength() const;

#if defined(NEX_SIGLO_FAMILY)
    /*!
     @brief    ワイドキャラクタ換算での文字列長を返します。結合文字列 (Combining Character Sequence) は、別々の文字として扱われます。
               

     @return    文字列長

     @see GetLength(), GetUtf8Length()
     */
#endif
    qInt32 GetWideCharLength() const;
#if defined(NEX_SIGLO_FAMILY)
    /*!
     @brief    UTF-8 符号を解釈した文字列長を返します。結合文字列 (Combining Character Sequence) は、別々の文字として扱われます。
               UTF-8 符号の解釈が失敗した場合は、 -1 を返します。

     @return    文字列長

     @see GetLength(), GetWideCharLength()
     */
    qInt32 GetUtf8Length() const { return GetWideCharLength(); }
#endif

    /*!
     @brief    文字列長が 0 なら true を返します。

     @return    文字列長が 0 かどうか。
     */
    qBool Empty() const
    {
        return GetLength() == 0;
    }

    /*
    Description: Uses a printf style format specifier to create a string.
    Parameters:
    szFormat: The format specifier string.

    Note: The parameter "..." represents all other arguments requested by szFormat.

    Remarks: The resulting string cannot be larger than NEX_LOG_STRING_BUFFER_SIZE (256) characters, otherwise it will
    be truncated. Refer to the documentation of the standard C function printf (in any good C++ book)
    for a description of the syntax and a list of the different format specifiers.
    */
    void __CDECL Format(const qChar* szFormat, ...);

    /*
    Description: Convert a string to uppercase
    */

    /*!
     @brief    大文字に変換します。
     */
    void ToUpper();

    /*
    Description: Convert a string to lowercase
    */

    /*!
     @brief    小文字に変換します。
     */
    void ToLower();

    /*
    Description: Find the location of a substring, regarding of the
    letter case
    Returns: position of first occurence of substring; -1 if not
    found
    */

    /*!
     @brief    文字列を検索します。大文字と小文字は区別されます。

     @param[in] szSubstring    検索文字列
     @param[in] poi    開始位置 0は先頭　(省略時0)
     @return    最初にマッチした文字列の開始位置。 0は先頭。存在しない場合は -1 を返します。
     */
    qInt FindSubstringCase(const qChar* szSubstring, qInt poi = 0) const;

    /*
    Description: Find the location of a substring, without regarding of the
    letter case
    Returns: position of first occurence of substring; -1 if not
    found
    */

    /*!
     @brief    文字列を検索します。大文字と小文字の区別はしません。

     @param[in] szSubstring    検索文字列

     @return    最初にマッチした文字列の開始位置。 存在しない場合は -1 を返します。
     */
    qInt FindSubstringNoCase(const qChar* szSubstring) const;

    /*
    Description: Determines if a String contains a certain substring without
    ignoring case.
    Returns: true if the string contains the string parameter, false otherwise
    */

    /*!
     @brief    引数の文字列を含むかどうかを返します。大文字と小文字は区別されます。

     @param[in] strSubstr    検索文字列

     @retval    true    検索文字列を含む
     @retval    false   検索文字列を含まない
     */
    qBool ContainsCase(const String &strSubstr) const;

    /*
    Description: Determines if a String contains a certain substring while
    ignoring case.
    Returns: true if the string contains the string parameter, false otherwise
    */

    /*!
     @brief    引数の文字列を含むかどうかを返します。大文字と小文字の区別はしません。

     @param[in] strSubstr    検索文字列

     @retval    true    検索文字列を含む
     @retval    false   検索文字列を含まない
     */
    qBool ContainsNoCase(const String &strSubstr) const;

    // StringをqUnsignedInt64に変換する。変換できなかったら0を返す。
    qUnsignedInt64 ToUInt64() const;

    /*!
     @brief    入力バイナリデータを base64 形式の qChar 文字列に変換します。

     szADest に NULL を指定した場合、変換を行わず、変換に必要な szADest のサイズを返します。必要サイズはNULL終端文字に必要な領域分を含みます。

     @param[in]   pbSrc         入力データへのポインタ
     @param[in]   uiSrcLength   入力データのサイズ
     @param[out]  szADest       base64 形式文字列の出力先バッファへのポインタ。変換に成功すると末尾にNULL終端文字が付与されます。
     @param[in]   uiDestLength  szADest で示されるバッファのサイズ(NULL終端文字分を含む)

     @return 変換に成功したかどうか、成功した場合は変換後の文字列長を返します
     @retval   -1   変換失敗(出力バッファのサイズ不足)
     @retval   それ以外   変換後の文字列長
     */
    static int ByteArrayToBase64(const qByte* pbSrc, size_t uiSrcLength, qChar* szADest, size_t uiDestLength);

    /*!
     @brief    base64 形式の文字列からバイナリデータに変換します。

     pbDest に NULL を指定した場合、変換を行わず、変換に必要な pbDest のサイズ(理論値)を返します。

     @param[in]   szASrc           入力 base64 形式文字列へのポインタ。
     @param[in]   uiSrcLength      入力 base64 形式文字列の長さ。4の倍数である必要があります。NULL終端文字列は含みません。
     @param[out]  pbDest           出力データ格納先バッファへのポインタ
     @param[in]   uiDestLength     pbDest で示されるバッファのサイズ

     @return 変換に成功したかどうか、成功した場合は変換後のバイナリデータ長を返します
     @retval   -1   変換失敗(出力バッファのサイズ不足)
     @retval   それ以外   変換後のバッファサイズ
     */
    static int Base64ToByteArray(const qChar* szASrc, size_t uiSrcLength, qByte* pbDest, size_t uiDestLength);

    /*!
     @brief    base64 形式の文字列からバイナリデータに変換します。

     pbDest に NULL を指定した場合、変換を行わず、変換に必要な pbDest のサイズ(理論値)を返します。

     @param[in]   strSrc           入力 base64 形式文字列
     @param[out]  pbDest           出力データ格納先バッファへのポインタ
     @param[in]   uiDestLength     pbDest で示されるバッファのサイズ

     @return 変換に成功したかどうかを返します
     @retval   -1   変換失敗(出力バッファのサイズ不足。base64 形式の文字列ではない)
     @retval   それ以外   変換後のバッファサイズ
     */
    static int Base64ToByteArray(const String& strSrc, qByte* pbDest, size_t uiDestLength);

    //  Trace String contents at byte level
    // {secret}
    void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS);

    /*
    Description: Defines how your game/application encodes strings in
    $PRODUCTNAME$ RMCs.
    Parameters:
    uiCode: Accepted values are UTF8 (the default).
    Remarks: $PRODUCTNAME$ uses UTF-8 in its RMCs.
    */
    static void SetDefaultStringEncoding(qUnsignedInt32 uiCode);

    /*
    Description: Returns the type of encoding your game/application uses
    for strings in calls to/from $PRODUCTNAME$.
    Remarks: Returns values: StringEncoding::Latin1, StringEncoding::UTF8 or StringEncoding::Unicode
    */
    static qUnsignedInt32 GetDefaultStringEncoding()
    {
        return s_uiDefaultStringEncoding;
    }

    // {secret}
    static qBool IsEqual(const qChar* lhs, const qChar* rhs);

private:
    /*
    @internal
    @brief    TConvertFrom型の文字列をqCharに変換し内部にコピーします。コンストラクタで呼び出す場合は事前にInitializeContent()されている必要があります。

    ただし、変換後のサイズがdestinationCapacityを上回っている場合はpszDestinationにメモリを再割り当てしてから書き込みます。

    @param[in]    szSource    コピー元の文字列。
    */
    template<typename TConvertFrom>
    void Assign(const TConvertFrom* szSource);

    /*
    @internal
    @brief    内部値を空文字列に設定します。動的バッファを確保している場合は事前にDeleteContentなどで解放しておく必要があります。
    */
    void InitializeContent()
    {
        m_szContent = m_internalBuffer;
        m_szContent[0] = 0;
        m_contentCapacitySize = STRING_INTERNAL_BUFFER_SIZE;
    }

    /*
    @internal
    @brief    内部で保持しているバッファを解放します。
              内部値は不定になります。このオブジェクトを再使用するにはInitializeContent()を呼び出すか、m_szContentとm_contentCapacitySizeを再設定する必要があります。
    */
    void DeleteContent();

    qBool IsUsingInternalBuffer() const
    {
        return m_szContent == m_internalBuffer;
    }

    qChar m_internalBuffer[STRING_INTERNAL_BUFFER_SIZE];
    qChar* m_szContent;
    size_t m_contentCapacitySize;

    static qUnsignedInt32 s_uiDefaultStringEncoding;
};

// Additional String operators.  These are overloaded for speed.
// {secret}
String operator+(const String & strLeft, const String & strRight);
// {secret}
String operator+(const String & strLeft, const qChar * szRight);
// {secret}
String operator+(const qChar * szLeft, const String & strRight);

// String streaming.
// {secret}
extern StringStream& operator<<(StringStream& oStream, const String& strValue);

END_NAMESPACE_NEX

