﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDeclInternal.h>
#include <stdarg.h>

BEGIN_NAMESPACE_NEX

//==============================================================================
// Internal class for text strings stream.
//==============================================================================
/*!
 @brief 文字列ストリームを扱うクラスです。
 */
class StringStream: public RootObject
{
public:
    static const qUnsignedInt32 STRINGSTREAM_INITIAL_SIZE=256;

    StringStream();
    virtual ~StringStream();

    // Returns the stream length
    size_t GetLength() const;

    // Returns the m_szBuffer size (capacity)
    size_t GetSize() const
    {
        return m_uiSize;
    }

    /*!
     @brief  文字列を C 形式のポインタで返します。
     @return 文字列を C 形式のポインタで返します。
     */
    const qChar* CStr() const
    {
        return m_szBuffer;
    }

    void Clear();

    // Streaming of basic types
    /*!
     @brief  文字列を追加します。
     @param[in] szText 追加する文字列を指定します。
     @return 文字列追加後のストリームが返ります。
     */
    StringStream& operator<<(const qChar* szText);
    /*!
     @brief  ポインタ値を文字列に変換して追加します。
     @param[in] pPointer 文字列として追加するポインタを指定します。
     @return 文字列追加後のストリームが返ります。
     */
    StringStream& operator<<(const void* pPointer);
    /*!
     @brief  qBool値を文字列に変換して追加します。
     @param[in] bValue 文字列として追加するqBool値を指定します。
     @return 文字列追加後のストリームが返ります。
     */
    StringStream& operator<<(qBool bValue);
    /*!
     @brief  qDouble値を文字列に変換して追加します。
     @param[in] dValue 文字列として追加するqDouble値を指定します。
     @return 文字列追加後のストリームが返ります。
     */
    StringStream& operator<<(qDouble dValue);
    /*!
     @brief  qFloat値を文字列に変換して追加します。
     @param[in] fValue 文字列として追加するqFloat値を指定します。
     @return 文字列追加後のストリームが返ります。
     */
    StringStream& operator<<(qFloat fValue);
    /*!
     @brief  qByte値を文字列に変換して追加します。
     @param[in] uiValue 文字列として追加するqByte値を指定します。
     @return 文字列追加後のストリームが返ります。
     */
    StringStream& operator<<(qByte uiValue)
    {
        return StreamNumber(uiValue);
    }

    /*!
     @brief  qInt8値を文字列に変換して追加します。
     @param[in] iValue 文字列として追加するqInt8値を指定します。
     @return 文字列追加後のストリームが返ります。
     */
    StringStream& operator<<(qInt8 iValue)
    {
        return StreamNumber(iValue);
    }
    /*!
     @brief  qUnsignedInt16値を文字列に変換して追加します。
     @param[in] uiValue 文字列として追加するqUnsignedInt16値を指定します。
     @return 文字列追加後のストリームが返ります。
     */
    StringStream& operator<<(qUnsignedInt16 uiValue)
    {
        return StreamNumber(uiValue);
    }
    /*!
     @brief  qInt16値を文字列に変換して追加します。
     @param[in] iValue 文字列として追加するqInt16値を指定します。
     @return 文字列追加後のストリームが返ります。
     */
    StringStream& operator<<(qInt16 iValue)
    {
        return StreamNumber(iValue);
    }
    /*!
     @brief  qUnsignedInt32値を文字列に変換して追加します。
     @param[in] uiValue 文字列として追加するqUnsignedInt32値を指定します。
     @return 文字列追加後のストリームが返ります。
     */
    StringStream& operator<<(qUnsignedInt32 uiValue)
    {
        return StreamNumber(uiValue);
    }
    /*!
     @brief  qInt32値を文字列に変換して追加します。
     @param[in] iValue 文字列として追加するqInt32値を指定します。
     @return 文字列追加後のストリームが返ります。
     */
    StringStream& operator<<(qInt32 iValue)
    {
        return StreamNumber(iValue);
    }

    /*!
     @brief  qUnsignedInt64値を文字列に変換して追加します。
     @param[in] uiValue 文字列として追加するqUnsignedInt64値を指定します。
     @return 文字列追加後のストリームが返ります。
     */
    StringStream& operator<<(qUnsignedInt64 uiValue);

    /*!
     @brief  qInt64値を文字列に変換して追加します。
     @param[in] iValue 文字列として追加するqInt64値を指定します。
     @return 文字列追加後のストリームが返ります。
     */
    StringStream& operator<<(qInt64 iValue);

    /*!
     @brief  文字列ストリームを追加します。
     @param[in] oSS 追加する文字列ストリームを指定します。
     @return 追加後のストリームが返ります。
     */
    StringStream& operator<<(const StringStream& oSS);

#if !defined(NEX_LONG_IS_UINT64_T)
    /*!
     @brief  long値を文字列に変換して追加します。
     @param[in] iValue 文字列として追加するlong値を指定します。
     @return 文字列追加後のストリームが返ります。
     */
    StringStream& operator<<(long iValue)
    {
#ifdef NEX_LONG_IS_64BIT
        return (*this) << static_cast<qInt64>(iValue);
#else
        return (*this) << static_cast<qInt32>(iValue);
#endif
    }

    /*!
     @brief  unsigned long値を文字列に変換して追加します。
     @param[in] uiValue 文字列として追加するunsigned long値を指定します。
     @return 文字列追加後のストリームが返ります。
     */
    StringStream& operator<<(unsigned long uiValue)
    {
#ifdef NEX_LONG_IS_64BIT
        return (*this) << static_cast<qUnsignedInt64>(uiValue);
#else
        return (*this) << static_cast<qUnsignedInt32>(uiValue);
#endif
    }
#endif

#if defined(MACOSX_TARGET) || defined(WII_TARGET) || defined (CAFEIOP_TARGET)
    // How does size_t is mapped in Unix targets??
    StringStream& operator<<(size_t uiValue)
    {
        return StreamNumber(qUnsignedInt32(uiValue));
    }
#endif

    // Manipulators
    StringStream& operator<<(StringStream& (*pf)(StringStream&))
    {
        return (*pf)(*this);
    }

    StringStream& Manip_hex()
    {
        m_bHex = true;
        m_bZeroHex =false;
        return *this;
    }
    StringStream& Manip_dec()
    {
        m_bHex = false;
        m_bZeroHex = false;
        return *this;
    }
    StringStream& Manip_zerohex()
    {
        m_bHex = true;
        m_bZeroHex =true;
        return *this;
    }

    StringStream& Manip_showbase()
    {
        m_bShowBase = true;
        return *this;
    }
    StringStream& Manip_noshowbase()
    {
        m_bShowBase = false;
        return *this;
    }

    StringStream& Manip_boolalpha()
    {
        m_bBoolAlpha = true;
        return *this;
    }
    StringStream& Manip_noboolalpha()
    {
        m_bBoolAlpha = false;
        return *this;
    }

    const qChar& operator[](size_t uiPos) const
    {
        return m_szBuffer[uiPos];
    }

    StringStream& BytesDump(const qByte *qBytes, size_t len);

    StringStream& BytesAsciiDump(const qByte *qBytes, size_t len);

private:
    // Disable these methods.
    StringStream(const StringStream&);
    StringStream& operator=(const StringStream&);

private:
    void FreeBuffer();
    void FreeBuffer(qChar *szBuffer);
    void ResizeBuffer(size_t uiSize);
    void AddBaseIfRequired();

    StringStream& StreamNumber(qByte uiValue);
    StringStream& StreamNumber(qUnsignedInt32 uiValue);
    StringStream& StreamNumber(qInt32 iValue);
    void TestFreeRoom(size_t uiNewSize);

private:
    qChar* m_szBuffer;
    size_t m_uiSize;
    qChar* m_szCurrentPosition;

    qChar m_szInitialBuffer[STRINGSTREAM_INITIAL_SIZE];

    qBool m_bHex;
    qBool m_bShowBase;
    qBool m_bBoolAlpha;

    qBool m_bZeroHex;

};

// Manipulators.

/*!
 @brief  整数値の出力フォーマットを16進数にします。
 @param[in] oStringStream 文字列ストリームを指定します。
 @return 文字列ストリームが返ります。
*/
extern StringStream& hex(StringStream& oStringStream);
/*!
 @brief  整数値の出力フォーマットを10進数にします。
 @param[in] oStringStream 文字列ストリームを指定します。
 @return 文字列ストリームが返ります。
*/
extern StringStream& dec(StringStream& oStringStream);
/*!
 @brief  改行コードを追加します。
 @param[in] oStringStream 文字列ストリームを指定します。
 @return 文字列ストリームが返ります。
*/
extern StringStream& endl(StringStream& oStringStream);
/*!
 @brief  ポインタ値、整数値の出力時に0xを頭につけるようにします。
 @param[in] oStringStream 文字列ストリームを指定します。
 @return 文字列ストリームが返ります。
*/
extern StringStream& showbase(StringStream& oStringStream);
/*!
 @brief  ポインタ値、整数値の出力時に0xを頭につけないようにします。
 @param[in] oStringStream 文字列ストリームを指定します。
 @return 文字列ストリームが返ります。
*/
extern StringStream& noshowbase(StringStream& oStringStream);
/*!
 @brief  qBool値の出力フォーマットをtrue,falseにします。
 @param[in] oStringStream 文字列ストリームを指定します。
 @return 文字列ストリームが返ります。
*/
extern StringStream& boolalpha(StringStream& oStringStream);
/*!
 @brief  qBool値の出力フォーマットを1,0にします。
 @param[in] oStringStream 文字列ストリームを指定します。
 @return 文字列ストリームが返ります。
*/
extern StringStream& noboolalpha(StringStream& oStringStream);

extern StringStream& zerohex(StringStream& oStringStream);
END_NAMESPACE_NEX

