﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Platform/Core/Time.h>

BEGIN_NAMESPACE_NEX

// Group=Time Management

/*
Description: Sets, resets and determines the time.

Remarks:

$ifdef NETZ$
The time provided by SystemClock is not synchronized across the stations that belong to the session.
For synchronized times, refer to SessionClock.
$endif$

The SystemClock time is the time elapsed, in ms, since the process started.
The time never wraps (it is calculated and stored using 64 bits).

The precision of the Time value is subject to the capabilities of the time provider
implemented on the various platforms. $PRODUCTNAME$ provides
a default time provider, but you can also use your own (see
SystemClock::RegisterTimeProvider)

$PRODUCTNAME$ provides two time providers on windows, each with its own
restrictions, see SystemClock::UsePerformanceCounter and
SystemClock::UseMultimediaTimer for details.

See Also: Time, SessionClock
*/

/*!
 * @brief   時間を決定したり、設定したり、リセットしたりします。
 *
 * SystemClockクラスによって提供される時間は、セッションにいるすべてのステーションによって同期されたりはしません。
 * セッション中で同期される時刻管理については、@ref SessionClock クラスを利用してください。
 *
 * SystemClockが提供する時間はプロセス起動時からのミリ秒単位における時間経過を表します。
 * 時間経過は64bitで計算されていますので、実運用上でオーバーフローすることはほぼありません。
 *
 * 時刻値の精度はプラットフォームのタイムプロバイダの性能によって決定されます。
 * デフォルトでタイムプロバイダが設定されていますが、アプリケーションごとに
 * 別のタイムプロバイダを作成して設定しなおすことも出来ます(@ref RegisterTimeProvider 関数を参照)。
 *
 * 
 * デフォルトのタイムプロバイダ使用時の時間取得の処理負荷は小さくありませんので、頻繁に呼ぶ際はパフォーマンスに注意が必要です。
 * 
 */
class SystemClock: public RootObject
{
public:
    /*
     Remarks: Do not instantiate this object. It is created by
     $PRODUCTNAME$ when a station joins or creates a session and is initialized to 0 ms.
    */

    /*!
     * @brief     アプリケーション側からこのコンストラクタを呼び出さないでください。
     *
     * インスタンスは、セッションに参加するかセッションを作成したときに、
     * 0ミリ秒で初期化されながら自動的に作成されます。
     */
    SystemClock();
    /*
      Remarks: Do not destroy this object. $PRODUCTNAME$ destroys it when the local station leaves the session or the session terminates.
    */

    /*!
     * @brief     アプリケーション側からこのデストラクタを呼び出さないでください。
     *
     * インスタンスはセッションを抜けるか、セッションが終了したときに
     * システムによって自動的に解放されます。
     */
    virtual ~SystemClock();

    /*
    Description: Registers the specified TimeProvider to an object of this class.

    Parameters:
    pTimeProvider: A pointer to the TimeProvider to registered with this object.
    By default, this class uses the $PRODUCTNAME$ internal, platform-specific TimeProvider.

    Remarks: A custom TimeProvider (such as a motherboard timing
    chip providing quality high-resolution timing information, or time
    information retrieved from the game physics engine) may be used to provide the time. Before this class can retrieve
    the time from such a custom TimeProvider, you must register it through this method.

    Note that you must call RegisterTimeProvider before creating the $PRODUCTNAME$ object as
    follows:

            // Call these two lines before you create the NetZ object
            MyTimeProvider oTimeProvider;
            SystemClock::RegisterTimeProvider(&oTimeProvider);

    See Also:
    */

    /*!
     * @brief     このクラスに対してタイムプロバイダを設定します。
     *
     * タイムプロバイダ(@ref TimeProvider クラス)とは、時間値を提供するクラスオブジェクトです。
     * SystemClockクラスの時間精度は、タイムプロバイダの性能に左右されます。
     * この関数によって、カスタマイズされた(たとえば高精度モードの)タイムプロバイダを設定することが出来ます。
     *
     * このクラスは、 NEX の Core オブジェクトの生成前に設定しておく必要があります。
     * この関数が呼び出されない場合、デフォルトでシステムが用意しているタイムプロバイダが利用されます。
     *
     * @param[in] pTimeProvider  タイムプロバイダオブジェクトのポインタを指定します。 NULLの場合は、デフォルトのタイムプロバイダが使われます。
     * @param[in] needCorrection タイムプロバイダが返す値が、稼働中に一周回ってしまい0になることがあるため補正が必要かを設定します。
     *                           たとえば、TimeProviderとして、msecオーダーで32ビットの値を返す時刻をもとにしている場合に設定が必要です。
     */
    static void RegisterTimeProvider(TimeProvider *pTimeProvider, qBool needCorrection = false);

    // Description: Returns the time in millisecs.
    /*!
     * @brief     システム時間をミリ秒単位で取得します。
     *
     * 取得する時間の精度や処理負荷は、タイムプロバイダの性能に左右されます。
     *
     * 
     * デフォルトのタイムプロバイダ使用時の処理負荷は小さくありませんので、頻繁に呼ぶ際はパフォーマンスに注意が必要です。
     * 
     *
     * @return    システム時間がミリ秒単位で返ります。
     */
    inline static Time GetTime()
    {
        Time now = s_needCorrection ? GetTimeImpl(false) : GetTimeImplCorrectless();
        SetCachedTime(now); // SystemClock::GetTimeをするたびSystemClockクラスのキャッシュタイムを更新
        return now;
    }

    // {secret}
    static Time GetCachedTime()
    {
#if NEX_ENABLE_GET_TIME_CACHE_OPTIMIZATION
        return s_cachedTime;
#else
        // return non cached time
        return GetTime();
#endif
    }
    static void SetCachedTime(Time t)
    {
#if NEX_ENABLE_GET_TIME_CACHE_OPTIMIZATION
        s_cachedTime = t;
#else
        // do nothing
        (void)t;
#endif
    }

#ifdef WIN32_TARGET

    /* Description: Uses QueryPerformanceCounter to retrieve the time (with a high degree of precision).

      Remarks: Not required on most platforms, this method is required on Windows because there existes a timer bug in multicore CPU
      motherboards.  That is, $PRODUCTNAME$ can use QueryPerformanceCounter to access a very high resolution time, but a major drawback
      is that it is subject to bugs when run on some dual core systems.

      See http://msdn.microsoft.com/library/default.asp?url=/library/en-us/directx9_c/Game_Timing_and_Multicore_Processors.asp
       for details.

      You can resolve this problem in one of two ways:

      One option to resolve this issue consists of locking the entire process to one single CPU using
      SetProcessAffinityMask(GetCurrentProcess(), 1);

      Another option is to lock NetZ threads to one CPU.  When doing so, you must ensure that all threads calling into NetZ are
      run from the same CPU. To do this, set the thread affinity using Core::SetDefaultThreadAffinity(1);
       This sets the thread affinity of the calling thread, and of any threads NetZ is using. You may also need to call
       SetThreadAffinityMask as well for any game thread that calls into NetZ.
    */
    static void UsePerformanceCounter();


    /* Description: Resets the function to retrieve the system time to the default Windows timeGetTime(). The resolution is not
       garanteed to be 1ms.

       Remarks: If you set this method, the system clock precision returned by Windows system clock functions can vary beteween
       1 and 5 milliseconds depending on the system.  Setting the windows time precision with this function, however, may result in system
       hangs due to threading synchronization problems. For more detailed information about this problem,
       see http://msdn.microsoft.com/library/default.asp?url=/library/en-us/multimed/htm/_win32_timegettime.asp

       See also http://msdn.microsoft.com/library/default.asp?url=/library/en-us/multimed/htm/_win32_obtaining_and_setting_timer_resolution.asp
       for an example of what to do to increase the timer resolution.  This isn't done automatically, as it is intrusive. But it is an option
       you can decide to use.
    */
    static void UseMultimediaTimer();

    // Description: Returns true when the system is using the Performance Counter. The default is false.
    static qBool IsUsingPerformanceCounter();
#endif

    /* Description: Resets the state of the SystemClock.
       Remarks: Sets the state of the SystemClock to uninitialized. You may Reset the SystemClock at anytime.
       It is useful to reset the System Clock before providing it with a new time provider..
       See Also: RegisterTimeProvider
    */

    /*!
     * @brief     リセット処理を行います。
     *
     * SystemClockクラスの状態を未定義状態に設定します。
     * この関数はいつでも呼び出すことが出来ます。
     * 新しいタイムプロバイダを設定する前に呼び出すと便利です。
     *
     * @see       RegisterTimeProvider
     */
    static void Reset();

private:
    static Time ProtectedGetTime();
    static Time GetTimeImpl(qBool bProtected);
    static Time GetTimeImplCorrectless();

    static void ApplyCorrection(Time tNow, Time tDesired);
    inline static Time GetUncorrectedTime();

    static qBool s_bUseHighPerformanceCounter;
    static TimeProvider *s_pTimeProvider;

    static qBool s_bInitialized;
    static LargeTimeInterval s_tiCorrection;
    static NAMESPACE_NEX CriticalSection s_csTimeCorrection;
    static qBool s_needCorrection;
#if NEX_ENABLE_GET_TIME_CACHE_OPTIMIZATION
    static qUnsignedInt64 s_cachedTime;
#endif

};

END_NAMESPACE_NEX

