﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDeclInternal.h>
#include <OnlineCore/src/Platform/Core/Log.h>

// General flags - Exposed to user

// group=Trace Flags

/*!
  @file
 */
/*
Description: Traces marked with this trace flag will never be displayed.
Remarks: This trace flag gives you the ability to disable a trace as a function parameter.
Example:
    void Avatar::ComputeScore(qUnsignedInt64 uiTraceFlag)
    {
    [...]
    QTRACE(uiTraceFlag,NEX_T("Trace will not appear if uiTraceFlag is TRACE_NEVER"));
    [...]
    }
*/
#define TRACE_NEVER         0

/*
Description: Traces marked with this trace flag will always be displayed.
Remarks: This trace flag is set by default and cannot be disabled. Therefore, traces
marked with this flag will always be displayed.
*/

/*!
  @brief 常に表示させたいトレースに設定されるトレースフラグです
 */
#define TRACE_ALWAYS            1

//n1589
#define TRACE_TIME (1<<1)

/*
Description: Set this flag to trace the bootstrap initialization of $PRODUCTNAME$.

$ifdef ETERNA$
Remarks: Set this flag to trace the creation and initialization of system components
such as the SessionClock, duplication spaces, and well-known objects.
$endif$

$ifdef NETZ$
Remarks: Set this flag to trace the creation and initialization of system components
such as the SessionClock, duplication spaces, and well-known objects.
$endif$

$ifdef RENDEZVOUS$
Remarks: Set this flag to trace the creation and initialization of system components.
$endif$
*/
#define TRACE_INIT              (1<<2)

/*
Description: Set this flag to trace the updates sent to duplicas by duplication masters.
Remarks: Set this flag to trace the updates performed on datasets of duplicated objects.
See Also: DuplicatedObject::Update

$ifdef RENDEZVOUS$
{secret}
$endif$
*/
//n1589:todo3:TRACE_UPDATE, rename to TRACE_DO_UPDATE
#define TRACE_UPDATE            (1<<3)

/*
$ifdef RENDEZVOUS$
Description: Set this flag to trace the runtime operations of the system.
$else$
Description: Set this flag to trace the runtime operations of the system.
Remarks: Operations are events that occur within the system that relate primarily to
duplicated objects or datasets. Setting the TRACE_OPERATION
flag has the effect of tracing all of the operations. Filtering of specific types of
operation traces is then performed via the Operation::SetTraceFilter system callback.

For example, the following code will trace the operations selected via the user-defined
FilterTraces function:

        Operation::SetTraceFilter(FilterTraces);
        TraceLog::GetInstance()->SetFlag(TRACE_OPERATION);

See Also: System Operations, Operation, Operation::SetTraceFilter
$endif$
*/

/*!
 */
#define TRACE_OPERATION         (1<<4)

/*
$ifdef RENDEZVOUS$
Description: Set this flag to trace any faults that occur in the system.
$else$
Description: Set this flag to trace the fault tolerance behaviour of the system.
Remarks: This trace flag is associated with detection of faulty stations and
the fault recovery of duplicated objects.
See Also: DuplicatedObject::ApproveFaultRecovery, Fault Tolerance Overview
$endif$
*/
//n1589:todo3:TRACE_FAULT, rename to TRACE_DO_FAULT
#define TRACE_FAULT         (1<<5)

/*
$ifdef NETZ$
Description: Set this flag to trace the operations related to the duplication spaces of the
$PRODUCTNAME$ application.
Remarks: This trace flag is reserved for tracing DuplicationSpace operations such as
DuplicationSpace::MatchAcrossPSRelations. Set this flag to trace the operations
performed on $PRODUCTNAME$ internal spaces as well as the spaces declared
as a part of your application.
See Also: DuplicationSpace, Duplication Space Overview
$else$
{secret}
$endif$
*/
#define TRACE_DUPSPACE          (1<<6)

/*
$ifdef ETERNA$
Description: Set this flag to trace the load balancing behaviour of the system.
See Also: Object Migration Overview, Load Balancing
$else$
{secret}
$endif$
*/
#define TRACE_LOADBALANCING     (1<<7)

/*
Description: Set this flag to trace the dead reckoning behaviour of the system.
Remarks: Use this trace flag to to trace the values of the extrapolated datasets.
See Also: Dead Reckoning Overview

$ifdef RENDEZVOUS$
{secret}
$endif$
*/
#define TRACE_PHBDR         (1<<8)

/*
$ifdef RENDEZVOUS$
{secret}
$else$
Description: Set this flag to trace the messages sent and received by duplicated objects.
Remarks: Set this flag to trace the exact time, thread, and process of each message sent
and received between duplicated objects.
$endif$
*/
//n1589:todo3:TRACE_DOPROTOCOL, rename to TRACE_DO
#define TRACE_DOPROTOCOL            (1<<9)

/*
$ifdef RENDEZVOUS$
Description: Set this flag to trace the execution of the scheduler.
$else$
Description: Set this flag to trace the execution of the scheduler.
See Also: Scheduler
$endif$
*/
#define TRACE_SCHEDULER         (1<<10)

/*
Description: Use this flag to create a new custom flag.
Remarks: Use this flag as is, or rename it to trace custom functionality in your application.
For example, you could create a new flag called TRACE_AVATAR as follows:

            #define TRACE_AVATAR  TRACE_USER
*/
#define TRACE_USER          (1<<11)

/*
Description: Set this flag to trace the teardown (shutdown) of $PRODUCTNAME$.
*/
#define TRACE_TEARDOWN          (1<<12)

/*
$ifdef RENDEZVOUS$
{secret}
$else$
Description: Set this flag to trace the events related to the session discovery.
See Also: SessionDiscoveryProtocol
$endif$
*/
#define TRACE_SESSIONDISCOVERY      (1<<13)

/*
Description: Set this flag to trace the events related to the station connections.
*/
#define TRACE_CONNECTION        (1<<14)

//TRACE_SYNCSIM is actually documented in SyncSim.h as it creates too many problems to document it here
//and then remove all the other flags from the doc.

/*
$ifdef SYNCSIM$
Description: Set this flag to trace the events related to the synchronized simulation
extension.
$else$
{secret}
$endif$
*/
#define TRACE_SYNCSIM           (1<<15)

/*
$ifdef RENDEZVOUS$
Description: Set this flag to trace events related to any request made to a $PRODUCTNAME$ service.
Remarks: When this flag is enabled ALL service requests will be logged, meaning both client
traces and $PRODUCTNAME$'s internal service traces will be recorded. As this flag will result in
large log files when logged to the server (because there are a lot of things that the $PRODUCTNAME$
servers do in the background), if you set this trace flag it is advisable to log to a client rather
than the server.
$else$
{secret}
$endif$
*/
#define TRACE_SERVICE_REQUESTS      (1<<16)

/*
Description: Set this flag to trace state transitions of $PRODUCTNAME$ system components.
*/
#define TRACE_SYSTEM_STATE          (1<<17)

/*
Description: Set this flag to trace NAT traversal related packets.
*/
#define TRACE_NAT                   (1<<18)

//n1589:duplication
#define TRACE_CLUSTERING            (1<<19)

#ifdef WII_TARGET
// NOTE: We use the same bit than TRACE_CLUSTERING because they cannot be
// used at the same time. TRACE_CLUSTERING is uniquely on the backend while
// TRACE_DWC is uniquely in NetZ.
/*
$ifdef NETZ$
Description: Set this flag to trace DWC related messages.
Remarks: When this flag is enabled ALL messages sent and received will be logged.
$else$
{secret}
$endif$
*/
//n1589:duplication
#define TRACE_DWC                   (1<<19)
#endif

//n1589:duplication
#ifdef CTR_TARGET
#define TRACE_UDS                   (1<<19)
#endif

/** \brief Set this flag to trace HTTP related traces. */
#define TRACE_HTTP  (1<<20)

/*
Description: Set this flag to trace Spark related traces.
*/
//n1589:duplication
#define TRACE_SPARK (1<<21)

//n1589:Transportの重要ログだけ出力
#define TRACE_TRANSPORT (1<<22)

//n1589:NetZ::Terminate処理の重要ログだけ出力
#define TRACE_TERMINATE (1<<23)

/*
$ifdef RENDEZVOUS$
{secret}
$else$
Description: Set this flag to trace all of the available core information.
Remarks: Setting this trace flag is equivalent to setting all the trace flags that relate to
core functionality.
$endif$
*/
#define TRACE_ALL (TRACE_ALWAYS|TRACE_TIME|TRACE_INIT|TRACE_UPDATE|TRACE_OPERATION|TRACE_FAULT|TRACE_DUPSPACE|TRACE_LOADBALANCING|TRACE_PHBDR|TRACE_DOPROTOCOL|TRACE_USER|TRACE_TEARDOWN|TRACE_SESSIONDISCOVERY|TRACE_CONNECTION|TRACE_NAT|TRACE_HTTP|TRACE_SPARK|TRACE_TRANSPORT)

// Internal Dev Traces are prefixed with 'D_'


#define D_TRACE_SHIFT 32
//n2361 swigでキャスト判定できないためLLを使用するように修正
#define D_TRACE_BIT 1LL

#define D_TRACE_ALL 0xFFFFFFFFFFFFFFFFULL


//n1589:D_TRACE_?:0-31

#define D_TRACE_ALWAYS (D_TRACE_BIT << (D_TRACE_SHIFT + 0)) //n1589:default ON for the internal build
#define D_TRACE_DEBUG (D_TRACE_BIT << (D_TRACE_SHIFT + 1))
#define D_TRACE_TEST (D_TRACE_BIT << (D_TRACE_SHIFT + 2))

#define D_TRACE_THREAD (D_TRACE_BIT << (D_TRACE_SHIFT + 3))
#define D_TRACE_SESSION_CLOCK (D_TRACE_BIT << (D_TRACE_SHIFT + 4))
#define D_TRACE_PROFILING (D_TRACE_BIT << (D_TRACE_SHIFT + 5)) // memo:少ししか使用されていないが今後役立ちそう
#define D_TRACE_CONSTRUCT (D_TRACE_BIT << (D_TRACE_SHIFT + 6))
#define D_TRACE_MEMORY (D_TRACE_BIT << (D_TRACE_SHIFT + 7))

// D_TRACE_TRANSPORT_XXX
#define D_TRACE_TRANSPORT (D_TRACE_BIT << (D_TRACE_SHIFT + 8))
#define D_TRACE_TRANSPORT_QUEUE (D_TRACE_BIT << (D_TRACE_SHIFT + 9))
#define D_TRACE_TRANSPORT_BUFFER (D_TRACE_BIT << (D_TRACE_SHIFT + 10))
#define D_TRACE_TRANSPORT_END_POINT (D_TRACE_BIT << (D_TRACE_SHIFT + 11))
#define D_TRACE_TRANSPORT_PACKET (D_TRACE_BIT << (D_TRACE_SHIFT + 12))
#define D_TRACE_TRANSPORT_ROUTING (D_TRACE_BIT << (D_TRACE_SHIFT + 13))
#define D_TRACE_TRANSPORT_BUNDLING (D_TRACE_BIT << (D_TRACE_SHIFT + 14))
#define D_TRACE_TRANSPORT_RELIABLE (D_TRACE_BIT << (D_TRACE_SHIFT + 15))
#define D_TRACE_TRANSPORT_SEND_MULTI (D_TRACE_BIT << (D_TRACE_SHIFT + 16))
#define D_TRACE_TRANSPORT_UDS (D_TRACE_BIT << (D_TRACE_SHIFT + 17))
#define D_TRACE_TRANSPORT_HOST_MIGRATION (D_TRACE_BIT << (D_TRACE_SHIFT + 18))
#define D_TRACE_TRANSPORT_EXPORT (D_TRACE_BIT << (D_TRACE_SHIFT + 19))

// D_TRACE_DO_XXX
#define D_TRACE_DO_DETAIL (D_TRACE_BIT << (D_TRACE_SHIFT + 20))
#define D_TRACE_DO_MESSAGE_BUNDLING (D_TRACE_BIT << (D_TRACE_SHIFT + 21))
#define D_TRACE_DO_MIGRATION (D_TRACE_BIT << (D_TRACE_SHIFT + 22))
#define D_TRACE_DO_RMC (D_TRACE_BIT << (D_TRACE_SHIFT + 23))
#define D_TRACE_DO_STATION (D_TRACE_BIT << (D_TRACE_SHIFT + 24))

// If more trace flags are added (beyond 32 bits), do not forget to
// also update the GetTraceFlagsMap() function to map the new flags.

// A few special flags.
// This flag is used to trace information just before a SYSTEMCHECK
// would fail.
//{secret}
#define TRACE_SYSTEMCHECK   TRACE_ALWAYS


#define DEFAULT_TRACE_FLAGS (TRACE_ALWAYS)

//{secret}
#define NO_DEVICE           0
//{secret}
#define CONSOLE_DEVICE      1
//{secret}
#define DEBUGOUTPUT_DEVICE  2
//{secret}
#define FILE_DEVICE     3


#define BOOL2STR(C) ((C))?NEX_T("true"):NEX_T("false")


//{secret}
#define QTRACE_VAR(F,V)     {QTRACE(F,NEX_T(#V) << NEX_T("=") << (V));}

//{secret}
#define QTRACE_PTR QTRACE_VAR
//{secret}
#define QTRACE_UINT QTRACE_VAR
//{secret}
#define QTRACE_ULONG QTRACE_VAR
//{secret}
#define QTRACE_INT QTRACE_VAR
//{secret}
#define QTRACE_DOUBLE QTRACE_VAR
//{secret}
#define QTRACE_BOOL QTRACE_VAR
//{secret}
#define QTRACE_TIME QTRACE_VAR
//{secret}
#define QTRACE_DOHANDLE QTRACE_VAR

//{secret}
#define QTRACE_HEX(F,V)         {QTRACE(F,NEX_T(#V) << NEX_T("=") << hex << (V));}


//{secret}
#define QTRACE_CSTR(F,V)        {QTRACE(F,NEX_T(#V) << NEX_T("=\"") << (V) << NEX_T("\""));}
//{secret}
#define QTRACE_STRING(F,V)      {QTRACE(F,NEX_T(#V) << NEX_T("=\"") << (V) << NEX_T("\""));}

#ifdef FOR_DOC_PURPOSE
//group=Trace Functions

/*
Description: Logs traces, using a printf-like interface.
Remarks: When enabled this macro outputs a string to the debug log however, the string is only
outputted if the supplied flag is set in the TraceLog. When traces are disabled, the call
is discarded as well as any function invocation done in the parameters.

Note: This macro is automatically disabled when run in release mode.

An example of the _QTRACE macro can be defined as follows:

        _QTRACE(EventLog::Always, (NEX_T("This uses an int: %d"), 5));

See Also: QTRACE
*/

/*!
@brief    printfのようなインターフェースを使用し、ログを出力します。

printfのようなインターフェースを使用し、ログを出力します。
文字列は TraceLog に供給された TRACE_FLAG がセットされている場合のみに出力されます。
トレースが無効の場合、呼び出しおよびパラーメータで行われたいずれの関数呼び出しも破棄されます。

@note    このマクロはDevelopment, Release ビルドでは無効になります。

@see     QTRACE
*/
#define _QTRACE(FLAG,ARGS)

/*
Description: Logs traces, using a stream-like interface.
Remarks: When enabled this macro outputs a string to the debug log however, the string is only
outputted if the supplied flag is set in the TraceLog. When traces are disabled, the call
is discarded as well as any function invocation done in the parameters.

Note: This macro is automatically disabled when run in release mode.

An example of the QTRACE macro can be defined as follows:

        QTRACE(EventLog::Always, (NEX_T("This uses an int")<<5);

See Also: _QTRACE
*/

/*!
@brief    ストリームのようなインターフェースを使用し、ログを出力します。

ストリームのようなインターフェースを使用し、ログを出力します。
文字列は TraceLog に該当の TRACE_FLAG がセットされている場合のみに出力されます。
トレースが無効の場合、呼び出しおよびパラーメータで行われたいずれの関数呼び出しも破棄されます。

@note    このマクロはリリースモードで実行されると自動的に無効になります。
*/
#define QTRACE(FLAG,ARGS)
#endif

#ifdef NEX_TRACE_ENABLED
#define _QTRACE(FLAG,ARGS) \
    if ((NAMESPACE_NEX TraceLog::GetInstance()->GetFlags() & (FLAG))) { \
        NAMESPACE_NEX TraceLog::GetInstance()->Output ARGS;} else {}
#else
#define _QTRACE(FLAG,ARGS) \
    if(false){ \
        if ((NAMESPACE_NEX TraceLog::GetInstance()->GetFlags() & (FLAG))) { \
            NAMESPACE_NEX TraceLog::GetInstance()->Output ARGS;} else {}} else {}
#endif

//n1589
#ifdef CTR_TARGET

//n1589:nn::dbg::GetLevelString is not defined in NEX_RELEASE_VERSION
#if defined(NEX_DEBUG_VERSION) && defined(NEX_TRACE_ENABLED)
// nn::dbg::GetLevelString がなくなったために空定義化
#define _QTRACE_RESULT(FLAG, ARGS, result)
#else
#define _QTRACE_RESULT(FLAG, ARGS, result)
#endif

#endif

#ifndef NEX_TRACE_ENABLED

#define QTRACE(FLAG,ARGS) \
    if(false){ \
        if ((NAMESPACE_NEX TraceLog::GetInstance()->GetFlags() & (FLAG))) { \
            NAMESPACE_NEX StringStream __ss; __ss << ARGS; NAMESPACE_NEX TraceLog::GetInstance()->Output(NEX_T("%s"), __ss.CStr());} else {}} else {}

#define IS_QNEX_TRACE_ENABLED(FLAG) false

#else   // NEX_TRACE_ENABLED

#define IS_QNEX_TRACE_ENABLED(FLAG) ((NAMESPACE_NEX TraceLog::GetInstance()->GetFlags() & (FLAG)))


#if defined(CTR_UNICODE) || defined(CAFE_UNICODE)

#define QTRACE(FLAG,ARGS)       \
    if ((NAMESPACE_NEX TraceLog::GetInstance()->GetFlags() & (FLAG))) {\
        NAMESPACE_NEX StringStream __ss; __ss << ARGS; NAMESPACE_NEX TraceLog::GetInstance()->OutputString(__ss.CStr());} else {}


#else //NON_SUPPORT_C - Unicode

#define QTRACE(FLAG,ARGS)       \
    if ((NAMESPACE_NEX TraceLog::GetInstance()->GetFlags() & (FLAG))) {\
        NAMESPACE_NEX StringStream __ss; __ss << ARGS; NAMESPACE_NEX TraceLog::GetInstance()->Output(NEX_T("%s"), __ss.CStr());} else {}

#endif  //NON_SUPPORT_C - Unicode

#endif  // NEX_TRACE_ENABLED

BEGIN_NAMESPACE_NEX

//group=Trace Logging

/*
Description: Traces the runtime functionality of your $PRODUCTNAME$ application.

$ifdef RENDEZVOUS$
Remarks: This class is used to trace $PRODUCTNAME$'s runtime functionality for debugging purposes.
When run in release mode, traces are automatically not logged. When you instantiate a TraceLog object
you need to attach a LogDevice and an OutputFormat object to the TraceLog object to define to which
output device the data is logged and how it is outputted.

To acquire a reference to the TraceLog instance, use the GetInstance method.  For example,

        TraceLog::GetInstance()->SetFlag(TRACE_SERVICE_REQUESTS);

The above will display traces for requests made to any Quazal service.
See Also: LogDevice, OutputFormat, Services

$else$
Remarks: This class is used to trace $PRODUCTNAME$'s runtime functionality for debugging purposes.
When run in release mode, traces are automatically not logged. When you instantiate a TraceLog object
you need to attach a LogDevice and an OutputFormat object to the TraceLog object to define to which
output device the data is logged and how it is outputted.

To acquire a reference to the TraceLog instance, use the GetInstance method.  For example,

        TraceLog::GetInstance()->SetFlag(TRACE_FAULT);

The above will display traces for the relevant fault-tolerance related System Operations.
The TRACE_OPERATION flag allows for more fine tuning of
which operations are traced and which are not, via the Operation::SetTraceFilter system callback
See Also: LogDevice, OutputFormat, System Operations, Operation::SetTraceFilter, DuplicatedObject::Trace,
DOSelections::Trace
$endif$
*/

/*!
@brief    NEXのランタイム機能をトレースします。

このクラスは、デバッグの目的でNEXのランタイム機能のトレースに使用されます。
Development, Release ビルドで実行されると、自動的にトレースがログされません。
TraceLogインスタンスの参照を取得するには、 @ref GetInstance メソッドを使用します。
*/
class TraceLog : public Log
{
public:
    //Group=Methods

    //{secret}
    virtual ~TraceLog();
#ifndef SWIG
    // 64bit環境のswigにva_listの引数をうまくコード生成できないバグがあるのでswigでは除外する
    // {secret}
    void OutputArgp( const qChar *szFormat, va_list argp );
#endif

    /*!
    @brief    トレースフラグをアクティブに設定します。

    デフォルトのトレースフラグは @ref TRACE_ALWAYS のみです。
    このメソッドを使用して、追加のトレースフラグをアクティブに設定します。

    @param[in]    uiFlags    アクティブにするトレースフラグ。
    */
    void SetFlag( qUnsignedInt64 uiFlags )
    {
        m_uiTraceFlags |= uiFlags;
    }

    /*!
    @brief     指定されたトレースフラグを非アクティブに再設定します。

     @param[in] uiFlag 非アクティブにするトレースフラグ
    */
    void ClearFlag( qUnsignedInt64 uiFlag )
    {
        m_uiTraceFlags = m_uiTraceFlags & (~uiFlag);
    }
    /*!
    @brief    アクティブなトレースフラグを返します。

    @return     @ref SetFlag を使用して設定されたアクティブなトレースフラグ。
    */
    qUnsignedInt64 GetFlags()
    {
        return m_uiTraceFlags;
    }

    //n1589:memo:FlagIsSet(TRACE_NEVER)はtrueになる
    /*!
    @brief    特定のフラグが設定されているかどうかを返します。

    @return    特定のフラグが設定されれば真、そうでないかエラーが起きた場合は偽。

    @param[in]    uiFlag    問い合わせるトレースフラグ。
    */
    qBool FlagIsSet(qUnsignedInt64 uiFlag)
    {
        return (m_uiTraceFlags & uiFlag)==uiFlag;
    }

    //n1589:memo:TraceLog::SetFlag(qChar)
    // {secret}
    /*
    Set trace flags using the names of the defines instead of their values.
    Accepts a list of flag names separated by commas.
    E.g.: SetFlag(NEX_T("TRACE_INIT,TRACE_UPDATE"));
    Particularly usefull when setting flags from user input such as a command line parameter.
     */
    void SetFlag( const qChar * szFlags );

    /*
    Description: Resets all trace flags as inactive except for TRACE_ALWAYS.
    Remarks: Resets all trace flags, except for TRACE_ALWAYS, to inactive.
    See Also: SetFlag, ClearFlag, GetFlags
    */
    void ClearFlags()
    {
        //n1589
        m_uiTraceFlags = DEFAULT_TRACE_FLAGS;
    }

    /*
    {Group:Classes}
    Description: This class makes it easy to temporarily increase the trace indentation.
    Remarks: Upon creation and destruction of a ScopedIndent object, the trace indentation will
    be increased and decreased by the number of indentation spaces given by the /uiIndent/
    parameter declared in the constructor.
    See Also: OutputFormat::IncreaseIndent, OutputFormat::DecreaseIndent
    */
    class ScopedIndent
    {
    public:
#ifdef NEX_TRACE_ENABLED    // n2370
        /*
        Remarks: When this object is created the trace indentation will be increased
        by the number of spaces given by uiIndent.
        Parameters:
        uiIndent: The number of indentation spaces that will be added to the trace. The
        default value is two.
        */
        ScopedIndent(qUnsignedInt uiIndent=2);

        /*
        Remarks: When this object is deleted the trace indentation is reduced by the
        number of indentation spaces declared when the object was created.
        */
        ~ScopedIndent();
    private:
        qUnsignedInt m_uiIndent;

#else   // NEX_TRACE_ENABLED
        ScopedIndent(qUnsignedInt uiIndent=2)
        {
            (void)uiIndent;
        }
        ~ScopedIndent() {}
#endif  // NEX_TRACE_ENABLED
    };

    /*
    {Group:Classes}
    Description: This class makes it easy to temporarily adjust the active Trace Flags.
    Remarks: Upon creation and destruction of a ScopedFlags object, the specified Trace Flags will be activated
    and deactivated whereby changing the data that is logged. The general way to use a scoped flag is as follows:

            {
                // Set the Trace Flags.
                TraceLog::ScopedFlags oSF(TRACE_INIT);

                // Do something useful...

                // Oops, I need an extra flag.
                TraceLog::GetInstance()->SetFlag(TRACE_MORE_STUFF);

                // All done. Flags reset to what they were before this scope was entered.
            }

    See Also: Trace Flags
    */
    class ScopedFlags
    {
    public:
        /*
        Remarks: When this object is created the specified Trace Flags will be set to active.
        Parameters:
        uiFlags: The trace flag(s) that will be set to active. The default flag is TRACE_ALL.
        */
        ScopedFlags(qUnsignedInt64 uiFlags = TRACE_ALL)
            : m_uiOldFlags(TraceLog::GetInstance()->GetFlags())
        {
            TraceLog::GetInstance()->SetFlag(uiFlags);
        }

        /*
        Remarks: When this object is deleted the Trace Flags that were set within the scope will be deactivated.
        */
        ~ScopedFlags()
        {
            TraceLog::GetInstance()->ClearFlags();
            TraceLog::GetInstance()->SetFlag(m_uiOldFlags);
        }
    private:
        qUnsignedInt64 m_uiOldFlags;
    };

    /*
    Description: Returns a pointer to the TraceLog instance.
    Returns: A reference to the TraceLog instance.
    Remarks: Use the reference returned by this function to control what information is
    to be traced and how. For example:

          TraceLog::GetInstance()->SetFlag(TRACE_TEARDOWN);
    */

    /*!
    @brief     @ref TraceLog インスタンスへのポインタを返します。

    この関数により返される参照を使用して、どの情報がどのようにトレースされるかをコントロールします。

    @return     @ref TraceLog インスタンスへのポインタ
    */
    static TraceLog *GetInstance();
private:
    TraceLog();

    qUnsignedInt64 m_uiTraceFlags;

    friend class GlobalVariables;
};

//{secret}
inline qDouble FormatCast(qReal rValue)
{
    return static_cast<qDouble>(rValue);
}

END_NAMESPACE_NEX

