﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/HighResolutionClock.h>
#include <OnlineCore/src/Platform/Core/ObjectThread.h>
#include <OnlineCore/src/Platform/Core/ObjectThreadRoot.h>
#include <OnlineCore/src/Platform/Core/Platform.h>
#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/ScopedCS.h>
#include <OnlineCore/src/Platform/Core/SystemClock.h>
#include <OnlineCore/src/Platform/Core/TimeProvider.h>

#ifdef CTR_TARGET
#include <nn/os.h>
#endif

BEGIN_NAMESPACE_NEX

/*!
 * @brief キャッシュされた時間値を返すタイムプロバイダです。
 *
 * キャッシュの時間値は内部の専用スレッドで一定時間ごとに更新されます。
 *
 * SystemClock::RegisterTimeProvider で、UpdatableTimeProviderオブジェクトを登録する必要があります。
 *
 * @see     SystemClock::RegisterTimeProvider
 */
class UpdatableTimeProvider : public TimeProvider
{
public:
    /*! @brief 更新スレッドのデフォルトスタックサイズ */
    static const qUnsignedInt32 DEFAULT_THREAD_STACK_SIZE = 4 * 1024;
    /*! @brief 更新スレッドのデフォルトのsleep時間[msec] */
    static const qUnsignedInt32 DEFAULT_SLEEP_TIME = 4;

    /*!
     * @brief コンストラクタです。
     *
     * 内部のキャッシュ更新スレッドを開始させます。
     *
     * @param[in] sleepTime 更新スレッドがsleepするミリ秒
     * @param[in] threadPriority 更新スレッドの優先度
     * @param[in] cpuCoreNo CPUのCoreNoを指定します。 ObjectThreadRoot::DEFAULT_CORE_NO の場合、NEX の規定値で動きます。
     *                                               SDK 3.0 以降の CTR では 1を指定することによりシステムコアを選択できます。
     *                                               システムコアの利用の際には、 nn::os::CTR::SetApplicationCpuTimeLimit() で、CPU 時間を割り当てる必要があります。
     *                                                
     * @param[in] threadStackSize 更新スレッドのスタックサイズ(CTRのみ有効で、4096バイト単位で指定)
     *            ObjectThreadRoot::UseAutoStack(false)である必要があります。
     *            
     */
    UpdatableTimeProvider(qUnsignedInt32 sleepTime = DEFAULT_SLEEP_TIME,
                          qUnsignedInt32 threadPriority = NEX_THREAD_PRIORITY_NORMAL,
                          qInt32 cpuCoreNo = ObjectThreadRoot::DEFAULT_CORE_NO,
                          qUnsignedInt32 threadStackSize = DEFAULT_THREAD_STACK_SIZE)
        : m_bRun(true), m_timeUpdaterThread(NEX_T("TimeUpdatorThread"), threadPriority)
    {
        m_timeUpdaterThread.SetCoreNo(cpuCoreNo);
        m_timeUpdaterThread.Launch(this, &UpdatableTimeProvider::Update,
                                   sleepTime, threadStackSize);
    }

    /*!
     * @brief デストラクタです。
     *
     * 内部のキャッシュ更新スレッドを停止させます。
     */
    virtual ~UpdatableTimeProvider()
    {
        Stop();
    }

    /*!
     * @brief キャッシュされた時間値を返します。
     *
     * SystemClockから呼び出されます。
     *
     * @return ミリ秒単位の時間値
     */
    virtual Time GetTime()
    {
        return m_time;
    }

    /*!
     * @brief キャッシュ更新スレッドを停止させます。
     */
    void Stop()
    {
        m_bRun = false;
        m_timeUpdaterThread.Wait();
    }

    /*!
     *@internal
     * @brief 時間値のキャッシュを更新します。
     *
     * 内部の更新スレッドから呼び出されます。
     * 更新スレッドは1回の更新ごとに一定期間のsleepを行います。
     *
     * @param[in] sleepTime 更新スレッドがsleepするミリ秒
     */
    void Update(qUnsignedInt32 sleepTime)
    {
#ifdef CTR_TARGET
        QLOG(EventLog::Info, NEX_T("Start UpdatableTimeProvider::Update CurrentProcessorNumber:") << nn::os::Thread::GetCurrentProcessorNumber());
#endif
        while (m_bRun)
        {
            m_time = HighResolutionClock::GetTime();
            Platform::Sleep(sleepTime);
        }
    }

private:
    Time m_time;  ///< キャッシュされた時間値
    volatile qBool m_bRun;  ///< 更新スレッドの実行フラグ
    ObjectThread<UpdatableTimeProvider, qUnsignedInt32> m_timeUpdaterThread;  ///< キャッシュ更新スレッド
};

END_NAMESPACE_NEX

