﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>


BEGIN_NAMESPACE_NEX

//group=Return Results Classes

/*
Summary: Passes a user-defined value.
Remarks: The user-defined value passed via the UserContext class may be either
a qBool, qUnsignedInt32, qReal, or pointer to an object. It is typicaly used to pass a
value that the user wishes to retain but is not automatically retained by $PRODUCTNAME$.

$ifdef NETZ$
For example, when a ChangeMasterStationOperation event occurs the original StationID of the
object is not retained by Net-Z. However, if you wish to retain this information
it can be passed as user-defined data before the operation begins and retrieved after the
operation ends. When user-defined information is passed in the context of an operation the methods
Operation::SetUserData and Operation::GetUserData should be used.

Refer to CallMethodOperation for an example of a UserContext in use.
See also: Operation::SetUserData, Operation::GetUserData
$endif$

$ifdef ETERNA$
For example, when a ChangeMasterStationOperation event occurs the original StationID of the
object is not retained by Eterna. However, if you wish to retain this information
it can be passed as user-defined data before the operation begins and retrieved after the
operation ends. When user-defined information is passed in the context of an operation the methods
Operation::SetUserData and Operation::GetUserData should be used.

Refer to CallMethodOperation for an example of a UserContext in use.
See also: Operation::SetUserData, Operation::GetUserData
$endif$
*/

/*!
@brief    ユーザー定義値を保持するクラスです。

UserContextクラスで渡されるユーザー定義の値は、qBool、qUnsignedInt32、qReal、またはオブジェクトへのポインタです。
通常、ユーザーが保持する値を渡すのに用いられます
*/
class UserContext: public RootObject
{

    //group=Methods

public:

    /*
    Description: Defines the type of attribute that will be passed with the UserContext.
    Parameters:
    uiValue: The user-defined value as a qUnsignedInt32. The default value is zero.
    ui64Value: The user-defined value as a qUnsignedInt64.
    dValue: The user-defined value as a qReal.
    bValue: The user-defined value as a qBool.
    pPointer: The user-defined pointer value.
    Remarks: The user-defined value passed via the UserContext class may be either
    a qUnsignedInt32, qReal, qBool, or a pointer.
    */

    /*!
     @brief    コンストラクタです。
     @param[in]    uiValue    ユーザー定義の符号無し32bit整数
     */
    UserContext(qUnsignedInt32 uiValue=0)
    {
        m_uContextStorage.m_uiValue=uiValue;
    }

    /*!
     @brief    コンストラクタです。
     @param[in]    dValue    ユーザー定義の実数値
     */
    UserContext(qReal dValue)
    {
        m_uContextStorage.m_dValue=dValue;
    }
    /*!
     @brief    コンストラクタです。
     @param[in]    bValue    ユーザー定義のbool値
     */
    UserContext(qBool bValue)
    {
        m_uContextStorage.m_bValue=bValue;
    }
    /*!
     @brief    コンストラクタです。
     @param[in]    pPointer    ユーザー定義のポインタ
     */
    UserContext(void* pPointer)
    {
        m_uContextStorage.m_pPointer=pPointer;
    }

    /*!
     @brief    コンストラクタです。
     @param[in]    ui64Value    ユーザー定義の符号無し64bit整数
     */
    UserContext(qUnsignedInt64 ui64Value)
    {
        m_uContextStorage.m_ui64Value = ui64Value;
    }

    //Description: Returns the user-defined value that was set via the method SetUIntValue.
    //Return Value: Returns the user-defined value set via the method SetUIntValue.
    //
    //Note: It is the user's responsibility to ensure that this method is used in
    //conjunction with SetUIntValue and not with one of the alternative methods
    //that take either a qReal, qBool, or pointer as a variable.
    //See Also: SetUIntValue
    /*!
    @brief    @ref SetUIntValue を使って設定されたユーザー定義値を返します。

    @return    @ref SetUIntValue を使って設定されたユーザー定義値を返します。

    @note    このメソッドが @ref SetUIntValue と関連して用いられ、qUnsignedInt64、qReal、qBool、または変数
    としてのポインタをとる代替メソッドの一つと関連して用いられないことを保証するのは、
    ユーザーの責任です。

    @see     SetUIntValue
    */
    qUnsignedInt32 GetUIntValue() const
    {
        return m_uContextStorage.m_uiValue;
    }

    /*!
    @brief    @ref SetUInt64Value を使って設定されたユーザー定義値を返します。

    @return    @ref SetUInt64Value を使って設定されたユーザー定義値を返します。

    @note    このメソッドが @ref SetUInt64Value と関連して用いられ、qUnsignedInt32、qReal、qBool、または変数
    としてのポインタをとる代替メソッドの一つと関連して用いられないことを保証するのは、
    ユーザーの責任です。

    @see     SetUIntValue
    */
    qUnsignedInt64 GetUInt64Value() const
    {
        return m_uContextStorage.m_ui64Value;
    }

    //Description: Returns the user-defined value that was set via the method SetRealValue.
    //Return Value: Returns the user-defined value set via the method SetRealValue.
    //
    //Note: It is the user's responsibility to ensure that this method is used in
    //conjunction with SetRealValue and not with one of the alternative methods
    //that take either a qUnsignedInt32, qBool, or pointer as a variable.
    //See Also: SetRealValue
    /*!
    @brief    @ref SetRealValue を使って設定されたユーザー定義値を返します。

    @return    @ref SetRealValue を使って設定されたユーザー定義値を返します。

    @note    このメソッドが @ref SetRealValue と関連して用いられ、qUnsignedInt32、qUnsignedInt64、qBool、
    または pointer as a variableをとる代替メソッドの一つと関連して用いられないことを保証するのは、
    ユーザーの責任です。

    @see     SetRealValue
    */
    qReal GetRealValue() const
    {
        return m_uContextStorage.m_dValue;
    }

    //Description: Returns the user-defined value that was set via the method SetBoolValue.
    //Return Value: Returns the user-defined value set via the method SetBoolValue.
    //
    //Note: It is the user's responsibility to ensure that this method is used in
    //conjunction with SetBoolValue and not with one of the alternative methods
    //that take either a qUnsignedInt32, qReal, or pointer as a variable.
    //See Also: SetBoolValue
    /*!
    @brief    @ref SetBoolValue を使って設定されたユーザー定義値を返します。

    @return    @ref SetBoolValue を使って設定されたユーザー定義値を返します。

    @note    このメソッドが @ref SetBoolValue と関連して用いられ、qUnsignedInt32、qUnsignedInt64、qReal、
    またはポインタを変数としてとる代替メソッドの一つと関連して用いられないことを保証するのは、
    開発者の責任です。

    @see     SetBoolValue
    */
    qBool GetBoolValue() const
    {
        return m_uContextStorage.m_bValue;
    }

    //Description: Returns the user-defined value that was set via the method
    //SetPointer.
    //Return Value: Returns the user-defined value set via the method SetPointer.
    //
    //Note: It is the user's responsibility to ensure that this method is used in
    //conjunction with SetPointer and not with one of the alternative methods
    //that take either a qUnsignedInt32, qBool, or qReal as a variable.
    //See Also: SetPointer
    /*!
    @brief    @ref SetPointer を使って設定されたユーザー定義値を返します。

    @return    @ref SetPointer を使って設定されたユーザー定義値を返します。

    @note    このメソッドが @ref SetPointer と関連して用いられ、qUnsignedInt32, qBool、
    または変数としてのqRealをとる代替メソッドの一つと関連して用いられないことを保証するのは、
    ユーザーの責任です。

    @see     SetPointer
    */
    void* GetPointer() const
    {
        return m_uContextStorage.m_pPointer;
    }

    template<typename TYPE> TYPE* GetPointerType() const
    {
        return static_cast<TYPE*>(GetPointer());
    }

    /*
    Description: Sets a user-defined value that can be obtained via the method GetUIntValue.

    Parameters:
    uiValue: A user-defined value of the type qUnsignedInt32.

    See Also: GetUIntValue
    */

    /*!
    @brief    @ref GetUIntValue を使用して取得できるユーザー定義の値を設定します。

    @param[in]    uiValue    qUnsignedInt32のユーザー定義値。

    @see     GetUIntValue
    */
    void SetUIntValue(qUnsignedInt32 uiValue)
    {
        m_uContextStorage.m_uiValue=uiValue;
    }

    /*!
    @brief    @ref GetUInt64Value を使用して取得できるユーザー定義の値を設定します。

    @param[in]    ui64Value    qUnsignedInt64のユーザー定義値。

    @see     GetUInt64Value
    */
    void SetUInt64Value(qUnsignedInt64 ui64Value)
    {
        m_uContextStorage.m_ui64Value=ui64Value;
    }

    /*
    Description: Sets a user-defined value that can be obtained via the method GetRealValue.

    Parameters:
    dValue: A user-defined value of the type qReal.

    See Also: GetRealValue
    */

    /*!
    @brief    @ref GetRealValue を使用して取得できるユーザー定義の値を設定します。

    @param[in]    dValue    qRealのユーザー定義値。

    @see     GetRealValue
    */
    void SetRealValue(qReal dValue)
    {
        m_uContextStorage.m_dValue=dValue;
    }

    /*
    Description: Sets a user-defined value that can be obtained via the method GetBoolValue.

    Parameters:
    bValue:  A user-defined value of the type qBool.

    See Also: GetBoolValue
    */

    /*!
    @brief    @ref GetBoolValue を使用して取得できるユーザー定義の値を設定します。

    @param[in]    bValue    qBool のユーザー定義値。

    @see     GetBoolValue
    */
    void SetBoolValue(qBool bValue)
    {
        m_uContextStorage.m_bValue=bValue;
    }

    /*
    Description: Sets a user-defined value that can be obtained via the method GetPointer.

    Parameters:
    pPointer: A user-defined pointer to an object.

    See Also: GetPointer
    */

    /*!
    @brief     @ref GetPointer を使用して取得できるユーザー定義値を設定します。

    @param[in]    pPointer    オブジェクトへのユーザー定義のポインタ。

    @see     GetPointer
    */
    void SetPointer(void* pPointer)
    {
        m_uContextStorage.m_pPointer=pPointer;
    }

private:

    union UserContextStorage
    {
        qUnsignedInt32 m_uiValue;
        qUnsignedInt64 m_ui64Value;
        qReal m_dValue;
        qBool   m_bValue;
        void* m_pPointer;
    };

    UserContextStorage m_uContextStorage;
};

END_NAMESPACE_NEX



