﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>


BEGIN_NAMESPACE_NEX

/**
    \ingroup group_onlinecore_platform
    \brief This class is used to pass a user-defined value.
    \note The user-defined value passed via the Variant class may be either a qBool, integer, qDouble or String.
    This class is supposed to support only basic types and stay minimal.
    It is the user's responsibility to ensure that he calls the getter that matches the data type of the value set in the variant.
*/
class OSDK_API Variant: public RootObject
{

public:

    /**
        \brief Supported Data Types.
        \note Should not exceed a byte (On wire, it is transfered as a byte).
    */
    enum DataType
    {
        INVALID,
        INTEGER,
        DOUBLE,
        BOOLEAN,
        STRING,
        DATETIME,
        UINTEGER
    };

    /**
        \brief Default constructor.
        \note Creates an invalid Variant!
    */
    Variant();

    /**
        \brief Copynstructor.
    */
    Variant(const Variant& var);

#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    // ムーブコンストラクタ
    Variant(Variant&& rhs) NEX_NOEXCEPT
        : m_value()
        , m_dataType(INVALID)
    {
        *this = std::move(rhs);
    }

    // ムーブ代入演算子
    Variant& operator=(Variant&& rhs);
#endif

    /**
        \brief Destructor.
    */
    ~Variant();

    /**
        \brief Constructor.
        \param[in] value The value that will be passed with this Variant as a qInt64.
    */
    explicit Variant(qInt64 value);

    /**
        \brief Constructor.
        \param[in] value The value that will be passed with this Variant as a qUnsignedInt64.
    */
    explicit Variant(qUnsignedInt64 value);

    /**
        \brief Constructor.
        \param[in] value The value that will be passed with this Variant as a qInt32.
    */
    explicit Variant(qInt32 value);

    /**
        \brief Constructor.
        \param[in] value The value that will be passed with this Variant as a qUnsignedInt32.
    */
    explicit Variant(qUnsignedInt32 value);

    /**
        \brief Constructor.
        \param[in] value The value that will be passed with this Variant as a qDouble.
    */
    explicit Variant(qDouble value);

    /**
        \brief Constructor.
        \param[in] value The value that will be passed with this Variant as a qBool.
    */
    explicit Variant(qBool value);

    /**
        \brief Constructor.
        \param[in] value The value that will be passed with this Variant as a String.
    */
    explicit Variant(const String& value);

    /**
        \brief Constructor.
        \param[in] value The value that will be passed with this Variant as a DateTime.
    */
    explicit Variant(const DateTime& value);

    /**
        \brief Constructor.
        \param[in] value The value that will be passed with this Variant as a qChar*.
    */
    explicit Variant(const qChar* value);


    DataType GetType() const;
    qBool IsInvalid() const
    {
        return GetType() == INVALID;
    }

    /**
        \brief Gets the user-defined value that was set via the corresponding assignment operator.
        \returns The user-defined value as a qInt64.
    */
    qInt64 GetInt64Value() const;

    /**
        \brief Gets the user-defined value that was set via the corresponding assignment operator.
        \returns The user-defined value as a qUnsignedInt64.
    */
    qUnsignedInt64 GetUInt64Value() const;

    /**
        \brief Gets the user-defined value that was set via the corresponding assignment operator.
        \returns The user-defined value as a qInt32.
    */
    qInt32 GetInt32Value() const;

    /**
        \brief Gets the user-defined value that was set via the corresponding assignment operator.
        \returns The user-defined value as a qUnsignedInt32.
    */
    qUnsignedInt32 GetUInt32Value() const;

    /**
        \brief Gets the user-defined value that was set via the corresponding assignment operator.
        \returns The user-defined value as a qDouble.
    */
    qDouble GetDoubleValue() const;

    /**
        \brief Gets the user-defined value that was set via the corresponding assignment operator.
        \returns The user-defined value as a qBool.
    */
    qBool GetBoolValue() const;

    /**
        \brief Gets the user-defined value that was set via the corresponding assignment operator.
        \returns The user-defined value as a String.
    */
    String GetStringValue() const;

    /**
        \brief Gets the user-defined value that was set via the corresponding assignment operator.
        \returns The user-defined value as a DateTime.
    */
    DateTime GetDateTimeValue() const;

    qBool operator==(const Variant& var) const;
    qBool operator!=(const Variant& var) const;

    Variant& operator=(const Variant& var);

    void Trace(qUnsignedInt32 flag=TRACE_ALWAYS)const;

private:

    //Supported data Types
    //Note: This class is defined as custom ddl type. As a result, you
    //need to make sure that this class can be streamed in/out

    union Value
    {
        qInt64         m_int64;
        qDouble        m_double;
        qBool          m_bool;
        qUnsignedInt64 m_datetime;
        qChar*         m_string;
    } m_value;

    DataType m_dataType;
};

extern OSDK_API StringStream& operator<<(StringStream& strStream, const Variant& var);

END_NAMESPACE_NEX



