﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/ScopedCS.h>

BEGIN_NAMESPACE_NEX

//This is a list class which forces the user to lock a critical section (to prevent concurrent access) before manipulating it
//If the user manipulates this list without locking a critical section, the class will force an error
template <typename T> class qProtectedList: public RootObject
{
public:
    qProtectedList(): m_csList(LockMasks::Ignored) {}
    qProtectedList(const qProtectedList& oOther):RootObject(), m_csList(LockMasks::Ignored)
    {
        *this = oOther;
    }
    //n1589:CriticalSection:Ignored
    qProtectedList(const qList<T>& oOther): m_csList(LockMasks::Ignored)
    {
        *this = oOther;
    }

    virtual ~qProtectedList() {}

    qProtectedList& operator=(const qProtectedList& oOther)
    {
        ScopedCS oCS(m_csList);
        m_oList = oOther.m_oList;
        return *this;
    }
    qProtectedList& operator=(const qList<T>& oOther)
    {
        ScopedCS oCS(m_csList);
        m_oList = oOther;
        return *this;
    }

    typedef typename qList<T>::iterator iterator;
    typedef typename qList<T>::const_iterator const_iterator;

    void push_back(const T& e)
    {
        ScopedCS oCS(m_csList);
        m_oList.push_back(e);
    }
    void pop_back()
    {
        ScopedCS oCS(m_csList);
        m_oList.pop_back();
    }

    void push_front(const T& e)
    {
        ScopedCS oCS(m_csList);
        m_oList.push_front(e);
    }
    void pop_front()
    {
        ScopedCS oCS(m_csList);
        m_oList.pop_front();
    }

    T& front()
    {
        SYSTEMCHECK(m_csList.HasOwnership());
        return m_oList.front();
    }
    const T& front() const
    {
        SYSTEMCHECK(m_csList.HasOwnership());
        return m_oList.front();
    }

    iterator begin()
    {
        SYSTEMCHECK(m_csList.HasOwnership());
        return m_oList.begin();
    }
    const_iterator begin() const
    {
        SYSTEMCHECK(m_csList.HasOwnership());
        return m_oList.begin();
    }
    iterator end()
    {
        SYSTEMCHECK(m_csList.HasOwnership());
        return m_oList.end();
    }
    const_iterator end() const
    {
        SYSTEMCHECK(m_csList.HasOwnership());
        return m_oList.end();
    }

    iterator erase(iterator pos)
    {
        SYSTEMCHECK(m_csList.HasOwnership());
        return m_oList.erase(pos);
    }
    iterator erase(iterator first, iterator last)
    {
        SYSTEMCHECK(m_csList.HasOwnership());
        return m_oList.erase(first,last);
    }

    void remove(const T& x)
    {
        ScopedCS oCS(m_csList);
        m_oList.remove(x);
    }

    /* API is different than the standard: it would be unsafe to
     * return an iterator */
    void safe_erase(iterator pos)
    {
        ScopedCS oCS(m_csList);
        m_oList.erase(pos);
    }
    void safe_erase(iterator first, iterator last)
    {
        ScopedCS oCS(m_csList);
        m_oList.erase(first, last);
    }

    void clear()
    {
        ScopedCS oCS(m_csList);
        m_oList.clear();
    }
    qBool empty() const
    {
        ScopedCS oCS(m_csList);
        return m_oList.empty();
    }
    size_t size() const
    {
        ScopedCS oCS(m_csList);
        return m_oList.size();
    }

    void copy(qList<T> & lstCopy) const
    {
        ScopedCS oCS(m_csList);
        lstCopy = m_oList;
    }

    CriticalSection& CS() const
    {
        return m_csList;
    }

private:

    mutable NAMESPACE_NEX CriticalSection m_csList;

    qList<T> m_oList;
};

END_NAMESPACE_NEX

