﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qVector.h>

BEGIN_NAMESPACE_NEX
/*!
    @brief  一方向キューを実現する固定長リングバッファクラスです。

    コンストラクタに必ず要素数を指定しなければなりません。
    末尾への要素追加と、先頭から要素取得をすること以外はできませんが、push、frontを実行してもメモリ操作は行われません。

    @tparam T コンテナにするオブジェクト。
*/

template <typename T> class qReservedQueue : public RootObject
{
public:
    class iterator : public std::iterator<std::random_access_iterator_tag, T>
    {
    public:
        typedef T& reference;
        typedef const T& const_reference;
        typedef std::ptrdiff_t difference_type;

        // コンストラクタ
        iterator()
            : m_Associated(NULL)
            , m_CurrentIndex(0)
            , m_IsEnd(true)
        {}
        iterator(const qReservedQueue<T>* associated, qBool isEnd)
            : m_Associated(const_cast<qReservedQueue<T>*>(associated))
            , m_CurrentIndex(isEnd ? m_Associated->m_PushIndex : m_Associated->m_PopIndex)
            , m_IsEnd(isEnd)
        {}

        // コピーコンストラクタと代入演算子はデフォルトのものでよい。

        // 要素のアクセス
        reference operator*()
        {
            return *(m_Associated->m_vec.begin() + m_CurrentIndex);
        }
        const_reference operator*() const
        {
            return *(m_Associated->m_vec.begin() + m_CurrentIndex);
        }
        reference operator[](difference_type distance)
        {
            difference_type d = (m_CurrentIndex + distance) % m_Associated->capacity();
            return *(m_Associated->m_vec.begin() + d);
        }
        const_reference operator[](difference_type distance) const
        {
            difference_type d = (m_CurrentIndex + distance) % m_Associated->capacity();
            return *(m_Associated->m_vec.begin() + d);
        }

        // 移動系
        // 前置インクリメント
        iterator& operator++()
        {
            SetCurrentIndex((m_CurrentIndex + 1) % m_Associated->capacity(), true);
            return *this;
        }
        // 後置インクリメント
        iterator  operator++(int)
        {
            iterator it = *this;
            ++(*this);
            return it;
        }
        iterator operator+(difference_type distance) const
        {
            iterator it = *this;
            it.SetCurrentIndex((m_CurrentIndex + distance) % m_Associated->capacity(), true);
            return it;
        }
        iterator& operator+=(difference_type distance)
        {
            SetCurrentIndex((m_CurrentIndex + distance) % m_Associated->capacity(), true);
            return *this;
        }
        // 前置デクリメント
        iterator& operator--()
        {
            SetCurrentIndex(m_CurrentIndex == 0 ? m_Associated->capacity() - 1 : m_CurrentIndex - 1, false);
            return *this;
        }
        // 後置デクリメント
        iterator  operator--(int)
        {
            iterator it = *this;
            --(*this);
            return it;
        }
        iterator operator-(difference_type distance) const
        {
            if(distance < 0)
            {
                return operator+(-distance);
            }

            iterator it = *this;

            if(m_CurrentIndex < static_cast<size_t>(distance))
            {
                it.SetCurrentIndex(m_CurrentIndex + m_Associated->capacity() - distance, false);
            }
            else
            {
                it.SetCurrentIndex(m_CurrentIndex - distance, false);
            }
            return it;
        }
        difference_type operator-(iterator it) const
        {
            // 左辺のpopインデックス(内部バッファの先頭)からの差分
            qInt32 diffLhs = static_cast<qInt32>(m_CurrentIndex) - static_cast<qInt32>(m_Associated->m_PopIndex);
            if(diffLhs < 0 || m_IsEnd)
            {
                diffLhs += static_cast<qInt32>(m_Associated->capacity());
            }

            // 右辺のpopインデックス(内部バッファの先頭)からの差分
            qInt32 diffRhs = static_cast<qInt32>(it.m_CurrentIndex) - static_cast<qInt32>(it.m_Associated->m_PopIndex);
            if(diffRhs < 0 || it.m_IsEnd)
            {
                diffRhs += static_cast<qInt32>(it.m_Associated->capacity());
            }

            return diffLhs - diffRhs;
        }
        iterator& operator-=(difference_type distance)
        {
            if(distance < 0)
            {
                return operator+=(-distance);
            }
            if(m_CurrentIndex < static_cast<size_t>(distance))
            {
                SetCurrentIndex(m_CurrentIndex + m_Associated->capacity() - distance, false);
            }
            else
            {
                SetCurrentIndex(m_CurrentIndex - distance, false);
            }
            return *this;
        }

        // 比較系
        qBool operator==(const iterator& it) const
        {
            // 念のため同じqReservedQueueかチェック
            if(m_Associated != it.m_Associated)
            {
                SYSTEMCHECK(false);
                return false;
            }
            if(m_Associated->empty() && m_Associated->m_PushIndex == m_CurrentIndex && m_CurrentIndex == it.m_CurrentIndex)
            {
                // 要素が空のケースはm_IsEndによらずインデックスが同じならOK
                return true;
            }
            // 空じゃないケースはインデックスとm_IsEndの比較が必要。要素がいっぱいの時にbeginとendでインデックスが等しくなってしまうため。
            return m_CurrentIndex == it.m_CurrentIndex && m_IsEnd == it.m_IsEnd;
        }
        qBool operator!=(const iterator& it) const
        {
            return !(*this == it);
        }
        qBool operator<(const iterator& it) const
        {
            // インデックスが等しい場合に"<"となるのは左辺がendじゃなくて右辺がendのときのみ。それ以外の時は次の条件で必ずfalseになる。
            if(!m_IsEnd && it.m_IsEnd)
            {
                return true;
            }
            // 折り返していない状態
            if((m_CurrentIndex < m_Associated->m_PopIndex && it.m_CurrentIndex < m_Associated->m_PopIndex)
                    || (m_CurrentIndex >= m_Associated->m_PopIndex && it.m_CurrentIndex >= m_Associated->m_PopIndex))
            {
                return m_CurrentIndex < it.m_CurrentIndex;
            }
            // 折り返している状態
            return m_CurrentIndex > it.m_CurrentIndex;
        }
        qBool operator>(const iterator& it) const
        {
            return it < *this;
        }
        qBool operator<=(const iterator& it) const
        {
            return !(*this > it);
        }
        qBool operator>=(const iterator& it) const
        {
            return !(*this < it);
        }
    private:
        // m_CurrentIndexを操作する場合は必ずこの関数を使用してください。
        // 本関数内でendに到達したかどうかの判定を行います。
        // 第二引数はm_CurrentIndexが増加する操作を行った場合にtrueを指定してください。
        void SetCurrentIndex(size_t newIndex, qBool increased)
        {
            // 増加した場合でPushIndexと等しくなったということは最後まで来て有効なデータがなくなったということなのでendと判定
            m_IsEnd = increased && m_Associated->m_PushIndex == newIndex;
            m_CurrentIndex = newIndex;
        }

        qReservedQueue<T>* m_Associated;
        size_t m_CurrentIndex;
        qBool m_IsEnd;
    };
    typedef const iterator const_iterator;
    typedef std::reverse_iterator<iterator> reverse_iterator;
    typedef std::reverse_iterator<const_iterator> const_reverse_iterator;

    explicit qReservedQueue(size_t size):
        m_vec(size),
        m_PushIndex(0),
        m_PopIndex(0),
        m_ElemCnt(0),
        m_ReservedSize(size)
    {
        SYSTEMCHECK(m_ReservedSize > 0);
    }

#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    // ムーブコンストラクタ
    qReservedQueue(qReservedQueue&& rhs) NEX_NOEXCEPT
        : m_vec(0)
        , m_PushIndex(0)
        , m_PopIndex(0)
        , m_ElemCnt(0)
        , m_ReservedSize(0)
    {
        *this = std::move(rhs);
    }

    // ムーブ代入演算子
    qReservedQueue& operator=(qReservedQueue&& rhs) NEX_NOEXCEPT
    {
        if (this == &rhs)
        {
            return *this;
        }
        m_vec = std::move(rhs.m_vec);
        m_PushIndex = rhs.m_PushIndex;
        m_PopIndex = rhs.m_PopIndex;
        m_ElemCnt = rhs.m_ElemCnt;
        m_ReservedSize = rhs.m_ReservedSize;

        rhs.m_PushIndex = 0;
        rhs.m_PopIndex = 0;
        rhs.m_ElemCnt = 0;
        rhs.m_ReservedSize = 0;
        return *this;
    }
#endif

    size_t size() const
    {
        return m_ElemCnt;
    }
    qBool empty() const
    {
        return m_ElemCnt == 0;
    }
    qBool full() const
    {
        return m_ElemCnt == m_ReservedSize;
    }
    size_t capacity() const
    {
        return m_ReservedSize;
    }
    size_t room() const
    {
        return m_ReservedSize - m_ElemCnt;
    }

    // pushされる前の最後の要素に対するイテレータは無効になります。
    void push(const T& item)
    {
        SYSTEMCHECK(m_ElemCnt < m_ReservedSize);
        m_vec[m_PushIndex] = item;
        m_PushIndex++;
        m_PushIndex %= m_ReservedSize;
        m_ElemCnt++;
    }

    // qByteコンテナのみ対応
    // qByteコンテナの場合はmemcpyを使って高速にpush/popが可能です
    // qByteコンテナ以外では以下の二つの関数は実装されていないためコンパイルできません
    void push(const T*, size_t);
    void front(T* dst, size_t size);


    const T& front() const
    {
        SYSTEMCHECK(m_ElemCnt > 0);
        return m_vec[m_PopIndex];
    }
    void pop()
    {
        SYSTEMCHECK(m_ElemCnt > 0);
#ifdef WIN_FAMILY
        // WindowsのReleaseビルドで、warning C4724: 剰余の 2 番目のオペランドは、
        // コンパイル時に 0 と評価され、不定の結果を返します。
        // という警告が出るのを回避する。
        // 実際にはm_ReservedSizeが0になることはない。
        SYSTEMCHECK(m_ReservedSize > 0);
        if (m_ReservedSize == 0)
        {
            QLOG(EventLog::Fatal, NEX_T("m_ReservedSize is 0"));
            return;
        }
#endif
        m_vec[m_PopIndex++] = T();  // popするオブジェクトを破棄させる
        m_PopIndex %= m_ReservedSize;
        m_ElemCnt--;
    }
    void pop(size_t size)
    {
        SYSTEMCHECK(m_ElemCnt>=size);
        for (size_t i = 0; i < size; ++i)
        {
            m_vec[m_PopIndex++] = T();  // popするオブジェクトを破棄・初期化する
            m_PopIndex %= m_ReservedSize;
        }
        m_ElemCnt -= size;
    }
    const T& at(size_t i) const
    {
        VERBOSESYSTEMCHECK(i < m_ElemCnt, _QLOG(EventLog::Fatal, NEX_T("qReservedQueue::at:%d >= %d"), i, m_ElemCnt));
        i += m_PopIndex;
        i %= m_ReservedSize;
        return m_vec.at(i);
    }
    void clear()
    {
        pop(size());
    }

    // 要素を探す
    // true  : 見つかった
    // false : 見つからなかった
    qBool find(const T& item) const
    {
        qBool ret = false;
        size_t i = m_PopIndex;
        for(size_t cnt = 0 ; cnt < m_ElemCnt ; cnt++)
        {
            if(item == m_vec[i])
            {
                ret = true;
                break;
            }
            i++;
            i %= m_ReservedSize;
        }
        return ret;
    }

    iterator begin()
    {
        return iterator(this, false);
    }
    const_iterator begin() const
    {
        return const_iterator(this, false);
    }
    iterator end()
    {
        return iterator(this, true);
    }
    const_iterator end() const
    {
        return const_iterator(this, true);
    }
    reverse_iterator rbegin()
    {
        return reverse_iterator(end());
    }
    /* TODO: #45690 clangでビルドに失敗する。　error: 'mutable' and 'const' cannot be mixed
    const_reverse_iterator rbegin() const
    {
        return (const_reverse_iterator(end()));
    }
    */
    reverse_iterator rend()
    {
        return (reverse_iterator(begin()));
    }
    /*
    const_reverse_iterator rend() const
    {
        return (const_reverse_iterator(begin()));
    }
    */
private:
    qReservedQueue(): m_vec(0),m_PushIndex(0),m_PopIndex(0),m_ElemCnt(0), m_ReservedSize(0) {}
    qVector<T> m_vec;
    size_t m_PushIndex;
    size_t m_PopIndex;
    size_t m_ElemCnt;
    size_t m_ReservedSize;
};

// qByteの場合の特殊化の宣言
template<>
void qReservedQueue<qByte>::push(const qByte* src, size_t size);

template<>
void qReservedQueue<qByte>::front(qByte* dst, size_t size);

template<>
void qReservedQueue<qByte>::pop();

template<>
void qReservedQueue<qByte>::pop(size_t size);

template<>
void qReservedQueue<qByte>::clear();

END_NAMESPACE_NEX
