﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once


#include <OnlineCore/src/Platform/Stack/Core/InetDecl.h>
#include <OnlineCore/src/Platform/Core/qMap.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/qReservedQueue.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Platform/Stack/Core/SocketDriver.h>
#include <OnlineCore/src/Platform/Core/ForcedCriticalSection.h>
#include <OnlineCore/src/Core/SharedPtr.h>
#include <OnlineCore/src/Core/ByteOrder.h>

BEGIN_NAMESPACE_NEX

typedef SharedPtr<SocketDriver::Socket> DriverSocketSharedPtr;
typedef SharedPtr<qReservedQueue<qByte> > ReservedBufferSharedPtr;

// 受信情報。データグラム再構成用のバッファとリモートアドレスを管理する。
struct ReceivingInfo: public RootObject
{
    // 無効なReceivingInfoを生成する。
    // メモリアロケートは発生しない。
    ReceivingInfo()
        : remoteAddress()
        , receivedBuffer()  // 空のSharedPtr
        , isValid(false)
    {
    }

    // メモリアロケートが発生する。
    ReceivingInfo(const InetAddress& remoteAddress_, qBool bMakeDatagram)
        : remoteAddress(remoteAddress_)
        , receivedBuffer()
        , isValid(true)
    {
        // データグラム化する場合はそのために必要なバッファを確保。
        if (bMakeDatagram)
        {
            receivedBuffer = ReservedBufferSharedPtr(qNew qReservedQueue<qByte>((SocketConstants::DEFAULT_MTU + SocketConstants::TCP_DATAGRAM_SIZE_PREFIX_SIZE) * 2));
        }
    }

    qBool IsValid() const
    {
        return isValid;
    }
    // データグラム分のサイズの受信バッファを持っていればそのデータグラムのサイズを返す。
    // データグラム分そろっていなければ0を返す。
    qUnsignedInt16 HaveDatagram() const
    {
        qUnsignedInt16 datagramSize = 0;

        // データグラム化しない設定の場合は常に0を返す。
        if (!receivedBuffer)
        {
            return 0;
        }

        // データグラムを得るには最低3byte必要。ヘッダ2byte+ペイロード1byte
        if (receivedBuffer->size() <= SocketConstants::TCP_DATAGRAM_SIZE_PREFIX_SIZE)
        {
            return 0;
        }

        // 先頭2byteを見てデータグラムサイズを取得する
        receivedBuffer->front(reinterpret_cast<qByte*>(&datagramSize), SocketConstants::TCP_DATAGRAM_SIZE_PREFIX_SIZE);
        datagramSize = NexNetworkOrderToHostOrder(datagramSize);
        if (datagramSize == 0 || datagramSize > SocketConstants::DEFAULT_MTU)
        {
            // 不正なデータグラムサイズを受け取ったら受信データをクリアし0を返す。
            // データが破損するのでその後上位の署名チェックではじかれるようになり切断される。
            QLOG(EventLog::Error, NEX_T("Invalid datagram received. size: ") << datagramSize);
            receivedBuffer->clear();
            return 0;
        }

        if (receivedBuffer->size() >= SocketConstants::TCP_DATAGRAM_SIZE_PREFIX_SIZE + datagramSize)
        {
            return datagramSize;
        }
        return 0;
    }
    InetAddress remoteAddress;
    ReservedBufferSharedPtr receivedBuffer;
    qBool isValid;
};

// 送信情報。データグラム生成用のバッファとソケットを管理する。
struct SendingInfo: public RootObject
{
    // 無効なReceivingInfoを生成する。
    // メモリアロケートは発生しない。
    SendingInfo()
        : spSocket()
        , sendBuffer()  // 空のSharedPtr
        , isValid(false)
    {
    }

    // メモリアロケートが発生する。
    SendingInfo(DriverSocketSharedPtr spSocket_, qBool bMakeDatagram)
        : spSocket(spSocket_)
        , sendBuffer()
        , isValid(true)
    {
        // データグラム化する場合はそのために必要なバッファを確保。
        if (bMakeDatagram)
        {
            sendBuffer = ReservedBufferSharedPtr(qNew qReservedQueue<qByte>((SocketConstants::DEFAULT_MTU + SocketConstants::TCP_DATAGRAM_SIZE_PREFIX_SIZE) * 2));
        }
    }

    qBool IsValid() const
    {
        return isValid;
    }

    DriverSocketSharedPtr spSocket;
    ReservedBufferSharedPtr sendBuffer;
    qBool isValid;
};

// delete待ちのソケットに関する情報を保持する構造体
struct ClosedSocketInfo: public RootObject
{
    ClosedSocketInfo(const DriverSocketSharedPtr& spSocket_,
                     const ReservedBufferSharedPtr& spReceivedBuffer_,
                     const ReservedBufferSharedPtr& spSendingBuffer_)
        : spSocket(spSocket_)
        , spReceivedBuffer(spReceivedBuffer_)
        , spSendingBuffer(spSendingBuffer_)
    {
    }

    bool IsReadyToDelete() const
    {
        return spSocket->IsReadyToDelete() && spSocket.unique();
    }

    bool operator==(const ClosedSocketInfo& rhs)
    {
        return spSocket == rhs.spSocket;
    }
    DriverSocketSharedPtr spSocket;
    ReservedBufferSharedPtr spReceivedBuffer;
    ReservedBufferSharedPtr spSendingBuffer;
};

// コネクション指向プロトコル向けに以下の機能を提供するクラス。
// ・ソケットとアドレスを関連付け、多数のソケットを扱えるようにする。
// ・データグラムによる通信を行う。
class ConnectionOrientedSocketManager: public RootObject
{
public:
    ConnectionOrientedSocketManager(qBool bMakeDatagram, SocketDriver* pSocketDriver);

    void AddSocket(const InetAddress& remoteAddress, const DriverSocketSharedPtr& spAcceptedSocket);
    void RemoveSocket(DriverSocketSharedPtr spSocket);
    void RemoveSocketCompletely(DriverSocketSharedPtr spSocket);
    void AcquireAcceptedSocket(const InetAddress& remoteAddress);
    void ReleaseAcceptedSocket(const InetAddress& remoteAddress);
    SocketDriver::Result SendDatagram(const qByte* pSendBuffer, size_t sendBufferSize, const InetAddress& remoteAddress);
    SocketDriver::Result ReceiveDatagram(qByte* receivedDatagram, size_t receiveBufferSize, size_t* receivedDatagramSize, InetAddress* pInetAddress, SocketDriver::SocketFlag flag);
    void PrepareToListen(size_t numSocketsToAccept);
    void PrepareToConnect();
    void DoWork();
private:
    void ProcessRemovedSocket();
    void MoveToClosedSocket(const SendingInfo& sendingInfo, const ReceivingInfo& receivingInfo);
    void PurgeClosedSocket();
    qBool UpdateReceivableSockets(qUnsignedInt32 timeoutMs);
    SocketDriver::Result ReceiveDatagramFromOneSocket(const DriverSocketSharedPtr& spSocket, qByte* receivedDatagram,
        size_t receiveBufferSize, size_t* receivedDatagramSize, InetAddress* pInetAddress, SocketDriver::SocketFlag flag);
    SendingInfo FindSendingInfo(const InetAddress& remoteAddress);
    ReceivingInfo FindReceivingInfo(const DriverSocketSharedPtr& spSocket);

    // ブロッキングで受信するときにタイムアウトする時間
    static const TimeInterval BLOCKING_RECV_TIMEOUT_MS;

    // 受信時に、ソケットにひもづけられたアドレスとデータグラム再構成用バッファを探すためのmap。
    qMap<DriverSocketSharedPtr, ReceivingInfo> m_socketToReceiviengInfoMap;

    // 生のソケットからSharedPtrのソケットを探すためのmap
    qMap<SocketDriver::Socket*, DriverSocketSharedPtr> m_rawSocketToSharedPtrSocketMap;

    // 送信時に、アドレスにひもづけられたソケットを探すためのmap
    qMap<InetAddress, SendingInfo> m_addrToSendingInfoMap;

    // 削除予定のソケットを保持する。ProcessRemovedSocket()が呼び出されるとm_closedSocketsに移動する。
    qVector<DriverSocketSharedPtr> m_removingSockets;

    // Closeされてdelete待ち状態のソケット情報を保持するlist
    qList<ClosedSocketInfo> m_closedSockets;

    // データを受信可能なソケットを保持するキャッシュ
    SharedPtr<qReservedQueue<DriverSocketSharedPtr> > m_pReceivableSockets;
    SharedPtr<qVector<SocketDriver::PollInfo> > m_pTempPollInfo;
    SharedPtr<qVector<DriverSocketSharedPtr> > m_pTempSocketsToCheck;

    qBool m_bPrepared;      // PrepareToListen()もしくはPrepareToConnect()を呼び出して準備が完了したか。
    qBool m_bTempPrepared;  // PrepareToListen()もしくはPrepareToConnect()を呼び出して準備が完了したか。排他処理用の一時変数。

    // m_socketToReceiviengInfoMapとm_rawSocketToSharedPtrSocketMapと
    // m_addrToSocketMapとm_closedSocketsとm_bTempPreparedをマルチスレッドでアクセスしても大丈夫なように
    // 排他するためのクリティカルセクション
    // m_socketToReceiviengInfoMapおよびm_rawSocketToSharedPtrSocketMapおよび
    // m_addrToSocketMapおよびm_closedSocketsおよびm_bPreparedを単体で排他するためと、
    // お互いをアトミックに更新するために使う。
    mutable ForcedCriticalSection m_cs;

    // データグラム化して送受信するかどうか。TCP等のストリーム型の通信で有効にする必要がある。
    qBool m_bMakeDatagram;
    SocketDriver* m_pSocketDriver;      // 管理するSocketDriver::Socketに対応するSocketDriver
    qBool m_bCreateTempSocketsToCheck;  // UpdateReceivableSockets()の度にチェックするソケットをリストアップするかどうか
};

END_NAMESPACE_NEX
