﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/qSet.h>
#include <OnlineCore/src/Platform/Stack/Core/InetDecl.h>
#include <OnlineCore/src/Platform/Stack/Core/SocketDriver.h>

struct libwebsocket;

BEGIN_NAMESPACE_NEX

class ReceiveEventHandlerInterface
{
public:
    virtual void ReceiveEventRaised(SocketDriver::Socket* socket) = 0;
    virtual void ReceiveEventLowered(SocketDriver::Socket* socket) = 0;
};

class AcceptedSocketCreatorInterface
{
public:
    virtual SocketDriver::Socket* CreateAcceptedSocket(const SocketDriver::Socket& socket, libwebsocket* wsi) = 0;
};

class ServerWebSocketDriver: public WebSocketDriver, public ReceiveEventHandlerInterface, public AcceptedSocketCreatorInterface
{
public:
    class ServerWebSocket;

    ServerWebSocketDriver()
        : m_sslAllowSelfSignedCerts(false)
        , m_sslRootCaCertPath()
        , m_sslServerCertFilePath()
        , m_sslPrivateKeyFilePath()
        , m_ePollEventRaised(NULL)
        , m_pCsForPollEventRaised(NULL)
        , m_numCreatedSockets(0)
        , m_pReturnHttpStatusInterface(NULL)
        , m_additionalHandshakeHeader()
        , m_isHttpServerOnly(false)
    {}
    virtual ~ServerWebSocketDriver()
    {
        if (m_ePollEventRaised)
        {
            qDelete m_ePollEventRaised;
        }
    }

    virtual Socket *Create();
    virtual SocketDriver::Socket* CreateAcceptedSocket(const SocketDriver::Socket& listenSocket, libwebsocket* wsi);
    virtual void Delete(SocketDriver::Socket *pSocket);

    virtual qBool Poll(PollInfo aPollInfo[], qUnsignedInt uiPollInfoSize, qUnsignedInt32 uiTimeoutInMs);
    virtual qBool CanUseGetAllReceivableSockets() const
    {
        return true;
    }
    virtual size_t GetAllReceivableSockets(SocketDriver::Socket* raisedSockets[], size_t numRaisedSockets, qUnsignedInt32 uiTimeoutInMs);

    // 自己署名された証明書を許容するかどうかをセットする。Connect側のみ影響する。
    virtual void SetSslAllowSelfSignedCerts(qBool allow)
    {
        m_sslAllowSelfSignedCerts = allow;
    }

    // 自己署名された証明書を許容するかどうかを取得する。
    virtual qBool GetSslAllowSelfSignedCerts()
    {
        return m_sslAllowSelfSignedCerts;
    }

    // 使用するルートCA証明書のパスをセットする。Connect側のみ影響する。
    virtual void SetSslRootCaCertPath(const String path)
    {
        m_sslRootCaCertPath = path;
    }

    // 使用するルートCA証明書のパスを取得する。
    virtual String GetSslRootCaCertPath() const
    {
        return m_sslRootCaCertPath;
    }

    // サーバ証明書のファイルパスをセットする。Accept側のみ影響する。
    virtual void SetSslServerCertFilePath(const String& filePath)
    {
        m_sslServerCertFilePath = filePath;
    }

    // 秘密鍵のファイルパスをセットする。Accept側のみ影響する。
    virtual void SetSslPrivateKeyFilePath(const String& filePath)
    {
        m_sslPrivateKeyFilePath = filePath;
    }

    // 受信するものがあることを記録する
    virtual void ReceiveEventRaised(SocketDriver::Socket* socket)
    {
        ScopedPtrCS cs(m_pCsForPollEventRaised);
        SYSTEMCHECK(m_ePollEventRaised != NULL);
        m_ePollEventRaised->insert(socket);
    }

    // 受信するものがなくなったことを記録する
    virtual void ReceiveEventLowered(SocketDriver::Socket* socket)
    {
        ScopedPtrCS cs(m_pCsForPollEventRaised);
        SYSTEMCHECK(m_ePollEventRaised != NULL);
        m_ePollEventRaised->erase(socket);
    }

    virtual void SetReturnHttpStatusInterface(ReturnHttpStatusInterface* pReturnHttpStatusInterface)
    {
        m_pReturnHttpStatusInterface = pReturnHttpStatusInterface;
    }

    ReturnHttpStatusInterface* GetReturnHttpStatusInterface() const
    {
        return m_pReturnHttpStatusInterface;
    }

    // 接続時に使用する追加のヘッダをセットする。\r\nで区切り、\r\nで終わる必要がある。
    virtual void SetAdditionalHandshakeHeader(const String& additionalHandshakeHeader)
    {
        m_additionalHandshakeHeader = additionalHandshakeHeader;

        // テストでメモリリークを検出しないように、空文字列が指定された場合にメモリを解放する。
        m_additionalHandshakeHeader.ShrinkToFit();
    }

    const String& GetAdditionalHandshakeHeader() const
    {
        return m_additionalHandshakeHeader;
    }

    virtual void SetHttpServerOnly(qBool flag)
    {
        m_isHttpServerOnly = flag;
    }
private:
    qBool PollOnce(PollInfo aPollInfo[], qUnsignedInt uiPollInfoSize);
    size_t GetAllReceivableSocketsOnce(SocketDriver::Socket* raisedSockets[], size_t numRaisedSockets);
    void DoWorkPollInfo(PollInfo aPollInfo[], qUnsignedInt uiPollInfoSize);

    qBool m_sslAllowSelfSignedCerts;
    String m_sslRootCaCertPath;
    String m_sslServerCertFilePath; // サーバ証明書のファイルパス。初期化時にメモリアロケートされない。
    String m_sslPrivateKeyFilePath; // 秘密鍵のファイルパス。初期化時にメモリアロケートされない。
    qSet<SocketDriver::Socket*>* m_ePollEventRaised;  // イベントが発生したソケットのリスト。初期化時にメモリアロケートしないようにポインタにしている。
    ForcedCriticalSection* m_pCsForPollEventRaised;     // m_ePollEventRaisedの競合を回避するためのクリティカルセクション。トランスポートバッファスレッド利用時のみ生成される。
    size_t m_numCreatedSockets;     // Createで作成したServerWebSocketの数(Acceptしたものは含まない)
    ReturnHttpStatusInterface* m_pReturnHttpStatusInterface;    // WebSocketではないHTTP要求を受け付けるインタフェース
    String m_additionalHandshakeHeader;                         // ハンドシェイク時に追加で指定するヘッダ
    qBool m_isHttpServerOnly;
};


END_NAMESPACE_NEX
