﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once


#include <OnlineCore/src/Platform/Stack/Core/InetDecl.h>
#include <OnlineCore/src/Platform/Stack/Core/TransportProtocol.h>

BEGIN_NAMESPACE_NEX

    //group=Transport Protocol

/*
Description: This class is an abstract class that should be derived from to implement
a custom socket layer for $PRODUCTNAME$.  This should be use to replace the default
$PRODUCTNAME$ platform-specific socket interface implementation.  Example usage would be
using a different platform-provided socket implementation or any data
transmission API, so long as it can be mapped to the SocketDriver abstraction. The implementation
should be totally non-blocking to avoid system locks.
Remark: Normally, $PRODUCTNAME$ should only be opening UDP sockets.  TCP is used for the UPnP extension.
*/
class SocketDriver: public RootObject
{
public:
    //group=

    /*
       VirtualSocketDriverの互換性のために用意。TransportProtocol::Typeと同じ値が定義されます。
    */
    typedef enum _TrafficType
    {
        // Traffic is UDP
        UDP = TransportProtocol::TYPE_UDP,
        // Traffic is TCP
        TCP = TransportProtocol::TYPE_TCP,
        // Traffic is WebSocket
        WEB_SOCKET = TransportProtocol::TYPE_WEB_SOCKET,
        TRAFFIC_TYPE_UNKNOWN = TransportProtocol::TYPE_UNKNOWN
    } TrafficType;

    /*
    Description: This enumerates the various poll events that can be used by SocketDriver::PollInfo
    エラーはReadyToReadに含まれる事に注意。
    See Also: _PollEvent
    */
    typedef enum _PollEvent
    {
        // No events raised or to check for.
        None = 0,
        // ブロックなしで受信可能。もしくは何かエラーが発生した。
        ReadyToRead = 1,
        // ブロックなしで送信可能。もしくはコネクションが確立された。
        ReadyToWrite = 2
    } PollEvent;
    /*
    Description: This enumerates the various result codes.
    See Also: _PollEvent
    */
    typedef enum _Result
    {
        // The call completed successfully.
        Success,
        // The call completed with error.
        Error,
        // The call was not performed since it would have blocked.
        WouldBlock,
        // The call is in progress.
        InProgress,
        // Socket Operation on not-socket. may be closed
        NotSock,
        // サーバへ接続中にサーバからメンテナンス中というレスポンスがあった。
        UnderMaintenance,
        // サーバへ接続中にサーバからサービス終了というレスポンスがあった。
        ServiceNoLongerAvailable,
        // サーバへ接続中にサーバからアプリケーションバージョンエラーというレスポンスがあった。
        ApplicationVersionIsOld,
        // サーバへ接続中にサーバからサービス休止中というレスポンスがあった。
        OutOfService,
        // サーバへ接続中にサーバからコネクション数限界というレスポンスがあった。
        MaxConnectionsReached
     } Result;

    //n2967
    typedef enum _SocketFlag
    {
        FlagNone = (qInt32)0,
        NonBlock = (qInt32) 0x01
    } SocketFlag;

    //group=
    //n1589:memo:I think this class name should be changed not to be confused with Quazal::InetAddress
    class InetAddress
    {
    public:
        InetAddress()
            : m_ui32Address(0)
            , m_ui32ProxyAddress(0)
            , m_ui16Port(0)
        {}
        ~InetAddress() {}

        /*
        Parameters:
        ui32Address: The numeric IP address.
        ui16Port: The IP port number.
        Remarks: The InetAddress class representation is always host-byte-ordered.
        */
        InetAddress(qUnsignedInt32 ui32Address, qUnsignedInt16 ui16Port)
            : m_ui32Address(ui32Address)
            , m_ui32ProxyAddress(0)
            , m_ui16Port(ui16Port)
        {}

        /*
        Description: Sets the numeric IP address.
        Parameters:
        ui32Address: The numeric IP address.
        Remarks: The InetAddress class representation is always host-byte-ordered.
        */
        void SetAddress(const qUnsignedInt32 ui32Address)
        {
            m_ui32Address = ui32Address;
        }

        /*
        Description: Sets the IP port number.
        Parameters:
        ui16Port: The IP port number.
        Remarks: The InetAddress class representation is always host-byte-ordered.
        */
        void SetPortNumber(const qUnsignedInt16 ui16Port)
        {
            m_ui16Port = ui16Port;
        }

        /*
        Description: Returns the numeric IP address.
        Remarks: The InetAddress class representation is always host-byte-ordered.
        */
        qUnsignedInt32 GetAddress() const
        {
            return m_ui32Address;
        }

        /*
        Description: Returns the IP port number.
        Remarks: The InetAddress class representation is always host-byte-ordered.
        */
        qUnsignedInt16 GetPortNumber() const
        {
            return m_ui16Port;
        }

        void SetProxyAddress(qUnsignedInt32 ui32Address)
        {
            m_ui32ProxyAddress = ui32Address;
        }

        qUnsignedInt32 GetProxyAddress() const
        {
            return m_ui32ProxyAddress;
        }
    private:
        qUnsignedInt32 m_ui32Address;
        qUnsignedInt32 m_ui32ProxyAddress;
        qUnsignedInt16 m_ui16Port;
    };

    //group=
    class Socket: public RootObject
    {
        friend class SocketDriver;
    public:
        /*
        Description: Opens the socket according to the given traffic type.
        Parameters:
        transportProtocolType: Requested transport protocol type for the socket.
        Returns: A boolean indicating if the call succeeded.
        Remark: The socket should behave in a non-blocking mode.
        See Also: SocketDriver::Socket::Close, SocketDriver::Socket::Bind
        */
        virtual qBool Open(TransportProtocol::Type transportProtocolType) = 0;

        /*
        Description: Closes the socket.
        See Also: SocketDriver::Socket::Open
        */
        virtual void Close() = 0;

        // テスト用に内部のソケットをクローズするなどしてエラーを発生させる。
        virtual void CloseForTest()
        {
            SYSTEMCHECK(false);
        }

        /*
        Description: Binds the socket to a given port. Binds a dynamic port when given 0.
        Parameters:
        ui16Port: The host-byte-ordered IP port to bind.
                  Returns the actual port that was bound (if 0 was passed).
        Returns: A boolean indicating if the call succeeded.
        */
        virtual qBool Bind(qUnsignedInt16 & ui16Port) = 0;

        /*
        Description: Reads data from opened and bound socket.
        Parameters:
        pBuffer: Pointer to a byte buffer to put the read data.
        uiBufferSize: Size of the buffer pointed by pBuffer.
        pSourceAddress: Pointer to an InetAddress object to store the source address of the packet, the sender.
        puiNbBytesRecvd: Pointer to a qUnsignedInt to store the number of bytes read.
        Returns:
        Success: Data was successfully read from the socket.  pBuffer, *pSourceAddress and
        *puiNbBytesReceived are updated.
        Error: There was an error reading data from the socket.
        WouldBlock: Nothing is ready to be read on the socket.
        Remarks: This call should not block if there is nothing ready to be read on the socket
        and return WouldBlock.
        RecvFrom should only be called on UDP sockets.
        See Also: SocketDriver::Socket::SendTo, SocketDriver::Poll
        */
        virtual Result RecvFrom(
            qByte *pBuffer,
            size_t uiBufferSize,
            InetAddress *pSourceAddress,
            size_t *puiNbBytesRecvd,
            SocketFlag flags = NonBlock
        ) = 0;
        /*
        Description: Sends data from opened and bound socket.
        Parameters:
        pBuffer: Pointer to a byte buffer containing the data to send.
        uiBufferSize: Size of the buffer pointed by pBuffer.
        refDestinationAddress: A reference to the InetAddress to which pBuffer should be sent.
        puiNbBytesSent: Pointer to a qUnsignedInt to store the number of bytes actually sent.
        Returns:
        Success: Data was successfully sent from the socket and *puiNbBytesSent is updated.
        Error: There was an error sending data from the socket.
        WouldBlock: The socket is not ready to send data without blocking.
        Remarks: This call should never block and should return WouldBlock if the socket is not ready for
        sending data.
        現在のブロッキングモードにかかわらずノンブロックで送信する。
        SendTo should only be called on UDP sockets.
        See Also: SocketDriver::Socket::RecvFrom, SocketDriver::Poll
        */
        virtual Result SendTo(
            const qByte *pBuffer,
            size_t uiBufferSize,
            const InetAddress &refDestinationAddress,
            size_t *puiNbBytesSent
        ) = 0;

        //n2967
        virtual Result Shutdown() = 0;

        /*
        Description: Connect to the specified InetAddress.
        Parameters:
        refInetAddress: A reference to the InetAddress to connect to.
        Returns:
        Success: Socket is now connected.
        Error: There was an error connecting to the socket.
        InProgress: The connection has been initiated.
        See Also: SocketDriver::Recv, SocketDriver::Socket::Send, SocketDriver::Poll
        */
        virtual Result Connect(const SocketDriver::InetAddress &refInetAddress) = 0;

        // コネクション指向ではないプロトコルでは常にSuccessを返す
        // コネクション指向でないプロトコルでも呼び出しが必要。呼び出した後はIsListening()がtrueを返すようになる。
        // サーバ側で呼び出されることになる。
        // Accept可能な状態に遷移する
        virtual Result Listen()
        {
            return Success;
        }

        // Listen()を呼び出したかどうか。
        virtual qBool IsListening() const
        {
            return false;
        }

        // Accept()で作成されたソケットか
        virtual qBool IsAcceptedSocket() const = 0;

        // オブジェクトを削除して良いかどうか。削除できないタイミングの時はfalseを返す。
        virtual qBool IsReadyToDelete() const
        {
            return true;
        }

        // TCP等のコネクション指向のプロトコルを扱う場合に必要なインタフェース
        // コネクション指向ではないプロトコルでは常にErrorを返す
        // Accept済みのソケットに対してConnectしてきた接続を受け入れる。
        virtual Result Accept(SocketDriver::InetAddress* acceptedInetAddress, SocketDriver::Socket** acceptedSocket)
        {
            DUMMY_PARAM(acceptedInetAddress);
            DUMMY_PARAM(acceptedSocket);
            return Error;
        }

        // TCP等のコネクション指向のプロトコルを扱う場合に必要なインタフェース
        // コネクション指向ではないプロトコルでは常にErrorを返す
        /*
        Description: Reads data from opened and bound socket.
        Parameters:
        pBuffer: Pointer to a byte buffer to put the read data.
        uiBufferSize: Size of the buffer pointed by pBuffer.
        puiNbBytesRecvd: Pointer to a qUnsignedInt to store the number of bytes read.
        Returns:
        Success: Data was successfully read from the socket.  pBuffer, *pSourceAddress and
        *puiNbBytesReceived are updated.
        Error: There was an error reading data from the socket.
        WouldBlock: Nothing is ready to be read on the socket.
        NotSock: 切断を検知した。
        Remarks: This call should not block if there is nothing ready to be read on the socket
        and return WouldBlock.  Recv should only be called on TCP sockets.
        See Also: SocketDriver::Socket::SendTo, SocketDriver::Poll
        */
        virtual Result Recv(qByte *pBuffer, size_t uiBufferSize, size_t *puiNbBytesRecvd, SocketFlag flags)
        {
            DUMMY_PARAM(pBuffer);
            DUMMY_PARAM(uiBufferSize);
            DUMMY_PARAM(puiNbBytesRecvd);
            DUMMY_PARAM(flags);
            return Error;
        }

        /*
        Description: Sends data from opened and bound socket.
        Parameters:
        pBuffer: Pointer to a byte buffer containing the data to send.
        uiBufferSize: Size of the buffer pointed by pBuffer.
        puiNbBytesSent: Pointer to a qUnsignedInt to store the number of bytes actually sent.
        Returns:
        Success: Data was successfully sent from the socket and *puiNbBytesSent is updated.
        Error: There was an error sending data from the socket.
        WouldBlock: The socket is not ready to send data without blocking.
        Remarks: This call should never block and should return WouldBlock if the socket is not ready for
        sending data.
        現在のブロッキングモードにかかわらずノンブロックで送信する。
        Send should only be called on TCP sockets.
        See Also: SocketDriver::Socket::RecvFrom, SocketDriver::Poll
        */
        virtual Result Send(const qByte *pBuffer, size_t uiBufferSize, size_t *puiNbBytesSent) = 0;

        /*
        Description: Sets the multicast address
        Parameters:
        uiAddress: The numeric IP address.
        Returns: True on success, false otherwise.
        Remarks: uiAddress representation is always host-byte-ordered.
        SetMulticastAddress should only be called on TCP sockets.
        */
        virtual qBool SetMulticastAddress(qUnsignedInt32 uiAddress)
        {
            DUMMY_PARAM(uiAddress);
            return true;
        };

        virtual qBool SetTTL(qUnsignedInt8)
        {
            return false;
        }

#if defined(WIN32_TARGET) || defined(LINUX_TARGET)
        // 重複したアドレスをbindできるようにする/しない (設定しない状態では「しない」)
        virtual qBool SetReuseAddr(qBool bReuse)
        {
            (void)bReuse;
            return false;
        }
#endif
        // フレーム毎の定期実行処理を行う。
        virtual void DoWork()
        {
        }

        // 生のソケットハンドルを扱っているSocketならそれを返す。
        // この関数をサポートするSocketはBerkeleySocketDriver::StaticPoll()とBerkeleySocketDriver::CheckOrWaitForTransmission()を利用できる。
        virtual SocketHandle GetRawSocketHandle() const
        {
            return NEX_INVALID_SOCKET;
        }

        // クラス名を取得する
        virtual const qChar8* GetClassNameString() const
        {
            return "SocketDriver::Socket";
        }

        // トランスポートプロトコルによっては必要となるURI情報をセットする。
        // WebSocketの場合に接続先として利用する。Connect前に呼び出す必要がある。
        virtual void SetUri(const String& uri)
        {
            DUMMY_PARAM(uri);
            QLOG(EventLog::Warning, NEX_T("URI is set but this SocketDriver does not support full URI representation. Only use address/port. uri: ")
                 << uri);
        }

        // 参照数を上げる。
        void AcquireRef()
        {
            SYSTEMCHECK(m_refCount >= 0);
            ++m_refCount;
        }

        // 参照数を下げる。参照数が0以下になったらtrueを返す。
        // つまり同数のAcquireRefとReleaseRefが呼び出されるか、最初にReleaseRefが呼び出されるとtrueが返る。
        qBool ReleaseRef()
        {
            SYSTEMCHECK(m_refCount >= 0);
            return --m_refCount <= 0;
        }

        // 参照数を取得する。
        qInt GetRefCount() const
        {
            return m_refCount;
        }
    protected:
        Socket()
            : m_refCount(0)
        {
        }

        virtual ~Socket() {}

        // 参照数。生成時は0。0以下のときは安全にクローズできる。1以上のときは使っているコネクションがあるのでクローズするとそれが切れる。
        AtomicValue m_refCount;
    };

    // SocketDriver::Poll()に指定するクラス。
    // memcpyしたいのでこのクラスはPODクラスである必要がある。
    class PollInfo: public RootObject
    {
    public:
        PollInfo()
            : m_pSocket(NULL)
            , m_eEventsToPollFor(None)
            , m_eRaisedEvents(None)
        {
        }
        PollInfo(Socket* socket, PollEvent eEventsToPollFor)
            : m_pSocket(socket)
            , m_eEventsToPollFor(eEventsToPollFor)
            , m_eRaisedEvents(None)
        {
        }

        // 対象のソケットが一致しているか
        bool operator==(const PollInfo& rhs)
        {
            return m_pSocket == rhs.m_pSocket;
        }

        // Pointer to the socket to poll for.
        Socket          *m_pSocket;
        // Events to poll for.
        PollEvent       m_eEventsToPollFor;
        // Events raised by poll.
        PollEvent       m_eRaisedEvents;
    };

public:
    SocketDriver() {}

    virtual ~SocketDriver() {}

    /*
    Description: Creates a specific socket.
    Remarks: This method is a factory of SocketDriver::Socket.  If you derive from SocketDriver, you should
    return an instance of your specific SocketDriver::Socket object.
    See Also: SocketDriver::Delete
    */
    virtual Socket *Create() = 0;

    /*
    Description: Deletes a specific socket.
    See Also: SocketDriver::Create
    */
    virtual void Delete(Socket *pSocket) = 0;

    /*
    Description: Polls sockets for events.
    Parameters:
    aPollInfo: An array of SocketDriver::PollInfo on which to poll.
    uiPollInfoSize: Number of items in aPollInfo.
    uiTimeoutInMs: Timeout in milliseconds.
    Returns: True if at least one event is raised, thus one socket in aPollInfo is ready.  False if
    no events were raised within the specified timeout.
    Remarks: This call should block until at least one event is raised or timeout has occurred.
    See Also: SocketDriver::PollInfo
    */
    // m_eRaisedEventsは0クリアしてある必要がある。
    virtual qBool Poll(PollInfo aPollInfo[], qUnsignedInt uiPollInfoSize, qUnsignedInt32 uiTimeoutInMs) = 0;

    virtual qBool CanUseGetAllReceivableSockets() const
    {
        return false;
    }
    virtual size_t GetAllReceivableSockets(SocketDriver::Socket* raisedSockets[], size_t numRaisedSockets, qUnsignedInt32 uiTimeoutInMs)
    {
        DUMMY_PARAM(raisedSockets);
        DUMMY_PARAM(numRaisedSockets);
        DUMMY_PARAM(uiTimeoutInMs);
        SYSTEMCHECK(false);
        return 0;
    }

    /*!
     * @brief 接続時に使用するスレッドの優先度を設定します。
     *
     * @param[in] priority 優先度
     */
    static void SetConnectThreadPriority(qUnsignedInt priority)
    {
        s_connectThreadPriority = priority;
    }

    /*!
     * @brief 接続時に使用するスレッドの優先度を取得します。
     *
     * @param[in] priority 優先度
     */
    static qUnsignedInt GetConnectThreadPriority()
    {
        return s_connectThreadPriority;
    }

    /*!
     * @brief ゲームサーバーとの接続用スレッドが利用する CPU の コア番号を設定します。
     *
     * @param[in] no CPU コア番号
     */
    static void SetConnectThreadCoreNo(qInt32 no)
    {
        s_ConnectThreadCoreNo = no;
    }

    /*!
     * @brief ゲームサーバーとの接続用スレッドが利用する CPU の コア番号を取得します。
     *
     * @return CPU コア番号
     */
    static qInt32 GetConnectThreadCoreNo()
    {
        return s_ConnectThreadCoreNo;
    }

    static const qUnsignedInt DEFAULT_CONNECT_THREAD_PRIORITY;
protected:
    static qUnsignedInt s_connectThreadPriority;
    static qInt32 s_ConnectThreadCoreNo;
};

/*!
 !private
 @brief SharedPtrで使用するSocketDriver向けのdeleterです。
 */
struct SocketDriverDeleter : public RootObject
{
    SocketDriverDeleter()
        : m_pSocketDriver(NULL)
    {
    }

    SocketDriverDeleter(SocketDriver* socketDriver)
        : m_pSocketDriver(socketDriver)
    {
    }

    void operator()(SocketDriver::Socket* pSocket)
    {
        m_pSocketDriver->Delete(pSocket);
    }
private:
    SocketDriver* m_pSocketDriver;
};

class ReturnHttpStatusInterface
{
public:
    static const size_t MAX_HTML_BODY_SIZE = 256;
    static const size_t MAX_CONTENT_TYPE_SIZE = 64;

    /*! @brief HTTPのステータスコードをHTMLの本文とともに返すコールバック
     *
     * @param pCode HTTPステータスコード
     * @param pBody HTML本文文字列
     * @param pContentType Content-Typeを示す文字列。無変更、空文字列の場合は、pCodeとpBodyをHTMLとして返す。
     * @param pEndPointName HTTPのEndPointを文字列
     * @param endPointNameLen HTTPのEndPointを文字列長
     */
    virtual qBool ReturnHttpStatus(qUnsignedInt *pCode, qChar8 *pBody, qChar8 *pContentType, const qChar8* pEndPointName, size_t endPointNameLen) = 0;
};

// WebSocket向けのSocketDriverインタフェース
class WebSocketDriver : public SocketDriver
{
public:
    // 自己署名された証明書を許容するかどうかをセットする。Connect側のみ影響する。
    virtual void SetSslAllowSelfSignedCerts(qBool allow)
    {
        DUMMY_PARAM(allow);
        SYSTEMCHECK(false);
    }

    // 自己署名された証明書を許容するかどうかを取得する。
    virtual qBool GetSslAllowSelfSignedCerts()
    {
        SYSTEMCHECK(false);
        return false;
    }

    // 使用するルートCA証明書のパスをセットする。Connect側のみ影響する。
    virtual void SetSslRootCaCertPath(const String path)
    {
        DUMMY_PARAM(path);
        SYSTEMCHECK(false);
    }

    // 使用するルートCA証明書のパスを取得する。
    virtual String GetSslRootCaCertPath() const
    {
        SYSTEMCHECK(false);
        return String();
    }

    // サーバ証明書のファイルパスをセットする。Accept側のみ影響する。
    virtual void SetSslServerCertFilePath(const String& filePath)
    {
        DUMMY_PARAM(filePath);
        SYSTEMCHECK(false);
    }

    // 秘密鍵のファイルパスをセットする。Accept側のみ影響する。
    virtual void SetSslPrivateKeyFilePath(const String& filePath)
    {
        DUMMY_PARAM(filePath);
        SYSTEMCHECK(false);
    }

    // WebSocketにUpgradeしない素のHTTP接続を処理するコールバックを指定する。
    virtual void SetReturnHttpStatusInterface(ReturnHttpStatusInterface* pReturnHttpStatusInterface)
    {
        DUMMY_PARAM(pReturnHttpStatusInterface);
        SYSTEMCHECK(false);
    }

    // 接続時に使用する追加のヘッダをセットする。\r\nで区切り、\r\nで終わる必要がある。
    virtual void SetAdditionalHandshakeHeader(const String& additionalHandshakeHeader)
    {
        DUMMY_PARAM(additionalHandshakeHeader);
        SYSTEMCHECK(false);
    }

#ifdef LIVE_VERSION
    // WebSocket接続は拒否して、HTTPサーバーとしてのみ動作する。
    virtual void SetHttpServerOnly(qBool flag)
    {
        DUMMY_PARAM(flag);
        SYSTEMCHECK(false);
    }
#endif
};

END_NAMESPACE_NEX

