﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Core/PluginObject.h>
#include <OnlineCore/src/Transport/Interface/RootTransport.h>
#include <OnlineCore/src/Transport/Interface/EndPoint.h>
#include <OnlineCore/src/Transport/Interface/EndPointGroup.h>
#include <OnlineCore/src/Transport/Interface/TransportCaps.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>
#include <OnlineCore/src/Platform/Core/UserContext.h>

BEGIN_NAMESPACE_NEX

class TransportEventHandler;
class OutputEmulationDevice;
class InputEmulationDevice;


//Group=Transport Protocol

//Description: Specifies atributes for a transport protocol.
//Remarks: This class is used to specify specific atributes of a transport protocol.
class ConnectionOrientedStream: public Stream
{

    friend class EndPoint;
    friend class EndPointGroup;

public:

    //Group=Methods

    //{secret}
    typedef void (*Callback)(ConnectionOrientedStream *, Buffer *pBuffer, qUnsignedInt32 uiResult, UserContext *pContext);

    ConnectionOrientedStream(Stream::Type oType, RootTransport *pTransport = NULL);
    virtual ~ConnectionOrientedStream();

    //{secret}
    virtual qBool           StartListen(Stream::ID idStream = 0,
                                        Stream::ID *oStreamID = NULL);
    //{secret}
    virtual qBool           StopListen();

    //{secret}
    Stream::ID              GetID()
    {
        return m_idStream;
    }
    //{secret}
    qBool                   IsListening()
    {
        return m_idStream != 0;
    }

    //{secret}
    virtual qBool           ResponsibleForURL(const StationURL *pStationURL) = 0;
    //{secret}
    virtual const qChar     *GetURLType() = 0;

    //{secret}
    virtual EndPoint        *OpenEndPoint(const StationURL *pStationURL) = 0;
    //{secret}
    virtual EndPoint        *OpenEndPoint(const ConnectionID cidStation) = 0;
    //{secret}
    virtual qBool           OpenEndPoint(EndPoint* /*pEndPoint*/);
    //{secret}
    virtual EndPoint        *FindEndPointByPID(const PrincipalID /* pidStation */);
    //{secret}
    virtual qUnsignedInt32  GetEndPointNumber();
    //{secret}
    virtual EndPoint        *FindEndPointByCID(const ConnectionID /* cidStation */);
    //{secret}
    virtual const qVector<ConnectionID> GetAllConnectionIDs() const;

    virtual void            CloseEndPoint(EndPoint *pEndPoint) = 0;

    //{secret}
    virtual void            LockEndPointMap();
    //{secret}
    virtual void            UnlockEndPointMap();

    //{secret}
    virtual qResult         Initialize();
    //{secret}
    virtual qResult         Teardown();

    //{secret}
    virtual qResult         RegisterEventHandler(TransportEventHandler *pEventHandler);

    //{secret}
    virtual qResult         SendBroadcast(StationURL *pStationURL, Buffer *pBuffer);

    //{secret}
    virtual qResult         Send(StationURL *pStationURL, Buffer *pBuffer);

    //{secret}
    virtual qResult         AddEndPointToGroup(EndPoint *pEndPoint, EndPointGroup *pEndPointGroup);
    //{secret}
    virtual qResult         RemoveEndPointFromGroup(EndPoint *pEndPoint, EndPointGroup *pEndPointGroup);

    //{secret}
    TransportEventHandler   *GetEventHandler()
    {
        return m_pEventHandler;
    }

    /*
    Description: Returns whether or not the currently loaded transport protocol is capable of supporting the
    specified capability.
    Parameters:
    uiCapability: The capability of the transport protocol. The valid transport capabilites are listed in
    Transport Parameters.
    Returns: true if the currently loaded transport protocol is capable of supporting the specified
    capability, false if not or an error occurred.
    Remarks: Use this method to determine the capabilities of the currently loaded transport protocol.
    See Also: Transport Protocol
    */
    virtual qBool           IsCapable(qUnsignedInt32 uiCapability) = 0;


    //Description: Returns a pointer to an instance of the OutputEmulationDevice.
    //Returns: A pointer to an instance of the OutputEmulationDevice.
    OutputEmulationDevice   *GetOutputEmulationDevice()
    {
        return m_pUDPTransport->GetOutputEmulationDevice();
    }

    //Description: Returns a pointer to an instance of the InputEmulationDevice.
    //Returns: A pointer to an instance of the InputEmulationDevice.
    InputEmulationDevice    *GetInputEmulationDevice()
    {
        return m_pUDPTransport->GetInputEmulationDevice();
    }

    // このStreamに紐づけるローカルポートを設定する。
    // 0以外を設定すると、このポート以外宛てに来たパケットは破棄する。
    // 0(デフォルト)を指定すると、任意のポートに来たパケットを受け入れる。
    void SetPreferredPortNumber(qUnsignedInt16 uiPreferredPortNumber)
    {
        m_uiPreferredPortNumber = uiPreferredPortNumber;
    }
    qUnsignedInt16  GetPreferredPortNumber();

#ifdef NEX_PERFCOUNTER_ENABLED
    //{secret}
    TransportPerfCounters   *GetPerfCounters()
    {
        return m_pUDPTransport->GetPerfCounters();
    }
#endif // NEX_PERFCOUNTER_ENABLED

    // Description: Traces the content of this Transport object.
    virtual void Trace(qUnsignedInt64 /*uiFlags = TRACE_ALWAYS*/);

protected:
    // 以下の関数はフレンドクラスであるEndPointから参照される。
    //{secret}
    virtual void            AddPIDEndPointAssociation(const PrincipalID /* pidStation */, EndPoint* /* pEndPoint */);
    //{secret}
    virtual qBool           RemovePIDEndPointAssociation(const PrincipalID /* pidStation */, EndPoint* /* pEndPoint */);
    //{secret}
    virtual void            AddCIDEndPointAssociation(const ConnectionID /* cidStation */, EndPoint* /* pEndPoint */);
    //{secret}
    virtual qBool           RemoveCIDEndPointAssociation(const ConnectionID /* cidStation */, EndPoint* /* pEndPoint */);

    TransportEventHandler           *m_pEventHandler;
    //n1589:todo1:rename to m_pTransport
    RootTransport                   *m_pUDPTransport;
    Stream::ID                      m_idStream;
    qUnsignedInt16                  m_uiPreferredPortNumber;
};

// Helper class
class EndPointMapLocker
{
public:
    EndPointMapLocker(ConnectionOrientedStream* pStream) : m_pStream(pStream)
    {
        m_pStream->LockEndPointMap();
    }
    ~EndPointMapLocker()
    {
        m_pStream->UnlockEndPointMap();
    }
private:
    ConnectionOrientedStream* m_pStream;
};



END_NAMESPACE_NEX

