﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/StringStream.h>

BEGIN_NAMESPACE_NEX

//group=NAT Traversal Service

/*
Description: This is a container for Quazal's simplified NAT properties (based on the RFC-4787
classification) detected on a player's router during login.

Remarks: Contains Quazal's simplified NAT properties (based on the RFC-4787 classification).

Note: Since these values are retrieved during the login process to $PRODUCTNAME$, accessing this object before
the login will return unknown values in the mapping and filtering properties, and both the interface
and public addresses will be 0.0.0.0.
See Also: RootTransport::GetNATProperties
*/
class NATProperties: public RootObject
{
public:
    //group=Type Definitions

    //Description: Enumerates NAT mapping behavior
    enum MappingProperties
    {
        Mapping_u = 0,          // Description: 0, Unknown NAT device type
        Mapping_eim,            // Description: 1, endpoint-independent mapping
        Mapping_edm,            // Description: 2, endpoint-dependent mapping
        Mapping_local           // Description: 3, Local Communication
    };

    //Description: Enumerates NAT filtering behavior
    enum FilteringProperties
    {
        Filtering_u = 0,        // Description: 0, Unknown NAT device type
        Filtering_pif,          // Description: 1, port-independent filtering
        Filtering_pdf,          // Description: 2, port-dependent filtering
        Filtering_local         // Description: 3, Local Communication
    };

    enum PortIncreaseProperties
    {
        PortIncrease_u = 0, //Unknown PortIncrease
        PortIncrease_zero, //Not Port Increase
        PortIncrease_one,  //One Port Increase
        PortIncrease_local //Local Communication
    };


    struct AllProperties
    {
        MappingProperties mapping;
        FilteringProperties filtering;
        PortIncreaseProperties portIncrease;
    };


public:
    //group=Methods

    NATProperties();
    virtual ~NATProperties();

    //{secret}
    void SetInterfaceAddress(const String &strInterfaceAddress);

    /*
    Description: Retrieves the interface address used to detect the NAT properties.
    Returns: The interface address used to detect the NAT properties.
    */
    String GetInterfaceAddress() const;

    //{secret}
    void SetInterfacePort(qUnsignedInt16 uiInterfacePort);

    /*
    Description: Retrieves the interface port used to detect the NAT properties.
    Returns: The interface port used to detect the NAT properties.
    */
    qUnsignedInt16 GetInterfacePort() const;

    //{secret}
    void SetPublicAddress(const String &strPublicAddress);

    /*
    Description: Retrieves the public address returned when detecting NAT properties.
    Returns: The public address returned when detecting NAT properties.
    */
    String GetPublicAddress() const;

    //{secret}
    void SetPublicPort(qUnsignedInt16 uiPublicPort);

    /*
    Description: Retrieves the public port returned when detecting NAT properties.
    Returns: The public port returned when detecting NAT properties.
    */
    qUnsignedInt16 GetPublicPort() const;

    //{secret}
    void SetNATMapping(MappingProperties eMappingType);

    /*
    Description: Retrieves the NAT mapping property.
    Returns: A simplified NAT mapping property based on the RFC-4787 classification [unknown (u), endpoint-independent
    mapping (eim) or endpoint-dependent mapping (edm)].
    */
    MappingProperties GetNATMapping() const;

    //{secret}
    void SetNATFiltering(FilteringProperties eFilteringType);

    /*
    Description: Retrieves the NAT filtering property.
    Returns: A simplified NAT filtering property based on the RFC-4787 classification [unknown (u), port-independent
    filtering (pif) or port-dependent filtering (pdf)].
    */
    FilteringProperties GetNATFiltering() const;

    //{secret}
    void SetNATMappingPortIncrement(qUnsignedInt uiPortIncrement);

    /*
      アドレスごとに独立した空間でEDMな性質をもつルータ(=PD)に対応
     */
    void SetIsPD(qBool bIsPD)
    {
        m_bIsPD = bIsPD;
    }

    qBool IsPD()
    {
        return m_bIsPD;
    }

    //{secret}
    qUnsignedInt GetNATMappingPortIncrement() const;

    PortIncreaseProperties GetPortIncreaseProperties() const;

    /*!
    @brief      NATProperties オブジェクトの内容をトレースします。

    @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
    @internal
    @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const NATProperties& rhs);

    //{secret}
    static String ConvertMappingPropertiesToStr(MappingProperties eMappingType);

    //{secret}
    static String ConvertFilteringPropertiesToStr(FilteringProperties eFilteringType);

    static String ConvertPortIncreasePropertiesToStr(PortIncreaseProperties eType);

    AllProperties GetAllProperties();

    static qUnsignedInt32 ConvertAllPropertiesToInteger(const AllProperties& type);
    static AllProperties ConvertIntegerToAllProperties(qUnsignedInt32 value);



    // Data members
private:
    String m_strInterfaceAddress;
    qUnsignedInt16 m_uiInterfacePort;
    String m_strPublicAddress;
    qUnsignedInt16 m_uiPublicPort;
    MappingProperties m_eNATMapping;
    FilteringProperties m_eNATFiltering;
    qUnsignedInt m_uiPortIncrement;
    qBool m_bIsPD;
};

inline StringStream& operator<<(StringStream& oStr, const NATProperties& rhs)
{
    oStr << NEX_T("NATProperties: (")
        << NEX_T("Mapping=") << NATProperties::ConvertMappingPropertiesToStr(rhs.m_eNATMapping)
        << NEX_T(", Filtering=") << NATProperties::ConvertFilteringPropertiesToStr(rhs.m_eNATFiltering)
        << NEX_T(", PublicAddress=") << rhs.m_strPublicAddress
        << NEX_T(", PublicPort=") << rhs.m_uiPublicPort
        << NEX_T(", InterfacePort=") << rhs.m_uiInterfacePort
        << NEX_T(", PortIncrement=") << rhs.m_uiPortIncrement;
    oStr << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX
