﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Core/PseudoSingleton.h>
#include <OnlineCore/src/Core/Scheduler.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Platform/Core/qProtectedList.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <OnlineCore/src/Transport/Interface/ConnectivityManager.h>
#include <OnlineCore/src/Transport/Interface/TransportAdapter.h>
#include <OnlineCore/src/Core/Encryption/RC4/RC4Encryption.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#ifdef NEX_CLIENT_PBPOOL
#include <OnlineCore/src/Platform/Core/PacketBufferManager.h>
#endif  // NEX_CLIENT_PBPOOL

#include <OnlineCore/src/Core/LiteBuffer.h>
//n2208
#include <OnlineCore/src/Core/Compression/ZLib/ZLibCompression.h>
//n2967
#include <OnlineCore/src/Transport/PRUDP/ProtocolMessageHolder.h>

BEGIN_NAMESPACE_NEX

//group=Transport Protocol
// $ifdef WII_COMMON$
// group=Net-Z API
// $endif$

// The instances it manages.

class RootTransport;
class NATTraversalEngine;
class qBuffer;
class EndPoint;

//n2690
class KeyedChecksumAlgorithm;
/*
    Description: This class encapsulates a network object. It is created by the system when you create the $FACADENAME$ object.
    Remarks:  This singleton can be used to compose a list of StationURL s for the local station, to
    register a custom TransportAdapter and to retrieve pointers to either of the former. Use GetInstance to retrieve
    a pointer to the object.
*/

/*!
  @brief ネットワークオブジェクトをカプセル化するクラスです。
 */
class Network: public RefCountedObject
{
    // Group=
public:

    /*
    Description: Creates a network object.
    Remarks: This object should never be directly instantiated. Instead, it is created
    by the system when you create $FACADENAME$.
    */
    Network();

    /*
    Remarks: This object should never be directly destroyed. It is deleted when you delete
    $FACADENAME$.
    */
    virtual ~Network();

    // Group=Methods

    //  Note: Both AcquireInstance and ReleaseInstance are reserved for product facade objects, only.
    //  Calling either one of them directly may result into undefined behavior.

    // {secret}
    static void AcquireInstance();
    // {secret}
    static void ReleaseInstance();

    /*
    Description: Returns a pointer to the Network instance.
    Returns: A pointer to the Network object.
    */

    /*!
     * @brief Networkのインスタンスを取得します。
     * @return Networkのインスタンス
     */
    static Network* GetInstance()
    {
        InstanceControl* pInstance = InstanceControl::FindInstance(IC_TYPE_NETWORK, PseudoSingleton::GetCurrentContext());
        if (pInstance != NULL)
        {
            return static_cast<Network*>(pInstance->GetDelegatorInstance());
        }
        else
        {
            //n1589
            //              QTRACE(D_TRACE_CONSTRUCT, NEX_T("Network::GetInstance:pInstance == NULL"));
            return NULL;
        }
    }

    // {secret}
    NATTraversalEngine* GetNATTraversalEngine();
    // {secret}
    ConnectivityManager* GetConnectivityManager()
    {
        return &m_oConnectivityManager;
    }


    ProtocolMessageHolder* GetProtocolMessageHolder()
    {
        return &m_oProtocolMessageHolder;
    }

    // {secret}
    void SortAndFilterTargetURLs(qList<StationURL> *plstTargetURLs);

public:
    // {secret}
    RootTransport* GetRootTransport()
    {
        return m_pUDPTransport;
    }

    // Description: Populates purlLocal with the list of
    // StationURL s for the local station.
    // Parameters:
    // purlLocal: List to be populated
    void GetLocalURLs(qList<StationURL> * purlLocal) const;

    void GetMyLocalURLs(ConnectionID rvcid, qList<StationURL>* plstURLs) const;

    // {secret}
    // ローカルステーションのリストにURLを追加
    void AddLocalURL(const StationURL &oURL);
    // {secret}
    // ローカルステーションのリスト中 ポート番号とアドレスとプロトコルが一致するものを全て削除
    qBool RemoveLocalURL(const StationURL &oURL);
    // {secret}
    // ローカルステーションのリスト中 ポート番号とアドレスとプロトコルが一致するものを全て更新する
    qBool ReplaceLocalURL(const StationURL &oURL);

    // {secret}
    // ローカルステーションのリスト中 oTargetURLとポート番号とアドレスが一致するものがあったら、最初の一つをoURLへ更新する
    qBool ReplaceLocalURL(const StationURL &oTargetUrl, const StationURL &oURL);



    // n2619:begin
    // {secret}
    typedef enum
    {
        P2PStateDefault     = 0,
        P2PStateTerminating
    } P2PState;

    // {secret}
    static P2PState GetP2PState()
    {
        return s_eP2PState;
    }

    // {secret}
    static void SetP2PState(P2PState state)
    {
        s_eP2PState = state;
    }

    // {secret}
    const qProtectedList<StationURL>& GetLocalURLs() const;
    // {secret}
    qProtectedList<StationURL>& GetLocalURLs();

    // Description: Registers a custom TransportAdapter to override the system's default one.
    // Parameters:
    // pAdapter: Pointer to the TransportAdapter to register
    // Returns: True if the TransportAdapter was registered succesfully.
    // See Also: TransportAdapter
    static qBool RegisterTransportAdapter(TransportAdapter *pAdapter);

    //n1589
    static qBool UnregisterTransportAdapter();

    // Description: Returns a pointer to the currently registered TransportAdapter.
    // Returns: A pointer to the currently registered TransportAdapter.
    // See Also: TransportAdapter
    static TransportAdapter* GetTransportAdapter()
    {
        //n1589:memo:Networkインスタンスが作られた後はGetTransportAdapterはNULLにならない
        //n1589:UdsNode::PrepareTransportInterfaceで使用するのでコメントアウト
        //          SYSTEMCHECK(s_pTransportAdapter != NULL);
        return s_pTransportAdapter;
    }

    //  Encryption is ON by default
    // {secret}
    static void EnableEncryption();
    // {secret}
    static void DisableEncryption();

    //n1769
    /*!
     @brief P2P 通信で使用するデフォルトの署名計算に用いる鍵データをセットします。

     デフォルトでは、NEXライブラリの P2P 通信には署名によるデータチェックがなされます。
     このAPIを呼び出すと、アプリケーションが設定した鍵データを用いた署名計算が
     行われるようになりますので、通信の安全性を多少高めることができます。
     本関数はローカル通信向けです。
     インターネット通信を行う際は @ref SetP2PDataPacketSessionSignatureKey() を使用してください。

     セットできる鍵データ長は、@ref MIN_P2P_DEFAULT_SIGNATURE_KEY_LEN から @ref MAX_P2P_DEFAULT_SIGNATURE_KEY_LEN
     までの長さです。


     NEX の Core オブジェクトが生成される前に、本関数を実行してください。
     
     @param[in]    pKeyData    鍵データの先頭を指すポインタ
     @param[in]    uiKeyLen    鍵データの長さ（単位はバイト）。0をセットすると、
                               規定の鍵データが使用されます。
     */
    static void SetP2PDataPacketDefaultSignatureKey(const qByte *pKeyData, size_t uiKeyLen);


    static qByte *GetP2PDataPacketDefaultSignatureKey();
    static size_t GetP2PDataPacketDefaultSignatureKeyLength()
    {
        return s_P2PSignatureKeyLen;
    }

    /*!
     @brief P2P 通信で使用するセッションごとの署名計算に用いる鍵データをセットします。


     MatchmakeSession::GetSessionKey()、 MatchmakeExtensionClient::CreateMatchmakeSession()、
     MatchmakeExtensionClient::JoinMatchmakeSession()で取得できる P2P 通信用の署名鍵を設定します。
     MatchmakeSession::GetSessionKey()の取得方法は、マニュアルのセキュリティの章を参考にしてください。
     Network::GetInstance()->SetP2PDataPacketSessionSignatureKey()で設定できます。

     設定された場合には、 Network::SetP2PDataPacketDefaultSignatureKey()で設定されたキーの代わり
     に利用します。
     セットできる鍵データ長は、@ref MIN_P2P_SIGNATURE_KEY_LEN から @ref MAX_P2P_SIGNATURE_KEY_LEN
     までの長さです。

     Session::CreateSession()、 Session::JoinSession()実行前に、本関数を実行してください。
     ConnectivityManager::StartNATSession()がコールされているのに、本関数で鍵が設定されていない場合は、
     Session::CreateSession()、 Session::JoinSession()実行時にエラーとなります。

     LANSessionDiscovery が有効になった場合
     や、 VirtualSocketDriver が設定されており Session::CreateSession() 、 Session::JoinSession() の実行時
     には、クリアされます。

     @param[in]    sessionKey    署名のキーです。

     @see ClearP2PDataPacketSessionSignatureKey(), GetP2PDataPacketSessionSignatureKeyPaylod()
     @see GetP2PDataPacketSessionSignatureKeyLength()
     */
    void SetP2PDataPacketSessionSignatureKey(const qVector<qByte>& sessionKey);


    /*!
     @brief P2P 通信で使用するセッションごとの署名計算に用いる鍵データをクリアします。


     LANSessionDiscovery が有効になった場合
     や、 VirtualSocketDriver が設定されており Session::CreateSession() 、 Session::JoinSession() の実行時
     に、内部で実行されます。

     
     Session::CreateSession()、 Session::JoinSession()実行後は、NetZが破棄されるまで実行しないでください。
     

     @see SetP2PDataPacketSessionSignatureKey()
     */
    void ClearP2PDataPacketSessionSignatureKey();


    /*!
     * @brief P2P 通信で使用するセッションごとの署名計算に用いる鍵データを取得します。
     *
     * @return 署名キーの先頭ポインタ。設定されていない場合は、NULLが返ります。
     *
     * @see SetP2PDataPacketSessionSignatureKey()
     * @see GetP2PDataPacketSessionSignatureKeyLength()
     */
    const void* GetP2PDataPacketSessionSignatureKeyPaylod();
    /*!
     * @brief P2P 通信で使用するセッションごとの署名計算に用いる鍵データのサイズを取得します。
     *
     * @return 署名キーのサイズ。設定されていない場合は、0が返ります。
     *
     * @see SetP2PDataPacketSessionSignatureKey()
     * @see GetP2PDataPacketSessionSignatureKeyPaylod()
     */
    size_t GetP2PDataPacketSessionSignatureKeyLength()
    {
        return m_P2PSessionSignatureKey.GetContentSize();
    }

    //n:2690
    /*!
     @internal

     @brief P2Pで使用する署名アルゴリズムをセットします

     DisableP2PDataPakcetSignature() で署名機能を無効化している場合はセットできません。

     NEX の Core オブジェクトが生成される前に、本関数を実行してください。
     
     @param[in]    pAlgorithm    署名アルゴリズム
     */
    static void SetP2PChecksumAlgorithm(KeyedChecksumAlgorithm* pAlgorithm);

    /*!
     @internal

     @brief P2Pで使用するデータの署名機能を有効化します。

     P2Pで使用するデータの署名機能を有効化します。SetP2PChecksumAlgorithm() で署名アルゴリズムがセットしていない場合、
     デフォルトの署名アルゴリズムがセットされます。

     NEX の Core オブジェクトが生成される前に、本関数を実行してください。

     */
    static void EnableP2PDataPacketSignature();

    /*!
     @internal

     @brief P2Pで使用するデータの署名機能を無効化します。

     P2Pで使用するデータの署名機能を無効化します。再度有効化する場合でアプリケーション独自の署名アルゴリズムを使用する場合は、
     @ref EnableP2PDataPacketSignature() 実行後、 @ref SetP2PChecksumAlgorithm() で署名アルゴリズムを再設定する必要があります。

     NEX の Core オブジェクトが生成される前に、本関数を実行してください。

     */
    static void DisableP2PDataPacketSignature();

    /*! @internal
     @brief Ack受信済みパケットのCallContextのCallIDリストの参照を取得します。(DirectStream用)
     @return CallContextのCallIDリストの参照が返ります。
     */
    qVector<qUnsignedInt32>& GetAckedID()
    {
        return m_ackedID;
    }

private:
    void InitializeLocalURLs();
    void SetRC4Encryption();
    void ResetEncryption();

    //n1589
public:
    void SetRC4Encryption(Stream::Type stream);
    void ResetEncryption(Stream::Type stream);
    void SetCompression(Stream::Type stream);
    void ResetCompression(Stream::Type stream);

    void SetDefaultChecksumKeyToSessionKey();

private:
    void InitChecksumAlgorithm();
    void InitP2PChecksumAlgorithmKey();

    // 以下、変数宣言
    // Core::AcquireInstance()の呼び出しを最初に行う必要があるため、変数はScopedAcquireInstance以下に宣言する必要がある。
    // (m_oConnectivityManagerがCoreのSchedulerを参照する。)
    Core::ScopedAcquireInstance m_coreScopedAcquireInstance;

    //n1589
#ifdef NEX_CLIENT_OPTIMIZATION
    TransportAdapter* m_pVirtualTransportAdapter;
#endif

    CriticalSection     m_csNetworkLock;
    PseudoSingleton     m_psInstance;
    qProtectedList<StationURL>  m_lstStationURL;

    //n1589:todo1:rename to m_pTransport
    RootTransport       *m_pUDPTransport;

#ifdef NEX_CLIENT_PBPOOL
    // パケットバッファマネージャの実体(PseudoSingleton)
    // この下で宣言されているConnectivityManagerからPacketBufferManagerが使われる。
    // ConnectivityManager::~ConnectivityManager()からPacketBufferManager::Finalize()される可能性があるため、
    // その後にPacketBufferManagerが解放される必要がある。そのためConnectivityManagerの上で宣言する必要がある。
    PacketBufferManager     m_oPacketBufferManager;
#endif  // NEX_CLIENT_PBPOOL

    ConnectivityManager m_oConnectivityManager;

    NATTraversalEngine      *m_pNATTraversalEngine;

    ProtocolMessageHolder   m_oProtocolMessageHolder;

    RC4Encryption           m_oRC4Encryption;

    static TransportAdapter *s_pTransportAdapter;
    static qBool            s_bEncryptionEnabled;

    // n2690 データパケットの署名対応
    static qBool            s_EnableRendezVousChecksumAlgorithm;
    KeyedChecksumAlgorithm* m_pRendezVousChecksumAlgorithm;
    static qBool            s_EnableP2PChecksumAlgorithm;
    KeyedChecksumAlgorithm* m_pP2PChecksumAlgorithm;

    //n1769  データパケットの署名対応
public:
    static const size_t   MAX_P2P_SIGNATURE_KEY_LEN = 32; //!< インターネット通信用データパケットの署名に用いる鍵データ長の最大値です。
    static const size_t   MIN_P2P_SIGNATURE_KEY_LEN = 20; //!< インターネット通信用データパケットの署名に用いる鍵データ長の最小値です。
    static const size_t   MAX_P2P_DEFAULT_SIGNATURE_KEY_LEN = 32; //!< ローカル通信用データパケットの署名に用いる鍵データ長の最大値です。
    static const size_t   MIN_P2P_DEFAULT_SIGNATURE_KEY_LEN = 8; //!< ローカル通信用データパケットの署名に用いる鍵データ長の最小値です。
private:
    static qByte          s_P2PSignatureKeyData[];
    static size_t         s_P2PSignatureKeyLen;

    static qByte s_byDefaultChecksumKey[];
    LiteBuffer<MAX_P2P_SIGNATURE_KEY_LEN> m_P2PSessionSignatureKey;

    // n2208
    static qInt s_ZLibWindowBits;
    static qInt s_ZLibMemLevel;

    // n2619
    static P2PState s_eP2PState;

    // Ack受信したReliableパケットに関連づいたCallContextのCallID
    qVector<qUnsignedInt32> m_ackedID;

};

END_NAMESPACE_NEX

