﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <OnlineCore/src/Transport/Interface/StationContactInfo.h>
#include <OnlineCore/src/Platform/Core/CoreConstants.h>
#include <OnlineCore/src/Platform/Core/UserContext.h>

BEGIN_NAMESPACE_NEX

//group=Miscellaneous Classes

/*
Description: Contains the probe results for a given Station.
Remarks: A StationProbe pings the Station it pertains to in order to determine the ping time (round trip time) and preferred
StationURL of the station.

$ifdef RENDEZVOUS$
See Also: ConnectivityManager::ProbeStations
$endif$
*/

/*!
    @brief   端末のアドレス情報 (StationURL) 及び NAT トラバーサル処理で計測された PingTime (RTT) 値を保持するクラスです。

    通常、アプリケーションが直接生成することはありません。
    @ref StationProbeList のコンストラクタ または @ref StationProbeList::AddStationURL() で生成されます。
 */
class StationProbe: public RootObject
{
    //Group=Methods
public:
    /*!
        @brief コンストラクタです
     */
    StationProbe();
    /*
    Parameters:
    refStationContactInfo: The StationContactInfo object that will be used to construct this StationProbe.
    See Also: StationContactInfo
    */

    /*!
        @internal
        @brief コンストラクタです
     */
    StationProbe(const StationContactInfo &refStationContactInfo);
    /*!
        @brief デストラクタです。
     */
    virtual ~StationProbe();

    /*!
        @brief NAT トラバーサルを行って得られた接続情報です。

        VSocket::ProbeStations() の完了後に利用可能になります。
        得られた StationURL オブジェクトから StationURL::GetInetAddress() 及び InetAddress::GetAddress() と
        InetAddress::GetPortNumber() で IP アドレスとポート番号を取得してください。

        @return NAT トラバーサルを行って得られた接続情報
     */
    StationURL          &GetPreferredURL()
    {
        return m_oPreferredURL;
    }

    /*!
        @brief NAT　トラバーサルを行って得られた接続情報です。

        VSocket::ProbeStations() の完了後に利用可能になります。
        得られた StationURL オブジェクトから StationURL::GetInetAddress() 及び InetAddress::GetAddress() と
        InetAddress::GetPortNumber() で IP アドレスとポート番号を取得してください。

        @return NAT トラバーサルを行って得られた接続情報
     */
    const StationURL    &GetPreferredURL() const
    {
        return m_oPreferredURL;
    }

    /*
    Description: Returns the StationContactInfo of the Station that pertains to this StationProbe.
    Returns: The StationContactInfo of the Station that pertains to this StationProbe.
    See Also: StationContactInfo
    */
    StationContactInfo  &GetContactInfo()
    {
        return m_oStationContactInfo;
    }

    //{secret}
    const StationContactInfo  &GetContactInfo() const
    {
        return m_oStationContactInfo;
    }

    /*
    Description: Returns the lowest ping time to the Station that pertains to this StationProbe.
    Returns: The lowest ping time (round trip time) in milliseconds to the Station that pertains to this StationProbe.
    An infinite ping time indicates that the Station cannot be reached or that NAT traversal could not be performed,
    which may be due to the presence of a firewall or strict NAT device.

    $ifdef RENDEZVOUS$
    Remarks: Refer to ConnectivityManager::ProbeStations for an example of this method in use.
    $endif$
    */

    /*!
        @brief NAT トラバーサルで計測した PingTime (RTT) 値を取得します。

        VSocket::ProbeStations() の完了後に利用可能になります。
        本関数で NAT トラバーサルに成功したかどうかも判断出来ます。
        NAT トラバーサルが失敗した場合、 Infinite が返ります。

        @retval Infinite NAT トラバーサルに失敗した。
        @retval Infinite以外 この端末との PingTime (RTT) 値(ミリ秒)。
     */
    qUnsignedInt32  GetPingTime() const
    {
        return m_tiPingTime;
    }

    /*!
        @brief この端末に紐づけられているプリンシパル ID を取得します。

        @return この端末に紐づけられているプリンシパル ID。
     */
    PrincipalID GetPrincipalId() const
    {
        return m_oStationContactInfo.GetPrincipalID();
    }

    /*
    Description:  Returns the UserContext associated with this StationProbe.
    Remarks: A UserContext can be associated with a StationProbe by means of SetUserContext
    and can be retrieved through GetUserContext.
    Returns: A pointer to the UserContext that is associated with this StationProbe.
    See Also: SetUserContext
    */

    /*!
        @brief アプリケーション定義データを取得します。
        @return アプリケーションがセットした UserContext。
     */
    const UserContext& GetUserContext() const
    {
        return m_oUserContext;
    }

    /*
    Description:   Sets the UserContext associated with this StationProbe.
    Remarks:  A UserContext can be associated with a StationProbe through SetUserContext and retrieved using GetUserContext.
    Parameters:
    refUserContext: A pointer to the UserContext that is to be associated with this StationProbe.

    See Also: GetUserContext
    */

    /*!
        @brief アプリケーション定義データをセットします。
        @param[in] refUserContext アプリケーションが任意の値を指定した UserContext。
     */
    void SetUserContext(const UserContext &refUserContext)
    {
        m_oUserContext = refUserContext;
    }

    //{secret}
    void            Update();

    /*
    Description: Traces the content of this StationProbe object.
    Remarks: The data traced is the preferred URL and ping time of the Station that pertains to this StationProbe.
    */
    void            Trace(qUnsignedInt64 uiTraceFlag = TRACE_NAT);

    //{secret}
    void            SetPingTime(qUnsignedInt32 tiPingTime)
    {
        m_tiPingTime = tiPingTime;
    }
    //{secret}
    void            SetPreferredURL(const StationURL &oURL)
    {
        m_oPreferredURL = oURL;
    }
    //{secret}
    void            Reset()
    {
        SetPreferredURL(NEX_T(""));
        SetPingTime(Infinite);
    }
    qBool           IsFailed() const
    {
        return m_bFailed;
    }

public:
    //{secret}
    StationContactInfo  m_oStationContactInfo;
    //{secret}
    StationURL          m_oPreferredURL;
    //{secret}
    qUnsignedInt32      m_tiPingTime;
protected:
    //{secret}
    UserContext         m_oUserContext;
    qBool               m_bFailed;
};

END_NAMESPACE_NEX

