﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>

BEGIN_NAMESPACE_NEX
/*!
    @brief ストリームごとのパケットバンドリング設定をカプセル化したクラスです。

    Stream::GetSettings().GetBundling()のメソッド呼び出しによって、
    そのストリームに関連付けられた、
    本クラスのオブジェクトへの参照が得られます。このオブジェクトを利用して、
    ストリーム設定にアクセスすることができます。

    規定値は、有効で、タイムアウト時間は0で、Dispatchごとにバンドリングされて送信されます。

  Stream::GetSettings()で取得したストリームに対しては、DOやDirectStreamなどすべてのパケットのバンドリング設定となります。

    @see StreamSettings
 */
class StreamBundling: public RootObject
{
public:
    StreamBundling();
    virtual     ~StreamBundling();

    /*!
     * @brief パケットのバンドリングが有効か否かを返します。
     *
     * デフォルトは有効です。
     *
     * @return 有効フラグ
     */
    qBool IsEnabled() const
    {
        return m_bEnabled;
    }

    /*!
    @brief パケットのバンドリングを無効にします。
     */
    void Disable()
    {
        m_bEnabled = false;
    }

    /*!
     * @brief パケットのバンドリングを有効にします。
     *
     * 指定された時間まで、パケットをバッファリングしてからバンドリングします。
     * 規定値は、有効かつタイムアウト時間は、0です。
     *
     * @param[in] tiAutoFlushPeriod タイムアウト時間 [msec]
     * 規定値は0で、その時点でキューに溜まっているパケットをバンドリングして、バンドリングのためにバッファリングを行いません。
     */
    void Enable(TimeInterval tiAutoFlushPeriod = Time(0));
    void Flush();
    Time GetLastFlushTime();

    //n1589
    /*!
     * @brief パケットのバンドリングタイムアウト時間を指定します。
     *
     * 指定された時間パケットをバッファリングしてからバンドリングします。
     *
     * @param[in] ti タイムアウト時間 [msec]
     *
     * 0は、その時点でキューに溜まっているパケットをバンドリングして、バンドリングのためにバッファリングを行いません。
     */
    void SetMaxFlushDelay(TimeInterval ti);

    /*!
     * @brief Unreliableパケットをバンドリングする最大数を取得します。
     *
     * @return 最大バンドリングパケット数
     */
    size_t GetMaxUnreliableBundlingPackets()
    {
        return m_maxUnreliableBundlingPackets;
    }

    /*!
     * @brief Unreliableパケットをバンドリングする最大数を設定します。
     *
     * @param[in] packets 最大バンドリングパケット数
     * 0は、バンドリングせずに送信します。
     *　初期値は、std::numeric_limits<size_t>::max() です。
     */
    void SetMaxUnreliableBundlingPackets(size_t packets)
    {
        m_maxUnreliableBundlingPackets = packets;
    }

    /*!
     * @brief Reliableパケットをバンドリングする最大数を取得します。
     *
     * @return 最大バンドリングパケット数
     */
    size_t GetMaxReliableBundlingPackets()
    {
        return m_maxReliableBundlingPackets;
    }

    /*!
     * @brief Reliableパケットをバンドリングする最大数を設定します。
     *
     * ACKもReliableパケットの扱いとなります。
     *
     * @param[in] packets 最大バンドリングパケット数
     * 0は、バンドリングせずに送信します。
     *　初期値は、std::numeric_limits<size_t>::max() です。
     */
    void SetMaxReliableBundlingPackets(size_t packets)
    {
        m_maxReliableBundlingPackets = packets;
    }

    //n2619
    TimeInterval GetMaxFlushDelay()
    {
        return m_tiMaxFlushDelay;
    }

private:
    void UpdateFlushTime();

private:
    qBool                   m_bEnabled;
    TimeInterval            m_tiMaxFlushDelay;
    Time                    m_tLastFlushTime;
    size_t                  m_maxUnreliableBundlingPackets;
    size_t                  m_maxReliableBundlingPackets;

    //n1589
    mutable CriticalSection m_csFlushTime;
};

END_NAMESPACE_NEX

