﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Platform/Core/PerfCounter.h>

#ifdef NEX_PERFCOUNTER_ENABLED

BEGIN_NAMESPACE_NEX

//Group= Performance Counters

/*
Description: Lists the transport protocol performance counters.
*/
enum TransportPerfCounterType
{
    /*
    Description: Records the number of bits per second
    sent via the transport protocol to peer nodes.
    Remarks: This counter includes the overhead added by the UDP header,
    which is equal to 28 bytes.
    */
    PERF_TRANSPORT_BPS_SENT = 0,

    /*
    Description: Records the number of bits per second
    received by the transport protocol from peer nodes.
    Remarks: This counter includes the overhead added by the UDP header,
    which is equal to 28 bytes.
    */
    PERF_TRANSPORT_BPS_RECV,

    /*
    Description: Records the number of packets sent per
    second via the transport protocol to peer nodes.
    */
    PERF_TRANSPORT_PACKETS_SENT,

    /*
    Description: Records the number of packets received
    per second by the transport protocol from peer nodes.
    */
    PERF_TRANSPORT_PACKETS_RECV,

    /*
    Description: Records the number of packets to peer nodes
    reported lost per second.
    */
    PERF_TRANSPORT_PACKETS_LOST,

    /*
    Description: Records the number of unreliable packets transmitted per
    second by the transport protocol to peer nodes.
    */
    PERF_TRANSPORT_UNRELIABLE_TRANSMIT,

    /*
    Description: Records the number of reliable packets transmitted per
    second by the transport protocol to peer nodes.
    */
    PERF_TRANSPORT_RELIABLE_TRANSMIT,

    /*
    Description: Records the number of reliable packets retransmitted per
    second by the transport protocol to peer nodes.
    */
    PERF_TRANSPORT_RELIABLE_RETRANSMIT,

    /*
    Description: Records the number of routed packets transmitted per
    second by the transport protocol to peer nodes.
    */
    PERF_TRANSPORT_ROUTED_PACKETS,

    /*
    Description: Records the number of routed packets in bits per second
    transmitted by the transport protocol to peer nodes.
    */
    PERF_TRANSPORT_ROUTED_BPS,

    /*
    Description: Records the number of packets forwarded by the transport
    protocol to peer nodes.
    */
    PERF_TRANSPORT_FORWARED_PACKETS,

    /*
    Description: Records the number of packets in bits per second forwarded
    by the transport protocol to peer nodes.
    */
    PERF_TRANSPORT_FORWARDED_BPS,

    /*
    Description: Records the counts of fulling syncache entries.
     */
    PERF_TRANSPORT_SYNCACHE_ENTRY_FULL_COUNTS,

    /*
    Description: Records the numbers of  unmatch packets in syn cache entries.
     */
    PERF_TRANSPORT_SYNCACHE_UNMATCH_PACKETS,

    // {Secret}
    PERF_TRANSPORT_HIGHEST_ID = PERF_TRANSPORT_SYNCACHE_UNMATCH_PACKETS
};

/*
Description: This class encapsulates the performance counters associated
with the transport protocol. Access the object using RootTransport::GetPerfCounters.
Remarks: This class contains the performance counters associated with the
transport protocol in use. Access the object using RootTransport::GetPerfCounters. For example,

        TransportPerfCounters *pCounters = RootTransport::GetInstance()->GetPerfCounters();

It should be noted that some transport protocols may not support all
the tansport performance counters available.
See also: DataSetPerfCounters, RootTransport::GetPerfCounters
*/
class TransportPerfCounters: public RootObject
{

    //group=Methods
public:
    //{secret}
    class TransportPerfCounter: public PerfCounter
    {
    public:
        TransportPerfCounter(): PerfCounter(TRANSPORT_UNKNOWN_PERF_C_STR) {}
    };


    /*
    Remarks: A TransportPerfCounters object should never be directly instantiated. Instead, it is created by
    $PRODUCTNAME$ before a session is created. To access the TransportPerfCounters object from within your
    game, use the RootTransport::GetPerfCounters method.
    See Also: RootTransport::GetPerfCounters
    */
    TransportPerfCounters();

    /*
    Remarks: A TransportPerfCounters object should never be directly destroyed. It is destroyed by $PRODUCTNAME$
    when the session terminates.
    */
    virtual ~TransportPerfCounters();

    //Summary: Traces the values of the transport performance counters to the TraceLog.
    void Trace();

    //Summary: Resets all the transport performance counters to zero.
    void Reset();

    /*
    Summary: Returns a performance counter value.
    Parameters:
    eCounterID: Queried performance counters. Can be one of the following: PERF_TRANSPORT_BPS_SENT,
    PERF_TRANSPORT_BPS_RECV, PERF_TRANSPORT_PACKETS_SENT, PERF_TRANSPORT_PACKETS_LOST, and PERF_TRANSPORT_PACKETS_RECV.
    eValue: Requested value. For each counter, the following values can be queried: PERF_FREQUENCY_LAST, PERF_FREQUENCY_AVERAGE,
    PERF_FREQUENCY_MIN, PERF_FREQUENCY_MAX and PERF_COUNT_TOTAL.
    Returns: The requested performance value for the specified performance counter if the call succeeds. Or 0 and
    SYSTEMERROR_GEN_OUT_OF_RANGE if an error occurred.
    See Also: RootTransport::GetPerfCounters for an example of how to use this method.
    */
    qUnsignedInt32 GetValue(
        TransportPerfCounterType eCounterID,
        PerfCounterValueType eValue);

    //{secret}
    void Inc(TransportPerfCounterType eCounterID, qInt iValue);

    // {Secret}
    PerfCounter* GetCounter(TransportPerfCounterType eCounterID);

private:
    TransportPerfCounter m_aPerfCounters[PERF_TRANSPORT_HIGHEST_ID+1];
};

END_NAMESPACE_NEX

#endif // NEX_PERFCOUNTER_ENABLED

