﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <cstring>

#include <OnlineCore/src/Transport/Interface/NetFaults.h>
#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/RootTransport.h>
#include <OnlineCore/src/Transport/Interface/VirtualPort.h>

#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Platform/Core/qChain.h>
#include <OnlineCore/src/Core/LogicalClock.h>
#include <OnlineCore/src/Transport/UDP/PacketHistoryChainDefine.h>
#include <OnlineCore/src/Core/Checksum/Interface/SignatureBytes.h>
#include <OnlineCore/src/Transport/UDP/PacketDefines.h>
#include <OnlineCore/src/Core/LiteByteStream.h>

BEGIN_NAMESPACE_NEX


class Buffer;


CHAINPOLICY_DEFINE(Temp); //ChainPolicyTempを定義


class Packet: public RefCountedObject
{
    friend class PRUDPMessageV0; //n2967 Pack, Unpackの移動に伴うコード変更を少なくするためにfrind宣言

public:
    static const qUnsignedInt32 HeaderSizeForSignatureCalc=28;
    static const qUnsignedInt32 OptionSizeForSignatureCalc=52;
    static const qUnsignedInt16 MaxHeaderOptionSize = 52; //NEX_STATIC_ASSERTで各バージョンごと下回っていないかチェックしている
    static const qUnsignedInt16 MaxOverHead = HeaderSizeForSignatureCalc + MaxHeaderOptionSize; // 最大オーバーヘッド
    static const qUnsignedInt16 MaxHeaderSynConnectionOptionSize = 80; //ConnectやSyn用のオプションサイズ
    static const qByte DefaultVersion=0x0;
    CHAINABLE(Packet);
    CHAINABLE_POLICY(Packet, Temp);
    CHAINABLE_POLICY(Packet, HistoryPacket); //ChainPolicyHistoryPacketは、PacketHistoryChainDefine.hで定義

    Packet();

    virtual     ~Packet();

    Buffer      *GetPayload()
    {
        return m_pPayload;
    }
    const Buffer      *GetPayload() const
    {
        return m_pPayload;
    }
    //qUnsignedInt32    GetHeader(void **pBuffer);

    qUnsignedInt16      GetSize();

    qUnsignedInt16      GetPayloadSize()
    {
        if (m_pPayload == NULL)
        {
            return 0;
        }
        else
        {
            return static_cast<qUnsignedInt16>(m_pPayload->GetContentSize());
        }
    }

    const SignatureBytes&  GetSignature() const
    {
        return m_oSignature;
    }
    SignatureBytes&        GetSignatureRef()
    {
        return m_oSignature;
    }
    void                        SetSignature(const qByte sign[], qUnsignedInt size)
    {
        m_oSignature.Set(sign, size);
    }
    void                        SetSignature(const SignatureBytes& obj)
    {
        m_oSignature = obj;
    }

    const SignatureBytes&  GetConnectionSignature() const
    {
        return m_oConnectionSignature;
    }
    SignatureBytes&        GetConnectionSignatureRef()
    {
        return m_oConnectionSignature;
    }
    void                        SetConnectionSignature(const qByte sign[], qUnsignedInt size)
    {
        m_oConnectionSignature.Set(sign, size);
    }
    void                        SetConnectionSignature(const SignatureBytes& obj)
    {
        m_oConnectionSignature = obj;
    }

    //n1589:memo:this is not joining session id but endpoint session id
    qByte               GetSessionID() const
    {
        return m_bySessionID;
    }
    void                SetSessionID(qByte bySessionID)
    {
        m_bySessionID = bySessionID;
    }

    PacketType      GetType() const
    {
        return static_cast<PacketType>(m_uiPacketTypeFlags & PACKET_TYPE_MASK);
    }
    
    void        SetType(PacketType byPacketType)
    {
        m_uiPacketTypeFlags |= (byPacketType & PACKET_TYPE_MASK);
    }
    

    void        SetFlag(qUnsignedInt16 uiFlag)
    {
        m_uiPacketTypeFlags |= (uiFlag & PACKET_FLAG_MASK);
    }

    //n1589:memo:not used in NetZ
    void        ClearFlag(qUnsignedInt16 uiFlag)
    {
        m_uiPacketTypeFlags &= ~(uiFlag & PACKET_FLAG_MASK);
    }
    qBool       FlagSet(qUnsignedInt16 uiFlag) const
    {
        return (m_uiPacketTypeFlags & uiFlag)!=0;
    }

    //n1589
    qUnsignedInt16 GetFlags() const
    {
        return (m_uiPacketTypeFlags & PACKET_FLAG_MASK) >> PACKET_TYPE_MASK_BIT;
    }

    //n1589
    qUnsignedInt16 GetTypeFlags() const
    {
        return m_uiPacketTypeFlags;
    }

    void SetTypeFlags(qUnsignedInt16 typeFlag)
    {
        m_uiPacketTypeFlags = typeFlag;
    }


    void            SetSeqId(LogicalClock16 lcSeqId)
    {
        m_lcSeqId = lcSeqId;
    }
    LogicalClock16  GetSeqId() const
    {
        return m_lcSeqId;
    }

    void        SetDestinationVirtualPort(const VirtualPort& oVPort)
    {
        m_oDestinationVPort = oVPort;
    }
    const VirtualPort& GetDestinationVirtualPort() const
    {
        return m_oDestinationVPort;
    }

    void        SetSourceVirtualPort(const VirtualPort& oVPort)
    {
        m_oSourceVPort = oVPort;
    }
    const VirtualPort& GetSourceVirtualPort() const
    {
        return m_oSourceVPort;
    }

    qByte       GetPartNumber() const
    {
        return m_byPartNumber;
    }
    void        SetPartNumber(qByte byPartNumber)
    {
        m_byPartNumber = byPartNumber;
    }

    void        SetPeerAddress(const InetAddress *pPeerAddress)
    {
        m_oPeerAddress = *pPeerAddress;
    }
    //-----------------
    //n1589:warning:#815-D:type qualifier on return type is meaningles
    //const InetAddress* const GetPeerAddress() const { return &m_oPeerAddress; }
    //InetAddress * const GetPeerAddress() { return &m_oPeerAddress; }
    //-----------------
    const InetAddress* GetPeerAddress() const
    {
        return &m_oPeerAddress;
    }
    InetAddress * GetPeerAddress()
    {
        return &m_oPeerAddress;
    }

    void        SetIOTime(Time oIOTime)
    {
        m_oIOTime = oIOTime;
    }
    Time        GetIOTime() const
    {
        return m_oIOTime;
    }
    static      qUnsignedInt16 GetOverhead();

    qUnsignedInt16  GetLocalPort()
    {
        return m_ui16LocalPort;
    }
    void            SetLocalPort(qUnsignedInt16 ui16LocalPort)
    {
        m_ui16LocalPort = ui16LocalPort;
    }
    
    
    void            SetQueuedTime(Time tTime = Time::GetTime())
    {
        m_tQueuedTime = tTime;
    }
    Time            GetQueuedTime()
    {
        return m_tQueuedTime;
    }


    void            SetPacketHistoryQueuedTime(Time tTime = Time::GetTime())
    {
        m_tPacketHistoryQueuedTime = tTime;
    }
    Time            GetPacketHistoryQueuedTime() const
    {
        return m_tPacketHistoryQueuedTime;
    }

    void            SetTransportProtocolType(TransportProtocol::Type transportProtocolType)
    {
        m_transportProtocolType = transportProtocolType;
    }
    TransportProtocol::Type GetTransportProtocolType() const
    {
        return m_transportProtocolType;
    }
    void            SetSubStreamID(qByte subStreamID)
    {
        m_byChunkId = subStreamID;
    }
    qByte           GetSubStreamID() const
    {
        return m_byChunkId;
    }

    void            SetMaxUserReliableSubStreams(qByte subStreamID)
    {
        m_byMaxUserReliableSubStreams = subStreamID;
    }
    qByte           GetMaxUserReliableSubStreams() const
    {
        return m_byMaxUserReliableSubStreams;
    }

    void            SetInitialUnreliableSeqId(LogicalClock16 lcSeqId)
    {
        m_lcInitialUnreliableSeqId=lcSeqId;
    }
    LogicalClock16  GetInitialUnreliableSeqId() const
    {
        return m_lcInitialUnreliableSeqId;
    }
    
    void SetDisconnectReason(DisconnectReason reason)
    {
    	m_disconnectReason = reason;
    }
    
    DisconnectReason GetDisconnectReason() const
    {
    	return m_disconnectReason;
    }

    //protected:
    qBool           SetPayload(Buffer *pPayload); //要ACQUIREREF or 解放義務と所有権の放棄

#ifdef NEX_WATERMARK_ENABLED
    static WaterMark    m_wmNbPackets;
#endif

    void   SetTTL(qUnsignedInt8 uiTTL)
    {
        m_uiTTL = uiTTL;
    }
    qUnsignedInt8   GetTTL()
    {
        return m_uiTTL;
    }

public:
    static qBool IsDataPacketSinatureEnabled(void)
    {
        return true;
    }

    //n2690:
    void SetIsOldRVSecure(qBool flag)
    {
        m_IsOldRVSecure = flag;
    }
    qBool IsOldRVSecure() const
    {
        return m_IsOldRVSecure;
    }

#ifdef NEX_TRACE_ENABLED
    static void SetTracePacketSizeMin(qUnsignedInt16 o)
    {
        s_uiTracePacketSizeMin = o;
    }

    static void SetTracePacketSizeMax(qUnsignedInt16 o)
    {
        s_uiTracePacketSizeMax = o;
    }

    virtual void Trace(qUnsignedInt64 uiTraceFlags) = 0;
    qBool TraceSimple(qUnsignedInt64 uiTraceFlags, qBool bIncoming);
    qBool TraceVerbose(qUnsignedInt64 uiTraceFlags, qBool bIncoming);
#endif  // NEX_TRACE_ENABLED

    inline qBool    IsReliable() const
    {
        return FlagSet(FLAG_RELIABLE) || FlagSet(FLAG_ACK);
    }

    inline qBool    CanHistoryBundle() const
    {
        // 履歴バンドリングで他のパケットとまとめることができるか？
        // 
        // このパケット自体が履歴バンドリングするよう指定されていなくても、
        // 他に履歴バンドリングが必要なパケットがあるか判断し、あればバンドリング処理を
        // 行うべきであることを示す。
        // 
        // NetZ で履歴バンドリングの対象になっているのが一部のパケットだけであっても、
        // それらのパケットは履歴バンドリングの対象にならない NetZ パケット送信時に
        // まとめて送信することができる。
        // 
        // * データパケットじゃなかったら (制御パケットだったら) 対象外
        // * NetZ のパケットじゃなかったら対象外
        // * Unreliable に対してバンドリング有効の指定がされておらず、かつ Reliable じゃなかったら (=Unreliable だったら) 対象外
        // * ACK パケット (Reliable のみで発生) だったら対象外
        // * Reliable に対してバンドリング有効の指定がされておらず、かつ Reliable だったら対象外
        return  GetType() == TYPE_DATA
                && GetSourceVirtualPort().GetStreamType() == Stream::DO
                && (RootTransport::IsEnableUnreliablePacketHistoryBundling() || IsReliable() )
                && ((RootTransport::IsEnableReliablePacketHistoryBundling() && !FlagSet(FLAG_ACK)) || !IsReliable() );
    }

    inline qBool    ShouldDuplicateIfHistoryBundle() const
    {
        // 履歴バンドリングによって重複処理を行うべきパケットか？
        // 
        // CanHistoryBundle(履歴バンドリングで他のパケットとまとめることができるか)
        // に加え、以下の条件を付加する。
        // * DO に対してバンドリング有効の指定がされておらず、かつ DirectStrem じゃなかったら(=DOだったら) 対象外
        return  CanHistoryBundle()
                && (RootTransport::IsEnableDOPacketHistoryBundling() || FlagSet(FLAG_PRUDP_USER));
    }

    inline qByte GetVersion() const
    {
        return m_version;
    }
    inline void SetVersion(qByte version)
    {
        m_version = version;
    }
    
    template<typename T> T& GetHeaderForSignatureCalc()
    {
        return *reinterpret_cast<T*>(m_byHeaderForSignatureCalc);
    }
    qByte* GetHeaderForSignatureCalc()
    {
        return (qByte *)m_byHeaderForSignatureCalc;
    }
    template<typename T> const T& GetHeaderForSignatureCalc() const
    {
        return *reinterpret_cast<T*>(m_byHeaderForSignatureCalc);
    }
    const qByte* GetHeaderForSignatureCalc() const
    {
        return (const qByte *)m_byHeaderForSignatureCalc;
    }

    inline void SetSupportedFunctions(qUnsignedInt32 functions)
    {
        m_uiSupportedFunctions = functions;
    }
    inline qUnsignedInt32 GetSupportedFunctions() const
    {
        return m_uiSupportedFunctions;
    }

    inline void SetHasSerializedHeader(qBool flag)
    {
        m_HasSerializedHeader = flag;
    }
    inline qBool HasSerializedHeader() const
    {
        return m_HasSerializedHeader;
    }

    inline LiteByteStream<OptionSizeForSignatureCalc>& GetOptionForSignatureCalc()
    {
        return m_optionForSignatureCalc;
    }
    inline const LiteByteStream<OptionSizeForSignatureCalc>& GetOptionForSignatureCalc() const
    {
        return m_optionForSignatureCalc;
    }
    
    template<typename T> T& GetOptionForSignatureCalc()
    {
        return *reinterpret_cast<T*>(m_optionForSignatureCalc.GetBuffer()->GetContentPtr());
    }
    
    template<typename T> const T& GetOptionForSignatureCalc() const
    {
        return *reinterpret_cast<const T*>(m_optionForSignatureCalc.GetBuffer()->GetContentPtr());
    }


protected:

    void InitHeader(qByte byPacketType);        // n2370: not used?  there's no implementation...

    qByte m_version;
    // Packet Header for all packets
    VirtualPort         m_oSourceVPort;
    VirtualPort         m_oDestinationVPort;
    SignatureBytes      m_oSignature;

    qUnsignedInt16      m_uiPacketTypeFlags;
    LogicalClock16      m_lcSeqId;
    // This is used for CONNECT/SYN packets
    LogicalClock16      m_lcInitialUnreliableSeqId;


    SignatureBytes      m_oConnectionSignature;
    // This is used for DATA packets

    qByte               m_byPartNumber;
    qByte               m_bySessionID;
    qUnsignedInt8       m_uiTTL;
    qByte               m_byMaxUserReliableSubStreams;
    qByte               m_byChunkId;
    DisconnectReason    m_disconnectReason;
    qUnsignedInt16      m_ui16LocalPort;

    qUnsignedInt32      m_byHeaderForSignatureCalc[HeaderSizeForSignatureCalc/sizeof(qUnsignedInt32)];
    //n2690: FPDとの互換性維持のため旧RVSecureであるかを判定するためのフラグ
    qBool m_IsOldRVSecure;
    qBool m_HasSerializedHeader; //Headerがすでに解釈済であるか

    qUnsignedInt32 m_uiSupportedFunctions;
    LiteByteStream<OptionSizeForSignatureCalc> m_optionForSignatureCalc;


    // Transient info
    Buffer          *m_pPayload;
    InetAddress     m_oPeerAddress;
    Time            m_oIOTime;
    Time            m_tQueuedTime;
    Time            m_tPacketHistoryQueuedTime;
    TransportProtocol::Type m_transportProtocolType;    // SocketTransport::Send()からSocketTransport::DeliverOutgoingImpl()までに間に
                                                        // TransportProtocol::Typeを覚えるために用意。

#ifdef NEX_TRACE_ENABLED
    static qUnsignedInt16 s_uiTracePacketSizeMin;
    static qUnsignedInt16 s_uiTracePacketSizeMax;

    void TraceImpl(qUnsignedInt64 uiTraceFlags, qBool bIncoming);
#endif  // NEX_TRACE_ENABLED
};

END_NAMESPACE_NEX

