﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/UDP/Packet.h>
#include <OnlineCore/src/Platform/Core/qChain.h>
#include <OnlineCore/src/Platform/Core/WaterMark.h>

BEGIN_NAMESPACE_NEX

class PacketQueue: public RootObject
{
public:
    typedef qChain<Packet*>::iterator iterator;

    PacketQueue(const qChar *szQueueName);
    virtual ~PacketQueue();

    virtual void Purge();

    qBool empty() const
    {
        return m_Chain.empty();
    }

    virtual void Queue(Packet *pPacket, qBool isSetTime = true);
    virtual void QueueFront(Packet *pPacket, qBool isSetTime = true);
    virtual iterator Dequeue(iterator it);

    const iterator begin() const
    {
        return m_Chain.begin();
    }
    iterator begin()
    {
        return m_Chain.begin();
    }
    const iterator end() const
    {
        return m_Chain.end();
    }
    iterator end()
    {
        return m_Chain.end();
    }
    const iterator last() const
    {
        return m_Chain.last();
    }
    iterator last()
    {
        return m_Chain.last();
    }
    virtual CriticalSection* GetLock()
    {
        return NULL;
    }

    //n1589
    size_t GetPacketCount()
    {
        return m_Chain.size();
    }

#ifdef NEX_TRACE_ENABLED    // n2370
    void Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS);
#endif  // NEX_TRACE_ENABLED

#ifdef NEX_WATERMARK_ENABLED
    qUnsignedInt64    GetSize() const
    {
        return m_wmSize.GetCurrent();
    }
#endif

private:
    qChain<Packet*>             m_Chain;
#ifdef NEX_WATERMARK_ENABLED
    WaterMark                   m_wmSize;
#endif
};

//n1589:memo:used only in SocketTransport
class ProtectedPacketQueue : public PacketQueue
{
public:
    ProtectedPacketQueue(
        const qChar *szQueueName,
        CriticalSection &cs
    ):PacketQueue(szQueueName), m_csQueue(cs)
    {
    }

    virtual ~ProtectedPacketQueue()
    {
        Purge();
    }
    virtual void Purge()
    {
        ScopedCS oCS(m_csQueue);
        PacketQueue::Purge();
    }
    virtual void Queue(Packet *pPacket, qBool isSetTime = true)
    {
        ScopedCS oCS(m_csQueue);
        PacketQueue::Queue(pPacket, isSetTime);
    }
    virtual void QueueFront(Packet *pPacket, qBool isSetTime = true)
    {
        ScopedCS oCS(m_csQueue);
        PacketQueue::QueueFront(pPacket, isSetTime);
    }
    virtual iterator Dequeue(iterator it)
    {
        ScopedCS oCS(m_csQueue);
        return PacketQueue::Dequeue(it);
    }
#ifdef NEX_TRACE_ENABLED    // n2370
    void Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS)
    {
        ScopedCS oCS(m_csQueue);
        PacketQueue::Trace(uiTraceFlags);
    }
#endif  // NEX_TRACE_ENABLED
    virtual CriticalSection* GetLock()
    {
        return &m_csQueue;
    }

private:

    //n1589:warning:#2700-D: declaring a reference with "mutable" is nonstandard
    CriticalSection &m_csQueue;

    // Cannot copy
    ProtectedPacketQueue(const ProtectedPacketQueue &);
    ProtectedPacketQueue &operator=(const ProtectedPacketQueue &);
};
END_NAMESPACE_NEX

