﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Stack/Core/TransportProtocol.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <OnlineCore/src/Core/Scheduler.h>
#include <OnlineCore/src/Core/Core.h>
#include <OnlineCore/src/Core/Holder.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <RendezVous/Core/src/Common/ProtocolCallContext.h>
#include <RendezVous/Core/src/Common/DataHolder.h>
#include <RendezVous/Core/src/Common/MembershipInfo.h>
#include <RendezVous/Core/src/Common/NotificationEventHandler.h>
#include <RendezVous/Core/src/Common/NintendoProtocolFoundation/NintendoNotificationEventHandler.h>
#include <RendezVous/Core/src/Client/SecureConnection/SecureConnectionClient.h>

/** \internal */
BEGIN_NAMESPACE_NEX

class RemoteLogDeviceServer;
class SecureStream;
class Credentials;
class AuthenticationClient;
class SecureConnectionClient;
class StreamManager;
class NotificationEventManager;
class NintendoNotificationEventManager;
class RendezVousOperation;
class ProtocolRegistry;


/*!
 @brief    ゲームサーバーとの接続を管理するクラスです。本クラスを継承した @ref NgsFacade クラスを使用してください。
 */
class OSDK_API BackEndServices: public RootObject
{
public:

    /** \internal */
    virtual ~BackEndServices();

    /*
    \brief Returns whether or not a given Principal is connected to both the
    lobby's Authentication Service and Secure Connection Service.
    \returns true if the given Principal is connected to both the lobby's Authentication Service and
    Secure Connection Service, /i.e./ the Principal is logged in to the lobby, false if not or an error
    occurred.
    \remarks When a Principal logs in to the lobby using the platform specific back-end services Login()
    method, it automatically connects to both the Authentication Service and the Secure Connection
    Service in order to create a secure transport layer that guarantees the integrity, confidentiality,
    and authenticity of data sent across the network. The protocol used is PRUDPS, Quazal's proprietary,
    reliable, secure UDP transport protocol. Once a Principal logs out from the lobby this method will return false.

    By default each BackEndServices client sends a heart beat signal to the Rendez-Vous servers every 10 seconds.
    If the servers do not receive a signal for a period of one minute, then it is assumed that the client is no longer
    connected. If you suspect that a fault has occured on the client, you can use HealthClient::PingDaemon. If the
    client is connected, the call will succeed rapidly. To test for a fault, you can issue the call and wait on the
    CallContext for 5 seconds. If it successfull within that time, you know that no fault has occured. If it has failed
    or it still pending, you can assume that a fault has occured.
    \sa Login, Logout, HealthClient::PingDaemon
    */

    /*!
     @brief    ゲームサーバーへの接続状態を取得します。

     ゲームサーバーへの接続状態を取得します。ゲームサーバーへのログイン後に本関数が QSUCCESS(Core, Success) 以外を返した場合は明示的に @ref NgsFacade::Logout() を呼ぶ必要があります。
     本関数は毎フレーム呼び出してもパフォーマンスには影響がありません。
     ログイン処理が成功した時点から本関数は QSUCCESS(Core, Success) を返すようになります。

     ネットワーク接続が切れた場合や、一定時間サーバーとの通信ができない場合にオフラインとなります。

     ゲームサーバーと接続されていない場合には、 QERROR(RendezVous, ConnectionDisconnected) が返ります。

     ユーザーに対してゲームサーバーとの通信が切断された旨を通知したい場合、
     この値を ErrorCodeConverter::ConvertToNetworkErrorCode() でネットワークエラーコードに変換して、エラー・EULAアプレットに渡すことで、
     「通信エラーが発生しました」というメッセージが表示されます。
     
     
    
    


    @retval     "QSUCCESS(Core, Success)"    ゲームサーバーに接続中（オンライン）
    @retval     "QERROR(RendezVous, ConnectionDisconnected)"   オフライン*/
    qResult IsConnected() const;

    /*
    \brief Disconnects a Principal from both the lobby's Authentication Service and
    Secure Connection Service.

    \param pContext A pointer to the CallContext for this call.
    \returns true if the Principal was successfully disconnected from both the lobby's Authentication Service
    and Secure Connection Service, false if not or an error occurred.
    \remarks This method should be called after Logout to correctly disconnect the Principal from both the
    Authentication Service and the Secure Connection Service. Refer to Logout for an example of this
    method in use.

    \note This method is now obsolete. To log out from the lobby you simply need to call Logout.
    \sa IsConnected, Logout
    */
    qBool Disconnect(CallContext *pContext);

    /** \internal */
    qBool Login_(CallContext *pContext, const qChar* szUserName, const qChar8* szPassword, const qChar* szAddress,
                 qUnsignedInt16 uiBasePort, Credentials** ppCredentials, String *pstrReturnMsg=NULL)
    {
        return LoginImpl(pContext, NULL, szUserName, szPassword, szAddress, uiBasePort, NULL, ppCredentials,
                         pstrReturnMsg, NULL);
    }

    // IPアドレス/ポートではなくhttpから始まるURIで接続先を指定するログイン
    qBool Login_(CallContext *pContext, const qChar* szUserName, const qChar8* szPassword,
                 const qChar* connectUri, Credentials** ppCredentials, String *pstrReturnMsg=NULL)
    {
        return LoginImpl(pContext, NULL, szUserName, szPassword, NULL, 0, connectUri, ppCredentials,
                         pstrReturnMsg, NULL);
    }

    /** \internal */
    qBool Login_(CallContext *pContext, const qChar* szUserName, const qChar8* szPassword,
                 const qChar* szAddress, qUnsignedInt16 uiBasePort, const DataHolder &oAnyData,
                 Credentials** ppCredentials, String *pstrReturnMsg)
    {
        return LoginImpl(pContext, NULL, szUserName, szPassword, szAddress, uiBasePort, NULL, ppCredentials,
                         pstrReturnMsg, (DataHolder *)&oAnyData);
    }

    // IPアドレス/ポートではなくhttpから始まるURIで接続先を指定するログイン
    qBool Login_(CallContext *pContext, const qChar* szUserName, const qChar8* szPassword,
                 const qChar* connectUri, const DataHolder &oAnyData,
                 Credentials** ppCredentials, String *pstrReturnMsg)
    {
        return LoginImpl(pContext, NULL, szUserName, szPassword, NULL, 0, connectUri, ppCredentials,
                         pstrReturnMsg, (DataHolder *)&oAnyData);
    }

    /** \internal */
    qBool Logout_(Credentials *pCredentials);

    /** \internal */
    qBool Logout_(CallContext *pContext, Credentials *pCredentials);

    /** \internal */
    qBool LogoutImpl(CallContext *pContext, Credentials *pCredentials);

    /*
    \brief Terminates and cleans up outstanding Login/Logout calls, ensuring that the object is in an appropriate
    state for deletion.

    \param pContext A pointer to the CallContext for this call.
    \remarks To ensure that you can delete the appropriate back end services object even if there are incomplete login or
    logout calls. To prevent problems, you should always call this method before deleting the object.
    An example of the Terminate() method is:
    \code
            ProtocolCallContext oContext;
            pBackEndServices->Terminate(&oContext);
            oContext.Wait();
            delete pBackEndServices;
    \endcode
    */

    /*!
    @brief    @ref NgsFacade を終了し、deleteできる状態にします。

    本関数はログイン処理中・ログアウト処理中・オンライン・オフラインのいずれの状態であっても呼び出し可能で、
    非同期処理が完了するとオフライン状態となることが保証されます。

    ログイン処理中・ログアウト処理中・ログイン中のいずれかの状態のとき、 @ref NgsFacade のデストラクタ内で NgsFacade::Terminate が呼ばれ、
    オフライン状態に遷移するまでブロックします。NgsFacade を破棄する前に NgsFacade::Terminate を呼ぶことを推奨します。

    本関数を呼ぶ前に @ref ServiceClient 継承クラス（ @ref MatchmakeExtensionClient , @ref NATTraversalClient など）の
    Unbind() および delete を実行してください。

    ネットワークに繋がっていない場合、非同期処理に時間がかかることがあるため、
    @ref Core::SetTerminateImmediately() で強制終了モードを設定しても構いません。

    @param[in,out]    pContext    呼び出し情報。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool  Terminate(CallContext* pContext);

    /** \internal */

    /*!
     @brief    @ref Credentials (クライアントの証明書情報)を取得します。

     @ref Credentials (クライアントの証明書情報)を取得します。有効な @ref Credentials を取得するには、
     NGS にログインしている必要があります。

     @return    Credentials (クライアントの証明書情報)。ログインしていない場合は NULL が返ります。
     */
    Credentials* GetCredentials()
    {
        return m_pCurrentCredentials;
    }

#ifndef NEX_SIGLO_FAMILY    // NX では本メソッドでログイン時間を短縮する必要はありません。
    /*!
     @brief    ログインに必要な情報の一部をキャッシュし、2回目以降のログイン時間を短縮できるようにします

     ログインに必要な情報をキャッシュすることで、本関数の呼び出し後、2回目以降のログインでは
     ログインに要する時間を   2.0 秒 程度短縮します。

     NEX の初期化を行った後（@ref Initialize() などを呼び出した後）の初回ログイン前に呼び出してください。

     NEX ライブラリを終了する際には @ref DisableLoginDataCache() を呼び出してください
     */
#endif // NEX_SIGLO_FAMILY
    static void EnableLoginDataCache();

#ifndef NEX_SIGLO_FAMILY
    /*!
     @brief    ログインに必要な情報のキャッシュをクリアします

     EnableLoginDataCache() を呼び出している場合、NEX の終了処理を行う前（@ref Finalize() などを呼び出す前）に本関数を呼び出してください。
     */
#endif // NEX_SIGLO_FAMILY
    static void DisableLoginDataCache();

    /*!
     @brief    ログインに必要な情報のキャッシュを有効化しているかを返します

     @return    ログインに必要な情報のキャッシュを有効化しているかどうか
     @retval    true     キャッシュが有効化されている
     @retval    false    キャッシュが有効化されていない
     */
    static qBool IsAbleLoginDataCache();

protected:
    /** \internal */
    typedef  void (*PreTerminateCallback)(CallContext*, const UserContext*);

    /** \internal
    Alternative design would be to have Terminate method as
    virtual and drive classes would invoke
    BackendServices::Terminate. The drawback would be that since Terminate is an Asyn call, all
    derived classes need to keep the user CallContext,
    RegisterCompletionCallback, .... .
    */
    void RegisterPreTerminateCallback(PreTerminateCallback pfPreTerminateCallback, const UserContext& oContext);

private:

    struct OSDK_API InternalData
    {
        PreTerminateCallback m_Callback;
        UserContext        m_oUserContext;
        qBool              m_bPreTerminateCallbackRegistered;
        InternalData():m_bPreTerminateCallbackRegistered(false) {};
        ~InternalData()
        {
            m_bPreTerminateCallbackRegistered = false;
        }

        void Set(const PreTerminateCallback &cb, const UserContext &c);
    };
    InternalData  m_PreTerminateCB;
    static void  TerminateCB(CallContext* pContext, const UserContext* pUserContext);
    CallContext m_oContext;
    CallContext* m_pUserCallContext;

    qBool  TerminateImpl();

    /** \internal
    Now called from inside PostLogoutCleanup().
    */
    void ReleaseCredentials();

public:
    /** \internal
    Function is virtual to let subclasses add more cleanup
    without having to create new background jobs to wrap the logout job.
    */
    virtual void PostLogoutCleanup();

    /** \internal */
    StreamManager *GetStreamManagerForSecure() const;

    /** \internal */
    SecureStream *GetSecureStream() const;

    /*
    \brief Returns a pointer to the AuthenticationClient of the Principal who has logged in.
    \returns A pointer to the AuthenticationClient of the Principal who has logged in.
    \remarks This call is only valid once a Principal has logged in to the lobby via the BackEndServices
    Login method.
    \sa Authentication Service, Login
    */
    AuthenticationClient* GetAuthenticationClient() const;

    // TODO: API to select no incoming connections, unsecure
    // incoming connections or secure incoming connections.

    /*
    \brief Returns a pointer to the SecureConnectionClient of the Principal who has logged in.
    \returns A pointer to the SecureConnectionClient of the Principal who has logged in.
    \remarks This call is only valid once a Principal has logged in to the lobby via the BackEndServices
    Login method.
    \sa Secure Connection Service, Login
    */
    SecureConnectionClient* GetSecureConnectionClient() const;

    /*!
    @internal
    @brief    ローカルポートを共有する設定にした場合に、次回RVへ接続する際に使用するローカルポートを取得します。

    BackEndServices::SetShouldShareLocalPort()でローカルポートを再利用する設定をした場合
    もしくはコネクション指向でないプロトコル使用時に、
    このポート番号を使用してRVに接続します。もしこのポートがバインドできなかったら代わりに任意のポートが使用され、
    本関数で取得されるポート番号がそのポート番号に変更されます。
    コネクション指向プロトコル使用時かつローカルポートを再利用しない設定の場合はポート0による自動割り当てが行われます。

    @return    次回RVへ接続する際に使用するローカルポート。
    */
    qUnsignedInt16 GetNextSharedLocalPort() const;

    /*!
    @internal
    @brief    ローカルポートを共有する設定にした場合に、次回RVへ接続する際に使用するローカルポートを設定します。

    @param[in] uiPort 次回RVへ接続する際に使用するローカルポート。
    */
    void SetNextSharedLocalPort(qUnsignedInt16 uiPort);

    /*!
    @internal
    @brief    ローカルポートを共有すべきかどうかを取得します。

    @return    ローカルポートを共有すべきかどうか。
    */
    static qBool GetShouldShareLocalPort()
    {
        return s_shouldShareLocalPort;
    }

    /*!
    @internal
    @brief    ローカルポートを共有すべきかどうかを設定します。

    RVにログインする際にローカルポートを共有して使用するソケットの数を削減するかどうかを設定します。
    コネクション指向でないプロトコルを使用している場合は本設定にかかわらず共有します。
    コネクション指向プロトコルを使用時に共有する場合は接続するRVのアドレスが全て同じである必要があります。

    @param[in] shouldShareLocalPort ローカルポートを共有すべきかどうか。
    */
    static void SetShouldShareLocalPort(qBool shouldShareLocalPort)
    {
        s_shouldShareLocalPort = shouldShareLocalPort;
    }

    /*!
    @internal
    @brief    ローカルポートを共有すべきかどうかをスコープで設定します。
    */
    class ScopedSetShouldShareLocalPort : public RootObject
    {
    public:
        explicit ScopedSetShouldShareLocalPort(qBool shouldShareLocalPort)
        {
            m_originalShouldShareLocalPort = GetShouldShareLocalPort();
            SetShouldShareLocalPort(shouldShareLocalPort);
        }
        ~ScopedSetShouldShareLocalPort()
        {
            SetShouldShareLocalPort(m_originalShouldShareLocalPort);
        }
    private:
        qBool m_originalShouldShareLocalPort;
    };

    /*
    \brief The Process will accept (log) remote service traces.
    \remarks In order to remotely debug a service you can log the service traces via a remote Process,
    such as a client Process. In order to do so, in your /UserConfiguration.py/ script you must first configure which
    traces will be outputed by the back-end services via the TraceLog.SetFlag method of the quazal module. If in your
    /UserConfiguration.py/ script log are enabled, then when AcceptRemoteTraces is called on a remote process, such as a client,
    all traces and service events that were configured to be logged and that result from the remote Process's actions, will
    be logged on that remote Process. In order to stop a remote Process logging service traces, call IgnoreRemoteTraces.

    This method should be called immediately after the BackEndServices object is created.
    The LogDevice where the traces will be logged to must be specified on the remote Process. Refer to the
    Data Logging section for further details.
    \sa IgnoreRemoteTraces, Data Logging, SandboxConfiguration
    */
    void AcceptRemoteTraces();

    /*
    \brief The Process will ignore all traces sent by a remote service.
    \remarks If a back-end service is configured to remotely log traces and the remote Process calls
    IgnoreRemoteTraces, any traces that the Process receives will be disregarded. In order for a remote
    Process to log service traces AcceptRemoteTraces must be called.
    \sa AcceptRemoteTraces
    */
    void IgnoreRemoteTraces();

#ifdef NEX_LOG_ENABLED
    /** \internal */
    RemoteLogDeviceServer* GetRemoteLogDeviceServer()
    {
        return m_pRemoteLogDeviceServer;
    }
#endif

    /*
    \brief Registers a specific NotificationEventHandler to the back-end services.

    \param pNotificationEventHandler A pointer to the NotificationEventHandler that will be registered to the
    back-end services.

    \returns true if the specified NotificationEventHandler was successfully registered to the back-end
    services, false if not or an error occurred.

    \remarks This method should be called after Login is called and before the required
    ServiceClient objects are created.  Refer to the Integration page for further details as to the
    correct order of calls. A NotificationEventHandler determines what should happen when a Process
    receives a given NotificationEvent. If no handler is registered then the Process will not
    receive any NotificationEvents. Once a NotificationEventHandler is registered, the Process
    associated to this BackEndServices object will receive all built-in NotificationEvents as well
    as custom NotificationEvents (as long as custom events are not implemented otherwise) until
    UnregisterNotificationEventHandler is called. Several NotificationEventHandler s may be
    registered to the back-end services at any given time.

    \sa UnregisterNotificationEventHandler, NotificationEventHandler, Integration
    */

    /*!
     @brief    通知イベントハンドラーを登録します

     通知イベントハンドラーを登録します。本関数を実行することで、通知イベントを受信するたびに
     NotificationEventHandler::ProcessNotificationEvent コールバックが実行されるようになります。
     すべての通知イベントを受信するためには、本関数を NgsFacade::Login() 呼び出しよりも前に実行してください。
     NgsFacade::Login() 実行後に本関数を実行すると、本関数の実行までの間に送られた通知イベントを受信できないことがあります。
     イベントハンドラーは複数個登録できます。

     @param[in]    pNotificationEventHandler    登録する通知イベントハンドラーへのポインタ。
     @retval    true    成功。
     @retval    false   同じイベントハンドラーを複数個登録しようとした。
     */
    qBool RegisterNotificationEventHandler(NotificationEventHandler* pNotificationEventHandler);
    qBool RegisterNintendoNotificationEventHandler(NintendoNotificationEventHandler* pNintendoNotificationEventHandler);

    /*
    \brief Unregisters a specific NotificationEventHandler from the back-end services.

    \param pNotificationEventHandler A pointer to the NotificationEventHandler that will be unregistered
                            from the back-end services.

    \returns true if the specified NotificationEventHandler was successfully unregistered from the
          back-end services, false if not or an error occurred.

    \remarks This method should be called after Logout is called. Refer to the Integration page for
    further details as to the correct order of calls. When there is no NotificationEventHandler
    registered to the BackEndServices, a Process will not receive any NotificationEvents.

    \sa RegisterNotificationEventHandler, NotificationEventHandler, Integration
    */

    /*!
     @brief    登録した通知イベントハンドラーを削除します。

     @param[in]    pNotificationEventHandler    登録を取り消す通知イベントハンドラーへのポインタ。
     @retval    true    成功。
     @retval    false   指定したイベントハンドラーが見つからなかった。
    */
    qBool UnregisterNotificationEventHandler(NotificationEventHandler* pNotificationEventHandler);
    qBool UnregisterNintendoNotificationEventHandler(NintendoNotificationEventHandler* pNintendoNotificationEventHandler);

    /** \internal */
    StreamManager* CreateStreamManager() const;
    /** \internal */
    void InitStreamManagerFactory(StreamManager* (*pfStreamManagerFactory)());
    void InitStreamManagerInitializationFlagSpecifier(qUnsignedInt32 (*pfGetStreamManagerInitializationFlagSpecifier)(qBool));
    void InitStreamManagerInitializationLocalPortSpecifier(qUnsignedInt16 (*pfGetStreamManagerInitializationLocalPortSpecifier)(qBool));

    /*
    \brief Sets the access key of the sandbox to which this BackEndServices client wishes to connect.

    \param strSandboxAccessKey The access key of the sandbox to which this BackEndServices client wishes to connect.
    \remarks Quazal assigns a unique access key to each sandbox on the Rendez-Vous servers. In order
    to connect to a given sandbox, the correct access key for that sandbox must be passed via this method
    before Login is called.

    Refer to the BackEndServices class overview for an example of this method in use.
    */
    static void SetSandboxAccessKey(const String& strSandboxAccessKey);

    /*
    \brief Sets the timeout for the Login call.

    \param tiTimeout The maximum time in milliseconds that the system will wait. The default timeout is 30 000 milliseconds (30 s).
    \remarks This method sets how long a client will wait for the Login call to succeed. It must be called before the
    call to Login is made.
    \sa Login
    */
    void SetLoginTimeout(const TimeInterval& tiTimeout)
    {
        m_tiLoginTimeout = tiTimeout;
    }

    /*
    \brief Enable or Disable advertising of Public Url returned at login to Network.

    \param[in] enable The flag to enable or disable advertising of url to Netkwork
    \remarks This method is only used by the JobLogin. It must be called before the call to Login is made.
    \sa Login
    */
    void SetAdvertisePublicUrlFlag(qBool enable)
    {
        m_advertisePublicURL = enable;
    }
    qBool GetAdvertisePublicUrlFlag() const
    {
        return m_advertisePublicURL;
    }

    /** \internal */
    void SetPublicUrl(const StationURL& publicUrl)
    {
        m_urlPublic = publicUrl;
    }

    /*
    \brief Returns the Public Url returned at login from RendezVous.
    \remarks Public Url is only valid after a successfull login is made.
    \sa Login
    */
    StationURL GetPublicUrl() const
    {
        return m_urlPublic;
    }

    /*
    \brief Returns a pointer to an OperationManager.
    \returns A pointer to an OperationManager.

    \remarks This method returns a pointer to the OperationManager
    that manages all RendezVousOperation s. The main purpose of
    the OperationManager is to manage the callbacks called at the
    beginning and end of the various RendezVous System Operations.
    Before using it, be sure to have at least one BackEndServices
    instance created.

    \sa OperationManager
    */
    //static OperationManager* GetOperationManager();

    /** \internal */
    qBool LoginJobIsInProgress() const;

    /** \internal */
    qBool LogoutJobIsInProgress() const;

    /** \internal */
    qBool TerminateJobIsInProgress() const;

    /*
    \brief Returns a user-friendly error string given an error code returned by qResult,

    \param strErrorCause    The user-friendly error string. Not all codes are translated yet
    \param uiReturnCode     Provides the value returned by qResult::GetReturnCode.
    \remarks Returns a user-friendly error string when given a return code from qResult
    */
    static String FormatQErrorCodeString(const String& strErrorCause, qUnsignedInt32 uiReturnCode);

    /** \internal */
    static void SetProtocolVersion(qUnsignedInt32 uiProtocolVersion);

    /** \internal */
    static void SetCustomData(const DataHolder &hCustomData)
    {
        m_hCustomData = hCustomData;
    }
    /** \internal */
    static const DataHolder& GetCustomData()
    {
        return m_hCustomData;
    }

    /*
    \brief This method compares the parameter that is passed against the current password.
    \param szPassword: Accepts a string that is compared against the current password. Returns true if
    the strings match, otherwise it returns false.
    */
    qBool ValidatePassword(const qChar8* szPassword);

    /** \internal */
    TimeInterval    GetLoginTimeout() const
    {
        return m_tiLoginTimeout;
    }

    /** \internal */
    void SetEffectiveLoginTimeout(TimeInterval tiTimeout);

    /** \internal */
    qBool EffectiveTimeoutIsSet() const;

    /*
    \brief This method enables traces and specifies the trace flags for
    NotificationsEvents that are received from the server.

    \param[in] uiTraceFlags The trace flags to use for the NotificationsEvents traces.
    \note By default the trace flag is TRACE_NEVER.
    \sa NotificationEventHandler
    */
    void SetNotificationEventTraceFlags(qUnsignedInt64 uiTraceFlags);

    /*
    \brief This method returns the NotificationsEvents trace flags.
    \returns The NotificationsEvents trace flags
    \sa NotificationEventHandler
    */
    //n1589:qUnsignedInt64
    qUnsignedInt64 GetNotificationEventTraceFlags();
    qUnsignedInt64 GetNintendoNotificationEventTraceFlags();

    /*
    \brief This method returns the global ProtocolRegistry which can be used to register client protocols.
                 The Registry is used to parse Reponses on a specific client protocol.
    \returns A pointer to the global ProtocolRegistry
    \sa ProtocolRegistry
    */
    ProtocolRegistry* GetProtocolRegistry() const;

    /** \internal
    These will move to the ServerCallManager once we refactor the Server side of protocols.
    */
    virtual void RegisterServerProtocols();
    /** \internal */
    virtual void UnregisterServerProtocols();
    /** \internal */
    void UpdateRemoteLogDeviceServer();

    void SetIsCombinedAuthAndServiceNode(qBool isCombinedAuthAndServiceNode)
    {
        m_isCombinedAuthAndServiceNode = isCombinedAuthAndServiceNode;
    }

    // Authとサービスノードを共用する設定かどうかを取得する。ログイン後に利用可能。
    qBool GetIsCombinedAuthAndServiceNode() const
    {
        return m_isCombinedAuthAndServiceNode;
    }

    /*!
     @internal
     @brief    メンバシップ情報をセットします。

     @param[in] info メンバシップ情報
     */
    void SetMembershipInfo(const MembershipInfo& info)
    {
        m_membershipInfo = info;
    }

    /*!
     @internal
     @brief    メンバシップ情報を取得します。ログイン後に利用可能です。

     @return メンバシップ情報
     */
    const MembershipInfo& GetMembershipInfo() const
    {
        return m_membershipInfo;
    }

    /*!
        @internal
        @brief         使用する標準APIバージョンを設定します。ログイン前に設定する必要があります。

        本関数で設定しない状態ではNEX_NGS_API_VERSION_GENERALが使用されます。

        @param[in]  apiVersionGeneral 使用する標準APIバージョン。
    */
    void SetApiVersionGeneral(qUnsignedInt32 apiVersionGeneral)
    {
        m_apiVersionGeneral = apiVersionGeneral;
    }

    /*!
        @internal
        @brief         使用する標準APIバージョンを取得します。

        @return        使用する標準APIバージョン。
    */
    qUnsignedInt32 GetApiVersionGeneral() const
    {
        return m_apiVersionGeneral;
    }

    /*!
        @internal
        @brief         使用するカスタムAPIバージョンを設定します。ログイン前に設定する必要があります。

        本関数で設定しない状態ではNEX_NGS_API_VERSION_CUSTOMが使用されます。

        @param[in]  apiVersionCustom 使用するカスタムAPIバージョン。
    */
    void SetApiVersionCustom(qUnsignedInt32 apiVersionCustom)
    {
        m_apiVersionCustom = apiVersionCustom;
    }

    /*!
        @internal
        @brief         使用するカスタムAPIバージョンを取得します。

        @return        使用するカスタムAPIバージョン。
    */
    qUnsignedInt32 GetApiVersionCustom() const
    {
        return m_apiVersionCustom;
    }

    /*!
        @internal
        @brief         APIバージョンの互換性チェックを無視してログインを行うかどうかを設定します。ログイン前に設定する必要があります。

        @param[in]  ignoreApiVersionCheck trueならAPIバージョンの互換性チェックを無視してログインを行う。
                                          falseならAPIバージョンの互換性チェックします。初期状態ではチェックします。
    */
    void SetIgnoreApiVersionCheck(qBool ignoreApiVersionCheck)
    {
        m_ignoreApiVersionCheck = ignoreApiVersionCheck;
    }

    /*!
        @internal
        @brief         APIバージョンの互換性チェックを無視してログインを行うかどうかを取得します。

        @return        APIバージョンの互換性チェックを無視してログインを行うかどうか。
    */
    qBool GetIgnoreApiVersionCheck() const
    {
        return m_ignoreApiVersionCheck;
    }

private:
    /** \internal */
    qBool LoginImpl(CallContext *pContext, qResult *pqResult, const qChar* szUserName, const qChar8* szPassword,
                    const qChar* szAddress, qUnsignedInt16 uiBasePort, const qChar* connectUri, Credentials** ppCredentials,
                    String *pstrReturnMsg, DataHolder *pAnyData);
private:
    /** \internal
    Invalid operations.
    */
    BackEndServices(const BackEndServices&);
    BackEndServices& operator=(const BackEndServices&);

protected:

    /** \internal */
    BackEndServices();

    /** \internal
    Must be called by the children of the class.
    */
    void OnLoginStart(CallContext& context);

    /** \internal
    Must be called by the children of the class.
    */
    void OnLogoutStart(CallContext& context);

    /** \internal */
    Job* GetJobLogin() const
    {
        return m_pJobLogin;
    }

private:

    void RegisterOperationOnCallContext(CallContext& context, RendezVousOperation& op);

    static String GetResultFacilityCore(const qUnsignedInt32 uiErrorCode);
    static String GetResultFacilityTransport(const qUnsignedInt32 uiErrorCode);
    static String GetResultFacilityRendezVous(const qUnsignedInt32 uiErrorCode);
    static String GetResultFacilityPythonCore(const qUnsignedInt32 uiErrorCode);
    // static String GetResultFacilityNews(const qUnsignedInt32 uiErrorCode);
    // static String GetResultFacilityGameSession(const qUnsignedInt32 uiErrorCode);
    //  static String GetResultFacilityZZZAuthentication(const qUnsignedInt32 uiErrorCode);
    //  static String GetResultFacilityPrivilege(const qUnsignedInt32 uiErrorCode);
    //   static String GetResultFacilityTracking(const qUnsignedInt32 uiErrorCode);
    static String GetResultFacilityFriends(const qUnsignedInt32 uiErrorCode);
    //  static String GetResultFacilityTitleStorage(const qUnsignedInt32 uiErrorCode);
    // static String GetResultFacilitySocialNetworks(const qUnsignedInt32 uiErrorCode);
    //  static String GetResultFacilitySubAccountManagement(const qUnsignedInt32 uiErrorCode);
    //static String GetResultFacilityZZZFriends(const qUnsignedInt32 uiErrorCode);

    friend class JobBackEndServicesTerminate;   //JobBackEndServicesTerminate calls CancelLoginJob
    void CancelLoginJob();

    TimeInterval    m_tiLoginTimeout;
    Credentials*    m_pCurrentCredentials;

#ifdef NEX_LOG_ENABLED
    RemoteLogDeviceServer*      m_pRemoteLogDeviceServer;
#endif
    NotificationEventManager*   m_pNotificationEventManager;
    NintendoNotificationEventManager*   m_pNintendoNotificationEventManager;

    qBool           m_bAcceptRemoteTraces;

    StreamManager* (*m_pfStreamManagerFactory)();
    qUnsignedInt32 (*m_pfGetStreamManagerInitializationFlagSpecifier)(qBool);
    qUnsignedInt16 (*m_pfGetStreamManagerInitializationLocalPortSpecifier)(qBool);

    static DataHolder m_hCustomData;

    Job*    m_pJobLogin;
    Job*    m_pJobLogout;
    Job*    m_pJobTerminate;

    //  See bug:4123
    class OSDK_API EffectiveLoginTimeout
    {
    public:
        EffectiveLoginTimeout();

        void Set(TimeInterval tiTimeout);
        TimeInterval Get() const;

        qBool IsSet() const;

    private:
        TimeInterval    m_tiTimeout;
    };
    EffectiveLoginTimeout   m_oEffectiveLoginTimeout;

    //static PseudoGlobalVariable<OperationManager> s_operationManager;
    RendezVousOperation* m_loginOperation;
    RendezVousOperation* m_logoutOperation;

    qBool m_advertisePublicURL;
    StationURL m_urlPublic;
    qBool m_isCombinedAuthAndServiceNode;

    static qBool s_IsAbleLoginDataCache;
    static qBool s_shouldShareLocalPort;
    MembershipInfo m_membershipInfo;
    qUnsignedInt32 m_apiVersionGeneral; // 使用する標準APIバージョン
    qUnsignedInt32 m_apiVersionCustom;  // 使用するカスタムAPIバージョン
    qBool   m_ignoreApiVersionCheck;    // APIバージョンのチェックを無視してログインするか。
};

/** \internal */
END_NAMESPACE_NEX

