﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Core/src/Common/HealthProtocolDDL.h>
#include <RendezVous/Core/src/Common/MonitoringProtocolDDL.h>
#include <RendezVous/Core/src/Common/Constants/ServiceID.h>

BEGIN_NAMESPACE_NEX

class StreamManager;

/**
   \ingroup group_rdv_core_services_health_client
   \brief The client class used to access the Health service that manages the general well-being of the \projectname database and back end.
*/
class OSDK_API HealthClient: public ServiceClient
{
public:

    /**
       \brief Default constructor.
    */
    HealthClient();

    /**
       \brief Destructor.
    */
    virtual ~HealthClient();

    /**
       \brief Pings the \projectname deamon.
       \param[in, out] pContext A pointer to the ProtocolCallContext for this call.
       \returns true if the call was successfully initiated, false if not or an error occurred.
       \remark If you suspect that a fault has occured on the client, you can use HealthClient::PingDaemon.
       If the client is  connected, the call will succeed rapidly. To test for a fault, you can issue the call
       and wait on the CallContext for 5 seconds. If it successfull within that time, you know that no fault has occured.
       If it has failed or it still pending, you can assume that a fault has occured.
       \note The signature that contains a qBool as a parameter is now obsolete. The result of the call is obtained via CallContext::GetOutcome,
       which will return the appropriate qResult.
       \sa BackEndServices::IsConnected
    */
    qBool PingDaemon(ProtocolCallContext *pContext);

#ifdef OBSOLETE_API_SUPPORT
    /*
       \param[in, out] pbResult A pointer to a Boolean that returns true if the call succeeded, false if not.
       \remark If this method returns true then pbResult will point to a boolean that determines whether or not
       the ping to the \projectname daemon was successful.
    */
    qBool PingDaemon(ProtocolCallContext *pContext, qBool *pbResult);
#endif

    /**
       \brief Pings the \projectname database.
       \param[in, out] pContext A pointer to the ProtocolCallContext for this call.
       \returns true if the call was successfully initiated, false if not or an error occurred.
       \note The signature that contains a qBool as a parameter is now obsolete. The result of the call is obtained via
       CallContext::GetOutcome, which will return the appropriate qResult.
    */
    qBool PingDatabase(ProtocolCallContext *pContext);

#ifdef OBSOLETE_API_SUPPORT
    /*
    \param[in, out] pbResult A pointer to a Boolean that returns true if the call succeeded, false if not.
    \remark If this method returns true then pbResult will point to a Boolean that determines whether or not the ping to the \projectname
    database was successful.
    */
    qBool PingDatabase(ProtocolCallContext *pContext, qBool *pbResult);
#endif

    /**
       \brief Verifies the integrity of the \projectname database and back end.
       \param[in, out] pContext A pointer to the ProtocolCallContext for this call.
       \returns true if the call was successfully initiated, false if not or an error occurred.
       \remarks This method performs checks on the validity of database entries such as ensuring the entry is of the valid type,
       falls within the valid range, and if it is a foreign key that it exists in the corresponding parent table.
       It also checks if certain required files are located on the correct directory path.
       \note The signature that contains a qBool as a parameter is now obsolete. The result of the call is obtained via
       CallContext::GetOutcome, which will return the appropriate qResult.
    */
    qBool RunSanityCheck(ProtocolCallContext *pContext);

#ifdef OBSOLETE_API_SUPPORT
    /*
    \param[in, out] pbResult A pointer to a Boolean that returns true if the call succeeded, false if not.
    \remark If this method  returns true then pbResult will point to a Boolean that determines whether or not the sanity checks were
    successfully performed.
    */
    qBool RunSanityCheck(ProtocolCallContext *pContext, qBool *pbResult);
#endif

    /**
       \brief Corrects any \projectname database and/or back-end errors.
       \param[in, out] pContext A pointer to the ProtocolCallContext for this call.
       \returns true if the call was successfully initiated, false if not or an error occurred.
       \remarks This method will correct any errors found by RunSanityCheck. If an error cannot be corrected then a warning will be returned.
       \note The signature that contains a qBool as a parameter is now obsolete.
       The result of the call is obtained via CallContext::GetOutcome, which will return the appropriate qResult.
    */
    qBool FixSanityErrors(ProtocolCallContext *pContext);

#ifdef OBSOLETE_API_SUPPORT
    /*
    \param[in, out] pbResult A pointer to a Boolean that returns true if the call succeeded, false if not.
    \remark If this method returns true then pbResult will point to a Boolean that determines whether or not the errors returned by RunSanityCheck were successfully corrected.
    */
    qBool FixSanityErrors(ProtocolCallContext *pContext, qBool *pbResult);
#endif

    /**
       \internal
       \brief Pings the deamon (in its C++ part).
       \note This is used internally by Siege to test the ping (in C++) performance of the deamon.
       \param[in, out] pContext A pointer to the ProtocolCallContext for this call.
       \returns true if the call was successfully initiated, false if not or an error occurred.
    */
    qBool PingCppDaemon(ProtocolCallContext *pContext);

    /**
       \internal
       TO BE REMOVED.  Unused.
    */
    qBool GetClusterMembers(ProtocolCallContext *pContext, qList<String>* pMembers);

protected:
    virtual void UpdateProtocolsDefaultCredentials(Credentials* pCredentials);

private:
    //OSDK_DISABLE_EXPORT_WARNING_BEGIN
    HealthProtocolClient m_oProtocol;
    MonitoringProtocolClient m_oMonitoringProtocol;
    //OSDK_DISABLE_EXPORT_WARNING_END
};

END_NAMESPACE_NEX

