﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/NonCopyable.h>
#include <OnlineCore/src/Core/StepSequenceJob.h>
#include <OnlineCore/src/Core/Chrono.h>
#include <OnlineCore/src/Core/qBuffer.h>
#include <RendezVous/Core/src/Common/NotificationEvent.h>
#include <RendezVous/Core/src/Common/ProtocolCallContext.h>

BEGIN_NAMESPACE_NEX

class MyNotificationEventHandler;
class NotificationEventManager;

class NotificationEventFilterInterface
{
public:
    NotificationEventFilterInterface(){}
    virtual ~NotificationEventFilterInterface(){}

    /*!
        @internal

        @brief      待ち受ける Param1 をセットします

            JobWaitForNotification::Start() を呼び出す段階では待ち受ける Param1 が決定していない場合にこの関数でセットします。
            本関数呼び出し前に SystemLock を取得する必要があります。

        @param[in]  eventParam1 待ち受ける Param1。
    */
    virtual void SetEventParam1(qUnsignedInt64 eventParam1) = 0;
};

class JobWaitForNotification : public StepSequenceJob, public NonCopyable
{
public:
    /*!
        @internal

        @brief      指定した通知を受信するのを待ちます。

            引数で指定した Type, SubType, Param1 全てと一致する物を受信したときに成功します。

        @param[in]  pCC 呼び出し情報。有効な(NULLでない)Credentialsがセットされている必要があります。
        @param[in]  eventType 待ち受ける Type。
        @param[in]  eventSubType 待ち受ける SubType。
        @param[in]  timeout 指定した通知が送られてくるまで待つ時間(ミリ秒)。この時間が過ぎると QERROR(Core, Timeout) が発生します。
        @param[in]  eventParam1 待ち受ける Param1。
        @param[out] notificationEvent 待ち受け条件と一致した NotificationEvent。
                    NULL を指定した場合は取得しません。デフォルトは NULL です。
    */
    static qBool Start(ProtocolCallContext* pCC, qUnsignedInt32 eventType, qUnsignedInt32 eventSubType, TimeInterval timeout,
                      qUnsignedInt64 eventParam1, NotificationEvent* notificationEvent=NULL);

    /*!
        @internal

        @brief      指定した通知を受信するのを待ちます。

            引数で指定した Type, SubType と後でセットした Param1 全てと一致する物を受信したときに成功します。
            NotificationEventFilterInterface::SetEventParam1() で Param1、
            をセットするまでに Type, SubType と一致する通知を
            MyNotificationEventHandler::MAX_NOTIFIED_PARAM1_SIZE (16) 個以上受け取ると Assert による停止もしくは正常に動作しないという制限があります。

        @param[in]  pCC 呼び出し情報。有効な(NULLでない)Credentialsがセットされている必要があります。
        @param[in]  eventType 待ち受ける Type。
        @param[in]  eventSubType 待ち受ける SubType。
        @param[in]  timeout 指定した通知が送られてくるまで待つ時間(ミリ秒)。この時間が過ぎると QERROR(Core, Timeout) が発生します。
        @param[out] notificationEventFilterInterface あとで Param1 をセットするためのインタフェース。
                    関数呼び出しから返されたときに インタフェース へのポインタが代入されます。
                    任意のタイミングで NotificationEventFilterInterface::SetEventParam1() で Param1 をセットしてください。
        @param[out] notificationEvent 待ち受け条件と一致した NotificationEvent。
                    NULL を指定した場合は取得しません。デフォルトは NULL です。
    */
    static qBool Start(ProtocolCallContext* pCC, qUnsignedInt32 eventType, qUnsignedInt32 eventSubType, TimeInterval timeout,
                       NotificationEventFilterInterface** notificationEventFilterInterface, NotificationEvent* notificationEvent=NULL);
protected:
    JobWaitForNotification();
    virtual ~JobWaitForNotification();
    virtual qResult Initialize(
        qUnsignedInt32 uiAssociatedCallID,
        qUnsignedInt32 eventType,
        qUnsignedInt32 eventSubType,
        TimeInterval timeout,
        qUnsignedInt64 eventParam1,
        NotificationEventFilterInterface** notificationEventFilterInterface,
        NotificationEvent* notificationEvent);

    qUnsignedInt32 m_AssociatedCallID;
    MyNotificationEventHandler* m_pNotificationEventHandler;
    NotificationEventManager* m_pNotificationEventManager;
    Chrono m_NotificationWaitingTime;
    qBool m_IsNotificationEventHandlerRegistered;
    NotificationEvent* m_pNotifiedNotificationEventOut;
    NotificationEvent m_NotifiedNotificationEvent;
    TimeInterval m_Timeout;

    void StepWaitingNotify();
    void CompleteJob(const qResult& result);

    static qBool StartImpl(ProtocolCallContext* pCC, qUnsignedInt32 eventType, qUnsignedInt32 eventSubType, TimeInterval timeout, qUnsignedInt64 eventParam1,
                           NotificationEventFilterInterface** notificationEventFilterInterface, NotificationEvent* notificationEvent);
};

END_NAMESPACE_NEX

