﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>
#include <RendezVous/Core/src/Common/Constants/ServiceID.h>
#include <RendezVous/Core/src/Common/Protocol.h>
#include <OnlineCore/src/Platform/Core/CoreConstants.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <RendezVous/Core/src/Client/Credentials.h>


BEGIN_NAMESPACE_NEX

class EndPoint;
class StationURL;
class Credentials;
class CallContext;
class UserContext;
class StreamManager;
class ProtocolRegistry;

/*
\ingroup group_rdv_core_client
\brief This is the root class of all lobby services.
\remarks This is the root class of all lobby services and the class via which a client links to
a given back-end or peer service. Before being able to connect to a given service, a client
must first log in to the lobby via the RendezVous class.

As this is the root class of all lobby services, if you create your own lobby service it must
inherit from this class.
*/

/*!
 @brief    ゲームサーバーとの通信を行う Client 機能のルートクラスです。
 */
class OSDK_API ServiceClient: public RootObject
{
public:

    /**
    \param uiServiceID  The ServiceID of the service.
    \remarks Only classes that inherit from the ServiceClient object should be instantiated and not
    the ServiceClient object itself. ServiceClient objects should only be created after a client
    has logged in to the lobby (via RendezVous::Login). Refer to the Integration page for
    further details as to the correct order of calls.
    */
    ServiceClient(qUnsignedInt32 uiServiceID);

    /**
    Only classes that inherit from the ServiceClient object should be destroyed and not the
    ServiceClient object itself. ServiceClient objects should be destroyed before the client logs
    out of the lobby (via RendezVous::Logout). Refer to the Integration page for further details
    as to the correct order of calls.
    */
    virtual ~ServiceClient();

    /**
    \param pCredentials The default Credentials for this service.
    \remarks It is up to each ServiceClient to use this method to set the current Credentials in
    each of his protocols.
    */
    void SetDefaultCredentials(Credentials* pCredentials);
    Credentials* GetDefaultCredentials() const
    {
        return m_pCredentials;
    }

    ProtocolRegistry* GetProtocolRegistry() const;

    /** \internal */
    virtual void UseLocalLoopback(PrincipalID pidLocal, ConnectionID cidLocal);
    /** \internal */
    qBool IsUsingLocalLoopback() const
    {
        return m_bUseLocalLoopback;
    }

#ifdef OBSOLETE_API_SUPPORT
    /*
    \remarks As part of client side protocols refactoring, we removed a lot of uneeded functionality in ServiceClient
       for backward compatability, the methods are still implemented, but will eventually be obsoleted.
       We recommend that you keep your ServiceClient connection unaware, since ServiceClient are not required to
       bind anymore.
    */

    /*
    \brief Links a specific Principal with this ServiceClient.

    \param pCredentials A pointer to the Credentials of the Principal who will be bound to this ServiceClient.
    \returns true if the specified Principal was successfully bound to this ServiceClient, false if not
    or an error occurred.
    \remarks Once a given Principal has logged in to the lobby and the ServiceClient objects have been created,
    the Principal must then be linked to each ServiceClient that it wishes to use before that Principal may actually
    use a given ServiceClient. For example, assuming the player has successfully logged in to the lobby he then needs
    to bind to the services as follows:
    \include RendezVous\Samples\SimpleMatch\SimpleMatch.cpp
    \sa Unbind
    */

    /*!
     @brief    サービスを利用するための証明書を登録します。

     サービスを利用するための証明書をセットします。
     機能を利用する前にセットする必要があります。@ref NgsFacade::Login() の非同期処理に成功後、
     @ref NgsFacade::GetCredentials() で取得した証明書を引数に指定してください。
     @ref Bind() で証明書をセットした後もう一度 @ref Bind() を呼び出すと証明書を差し替える事ができます。

     @param[in]     pCredentials    サービスにアクセスするための証明書。

     @retval     true 成功。pCredentials が NULL でない場合は必ず成功します。
     @retval     false pCredentials に NULL が指定された。
     */
    virtual qBool Bind(Credentials* pCredentials);

    /*
    \brief Unlinks a specific Principal from this ServiceClient.
    \returns true if the specified Principal was successfully unbound from this ServiceClient, false if not
    or an error occurred.
    \remarks If this method is not called before the ServiceClient is deleted, then it will automatically be called
    in the ServiceClient destructor. If there are any CallContext s in progress when a service is unbound, then
    those CallContext s will automatically be terminated, /i.e./ the CallContext::State will be set to /CallFailure/.
    \sa Bind
    */

    /*!
    @brief     証明書の登録を解除します。

       @ref Bind() でセットされた証明書を解除します。@ref Bind() が呼び出されていなくても本関数を呼び出すことができます。
       本オブジェクトを破棄する際に自動的に @ref Unbind() と同等の処理行われるため、明示的に呼び出す必要はありません。

    @retval      true 証明書の解除を行った。
    @retval      false Bind() が行われていないため何も行わなかった。
    */
    virtual qBool Unbind();

    /** \internal */
    qBool BindLocal(Credentials* pCredentials);

    /*
    \brief Returns the Credentials of the client who is bound to this ServiceClient.
    \returns A pointer to the Credentials of the client who is bound to this ServiceClient.
    \sa Bind
    */

    /*!
     @brief    @ref Bind() された @ref Credentials オブジェクトを取得します。

     @return   @ref Bind() された @ref Credentials オブジェクト。@ref Bind() されていない場合、NULL を返します。
     */
    Credentials* GetCredentials() const
    {
        return GetDefaultCredentials();
    }

    /*
    \brief Returns the ServiceID of a client associated service for a given service.
    \returns The ServiceID of the client associated service.
    */
    qUnsignedInt32 GetServiceID() const
    {
        return m_uiServiceID;
    }

    /*
    \brief Returns whether or not a given client is connected to a given service.
    \returns true if the given client is currently connected to the given service, false if not
    or an error occurred.
    */
    virtual qBool IsConnected() const;

    virtual qBool IsFaulty() const;

    /* \internal */
    StreamManager *GetStreamManager() const;

    /* \internal */
    qBool PrepareForConnection(Credentials* pCredentials);

protected:
    /** \internal */
    EndPoint *GetConnection(ProtocolID uiProtocolID) const;

    /** \internal */
    virtual void ConnectionStateHasChanged();

#endif // OBSOLETE_API_SUPPORT
    virtual void UpdateProtocolsDefaultCredentials(Credentials* pCredentials) = 0;

    void SetCredentials(Credentials* pCredentials);

    virtual qBool SetDependentRvConnection(ProtocolCallContext* pContext) const;

private:
    qUnsignedInt32 m_uiServiceID;

    Credentials* m_pCredentials;

    qBool m_bUseLocalLoopback;
};

END_NAMESPACE_NEX


