﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/Operation.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Core/OperationManager.h>

BEGIN_NAMESPACE_NEX

class Message;

/**
    \ingroup group_rdv_core_common
    \brief TODO
*/
class OSDK_API CallProtocolMethodOperation: public Operation
{
public:

    /** \internal */
    class OperationWrapper
    {
    public:
        OperationWrapper(OperationManager& refOperationManager, CallProtocolMethodOperation* pOperation):
            m_refOperationManager(refOperationManager), m_pOperation(pOperation)
        {
            m_pOperation->InitializeForNewCallAttempt();
            m_refOperationManager.OperationBegins(m_pOperation);
        };
        ~OperationWrapper()
        {
            m_refOperationManager.OperationEnds(m_pOperation);
        }
    private:
        OperationWrapper(const OperationWrapper& o);
        OperationWrapper* operator=(const OperationWrapper& o);
    private:
        OperationManager& m_refOperationManager;
        CallProtocolMethodOperation* m_pOperation;
    };


    /** \internal */
    CallProtocolMethodOperation();
    /** \internal */
    virtual ~CallProtocolMethodOperation();

    NEX_IMPL_OPERATION_COMMON_METHODS(CallProtocolMethod);

    // Access to members.

    /**
        \brief Use this method to postpone a CallProtocolMethodOperation.

        To avoid deadlocks in embeded RMCs, the original RMC call can be postponed if it
        is not yet ready to return the result and then recalled a short time later. During call
        postponement, any queued messages are processed so that if the original RMC relies on a
        queued message, that message will be processed and the RMC call can complete.

        \param[in] tiPostponeDelay The amount of time in milliseconds that this operation will be postponed.
        \note
        When either of the DOCallContext flags TargetObjectMustBeMaster or TargetObjectMustHaveAuthority
        is set, these flags are only guaranteed to be true the initial time that the method is called.
        Therefore, if the call is posponed these flags are not rechecked and may no longer be true.
        \sa OperationIsPostponed, GetDelay, DOCallContext::_Flags
    */
    void PostponeOperation(TimeInterval tiPostponeDelay);

    /**
        \brief Returns whether or not this CallProtocolMethodOperation has been postponed.

        A CallProtocolMethodOperation can be postponed and then recalled via PostponeOperation.

        \returns true if this operation has been postponed, false if not or an error occurred.
        \sa PostponeOperation, GetDelay
    */
    qBool OperationIsPostponed() const
    {
        return m_bPostponed;
    };

    /**
        \brief Returns the amount of time for which this CallProtocolMethodOperation will be postponed.
        \returns The PostponeDelay in milliseconds for this operation.
        \sa PostponeOperation, OperationIsPostponed
    */
    TimeInterval GetDelay() const
    {
        return m_tiPostponeDelay;
    }

    /** \internal */
    void InitializeForNewCallAttempt();

    /**
    \brief Returns the number of times that this CallProtocolMethodOperation has been called.

    The attempts count starts at zero and is incremented by one each time the
    CallProtocolMethodOperation is postponed and then recalled.

    \returns The number of times that this CallProtocolMethodOperation has been called.
    \sa PostponeOperation
    */
    size_t GetAttemptCount();

    /**
        \brief Attaches a UserContext to this CallProtocolMethodOperation object.
        \param[in] oDelayContext The UserContext to be attached to this CallProtocolMethodOperation object.
        \sa GetUserContext
    */
    void AttachUserContext(const UserContext& oDelayContext);

    /**
        \brief Returns the UserContext that refers to this CallProtocolMethodOperation.

        A UserContext is attached to a CallProtocolMethodOperation object via AttachUserContext.

        \returns The UserContext that relates to this CallProtocolMethodOperation.
        \sa AttachUserContext
    */
    UserContext& GetUserContext();

    /** \internal */
    qResult GetCallResult() const
    {
        return m_rCallResult;
    }

    /** \internal */
    void SetCallError(qResult rResult)
    {
        SYSTEMCHECK(rResult==false);
        m_rCallResult = rResult;
    }

private:
#ifdef NEX_TRACE_ENABLED    // n2370
    /** \internal */
    virtual void TraceImpl(Event eEvent, qUnsignedInt64 uiTraceFlags) const;
#endif  // NEX_TRACE_ENABLED

private:

    size_t m_uiAttemptNumber;
    qBool m_bPostponed;
    UserContext m_oDelayContext;
    TimeInterval m_tiPostponeDelay;

    qResult m_rCallResult;
};

END_NAMESPACE_NEX

