﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

BEGIN_NAMESPACE_NEX

/*
\brief Namespace for enum NotificationEvents.
*/

/*!
 @brief NotificationEvents の名前空間
*/
#ifdef SWIG
class NotificationEvents
{
public:
#else
namespace NotificationEvents
{
#endif

    /*
    \ingroup group_rdv_core_common
    \brief Enumerates the types of Notification event s that may be sent to a given Principal.
    \remarks A NotificationEvent is a message sent from a back-end service to another process, which is
    typically a client process, that is triggered when a given event occurs on the back-end service.
    The purpose of these messages is to notify the remote process that a specific event has occurred
    on a given back-end service.
    You cannot modify the built-in Notification event s however, you may add an almost unlimited
    number of user-defined Notification event s via the NotificationEvent class.

    \note In order to access these types you must include the RendezVous.h file.
    \sa Principal, PrincipalID
    */

    /*!
     @brief     通知イベントの種別を定義する列挙子です。

     通知イベントの種別を定義する列挙子です。NotificationEventHandler::ProcessNotiticationEvernt() に渡される
     NotificationEvent クラスオブジェクトにセットされます。
     */
    enum NotificationEvents
    {

        /*
        A given player is in the process of adding another player to one of his/her Friends lists.
        This Notification event will be triggered when FriendsClient::AddFriend,
        FriendsClient::AcceptFriendship, or FriendsClient::DeclineFriendship is called and will have
        a different meaning according to which method triggers the event.

        AddFriend:
        This event is sent to the player with whom a given player wishes to be friends.
        The first parameter of the NotificationEvent indicates the ID of the player (PrincipalID) who
        wishes to be friends with another player ( /i.e./ the ID of the player who called FriendsClient::AddFriend),
        the second parameter indicates the new PlayerRelationships, which is Pending, and the string parameter
        indicates the associated message, if any.

        AcceptFriendship:
        This event is sent to the player who wants to be friends with another player ( /i.e./ the player who called
        FriendsClient::AddFriend).
        The first parameter of the NotificationEvent indicates the ID of the player (PrincipalID) who
        accepts to be friends with the other player ( /i.e./ the ID of the player who called FriendsClient::AcceptFriendship),
        and the second parameter indicates the new PlayerRelationships, which is Friend.

        DeclineFriendship:
        This event is sent to the player who wants to be friends with another player ( /i.e./ the player who called
        FriendsClient::AddFriend).
        The first parameter of the NotificationEvent indicates the ID of the player (PrincipalID) who
        declines to be friends with the other player ( /i.e./ the ID of the player who called FriendsClient::DeclineFriendship),
        and the second parameter indicates the new PlayerRelationships, which is None.
        */

        /*!
         @internal

         @brief    使用されません
        */
        FriendEvent = 1,

        /*
        A session has been launched.
        This event is sent to all participants of a given Gathering when the gathering session is
        launched (MatchMakingClient::LaunchSession is called).
        The first parameter of the NotificationEvent indicates the ID of the Gathering (GatheringID)
        and the string parameter indicates the URL of the launched session.
        */

        /*!
         @internal

         @brief    使用されません
        */
        SessionLaunched = 2,

        /*
        A Participation event has occurred.
        The subtype of this event is ParticipationEvents.
        This event is sent to the target Process for the given Participation event that occurred. Refer to the
        description of each individual ParticipationEvents for details as to the type of ParticipationEvents that
        occurred and the target Process.
        The first parameter of the NotificationEvent indicates the type of ParticipationEvents, the second parameter
        the ID of the Gathering (GatheringID), and the string parameter indicates the associated message, if any.
        */

        // 第 3 パラメータには参加したクライアントからの参加人数がセットされます。
        /*!
         @brief     ギャザリングの参加者の状態変更時（参加・退出など）にギャザリングのオーナーもしくは参加者に通知されるイベントです。
         イベントの詳細（サブタイプ）は ParticipationEvents で定義されています。
         NotificationEvent::GetParam1() でイベントが発生した GatheringID 、 NotificationEvent::GetParam2() で参加・退出したクライアントの PrincipalID、
         NotificationEvent::GetStringParam() で付随するメッセージが取得できます。
         */
        ParticipationEvent = 3,

        /*
        The ownership of a Gathering has been transferred to another Principal.
        This event is sent to all participants in the Gathering when either MatchMakingClient::MigrateGatheringOwnership
        is called or the Gathering owner disconnects from the gathering and the GatheringFlags /MigrateOwner/ is set.
        The first parameter of the NotificationEvent indicates the ID of the Gathering (GatheringID) and the
        second parameter indicates the PrincipalID of the new owner.
        */

        /*!
         @brief     ギャザリングのオーナー変更時にギャザリングの参加者全員に通知されるイベントです。
         NotificationEvent::GetParam1() でイベントが発生した GatheringID 、 NotificationEvent::GetParam2() で新しいオーナーの PrincipalIDが取得できます。
        */
        OwnershipChangeEvent = 4,

        /*
         The status (online/offline) of a friend has changed.
         The first parameter of the NotificationEvent is the Pid of the principal that has changed status.
         The second parameter of the NotificationEvent is the FriendStatus of the principal that has changed status.
        */

        /*!
         @internal

         @brief    使用されません
        */
        FriendStatusChangeEvent = 5,

        /*
        rdv node forces user disconnection.
        This is used to implement preemptive login.
        The first parameter of the NotificationEvent is the Pid of the principal to disconnect.
        The second parameter is currently set to 0
        */

        /*!
         @internal

         @brief    使用されません
        */
        ForceDisconnectEvent = 6,

        /*
        An event triggered by the GameSession service has occurred.
        The type of the GameSessionEvent can be retrieved through the NotificationEvent::GetSubType method.
        */

        /*!
         @internal

         @brief    使用されません
        */
        GameSessionEvent = 7,

        /*
        RDV node forces user disconnection.
        This is used to implement kick out principal.
        The first parameter of the NotificationEvent is the did of the deamon owner of the principal to disconnect.
        The second parameter of the NotificationEvent is the Pid of the principal to disconnect.
        */

        /*!
         @internal

         @brief    使用されません
        */
        ForceDisconnectPrincipalEvent = 8,

        /*
        RDV node forces all user disconnection.
        This is used to implement kick principal.
        The first parameter of the NotificationEvent is the pid of the principal generating the request (not to disconnect).
        The second parameter is currently set to 0
        */

        /*!
         @internal

         @brief    使用されません
        */
        ForceDisconnectAllPrincipalsEvent = 9,

        /*
          for UpdateNotificationData, GetFriendNotificationData
        */

        /*!
         @brief    同じゲームサーバーにログインしているフレンドに通知されるイベントです。
         通知タイミング・通知データはアプリケーションが自由に定義することができます。
        */
        GameNotificationType1 = 101,

        /*!
         @brief    ゲーム定義イベント 2 です。
        */
        GameNotificationType2 = 102,

        /*!
         @brief    ゲーム定義イベント 3 です。
        */
        GameNotificationType3 = 103,

        /*!
         @brief    ゲーム定義イベント 4 です。
        */
        GameNotificationType4 = 104,

        /*!
         @brief    ゲーム定義イベント 5 です。
        */
        GameNotificationType5 = 105,

        /*!
         @brief    ゲーム定義イベント 6 です。
        */
        GameNotificationType6 = 106,

        /*!
         @brief    ゲーム定義イベント 7 です。
        */
        GameNotificationType7 = 107,

        /*!
         @brief    ゲーム定義イベント 8 です。
        */
        GameNotificationType8 = 108,

        /*!
         @brief     ギャザリングが削除されたときにギャザリングの参加者全員に通知されるイベントです。
         NotificationEvent::GetParam1() でイベントが発生した GatheringID が取得できます。
        */
        GatheringUnregistered = 109,

        /*!
         @brief     ギャザリングのホスト変更時に、前回のホストに通知されるイベントです。
         NotificationEvent::GetParam1() でイベントが発生した GatheringID が取得できます。
        */
        HostChangeEvent = 110,

        /*!
         @brief     ゲーム定義イベントをセットしていた場合に、
         ゲームサーバーからログアウトしたときにフレンドに通知されるイベントです。
         このログアウトは電源断などの非明示なログアウトも含みます。
        */
        GameNotificationLogout = 111,

        /*!
         @brief    Subscriptionで通知されるイベントです。
        */

        SubscriptionEvent = 112,
        /*!
         @brief     ゲームサーバーのメンテナンス開始通知イベントです。
        */
        GameServerMaintenance = 113,

        /*!
         @brief    ゲームサーバーのメンテナンス予告通知イベントです。
         NotificationEvent::GetParam1() でメンテナンス開始時刻が "Unix Epoch Time" で取得でき、
         DateTime::FromUnixEpochTime() を使用してサーバーメンテナンス時刻 （UTC） を取得できます。
        */
        MaintenanceAnnouncement = 114,

        /*!
         @internal

         @brief    ライブラリ内部用のイベントです。アプリケーションには通知されません。
        */
        SystemEvent = 115,

        /*!
         @brief    ラウンド開始時にラウンド参加者全員に通知されるイベントです。
                   NotificationEvent::GetParam1() でラウンドIDが取得できます。
        */
        RoundStarted = 116,

        /*!
         @brief    ラウンドの最初のレポートをサーバが受信したときに参加者全員に通知されるイベントです。
                   NotificationEvent::GetParam1() でラウンドIDが取得できます。
        */
        FirstRoundReportReceived = 117,

        /*!
         @brief    ラウンドの集計処理が終了したときに参加者全員に通知されるイベントです。
                   NotificationEvent::GetParam1() でラウンドIDが取得できます。
        */
        RoundSummarized = 118,

        /*!
         @internal

         @brief    Matchmakeで使用する通知用のイベントです。アプリケーションには通知されません。
        */
        MatchmakeSystemConfigurationNotification = 119,

        /*!
         @brief    参加しているマッチメイクセッションにマッチメイクセッションシステムパスワードが
                   設定されたときに参加者全員に通知されるイベントです。
                   NotificationEvent::GetStringParam() でマッチメイクセッションシステムパスワードが取得できます。
        */
        MatchmakeSessionSystemPasswordSet = 120,

        /*!
         @brief    参加しているマッチメイクセッションのマッチメイクセッションシステムパスワードが
                   解除されたときに参加者全員に通知されるイベントです。
        */
        MatchmakeSessionSystemPasswordClear = 121,

        /*!
         @brief    自分自身がギャザリングに追加されたことを表します。
         他のユーザーから MatchmakeExtensionClient::AutoMatchmake(), MatchmakeExtensionClient::JoinMatchmakeSession(),
         MatchmakeExtensionClient::CreateMatchmakeSession でギャザリングに追加されたときに追加されたユーザーに対して通知されます。
         NotificationEvent::GetParam1() で追加されたギャザリングの ID、 NotificationEvent::GetParam2() で追加されたユーザのプリンシパル ID (つまり自分のプリンシパル ID)が取得できます。
         NotificationEvent::GetStringParam() でイベント発生元ユーザーが参加時に指定した参加メッセージが取得できます。
         NotificationEvent::GetSource() で参加させる RMC を呼び出したユーザーのプリンシパル ID を取得可能です。
         */
        AddedToGathering = 122,

        /*!
         @internal

         @brief   ライブラリ内部用のイベントです。アプリケーションには通知されません。
         */
        SharedLocalCacheUpdated = 123,

        /*!
         @internal

         @brief   サーバーサイドマッチメイクがリクエストされたときに通知されます。
         NotificationEvent::GetParam1() でリクエストIDを取得できます。
         このリクエストIDを MatchmakeExtensionClient::WithdrawMatchmaking() に指定するとそのリクエストを取り下げる事ができます。
         */
        ServerSideMatchmakingRequested = 124,

        /*!
         @internal

         @brief   サーバーサイドマッチメイクが完了したときに通知されます。
         NotificationEvent::GetParam1() でリクエストIDを取得できます。
         NotificationEvent::GetParam3() でマッチメイク結果 (MatchmakingCompletionResult) を取得できます。
         NotificationEvent::GetStringParam() で参加したマッチメイクセッションのギャザリング ID を文字列化したものを取得できます。
         */
        ServerSideMatchmakingCompleted = 125,

        /*!
         @internal

         @brief   ライブラリ内部用のイベントです。アプリケーションには通知されません。
         */
        ServiceSettingUpdated = 126,

        /*!
         @internal

         @brief   ライブラリ内部用のイベントです。アプリケーションには通知されません。
         */
        ClusterHealthCheck = 127,

        // Custom reserved event
        CustomEvent1 = 200,
        CustomEvent2 = 201,
        CustomEvent3 = 202,
        CustomEvent4 = 203,
        CustomEvent5 = 204,
        CustomEvent6 = 205,
        CustomEvent7 = 206,
        CustomEvent8 = 207,
        CustomEvent9 = 208,
        CustomEvent10 = 209,
        CustomEvent11 = 210,
        CustomEvent12 = 211,
        CustomEvent13 = 212,
        CustomEvent14 = 213,
        CustomEvent15 = 214,
        CustomEvent16 = 215,

        /*!
         @internal

         @brief    使用されません
        */
        FirstUserNotification = 1000
    };
#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX

