﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX


/*!
    @internal
    @brief      64bit 型データ ID と表示用のデータコード文字列を相互変換するクラスです。

    データコード文字列とは、システムで連番で発番する ID（データ ID）と一対一対応する文字列です。
    0 1 2 3 4 5 6 7 8 9 B C D F G H J K L M N P R T V W X Y の28文字 から構成されます。
    データコード文字列から元の ID の推測は困難なため、安全にユーザーの表示に利用できます。
    また、チェックサムを含むため、ユーザーの誤入力を検出できます。

    64bit 型のデータ ID に対するデータコード文字列は最大 14 文字の文字列です。
    データ ID は、最大 56 ビット まで利用可能です。

*/
class DataCode64: public RootObject
{
protected:
    /*!
        @brief    DataCode64 オブジェクトを作成します。

        @param[in]    invalidDataId         このオブジェクトで使用する無効なデータ ID
        @param[in]    invalidDataCodeRaw    このオブジェクトで使用する無効な内部データコード
     */
    DataCode64(qUnsignedInt64 invalidDataId, qUnsignedInt64 invalidDataCodeRaw);

private:
    // 事実上 const だが、代入演算子扱いが難しくなるため const にしない。
    /*const*/ qUnsignedInt64 m_invalidDataId;
    /*const*/ qUnsignedInt64 m_invalidDataCodeRaw;
    qUnsignedInt64 m_dataCodeRaw;
    qBool m_isValid;

    qUnsignedInt64 DataIdToDataCode(const String& key, qUnsignedInt64 dataId) const;
    qBool CheckDataCode(const String& key, qUnsignedInt64 dataCode) const;
    qUnsignedInt64 DataCodeStr(const String& dataCodeStr) const;
protected:
    /*!
        @brief    データ ID を DataCode64 オブジェクトにセットします。

        56 ビットを超える場合は、  IsValid() が False となります。

        @param[in]    key       チェックサム算出時の鍵。自タイトルのアクセスキーを指定してください。
        @param[in]    dataId    データ ID

        @return    データ ID が利用可能な値の場合は true (IsValid() と同じ値)

        @see IsValid(), GetDataId()
     */
    qBool SetDataId(const String& key, qUnsignedInt64 dataId);

    /*!
        @brief    データコード文字列を DataCode64 オブジェクトにセットします。

        データコード文字列の妥当性検証時に、データコード文字列に小文字が含まれる場合には大文字に変換され、入力ミスが疑われる文字は適切な文字に変換されます。
        (I->1, Q->0, O->0, Z->2, S->5)

        データコード文字列が不正な場合、IsValid() が false を返します。

        @param[in]    key       チェックサム算出時の鍵。自タイトルのアクセスキーを指定してください。
        @param[in]    dataCodeString    データコード文字列

        @return    内部データコードが正当な値の場合 true (IsValid() と同じ値)

        @see IsValid(), ToDataCodeString(), DataCodeConstants::MAX_DATACODE64_STR_LEN
     */
    qBool SetDataCodeString(const String& key, const String& dataCodeString);


    /*!
        @brief    64 ビット整数値の内部データコードを DataCode64 オブジェクトにセットします。

        内部データコードが不正な場合、IsValid() が false を返します。


        @param[in]    key       チェックサム算出時の鍵。自タイトルのアクセスキーを指定してください。
        @param[in]    dataCodeRaw    内部データコード

        @return    内部データコードが正当な値の場合 true (IsValid() と同じ値)

        @see IsValid(), GetDataCodeRaw()
     */
    qBool SetDataCodeRaw(const String& key, qUnsignedInt64 dataCodeRaw);

public:
    /*!
        @brief データ ID を取得します。

        このオブジェクトが無効の場合は、 無効値を返します。

        @return データ ID

    */
    qUnsignedInt64 GetDataId() const;

    /*!
        @brief データコード文字列を生成します。

        データコード文字列とは、データ ID と チェックサムを含む、 0 1 2 3 4 5 6 7 8 9 B C D F G H J K L M N P R T V W X Y
        の28文字 からなる 最大 14 文字の文字列です。 データ ID は、最大 56 ビット まで利用可能です。

        このオブジェクトが無効の場合は、空文字列が返ります。

        @return データコード文字列

        @see IsValid(), DataCodeConstants::MAX_DATACODE64_STR_LEN
     */
    String ToDataCodeString() const;

    /*!
        @brief データコード文字列を生成します。 14文字に満たない場合は、上位を 0 で埋めます。

        データコード文字列とは、データ ID と チェックサムを含む、 0 1 2 3 4 5 6 7 8 9 B C D F G H J K L M N P R T V W X Y
        の28文字 からなる 最大 14 文字の文字列です。 データ ID は、最大 56 ビット まで利用可能です。

        このオブジェクトが無効の場合は、空文字列が返ります。

        @return データコード文字列

        @see IsValid()
     */
    String ToDataCodeStringZeroPad() const;

    /*!
        @brief 内部データコードを取得します。

        データ ID (56bit) とタイトル固有の チェックサム値 (8bit) から成る 64 ビットの整数値です。

        この関数を使う前に IsValid() で、オブジェクトが有効であることを確認してください。

        @return 内部データコード

        @see IsValid()
     */
    qUnsignedInt64 GetDataCodeRaw() const
    {
        return m_dataCodeRaw;
    }

    /*!
        @brief このオブジェクトが有効なデータコード文字列、有効なデータ ID もしくは有効な内部データコードから
               生成されたものか判定します。

        @retval true 有効なオブジェクトである
        @retval false 有効なオブジェクトでない

        @see CreateFromDataId(), CreateFromDataCodeString(), CreateFromDataCodeRaw()
        @see ToDataCodeString(), ToDataCodeStringZeroPad(), GetDataCodeRaw()
     */
    qBool IsValid() const
    {
        return m_isValid;
    }

    /*!
        @internal
        @brief チェックサム算出時の鍵が適切かどうかをチェックします。
            適切でない鍵に対するデータコード文字列の挙動は保証されません。

        @retval true 適切なチェックサム算出時の鍵である。
        @retval false 適切なチェックサム算出時の鍵でない。
     */
    static qBool IsValidKey(const String& key);
};


END_NAMESPACE_NEX
