﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Common/Data.h>
#include <RendezVous/Core/src/Common/AnyDataAdapter.h>
#include <RendezVous/Core/src/Common/AnyObjectHolder.h>

BEGIN_NAMESPACE_NEX

OSDK_EXPORT_ANY_OBJECT_HOLDER(Data, String)

typedef AnyObjectHolder<Data, String> DataHolder;

/*
    \ingroup group_rdv_core_common
    \brief This template class is used to customize a DataHolder for a particular type of Data.

    Use this template class to customize a DataHolder for a particular type of Data. The custom
    Data type is designated by T.
*/

/*!
 @brief    DataHolder クラスをカスタマイズするための templete クラスです。

 templete 引数に渡したクラス専用のDataHolderクラスが利用できます。
 */
template <class T=Data>
class CustomDataHolder: public AnyObjectHolder<Data, String>
{
public:
    /*
        \brief Constructor
        \param pData A pointer to the type of Data that this CustomDataHolder will refer to.
    */
    CustomDataHolder(T* pData=NULL): AnyObjectHolder<Data, String>(pData) {}

    /*
        \brief Gets the pointer to the Data object that is referenced by this CustomDataHolder.
        \returns A pointer to the Data object that is referenced by this CustomDataHolder.
        \returns NULL if no Data object is referenced
        \sa Release
    */
    T* Get() const
    {
        return static_cast<T*>(AnyObjectHolder<Data, String>::Get());
    }
    T& operator* () const
    {
        return *Get();
    }
    T* operator-> () const
    {
        return Get();
    }

    /*
     \brief Releases ownership of the Data object that this CustomDataHolder references.
     \sa Get
     */
    T* Release()
    {
        return static_cast<T*>(AnyObjectHolder<Data, String>::Release());
    }
};

#ifdef FOR_DOC_PURPOSE
/*!
 @brief    データのホルダークラスです

 @ref Data クラス及び Data の派生クラス( @ref UserMessage , @ref TextMessage , @ref BinaryMessage ) をゲームサーバーに送受信するために使用されます。
 内部で @ref Data クラスオブジェクトのポインタを保持しており、代入やデストラクタ呼び出し時に保持しているオブジェクトのデストラクタも呼び出します。
*/
class DataHolder: public RootObject
{
public:
    //group=Methods

    /*
    Parameters:
    p: A pointer that determines to where the DataHolder will refer.

    Remarks: If you use the first (default) constructor the DataHolder will not refer to
    anything, whereas  if you use the second constructor the DataHolder will refer to where
    the pointer refers.
    */

    /*!
     @brief    コンストラクタです。

     初期値として NULL ポインタがセットされます
     */
    DataHolder() : m_ptr(NULL)
    {
    }
    explicit DataHolder (Data* p) : m_ptr(p)
    {
    }

    // Remarks: When a DataHolder is deleted it releases the object, if any, to which it refers.
    /*!
     @brief    デストラクタです。

     保持している Data クラスオブジェクトを解放します。
     DataHolder のデストラクト後も Data クラスオブジェクトを使用する場合は、
     @ref Release を呼び出してオブジェクトの所有権を変更してください。
     */
    ~DataHolder()
    {
        if (m_ptr!=NULL)
        {
            delete m_ptr;
        }
    }

    // Remarks: Assigns a new pointer to the DataHolder. This method automatically deletes
    // and deallocates the previously held object, if any.
    /*!
     @brief    代入演算子です。

     保持している Data クラスオブジェクトをデストラクトした上で、引数で与えられた Data クラスオブジェクトをセットします。

     @param[in]    p    Data クラスオブジェクトのポインタ
     @return     自分自身への参照
     */
    DataHolder<T>& operator= (Data* p)
    {
        if (m_ptr!=NULL)
        {
            delete m_ptr;
        }
        m_ptr = p;
        return *this;
    }
    /*!
     @brief    保持しているオブジェクトのポインタを返します

     @return     保持しているオブジェクトのポインタ。セットされていない場合 NULL を返します。
     */
    Data& operator* () const
    {
        return *m_ptr;
    }

    /*!
     @brief    保持しているオブジェクトのポインタを返します

     @return     保持しているオブジェクトのポインタ。セットされていない場合 NULL を返します。
     */
    Data* operator-> () const
    {
        return m_ptr;
    }

    /*
    Description: Returns a pointer to the Data object that is referenced by this DataHolder.
    Returns: A pointer to the Data object that is referenced by this DataHolder.
    If no Data object is referenced, a NULL pointer is returned.
    See Also: Release, ExchangeWith
    */

    /*!
     @brief    保持しているオブジェクトのポインタを返します。

     保持しているオブジェクトのポインタを返します。セットされていない場合 NULL を返します。

     @return     保持しているオブジェクトのポインタ
     */
    Data* Get() const
    {
        return m_ptr;
    }

    /*
    Description: Releases ownership of the Data object that this DataHolder references.
    See Also: Get, ExchangeWith
    */

    /*!
     @brief    保持しているオブジェクトの所有権を解放します。

     保持しているオブジェクトの所有権を解放し、そのポインタを返します。

     @return     保持していたオブジェクトのポインタ
     */
    Data* Release()
    {
        Data* ret=m_ptr;
        m_ptr = 0;
        return ret;
    }

    /*
    Description: Exchanges ownership of a Data object with another DataHolder.
    Parameters:
    h: The DataHolder with which Data object ownership will be exchanged.
    p: A pointer to the Data object that will be exchanged with the current Data
    object

    Remarks: This method enables the held Data object to be replaced without destroying the
    object. The object that will replace the current object can be specified via either its
    current DataHolder owner or via a pointer to the said Data object.
    See Also: Get, Release
    */
    void ExchangeWith (DataHolder<T>& h)
    {
        std::swap(m_ptr,h.m_ptr);
    }
    void ExchangeWith (Data*& p)
    {
        std::swap(m_ptr,p);
    }
};
#endif


END_NAMESPACE_NEX

