﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Common/Constants/MembershipConstants.h>

BEGIN_NAMESPACE_NEX

class Ticket;

/*!
    @internal
    @brief メンバーシップ情報を表すクラスです。
*/
class MembershipInfo: public RootObject
{
public:
    /*!
        @internal
        @brief コンストラクタ。

        無効なオブジェクトを生成する。MembershipInfo::ParseFromTicket()を呼び出すまでMembershipInfo::IsInvalid()はtrueを返す。
     */
    MembershipInfo();

    /*!
        @internal
        @brief コンストラクタ。チケットからオブジェクトを生成する。

        @param ticket チケット。
        @param cid RVCID。

        パースが失敗した場合、CanPlayAll()を呼び出すとtrueを返すが、実際にはNGSからエラーが返される可能性がある。
        何らかの理由でメンバシップ情報が取得出来なかった場合でも接続は可能かもしれないので規定値はtrueとしている。
     */
    MembershipInfo(const Ticket& ticket, ConnectionID cid);

    /*!
        @internal
        @brief 無効なオブジェクトかどうかを取得する。
        @return 無効なオブジェクトかどうか。
     */
    qBool IsInvalid() const
    {
        return m_isInvalid;
    }

    /*!
    @internal
    @brief このメンバシップ情報に対応するプリンシパルIDを取得する。
    @return このメンバシップ情報に対応するプリンシパルID。
    */
    PrincipalID GetPrincipalId() const
    {
        return m_pid;
    }

    /*!
    @internal
    @brief このメンバシップ情報に対応するRVCIDを取得する。
    @return このメンバシップ情報に対応するRVCID。
    */
    ConnectionID GetConnectionId() const
    {
        return m_cid;
    }

    /*!
        @internal
        @brief チケットをパースしてメンバ変数に値をセットする。有効期限のチェックはしない。
        @param ticket チケット。
        @param cid RVCID。
        @retval true パースが全て正常に完了しメンバ変数に値がセットされた。
        @retval false パースが失敗した。メンバ変数の値は初期値になる。
     */
    qBool ParseFromTicket(const Ticket& ticket, ConnectionID cid);

    /*!
        @internal
        @brief 標準APIバージョンを取得する。
        @return 標準APIバージョン。
     */
    qUnsignedInt32 GetApiVersionGeneral() const
    {
        return m_apiVersionGeneral;
    }

    /*!
        @internal
        @brief カスタムAPIバージョンを取得する。
        @return カスタムAPIバージョン。
     */
    qUnsignedInt32 GetApiVersionCustom() const
    {
        return m_apiVersionCustom;
    }

    /*!
        @internal
        @brief メンバシップステータスを取得する。
        @return メンバシップステータス。
     */
    MembershipStatus::MembershipStatus GetMembershipStatus() const
    {
        return m_membershipStatus;
    }

    /*!
        @internal
        @brief 現在のメンバシップ情報で全てのNGSの機能を利用可能かどうかを返す。
        @retval true 全て利用可能。もしくは可能かどうか不明。実際にはNGSからメンバシップに関するエラーが返される場合がある。
        @retval false 全てもしくは一部の機能が利用できない。
                      (利用できない範囲はOMASで設定したメンバシップポリシーに依存する。)
                      全て利用出来るようにするにはメンバシップを購入する必要がある。
     */
    qBool CanPlayAll() const
    {
        return m_canPlayAll;
    }

    void Reset()
    {
        m_isInvalid = false;
        m_pid = INVALID_PRINCIPALID;
        m_cid = INVALID_CONNECTIONID;
        m_expirationUnixTimestamp = 0;
        m_apiVersionGeneral = 0;
        m_apiVersionCustom = 0;
        m_membershipPolicy = MembershipPolicy::NOT_REQUIRED;
        m_membershipStatus = MembershipStatus::NOT_FOUND;
        m_canPlayAll = true;    // 何らかの理由でメンバシップ情報が取得出来なかった場合でも接続は可能かもしれないので規定値はtrueとする。
    }

    qBool m_isInvalid;
    PrincipalID m_pid;
    ConnectionID m_cid; // RVCID
    qInt64 m_expirationUnixTimestamp;
    qUnsignedInt32 m_apiVersionGeneral;
    qUnsignedInt32 m_apiVersionCustom;
    MembershipPolicy::MembershipPolicy m_membershipPolicy;
    MembershipStatus::MembershipStatus m_membershipStatus;
    qBool   m_canPlayAll;

};

END_NAMESPACE_NEX
