﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/ConnectionID.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>
#include <RendezVous/Core/src/Common/CallProtocolMethodOperation.h>
#include <OnlineCore/src/Core/SystemComponent.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <OnlineCore/src/Core/OperationManager.h>
#include <RendezVous/Core/src/Common/ProtocolFlags.h>
#include <OnlineCore/src/Core/Buffer.h>

BEGIN_NAMESPACE_NEX

class Message;
class EndPoint;
class ProtocolRequestBrokerInterface;
class ProtocolCallContext;
class CallProtocolMethodOperation;
class ClientProtocol;
class ServerProtocol;

typedef qUnsignedInt16 ProtocolID;
// ReturnFunction(結果リターン時のデシリアライズ関数)の型
typedef void (*ProtoReturnFunction)( Message*, ProtocolCallContext* );

//==============================================================================
// Abstract base class for Server-side and client-side protocol classes
//==============================================================================


class OSDK_API Protocol: public SystemComponent
{
public:
    NEX_DECLARE_SYSCOMPONENT_METHODS(Protocol,SystemComponent);

    typedef enum _Type
    {
        Client,
        Server
    } Type;

    typedef enum _Command
    {
        Response,
        Request
    } Command;

    typedef enum _Reserved
    {
        UseExtendedProtocolIDMapping=127
    } Reserved;

    class ProtocolKey
    {
    public:
        ProtocolKey():m_value(0) {}
        void SetCommand(Protocol::Command eCommand)
        {
            if (eCommand == Protocol::Response)
            {
                m_value &= 0x7F;
            }
            else
            {
                m_value |= 0x80;
            }
        }
        Protocol::Command GetCommand() const
        {
            return Protocol::Command(m_value >> 7);
        }
        void SetProtocolID(ProtocolID uiProtocolID)
        {
            m_value = (qByte(uiProtocolID & 0x7F) | (m_value & 0x80));
        }
        ProtocolID GetProtocolID() const
        {
            return ProtocolID(m_value & 0x7F);
        }
        qByte GetValue() const
        {
            return m_value;
        }
        void SetValue(qByte bValue)
        {
            m_value = bValue;
        }
    private:
        qByte m_value;
    };

    Protocol(qUnsignedInt32 uiFlags);
    virtual ~Protocol();

#ifdef NEX_TRACE_ENABLED    // n2370
    virtual void TraceImpl(qUnsignedInt64 uiTraceFlags) const;
#endif  // NEX_TRACE_ENABLED

    virtual Protocol::Type GetProtocolType() const=0;

    qBool IsClientSide() const
    {
        return GetProtocolType()==Client;
    }
    qBool IsServerSide() const
    {
        return GetProtocolType()==Server;
    }

    qBool FlagIsSet(qUnsignedInt32 uiFlag) const;
    void ClearFlag(qUnsignedInt32 uiFlag);
    void SetFlag(qUnsignedInt32 uiFlag);

    void SetProtocolID(ProtocolID uiProtocolID);
    ProtocolID GetProtocolID() const
    {
        return m_uiProtocolID;
    }

public:
    OperationManager& GetOperationManager() const;

    ////////////////////////////////////////////////////////////////////////////////
public:
    virtual void EndPointDisconnected( EndPoint * /*pEndPoint*/ ) {};
    virtual void FaultDetected(EndPoint * /*pEndPoint*/, qUnsignedInt32 /*uiFault*/) {};
    ////////////////////////////////////////////////////////////////////////////////

    // This is mostly used by services, in a client/service scenario. On the
    // server side, the incoming call context represents a connection to the
    // client that is performing the call.
    EndPoint* GetIncomingConnection() const;

    // This is mostly used by clients, in a client/service scenario. On the
    // client side, the outgoing call context represents a connection to the
    // server.
    EndPoint* GetOutgoingConnection() const;
    void SetOutgoingConnection(EndPoint* pEndPoint);

    qBool IsUsingLocalLoopback() const
    {
        return m_bUseLocalLoopback;
    }
    void UseLocalLoopback(PrincipalID pidLocal, ConnectionID cidLocal);

    PrincipalID GetCallerPID() const;
    ConnectionID GetCallerCID() const;
    PrincipalID GetOwnPID() const;

    ProtocolRequestBrokerInterface* GetRequestBroker() const
    {
        return m_pAssociatedRequestBroker;
    }

    CallProtocolMethodOperation* GetCurrentOperation();

    void AssociateProtocolRequestBroker(ProtocolRequestBrokerInterface *pAssociatedRequestBroker);

    static void AddCallResponse(Message* pMessage, CallProtocolMethodOperation* pOp, qUnsignedInt32 uiContextID, const Message& msgCallResult);

    // Code generation helper.
    static void AddProtocolKey(Message *pMessage, Protocol::Command eProtocolCommand, ProtocolID uiProtocolID);
    static void ExtractProtocolKey(Message *pMessage, Protocol::Command& eProtocolCommand, ProtocolID& uiProtocolID);

    static void AddCallOutcome(Message *pResponse, qResult rCallResult);
    static void ExtractCallOutcome(Message *pMsg, qResult *prCallResult);

    static qBool RegisterCallContext(Message *pMessage, ProtocolCallContext* pCallContext);
    static void AddCallContextID(Message *pMessage, qUnsignedInt32 uiCallContextID);
    static void AddSignature(Message *pMessage, const Buffer& signature);
    static void ExtractSignature(Message *pMessage, Buffer& buffer);

    static void AddOriginalMessage(Message *pMessage, Message* pOriginalMessage);
    static void ExtractOriginalMessageHeader(Message *pMessage, Buffer& buffer);

    static void AddAlternativePrincipalID(Message *pMessage, PrincipalID principalID);
    static PrincipalID ExtractAlternativePrincipalID(Message *pMessage);

    static qUnsignedInt32 ExtractCallContextID(Message *pMessage);
    static ProtocolCallContext* RetrieveCallContext(Message* pMsg);

    static qUnsignedInt32 ExtractMethodID(Message *pMessage);
    static void AddMethodID(Message *pMessage, qUnsignedInt32 uiMethodID);

    static qBool IsOldRVDDLVersion(EndPoint *pEndPoint);

protected:

    void CopyMembers(const Protocol* pOriginal);

private:
    // Component API
    virtual qBool BeginInitialization();
    virtual qBool BeginTermination();

public:

    // Default implementation: Can't be cloned.
    virtual Protocol* Clone() const
    {
        return NULL;
    }

    // プロトコルのリロードを行う。動的なプロトコル向けインタフェース。
    // C++で実装されているプロトコルはServerProtocolクラスが使われる。C++の場合は動的にプロトコルが変化することはないため何もしなくてよい。
    // Pythonで実装されているプロトコルはPythonScriptEngineが使われる。Pythonの場合はHookなどによりRMCのエントリポイントが動的に変化する事があるため、
    // その場合はReload()で読み直す必要がある。(読み直す処理はScriptEngineクラスで実装されている)
    virtual qBool Reload()
    {
        return true;
    }

    void SetIncomingConnection(EndPoint* pEndPoint);

private:
    ProtocolID m_uiProtocolID;
    EndPoint *m_pOutgoingConnection;
    ProtocolRequestBrokerInterface *m_pAssociatedRequestBroker;

    qUnsignedInt32 m_uiFlags;

    // When we use a connection...
    EndPoint *m_pIncomingConnection;

    // When we use a localloopback
    qBool m_bUseLocalLoopback;

    PrincipalID m_pidCaller;
    ConnectionID m_cidCaller;

};


END_NAMESPACE_NEX

