﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

// A RendezVousStream encapsulates a Stream, manages its connections to
// it via a ConnectionManager and forwards its messages to a or many
// ProtocolRequestBroker.

// There are several kinds of RendezVousStreams: the
// RendezVousClientStream, a RendezVousAuthenticationStream and a
// RendezVousSecureStream.

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Core/SystemSetting.h>
#include <OnlineCore/src/Transport/Interface/ConnectionManager.h>
#include <OnlineCore/src/Transport/Interface/Network.h>
#include <OnlineCore/src/Transport/Interface/EndPointEventHandler.h>

BEGIN_NAMESPACE_NEX

class ConnectionOrientedStream;
class PRUDPStream;
class ProtocolRequestBrokerInterface;
class Credentials;

class OSDK_API StreamManager: public EndPointEventHandler
{
public:
#ifndef SWIG
    StreamManager(Stream::Type oStreamType, RootTransport* pTransport);
    virtual ~StreamManager();

    typedef enum _Flags
    {
        AcceptIncomingConnections   = (1<<0), // Listen用のローカルポートをbindするかどうか
        MustUseSpecifiedPort        = (1<<1), // Listenする場合、指定したローカルポートを必ず使うかどうか
        ConnectToOthers             = (1<<2), // Connect用のローカルポートをbindするかどうか
        AcceptIncomingStreams       = (1<<3), // 相手からのStream接続を許可するかどうか。AcceptIncomingConnectionsを指定した場合は自動的に有効になります。
                                              // このフラグを指定するとConnectToOthersを指定した場合でも相手側からのEndPoint::Connect()が可能になります。
                                              // RVを経由したクライアント同士でSecureStreamの通信をする場合に必要になります。
        ForceUseExistenceListenSocket   = (1<<4), // AcceptIncomingConnectionsが指定されていたとき、指定したローカルポートのソケットを必ず再利用する
        ForceUseExistenceConnectSocket  = (1<<5), // ConnectToOthersが指定されていたとき、指定したローカルポートのソケットを必ず再利用する
        UseExistenceConnectSocketIfPossible = (1<<6)  // ConnectToOthersが指定されていたとき、コネクション指向プロトコルを使っている場合
                                                      // ローカルポートが同じソケットがあれば指定したローカルポートのソケットを再利用する。
                                                      // コネクション指向でないプロトコル使用時はこのフラグにかかわらず再利用しようとする。
                                                      // 再利用するソケットがなければそのポートでのbindを試した後に任意ポートのbindを試す。
    } Flags;

    qBool Initialize(qUnsignedInt16 uiPort, Stream::ID idStream, qUnsignedInt32 uiFlags);

    qBool InitializeAsService(qUnsignedInt16 uiPort)
    {
        return Initialize(uiPort, StreamID::WellKnown, AcceptIncomingConnections | MustUseSpecifiedPort);
    }

    void Terminate();
    qBool IsTerminated() const;

    qBool IsInitialized() const
    {
        return m_bInitialized;
    }

    ProtocolRequestBrokerInterface* GetProtocolRequestBroker()
    {
        return m_pBroker;
    }
    const ProtocolRequestBrokerInterface* GetProtocolRequestBroker() const
    {
        return m_pBroker;
    }

    ConnectionManager* GetConnectionManager()
    {
        return m_pConnectionManager;
    }
    const ConnectionManager* GetConnectionManager() const
    {
        return m_pConnectionManager;
    }

    // Connect用のStreamを取得する
    PRUDPStream *GetPRUDPStream()
    {
        return m_pStream;
    }
    const PRUDPStream *GetPRUDPStream() const
    {
        return m_pStream;
    }

    // Listen用のStreamを取得する
    PRUDPStream *GetPRUDPStreamForAcceptedConnection()
    {
        return m_pStreamForAcceptedConnection;
    }
    const PRUDPStream *GetPRUDPStreamForAcceptedConnection() const
    {
        return m_pStreamForAcceptedConnection;
    }

    ConnectionOrientedStream *GetSecureStream()
    {
        return m_pSecureStream;
    }
    void AssociateSecureStream(ConnectionOrientedStream *pTransport, ConnectionManager::ConnectionFilter pfConnectionFilter);
    void ClearAssociatedSecureStream();

    static void SetSandboxAccessKey(const String& strSandboxAccessKey);
    static String GetSandboxAccessKey();

    static void SetProtocolVersion(qUnsignedInt32 uiProtocolVersion)
    {
        s_uiProtocolVersion = uiProtocolVersion;
    }
#endif

    void SetMaxSilenceTimeout(qUnsignedInt32 uiTimeout);
    void SetKeepAliveTimeout(qUnsignedInt32 uiTimeout);
    void EmulateSymmetricNAT();
    qBool SimulateFault(PrincipalID pid);

    // used for test purposes
    static void SetPacketDropProbability(qFloat fProbability)
    {
        s_fPacketDropProbability = fProbability;
    }

    virtual void SetCredentials(Credentials *) {};

    qUnsignedInt16 GetConnectingLocalPort() const
    {
        return m_uiConnectingLocalPort;
    }
    qUnsignedInt16 GetListeningLocalPort() const
    {
        return m_uiListeningLocalPort;
    }
private:

    void SetStreamKey();
    qBool InitializeImpl(qUnsignedInt16 uiPort, Stream::ID idStream, qUnsignedInt32 uiFlags);
    void Cleanup();

    virtual void Receive(EndPoint *pEndPoint, Buffer *pBuffer, qByte subStreamID)=0;
    virtual void PeerDisconnected(EndPoint *pEndPoint)=0;
    virtual void FaultDetection(EndPoint *pEndPoint, qUnsignedInt32 uiFault)=0;

protected:
    void SetProtocolRequestBroker(ProtocolRequestBrokerInterface* pBroker);

private:
    PRUDPStream *m_pStream;
    PRUDPStream *m_pStreamForAcceptedConnection;

    ConnectionOrientedStream *m_pSecureStream;

    ProtocolRequestBrokerInterface *m_pBroker;
    ConnectionManager *m_pConnectionManager;
    qBool m_bInitialized;

    //  Char array instead of String, to prevent memory leak
    enum { SANDBOX_ACCESS_KEY_SIZE = 128 };
    static qChar s_szSandboxAccessKey[SANDBOX_ACCESS_KEY_SIZE];

    static qUnsignedInt32 s_uiProtocolVersion;

    static qFloat s_fPacketDropProbability;

    RootTransport* m_pTransport;
    Stream::Type m_oStreamType;

    qUnsignedInt16 m_uiConnectingLocalPort; // Connect用ソケットに割り当てられたローカルポート
    qUnsignedInt16 m_uiListeningLocalPort;  // Listen用ソケットに割り当てられたローカルポート

    static SystemSetting s_ssUseDistinctPort;
};

END_NAMESPACE_NEX


