﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>
#include <RendezVous/Core/src/Common/MessageRecipientDDL.h>
#include <RendezVous/Core/src/Common/UserMessageDDL.h>
#include <RendezVous/Core/src/Common/MessageRecipient.h>
#include <RendezVous/Core/src/Common/Constants/IdentifierTypes.h>


BEGIN_NAMESPACE_NEX


/*
    \ingroup group_rdv_core_common
    \brief Use this class to construct and retrieve messages sent via
           the Messaging Service.

    This class is used as a header to construct and retrieve messages sent via
    the Messaging Service. You can derive your own custom message from
    UserMessage, to add specific information, or use a predefined class
    like TextMessage, to send supplementary text information.
    \sa Messaging service, TextMessage
*/
#ifdef FOR_DOC_PURPOSE
/*!
 @brief    メッセージデータを格納するクラスです。

 インスタントメッセージング機能を利用する場合は本クラス、@ref TextMessage 、@ref BinaryMessage のいずれかのクラスを利用してください。
 */
class OSDK_API UserMessage: public Data
{

#else
class OSDK_API UserMessage: public DDLCLASS(UserMessage)
{

#endif

public:
    /** \internal */
    void Clear()
    {
        m_uiID=0;
        m_uiParentID=0;
        m_pidSender=INVALID_PRINCIPALID;
        m_receptiontime = DateTime::Never;
        m_uiLifeTime = 0;
        m_uiFlags = 0;
        m_strSubject = NEX_T("");
        SetRecipient(MessageRecipient());
    }

    /*
    \brief Default constructor.
    */

    /*!
     @brief    コンストラクタです。
     */
    UserMessage()
    {
        Clear();
    }

    /*
    \brief Returns the ID of this UserMessage.

    The ID corresponds to the message unique ID in the database. Any message
    sent through Instant Messaging is not stored into the database, which means
    that GetID()==0 in this case.
    \returns Returns the ID of this UserMessage.
    */
    qUnsignedInt32 GetID() const
    {
        return m_uiID;
    }

    /** \internal */
    void SetID( qUnsignedInt32 uiID )
    {
        m_uiID = uiID;
    }

    /*
    \brief Returns the parent ID of this UserMessage.

    The purpose of this attribute is to allow usage of parent/child relation
    between messages, like for forum messages, for example.
    */
    qUnsignedInt32 GetParentID() const
    {
        return m_uiParentID;
    }

    /*
    \brief Sets the parent ID of this UserMessage.

    The purpose of this attribute is to allow usage of parent/child relation between
    messages, like for forum messages, for example.
    */
    void SetParentID(qUnsignedInt32 uiParentMsgID)
    {
        m_uiParentID = uiParentMsgID;
    }

    /*
    \brief Returns the timestamp of this UserMessage, as reception time on server side.
    \returns The timestamp of this UserMessage
    */

    /*!
     @brief    メッセージの受信時刻を取得します。

     ゲームサーバーがメッセージを受信した時刻、すなわち送信元が送信した時刻を取得します。
     クライアントがメッセージを受信した時刻ではないことに注意してください。

     時刻は UTC でセットされています。

     @return    ゲームサーバーがメッセージを受信した時刻
     */
    const DateTime& GetReceptionTime() const
    {
        return m_receptiontime;
    }

    /**
    \internal
    for debug (mainly set by script)
    */
    void SetReceptionTime(const DateTime& oDateTime)
    {
        m_receptiontime = oDateTime;
    }

    /** \internal */
    qUnsignedInt32 GetLifeTime() const
    {
        return m_uiLifeTime;
    }

    /** \internal */
    void SetLifeTime(qUnsignedInt32 uiLifeTime)
    {
        m_uiLifeTime = uiLifeTime;
    }

    /** \internal */
    qUnsignedInt32 GetFlags() const
    {
        return m_uiFlags;
    }

    /** \internal */
    void SetFlags(qUnsignedInt32 uiFlags)
    {
        m_uiFlags = uiFlags;
    }

    /*
    \brief Gets the PrincipalID for sender of this message.
    \returns PrincipalID of the message's sender
    */

    /*!
     @brief    メッセージの送信元の PrincipalID を取得します。

     @return    メッセージの送信元の PrincipalID
     */
    PrincipalID GetSenderID() const
    {
        return m_pidSender;
    }

    /**
    \internal
    for debug (mainly set by script)
    */
    void SetSenderID(PrincipalID pidSender)
    {
        m_pidSender = pidSender;
    }

    /*
    \brief Gets the username for sender of this message.
    \returns username of the message's sender
    */
    const String& GetSenderName() const
    {
        return m_strSender;
    }

    /**
    \internal
    for debug (mainly set by script)
    */
    void SetSenderName(String strSender)
    {
        m_strSender = strSender;
    }

    /*!
     * @brief メッセージの送信時に送信先としてセットされた プリンシパル ID を返します。
     *
     * GetRecipientType() が返す ID のタイプが IdentifierTypes::PIDType の場合に使用します。
     * MessagingClient::SendInstant() で送信された場合はプリンシパル ID が返ります。
     * MessagingClient::SendChat() で送信された場合は INVALID_PRINCIPALID が返ります。
     *
     * @return プリンシパル ID
     *
     * @see IdentifierTypes::Types, GetRecipientType()
     *
     */
    PrincipalID GetRecipientPrincipalID() const
    {
        return GetRecipient().GetRecipientPrincipalID();
    }

    /*!
     * @brief メッセージの送信時に送信先としてセットされた ギャザリング ID を返します。
     *
     * GetRecipientType() が返す IDのタイプ が IdentifierTypes::GIDType の場合に使用します。
     * MessagingClient::SendInstant() で送信された場合は INVALID_GATHERINGID が返ります。
     * MessagingClient::SendChat() で送信された場合は ギャザリング ID が返ります。
     *
     * @return ギャザリング ID
     *
     * @see IdentifierTypes::Types, GetRecipientType()
     *
     */
    GatheringID GetRecipientGatheringID() const
    {
        return GetRecipient().GetRecipientGatheringID();
    }

    /*
    \brief Returns the message's recipient type.
    Types are defined in IdentifierTypes.h
    */

    /*!
     * @brief GetRecipientPrincipalID(), GetRecipientGatheringID() どちらの関数を呼ぶ出すべきかを判断するための ID のタイプを返します。
     *
     * @return IdentifierTypes::Types の値
     *
     * @see IdentifierTypes::Types, GetRecipientPrincipalID(), GetRecipientGatheringID()
     */
    IdentifierTypes::Types GetRecipientType() const
    {
        return static_cast<IdentifierTypes::Types>(GetRecipient().GetRecipientType());
    }

    /** \internal */
    void SetRecipient(const MessageRecipient& oRecipient)
    {
        m_messageRecipient = oRecipient;
    }

    /** \internal */
    const MessageRecipient& GetRecipient() const
    {
        return m_messageRecipient;
    }

    /*
    \brief Returns the subject part of the message.
    */

    /*!
     @brief    メッセージの Subject を取得します。

     @return    メッセージの Subject
    */
    const String& GetSubject() const
    {
        return m_strSubject;
    }

    /*
    \brief Sets the subject part of the message.
    */

    /*!
     @brief    メッセージの Subject をセットします。
     MESSAGING_STRING_MAX_LENGTH 文字( NULL終端含まず ) を超えるテキストは、
     インスタントメッセージング機能では使用できません。サーバー送信時にエラーとなります。

     @param[in]    strSubject    メッセージの Subject
    */
    void SetSubject(const String& strSubject)
    {
        m_strSubject = strSubject;
    }

    /*
     for debug
     */
    virtual void Trace(qUnsignedInt64 traceFlag);
};

/**
\internal
*/
extern StringStream& operator<<(StringStream& oStr, const MessageRecipient& oMsgRecipient);

/**
\internal
*/
extern OSDK_API StringStream& operator<<(StringStream& oStr, const UserMessage& oUserMsg);


END_NAMESPACE_NEX

