﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/Jugem/DataStore/src/Client/DataStoreApiVersionTraits.h>
#include <RendezVous/Services/Jugem/DataStore/src/Client/DataStoreClientInterface.h>
#include <RendezVous/Services/Jugem/DataStore/src/Client/DataStoreEventListener.h>
#include <RendezVous/Services/Jugem/DataStore/src/Client/DataStoreLogicServerClient.h>
#include <RendezVous/Services/Jugem/DataStore/src/Common/DataStoreErrors.h>
#include <RendezVous/Services/Jugem/DataStore/src/Common/DataStoreDataCode.h>

// GetObjectのdefine回避開始 #29450
#ifdef WIN32_TARGET
#ifdef GetObject
#define NEX_PREVIOUS_GETOBJECT_DEFINED
#undef GetObject
#endif
#endif

BEGIN_NAMESPACE_NEX

#ifdef FOR_DOC_PURPOSE
/*!
    @brief DataStore サーバーとの通信を行うためのライブラリです。
*/
class DataStoreClient : public DataStoreLogicServerClient, public DataStoreClientInterface
#else
template<class TLogicServerClient>
class DataStoreClientTemplate : public TLogicServerClient, public DataStoreClientInterface
#endif  // FOR_DOC_PURPOSE
{
public:

#ifdef FOR_DOC_PURPOSE
    /*!
        @brief      コンストラクタです。
    */
    DataStoreClient();
#else
    DataStoreClientTemplate();
#endif  // FOR_DOC_PURPOSE

#ifdef FOR_DOC_PURPOSE
    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreClient() {};
#else
    virtual ~DataStoreClientTemplate() {};
#endif  // FOR_DOC_PURPOSE

    /*!
        @brief      DataStore サーバーからデータを取得します。

        
        本関数呼び出し前に nn::http::Initialize() を呼び出し、HTTP ライブラリを初期化する必要があります。
        
        
        本関数を呼び出すと NEX のスレッドモードにかかわらず通信用スレッドが一つ作成されます。
        ダウンロードしたデータは指定した eventListener からアプリケーションに渡されます。
        ダウンロードするデータを保存するメモリを一度に確保できない場合などに使用してください。
        メタ情報のみをアップロードする DataStoreClient::PostObject() を使いアップロードされたデータに対して本関数を呼び出した場合の結果は
        QERROR(DataStore, OperationNotAllowed) となります。代わりに DataStoreClient::GetMeta() を使用して下さい。
        対象のデータの参照権限を持っている人のみが参照できます。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[in]  eventListener ダウンロードされたデータを受け取るリスナーオブジェクト。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool GetObject(ProtocolCallContext *pContext, const DataStoreGetParam& param, DataStoreGetObjectEventListener* eventListener)
    {
        return GetObjectImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_LATEST> >(pContext, param, eventListener, NULL, 0, false, NULL);
    }

#ifdef WIN32_TARGET
    // GetObjectのdefine問題回避用定義
    virtual qBool GetObjectA(ProtocolCallContext *pContext, const DataStoreGetParam& param, DataStoreGetObjectEventListener* eventListener)
    {
        return GetObject(pContext, param, eventListener);
    }

    // GetObjectのdefine問題回避用定義
    virtual qBool GetObjectW(ProtocolCallContext *pContext, const DataStoreGetParam& param, DataStoreGetObjectEventListener* eventListener)
    {
        return GetObject(pContext, param, eventListener);
    }
#endif  // WIN32_TARGET

    /*!
        @internal
        @brief      DataStore サーバーからデータを取得します。

        
        本関数呼び出し前に nn::http::Initialize() を呼び出し、HTTP ライブラリを初期化する必要があります。
        
        
        本関数を呼び出すと NEX のスレッドモードにかかわらず通信用スレッドが一つ作成されます。
        ダウンロードしたデータは指定した eventListener からアプリケーションに渡されます。
        ダウンロードするデータを保存するメモリを一度に確保できない場合などに使用してください。
        もし DataStoreConstants::DATA_FLAG_NOT_USE_FILESERVER のフラグが立っているデータを取得しようとした場合、メタバイナリが代わりにダウンロードされます。
        このときメタバイナリのサイズが 0 なら非同期処理の結果は QERROR(DataStore, OperationNotAllowed) となります
        対象のデータの参照権限を持っている人のみが参照できます。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[in]  eventListener ダウンロードされたデータを受け取るリスナーオブジェクト。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool GetObjectOrMetaBinary(ProtocolCallContext *pContext, const DataStoreGetParam& param, DataStoreGetObjectEventListener* eventListener)
    {
        return GetObjectImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_LATEST> >(pContext, param, eventListener, NULL, 0, true, NULL);
    }

    /*!
        @internal
        @brief      DataStore サーバーからデータを取得します。古いインタフェースの互換性維持用。

        
        本関数呼び出し前に nn::http::Initialize() を呼び出し、HTTP ライブラリを初期化する必要があります。
        
        
        本関数を呼び出すと NEX のスレッドモードにかかわらず通信用スレッドが一つ作成されます。
        ダウンロードしたデータは指定した eventListener からアプリケーションに渡されます。
        ダウンロードするデータを保存するメモリを一度に確保できない場合などに使用してください。
        対象のデータの参照権限を持っている人のみが参照できます。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[in]  eventListener ダウンロードされたデータを受け取るリスナーオブジェクト。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool GetObjectV1(ProtocolCallContext *pContext, const DataStoreGetParamV1& param, DataStoreGetObjectEventListenerV1* eventListener)
    {
        return GetObjectImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_1> >(pContext, param, eventListener, NULL, 0, false, NULL);
    }

    /*!
        @brief      DataStore サーバーからデータを取得します。

        
        本関数呼び出し前に nn::http::Initialize() を呼び出し、HTTP ライブラリを初期化する必要があります。
        
        
        本関数を呼び出すと NEX のスレッドモードにかかわらず通信用スレッドが一つ作成されます。
        ダウンロードしたデータは buffer に書き込まれます。非同期処理が成功したときのみ buffer からデータを読み出すことが可能です。
        メタ情報のみをアップロードする DataStoreClient::PostObject() を使いアップロードされたデータに対して本関数を呼び出した場合の結果は
        QERROR(DataStore, OperationNotAllowed) となります。代わりに DataStoreClient::GetMeta() を使用して下さい。
        対象のデータの参照権限を持っている人のみが参照できます。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] buffer ダウンロードしたデータを書き込むバッファ。
        @param[in]  bufferSize buffer のサイズ。ダウンロードしたいデータのサイズ以上の値を指定してください。
        @param[out] pWrittenSize ダウンロードしたデータのサイズを格納するポインタ。NULL を指定すると格納されません。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool GetObject(ProtocolCallContext *pContext, const DataStoreGetParam& param, void* buffer, size_t bufferSize, size_t* pWrittenSize)
    {
        return GetObjectImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_LATEST> >(pContext, param, NULL, static_cast<qUnsignedInt8*>(buffer), bufferSize, false, pWrittenSize);
    }

#ifdef WIN32_TARGET
    // GetObjectのdefine問題回避用定義
    virtual qBool GetObjectA(ProtocolCallContext *pContext, const DataStoreGetParam& param, void* buffer, size_t bufferSize, size_t* pWrittenSize)
    {
        return GetObject(pContext, param, buffer, bufferSize, pWrittenSize);
    }

    // GetObjectのdefine問題回避用定義
    virtual qBool GetObjectW(ProtocolCallContext *pContext, const DataStoreGetParam& param, void* buffer, size_t bufferSize, size_t* pWrittenSize)
    {
        return GetObject(pContext, param, buffer, bufferSize, pWrittenSize);
    }
#endif  // WIN32_TARGET

    /*!
        @brief      DataStore サーバーにデータをアップロードします。

        
        本関数呼び出し前に nn::http::Initialize() を呼び出し、HTTP ライブラリを初期化する必要があります。
        
        
        本関数を呼び出すと NEX のスレッドモードにかかわらず通信用スレッドが一つ作成されます。
        アップロードするデータは eventListener からコールバックされる関数で指定します。
        本関数を使用してアップロードしたデータはメタ情報とは別に専用のストレージサーバー領域に保存されます。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] dataId アップロードされたデータに割り当てられたデータ ID。データ ID はゲーム内でユニークな値になります。
                    NULL を指定すると格納されません。
        @param[in]  eventListener アップロードするデータを指定するリスナーオブジェクト。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool PostObject(ProtocolCallContext *pContext, const DataStorePostParam& param, qUnsignedInt64* dataId, DataStorePostObjectEventListener* eventListener)
    {
        return PostObjectImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_LATEST> >(pContext, param, dataId, eventListener, NULL, false);
    }

    /*!
        @internal
        @brief      DataStore サーバーにデータをアップロードします。古いインタフェースの互換性維持用。

        
        本関数呼び出し前に nn::http::Initialize() を呼び出し、HTTP ライブラリを初期化する必要があります。
        
        
        本関数を呼び出すと NEX のスレッドモードにかかわらず通信用スレッドが一つ作成されます。
        アップロードするデータは eventListener からコールバックされる関数で指定します。
        本関数を使用してアップロードしたデータはメタ情報とは別に専用のストレージサーバー領域に保存されます。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] dataId アップロードされたデータに割り当てられたデータID。データIDはゲーム内でユニークな値になります。
        @param[in]  eventListener アップロードするデータを指定するリスナーオブジェクト。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool PostObjectV1(ProtocolCallContext *pContext, const DataStorePostParamV1& param, qUnsignedInt32* dataId, DataStorePostObjectEventListener* eventListener)
    {
        return PostObjectImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_1> >(pContext, param, dataId, eventListener, NULL, false);
    }

    /*!
        @brief      DataStore サーバーにデータをアップロードします。

        
        本関数呼び出し前に nn::http::Initialize() を呼び出し、HTTP ライブラリを初期化する必要があります。
        
        
        本関数を呼び出すと NEX のスレッドモードにかかわらず通信用スレッドが一つ作成されます。
        アップロードするデータは buffer から指定します。
        本関数を使用してアップロードしたデータはメタ情報とは別に専用のストレージサーバーに保存されます。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] dataId アップロードされたデータに割り当てられたデータ ID。データ ID はゲーム内でユニークな値になります。
                    NULL を指定すると格納されません。
        @param[in]  buffer アップロードするデータ。 param に対して DataStorePreparePostParam::SetSize() で指定したサイズ分がアップロードされます。
        @param[in]  useInternalBuffer true を指定した場合(デフォルト)、 buffer の内容はライブラリ内部にコピーして保持されるため呼び出し側では破棄して構いません。
                                      この場合、DataStorePostParam::SetSize()で指定したサイズ分のメモリを内部に確保します。
                                      false を指定した場合、非同期処理が完了するまで buffer の内容を保持してください。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool PostObject(ProtocolCallContext *pContext, const DataStorePostParam& param, qUnsignedInt64* dataId, const void* buffer, qBool useInternalBuffer=true)
    {
        return PostObjectImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_LATEST> >(pContext, param, dataId, NULL, static_cast<const qUnsignedInt8*>(buffer), useInternalBuffer);
    }

    /*!
        @brief      DataStore サーバーにメタ情報のみをアップロードします。

        本関数を使用してメタ情報のみアップロードした場合は専用のストレージサーバーへのデータ保存は行いません。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] dataId アップロードされたデータに割り当てられたデータ ID。データ ID はゲーム内でユニークな値になります。
                           NULL を指定すると格納されません。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool PostObject(ProtocolCallContext *pContext, const DataStorePostParam& param, qUnsignedInt64* dataId)
    {
        return TLogicServerClient::PostMetaBinary(pContext, param, dataId);
    }

    /*!
        @brief      データ ID を指定して DataStore サーバーにメタ情報のみをアップロードします。

        データ ID 指定版です。通常のアップロードでは、データ ID をサーバーが自動的に割り当てますが、
        本関数を仕様すると、データ ID をクライアントが指定できます。
        詳しくはプログラミングマニュアルを参照してください。
        本関数を使用してメタ情報のみアップロードした場合は専用のストレージサーバーへのデータ保存は行いません。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  dataId アップロードされたデータに割り当てられたデータID。データIDはゲーム内でユニークな値になります。
        @param[in]  param 詳細情報。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool PostObject(ProtocolCallContext *pContext, const qUnsignedInt64 dataId, const DataStorePostParam& param)
    {
        return TLogicServerClient::PostMetaBinaryWithDataId(pContext, dataId, param);
    }

    /*!
        @brief      データ ID を指定して DataStore サーバーにメタ情報のみを一括アップロードします。

        データ ID 指定版です。通常のアップロードでは、データ ID をサーバーが自動的に割り当てますが、
        本関数を仕様すると、データ ID をクライアントが指定できます。
        詳しくはプログラミングマニュアルを参照してください。
        本関数を使用してメタ情報のみアップロードした場合は専用のストレージサーバーへのデータ保存は行いません。

        一括処理の上限数は100件です。
        pResultにはparamsで指定したデータIDに対する処理の結果が指定した数だけ同順で返ります。

        @param[in,out]   pContext 呼び出し情報。
        @param[in]   params 詳細情報。
        @param[in]   transactional すべて成功する場合のみ処理を行うか (true)、できる限り処理を行うか。処理に失敗したデータについては処理が取り消されます。(false)。
        @param[out]  pResults 成功したかどうかの結果 (省略可)

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool PostObject(ProtocolCallContext *pContext, const qVector< std::pair<qUnsignedInt64, DataStorePostParam> >& params, const qBool transactional = true, qVector<qResult>* pResults = NULL)
    {
        return TLogicServerClient::PostMetaBinariesWithDataId(pContext, params, transactional, pResults);
    }

    /*!
        @brief      データ ID を指定して DataStore サーバーにメタ情報のみを一括アップロードします。

        データ ID 指定版です。通常のアップロードでは、データ ID をサーバーが自動的に割り当てますが、
        本関数を仕様すると、データ ID をクライアントが指定できます。
        詳しくはプログラミングマニュアルを参照してください。
        本関数を使用してメタ情報のみアップロードした場合は専用のストレージサーバーへのデータ保存は行いません。

        一つのメタ情報を複数のデータ ID に設定することができます。
        一括処理の上限数は100件です。
        pResultにはdataIdsで指定したデータIDに対する処理の結果が指定した数だけ同順で返ります。

        @param[in,out]   pContext 呼び出し情報。
        @param[in]   dataIds アップロードされたデータに割り当てられたデータID配列。データIDはゲーム内でユニークな値になります。
        @param[in]   param 詳細情報。
        @param[in]   transactional すべて成功する場合のみ処理を行うか (true)、できる限り処理を行うか。処理に失敗したデータについては処理が取り消されます。(false)。
        @param[out]  pResults 成功したかどうかの結果 (省略可)

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool PostObject(ProtocolCallContext *pContext, const qVector< qUnsignedInt64 >& dataIds, const DataStorePreparePostParam& param, const qBool transactional = true, qVector<qResult>* pResults = NULL)
    {
        return TLogicServerClient::PostMetaBinaryWithDataIds(pContext, dataIds, param, transactional, pResults);
    }


    /*!
        @brief      DataStore サーバーにアップロードしてあるデータを置き換えます。

        
        本関数呼び出し前に nn::http::Initialize() を呼び出し、HTTP ライブラリを初期化する必要があります。
        
        
        本関数を呼び出すと NEX のスレッドモードにかかわらず通信用スレッドが一つ作成されます。
        アップロードするデータは eventListener からコールバックされる関数で指定します。データIDは変化しません。
        メタ情報のみをアップロードする DataStoreClient::PostObject() を使いアップロードされたデータに対して本関数を呼び出した場合の結果は
        QERROR(DataStore, OperationNotAllowed) となります。メタ情報を変更するには DataStoreClient::ChangeMeta() を使用して下さい。
        更新権限を持っているデータのみ更新することが可能です。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[in]  eventListener アップロードするデータを指定するリスナーオブジェクト。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool UpdateObject(ProtocolCallContext *pContext, const DataStoreUpdateParam& param, DataStorePostObjectEventListener* eventListener)
    {
        return UpdateObjectImpl(pContext, param, eventListener, NULL, false);
    }

    /*!
        @brief      DataStore サーバーにアップロードしてあるデータを置き換えます。

        
        本関数呼び出し前に nn::http::Initialize() を呼び出し、HTTP ライブラリを初期化する必要があります。
        
        
        本関数を呼び出すと NEX のスレッドモードにかかわらず通信用スレッドが一つ作成されます。
        アップロードするデータは buffer から指定します。データIDは変化しません。
        メタ情報のみをアップロードする DataStoreClient::PostObject() を使いアップロードされたデータに対して本関数を呼び出した場合の結果は
        QERROR(DataStore, OperationNotAllowed) となります。メタ情報を変更するには DataStoreClient::ChangeMeta() を使用して下さい。
        更新権限を持っているデータのみ更新することが可能です。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[in]  buffer アップロードするデータ。param に対して DataStoreUpdateParam::SetSize() で指定したサイズ分がアップロードされます。
        @param[in]  useInternalBuffer true を指定した場合(デフォルト)、 buffer の内容はライブラリ内部にコピーして保持されるため呼び出し側では破棄して構いません。
                                      この場合、DataStoreUpdateParam::SetSize()で指定したサイズ分のメモリを内部に確保します。
                                      false を指定した場合、非同期処理が完了するまで buffer の内容を保持してください。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool UpdateObject(ProtocolCallContext *pContext, const DataStoreUpdateParam& param, const void* buffer, qBool useInternalBuffer=true)
    {
        return UpdateObjectImpl(pContext, param, NULL, static_cast<const qUnsignedInt8*>(buffer), useInternalBuffer);
    }

    /*!
        @brief      データの送受信に使用される HTTP 通信スレッドの優先度を設定します。

        HTTP 通信は DataStoreClient::GetObject() 、 DataStoreClient::PostObject() 、 DataStoreClient::UpdateObject() 呼び出しにより発生します。
        HTTP 通信を発生させる関数を呼び出した時点での設定が使用されます。
        そのため、 HTTP 通信を発生させる関数呼び出しごとに本関数を呼び出すことにより、セッションごとに優先度を変更することができます。
        本関数を呼び出さない場合の優先度は DataStoreConstants::DEFAULT_HTTP_THREAD_PRIORITY となります。

        @param[in]  priority HTTP 通信スレッドの優先度。
    */
    void SetHttpThreadPriority(qUnsignedInt32 priority)
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        m_HttpThreadPriority = priority;
    }

    /*!
        @brief      データの送受信に使用される HTTP 通信スレッドの優先度を取得します。

        @return     設定されている HTTP 通信スレッドの優先度。
    */
    qUnsignedInt32 GetHttpThreadPriority() const
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        return m_HttpThreadPriority;
    }

    /*!
        @internal
        @brief      データの送受信に HTTPS を使用するかどうかを指定します。

        データの送受信は DataStoreClient::GetObject() 、 DataStoreClient::PostObject() 、 DataStoreClient::UpdateObject() 呼び出しにより発生します。
        それ以外のサーバーとの通信はこの設定にかかわらず常に暗号化されます。
        HTTP 通信を発生させる関数を呼び出した時点での設定が使用されます。
        そのため、 HTTP 通信を発生させる関数呼び出しごとに本関数を呼び出すことにより、セッションごとに HTTPS/HTTP を切り替えることができます。
        本関数を呼び出さない場合は HTTPS が使用されます。

        @param[in]  useHttps HTTPS を使用する場合は true。HTTPを使用する場合は false。
    */
    void SetUseHttps(qBool useHttps)
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        m_UseHttps = useHttps;
    }

    /*!
        @internal
        @brief      データの送受信に HTTPS を使用するかどうかを取得します。

        @return     HTTPS を使用する場合は true。HTTP を使用する場合は false。
    */
    qBool GetUseHttps() const
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        return m_UseHttps;
    }

    /*!
        @brief      データを受け渡す際の最大サイズを設定します。

        DataStoreGetObjectEventListener::ProcessResponse() 及び DataStorePostObjectEventListener::PostChunkedBuffer()
        により渡されるバッファサイズの最大値を設定します。
        HTTP 通信を発生させる関数を呼び出した時点での設定が使用されます。
        そのため、 HTTP 通信を発生させる関数呼び出しごとに本関数を呼び出すことにより、セッションごとにサイズを切り替えることができます。
        本関数を呼び出さない場合のサイズは DataStoreConstants::DEFAULT_RELAY_BUFFER_SIZE です。
        DataStoreClient::SetHttpBufferSize() で設定する HTTP 用のバッファサイズ以下の値を指定してください。
        HTTP 用のバッファサイズより大きな値を指定した場合は HTTP 用バッファサイズの値がデータを受け渡す際の最大値となります。

        @param[in]  relayBufferSize データを受け渡す際の最大サイズ。
    */
    void SetRelayBufferSize(size_t relayBufferSize)
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        m_RelayBufferSize = relayBufferSize;
    }

    /*!
        @brief      データを受け渡す際の最大サイズを取得します。

        @return     データを受け渡す際の最大サイズ。
    */
    size_t GetRelayBufferSize() const
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        return m_RelayBufferSize;
    }

    /*!
        @brief      データの送受信で使用される HTTP 通信用に DataStore ライブラリが確保するバッファサイズを設定します。

        データの送受信は DataStoreClient::GetObject() 、 DataStoreClient::PostObject() 、 DataStoreClient::UpdateObject() 呼び出しにより発生します。
         HTTP 通信の際はダブルバッファリングが行われるためここで設定したサイズの二倍のメモリが確保されます。 
        HTTP 通信を発生させる関数を呼び出した時点での設定が使用されます。
        そのため、 HTTP 通信を発生させる関数呼び出しごとに本関数を呼び出すことにより、セッションごとにサイズを切り替えることができます。
        本関数を呼び出さない場合のサイズは DataStoreConstants::DEFAULT_HTTP_BUFFER_SIZE です。

        @param[in]  httpBufferSize HTTP 通信で使用されるバッファサイズ。
    */
    void SetHttpBufferSize(size_t httpBufferSize)
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        m_HttpBufferSize = httpBufferSize;
    }

    /*!
        @brief      HTTP 通信で使用するバッファサイズを取得します。

        @return     HTTP 通信で使用するバッファサイズ。
    */
    size_t GetHttpBufferSize() const
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        return m_HttpBufferSize;
    }

#ifdef NEX_SIGLO_FAMILY
    /*!
     * @brief ストレージサーバーへの HTTP 通信で使用するソケットの送信バッファサイズを指定します。
     *
     *  0 の場合は、 システム標準値が使われます。
     *  この値は、NXでのみ使用されます。Windows では使用されず自動設定されます。
     *
     * @see DataStoreConstants::DEFAULT_HTTP_SEND_SOCKET_BUFFER_SIZE
     */
#endif
    void SetHttpSendSocketBufferSize(size_t size)
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        m_HttpSendSocketBufferSize = size;
    }

#ifdef NEX_SIGLO_FAMILY
    /*!
     * @brief ストレージサーバーへの HTTP 通信で使用するソケットの送信バッファサイズを取得します。
     *
     *  0 の場合は、 システム標準値が使われます。
     *
     * @see DataStoreConstants::DEFAULT_HTTP_SEND_SOCKET_BUFFER_SIZE
     */
#endif
    size_t GetHttpSendSocketBufferSize() const
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        return m_HttpSendSocketBufferSize;
    }

#ifdef NEX_SIGLO_FAMILY
    /*!
     * @brief ストレージサーバーへの HTTP 通信で使用するソケットの受信バッファサイズを指定します。
     *
     *  0 の場合は、 システム標準値が使われます。
     *  この値は、NXでのみ使用されます。Windows では使用されず自動設定されます。
     *
     * @see DataStoreConstants::DEFAULT_HTTP_RECV_SOCKET_BUFFER_SIZE
     */
#endif
    void SetHttpRecvSocketBufferSize(size_t size)
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        m_HttpRecvSocketBufferSize = size;
    }

#ifdef NEX_SIGLO_FAMILY
    /*!
     * @brief ストレージサーバーへの HTTP 通信で使用するソケットの受信バッファサイズを取得します。
     *
     *  0 の場合は、 システム標準値が使われます。
     *
     * @see DataStoreConstants::DEFAULT_HTTP_RECV_SOCKET_BUFFER_SIZE
     */
#endif
    size_t GetHttpRecvSocketBufferSize() const
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        return m_HttpRecvSocketBufferSize;
    }



    /*!
        @brief      データの送受信時のタイムアウト値を設定します。

        データの送受信は DataStoreClient::GetObject() 、 DataStoreClient::PostObject() 、 DataStoreClient::UpdateObject() 呼び出しにより発生します。
        このときに適用するタイムアウト値を設定します。タイムアウト値(ミリ秒)は下記の計算によって設定されます。

        std::max(<送受信するデータサイズ(byte)> / timeoutBytesPerSecond * 1000, minimumTimeoutMilliSecond);

        本関数を使用しない場合(未設定の場合)は timeoutBytesPerSecond に DataStoreConstants::DEFAULT_DATA_TRANSFER_TIMEOUT_BYTES_PER_SECOND 、
        minimumTimeoutMilliSecond に DataStoreConstants::DEFAULT_DATA_TRANSFER_MINIMUM_TIMEOUT が設定されたタイムアウト値が適用されます。
        この初期設定では、例えば 100KB のデータの送受信が 10 分かかっても終わらない場合や、1byte のファイルが 60 秒たってもダウンロードできない場合にタイムアウトされます。
        タイムアウト値は本関数で設定した値と送受信するデータサイズにより決定されます。
        (一時的に通信速度が timeoutBytesPerSecond を下回ったときにタイムアウトするという処理は行いません。)
        HTTP 通信を発生させる関数呼び出しごとに本関数を呼び出すことにより、セッションごとにタイムアウト値を切り替えることができます。
        各 RMC を固定の時間でタイムアウトさせたい場合は CallContext::SetTimeout() を使用してください。

        @param[in]  timeoutBytesPerSecond タイムアウトの計算元となる最低通信速度。(バイト毎秒) 0を指定した場合はタイムアウト処理を行いません。
        @param[in]  minimumTimeoutMilliSecond 送受信するデータサイズによらず最低限待機する時間。(ミリ秒)
    */
    void SetDataTransferTimeout(qUnsignedInt32 timeoutBytesPerSecond, TimeInterval minimumTimeoutMilliSecond=DataStoreConstants::DEFAULT_DATA_TRANSFER_MINIMUM_TIMEOUT)
    {
        // ReceiveNewArrivedObjects()内部のJobから参照されるのでSystemLockが必要。
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        m_TimeoutBytesPerSecond = timeoutBytesPerSecond;
        m_MinimumTimeoutMilliSecond = minimumTimeoutMilliSecond;
    }

    /*!
        @internal
        @brief      更新が通知されているかを確認します。本関数を直接アプリケーションから呼び出さないでください。

        
        本関数呼び出し前に nn::http::Initialize() を呼び出し、HTTP ライブラリを初期化する必要があります。
        
        
        本関数を呼び出すと NEX のスレッドモードにかかわらず通信用スレッドが一つ作成されます。
        宛先が自分になっているデータが新しくアップロードされているかどうかを調べます。
        通知ファイルの整合性がとれていない場合は urlOut に DataStoreConstants::INVALID_NOTIFICATION_URL 、 notificationIdOut に 0 が格納されます。
        urlIn に空文字列もしくは DataStoreConstants::INVALID_NOTIFICATION_URL 、sandboxInstanceIdIn に DataStoreConstants::INVALID_SANDBOX_INSTANCEID 、
        forceRetrieveUrl を true にしたときは必ず新しいURLを取得しに行きます。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  urlIn 通知ファイルをチェックする URL 。初回利用時は空文字列を渡してください。サーバーで指定した値と正しい値とが比較され、異なっていれば 通知ファイルが設置されます。
        @param[out] urlOut 新しい通知 URL 。本 RMC が成功した場合、この値を毎回保存して次回本 API に渡してください。
                           ただし、 URL が DataStoreConstants::INVALID_NOTIFICATION_URL だった場合は通知ファイルの整合性がとれていない事を示します。
        @param[in]  notificationIdIn 前回の通知ID。
        @param[out] notificationIdOut 新しい通知ID。本 RMC が成功した場合、この値を毎回保存して次回本 API に渡してください。
                                      ただし、通知IDが0だった場合は通知ファイルの整合性がとれていない事を示します。
        @param[in]  sandboxInstanceIdIn 前回のsandboxInstanceId 。正しい Sandbox 向けの通知ファイルを取得したか確認します。
        @param[out] sandboxInstanceIdOut 通知ファイルの整合性を確認するための ID 。この値を毎回保存して次回本 API に渡してください。
        @param[in]  principalId 通知ファイルの整合性を確認するための プリンシパル ID  。通常自分の プリンシパル ID  を指定します。
        @param[out] isFailedRetrieveNotificationUrl 通知URLを取得しようとして失敗したかが格納されます。この値はRMCが失敗したときでも参照可能です。
        @param[in]  forceRetrieveUrl true を指定すると強制的にURLを再取得します。デフォルト値は false です。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool CheckNotification(ProtocolCallContext *pContext,
                                    const String& urlIn,
                                    String* urlOut,
                                    qUnsignedInt64 notificationIdIn,
                                    qUnsignedInt64* notificationIdOut,
                                    qUnsignedInt32 sandboxInstanceIdIn,
                                    qUnsignedInt32* sandboxInstanceIdOut,
                                    PrincipalID principalId,
                                    qBool* isFailedRetrieveNotificationUrl,
                                    qBool forceRetrieveUrl=false
                                   )
    {
        return CheckNotificationImpl(pContext, urlIn, urlOut, notificationIdIn, notificationIdOut,
                                     sandboxInstanceIdIn, sandboxInstanceIdOut, principalId, isFailedRetrieveNotificationUrl, forceRetrieveUrl);
    }

    /*!
        @internal
        @brief      更新が通知されているかを確認します。チェックトークンを指定して更新の有無のみを調べます。本関数を直接アプリケーションから呼び出さないでください。

        
        本関数呼び出し前に nn::http::Initialize() を呼び出し、HTTP ライブラリを初期化する必要があります。
        
        
        本関数を呼び出すと NEX のスレッドモードにかかわらず通信用スレッドが一つ作成されます。
        宛先が自分になっているデータが新しくアップロードされているかどうかを調べます。
        通知ファイルを確認し、ダウンロードするものが無いと判断された場合に needReceiving に false が格納されます。
        それ以外の場合、つまり以下の場合に true が格納されます。この場合次回 DataStoreClient::ReceiveNewArrivedObjects() を呼び出す必要があります。

        @li 新しく受信したものがあると確認した。
        @li チェックトークンが NULL。

        また、newArrivalCheckToken に NULL を指定し、新たにチェックトークンを取得した後は一度 DataStoreClient::ReceiveNewArrivedObjects() を呼び出すまで
        needReceiving は常に true となります。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  newArrivalCheckToken チェックトークン。NULL を指定したときはトークンの更新のみ行われ、データのダウンロードは発生しません。
        @param[in]  receiveNewArrivedObjectsEventListener チェックトークンが更新されたことを通知するイベントリスナー
        @param[out] pNeedReceiving 新しくデータを受信した可能性があるかどうか。可能性がある場合は true が格納され、
                    次に DataStoreClient::ReceiveNewArrivedObjectsを呼び出す必要があります。

        @retval     "QSUCCESS(Core, Success)" RMC の開始に成功。
        @retval     "QERROR(Core, CallInitiationFailure)" 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
        @retval     "QERROR(DataStore, InvalidCheckToken)" 指定したチェックトークンが破損しています。
                                                                新規にトークンを取得してください。
                                                                (本関数の newArrivalCheckToken パラメータに NULL を指定し取得してください。)
    */
    virtual qResult CheckNewArrival(ProtocolCallContext* pContext,
            const qByte newArrivalCheckToken[DataStoreConstants::NEW_ARRIVAL_CHECK_TOKEN_SIZE],
            DataStoreReceiveNewArrivedObjectsEventListener* receiveNewArrivedObjectsEventListener,
            qBool* pNeedReceiving
                           )
    {
        return CheckNewArrivalImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_LATEST> >(pContext, newArrivalCheckToken, receiveNewArrivedObjectsEventListener, pNeedReceiving);
    }

    /*!
        @internal
        @brief      更新が通知されているかを確認し、新着データがあればそれをダウンロードします。本関数を直接アプリケーションから呼び出さないでください。

        
        本関数呼び出し前に nn::http::Initialize() を呼び出し、HTTP ライブラリを初期化する必要があります。
        
        
        本関数を呼び出すと NEX のスレッドモードにかかわらず通信用スレッドが一つ作成されます。
        宛先が自分になっているデータが新しくアップロードされているかどうかを調べ、あればそのデータをダウンロードします。
        以前使っていたトークンを破棄し、再度 newArrivalCheckToken に NULL を指定した場合は前回本関数呼び出しでダウンロードしたものから再度ダウンロードが行われます。
        (今まで受信したものを全てダウンロードし直すことにはなりません。)

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  newArrivalCheckToken 前回 DataStoreReceiveNewArrivedObjectsEventListener::Updated() から返されたチェックトークン。初回利用時は NULL を指定してください。
                                         NULL を指定したときはトークンの更新のみ行われ、データのダウンロードは発生しません。
        @param[in]  getObjectEventListeners ダウンロードするデータを受信するためのイベントリスナー。
                                            ダウンロードデータ一つに対して一つのイベントリスナーがコンテナの要素の順に使用されます。
                                            最大で指定した要素数分のファイルがダウンロードされます。
                                            各イベントリスナーは同一の実体を指すポインタを複数使用することもできます。
                                            各データは並列にはダウンロードされず、一つずつダウンロードされます。
        @param[in]  receiveNewArrivedObjectsEventListener チェックトークンが更新されたことを通知するイベントリスナー
        @param[out] hasNext まだダウンロードすべき新着データがある場合にtrueがセットされます。
                            true がセットされた場合、処理時間などに余裕があればもう一度本 API を呼び出してください。
                            指定した getObjectEventListeners の個数より少ない回数のダウンロードしか発生しなかった場合でも
                            このパラメータが true になることがあります。

        @retval     "QSUCCESS(Core, Success)" RMC の開始に成功。
        @retval     "QERROR(Core, CallInitiationFailure)" 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
        @retval     "QERROR(DataStore, InvalidCheckToken)" 指定したチェックトークンが破損しています。
                                                                新規にトークンを取得してください。
                                                                (本関数の newArrivalCheckToken パラメータに NULL を指定し取得してください。)
    */
    virtual qResult ReceiveNewArrivedObjects(ProtocolCallContext* pContext,
            const qByte newArrivalCheckToken[DataStoreConstants::NEW_ARRIVAL_CHECK_TOKEN_SIZE],
            qVector<DataStoreGetObjectEventListener*> getObjectEventListeners,
            DataStoreReceiveNewArrivedObjectsEventListener* receiveNewArrivedObjectsEventListener,
            qBool* hasNext
                                    )
    {
        return ReceiveNewArrivedObjectsImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_LATEST> >(pContext, newArrivalCheckToken, getObjectEventListeners,
                receiveNewArrivedObjectsEventListener, hasNext);
    }

    /*!
        @internal
        @brief      更新が通知されているかを確認し、新着データがあればそれをダウンロードします。古いインタフェースの互換性維持用。本関数を直接アプリケーションから呼び出さないでください。

        
        本関数呼び出し前に nn::http::Initialize() を呼び出し、HTTP ライブラリを初期化する必要があります。
        
        
        本関数を呼び出すと NEX のスレッドモードにかかわらず通信用スレッドが一つ作成されます。
        宛先が自分になっているデータが新しくアップロードされているかどうかを調べ、あればそのデータをダウンロードします。
        以前使っていたトークンを破棄し、再度 newArrivalCheckToken に NULL を指定した場合は前回本関数呼び出しでダウンロードしたものから再度ダウンロードが行われます。
        (今まで受信したものを全てダウンロードし直すことにはなりません。)

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  newArrivalCheckToken 前回 DataStoreReceiveNewArrivedObjectsEventListener::Updated() から返されたチェックトークン。初回利用時は NULL を指定してください。
                                         NULL を指定したときはトークンの更新のみ行われ、データのダウンロードは発生しません。
        @param[in]  getObjectEventListeners ダウンロードするデータを受信するためのイベントリスナー。
                                            ダウンロードデータ一つに対して一つのイベントリスナーがコンテナの要素の順に使用されます。
                                            最大で指定した要素数分のファイルがダウンロードされます。
                                            各イベントリスナーは同一の実体を指すポインタを複数使用することもできます。
                                            各データは並列にはダウンロードされず、一つずつダウンロードされます。
        @param[in]  receiveNewArrivedObjectsEventListener チェックトークンが更新されたことを通知するイベントリスナー
        @param[out] hasNext まだダウンロードすべき新着データがある場合にtrueがセットされます。
                            true がセットされた場合、処理時間などに余裕があればもう一度本 API を呼び出してください。
                            指定した getObjectEventListeners の個数より少ない回数のダウンロードしか発生しなかった場合でも
                            このパラメータが true になることがあります。

        @retval     "QSUCCESS(Core, Success)" RMC の開始に成功。
        @retval     "QERROR(Core, CallInitiationFailure)" 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
        @retval     "QERROR(DataStore, InvalidCheckToken)" 指定したチェックトークンが破損しています。
                                                                新規にトークンを取得してください。
                                                                (本関数の newArrivalCheckToken パラメータに NULL を指定し取得してください。)
    */
    virtual qResult ReceiveNewArrivedObjectsV1(ProtocolCallContext* pContext,
            const qByte newArrivalCheckToken[DataStoreConstants::NEW_ARRIVAL_CHECK_TOKEN_SIZE],
            qVector<DataStoreGetObjectEventListenerV1*> getObjectEventListeners,
            DataStoreReceiveNewArrivedObjectsEventListenerV1* receiveNewArrivedObjectsEventListener,
            qBool* hasNext
                                      )
    {
        return ReceiveNewArrivedObjectsImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_1> >(pContext, newArrivalCheckToken, getObjectEventListeners,
                receiveNewArrivedObjectsEventListener, hasNext);
    }

    /*!
        @brief      データの評価情報を取得します

        データIDとスロットを指定し、1個の評価情報を取得します。
        対象のデータの参照権限を持っている人のみが取得できます。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  target 評価情報を取得する対象。
        @param[out] pRating 評価情報。
        @param[in]  accessPassword 参照パスワード。デフォルトは DataStoreConstants::INVALID_PASSWORD となっておりパスワードによる認証を使用しません。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool GetRating(ProtocolCallContext* pContext, const DataStoreRatingTarget& target, DataStoreRatingInfo* pRating, qUnsignedInt64 accessPassword=DataStoreConstants::INVALID_PASSWORD)
    {
        return TLogicServerClient::GetRating(pContext, target, pRating, accessPassword);
    }

    /*!
        @internal
        @brief      データの評価情報を取得します

        データIDのリストを指定し、複数の評価値をまとめて取得します。
        対象のデータの参照権限を持っている人のみが取得できます。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  dataIds 評価情報を取得するデータIDのリスト。
        @param[out] pRatings 評価情報。
        @param[out] pResults 成功したかどうかの結果。 (省略可)
        @param[in]  accessPassword 参照パスワード。デフォルトは DataStoreConstants::INVALID_PASSWORD となっておりパスワードによる認証を使用しません。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool GetRating(ProtocolCallContext* pContext, const qVector<qUnsignedInt64>& dataIds, qVector< qVector< DataStoreRatingInfoWithSlot > > *pRatings, qVector<qResult>* pResults=NULL, qUnsignedInt64 accessPassword=DataStoreConstants::INVALID_PASSWORD)
    {
        return TLogicServerClient::GetRating(pContext, dataIds, pRatings, pResults, accessPassword);
    }

    /*!
        @brief      データの評価情報を取得します

        データIDを指定し、全てのスロットの評価情報を取得します。
        対象のデータの参照権限を持っている人のみが取得できます。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  dataId 評価情報を取得するデータID。
        @param[out] pRatings 評価情報。
        @param[in]  accessPassword 参照パスワード。デフォルトは DataStoreConstants::INVALID_PASSWORD となっておりパスワードによる認証を使用しません。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool GetRating(ProtocolCallContext* pContext, const qUnsignedInt64 dataId, qMap<qInt8, DataStoreRatingInfo> *pRatings, qUnsignedInt64 accessPassword=DataStoreConstants::INVALID_PASSWORD)
    {
        return GetRatingImpl(pContext, dataId, pRatings, accessPassword);
    }

    /*!
        @brief      データの評価情報を一括取得します

        データIDのリストを指定し、複数の評価情報をまとめて取得します。
        pRatingsにはdataIdsで指定したデータIDに対する処理の結果が指定した数だけ同順で返ります。
        対象のデータの参照権限を持っている人のみが取得できます。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  dataIds 評価情報を取得するデータIDのリスト。
        @param[out] pRatings 評価情報。
        @param[out] pResults 成功したかどうかの結果 (省略可)

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool GetRating(ProtocolCallContext* pContext, const qVector<qUnsignedInt64>& dataIds, qVector< qMap<qInt8, DataStoreRatingInfo> > *pRatings, qVector<qResult>* pResults=NULL)
    {
        return GetRatingImpl(pContext, dataIds, pRatings, pResults);
    }

    /*!
        @brief      データの評価情報と評価ログを取得します

        データIDとスロットを指定し、1個の評価情報と評価ログを取得します。
        本関数は評価スロットの初期化設定の際に DataStoreConstants::RATING_FLAG_MODIFIABLE を指定したものもしくは、
        重複ロックを設定したデータのみに使用できます。(両方を指定している場合でも使用できます。)
        これらが指定されていなかった場合の結果は QERROR(DataStore, OperationNotAllowed) となります。
        対象のデータの参照権限を持っている人のみが取得できます。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  target 評価情報を取得する対象。
        @param[out] pRating 評価情報。
        @param[out] pRatingLog 評価ログ。
        @param[in]  accessPassword 参照パスワード。デフォルトは DataStoreConstants::INVALID_PASSWORD となっておりパスワードによる認証を使用しません。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool GetRating(ProtocolCallContext* pContext, const DataStoreRatingTarget& target, DataStoreRatingInfo* pRating, DataStoreRatingLog* pRatingLog, qUnsignedInt64 accessPassword=DataStoreConstants::INVALID_PASSWORD)
    {
        return TLogicServerClient::GetRatingWithLog(pContext, target, pRating, pRatingLog, accessPassword);
    }

    /*!
        @internal
        @brief      ストレージのデータをファイルサーバーから取得します。

        
        本関数呼び出し前に nn::http::Initialize() を呼び出し、HTTP ライブラリを初期化する必要があります。
        
        
        本関数を呼び出すと NEX のスレッドモードにかかわらず通信用スレッドが一つ作成されます。
        ダウンロードしたデータは指定した eventListener からアプリケーションに渡されます。
        ダウンロードするデータを保存するメモリを一度に確保できない場合などに使用してください。
        DataStoreClient::GetObjectInfo() で取得した情報を指定し、ストレージのデータをファイルサーバーから取得します。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  info ストレージからデータを取得するための情報
        @param[in]  pEventListener ダウンロードされたデータを受け取るリスナーオブジェクト

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool GetObjectByInfo(ProtocolCallContext* pContext, const DataStoreReqGetInfo& info, DataStoreGetObjectEventListener* pEventListener)
    {
        return GetObjectByInfoImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_LATEST> >(pContext, info, pEventListener, NULL, 0, NULL);
    }

    /*!
        @internal
        @brief      ストレージのデータをファイルサーバーから取得します。

        
        本関数呼び出し前に nn::http::Initialize() を呼び出し、HTTP ライブラリを初期化する必要があります。
        
        
        本関数を呼び出すと NEX のスレッドモードにかかわらず通信用スレッドが一つ作成されます。
        ダウンロードしたデータは buffer に書き込まれます。非同期処理が成功したときのみ buffer からデータを読み出すことが可能です。
        DataStoreClient::GetObjectInfo() で取得した情報を指定し、ストレージのデータをファイルサーバーから取得します。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  info ストレージからデータを取得するための情報
        @param[out]  buffer ダウンロードしたデータを書き込むバッファ
        @param[in]  bufferSize buffer のサイズ。ダウンロードしたいデータのサイズ以上の値を指定してください
        @param[out]  pWrittenSize ダウンロードしたデータのサイズを格納するポインタ。NULLを指定すると格納されません

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    virtual qBool GetObjectByInfo(ProtocolCallContext* pContext, const DataStoreReqGetInfo& info, void* buffer, size_t bufferSize, size_t* pWrittenSize)
    {
        return GetObjectByInfoImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_LATEST> >(pContext, info, NULL, static_cast<qUnsignedInt8*>(buffer), bufferSize, pWrittenSize);
    }

    /*!
        @brief      DataStoreClientオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, NEX_T("DataStoreClient: (")
               << NEX_T("m_HttpThreadPriority=") << m_HttpThreadPriority
               << NEX_T(", m_UseHttps=") << m_UseHttps
               << NEX_T(", m_TimeoutBytesPerSecond=") << m_TimeoutBytesPerSecond
               << NEX_T(", m_MinimumTimeoutMilliSecond=") << m_MinimumTimeoutMilliSecond
               << NEX_T(", m_RelayBufferSize=") << m_RelayBufferSize
               << NEX_T(", m_HttpBufferSize=") << m_HttpBufferSize
               << NEX_T(", m_HttpSendSocketBufferSize=") << m_HttpSendSocketBufferSize
               << NEX_T(", m_HttpRecvSocketBufferSize=") << m_HttpRecvSocketBufferSize
               << NEX_T(")")
              );
    }

protected:
    qBool SetDependentRvConnection(ProtocolCallContext* pContext) const;
    template<class TVersionTraits>
    qBool GetObjectImpl(ProtocolCallContext *pContext, const typename TVersionTraits::TPrepareGetParam& param,
                        typename TVersionTraits::TGetObjectEventListener* eventListener,
                        qUnsignedInt8* buffer, size_t bufferSize, qBool getMetaBinary, size_t* pWrittenSize);
    template<class TVersionTraits>
    qBool PostObjectImpl(ProtocolCallContext *pContext, const typename TVersionTraits::TPreparePostParam& param,
                         typename TVersionTraits::TDataId* dataId, DataStorePostObjectEventListener* eventListener,
                         const qUnsignedInt8* buffer, qBool useInternalBuffer);
    qBool UpdateObjectImpl(ProtocolCallContext *pContext, const DataStorePrepareUpdateParam& param,
                           DataStorePostObjectEventListener* eventListener, const qUnsignedInt8* buffer, qBool useInternalBuffer);
    qBool CheckNotificationImpl(ProtocolCallContext *pContext,
                                const String& urlIn,
                                String* urlOut,
                                qUnsignedInt64 notificationIdIn,
                                qUnsignedInt64* notificationIdOut,
                                qUnsignedInt32 sandboxInstanceIdIn,
                                qUnsignedInt32* sandboxInstanceIdOut,
                                PrincipalID principalId,
                                qBool* isFailedRetrieveNotificationUrl,
                                qBool forceRetrieveUrl=false
                               );
    template<class TVersionTraits>
    qResult CheckNewArrivalImpl(ProtocolCallContext* pContext,
                            const qByte newArrivalCheckToken[DataStoreConstants::NEW_ARRIVAL_CHECK_TOKEN_SIZE],
                            DataStoreReceiveNewArrivedObjectsEventListener* receiveNewArrivedObjectsEventListener,
                            qBool* pNeedReceiving
                           );
    template<class TVersionTraits>
    qResult ReceiveNewArrivedObjectsImpl(ProtocolCallContext* pContext,
                                         const qByte newArrivalCheckToken[DataStoreConstants::NEW_ARRIVAL_CHECK_TOKEN_SIZE],
                                         qVector<typename TVersionTraits::TGetObjectEventListener*> getObjectEventListeners,
                                         typename TVersionTraits::TReceiveNewArrivedObjectsEventListener* receiveNewArrivedObjectsEventListener,
                                         qBool* hasNext
                                        );
    qBool GetRatingImpl(ProtocolCallContext* pContext, const qUnsignedInt64 dataId, qMap<qInt8, DataStoreRatingInfo> *pRatings, qUnsignedInt64 accessPassword);
    qBool GetRatingImpl(ProtocolCallContext* pContext, const qVector<qUnsignedInt64>& dataIds, qVector< qMap<qInt8, DataStoreRatingInfo> > *pRatings, qVector<qResult>* pResults);
    template<class TVersionTraits>
    qBool GetObjectByInfoImpl(ProtocolCallContext* pContext,
                                  const DataStoreReqGetInfo& info,
                                  typename TVersionTraits::TGetObjectEventListener* pEventListener,
                                  qUnsignedInt8* buffer,
                                  size_t bufferSize,
                                  size_t* pWrittenSize);

    qUnsignedInt32 m_HttpThreadPriority;
    qBool m_UseHttps;
    qByte padding1[3];
    qUnsignedInt32 m_TimeoutBytesPerSecond;     // ファイルサーバーとのデータ通信に対しての速度によるタイムアウト設定
    TimeInterval m_MinimumTimeoutMilliSecond;   // ファイルサーバーとのデータ通信に対しての最小タイムアウト時間設定
    size_t m_RelayBufferSize;           // HTTP通信のリレーに使われるバッファサイズ
    size_t m_HttpBufferSize;            // HTTP通信用バッファサイズ
    size_t m_HttpSendSocketBufferSize;  // HTTP通信送信用ソケットバッファサイズ
    size_t m_HttpRecvSocketBufferSize;  // HTTP通信受信用ソケットバッファサイズ

};

typedef DataStoreClientTemplate<DataStoreLogicServerClient> DataStoreClient;

END_NAMESPACE_NEX

// GetObjectのdefine回避終了 #29450
#ifdef WIN32_TARGET
#ifdef NEX_PREVIOUS_GETOBJECT_DEFINED
#ifdef UNICODE
#define GetObject GetObjectW
#else
#define GetObject GetObjectA
#endif
#undef NEX_PREVIOUS_GETOBJECT_DEFINED
#endif
#endif

