﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are 
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreChangeMetaCompareParamDDL.h"
#include "DataStoreConstants.h"
#include "DataStoreMetaInfo.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      メタ情報をコンペアアンドスワップで変更時、比較対象にするパラメータを表すクラスです。
*/
class DataStoreChangeMetaCompareParam: public DDLCLASS(DataStoreChangeMetaCompareParam) {
public:
    /*!
        @brief      コンストラクタです。

        コンペアアンドスワップが行われない設定で生成します。コンペアアンドスワップを行うには、比較対象とするフラグと値を設定する必要があります。
    */
    DataStoreChangeMetaCompareParam()
    {
        Reset();
    }

    /*!
        @brief      コンストラクタです。

        メタデータからフラグで指定した値を参照し、比較対象とする設定で生成します。

        @param[in]  comparisonFlag 参照する値を設定するビットフラグ。DataStoreConstants::ComparisonFlag に対するビット単位の論理和です。すべて参照する場合は DataStoreConstants::COMPARISON_FLAG_ALL を指定します。
        @param[in]  metaInfo       参照元となるメタデータ
    */
    DataStoreChangeMetaCompareParam(qUnsignedInt32 comparisonFlag, const DataStoreMetaInfo& metaInfo)
    {
        Set(comparisonFlag, metaInfo);
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreChangeMetaCompareParam(){}

    /*!
        @brief      メタデータから値を参照し、設定します。

        メタデータからフラグで指定した値を参照し、設定します。
        この関数では、設定前にすべての値を初期化します。これは、フラグが設定されていないものも対象です。

        @param[in]  comparisonFlag_ 参照する値を設定するビットフラグ。DataStoreConstants::ComparisonFlag に対するビット単位の論理和です。すべて参照する場合は DataStoreConstants::COMPARISON_FLAG_ALL を指定します。
        @param[in]  metaInfo_       参照元となるメタデータ
    */
    void Set(qUnsignedInt32 comparisonFlag_, const DataStoreMetaInfo& metaInfo_)
    {
        Reset();
        SetComparisonFlag(comparisonFlag_);
        if (comparisonFlag_ & DataStoreConstants::COMPARISON_FLAG_NAME)
        {
            SetName(metaInfo_.GetName());
        }
        if (comparisonFlag_ & DataStoreConstants::COMPARISON_FLAG_ACCESS_PERMISSION)
        {
            SetAccessPermission(metaInfo_.GetAccessPermission());
        }
        if (comparisonFlag_ & DataStoreConstants::COMPARISON_FLAG_UPDATE_PERMISSION)
        {
            SetUpdatePermission(metaInfo_.GetUpdatePermission());
        }
        if (comparisonFlag_ & DataStoreConstants::COMPARISON_FLAG_PERIOD)
        {
            SetPeriod(metaInfo_.GetPeriod());
        }
        if (comparisonFlag_ & DataStoreConstants::COMPARISON_FLAG_METABINARY)
        {
            qVector<qByte> bin = metaInfo_.GetMetaBinary();
            if (!bin.empty())
            {
                SetMetaBinary(&bin[0], bin.size());
            }
        }
        if (comparisonFlag_ & DataStoreConstants::COMPARISON_FLAG_TAGS)
        {
            SetTags(metaInfo_.GetTags());
        }
        if (comparisonFlag_ & DataStoreConstants::COMPARISON_FLAG_DATA_TYPE)
        {
            SetDataType(metaInfo_.GetDataType());
        }
        if (comparisonFlag_ & DataStoreConstants::COMPARISON_FLAG_REFERRED_CNT)
        {
            SetReferredCount(metaInfo_.GetReferredCount());
        }
        if (comparisonFlag_ & DataStoreConstants::COMPARISON_FLAG_STATUS)
        {
            SetDataStatus(metaInfo_.GetDataStatus());
        }
    }
    
    /*!
        @brief      比較フラグをセットします。

        メタデータを比較する箇所を表すフラグを指定します。本パラメータは必ず指定する必要があります。
        DataStoreConstants::ComparisonFlag に対してビット単位の論理和を用いて複数指定することができます。

        @param[in]  comparisonFlag_ メタデータを比較する箇所を表すビットフラグ。 DataStoreConstants::ComparisonFlag に対するビット単位の論理和です。
    */
    void SetComparisonFlag(qUnsignedInt32 comparisonFlag_) { this->comparisonFlag = comparisonFlag_; }
    
    /*!
        @brief      比較フラグを取得します。

        @return     メタデータを比較する箇所を表すビットフラグ。 DataStoreConstants::ComparisonFlag に対するビット単位の論理和です。
    */
    qUnsignedInt32 GetComparisonFlag() const { return this->comparisonFlag; }
    
    /*!
        @brief      タイトルをセットします

        タイトル(要約情報)を指定します。本パラメータの指定は必須ではありません。デフォルトは空文字列です。

        @param[in]  name_ タイトル(要約情報)。
    */
    void SetName(const String& name_)
    {
#ifdef NEX_LOG_ENABLED
        qInt32 length = name_.GetWideCharLength();
        if (length > DataStoreConstants::MAX_NAME_LENGTH)
        {
            QLOG(EventLog::Warning, NEX_T("DataStoreChangeMetaCompareParam::SetName(): name length is over the limit. ") << NEX_T("name=") << name_ << NEX_T(", length=") << length);
        }
#endif
        this->name = name_;
    }
    
    /*!
        @brief      タイトルを取得します。

        @return     タイトル。
    */
    const String& GetName() const { return this->name; }
    
    /*!
        @brief      参照権限をセットします。

        参照権限を指定します。本パラメータの指定は必須ではありません。デフォルトは DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE) です。

        @param[in]  permission_ 参照権限。
    */
    void SetAccessPermission(const DataStorePermission& permission_) { this->permission = permission_; }
    
    /*!
        @brief      参照権限を取得します。

        @return     参照権限。
    */
    const DataStorePermission& GetAccessPermission() const { return this->permission; }
    
    /*!
        @brief      更新権限をセットします。

        更新権限を指定します。本パラメータの指定は必須ではありません。デフォルトは DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE) です。

        @param[in]  updatePermission 更新権限。
    */
    void SetUpdatePermission(const DataStorePermission& updatePermission) { this->delPermission = updatePermission; }
    
    /*!
        @brief      更新権限を取得します。

        @return     更新権限。
    */
    const DataStorePermission& GetUpdatePermission() const { return this->delPermission; }
    
    /*!
        @brief      有効日数をセットします。

        有効日数を指定します。本パラメータの指定は必須ではありません。デフォルトは DataStoreConstants::DEFAULT_PERIOD です。

        @param[in]  period_ 有効日数。
    */
    void SetPeriod(qUnsignedInt16 period_) { this->period = period_; }
    
    /*!
        @brief      有効日数を取得します。

        @return     有効日数。
    */
    qUnsignedInt16 GetPeriod() const { return this->period; }
    
    /*!
        @brief      タグをセットします

        アップロードするデータにメタ情報として付加するタグを指定します。空文字列のタグは指定できません。
        既にタグが設定されてる場合は、既存のタグは削除され、本関数で指定したタグで上書きされます。
        本パラメータの指定は必須ではありません。デフォルトは空です。

        @param[in]  tags_ アップロードするデータにメタ情報として付加するタグ。
    */
    void SetTags(const qVector<String>& tags_)
    {
#ifdef NEX_LOG_ENABLED
        if (tags_.size() > DataStoreConstants::NUM_TAG_SLOT)
        {
            QLOG(EventLog::Warning, NEX_T("DataStoreChangeMetaCompareParam::SetTags(): slot size is over the limit. size=") << tags_.size());
        }

        qVector<String>::const_iterator it = tags_.begin();
        for (; it != tags_.end(); ++it)
        {
            qInt32 length = it->GetWideCharLength();
            if (length > DataStoreConstants::MAX_TAG_LENGTH)
            {
                QLOG(EventLog::Warning, NEX_T("DataStoreChangeMetaCompareParam::SetTags(): tag length is over the limit. ") << NEX_T("tag=") << *it << NEX_T(", length=") << length);
            }
        }
#endif
        this->tags = tags_;
    }
    
    /*!
        @brief      タグを取得します。

        @return     タグ。
    */
    const qVector<String>& GetTags() const { return this->tags; }

    /*!
        @brief      メタ情報内バイナリデータをセットします。

        アップロードするデータにメタ情報として付加するバイナリデータを指定します。
        0-1024byteまで設定できます。本パラメータの指定は必須ではありません。デフォルトは空です。

        @param[in]  p アップロードするデータにメタ情報として付加するバイナリデータのポインタ。NULLを指定すると空になります。
        @param[in]  size アップロードするバイナリデータのデータサイズ(0-1024)。
    */
    void SetMetaBinary(const void* p, size_t size) {
        this->metaBinary.clear();
        if (p != NULL) {
            const qByte* pb = static_cast<const qByte*>(p);
            this->metaBinary.push_back(pb, size);
        }
    }

    /*!
        @brief      メタ情報内バイナリデータを取得します。

        @return     メタ情報内バイナリデータ
    */
    qVector<qByte> GetMetaBinary() const {
        // qBuffer -> qVector<qByte>への変換
        qVector<qByte> vb;
        vb.assign(this->metaBinary.begin(), this->metaBinary.end()); 
        return vb;
    }

    /*!
        @brief      メタ情報内バイナリデータを取得します。

        こちらのオーバーロード関数の方が効率的です。

        @param[out]  metaBinary_ メタ情報内バイナリデータ。
    */
    void GetMetaBinary(qVector<qByte>& metaBinary_) const {
        // qBuffer -> qVector<qByte>への変換
        metaBinary_.assign(this->metaBinary.begin(), this->metaBinary.end());
    }

    /*!
        @brief      データタイプを設定します。

        @param[in]  dataType_ データタイプ。
    */
    void SetDataType(qUnsignedInt16 dataType_) { this->dataType = dataType_; }

    /*!
        @brief      データタイプを取得します。

        @return     データタイプ。
    */
    qUnsignedInt16 GetDataType() const { return this->dataType; }

    /*!
        @brief      ステータスを設定します。

        @param[in]  status_   ステータス。
    */
    void SetDataStatus(DataStoreConstants::DataStatus status_) { this->status = static_cast<qByte>(status_); }

    /*!
        @brief      ステータスを取得します。

        @return     ステータス。
    */
    DataStoreConstants::DataStatus GetDataStatus() const { return static_cast<DataStoreConstants::DataStatus>(this->status); }

    /*!
        @brief      ダウンロードされた回数を設定します。廃止されるため使用できません。

        @param[in]  referredCount   ダウンロードされた回数。
    */
    void SetReferredCount(qUnsignedInt32 referredCount) { this->referredCnt = referredCount; }

    /*!
        @brief      ダウンロードされた回数を取得します。廃止されるため使用できません。

        @return     ダウンロードされた回数。
    */
    qUnsignedInt32 GetReferredCount() const { return this->referredCnt; }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetComparisonFlag(DataStoreConstants::COMPARISON_FLAG_NONE);
        SetName(NEX_T(""));
        SetAccessPermission(DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE));
        SetUpdatePermission(DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE));
        SetPeriod(DataStoreConstants::DEFAULT_PERIOD);
        SetTags(qVector<String>());
        SetMetaBinary(NULL, 0);
        SetDataType(0);
        SetDataStatus(DataStoreConstants::DATA_STATUS_NONE);
        SetReferredCount(0);
    }

    /*!
        @brief      DataStoreChangeMetaCompareParamオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        StringStream ss;
        ss << NEX_T("DataStoreChangeMetaCompareParam: (")
           << NEX_T(", comparisonFlag=") << comparisonFlag
           << NEX_T(", name=") << name
           << NEX_T(", accessPermission=") << permission
           << NEX_T(", updatePermission=") << delPermission
           << NEX_T(", period=") << period
           << NEX_T(", dataType=") << dataType
           << NEX_T(", referredCnt=") << referredCnt
           << NEX_T(", status=") << status;

        ss << NEX_T(", tags=(");
        for (qVector<String>::const_iterator it = tags.begin(); it != tags.end(); ++it)
        {
            ss << *it << NEX_T(", ");
        }
        ss << NEX_T(")");

        ss << NEX_T(", metaBinary=");
        for (qBuffer::const_iterator it = metaBinary.begin(); it != metaBinary.end(); ++it)
        {
           ss << hex << *it;
        }

        ss << NEX_T(")");

        QTRACE(uiTraceFlags, ss.CStr());
    }
};

END_NAMESPACE_NEX

