﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreChangeMetaParamDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      メタ情報の変更に関するパラメータを表すクラスです。
*/
class DataStoreChangeMetaParam: public DDLCLASS(DataStoreChangeMetaParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    DataStoreChangeMetaParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreChangeMetaParam() {}

    /*!
        @brief      データIDをセットします。

        メタデータを変更するデータのデータIDを指定します。
        本パラメータを設定する場合は DataStoreChangeMetaParam::SetPersistenceTarget() で永続化対象(プリンシパルIDと永続化スロットID)を設定しないでください。

        @param[in]  dataId_ メタデータを変更するデータのデータID。
    */
    void SetDataId(qUnsignedInt64 dataId_)
    {
        SYSTEMCHECK(persistenceTarget == DataStorePersistenceTarget() && "PersistenceTarget is already set. Please specify either DataID or PersistenceTarget");
        this->dataId = dataId_;
    }

    /*!
        @brief      データIDを取得します。

        @return     データID。
    */
    qUnsignedInt64 GetDataId() const
    {
        return this->dataId;
    }

    /*!
        @brief      変更フラグをセットします。

        メタデータを変更する箇所を表すフラグを指定します。本パラメータは必ず指定する必要があります。
        DataStoreConstants::ModificationFlag に対してビット単位の論理和を用いて複数指定することができます。

        @param[in]  modificationFlag メタデータを変更する箇所を表すビットフラグ。 DataStoreConstants::ModificationFlag に対するビット単位の論理和です。
    */
    void SetModificationFlag(qUnsignedInt32 modificationFlag)
    {
        this->modifiesFlag = modificationFlag;
    }

    /*!
        @brief      変更フラグを取得します。

        @return     メタデータを変更する箇所を表すビットフラグ。 DataStoreConstants::ModificationFlag に対するビット単位の論理和です。
    */
    qUnsignedInt32 GetModificationFlag() const
    {
        return this->modifiesFlag;
    }

    /*!
        @brief      タイトルをセットします

        新しいタイトル(要約情報)を指定します。本パラメータの指定は必須ではありません。デフォルトは空文字列です。

        @param[in]  name_ 新しいタイトル(要約情報)。
    */
    void SetName(const String& name_)
    {
#ifdef NEX_LOG_ENABLED
        qInt32 length = name_.GetWideCharLength();
        if (length > DataStoreConstants::MAX_NAME_LENGTH)
        {
            QLOG(EventLog::Warning, NEX_T("DataStoreChangeMetaParam::SetName(): name length is over the limit. ") << NEX_T("name=") << name_ << NEX_T(", length=") << length);
        }
#endif
        this->name = name_;
    }

    /*!
        @brief      タイトルを取得します。

        @return     タイトル。
    */
    const String& GetName() const
    {
        return this->name;
    }

    /*!
        @brief      参照権限をセットします。

        新しい参照権限を指定します。本パラメータの指定は必須ではありません。デフォルトは DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE) です。

        @param[in]  permission_ 新しい参照権限。
    */
    void SetAccessPermission(const DataStorePermission& permission_)
    {
        this->permission = permission_;
    }

    /*!
        @brief      参照権限を取得します。

        @return     参照権限。
    */
    const DataStorePermission& GetAccessPermission() const
    {
        return this->permission;
    }

    /*!
        @brief      更新権限をセットします。推奨されません。

        これは互換性のために残されている古い関数です。 DataStoreChangeMetaParam::SetUpdatePermission() を使用して下さい。
        新しい更新権限を指定します。本パラメータの指定は必須ではありません。デフォルトは DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE) です。

        @param[in]  delPermission_ 新しい更新権限。
    */
    void SetDeletePermission(const DataStorePermission& delPermission_)
    {
        QLOG(EventLog::Warning, NEX_T("DataStoreChangeMetaParam::SetDeletePermission() is deprecated. Use DataStoreChangeMetaParam::SetUpdatePermission()"));
        this->delPermission = delPermission_;
    }

    /*!
        @brief      更新権限を取得します。推奨されません。

        これは互換性のために残されている古い関数です。 DataStoreChangeMetaParam::GetUpdatePermission() を使用して下さい。

       @return     更新権限。
    */
    const DataStorePermission& GetDeletePermission() const
    {
        QLOG(EventLog::Warning, NEX_T("DataStoreChangeMetaParam::GetDeletePermission() is deprecated. Use DataStoreChangeMetaParam::GetUpdatePermission()"));
        return this->delPermission;
    }

    /*!
        @brief      更新権限をセットします。

        新しい更新権限を指定します。本パラメータの指定は必須ではありません。デフォルトは DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE) です。

        @param[in]  updatePermission 新しい更新権限。
    */
    void SetUpdatePermission(const DataStorePermission& updatePermission)
    {
        this->delPermission = updatePermission;
    }

    /*!
        @brief      更新権限を取得します。

        @return     更新権限。
    */
    const DataStorePermission& GetUpdatePermission() const
    {
        return this->delPermission;
    }

    /*!
        @brief      有効日数をセットします。

        新しい有効日数を指定します。本パラメータの指定は必須ではありません。デフォルトは DataStoreConstants::DEFAULT_PERIOD です。

        @param[in]  period_ 新しい有効日数。
    */
    void SetPeriod(qUnsignedInt16 period_)
    {
        this->period = period_;
    }

    /*!
        @brief      有効日数を取得します。

        @return     有効日数。
    */
    qUnsignedInt16 GetPeriod() const
    {
        return this->period;
    }

    /*!
        @brief      タグをセットします

        アップロードするデータにメタ情報として付加するタグを指定します。空文字列のタグは指定できません。
        既にタグが設定されてる場合は、既存のタグは削除され、本関数で指定したタグで上書きされます。
        本パラメータの指定は必須ではありません。デフォルトは空です。

        @param[in]  tags_ アップロードするデータにメタ情報として付加するタグ。
    */
    void SetTags(const qVector<String>& tags_)
    {
#ifdef NEX_LOG_ENABLED
        if (tags_.size() > DataStoreConstants::NUM_TAG_SLOT)
        {
            QLOG(EventLog::Warning, NEX_T("DataStoreChangeMetaParam::SetTags(): slot size is over the limit. size=") << tags_.size());
        }

        qVector<String>::const_iterator it = tags_.begin();
        for (; it != tags_.end(); ++it)
        {
            qInt32 length = it->GetWideCharLength();
            if (length > DataStoreConstants::MAX_TAG_LENGTH)
            {
                QLOG(EventLog::Warning, NEX_T("DataStoreChangeMetaParam::SetTags(): tag length is over the limit. ") << NEX_T("tag=") << *it << NEX_T(", length=") << length);
            }
        }
#endif
        this->tags = tags_;
    }

    /*!
        @brief      タグを取得します。

        @return     タグ。
    */
    const qVector<String>& GetTags() const
    {
        return this->tags;
    }

    /*!
        @brief      メタ情報内バイナリデータをセットします。

        アップロードするデータにメタ情報として付加するバイナリデータを指定します。
        0-1024byteまで設定できます。本パラメータの指定は必須ではありません。デフォルトは空です。

        @param[in]  p アップロードするデータにメタ情報として付加するバイナリデータのポインタ。NULLを指定すると空になります。
        @param[in]  size アップロードするバイナリデータのデータサイズ(0-1024)。
    */
    void SetMetaBinary(const void* p, size_t size)
    {
        this->metaBinary.clear();
        if (p != NULL)
        {
            const qByte* pb = static_cast<const qByte*>(p);
            this->metaBinary.push_back(pb, size);
        }
    }

    /*!
        @brief      メタ情報内バイナリデータを取得します。

        @return     メタ情報内バイナリデータ
    */
    qVector<qByte> GetMetaBinary() const
    {
        // qBuffer -> qVector<qByte>への変換
        qVector<qByte> vb;
        vb.assign(this->metaBinary.begin(), this->metaBinary.end());
        return vb;
    }

    /*!
        @brief      メタ情報内バイナリデータを取得します。

        こちらのオーバーロード関数の方が効率的です。

        @param[out]  metaBinary_ メタ情報内バイナリデータ。
    */
    void GetMetaBinary(qVector<qByte>& metaBinary_) const
    {
        // qBuffer -> qVector<qByte>への変換
        metaBinary_.assign(this->metaBinary.begin(), this->metaBinary.end());
    }

    /*!
        @brief      更新パスワードを設定します。

        更新権限を持たないデータを操作する場合に更新パスワードを指定します。
        データのオーナーから受け取った更新パスワードを指定してください。
        更新権限を持っている場合は更新パスワードは無視され、常に操作可能です。
        DataStoreConstants::INVALID_PASSWORD を指定した場合は更新パスワードでの認証を行いません。
        本関数で指定しない場合は DataStoreConstants::INVALID_PASSWORD が使用されます。

        @param[in]  updatePassword_ 更新パスワード
    */
    void SetUpdatePassword(qUnsignedInt64 updatePassword_)
    {
        this->updatePassword = updatePassword_;
    }

    /*!
        @brief      更新パスワードを取得します。

        @return     更新パスワード。
    */
    qUnsignedInt64 GetUpdatePassword() const
    {
        return this->updatePassword;
    }

    /*!
        @brief      データタイプを設定します。

        @param[in]  dataType_ データタイプ。
    */
    void SetDataType(qUnsignedInt16 dataType_) { this->dataType = dataType_; }

    /*!
        @brief      データタイプを取得します。

        @return     データタイプ。
    */
    qUnsignedInt16 GetDataType() const { return this->dataType; }

    /*!
        @brief      ステータスを設定します。

        @param[in]  status_   ステータス。
    */
    void SetDataStatus(DataStoreConstants::DataStatus status_) { this->status = static_cast<qByte>(status_); }

    /*!
        @brief      ステータスを取得します。

        @return     ステータス。
    */
    DataStoreConstants::DataStatus GetDataStatus() const { return static_cast<DataStoreConstants::DataStatus>(this->status); }

    /*!
        @brief      ダウンロードされた回数を設定します。廃止されるため使用できません。

        @param[in]  referredCount   ダウンロードされた回数。
    */
    void SetReferredCount(qUnsignedInt32 referredCount) { this->referredCnt = referredCount; }

    /*!
        @brief      ダウンロードされた回数を取得します。廃止されるため使用できません。

        @return     ダウンロードされた回数。
    */
    qUnsignedInt32 GetReferredCount() const { return this->referredCnt; }

    /*!
        @brief      コンペアアンドスワップで変更するよう、比較用期待値を設定します。

        @param[in]  compareParam_ 比較用期待値

        コンペアアンドスワップで変更する場合、変更前に、設定した期待値とデータの値を比較します。
        この時、期待値とデータの値が等しい場合はメタ情報が変更され、異なる場合は変更されません。
        本パラメータの指定は必須ではありません。デフォルトでは、コンペアアンドスワップを使用しない設定がされています。
    */
    void SetChangeMetaCompareParam(const DataStoreChangeMetaCompareParam& compareParam_) { this->compareParam = compareParam_; }
    
    /*!
        @brief      コンペアアンドスワップで使用する、比較用期待値を取得します。

        @return     比較用期待値
    */
    const DataStoreChangeMetaCompareParam& GetChangeMetaCompareParam() const { return this->compareParam; }

    /*!
        @brief      変更するデータを永続化対象(プリンシパルIDと永続化スロットID)で指定します。

        変更するデータをプリンシパルIDと永続化スロットIDで指定します。
        本パラメータを設定する場合は DataStoreChangeMetaParam::SetDataId() でデータIDを設定しないでください。

        @param[in]  persistenceTarget_ 変更するデータの永続化対象。
    */
    void SetPersistenceTarget(DataStorePersistenceTarget persistenceTarget_)
    {
        SYSTEMCHECK(dataId == DataStoreConstants::INVALID_DATAID && "DataID is already set. Please specify either DataID or PersistenceTarget");
        this->persistenceTarget = persistenceTarget_;
    }

    /*!
        @brief      変更するデータの永続化対象を取得します。

        @return     変更するデータの永続化対象。
    */
    const DataStorePersistenceTarget& GetPersistenceTarget() const
    {
        return this->persistenceTarget;
    }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        // PersistenceTargetがすでにセットされているかのチェックを省くため直接代入する
        dataId = DataStoreConstants::INVALID_DATAID;
        SetModificationFlag(DataStoreConstants::MODIFICATION_FLAG_NONE);
        SetName(NEX_T(""));
        SetAccessPermission(DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE));
        SetUpdatePermission(DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE));
        SetPeriod(DataStoreConstants::DEFAULT_PERIOD);
        SetTags(qVector<String>());
        SetMetaBinary(NULL, 0);
        SetUpdatePassword(DataStoreConstants::INVALID_PASSWORD);
        SetDataType(0);
        SetDataStatus(DataStoreConstants::DATA_STATUS_NONE);
        SetReferredCount(0);
        SetChangeMetaCompareParam(DataStoreChangeMetaCompareParam());
        SetPersistenceTarget(DataStorePersistenceTarget());
    }

    /*!
        @brief      DataStoreChangeMetaParamオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        StringStream ss;
        ss << NEX_T("DataStoreChangeMetaParam: (")
           << NEX_T("dataId=") << dataId
           << NEX_T(", modificationFlag=") << modifiesFlag
           << NEX_T(", name=") << name
           << NEX_T(", accessPermission=") << permission
           << NEX_T(", updatePermission=") << delPermission
           << NEX_T(", period=") << period
           << NEX_T(", updatePassword=") << updatePassword
           << NEX_T(", dataType=") << dataType
           << NEX_T(", referredCnt=") << referredCnt
           << NEX_T(", status=") << status
           << NEX_T(", persistenceTarget=") << persistenceTarget;

        ss << NEX_T(", tags=(");
        for (qVector<String>::const_iterator it = tags.begin(); it != tags.end(); ++it)
        {
            ss << *it << NEX_T(", ");
        }
        ss << NEX_T(")");

        ss << NEX_T(", metaBinary=");
        for (qBuffer::const_iterator it = metaBinary.begin(); it != metaBinary.end(); ++it)
        {
            ss << hex << *it;
        }
        ss << NEX_T(")");

        compareParam.Trace(uiTraceFlags);

        QTRACE(uiTraceFlags, ss.CStr());
    }
};

END_NAMESPACE_NEX

