﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are 
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreChangeMetaParamV1DDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      メタ情報の変更に関するパラメータを表すクラスです。
*/
class DataStoreChangeMetaParamV1: public DDLCLASS(DataStoreChangeMetaParamV1) {
public:
    /*!
        @brief      コンストラクタです。
    */
    DataStoreChangeMetaParamV1()
    {
        Reset();
    }
    
    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreChangeMetaParamV1(){}
    
    /*!
        @brief      データIDをセットします。

        メタデータを変更するデータのデータIDを指定します。

        @param[in]  dataId メタデータを変更するデータのデータID。
    */
    void SetDataId(qUnsignedInt64 dataId_) { this->dataId = dataId_; }
    
    /*!
        @brief      データIDを取得します。

        @return     データID。
    */
    qUnsignedInt64 GetDataId() const { return this->dataId; }
    
    /*!
        @brief      変更フラグをセットします。

        メタデータを変更する箇所を表すフラグを指定します。本パラメータは必ず指定する必要があります。
        DataStoreConstants::ModificationFlag に対してビット単位の論理和を用いて複数指定することができます。

        @param[in]  modificationFlag メタデータを変更する箇所を表すビットフラグ。 DataStoreConstants::ModificationFlag に対するビット単位の論理和です。
    */
    void SetModificationFlag(qUnsignedInt32 modificationFlag) { this->modifiesFlag = modificationFlag; }
    
    /*!
        @brief      変更フラグを取得します。

        @return     メタデータを変更する箇所を表すビットフラグ。 DataStoreConstants::ModificationFlag に対するビット単位の論理和です。
    */
    qUnsignedInt32 GetModificationFlag() const { return this->modifiesFlag; }
    
    /*!
        @brief      タイトルをセットします

        新しいタイトル(要約情報)を指定します。本パラメータの指定は必須ではありません。デフォルトは空文字列です。

        @param[in]  name 新しいタイトル(要約情報)。
    */
    void SetName(const String& name_)
    {
#ifdef NEX_LOG_ENABLED
        qInt32 length = name_.GetWideCharLength();
        if (length > DataStoreConstants::MAX_NAME_LENGTH)
        {
            QLOG(EventLog::Warning, NEX_T("DataStoreChangeMetaParamV1::SetName(): name length is over the limit. ") << NEX_T("name=") << name_ << NEX_T(", length=") << length);
        }
#endif
        this->name = name_;
    }
    
    /*!
        @brief      タイトルを取得します。

        @return     タイトル。
    */
    const String& GetName() const { return this->name; }
    
    /*!
        @brief      参照権限をセットします。

        新しい参照権限を指定します。本パラメータの指定は必須ではありません。デフォルトは DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE) です。

        @param[in]  permission 新しい参照権限。
    */
    void SetAccessPermission(const DataStorePermission& permission_) { this->permission = permission_; }
    
    /*!
        @brief      参照権限を取得します。

        @return     参照権限。
    */
    const DataStorePermission& GetAccessPermission() const { return this->permission; }
    
    /*!
        @brief      更新権限をセットします。推奨されません。

        これは互換性のために残されている古い関数です。 DataStoreChangeMetaParamV1::SetUpdatePermission() を使用して下さい。
        新しい更新権限を指定します。本パラメータの指定は必須ではありません。デフォルトは DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE) です。

        @param[in]  delPermission 新しい更新権限。
    */
    void SetDeletePermission(const DataStorePermission& delPermission_)
    {
        QLOG(EventLog::Warning, NEX_T("DataStoreChangeMetaParamV1::SetDeletePermission() is deprecated. Use DataStoreChangeMetaParamV1::SetUpdatePermission()"));
        this->delPermission = delPermission_;
    }
    
    /*!
        @brief      更新権限を取得します。推奨されません。

        これは互換性のために残されている古い関数です。 DataStoreChangeMetaParamV1::GetUpdatePermission() を使用して下さい。

       @return     更新権限。
    */
    const DataStorePermission& GetDeletePermission() const
    {
        QLOG(EventLog::Warning, NEX_T("DataStoreChangeMetaParamV1::GetDeletePermission() is deprecated. Use DataStoreChangeMetaParamV1::GetUpdatePermission()"));
        return this->delPermission;
    }
    
    /*!
        @brief      更新権限をセットします。

        新しい更新権限を指定します。本パラメータの指定は必須ではありません。デフォルトは DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE) です。

        @param[in]  updatePermission 新しい更新権限。
    */
    void SetUpdatePermission(const DataStorePermission& updatePermission) { this->delPermission = updatePermission; }
    
    /*!
        @brief      更新権限を取得します。

        @return     更新権限。
    */
    const DataStorePermission& GetUpdatePermission() const { return this->delPermission; }
    
    /*!
        @brief      有効日数をセットします。

        新しい有効日数を指定します。本パラメータの指定は必須ではありません。デフォルトは DataStoreConstants::DEFAULT_PERIOD です。

        @param[in]  period 新しい有効日数。
    */
    void SetPeriod(qUnsignedInt16 period_) { this->period = period_; }
    
    /*!
        @brief      有効日数を取得します。

        @return     有効日数。
    */
    qUnsignedInt16 GetPeriod() const { return this->period; }
    
    /*!
        @brief      タグをセットします

        アップロードするデータにメタ情報として付加するタグを指定します。
        既にタグが設定されてる場合は、それは削除され本関数で指定したタグで上書きされます。
        本パラメータの指定は必須ではありません。デフォルトは空です。

        @param[in]  tags アップロードするデータにメタ情報として付加するタグ。
    */
    void SetTags(const qVector<String>& tags_)
    {
#ifdef NEX_LOG_ENABLED
        if (tags_.size() > DataStoreConstants::NUM_TAG_SLOT)
        {
            QLOG(EventLog::Warning, NEX_T("DataStoreChangeMetaParamV1::SetTags(): slot size is over the limit. size=") << tags_.size());
        }

        qVector<String>::const_iterator it = tags_.begin();
        for (; it != tags_.end(); ++it)
        {
            qInt32 length = it->GetWideCharLength();
            if (length > DataStoreConstants::MAX_TAG_LENGTH)
            {
                QLOG(EventLog::Warning, NEX_T("DataStoreChangeMetaParamV1::SetTags(): tag length is over the limit. ") << NEX_T("tag=") << *it << NEX_T(", length=") << length);
            }
        }
#endif
        this->tags = tags_;
    }
    
    /*!
        @brief      タグを取得します。

        @return     タグ。
    */
    const qVector<String>& GetTags() const { return this->tags; }

    /*!
        @brief      メタ情報内バイナリデータをセットします。

        アップロードするデータにメタ情報として付加するバイナリデータを指定します。
        0-1024byteまで設定できます。本パラメータの指定は必須ではありません。デフォルトは空です。

        @param[in]  p アップロードするデータにメタ情報として付加するバイナリデータのポインタ。NULLを指定すると空になります。
        @param[in]  size アップロードするバイナリデータのデータサイズ(0-1024)。
    */
    void SetMetaBinary(const void* p, size_t size) {
        this->metaBinary.clear();
        if (p != NULL) {
            const qByte* pb = static_cast<const qByte*>(p);
            this->metaBinary.push_back(pb, size);
        }
    }

    /*!
        @brief      メタ情報内バイナリデータを取得します。

        @return     メタ情報内バイナリデータ
    */
    qVector<qByte> GetMetaBinary() const {
        // qBuffer -> qVector<qByte>への変換
        qVector<qByte> vb;
        vb.assign(this->metaBinary.begin(), this->metaBinary.end()); 
        return vb;
    }

    /*!
        @brief      メタ情報内バイナリデータを取得します。

        こちらのオーバーロード関数の方が効率的です。

        @param[out]  metaBinary メタ情報内バイナリデータ。
    */
    void GetMetaBinary(qVector<qByte>& metaBinary_) const {
        // qBuffer -> qVector<qByte>への変換
        metaBinary_.assign(this->metaBinary.begin(), this->metaBinary.end());
    }

    /*!
        @brief      更新パスワードを設定します。

        更新権限を持たないデータを操作する場合に更新パスワードを指定します。
        データのオーナーから受け取った更新パスワードを指定してください。
        更新権限を持っている場合は更新パスワードは無視され、常に操作可能です。
        DataStoreConstants::INVALID_PASSWORD を指定した場合は更新パスワードでの認証を行いません。
        本関数で指定しない場合は DataStoreConstants::INVALID_PASSWORD が使用されます。

        @param[in]  updatePassword 更新パスワード
    */
    void SetUpdatePassword(qUnsignedInt64 updatePassword_) { this->updatePassword = updatePassword_; }
    
    /*!
        @brief      更新パスワードを取得します。

        @return     更新パスワード。
    */
    qUnsignedInt64 GetUpdatePassword() const { return this->updatePassword; }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetDataId(DataStoreConstants::INVALID_DATAID);
        SetModificationFlag(DataStoreConstants::MODIFICATION_FLAG_NONE);
        SetName(NEX_T(""));
        SetAccessPermission(DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE));
        SetUpdatePermission(DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE));
        SetPeriod(DataStoreConstants::DEFAULT_PERIOD);
        SetTags(qVector<String>());
        SetMetaBinary(NULL, 0);
        SetUpdatePassword(DataStoreConstants::INVALID_PASSWORD);
    }

    /*!
        @brief      DataStoreChangeMetaParamV1オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        StringStream ss;
        ss << NEX_T("DataStoreChangeMetaParamV1: (")
           << NEX_T("dataId=") << dataId
           << NEX_T(", modificationFlag=") << modifiesFlag
           << NEX_T(", name=") << name
           << NEX_T(", accessPermission=") << permission
           << NEX_T(", updatePermission=") << delPermission
           << NEX_T(", period=") << period
           << NEX_T(", updatePassword=") << updatePassword;

        ss << NEX_T(", tags=(");
        for (qVector<String>::const_iterator it = tags.begin(); it != tags.end(); ++it)
        {
            ss << *it << NEX_T(", ");
        }
        ss << NEX_T(")");

        ss << NEX_T(", metaBinary=");
        for (qBuffer::const_iterator it = metaBinary.begin(); it != metaBinary.end(); ++it)
        {
           ss << hex << *it;
        }

        ss << NEX_T(")");

        QTRACE(uiTraceFlags, ss.CStr());
    }
};

END_NAMESPACE_NEX

