﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Common/DataCode64.h>
#include <RendezVous/Services/Jugem/DataStore/src/Common/DataStoreConstants.h>

BEGIN_NAMESPACE_NEX


/*!
    @brief      データ ID と表示用のデータコード文字列を相互変換するクラスです。

    データコード文字列とは、データ ID と チェックサムを含む、 0 1 2 3 4 5 6 7 8 9 B C D F G H J K L M N P R T V W X Y
    の28文字 からなる 最大 14 文字の文字列です。 データ ID は、最大 56 ビット まで利用可能です。

*/
#ifndef FOR_DOC_PURPOSE
class DataStoreDataCode: public DataCode64
{
#else
class DataStoreDataCode : public RootObject
{
#endif  // FOR_DOC_PURPOSE
private:
    DataStoreDataCode();

public:
    /*!
        @brief    データ ID から DataStoreDataCode オブジェクトを生成します

        56 ビットを超える場合は、  IsValid() が False となります。

        @param[in]    key       チェックサム算出時の鍵。自タイトルのアクセスキーを指定してください。
        @param[in]    dataId    データ ID

        @return    DataStoreDataCode オブジェクト

        @see IsValid(), GetDataId()
     */
    static DataStoreDataCode CreateFromDataId(const String& key, qUnsignedInt64 dataId);

    /*!
        @brief    データコード文字列 から DataStoreDataCode オブジェクトを生成します

        データコード文字列の妥当性検証時に、データコード文字列に小文字が含まれる場合には大文字に変換され、入力ミスが疑われる文字は適切な文字に変換されます。
        (I->1, Q->0, O->0, Z->2, S->5)

        データコード文字列が妥当であるかは、 オブジェクト生成後に、 IsValid() でチェックしてくてください。

        @param[in]    key       チェックサム算出時の鍵。自タイトルのアクセスキーを指定してください。
        @param[in]    dataCodeString    データコード文字列

        @return    DataStoreDataCode オブジェクト

        @see IsValid(), ToDataCodeString(), DataStoreConstants::MAX_DATACODE_STR_LEN
     */
    static DataStoreDataCode CreateFromDataCodeString(const String& key, const String& dataCodeString);


    /*!
        @brief    64 ビット整数値の内部データコード から DataStoreDataCode オブジェクトを生成します

        内部データコードが妥当であるかは、 オブジェクト生成後に、 IsValid() でチェックしてくてください。


        @param[in]    key       チェックサム算出時の鍵。自タイトルのアクセスキーを指定してください。
        @param[in]    dataCodeRaw    内部データコード

        @return    DataStoreDataCode オブジェクト

        @see IsValid(), GetDataCodeRaw()
     */
    static DataStoreDataCode CreateFromDataCodeRaw(const String& key, qUnsignedInt64 dataCodeRaw);

#ifdef FOR_DOC_PURPOSE
    /*!
        @brief データ ID を取得します。

        このオブジェクトが無効の場合は、 DataStoreConstants::INVALID_DATAID を返します。

        @return データ ID

    */
    qUnsignedInt64 GetDataId() const;

    /*!
        @brief データコード文字列を生成します。

        データコード文字列とは、データ ID と チェックサムを含む、 0 1 2 3 4 5 6 7 8 9 B C D F G H J K L M N P R T V W X Y
        の28文字 からなる 最大 14 文字の文字列です。 データ ID は、最大 56 ビット まで利用可能です。

        このオブジェクトが無効の場合は、空文字列が返ります。

        @return データコード文字列

        @see IsValid(), DataStoreConstants::MAX_DATACODE_STR_LEN
     */
    String ToDataCodeString() const;

    /*!
        @brief データコード文字列を生成します。 14文字に満たない場合は、上位を 0 で埋めます。

        データコード文字列とは、データ ID と チェックサムを含む、 0 1 2 3 4 5 6 7 8 9 B C D F G H J K L M N P R T V W X Y
        の28文字 からなる 最大 14 文字の文字列です。 データ ID は、最大 56 ビット まで利用可能です。

        このオブジェクトが無効の場合は、空文字列が返ります。

        @return データコード文字列

        @see IsValid()
     */
    String ToDataCodeStringZeroPad() const;

    /*!
        @brief 内部データコードを取得します。

        データ ID (56bit) とタイトル固有の チェックサム値 (8bit) から成る 64 ビットの整数値です。

        この関数を使う前に IsValid() で、オブジェクトが有効であることを確認してください。

        @return 内部データコード

        @see IsValid()
     */
    qUnsignedInt64 GetDataCodeRaw() const;

    /*!
        @brief このオブジェクトが有効なデータコード文字列、有効なデータ ID もしくは有効な内部データコードから
               生成されたものか判定します。

        @retval true 有効なオブジェクトである
        @retval false 有効なオブジェクトでない

        @see CreateFromDataId(), CreateFromDataCodeString(), CreateFromDataCodeRaw()
        @see ToDataCodeString(), ToDataCodeStringZeroPad(), GetDataCodeRaw()
     */
    qBool IsValid() const;
#endif  // FOR_DOC_PURPOSE
};


END_NAMESPACE_NEX

