﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStorePersistenceInitParamDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      永続化設定を表すクラスです。
*/
class DataStorePersistenceInitParam: public DDLCLASS(DataStorePersistenceInitParam)
{
public:
    /*!
        @brief      コンストラクタです。

        永続化を行う場合は対象の永続化スロットIDを指定してください。
        デフォルト引数である DataStoreConstants::INVALID_PERSISTENCE_SLOT_ID を指定した場合は永続化を行いません。
        deleteLastObjectには指定した永続化スロットIDに既にデータがある場合、そのデータを削除するかどうかを設定します。

        @param[in]  persistenceSlotId 永続化スロットID
        @param[in]  deleteLastObject 以前のデータを削除するかどうか
    */
    explicit DataStorePersistenceInitParam(qUnsignedInt16 persistenceSlotId = DataStoreConstants::INVALID_PERSISTENCE_SLOT_ID, qBool deleteLastObject = true)
    {
        SetPersistenceSlotId(persistenceSlotId);
        SetDeleteLastObject(deleteLastObject);
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStorePersistenceInitParam() {}


    /*!
        @brief      永続化スロットIDを設定します。

        永続化スロットIDは 0 以上 DataStoreConstants::NUM_PERSISTENCE_SLOT 未満の範囲もしくは DataStoreConstants::INVALID_PERSISTENCE_SLOT_ID を指定してください。
        DataStoreConstants::INVALID_PERSISTENCE_SLOT_ID を指定した場合は永続化を行いません。

        @param[in]  persistenceSlotId_ 永続化スロットID
    */
    void SetPersistenceSlotId(qUnsignedInt16 persistenceSlotId_)
    {
        this->persistenceSlotId = persistenceSlotId_;
    }

    /*!
        @brief      永続化スロットIDを取得します。

        @return     永続化スロットID
    */
    qUnsignedInt16 GetPersistenceSlotId() const
    {
        return this->persistenceSlotId;
    }

    /*!
        @brief      以前のデータを削除するかどうかを設定します。

        SetPersistenceSlotId() で指定した永続化スロットIDに既にデータがある場合、そのデータを削除するかどうかを設定します。

        @param[in]  deleteLastObject_ 以前のデータを削除するかどうかを設定します。
    */
    void SetDeleteLastObject(qBool deleteLastObject_)
    {
        this->deleteLastObject = deleteLastObject_;
    }

    /*!
        @brief      以前のデータを削除するかどうかを取得します。

        @return     以前のデータを削除するかどうか。
    */
    qBool GetDeleteLastObject() const
    {
        return this->deleteLastObject;
    }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetPersistenceSlotId(DataStoreConstants::INVALID_PERSISTENCE_SLOT_ID);
        SetDeleteLastObject(true);
    }

    /*!
        @brief      DataStorePersistenceInitParamオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド定義です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const DataStorePersistenceInitParam& oPersistenceInitParam);
};

inline StringStream& operator<<(StringStream& oStr, const DataStorePersistenceInitParam& oPersistenceInitParam)
{
    oStr <<NEX_T("DataStorePersistenceInitParam: (")
         << NEX_T("persistenceSlotId=") << oPersistenceInitParam.persistenceSlotId
         << NEX_T(", deleteLastObject=") << oPersistenceInitParam.deleteLastObject
         << NEX_T(")");

    return oStr;
}

END_NAMESPACE_NEX

