﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStorePrepareGetParamDDL.h"
#include "DataStorePersistenceTarget.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      データのダウンロードに関するパラメータを表すクラスです。

    データIDもしくは永続化対象のいずれかを指定してダウンロードする対象を指定します。

    このクラスは、 DataStoreGetParam クラスと同一です。

*/
class DataStorePrepareGetParam: public DDLCLASS(DataStorePrepareGetParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    DataStorePrepareGetParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStorePrepareGetParam() {}

    /*!
        @brief      データIDをセットします。

        取得するデータのデータIDを指定します。
        本パラメータを設定する場合は DataStorePrepareGetParam::SetPersistenceTarget() で永続化対象(プリンシパルIDと永続化スロットID)を設定しないでください。

        @param[in]  dataId_ 取得するデータのデータID。
    */
    void SetDataId(qUnsignedInt64 dataId_)
    {
        SYSTEMCHECK(persistenceTarget == DataStorePersistenceTarget() && "PersistenceTarget is already set. Please specify either DataID or PersistenceTarget");
        this->dataId = dataId_;
    }

    /*!
        @brief      データIDを取得します。

        @return     データID。
    */
    qUnsignedInt64 GetDataId() const
    {
        return this->dataId;
    }

    /*!
        @brief      取得するデータを永続化対象(プリンシパルIDと永続化スロットID)で指定します。

        取得するデータをプリンシパルIDと永続化スロットIDで指定します。
        本パラメータを設定する場合は DataStorePrepareGetParam::SetDataId() でデータIDを設定しないでください。

        @param[in]  persistenceTarget_ 取得するデータの永続化対象。
    */
    void SetPersistenceTarget(const DataStorePersistenceTarget& persistenceTarget_)
    {
        SYSTEMCHECK(dataId == DataStoreConstants::INVALID_DATAID && "DataID is already set. Please specify either DataID or PersistenceTarget");
        this->persistenceTarget = persistenceTarget_;
    }

    /*!
        @brief      永続化対象取得します。

        @return     永続化対象。
    */
    const DataStorePersistenceTarget& GetPersistenceTarget() const
    {
        return this->persistenceTarget;
    }

    /*!
        @brief      参照パスワードを設定します。

        参照権限を持たないデータを参照する場合に参照パスワードを指定します。
        データのオーナーから受け取った参照パスワードを指定してください。
        参照権限を持っている場合は参照パスワードは無視され、常に参照可能です。
        DataStoreConstants::INVALID_PASSWORD を指定した場合は参照パスワードでの認証を行いません。
        本関数で指定しない場合は DataStoreConstants::INVALID_PASSWORD が使用されます。

        @param[in]  accessPassword_ 参照パスワード
    */
    void SetAccessPassword(qUnsignedInt64 accessPassword_)
    {
        this->accessPassword = accessPassword_;
    }

    /*!
        @brief      参照パスワードを取得します。

        @return     参照パスワード。
    */
    qUnsignedInt64 GetAccessPassword() const
    {
        return this->accessPassword;
    }

    /*
        @internal
        @brief      ロックIDをセットします。

        ダウンロードするデータに対するロックIDを指定します。
        データをアップロードするときに DataStoreConstants::DATA_FLAG_USE_READ_LOCK を指定した場合、
        初めて DataStoreClient::GetObject() で参照されたときにロックIDがサーバーで保存されます。以後はそのロックIDと一致する場合のみダウンロード可能となります。
        例えば誰か一人にデータを渡したいときに利用します。
        本パラメータの指定は必須ではありません。デフォルトは0です。

        @param[in]  lockId データにひも付けるロックID。
    */
    //    void SetLockId(qUnsignedInt32 lockId) { this->lockId = lockId; }

    /*
        @internal
        @brief      ロックIDを取得します。

        @return     ロックID。
    */
    //    qUnsignedInt32 GetLockId() const { return this->lockId; }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        dataId = DataStoreConstants::INVALID_DATAID;
        persistenceTarget = DataStorePersistenceTarget();
        SetAccessPassword(DataStoreConstants::INVALID_PASSWORD);
        // SetLockId(0);
        // 変な値が入るのは困るので0を入れておく
        lockId = 0;
        this->extraData.clear();
    }

    /*!
        @brief      DataStorePrepareGetParamオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, NEX_T("DataStorePrepareGetParam: (")
               << NEX_T("dataId=") << dataId
               << NEX_T(", persistenceTarget=") << persistenceTarget
               << NEX_T(", accessPassword=") << accessPassword
               << NEX_T(")")
              );
    }

};

END_NAMESPACE_NEX

