﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreSearchParamDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      メタ情報の検索に関するパラメータを表すクラスです。
*/
class DataStoreSearchParam: public DDLCLASS(DataStoreSearchParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    DataStoreSearchParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreSearchParam() {}

    /*!
        @brief      検索タイプをセットします。

        デフォルトは DataStoreConstants::SEARCH_TYPE_PUBLIC です。
        本関数を使用した場合の動作の詳細についてプログラミングマニュアルを参照してください。

        @param[in]  searchType 検索タイプ。
    */
    void SetSearchType(DataStoreConstants::SearchType searchType)
    {
        this->searchTarget = static_cast<qByte>(searchType);
    }

    /*!
        @brief      検索タイプを取得します。

        @return     検索タイプ。
    */
    DataStoreConstants::SearchType GetSearchType() const
    {
        return static_cast<DataStoreConstants::SearchType>(this->searchTarget);
    }

    /*!
        @brief      検索するアップロードした人の プリンシパル ID  をセットします。

        指定しなかった場合はこの条件が検索条件から除外されます。

        @param[in]  ownerIds_ 検索するアップロードした人の プリンシパル ID  のリスト。
    */
    void SetOwnerIds(const qVector<PrincipalID>& ownerIds_)
    {
        this->ownerIds = ownerIds_;
    }

    /*!
        @brief      検索するアップロードした人の プリンシパル ID  を取得します。

        @return     検索するアップロードした人の プリンシパル ID  のリスト。
    */
    const qVector<PrincipalID>& GetOwnerIds() const
    {
        return this->ownerIds;
    }

    /*!
        @brief      検索するアップロードした人の プリンシパル ID  のタイプをセットします。

        @param[in]  ownerType_ 検索するアップロードした人の プリンシパル ID  のタイプ。
    */
    void SetOwnerType(DataStoreConstants::SearchTarget ownerType_)
    {
        this->ownerType = static_cast<qByte>(ownerType_);
    }

    /*!
        @brief      検索するアップロードした人の プリンシパル ID  のタイプを取得します。

        @return     検索するアップロードした人の プリンシパル ID  のタイプ。
    */
    DataStoreConstants::SearchTarget GetOwnerType() const
    {
        return static_cast<DataStoreConstants::SearchTarget>(this->ownerType);
    }

    /*!
        @brief      検索する参照権限保持者の プリンシパル ID  をセットします。

        データアップロード時に参照権限をセットする際に DataStorePermission::DataStorePermission() の第二引数で権限保持者の プリンシパル ID  を指定できますが、
        本関数を使用するとその権限保持者の プリンシパル ID  を指定してデータを検索することができます。
        指定しなかった場合はこの条件が検索条件から除外されます。
        検索タイプが DataStoreConstants::SEARCH_TYPE_PUBLIC もしくは DataStoreConstants::SEARCH_TYPE_FRIEND の場合に指定できます。
        自分自身が参照権限に含まれているものを検索する場合は検索タイプを DataStoreConstants::SEARCH_TYPE_RECEIVED とするのが便利です。
        DataStoreConstants::SEARCH_TYPE_RECEIVED を使用した場合は本関数で自分の プリンシパル ID  を指定する必要はありません。
        (自分の プリンシパル ID  を指定してもエラーとはなりません。)
        参照権限保持者を送信宛先と見立てて、特定の人が受け取ったデータを列挙するという使い方が可能です。

        @param[in]  destinationIds_ 検索する、参照権限として指定された プリンシパル ID  のリスト。複数の プリンシパル ID  が指定されたときは OR で検索されます。
    */
    void SetDestinationIds(const qVector<PrincipalID>& destinationIds_)
    {
        this->destinationIds = destinationIds_;
    }

    /*!
        @brief      検索する参照権限保持者の プリンシパル ID  を取得します。

        @return     検索する参照権限保持者の プリンシパル ID  のリスト。
    */
    const qVector<PrincipalID>& GetDestinationIds() const
    {
        return this->destinationIds;
    }

    /*!
        @brief      検索するデータタイプ 1 つをセットします。

        デフォルトでは検索条件にデータタイプを指定しません。

        @param[in]  dataType_ 検索するデータタイプ。
    */
    void SetDataType(const qUnsignedInt16 dataType_)
    {
        this->dataTypes.clear();
        this->dataTypes.push_back(dataType_);
    }

    /*!
        @brief      検索するデータタイプのリストをセットします。

        デフォルトでは検索条件にデータタイプを指定しません。
        最大 DataStoreConstants::MAX_SEARCH_DATA_TYPE_SIZE 個のデータタイプを指定できます。

        @param[in]  dataTypes_ 検索するデータタイプのリスト。
    */
    void SetDataType(const qVector<qUnsignedInt16>& dataTypes_)
    {
        this->dataTypes = dataTypes_;
    }

    /*!
        @brief      検索するデータタイプを 1 つ取得します。

        DataStoreSearchParam::SetDataType() で複数のデータタイプを指定している場合は、指定したデータタイプのリスト中の先頭のデータタイプが返ります。
        検索条件にデータタイプを指定していない場合は、DataStoreConstants::INVALID_DATA_TYPE が返ります。

        @return     検索するデータタイプ。
    */
    qUnsignedInt16 GetDataType() const
    {
        return dataTypes.empty() ? DataStoreConstants::INVALID_DATA_TYPE : this->dataTypes[0];
    }

    /*!
        @brief      検索するデータタイプのリストを取得します。

        検索条件にデータタイプを指定していない場合は、空のリストが返ります。

        @param[out]  dataTypes_ 検索するデータタイプのリスト。
    */
    void GetDataType(qVector<qUnsignedInt16>& dataTypes_) const
    {
        dataTypes_.assign(this->dataTypes.begin(), this->dataTypes.end());
    }

    /*!
        @brief      検索するアップロードされた日時の開始点をセットします。

        アップロードされた日時の開始点は検索のパラメータから将来的に廃止予定であるため、本関数の使用は推奨されません。

        DataStoreSearchParam::SetCreatedAfter() で指定した日時以降、 DataStoreSearchParam::SetCreatedBefore() で指定した日時以前のものを検索します。
        DataStoreConstants::INVALID_DATE_TIME を指定した場合はこの条件が検索条件から除外されます。デフォルトは DataStoreConstants::INVALID_DATE_TIME です。

        @param[in]  createdAfter_ 検索するアップロードされた日時の開始点。
    */
    void SetCreatedAfter(const DateTime& createdAfter_)
    {
        this->createdAfter = createdAfter_;
    }

    /*!
        @brief      検索するアップロードされた日時の開始点を取得します。

        アップロードされた日時の開始点は検索のパラメータから将来的に廃止予定であるため、本関数の使用は推奨されません。

        @return     検索するアップロードされた日時の開始点。
    */
    const DateTime& GetCreatedAfter() const
    {
        return this->createdAfter;
    }

    /*!
        @brief      検索するアップロードされた日時の終了点をセットします。

        アップロードされた日時の終了点は検索のパラメータから将来的に廃止予定であるため、本関数の使用は推奨されません。

        DataStoreSearchParam::SetCreatedAfter() で指定した日時以降、 DataStoreSearchParam::SetCreatedBefore() で指定した日時以前のものを検索します。
        DataStoreConstants::INVALID_DATE_TIME を指定した場合はこの条件が検索条件から除外されます。デフォルトは DataStoreConstants::INVALID_DATE_TIME です。

        @param[in]  createdBefore_ 検索するアップロードされた日時の終了点。
    */
    void SetCreatedBefore(const DateTime& createdBefore_)
    {
        this->createdBefore = createdBefore_;
    }

    /*!
        @brief      検索するアップロードされた日時の終了点を取得します。

        アップロードされた日時の終了点は検索のパラメータから将来的に廃止予定であるため、本関数の使用は推奨されません。

        @return     検索するアップロードされた日時の終了点。
    */
    const DateTime& GetCreatedBefore() const
    {
        return this->createdBefore;
    }

    /*!
        @brief      検索する更新日時の開始点をセットします。

        更新日時の開始点は検索のパラメータから将来的に廃止予定であるため、本関数の使用は推奨されません。

        DataStoreSearchParam::SetUpdatedAfter() で指定した日時以降、 DataStoreSearchParam::SetUpdatedBefore() で指定した日時以前のものを検索します。
        DataStoreConstants::INVALID_DATE_TIME を指定した場合はこの条件が検索条件から除外されます。デフォルトは DataStoreConstants::INVALID_DATE_TIME です。

        @param[in]  updatedAfter_ 検索するアップロードされた日時の開始点。
    */
    void SetUpdatedAfter(const DateTime& updatedAfter_)
    {
        this->updatedAfter = updatedAfter_;
    }

    /*!
        @brief      検索する更新日時の開始点を取得します。

        更新日時の開始点は検索のパラメータから将来的に廃止予定であるため、本関数の使用は推奨されません。

        @return     検索する更新日時の開始点。
    */
    const DateTime& GetUpdatedAfter() const
    {
        return this->updatedAfter;
    }

    /*!
        @brief      検索する更新日時の終了点をセットします。

        更新日時の終了点は検索のパラメータから将来的に廃止予定であるため、本関数の使用は推奨されません。

        DataStoreSearchParam::SetUpdatedAfter() で指定した日時以降、 DataStoreSearchParam::SetUpdatedBefore() で指定した日時以前のものを検索します。
        DataStoreConstants::INVALID_DATE_TIME を指定した場合はこの条件が検索条件から除外されます。デフォルトは DataStoreConstants::INVALID_DATE_TIME です。

        @param[in]  updatedBefore_ 検索するアップロードされた日時の終了点。
    */
    void SetUpdatedBefore(const DateTime& updatedBefore_)
    {
        this->updatedBefore = updatedBefore_;
    }

    /*!
        @brief      検索する更新日時の終了点を取得します。

        更新日時の終了点は検索のパラメータから将来的に廃止予定であるため、本関数の使用は推奨されません。

        @return     検索する更新日時の終了点。
    */
    const DateTime& GetUpdatedBefore() const
    {
        return this->updatedBefore;
    }

    /*!
        @internal
        @brief      検索する関連づけられたデータIDをセットします。

        DataStoreConstants::INVALID_DATAID を指定した場合はこの条件が検索条件から除外されます。デフォルトは DataStoreConstants::INVALID_DATAID です。

        @param[in]  referDataId_ 検索する関連づけられたデータID。
    */
    void SetReferDataId(qUnsignedInt32 referDataId_)
    {
        this->referDataId = referDataId_;
    }

    /*!
        @internal
        @brief      検索する関連づけられたデータIDを取得します。

        @return     検索する関連づけられたデータID。
    */
    qUnsignedInt32 GetReferDataId() const
    {
        return this->referDataId;
    }

    /*!
        @brief      検索結果のソート対象とするカラムをセットします。

        デフォルトは DataStoreConstants::SEARCH_SORT_COLUMN_DATAID です。

        ソート対象に DataStoreConstants::SEARCH_SORT_COLUMN_DATAID 以外のカラムをセットする場合は、 DataStoreSearchParam::SetDataType() で検索対象のデータタイプを 1 つセットする必要があります。
        DataStoreConstants::SEARCH_SORT_COLUMN_DATAID 以外のカラムでソートする際は、複数のデータタイプを指定することはできません。

        @param[in]  resultOrderColumn_ 検索結果のソート対象とするカラム。
    */
    void SetSearchSortOrderColumn(DataStoreConstants::SearchSortColumn resultOrderColumn_)
    {
        this->resultOrderColumn = static_cast<qByte>(resultOrderColumn_);
    }

    /*!
        @brief      検索結果のソート対象とするカラムを取得します。

        @return     検索結果のソート対象とするカラム。
    */
    DataStoreConstants::SearchSortColumn GetSearchSortOrderColumn() const
    {
        return static_cast<DataStoreConstants::SearchSortColumn>(this->resultOrderColumn);
    }

    /*!
        @brief      検索結果のソート順をセットします。

        デフォルトは DataStoreConstants::SEARCH_SORT_ORDER_ASC です。

        @param[in]  resultOrder_ 検索結果のソート順。
    */
    void SetSearchSortOrder(DataStoreConstants::SearchSortOrder resultOrder_)
    {
        this->resultOrder = static_cast<qByte>(resultOrder_);
    }

    /*!
        @brief      検索結果のソート順を取得します。

        @return     検索結果のソート順。
    */
    DataStoreConstants::SearchSortOrder GetSearchSortOrder() const
    {
        return static_cast<DataStoreConstants::SearchSortOrder>(this->resultOrder);
    }

    /*!
        @brief      検索範囲をセットします。

        デフォルトは ResultRange() です。
        本関数で検索結果の取得数を設定する際、指定可能な最大数は DataStoreConstants::MAX_SEARCH_RESULT_SIZE です。

        RESULTRANGE_ANY_OFFSET を設定すると、検索結果は、ソート対象とソート順をもとにした擬似的に無作為な順序になります。なお、この時指定可能な最大数は DataStoreConstants::MAX_SEARCH_ANY_RESULT_SIZE です。

        @param[in]  resultRange_ 検索範囲。
    */
    void SetResultRange(const ResultRange& resultRange_)
    {
        this->resultRange = resultRange_;
    }

    /*!
        @brief      検索範囲を取得します。

        @return     検索範囲。
    */
    const ResultRange& GetResultRange() const
    {
        return this->resultRange;
    }

    /*!
        @brief      検索の戻り値オプションをセットします。

        DataStoreConstants::ResultFlag に対してビット単位の論理和を用いて複数指定することができます。
        指定したオプションの情報を検索結果で取得できる情報に加えることができます。
        デフォルトは 0 です。

        @param[in] resultOption_ 検索の戻り値オプション値。
    */
    void SetResultOption(qUnsignedInt32 resultOption_)
    {
        this->resultOption = static_cast<qByte>(resultOption_);
    }

    /*!
        @brief      検索の戻り値オプションを取得します。

        @return     検索の戻り値オプション値。
    */
    qByte GetResultOption() const
    {
        return this->resultOption;
    }

    /*!
        @brief      検索するタグをセットします。

        デフォルトは qVector<String>() です。指定した全てのタグと一致するものを検索します。
        最大 DataStoreConstants::NUM_TAG_SLOT 個のタグを指定できます。空文字列のタグは指定できません。

        タグを指定する際は DataStoreSearchParam::SetDataType() で検索対象のデータタイプを 1 つセットする必要があります。
        タグ指定時に複数のデータタイプを指定することはできません。

        DataStoreClient::SearchObjectLight() を使用する場合はタグをセットすることはできません。

        @param[in] tags_ 検索するタグ。
    */
    void SetTags(const qVector<String>& tags_)
    {
#ifdef NEX_LOG_ENABLED
        if (tags_.size() > DataStoreConstants::NUM_TAG_SLOT)
        {
            QLOG(EventLog::Warning, NEX_T("DataStoreSearchParam::SetTags(): slot size is over the limit. size=") << tags_.size());
        }

        qVector<String>::const_iterator it = tags_.begin();
        for (; it != tags_.end(); ++it)
        {
            qInt32 length = it->GetWideCharLength();
            if (length > DataStoreConstants::MAX_TAG_LENGTH)
            {
                QLOG(EventLog::Warning, NEX_T("DataStoreSearchParam::SetTags(): tag length is over the limit. ") << NEX_T("tag=") << *it << NEX_T(", length=") << length);
            }
        }
#endif
        this->tags = tags_;
    }

    /*!
        @brief      検索するタグを取得します。

        @return     検索するタグ。
    */
    const qVector<String>& GetTags() const
    {
        return this->tags;
    }

    /*!
        @brief      最低評価回数をセットします。

        このパラメータは、ソート順に評価値もしくは評価平均値を指定した場合のみ有効です。

        @param[in] minimalRatingFrequency_ 最低評価回数
    */
    void SetMinimalRatingFrequency(const qUnsignedInt32 minimalRatingFrequency_)
    {
        this->minimalRatingFrequency = minimalRatingFrequency_;
    }

    /*!
        @brief      最低評価回数を取得します。

        @return     最低評価回数
    */
    qUnsignedInt32 GetMinimalRatingFrequency()
    {
        return this->minimalRatingFrequency;
    }

    /*!
        @brief      検索結果のキャッシュを使用するかどうかをセットします。

        検索結果のキャッシュを使用する場合は、 DataStoreSearchParam::SetSearchType() で検索タイプに DataStoreConstants::SEARCH_TYPE_PUBLIC を、
        DataStoreSearchParam::SetOwnerType() でアップロードした人のプリンシパル ID のタイプに DataStoreConstants::SEARCH_TARGET_ANYBODY を
        セットする必要があります。

        @param[in] useCache_ 検索結果のキャッシュを使用するかどうか
    */
    void SetUseCache(const qBool useCache_)
    {
        this->useCache = useCache_;
    }

    /*!
        @brief      検索結果のキャッシュを使用するかどうかを取得します。

        デフォルトはfalseです。
        trueが指定された場合は、検索結果のキャッシュが存在すればキャッシュから応答を生成します。

        @return     検索結果のキャッシュを使用するかどうか
    */
    qBool GetUseCache()
    {
        return this->useCache;
    }

    /*!
        @brief      検索条件に一致した総数を取得するかどうかをセットします。

        DataStoreClient::SearchObjectLight() を使用する場合は検索条件に一致した総数を取得することはできません。

        @param[in] totalCountEnabled_ 検索条件に一致した総数を取得するか
    */
    void SetTotalCountEnabled(const qBool totalCountEnabled_)
    {
        this->totalCountEnabled = totalCountEnabled_;
    }

    /*!
        @brief      検索条件に一致した総数を取得するかどうかを取得します。

        デフォルトはfalseです。
        false の場合は、検索条件に一致した総数は常に0になります。(検索にヒットしたデータそのものは検索結果に含まれます)
        true の場合は、検索条件に一致した総数が検索結果に反映されます。

        @return     検索条件に一致した総数を取得するかどうか
    */
    qBool GetTotalCountEnabled()
    {
        return this->totalCountEnabled;
    }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetSearchType(DataStoreConstants::SEARCH_TYPE_PUBLIC);
        SetOwnerIds(qVector<PrincipalID>());
        SetOwnerType(DataStoreConstants::SEARCH_TARGET_ANYBODY);
        SetDestinationIds(qVector<PrincipalID>());
        this->dataType = DataStoreConstants::INVALID_DATA_TYPE;
        SetDataType(qVector<qUnsignedInt16>());
        SetCreatedAfter(DataStoreConstants::INVALID_DATE_TIME);
        SetCreatedBefore(DataStoreConstants::INVALID_DATE_TIME);
        SetUpdatedAfter(DataStoreConstants::INVALID_DATE_TIME);
        SetUpdatedBefore(DataStoreConstants::INVALID_DATE_TIME);
        SetReferDataId(DataStoreConstants::INVALID_DATAID);
        SetSearchSortOrderColumn(DataStoreConstants::SEARCH_SORT_COLUMN_DATAID);
        SetSearchSortOrder(DataStoreConstants::SEARCH_SORT_ORDER_ASC);
        SetResultRange(ResultRange());
        SetResultOption(0);
        SetTags(qVector<String>());
        SetMinimalRatingFrequency(0);
        SetUseCache(false);
        SetTotalCountEnabled(false);
    }

    /*!
        @brief      DataStoreChangeMetaParamオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        StringStream ss;

        ss << NEX_T("DataStoreSearchParam: (")
           << NEX_T("searchTarget=") << searchTarget
           << NEX_T(", ownerIds=(");

        for (qVector<PrincipalID>::const_iterator it = ownerIds.begin(); it != ownerIds.end(); ++it)
        {
            ss << *it << NEX_T(", ");
        }

        ss << NEX_T("), ownerType=") << ownerType
           << NEX_T(", destinationIds=(");

        for (qVector<PrincipalID>::const_iterator it = destinationIds.begin(); it != destinationIds.end(); ++it)
        {
            ss << *it << NEX_T(", ");
        }

        ss << NEX_T("), dataTypes=(");
        for (qVector<qUnsignedInt16>::const_iterator it = dataTypes.begin(); it != dataTypes.end(); ++it)
        {
            ss << *it << NEX_T(", ");
        }

        ss << NEX_T("), createdAfter=") << createdAfter << NEX_T(", createdBefore=") << createdBefore
           << NEX_T(", updatedAfter=") << updatedAfter << NEX_T(", updatedBefore=") << updatedBefore
           << NEX_T(", referDataId=") << referDataId
           << NEX_T(", resultOrderColumn=") << resultOrderColumn << NEX_T(", resultOrder=") << resultOrder
           << NEX_T(", resultRange=(") << resultRange << NEX_T(")")
           << NEX_T(", resultOption=") << resultOption
           << NEX_T(", tags=(");

        for (qVector<String>::const_iterator it = tags.begin(); it != tags.end(); ++it)
        {
            ss << *it << NEX_T(", ");
        }

        ss << NEX_T("), minimalRatingFrequency=") << minimalRatingFrequency;
        ss << NEX_T(", useCache=") << useCache;
        ss << NEX_T(", totalCountEnabled=") << totalCountEnabled;

        ss << NEX_T(")");

        QTRACE(uiTraceFlags, ss.CStr());
    }
};

END_NAMESPACE_NEX

