﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>

BEGIN_NAMESPACE_NEX

class Credentials;

class ServiceHttpClient : public RootObject
{
public:

    /*!
      @brief 優先度タイプ
    */
    enum PriorityType
    {
        PRIORITY_TYPE_UNKNOWN    = 0,    //!< 優先度タイプ不明
        PRIORITY_TYPE_FOREGROUND = 1,    //!< フォアグラウンド通信
        PRIORITY_TYPE_BACKGROUND = 2     //!< バックグラウンド通信
    };

    ServiceHttpClient();

    virtual ~ServiceHttpClient();

    // 必ずNULLを返します。ServiceClientとの互換性のために用意しています。
    Credentials* GetCredentials() const
    {
        return NULL;
    }
#if defined(CAFE_TARGET) || defined(CAFEIOP_TARGET)
    /*!
        @brief      HTTP to PRUDP Proxy 経由で接続する RV の情報とアカウント情報を設定します。

            RMC を呼ぶ前に設定してください。
            falseを返した場合、続けて RMC を呼び出すことはできません。
            指定したアクセスキーが不正な場合、本API呼び出しの結果は true となりますが、続く RMC 呼び出しの結果で QRESULT(Transport, ServerRequestError) が返されます。
            同様にゲームサーバ ID が不正な場合、もしくはゲームサーバ ID に対応するサーバー設定が未完了の場合は続く RMC 呼び出しの結果で  QRESULT(Transport, DnsError) が返されます。

        @param[in] gameServerId アプリケーションに割り当てられたゲームサーバ ID を指定します。
        @param[in] pAccessKey アプリケーションに割り当てられたアクセスキーを指定します。
        @param[in] accountSlotNo サーバーにアクセスしたいアカウントスロット ID を指定します。

        @retval true 成功
        @retval false 指定したスロットにアカウントが見つからないか、ネットワークアカウントではないか、フレンドのアカウントがまだ作成されていません。そのため続行できません。
    */
    virtual qBool SetSandboxContext(qUnsignedInt32 gameServerId, const qChar* pAccessKey, qUnsignedInt8 accountSlotNo);
#else
    /*!
        @brief      HTTP to PRUDP Proxy 経由で接続する RV の情報とアカウント情報を設定します。

            RMC を呼ぶ前に設定してください。
            falseを返した場合、続けて RMC を呼び出すことはできません。
            指定したアクセスキーが不正な場合は続く RMC 呼び出しの結果で QRESULT(Transport, ServerRequestError) が返されます。
            ゲームサーバ ID が不正な場合、もしくはゲームサーバ ID に対応するサーバー設定が未完了の場合は続く RMC 呼び出しの結果で  QRESULT(Transport, DnsError) が返されます。

        @param[in] gameServerId アプリケーションに割り当てられたゲームサーバ ID を指定します。
        @param[in] pAccessKey アプリケーションに割り当てられたアクセスキーを指定します。

        @retval true 成功
        @retval false この本体はプリンシパル ID をまだ取得していないため続行できません。
    */
    virtual qBool SetSandboxContext(qUnsignedInt32 gameServerId, const qChar* pAccessKey);
#endif

    /*!
        @internal
        @brief      デバッグ用にユーザー情報を設定します。強制的に情報を変更する場合に使用します。内部テスト用の関数です。アプリケーションから呼び出さないでください。

            SetSandboxContextの後、 RMC を呼ぶ前に設定してください。

        @param[in] principalId 使用するPID。
        @param[in] password 使用するパスワード。
    */
    void SetUserContext(PrincipalID principalId, const String& password)
    {
        m_PrincipalId = principalId;
        m_Password = password;
    }

    /*!
        @brief      HTTP to PRUDP Proxy との通信に使用される HTTP 通信スレッドの優先度を設定します。

       ロジック制御用サーバーにアクセスするための HTTP 通信スレッドの優先度を指定します。
       セッションごとに優先度を変更することができます。
       データの送受信用 HTTP スレッドの優先度を指定するには DataStoreClient::SetHttpThreadPriority() を使用してください。

        @param[in]  priority HTTP to PRUDP Proxy との通信に使用される HTTP 通信スレッドの優先度。デフォルトは DataStoreConstants::DEFAULT_HTTP_THREAD_PRIORITY です。
    */
    void SetHppHttpThreadPriority(qUnsignedInt32 priority)
    {
        m_HppHttpThreadPriority = priority;
    }

    /*!
        @brief      HTTP to PRUDP Proxy との通信に使用される HTTP 通信スレッドの優先度を取得します。

        @return     設定されている HTTP 通信スレッドの優先度。
    */
    qUnsignedInt32 GetHppHttpThreadPriority() const
    {
        return m_HppHttpThreadPriority;
    }

    /*!
        @internal
        @brief      HTTP to PRUDP Proxy のアドレスを設定します。(デバッグ用)

       HTTP to PRUDP Proxy のアドレスを設定します。本来は SetSandboxContext() により適切なURLがセットされます。
       強制的に URL を変更したい場合に SetSandboxContext() 呼び出しの後本APIを呼び出してください。

        @param[in]  hppUrl HTTP to PRUDP Proxy のアドレスを設定します。
    */
    void SetHppUrl(const String& hppUrl)
    {
        m_HppUrl = hppUrl;
        QLOG(EventLog::Info, NEX_T("HPP URL has been changed: ") << m_HppUrl);
    }

    /*!
        @brief      HTTP to PRUDP Proxy のアドレスを取得します。

        @return     設定されている HTTP to PRUDP Proxy のアドレス。
    */
    String GetHppUrl() const
    {
        return m_HppUrl;
    }

    /*!
        @internal
        @brief      HTTP to PRUDP Proxy との通信に HTTPS を使用するかどうかを指定します。(デバッグ用)

            RMC 呼び出しごとに本関数を呼び出すことにより、セッションごとに HTTPS/HTTP を切り替えることができます。
            ファイルサーバーとの HTTP 通信設定は DataStoreHttpClient::SetUseHttps() を使用してください。

        @param[in]  useHttps HTTPS を使用する場合は true。HTTPを使用する場合は false。デフォルトは true です。
    */
    void SetUseHttps(qBool useHttps)
    {
        m_UseHttps = useHttps;
    }

    /*!
        @brief      HTTP to PRUDP Proxy との通信に HTTPS を使用するかどうかを取得します。

        @return     HTTPS を使用する場合は true。HTTP を使用する場合は false。
    */
    qBool GetUseHttps() const
    {
        return m_UseHttps;
    }

    /*!
        @brief      HTTP to PRUDP Proxy との通信タイムアウト時間を設定します。

            本関数で設定しない場合は 60 秒となります。特に理由がない限りデフォルトのまま使用してください。

        @param[in]  timeoutMilliSecond HTTPS HTTP to PRUDP Proxy との通信タイムアウト時間(ミリ秒)。
    */
    void SetHppTransferTimeout(TimeInterval timeoutMilliSecond)
    {
        m_TimeoutMilliSecond = timeoutMilliSecond;
    }

    /*!
        @brief      HTTP to PRUDP Proxy との通信タイムアウト時間を取得します。

        @return     HTTP to PRUDP Proxy との通信タイムアウト時間。
    */
    qUnsignedInt32 GetHppTransferTimeout() const
    {
        return m_TimeoutMilliSecond;
    }

    /*!
        @internal
        @brief      HTTP to PRUDP Proxy との通信する前に認証サーバーで認証を行うかどうかを設定します。(デバッグ用)

            古いAPIです。本API呼び出しは動作に影響しません。

        @param[in]  skipAuthentication 動作に影響を与えません
    */
    void SetSkipAuthentication(qBool skipAuthentication)
    {
        (void)skipAuthentication;
    }

    /*!
      @brief      HTTP 通信の優先度タイプを設定します。

      HTTP 通信に優先度を表すヘッダーを設定します。本関数で設定後、再度本関数で設定するまで設定した値が有効となります。
      設定値が ServiceHttpClient::PRIORITY_TYPE_UNKNOWN の場合はヘッダーは付加されません。

      @param[in]  priorityType デフォルトは ServiceHttpClient::PRIORITY_TYPE_UNKNOWN です。
    */
    void SetPriorityType(ServiceHttpClient::PriorityType priorityType) { m_PriorityType = priorityType; }

    /*!
      @brief      HTTP 通信の優先度タイプを取得します。

      @return     HTTP 通信の優先度タイプ。
    */
    ServiceHttpClient::PriorityType GetPriorityType() const { return m_PriorityType; }

    /// HTTP to PRUDP Proxy との通信に使用される HTTP 通信スレッドの優先度のデフォルト値です。
    static const qUnsignedInt32 DEFAULT_HTTP_THREAD_PRIORITY = 16;

    /// HTTP to PRUDP Proxy との通信に使用されるデフォルトのタイムアウト値です。(ミリ秒)
    static const qUnsignedInt32 DEFAULT_HPP_TRANSFER_TIMEOUT = 60 * 1000;
protected:
    String MakeHppUrl() const;

    qUnsignedInt32 m_GameId;
    String m_AccessKey;
    PrincipalID m_PrincipalId;
    String m_Password;
    qUnsignedInt32 m_HppHttpThreadPriority;
    String m_HppUrl;
    qBool m_UseHttps;
    TimeInterval m_TimeoutMilliSecond;
#if defined(CAFE_TARGET) || defined(CAFEIOP_TARGET)
    qUnsignedInt8 m_AccountSlotNo;
    String m_NfsEnv;            // Cafeのみenumではなく文字列で取得する仕様
#else
    qUnsignedInt32 m_NfsType;   // NFSTypeは外に公開したくないのでqUnsignedInt32で代用
    qUnsignedInt8 m_NfsNumber;
#endif
    ServiceHttpClient::PriorityType m_PriorityType;
private:
    qBool InitializeNfsLibrary() const;
    void FinalizeNfsLibrary() const;
    qBool GetPrincipalId(PrincipalID& outPrincipalId) const;
    qBool GetPassword(String& outPassword) const;
#if defined(CAFE_TARGET) || defined(CAFEIOP_TARGET)
    qBool GetNfsSettings(String& nfsEnv) const;
#else
    qBool GetNfsSettings(qUnsignedInt32& nfsType, qUnsignedInt8& nfsNumber) const;
#endif
    qBool SetSandboxContextImpl(qUnsignedInt32 gameId, const qChar* pAccessKey);
};

END_NAMESPACE_NEX

