﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Services/Jugem/Debug/src/Common/DebugDDF.h>
#include <RendezVous/Services/Jugem/Debug/src/Common/DebugProtocolDDL.h>
#include <RendezVous/Services/Jugem/Debug/src/Common/ApiCallData.h>
#include <RendezVous/Services/Jugem/Debug/src/Common/ApiCallSummary.h>
#include <RendezVous/Services/Jugem/Debug/src/Common/DebugConstants.h>

BEGIN_NAMESPACE_NEX

/*!
    @brief デバッグ補助機能を提供するライブラリです。

    注意：本機能はデバッグ専用の機能のため、製品ロムに含めてはいけません。

    DebugClient ではNEXでのサーバーを介するAPI呼び出しをサーバー側で記録・履歴の取得や、
    デバッグを効率化するために、ゲームサーバーの設定を変更することができます。

    将来的には、ゲームサーバー全体の設定変更機能については管理用のWebシステムが提供される予定です。
    その際には、 DebugClient からゲームサーバーの設定変更関数は削除されます。
*/
class DebugClient : public ServiceClient
{
public:

    /*!
        @brief         コンストラクタです。
    */
    DebugClient();

    /*!
        @brief         デストラクタです。
    */
    virtual ~DebugClient();

    void UpdateProtocolsDefaultCredentials(Credentials* pCredentials);

    /*!
     @brief    サーバーでのAPI呼び出しの記録を有効化する。

     サーバーでのAPI呼び出しの記録を有効化します。

     通常、API呼び出しの記録は無効化されていますので、 DebugClient::EnableApiRecorder() を用いて記録を有効化してください。
     なおAPI呼び出しが1時間以上なかった場合には自動的に無効化されます。

     @param[in,out]    pContext        呼び出し情報。

     @retval     true 非同期処理の開始に成功。
     @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool EnableApiRecorder(ProtocolCallContext* pContext);

    /*!
     @brief    サーバーでのAPI呼び出しの記録を無効化する。

     サーバーでのAPI呼び出しの記録を無効化します。
     無効化に伴ってサーバー上の記録したAPI呼び出しの履歴も全て消去されます。

     API呼び出しの記録の有効・無効はサーバー全体で一つの状態しか持たないため、
     DisableApiRecorder を呼ぶことで、他の DebugClient を利用しているクライアントにも
     影響が出ることにご注意ください。

     @param[in,out]    pContext        呼び出し情報。

     @retval     true 非同期処理の開始に成功。
     @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool DisableApiRecorder(ProtocolCallContext* pContext);

    /*!
     @brief    サーバーでのAPI呼び出しの記録が有効であるかを取得する。

     @param[in,out]    pContext        呼び出し情報。
     @param[out]   IsEnabled       サーバーの状態。

     @retval     true 非同期処理の開始に成功。
     @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool IsApiRecorderEnabled(ProtocolCallContext* pContext, bool* IsEnabled);

    /*!
     @brief    サーバーよりAPI呼び出しの履歴を取得する。

     指定した条件でAPI呼び出しの履歴をリストとして取得します。
     取得したリストは呼び出し順に並んでいます。

     取得するAPI呼び出しの条件指定には、呼び出し時刻と呼び出したクライアントの プリンシパル ID  が指定できます。

     指定した時刻より後の履歴を取得するには、 sinceTime 引数で指定してください。
     同様に指定した時刻より前の履歴を取得するには、 untilTime 引数で指定してください。
     指定したくない場合には DateTime::Never を引数として渡しください。

     特定の プリンシパル ID  の履歴のみ取得する際には、principalIds に取得したい プリンシパル ID  のリストを与えてください。
     指定しない場合には空のリストを与えてください。

     非同期処理の結果、以下の Result が返ることがあります。

     @li    QERROR(RendezVous, InvalidConfiguration) : API呼び出しの記録機能が無効化されている

     @param[in,out]    pContext        呼び出し情報。
     @param[out]   pApiCallDatas   取得したAPI呼び出し履歴。
     @param[in]    principalIds    プリンシパル ID  のリスト。
     @param[in]    sinceTime       指定する時刻。
     @param[in]    untilTime       指定する時刻。

     @retval     true 非同期処理の開始に成功。
     @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool GetApiCalls(ProtocolCallContext *pContext,
                      qList<ApiCallData>* pApiCallDatas,
                      qList<PrincipalID>& principalIds,
                      DateTime sinceTime = DateTime::Never,
                      DateTime untilTime = DateTime::Never
                     );

    /*!
     @brief    API呼び出し履歴を集計し、呼び出し頻度の制限内にあるかどうかを取得します

     指定された条件に合致するAPIの呼び出し情報を集計し、呼び出し回数の総計や、呼び出し回数がガイドラインで
     規定されているAPIの呼び出し制限内に収まっているかの判定結果等を取得します。

     取得するAPI呼び出しの条件指定には、呼び出し時刻と呼び出したクライアントの プリンシパル ID  が指定できます。

     指定した時刻より後の履歴を取得するには、 sinceTime 引数で指定してください。
     同様に指定した時刻より前の履歴を取得するには、 untilTime 引数で指定してください。
     指定したくない場合には DateTime::Never を引数として渡しください。

     非同期処理の結果、以下の Result が返ることがあります。

     @li    QERROR(RendezVous, InvalidConfiguration) : API呼び出しの記録機能が無効化されている

     @param[in,out]    pContext              呼び出し情報。
     @param[out]   pApiCallSummaryList   呼び出し履歴の集計情報のリスト
     @param[in]    principalId           取得対象のプリンシパル ID
     @param[in]    sinceTime             開始時刻
     @param[in]    untilTime             終了時刻
     @param[in]    isLimitOverOnly       呼び出し頻度制限の目安を超えているAPIのみ取得するかどうか

     @retval     true 非同期処理の開始に成功。
     @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool GetApiCallSummary(ProtocolCallContext *pContext,
                            qList<ApiCallSummary> *pApiCallSummaryList,
                            PrincipalID principalId,
                            DateTime sinceTime = DateTime::Never,
                            DateTime untilTime = DateTime::Never,
                            qBool isLimitOverOnly = true);

    /*!
     @brief    オートマッチで過去参加したマッチメイクセッションを参加対象外とするかどうかを設定します

     デフォルト設定では、 @ref MatchmakeExtensionClient::AutoMatchmake() では、過去3回にわたって参加したマッチメイクセッションに関して、
     サーバー側で参加対象から外しています。アプリケーションのデバッグ時に、 @ref MatchmakeExtensionClient::AutoMatchmake() 同じマッチメイクセッションに
     参加したいという場合に本関数を使用してください。

     ゲームサーバー全体の設定となりますので使用にあたっては注意してください。
     なお、ゲームサーバーのデバッグ設定管理用 Web システムが提供され次第、本関数は使用できなくなります。

     非同期処理の結果、以下の Result が返ることがあります。

     @li    QERROR(Core, NotImplemented) : マッチメイク機能を使用しないゲームサーバー

     @param[in,out]    pContext        呼び出し情報。
     @param[in]    bAvoidPreviousGathering    オートマッチで過去参加したマッチメイクセッションを参加対象外とするかどうか。
                                              trueの場合、過去に参加したマッチメイクセッションに参加しない(ゲームサーバーのデフォルト設定)、
                                              false の場合、過去に参加したマッチメイクセッションに参加する。
     */
    qBool SetExcludeJoinedMatchmakeSession(ProtocolCallContext *pContext, qBool bAvoidPreviousGathering);

    /*!
     @brief    オートマッチで過去参加したマッチメイクセッションを参加対象外とするかどうかを取得します

     非同期処理の結果、以下の Result が返ることがあります。

     @li    QERROR(Core, NotImplemented) : マッチメイク機能を使用しないゲームサーバー

     @param[in,out]    pContext        呼び出し情報。
     @param[in]    bpAvoidPreviousGathering    オートマッチで過去参加したマッチメイクセッションを参加対象外とするかどうか。
                                               trueの場合、過去に参加したマッチメイクセッションに参加しない(ゲームサーバーのデフォルト設定)、
                                               false の場合、過去に参加したマッチメイクセッションに参加する。
     */
    qBool GetExcludeJoinedMatchmakeSession(ProtocolCallContext *pContext, qBool* bpAvoidPreviousGathering);

private:
    DebugProtocolClient m_oProtocol;
};

END_NAMESPACE_NEX

