﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <RendezVous/Core/src/Common/ProtocolCallContext.h>

BEGIN_NAMESPACE_NEX

class JobIndependentServer;

/*!
    @brief 独自サーバーの処理を行うライブラリです。

    内部で nn::os::Event オブジェクトを一つ使用します。

    非同期処理が行われている間は定期的に@ref Scheduler::Dispatch() を呼び出す必要があります。
*/
class IndependentServer : public RootObject
{
public:
    /*!
        @brief 認証トークン取得時に必要なバッファサイズです。認証トークンのNULL終端文字分を含みます。
    */
    static const qUnsignedInt32 AUTH_TOKEN_SIZE = 256;

public:
    /*!
        @brief         コンストラクタです。
    */
    IndependentServer();

    /*!
        @brief         デストラクタです。
    */
    virtual ~IndependentServer();

    /*!
        @brief         独自サーバーの認証トークンを要求します。

        認証サーバーでゲームの認証処理を行った後、認証トークンの取得を行います。
        本体が プリンシパル ID 未取得の場合、本関数の非同期処理中にフレンドサーバへのアクセスを行い プリンシパル ID を取得します。

        本関数はライブラリ内部で nn::friends ライブラリの非同期処理を呼び出します。
        本関数の非同期処理実行中に nn::friends::Login() などの nn::friends ライブラリの
        非同期処理を呼びださないでください。

        本関数では、 ProtocolCallContext::SetTimeout() や ProtocolCallContext::Cancel() の指定は無効です。これらの操作を行わないでください。

        非同期処理の結果は @ref CallContext::GetOutcome() で成功・失敗の判断を行います。
        失敗の場合、ネットワークエラーコードは @ref GetLastErrorCode() で取得して下さい。

        checkParentalControl が false の場合、「他のユーザーとのインターネット通信」のペアレンタルコントロールの
        チェックを行いません。そのため、サービス内容に応じて正しく設定するようにしてください。
        

        @param[in,out]     pContext   呼び出し情報。
        @param[in]     gameServerId ゲームサーバ ID。
        @param[in]     pKeyHash   キーハッシュ値。
        @param[out]    pAuthToken 認証トークンを受け取るバッファ。@n
                                  バッファサイズは @ref AUTH_TOKEN_SIZE 以上にして下さい。
        @param[in]     checkParentalControl ペアレンタルコントロールのチェックを行うかどうか。

        @return        処理の結果が返ります。
        @retval        "QSUCCESS(Core,Success)"               処理の起動に成功。
        @retval        "QERROR(Core,InvalidArgument)"         引数が不正。
        @retval        "QERROR(Core,CallInitiationFailure)"   CallContext の状態が不正。
        @retval        "QERROR(FPD,NotInitialized)"           フレンドライブラリが初期化されていない。
        @retval        "QERROR(FPD,NotConnected)"             NFS にログインしていない。(SDK 3.2より前のSDKを使用している場合)
    */
    qResult RequestAuthenticationToken(ProtocolCallContext* pContext,
                                       qUnsignedInt32 gameServerId,
                                       const qChar8* pKeyHash,
                                       qChar8* pAuthToken,
                                       qBool checkParentalControl=true);

    /*!
        @brief         認証トークンの要求処理で発生したネットワークエラーコードを取得します。

                       @ref RequestAuthenticationToken() の非同期処理に失敗した場合にのみ有効な値がセットされます。

        @return        ネットワークエラーコード。
    */
    qInt32 GetLastErrorCode() const;

    /*!
        @brief         サーバー時刻(UTC)を取得します。

                       フレンドサーバーへログインしている必要があります。

        @param[out]    pServerTime  サーバー時刻。

        @return        処理結果が返ります。

        @retval        true     成功
        @retval        false    失敗(フレンドサーバーにログインしていない)
    */
    static qBool GetServerTime(DateTime* pServerTime);

protected:
    friend class JobIndependentServer;

protected:
    // ネットワークエラーコード
    qInt32 m_errorCode;
};

END_NAMESPACE_NEX

