﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringID.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/AutoMatchmakeParamDDL.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeConstants.h>


BEGIN_NAMESPACE_NEX

/*!
@brief      MatchmakeExtensionClient::AutoMatchmake() に関するパラメータを格納するデータクラスです。
*/
class AutoMatchmakeParam: public DDLCLASS(AutoMatchmakeParam)
{
public:

    /*!
    @brief      コンストラクタです。

    コンストラクタです。最低限 AutoMatchmakeParam::SetSourceMatchmakeSession() で作成するマッチメイクセッションを設定する必要があります。
    */
    AutoMatchmakeParam()
    {
        Reset();
    };
    
    /*!
     @brief    デストラクタです。
     */
    virtual ~AutoMatchmakeParam() {}

    /*!
    @brief      適切なマッチメイクセッションが見つからなかった場合に作成するマッチメイクセッションを設定します。

    AutoMatchmakeParam::SetSearchCriteria() もしくは AutoMatchmakeParam::SetTargetGatheringId() で条件が指定されない場合はここで指定したマッチメイクセッションの

    @li ゲームモード
    @li 最小・最大参加人数
    @li マッチメイクシステムタイプ
    @li 検索属性

    が全て一致する物でかつ参加がロックされていない物、人数に余裕がある物、P2P 通信のホストが決定している物を検索します。
    AutoMatchmakeParam::SetSearchCriteria() もしくは AutoMatchmakeParam::SetTargetGatheringId() で条件が指定された場合はその条件で検索します。
    検索条件に一致する物が見つからなかった場合は sourceMatchmakeSession に指定したパラメータでマッチメイクセッションを作成します。

    @param[in] sourceMatchmakeSession_ 適切なマッチメイクセッションが見つからなかった場合に作成するマッチメイクセッション
    */
    void SetSourceMatchmakeSession(const MatchmakeSession& sourceMatchmakeSession_) { this->sourceMatchmakeSession = sourceMatchmakeSession_; }
    /*!
    @brief      適切なマッチメイクセッションが見つからなかった場合に作成するマッチメイクセッションを取得します。

    @return 適切なマッチメイクセッションが見つからなかった場合に作成するマッチメイクセッション。
    */
    const MatchmakeSession& GetSourceMatchmakeSession() const { return sourceMatchmakeSession; }

    /*!
    @brief      追加で参加させるユーザーを設定します。

    自分が参加すると同時に他人も同時にマッチメイクセッションに参加させたい場合、本関数でそのプリンシパル ID を指定します。
    例えば、フレンドとチームを組んで対戦したいといった場合に利用できます。
    対象のユーザーは NGS にログインしている必要があります。
    追加で参加したユーザに対しては NotificationEvents::AddedToGathering の通知が送信されます。
    追加で参加したユーザは MatchmakeExtensionClient::FindMatchmakeSessionByGatheringIdDetail() で P2P 通信用の共通鍵を含んだマッチメイクセッションの詳細情報を取得できます。
    初期状態は空です。

    @param[in] additionalParticipants_ 追加で参加させるユーザー。

    @see AutoMatchmakeParam::SetGatheringIdForParticipationCheck()
    */
    void SetAdditionalParticipant(const qList<PrincipalID>& additionalParticipants_) { this->additionalParticipants = additionalParticipants_; }
    /*!
    @brief      追加で参加させるユーザーを取得します。

    @return 追加で参加させるユーザー。
    */
    const qList<PrincipalID>& GetAdditionalParticipants() const { return additionalParticipants; }

    /*!
    @brief      自分と追加で参加させるユーザーが、指定したギャザリングに参加しているかを確認するためのギャザリング ID を設定します。

    自分を含む参加者全員が特定のギャザリングに参加している事を条件とする場合は本関数でそのギャザリング ID を指定します。
    初期状態は INVALID_GATHERINGID です。

    @param[in] gidForParticipationCheck_ 参加を条件とするギャザリングの ID 。INVALID_GATHERINGID を指定するとチェックは行われません。

    @see AutoMatchmakeParam::SetAdditionalParticipant()
    */
    void SetGatheringIdForParticipationCheck(GatheringID gidForParticipationCheck_) { this->gidForParticipationCheck = gidForParticipationCheck_; }
    /*!
    @brief      自分と追加で参加させるユーザーが、指定したギャザリングに参加しているかを確認するためのギャザリング ID を取得します。

    @return 参加を条件とするギャザリングの ID 。
    */
    GatheringID GetGatheringIdForParticipationCheck() const { return gidForParticipationCheck; }

    /*!
    @brief      AutoMatchmakeOption で列挙されるオプションビットフラグを設定します。

    初期状態は AUTO_MATCHMAKE_OPTION_NONE です。

    @param[in] autoMatchmakeOption_ オプションビットフラグ。
    */
    void SetAutoMatchmakeOption(qUnsignedInt32 autoMatchmakeOption_) { this->autoMatchmakeOption = autoMatchmakeOption_; }
    /*!
    @brief      AutoMatchmakeOption で列挙されるオプションビットフラグを取得します。

    @return オプションビットフラグ。
    */
    qUnsignedInt32 GetAutoMatchmakeOption() const { return autoMatchmakeOption; }

    /*!
    @brief      ギャザリングのオーナーもしくは参加者に通知されるメッセージを設定します。

    このメッセージはユーザーごとにサーバーに保存され、MatchmakeExtensionClient::GetDetailedParticipants() で取得できます。
    AutoMatchmakeParam::SetAdditionalParticipant() で参加させたユーザーのメッセージも本関数で設定したものになります。
    初期状態は空文字列です。

    @param[in] joinMessage_ ギャザリングのオーナーもしくは参加者に通知されるメッセージ。
    @see MatchmakeExtensionClient::GetDetailedParticipants()
    */
    void SetJoinMessage(const String& joinMessage_) { this->joinMessage = joinMessage_; }
    /*!
    @brief      ギャザリングのオーナーもしくは参加者に通知されるメッセージを取得します。

    @return ギャザリングのオーナーもしくは参加者に通知されるメッセージ。
    */
    const String& GetJoinMessage() const { return joinMessage; }

    /*!
    @brief      マッチメイクセッションの検索条件を設定します。

    指定したマッチメイクセッション検索条件でマッチメイクセッション(ギャザリング)を検索し、一致するものが見つかればそこに参加します。
    検索条件はリストとして複数渡すことができ、先頭の検索条件から順番に使用されます。一つの検索条件で最大 50 件のマッチメイクセッションを列挙し、
    それらへの参加を試みます。参加できた場合はその後の検索は行われません。
    マッチメイクセッションが見つからなかったり、見つかった全てのマッチメイクセッションへの参加ができなかった場合は次の検索条件で検索します。
    したがって条件を徐々に緩めながらマッチメイクセッションを検索していくことが可能です。
    全ての検索条件に合致するものが存在しない場合、AutoMatchmakeParam::SetSourceMatchmakeSession() に指定したパラメータでマッチメイクセッションを作成します。
    検索条件は最大 2 つまで指定できます。3 つ以上指定した場合は QERROR(Core, InvalidArgument) となります。
    本関数と AutoMatchmakeParam::SetTargetGatheringId() は排他です。両方を指定することはできません。
    初期状態は空です。

    @param[in] searchCriteria マッチメイクセッションの検索条件。最大 2 つまで指定できます。
    @see AutoMatchmakeParam::SetTargetGatheringId()
    */
    void SetSearchCriteria(const qList<MatchmakeSessionSearchCriteria>& searchCriteria) { this->lstSearchCriteria = searchCriteria; }
    /*!
    @brief      マッチメイクセッションの検索条件を取得します。

    @return マッチメイクセッションの検索条件。
    */
    const qList<MatchmakeSessionSearchCriteria>& GetSearchCriteria() const { return lstSearchCriteria; }

    /*!
    @brief      参加したいギャザリング ID を設定します。

    ギャザリング ID を指定してマッチメイクセッション(ギャザリング)に参加します。
    ギャザリング ID は複数指定でき、与えられたギャザリング ID のリストの先頭から順に参加を試みます。
    本関数を使用して MatchmakeExtensionClient::AutoMatchmake() を行った場合、過去 3 回にわたって参加したマッチメイクセッションを参加の対象から取り除く処理は含まれていません。
    アプリケーション側で MatchmakeExtensionClient::BrowseMatchmakeSession() を用いて検索を行い、検索結果のフィルタリングを行った後で、本関数を用いてマッチメイクセッションに参加、
    もしくはを作成するような実装は許容されています。しかし多くの場合、 検索条件を指定できる MatchmakeExtensionClient::AutoMatchmake() を使用しても、
    MatchmakeSessionSearchCriteria を適切に指定することで同様のことは実現できます。まずは検索条件を指定できる AutoMatchmakeParam::SetSearchCriteria() の利用をご検討ください。

    その上で本関数を使用される場合、指定したマッチメイクセッションに参加できないなどの理由で不必要にマッチメイクセッションが作成されないよう、
    ギャザリング ID は複数指定し、リストの並び順はなるべくクライアント毎に異なるようにするような実装を強く推奨します。
    全てのマッチメイクセッションに参加できなかった場合、AutoMatchmakeParam::SetSourceMatchmakeSession() に指定したパラメータでマッチメイクセッションを作成します。
    本関数と AutoMatchmakeParam::SetSearchCriteria() は排他です。両方を指定することはできません。
    初期状態は空です。

    @param[in] targetGids_ 参加したいギャザリング ID。
    @see AutoMatchmakeParam::SetSearchCriteria()
    */
    void SetTargetGatheringId(const qList<GatheringID>& targetGids_) { this->targetGids = targetGids_; }
    /*!
    @brief      参加したいギャザリング ID を取得します。

    @return 参加したいギャザリング ID。
    */
    const qList<GatheringID>& GetTargetGatheringId() const { return targetGids; }

    /*!
    @internal
    @brief      このクライアントからの参加人数を設定します。。

    自分自身を何人分としてカウントするかを指定します。
    1 以上、かつ作成するマッチメイクセッションの最大参加人数以下の値を設定してください。
    2 以上を指定し、かつ CreateMatchmakeSessionParam::SetAdditionalParticipant() で追加参加者を指定することはできません。
    初期状態は 1 です。

    @param[in] participationCount_ このクライアントからの参加人数。
    */
    void SetParticipationCount(const qUnsignedInt16 participationCount_) { this->participationCount = participationCount_; }
    /*!
    @internal
    @brief      このクライアントからの参加人数を取得します。

    @return このクライアントからの参加人数。
    */
    qUnsignedInt16 GetParticipationCount() const { return participationCount; }

    /*!
    @brief      ブロックリストの挙動を指定するパラメータを設定します。

    マッチメイクセッションへ参加する際にブロックリストのチェックを行うかどうかを指定するパラメータを設定します。
    blockListParam_ の初期状態は MatchmakeBlockListParam::SetUseMyBlockList(true) および MatchmakeBlockListParam::SetUseBlockListByOthers(true) された状態です。

    @param[in] blockListParam_ ブロックリストの挙動を指定するパラメータ。
    */
    void SetBlockListParam(const MatchmakeBlockListParam& blockListParam_) { this->blockListParam = blockListParam_; }

    /*!
    @brief      ブロックリストの挙動を指定するパラメータを取得します。

    @return ブロックリストの挙動を指定するパラメータ。
    */
    const MatchmakeBlockListParam& GetBlockListParam() const { return this->blockListParam; }

    /*!
    @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetSourceMatchmakeSession(MatchmakeSession());
        SetAdditionalParticipant(qList<PrincipalID>());
        SetGatheringIdForParticipationCheck(INVALID_GATHERINGID);
        SetAutoMatchmakeOption(AUTO_MATCHMAKE_OPTION_NONE);
        SetJoinMessage(String());
        SetSearchCriteria(qList<MatchmakeSessionSearchCriteria>());
        SetTargetGatheringId(qList<GatheringID>());
        SetParticipationCount(1);
        SetBlockListParam(MatchmakeBlockListParam());
    }

    /*!
    @brief      AutoMatchmakeParam オブジェクトの内容をトレースします。

    @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
    @internal
    @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const AutoMatchmakeParam& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const AutoMatchmakeParam& rhs)
{
    oStr << NEX_T("AutoMatchmakeParam: (")
               << NEX_T("sourceMatchmakeSession=(not implemented)")
               << NEX_T(", joinMessage=") << rhs.joinMessage;

    {
        oStr << NEX_T(", additionalParticipants=(");
        qList<PrincipalID>::const_iterator itr = rhs.additionalParticipants.begin();
        for (int i = 0; itr != rhs.additionalParticipants.end(); ++i, ++itr)
        {
            if (i != 0)
            {
                oStr << NEX_T(", ");
            }
            oStr << *itr;
        }
        oStr << NEX_T(")");
    }

    oStr << NEX_T(", lstSearchCriteria=(not implemented)");

    {
        oStr << NEX_T(", targetGids=(");
        qList<GatheringID>::const_iterator itr = rhs.targetGids.begin();
        for (int i = 0; itr != rhs.targetGids.end(); ++i, ++itr)
        {
            if (i != 0)
            {
                oStr << NEX_T(", ");
            }
            oStr << *itr;
        }
        oStr << NEX_T(")");
    }
    oStr << NEX_T(", participationCount=") << rhs.participationCount;
    oStr << rhs.blockListParam;
    oStr << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX

