﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <RendezVous/Services/Jugem/Matchmake/src/Common/PersistentGathering.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeConstants.h>

BEGIN_NAMESPACE_NEX

// このヘッダファイルは API リファレンス生成のために用意しています。
// 実体はPersistentGathering.h にあります。
#ifdef FOR_DOC_PURPOSE
/*!
 @brief    コミュニティ種別
 */
namespace CommunityType
{
/*!
 @brief    コミュニティ種別
 */
typedef enum
{
    Open,              //!< 一般コミュニティ
    PasswordLocked,    //!< パスワード付コミュニティ
    Official           //!< 公式コミュニティ
} CommunityType;
}

/*!
 @brief コミュニティの情報を格納するクラスです。コミュニティ機能は将来的に廃止予定であるため、同等の機能である永続ギャザリングの使用を推奨します。
 */
class Community: public Gathering
{
public:

    /*!
    @brief      コンストラクタです。

       初期状態はコミュニティタイプはOpen、属性は全て 0 となります。
    */
    Community();

    /*!
    @brief      デストラクタです。
    */
    virtual ~Community() {};

    /*!
    @brief      セットしたデータを初期化します。

      コミュニティタイプはOpen、属性は全て 0 となります。
    */
    void Reset();

    /*!
    @brief      コミュニティタイプをセットします。

    @param[in] type コミュニティタイプ
    */
    void SetCommunityType(CommunityType::CommunityType type);

    /*!
     @brief    コミュニティタイプを取得します。

     @return   コミュニティタイプ
     */
    CommunityType::CommunityType GetCommunityType() const;

    /*!
     @brief    コミュニティ参加時のパスワードをセットします。

     パスワードはコミュニティタイプが PasswordLocked の場合にのみ有効です。

     @param[in] password コミュニティ参加時のパスワード
     */
    void SetPassword(const String& password)
    {
        m_Password = password;
    }

    /*!
     @brief    コミュニティ参加時のパスワードを取得します。

     ゲームサーバーからコミュニティを取得したときにはパスワードはセットされません。

     @return コミュニティ参加時のパスワード
     */
    const String& GetPassword() const
    {
        return m_Password;
    }

    /*!
    @brief      属性をセットします。

       このマッチメイクセッションの属性をセットします。最大6つ指定できます。

    @param[in]     uiIndex    セットする属性のインデックス。0以上NUM_MATCHMAKE_SESSION_ATTRIBUTES - 1以下の値を指定できます。
    @param[in]     val    アプリケーション定義の属性値。
    */
    void SetAttribute(qUnsignedInt32 uiIndex, qUnsignedInt32 val);

    /*!
    @brief      属性を取得します。

       このマッチメイクセッションにセットされた属性を取得します。

    @param[in]     uiIndex    取得する属性のインデックス。0以上NUM_MATCHMAKE_SESSION_ATTRIBUTES - 1以下の値を指定できます。

    @return     指定されたインデックスの属性値。
    */
    qUnsignedInt32 GetAttribute(qUnsignedInt32 uiIndex) const;

    /*!
    @brief      アプリケーション定義のデータをセットします。

       このマッチメイクセッションにアプリケーション定義のデータをセットします。最大512byteです。

    @param[in]     buffer    アプリケーション定義のデータ。
    */
    void SetApplicationBuffer(qVector<qByte> buffer)
    {
        m_ApplicationBuffer = buffer;
    }

    /*!
    @brief      アプリケーション定義のデータを取得します。

       このマッチメイクセッションにセットされたアプリケーション定義のデータを取得します。

    @return    アプリケーション定義のデータの参照
    */
    const qVector<qByte>& GetApplicationBuffer() const
    {
        return m_ApplicationBuffer;
    }

    /*!
    @brief      このコミュニティに参加可能となる日時を設定します。

       この日時より前に MatchmakeExtensionClient::JoinCommunity() を実行しても参加することはできません。
       @ref DateTime::Never を設定すると、参加可能日時の条件は無視されます。
       日時は UTC 時刻で入力してください。

    @param[in]     date    コミュニティに参加可能となる日時
    */
    void SetParticipationStartDate(const DateTime& date)
    {
        m_ParticipationStartDate = date;
    }

    /*!
    @brief      このコミュニティに参加可能となる日時を取得します。

    @return    コミュニティに参加可能となる日時
    */
    const DateTime& GetParticipationStartDate() const
    {
        return m_ParticipationStartDate;
    }

    /*!
    @brief      このコミュニティへの参加締め切り日を設定します。

       この日時より後に MatchmakeExtensionClient::JoinCommunity() を実行しても参加することはできません。
       @ref DateTime::Never を設定すると、参加締め切り日の条件は無視されます。
       日時は UTC 時刻で入力してください。

    @param[in]     date    コミュニティへの参加締め切る日時
    */
    void SetParticipationEndDate(const DateTime& date)
    {
        m_ParticipationEndDate = date;
    }

    /*!
    @brief      このコミュニティへの参加締め切り日を取得します。

    @return    コミュニティへの参加を締め切る日時
    */
    const DateTime& GetParticipationEndDate() const
    {
        return m_ParticipationEndDate;
    }

    /*!
    @brief      このコミュニティに関連するコミュニティマッチの数を取得します。

    @return    このコミュニティに関連するコミュニティマッチの数
    @see MatchmakeSession::SetReferId()
    */
    qUnsignedInt32 GetMatchmakeSessionCount() const
    {
        return m_MatchmakeSessionCount;
    }

    /*!
    @brief      このコミュニティへの参加人数を取得します

    @return    このコミュニティへの参加人数
    */
    qUnsignedInt32 GetParticipationCount() const
    {
        return m_ParticipationCount;
    }
};
#endif

END_NAMESPACE_NEX

