﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringID.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/CreateMatchmakeSessionParamDDL.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeConstants.h>


BEGIN_NAMESPACE_NEX

/*!
    @brief      MatchmakeExtensionClient::CreateMatchmakeSession() に関するパラメータを格納するデータクラスです。
*/
class CreateMatchmakeSessionParam: public DDLCLASS(CreateMatchmakeSessionParam)
{
public:

    /*!
    @brief      コンストラクタです。

    コンストラクタです。最低限 CreateMatchmakeSessionParam::SetSourceMatchmakeSession() で作成するマッチメイクセッションを設定する必要があります。
    */
    CreateMatchmakeSessionParam()
    {
        Reset();
    };
    
    /*!
     @brief    デストラクタです。
     */
    virtual ~CreateMatchmakeSessionParam() {}

    /*!
    @brief      作成するマッチメイクセッションを設定します。

    sourceMatchmakeSession に指定したパラメータでマッチメイクセッションを作成します。

    @param[in] sourceMatchmakeSession_ 作成するマッチメイクセッション。
    */
    void SetSourceMatchmakeSession(const MatchmakeSession& sourceMatchmakeSession_) { this->sourceMatchmakeSession = sourceMatchmakeSession_; }
    /*!
    @brief      作成するマッチメイクセッションを取得します。

    @return 作成するマッチメイクセッション。
    */
    const MatchmakeSession& GetSourceMatchmakeSession() const { return sourceMatchmakeSession; }

    /*!
    @brief      追加で参加させるユーザーを設定します。

    自分が参加すると同時に他人も同時にマッチメイクセッションに参加させたい場合、本関数でそのプリンシパル ID を指定します。
    例えば、フレンドとチームを組んで対戦したいといった場合に利用できます。
    対象のユーザーは NGS にログインしている必要があります。
    追加で参加したユーザに対しては NotificationEvents::AddedToGathering の通知が送信されます。
    追加で参加したユーザは MatchmakeExtensionClient::FindMatchmakeSessionByGatheringIdDetail() で P2P 通信用の共通鍵を含んだマッチメイクセッションの詳細情報を取得できます。
    初期状態は空です。

    @param[in] additionalParticipants_ 追加で参加させるユーザー。

    @see CreateMatchmakeSessionParam::SetGatheringIdForParticipationCheck()
    */
    void SetAdditionalParticipant(const qList<PrincipalID>& additionalParticipants_) { this->additionalParticipants = additionalParticipants_; }
    /*!
    @brief      追加で参加させるユーザーを取得します。

    @return 追加で参加させるユーザー。
    */
    const qList<PrincipalID>& GetAdditionalParticipants() const { return additionalParticipants; }

    /*!
    @brief      自分と追加で参加させるユーザーが、指定したギャザリングに参加しているかを確認するためのギャザリング ID を設定します。

    自分を含む参加者全員が特定のギャザリングに参加している事を条件とする場合は本関数でそのギャザリング ID を指定します。
    初期状態は INVALID_GATHERINGID です。

    @param[in] gidForParticipationCheck_ 参加を条件とするギャザリングの ID 。INVALID_GATHERINGID を指定するとチェックは行われません。

    @see CreateMatchmakeSessionParam::SetAdditionalParticipant()
    */
    void SetGatheringIdForParticipationCheck(GatheringID gidForParticipationCheck_) { this->gidForParticipationCheck = gidForParticipationCheck_; }
    /*!
    @brief      自分と追加で参加させるユーザーが、指定したギャザリングに参加しているかを確認するためのギャザリング ID を取得します。

    @return 参加を条件とするギャザリングの ID 。
    */
    GatheringID GetGatheringIdForParticipationCheck() const { return gidForParticipationCheck; }

    /*!
    @internal
    @brief      CreateMatchmakeSessionOption で列挙されるオプションビットフラグを設定します。

    初期状態は CREATE_MATCHMAKE_SESSION_OPTION_NONE です。

    @param[in] createMatchmakeSessionOption_ オプションビットフラグ。
    */
    void SetCreateMatchmakeSessionOption(qUnsignedInt32 createMatchmakeSessionOption_) { this->createMatchmakeSessionOption = createMatchmakeSessionOption_; }
    /*!
    @internal
    @brief      CreateMatchmakeSessionOption で列挙されるオプションビットフラグを取得します。

    @return オプションビットフラグ。
    */
    qUnsignedInt32 GetCreateMatchmakeSessionOption() const { return createMatchmakeSessionOption; }

    /*!
    @brief      設定によりギャザリングの参加者に通知されるメッセージを設定します。

    このメッセージはユーザーごとにサーバーに保存され、MatchmakeExtensionClient::GetDetailedParticipants() で取得できます。
    CreateMatchmakeSessionParam::SetAdditionalParticipant() で参加させたユーザーのメッセージも本関数で設定したものになります。
    初期状態は空文字列です。

    @param[in] joinMessage_ 設定によりギャザリングの参加者に通知されるメッセージ。
    @see MatchmakeExtensionClient::GetDetailedParticipants()
    */
    void SetJoinMessage(const String& joinMessage_) { this->joinMessage = joinMessage_; }
    /*!
    @brief      設定によりギャザリングの参加者に通知されるメッセージを取得します。

    @return 設定によりギャザリングの参加者に通知されるメッセージ。
    */
    const String& GetJoinMessage() const { return joinMessage; }

    /*!
    @internal
    @brief      このクライアントからの参加人数を設定します。。

    自分自身を何人分としてカウントするかを指定します。
    1 以上、かつ作成するマッチメイクセッションの最大参加人数以下の値を設定してください。
    2 以上を指定し、かつ CreateMatchmakeSessionParam::SetAdditionalParticipant() で追加参加者を指定することはできません。
    初期状態は 1 です。

    @param[in] participationCount_ このクライアントからの参加人数。
    */
    void SetParticipationCount(const qUnsignedInt16 participationCount_) { this->participationCount = participationCount_; }
    /*!
    @internal
    @brief      このクライアントからの参加人数を取得します。

    @return このクライアントからの参加人数。
    */
    const qUnsignedInt16& GetParticipationCount() const { return participationCount; }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetSourceMatchmakeSession(MatchmakeSession());
        SetAdditionalParticipant(qList<PrincipalID>());
        SetGatheringIdForParticipationCheck(INVALID_GATHERINGID);
        SetCreateMatchmakeSessionOption(CREATE_MATCHMAKE_SESSION_OPTION_NONE);
        SetJoinMessage(String());
        SetParticipationCount(1);
    }

    /*!
        @brief      CreateMatchmakeSessionParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const CreateMatchmakeSessionParam& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const CreateMatchmakeSessionParam& rhs)
{
    oStr << NEX_T("CreateMatchmakeSessionParam: (")
               << NEX_T("sourceMatchmakeSession=(not implemented)")
               << NEX_T(", joinMessage=") << rhs.joinMessage;

    {
        oStr << NEX_T(", additionalParticipants=(");
        qList<PrincipalID>::const_iterator itr = rhs.additionalParticipants.begin();
        for (int i = 0; itr != rhs.additionalParticipants.end(); ++i, ++itr)
        {
            if (i != 0)
            {
                oStr << NEX_T(", ");
            }
            oStr << *itr;
        }
        oStr << NEX_T(")");
    }

    oStr << NEX_T(", participationCount=") << rhs.participationCount;

    oStr << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX

