﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeBlockListParamDDL.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeConstants.h>


BEGIN_NAMESPACE_NEX

/*!
@brief      ブロックリストの挙動を指定するパラメータを格納するデータクラスです。

初期状態は MatchmakeBlockListParam::SetUseMyBlockList(true) および MatchmakeBlockListParam::SetUseBlockListByOthers(true) された状態です。
*/
class MatchmakeBlockListParam: public DDLCLASS(MatchmakeBlockListParam)
{
public:

    /*!
    @brief      コンストラクタです。

    コンストラクタです。
    */
    MatchmakeBlockListParam()
    {
        Reset();
    };

    /*!
    @brief    デストラクタです。
    */
    virtual ~MatchmakeBlockListParam() {}

    /*!
    @brief      自分がブロックリストに登録しているユーザをチェックするかどうかをセットします。

    マッチメイクセッション参加時に、マッチメイクセッションの参加者に自分がブロックリストに登録しているユーザが存在する場合に、参加できない（true）か強制的に参加する（false）かをセットします。
    初期状態は true です。

    @param[in] useMyBlockList 自分がブロックリストに登録しているユーザをチェックするかどうか。
    */
    void SetUseMyBlockList(qBool useMyBlockList)
    {
        if (useMyBlockList)
        {
            this->optionFlag |= MATCHMAKE_BLOCKLIST_OPTION_USE_MY_BLOCKLIST;
        }
        else
        {
            this->optionFlag &= ~MATCHMAKE_BLOCKLIST_OPTION_USE_MY_BLOCKLIST;
        }
    }

    /*!
    @brief      自分がブロックリストに登録しているユーザをチェックするかどうかを取得します。

    自分がブロックリストに登録しているユーザをチェックするかどうかを取得します。

    @return 自分がブロックリストに登録しているユーザをチェックするかどうか。
    */
    qBool GetUseMyBlockList()
    {
       return (this->optionFlag & MATCHMAKE_BLOCKLIST_OPTION_USE_MY_BLOCKLIST) > 0;
    }

    /*!
    @brief      自分をブロックリストに登録しているユーザをチェックするかどうかをセットします。

    マッチメイクセッション参加時に、マッチメイクセッションの参加者に自分をブロックリストに登録しているユーザが存在する場合に、参加できない（true）か強制的に参加する（false）かをセットします。
    初期状態は true です。

    @param[in] useBlockListByOthers 自分をブロックリストに登録しているユーザをチェックするかどうか。
    */
    void SetUseBlockListByOthers(qBool useBlockListByOthers)
    {
        if (useBlockListByOthers)
        {
            this->optionFlag |= MATCHMAKE_BLOCKLIST_OPTION_USE_BLOCKLIST_BY_OTHERS;
        }
        else
        {
            this->optionFlag &= ~MATCHMAKE_BLOCKLIST_OPTION_USE_BLOCKLIST_BY_OTHERS;
        }
    }

    /*!
    @brief      自分をブロックリストに登録しているユーザをチェックするかどうかを取得します。

    自分をブロックリストに登録しているユーザをチェックするかどうかを取得します。

    @return 自分をブロックリストに登録しているユーザをチェックするかどうか。
    */
    qBool GetUseBlockListByOthers()
    {
        return (this->optionFlag & MATCHMAKE_BLOCKLIST_OPTION_USE_BLOCKLIST_BY_OTHERS) > 0;
    }

    /*!
    @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。

    初期状態は MatchmakeBlockListParam::SetUseMyBlockList() および MatchmakeBlockListParam::SetUseBlockListByOthers() された状態です。
    */
    void Reset()
    {
        this->optionFlag = MATCHMAKE_BLOCKLIST_OPTION_NONE;
        SetUseMyBlockList(true);
        SetUseBlockListByOthers(true);
    }

    /*!
    @brief      MatchmakeBlockListParam オブジェクトの内容をトレースします。

    @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
    @internal
    @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const MatchmakeBlockListParam& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const MatchmakeBlockListParam& rhs)
{
    oStr << NEX_T("MatchmakeBlockListParam: (")
         << NEX_T("UseMyBlockList=") << ((rhs.optionFlag & MATCHMAKE_BLOCKLIST_OPTION_USE_MY_BLOCKLIST) > 0)
         << NEX_T(", UseBlockListByOthers=") << ((rhs.optionFlag & MATCHMAKE_BLOCKLIST_OPTION_USE_BLOCKLIST_BY_OTHERS) > 0)
         << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX
