﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringID.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeParamDDL.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeConstants.h>


BEGIN_NAMESPACE_NEX

/*!
@brief      マッチメイクセッションに関するパラメータを格納するデータクラスです。
*/
class MatchmakeParam: public DDLCLASS(MatchmakeParam)
{
public:

    /*!
    @brief      コンストラクタです。

       コンストラクタです。初期状態は空のパラメータです。
    */
    MatchmakeParam()
    {
        Reset();
    };
    
    /*!
     @brief    デストラクタです。
     */
    virtual ~MatchmakeParam() {}

    void SetParam(const String& key, const Variant& value) { m_Params[key] = value; }
    qBool GetParam(const String& key, Variant* value) const
    {
        qMap<String, Variant>::const_iterator v = m_Params.find(key);
        if (v == m_Params.end())
        {
            return false;
        }

        *value = v->second;
        return true;
    }

    /*!
    @brief      設定インデックスをセットします。

       スコア換算表の設定インデックスをセットします。設定インデックスをセットしていない場合は 0 が使用されます。
       MatchmakeSessionSearchCriteria::SetSelectionMethod() で指定する MatchmakeParam オブジェクトに指定した場合のみこのパラメータが参照されます。

    @param[in]  settingIndex 設定インデックス。未設定の設定インデックスを指定した場合はマッチメイクセッションを評価したときの
                             トータルスコアが 0 となります。
    */
    void SetSettingIndex(qUnsignedInt32 settingIndex) { SetParam(MATCHMAKE_PARAM_KEY_SETTING_INDEX, Variant(settingIndex)); }

    /*!
    @brief      設定インデックスを取得します。

       スコア換算表の設定インデックスを取得します。引数に NULL を渡すとパラメータの有無をのみを確認できます。
       サーバーから取得した MatchmakeSession に含まれる MatchmakeParam オブジェクトにはこのパラメータは含まれません。

    @param[out] settingIndex 取得した設定インデックスを書き込む先のポインタ。NULL を渡した場合は書き込まれません。

    @return パラメータがこのオブジェクトにセットされているかどうか。
    */
    qBool GetSettingIndex(qUnsignedInt32* settingIndex) const
    {
        Variant variant;
        qBool result = GetParam(MATCHMAKE_PARAM_KEY_RATING_VALUE, &variant);
        if (result && settingIndex)
        {
            *settingIndex = variant.GetUInt32Value();
        }
        return result;
    }

    /*!
    @brief      レーティング値をセットします。

       レーティング値をセットします。レーティング値をセットしない場合はレーティング値の差に関するスコアがデフォルトスコアとして評価されます。

    @param[in]  ratingValue レーティング値。
    */
    void SetRatingValue(qUnsignedInt32 ratingValue) { SetParam(MATCHMAKE_PARAM_KEY_RATING_VALUE, Variant(ratingValue)); }

    /*!
    @brief      レーティング値を取得します。

       レーティング値を取得します。引数に NULL を渡すとパラメータの有無をのみを確認できます。

    @param[out] ratingValue 取得したレーティング値を書き込む先のポインタ。NULL を渡した場合は書き込まれません。

    @return パラメータがこのオブジェクトにセットされているかどうか。
    */
    qBool GetRatingValue(qUnsignedInt32* ratingValue) const
    {
        Variant variant;
        qBool result = GetParam(MATCHMAKE_PARAM_KEY_RATING_VALUE, &variant);
        if (result && ratingValue)
        {
            *ratingValue = variant.GetUInt32Value();
        }
        return result;
    }

    /*!
    @brief      切断率をセットします。

       切断率をセットします。切断率をセットしない場合は切断率の差に関するスコアがデフォルトスコアとして評価されます。

    @param[in]  disconnectionRate 切断率。
    */
    void SetDisconnectionRate(qUnsignedInt32 disconnectionRate) { SetParam(MATCHMAKE_PARAM_KEY_DISCONNECTION_RATE, Variant(disconnectionRate)); }

    /*!
    @brief      切断率を取得します。

       切断率を取得します。引数に NULL を渡すとパラメータの有無をのみを確認できます。

    @param[out] disconnectionRate 取得した切断率を書き込む先のポインタ。NULL を渡した場合は書き込まれません。

    @return パラメータがこのオブジェクトにセットされているかどうか。
    */
    qBool GetDisconnectionRate(qUnsignedInt32* disconnectionRate) const
    {
        Variant variant;
        qBool result = GetParam(MATCHMAKE_PARAM_KEY_DISCONNECTION_RATE, &variant);
        if (result && disconnectionRate)
        {
            *disconnectionRate = variant.GetUInt32Value();
        }
        return result;
    }

    /*!
    @brief      違反率をセットします。

       違反率をセットします。違反率をセットしない場合は違反率の差に関するスコアがデフォルトスコアとして評価されます。

    @param[in]  violationRate 違反率。
    */
    void SetViolationRate(qUnsignedInt32 violationRate) { SetParam(MATCHMAKE_PARAM_KEY_VIOLATION_RATE, Variant(violationRate)); }

    /*!
    @brief      違反率を取得します。

       違反率を取得します。引数に NULL を渡すとパラメータの有無をのみを確認できます。

    @param[out] violationRate 取得した違反率を書き込む先のポインタ。NULL を渡した場合は書き込まれません。

    @return パラメータがこのオブジェクトにセットされているかどうか。
    */
    qBool GetViolationRate(qUnsignedInt32* violationRate) const
    {
        Variant variant;
        qBool result = GetParam(MATCHMAKE_PARAM_KEY_VIOLATION_RATE, &variant);
        if (result && violationRate)
        {
            *violationRate = variant.GetUInt32Value();
        }
        return result;
    }

#if defined(NEX_NINTENDO_COUNTRY_CODE_SUPPORT)
    /*!
    @brief      国コードをセットします。

       UseGeoIP() で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合に
       使用する国コードをセットします。位置情報が得られた場合はここで指定した国コードは使用されません。
       国コードをセットしない場合はUseGeoIP() で位置情報を使用しない設定をした場合、
       もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合に国コードの比較に関するスコアがデフォルトスコアとして評価されます。

       CountryCodeが設定してあればCountryCodeの値が優先され、こちらで設定した値は無視されます。

    @param[in]  nintendoCountryCode 国コード。
    */
#endif
    void SetNintendoCountryCode(qUnsignedInt32 nintendoCountryCode) { SetParam(MATCHMAKE_PARAM_KEY_NINTENDO_COUNTRY_CODE, Variant(nintendoCountryCode)); }

    /*!
    @brief ISO 3166-1 alpha-2 によるアルファベット 2 文字の国コードを設定します。

       UseGeoIP() で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合に
       使用する国コードをセットします。位置情報が得られた場合はここで指定した国コードは使用されません。
       国コードをセットしない場合はUseGeoIP() で位置情報を使用しない設定をした場合、
       もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合に国コードの比較に関するスコアがデフォルトスコアとして評価されます。

    @param[in]  countryCode 国コード。
    @return 国コードとして適切な文字列か。適切ではない場合には設定が行われません。
    */
    qBool SetCountryCode(const String& countryCode);
    /*
    @brief      国コードを取得します。

       国コードを取得します。引数に NULL を渡すとパラメータの有無をのみを確認できます。
       サーバーから取得した MatchmakeSession に含まれる MatchmakeParam オブジェクトにはクライアントが
       SetNintendoCountryCode() で指定した国コードが含まれます。
       サーバーで調べた位置情報は本関数では取得できません。代わりに GetCountryCode() を使用して下さい。

    @param[out] nintendoCountryCode 取得した国コードを書き込む先のポインタ。NULL を渡した場合は書き込まれません。

    @return パラメータがこのオブジェクトにセットされているかどうか。
    */
    qBool GetNintendoCountryCode(qUnsignedInt32* nintendoCountryCode) const
    {
        Variant variant;
        qBool result = GetParam(MATCHMAKE_PARAM_KEY_NINTENDO_COUNTRY_CODE, &variant);
        if (result && nintendoCountryCode)
        {
            *nintendoCountryCode = variant.GetUInt32Value();
        }
        return result;
    }

    /*
    @brief      ISO 3166 によるアルファベット 2 文字の国コードを取得します。

       ISO 3166 によるアルファベット2文字の国コードを取得します。引数に NULL を渡すとパラメータの有無をのみを確認できます。
       サーバーで位置情報を取得できた場合はそれを、取得できなかった場合は SetNintendoCountryCode() で指定された国コードを
       ISO 3166 によるアルファベット 2 文字の国コードに変換したものを取得します。
       サーバーから取得した位置情報が有効な MatchmakeSession オブジェクトからのみ ISO 3166 によるアルファベット 2 文字の国コードを取得できます。

    @param[out] countryCode 取得した国コードを書き込む先のポインタ。NULL を渡した場合は書き込まれません。

    @return パラメータがこのオブジェクトにセットされているかどうか。
    */
    qBool GetCountryCode(String* countryCode) const
    {
        Variant variant;
        qBool result = GetParam(MATCHMAKE_PARAM_KEY_COUNTRY_CODE, &variant);
        if (result && countryCode)
        {
            *countryCode = variant.GetStringValue();
        }
        return result;
    }

    /*!
    @brief      デバッグ用の IP アドレスをセットします。

       デバッグ用の IP アドレスをセットします。自身の IP アドレスの代わりにこの IP アドレスを使用して GeoIP による位置情報を取得します。
       デバッグ用にのみ使用可能です。
       サーバーから取得した MatchmakeSession に含まれる MatchmakeParam オブジェクトを
       MatchmakeExtensionClient::UpdateMatchmakeSession() で使用する際にデバッグ用の IP アドレスを使用する場合は
       再度本 API で再セットする必要があります。

    @param[in]  overrideIpAddress デバッグ用の IP アドレス。
    */
    void SetOverrideIpAddress(const String& overrideIpAddress) { SetParam(MATCHMAKE_PARAM_KEY_OVERRIDE_IP_ADDRESS, Variant(overrideIpAddress)); }

    /*!
    @brief      デバッグ用の IP アドレスを取得します。

       デバッグ用の IP アドレスを取得します。引数に NULL を渡すとパラメータの有無をのみを確認できます。
       サーバーから取得した MatchmakeSession に含まれる MatchmakeParam オブジェクトにはこのパラメータは含まれません。

    @param[out] overrideIpAddress 取得したデバッグ用の IP アドレスを書き込む先のポインタ。NULL を渡した場合は書き込まれません。

    @return パラメータがこのオブジェクトにセットされているかどうか。
    */
    qBool GetOverrideIpAddress(String* overrideIpAddress)
    {
        Variant variant;
        qBool result = GetParam(MATCHMAKE_PARAM_KEY_OVERRIDE_IP_ADDRESS, &variant);
        if (result && overrideIpAddress)
        {
            *overrideIpAddress = variant.GetStringValue();
        }
        return result;
    }

    /*
    @brief      緯度を取得します。

       緯度を取得します。引数に NULL を渡すとパラメータの有無をのみを確認できます。
       MatchmakeSession に指定する MatchmakeParam オブジェクトに対して SetUseGeoIp(true) を指定し、サーバーで位置情報を取得できた場合のみ
       サーバーから取得した位置情報が有効な MatchmakeSession オブジェクトからのみ緯度を取得できます。

    @param[out] latitude 取得した緯度を書き込む先のポインタ。NULL を渡した場合は書き込まれません。

    @return パラメータがこのオブジェクトにセットされているかどうか。
    */
    qBool GetLatitude(qDouble* latitude) const
    {
        Variant variant;
        qBool result = GetParam(MATCHMAKE_PARAM_KEY_LATITUDE, &variant);
        if (result && latitude)
        {
            *latitude = variant.GetDoubleValue();
        }
        return result;
    }

    /*
    @brief      経度を取得します。

       経度を取得します。引数に NULL を渡すとパラメータの有無をのみを確認できます。
       MatchmakeSession に指定する MatchmakeParam オブジェクトに対して SetUseGeoIp(true) を指定し、サーバーで位置情報を取得できた場合のみ
       サーバーから取得した位置情報が有効な MatchmakeSession オブジェクトからのみ経度を取得できます。

    @param[out] longitude 取得した経度を書き込む先のポインタ。NULL を渡した場合は書き込まれません。

    @return パラメータがこのオブジェクトにセットされているかどうか。
    */
    qBool GetLongitude(qDouble* longitude) const
    {
        Variant variant;
        qBool result = GetParam(MATCHMAKE_PARAM_KEY_LONGITUDE, &variant);
        if (result && longitude)
        {
            *longitude = variant.GetDoubleValue();
        }
        return result;
    }

    /*
    @brief      距離を取得します。

       距離を取得します。引数に NULL を渡すとパラメータの有無をのみを確認できます。
       MatchmakeExtensionClient::AutoMatchmake() において MATCHMAKE_SELECTION_METHOD_SCORE_BASED を使用した検索条件を指定し、
       かつサーバーに登録されているマッチメイクセッション及び MatchmakeExtensionClient::AutoMatchmake() に指定した
       MatchmakeSession に指定する MatchmakeParam オブジェクトに対して SetUseGeoIp(true) を指定し、サーバーで位置情報を取得でき、
       自身がマッチメイクセッションのオーナーにならなかった場合のみ MatchmakeExtensionClient::AutoMatchmake()
       で取得したマッチメイクセッションから距離を取得できます。

    @param[out] distance 取得した距離を書き込む先のポインタ。NULL を渡した場合は書き込まれません。

    @return パラメータがこのオブジェクトにセットされているかどうか。
    */
    qBool GetDistance(qDouble* distance) const
    {
        Variant variant;
        qBool result = GetParam(MATCHMAKE_PARAM_KEY_DISTANCE, &variant);
        if (result && distance)
        {
            *distance = variant.GetDoubleValue();
        }
        return result;
    }

    /*!
    @brief      トータルスコアを取得します。

       トータルスコアを取得します。引数に NULL を渡すとパラメータの有無をのみを確認できます。
       MatchmakeExtensionClient::AutoMatchmake() において MATCHMAKE_SELECTION_METHOD_SCORE_BASED を使用した検索条件を指定し、
       自身がマッチメイクセッションのオーナーにならなかった場合のみ MatchmakeExtensionClient::AutoMatchmake()
       で取得したマッチメイクセッションからトータルスコアを取得できます。

    @param[out] totalScore 取得したトータルスコアを書き込む先のポインタ。NULL を渡した場合は書き込まれません。

    @return パラメータがこのオブジェクトにセットされているかどうか。
    */
    qBool GetTotalScore(qUnsignedInt32* totalScore) const
    {
        Variant variant;
        qBool result = GetParam(MATCHMAKE_PARAM_KEY_TOTAL_SCORE, &variant);
        if (result && totalScore)
        {
            *totalScore = variant.GetUInt32Value();
        }
        return result;
    }

    /*!
    @brief      位置情報を使うかどうかをセットします。

       位置情報 (緯度経度、国コード) を使うかどうかをセットします。
       位置情報を使うかどうかをセットしない場合は距離の比較に関するスコアがデフォルトスコアとして評価されます。

    @param[in]  useGeoIp 位置情報を使うかどうか。
    */
    void SetUseGeoIp(qBool useGeoIp) { SetParam(MATCHMAKE_PARAM_KEY_USE_GEO_IP, Variant(useGeoIp)); }

    /*!
    @brief      位置情報を使うかどうかを取得します。

       位置情報を使うかどうかを取得します。引数に NULL を渡すとパラメータの有無をのみを確認できます。

    @param[out] useGeoIp 位置情報を使うかどうかを書き込む先のポインタ。NULL を渡した場合は書き込まれません。

    @return パラメータがこのオブジェクトにセットされているかどうか。
    */
    qBool GetUseGeoIp(qBool* useGeoIp) const
    {
        Variant variant;
        qBool result = GetParam(MATCHMAKE_PARAM_KEY_USE_GEO_IP, &variant);
        if (result && useGeoIp)
        {
            *useGeoIp = variant.GetBoolValue();
        }
        return result;
    }

    /*!
    @brief      位置情報を更新するかどうかをセットします。

       位置情報を更新するかどうかをセットします。
       MatchmakeExtensionClient::UpdateMatchmakeSession() に指定する MatchmakeSession の MatchmakeParam オブジェクトに対して使用できます。
       更新する場合は自身の位置情報で置き換えられます。
       位置情報を更新するかどうかをセットしない場合は更新されます。
       サーバーに記録されている位置情報を更新しない場合は該当のマッチメイクセッションをサーバーから取得し、その MatchmakeParam オブジェクトを利用する必要があります。
       クライアントで生成した MatchmakeParam を利用して更新しない設定をすると位置情報が削除されます。

    @param[in]  updateGeoIp 位置情報を使うかどうか。
    */
    void SetUpdateGeoIp(qBool updateGeoIp) { SetParam(MATCHMAKE_PARAM_KEY_UPDATE_GEO_IP, Variant(updateGeoIp)); }

    /*!
    @brief      位置情報を更新するかどうかを取得します。

       位置情報を更新するかどうかを取得します。引数に NULL を渡すとパラメータの有無をのみを確認できます。
       サーバーから取得した MatchmakeSession に含まれる MatchmakeParam オブジェクトにはこのパラメータは含まれません。

    @param[out] updateGeoIp 位置情報を更新するかどうかを書き込む先のポインタ。NULL を渡した場合は書き込まれません。

    @return パラメータがこのオブジェクトにセットされているかどうか。
    */
    qBool GetUpdateGeoIp(qBool* updateGeoIp) const
    {
        Variant variant;
        qBool result = GetParam(MATCHMAKE_PARAM_KEY_UPDATE_GEO_IP, &variant);
        if (result && updateGeoIp)
        {
            *updateGeoIp = variant.GetBoolValue();
        }
        return result;
    }

    /*!
    @brief      位置情報の取得結果を取得します。

       サーバーで位置情報を取得したときの結果を取得します。引数に NULL を渡すとパラメータの有無をのみを確認できます。
       サーバーから取得したマッチメイクセッションからこのパラメータを取得できます。

    @param[out] geoIpResult 位置情報を更新するかどうかを書き込む先のポインタ。NULL を渡した場合は書き込まれません。

    @return パラメータがこのオブジェクトにセットされているかどうか。
    */
    qBool GetGeoIpResult(MATCHMAKE_GEO_IP_RESULT* geoIpResult) const
    {
        Variant variant;
        qBool result = GetParam(MATCHMAKE_PARAM_KEY_GEO_IP_RESULT, &variant);
        if (result && geoIpResult)
        {
            *geoIpResult = static_cast<MATCHMAKE_GEO_IP_RESULT>(variant.GetUInt32Value());
        }
        return result;
    }

    /*!
        @internal
    @brief      参加元ギャザリング ID を取得します。

        参加元ギャザリング ID を取得します。引数に NULL を渡すとパラメータの有無をのみを確認できます。
        CREATE_MATCHMAKE_SESSION_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK, JOIN_MATCHMAKE_SESSION_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK, AUTO_MATCHMAKE_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK
        をそれぞれ、 CreateMatchmakeSessionParam::SetCreateMatchmakeSessionOption(), JoinMatchmakeSessionParam::SetJoinMatchmakeSessionOption(),
        AutoMatchmakeParam::SetAutoMatchmakeOption() に指定してマッチメイクした場合のみ対象のマッチメイクセッションに記録されます。

    @param[out] gid 参加元ギャザリング ID を書き込む先のポインタ。NULL を渡した場合は書き込まれません。

    @return パラメータがこのオブジェクトにセットされているかどうか。
    */
    qBool GetLastGidForParticipationCheck(GatheringID* gid) const
    {
        Variant variant;
        qBool result = GetParam(MATCHMAKE_PARAM_KEY_LAST_GID_FOR_PARTICIPATION_CHECK, &variant);
        if (result && gid)
        {
            *gid = variant.GetUInt32Value();
        }
        return result;
    }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        m_Params.clear();
    }

    /*!
        @brief      MatchmakeParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const MatchmakeParam& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const MatchmakeParam& rhs)
{
    oStr << NEX_T("MatchmakeParam: (")
               << NEX_T("m_Params=(");

    qMap<String, Variant>::const_iterator itr = rhs.m_Params.begin();
    for (int i = 0; itr != rhs.m_Params.end(); ++i, ++itr)
    {
        if (i != 0)
        {
            oStr << NEX_T(", ");
        }
        oStr << NEX_T("(key=") << itr->first << NEX_T(", value=") << itr->second << NEX_T(")");
    }

    oStr << NEX_T("))");
    return oStr;
}

END_NAMESPACE_NEX

