﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeSessionDDL.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeConstants.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeFlags.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakingUtil.h>

BEGIN_NAMESPACE_NEX

/*!
 @brief マッチメイクの情報を格納するクラスです。
 */
#ifdef FOR_DOC_PURPOSE
class MatchmakeSession: public Gathering
#else
class MatchmakeSession: public DDLCLASS(MatchmakeSession)
#endif
{
public:
    /*!
    @brief      コンストラクタです。

    初期状態はゲームモード、属性は全て0、マッチメイクシステムタイプは MATCHMAKE_SYSTEM_TYPE_INVALID 、ゲーム進行度は MAX_PROGRESS_SCORE となります。
    */
    MatchmakeSession();
    /*!
     @brief    デストラクタです。
     */
    virtual ~MatchmakeSession() {};

    /*!
    @brief      ゲームモードをセットします。

    このマッチメイクセッションのゲームモードをセットします。

    @param[in]     gameMode    アプリケーション定義のゲームモード。
    */
    void SetGameMode(qUnsignedInt32 gameMode);

    /*!
    @brief      属性をセットします。

    このマッチメイクセッションの属性をセットします。最大6つ指定できます。

    @param[in]     uiIndex    セットする属性のインデックス。0以上NUM_MATCHMAKE_SESSION_ATTRIBUTES - 1以下の値を指定できます。
    @param[in]     val    アプリケーション定義の属性値。
    */
    void SetAttribute(qUnsignedInt32 uiIndex, qUnsignedInt32 val);

    /*!
    @brief      マッチメイクシステムタイプをセットします。

    このマッチメイクセッションのマッチメイクシステムタイプをセットします。

    matchmakeSystemTypeにMATCHMAKE_SYSTEM_TYPE_FRIENDが指定されたマッチメイクセッションはフレンドのみが参加可能となるように制限されます。
    このとき @ref Gathering::SetFlag() により GatheringFlags::MigrateOwner がセットされていた場合は
    オーナーがマッチメイクセッションを退出して自動的にオーナーが他のプレイヤーに移行されたときにギャザリングが自動的に締め切られます。
    ただし、policyArgumentに @ref FriendOnlyParticipationPolicyArgument::WithoutClose を指定した場合は自動的に締め切られなくなります。

    matchmakeSystemTypeにMATCHMAKE_SYSTEM_TYPE_ANYBODYが指定されたマッチメイクセッションは誰でも参加可能になります。
    このとき @ref Gathering::SetFlag() により GatheringFlags::MigrateOwner がセットされていた場合は
    オーナーがマッチメイクセッションを退出して自動的にオーナーが他のプレイヤーに移行されてもギャザリングが自動的に締め切られませんが、
    policyArgumentに @ref AnybodyParticipationPolicyArgument::CloseOnOwnerMigration を指定した場合は自動的に締め切られるようになります。

    @attention MATCHMAKE_SYSTEM_TYPE_FRIEND の場合と MATCHMAKE_SYSTEM_TYPE_ANYBODY の場合とで、デフォルト動作(オーナー移行時に自動的に締め切られる／締め切られない)が異なることに注意してください。

    matchmakeSystemType に MATCHMAKE_SYSTEM_TYPE_FRIEND もしくは MATCHMAKE_SYSTEM_TYPE_ANYBODY のいずれでもない値が指定されたときは policyArgument は無視されます。

    matchmakeSystemTypeにMATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING(MATCHMAKE_SYSTEM_TYPE_COMMUNITY)を指定した場合、@ref SetAttribute() で 0 番目の属性値に永続ギャザリング（コミュニティ）のギャザリングIDをセットする必要があります。

    @param[in]     matchmakeSystemType    マッチメイクシステムタイプ。
    @param[in]     policyArgument    参加ポリシー引数（デフォルト値 0）
    */
    void SetMatchmakeSystemType(MatchmakeSystemType matchmakeSystemType, qUnsignedInt32 policyArgument=0);

    /*!
    @brief      アプリケーション定義のデータをセットします。

    このマッチメイクセッションにアプリケーション定義のデータをセットします。最大 MATCHMAKE_BUFFER_MAX_LENGTH byteです。

    @param[in]     applicationBuffer    アプリケーション定義のデータ。
    */
    void SetApplicationBuffer(const qVector<qUnsignedInt8>& applicationBuffer);

    /*!
    @brief      マッチメイクセッションへ参加可能であるかをセットします

    このマッチメイクセッションへ参加可能であるかをセットします

     @param[in]     openParticipation    マッチメイクセッションへ参加可能であるか。
    */
    void SetOpenParticipation(bool openParticipation);

    /*!
    @brief      セットしたデータを全て初期化します。

    このマッチメイクセッションにセットした情報を全て初期化します。ゲームモード、属性は全て0となり、マッチメイクシステムタイプは MATCHMAKE_SYSTEM_TYPE_INVALID 、ゲーム進行度は MAX_PROGRESS_SCORE となります。
    */
    void Reset();

    /*!
    @brief      ゲームモードを取得します。

    このマッチメイクセッションにセットされたゲームモードを取得します。

    @return     ゲームモード
    */
    qUnsignedInt32 GetGameMode() const
    {
        return m_GameMode;
    }

    /*!
    @brief      属性を取得します。

    このマッチメイクセッションにセットされた属性を取得します。

    @param[in]     uiIndex    取得する属性のインデックス。0以上NUM_MATCHMAKE_SESSION_ATTRIBUTES - 1以下の値を指定できます。

    @return     指定されたインデックスの属性値。
    */
    qUnsignedInt32 GetAttribute(qUnsignedInt32 uiIndex) const;

    /*!
    @brief      マッチメイクシステムタイプを取得します。

    このマッチメイクセッションにセットされたマッチメイクシステムタイプを取得します。

    @return     マッチメイクシステムタイプ
    */
    MatchmakeSystemType GetMatchmakeSystemType() const
    {
        return static_cast<MatchmakeSystemType>(m_MatchmakeSystemType);
    };

    /*!
    @brief      アプリケーション定義のデータを取得します。

    このマッチメイクセッションにセットされたアプリケーション定義のデータを取得します。

    @param[out]     pApplicationBuffer    アプリケーション定義のデータへのポインタ
    */
    void GetApplicationBuffer(qVector<qUnsignedInt8>* pApplicationBuffer) const
    {
        SYSTEMCHECK(pApplicationBuffer != NULL);
        pApplicationBuffer->assign(m_ApplicationBuffer.begin(), m_ApplicationBuffer.end());
    };

    /*!
    @brief      マッチメイクセッションへ参加可能であるかを取得します

    このマッチメイクセッションへ参加可能であるかを取得します

    @return     マッチメイクセッションへ参加可能であるか。
    */
    bool GetOpenParticipation() const
    {
        return m_OpenParticipation;
    }

    /*!
    @brief      マッチメイクセッションの参加人数を取得します

    このマッチメイクセッションへの参加人数を取得します。

    @return     マッチメイクセッションの参加人数。
    */
    qUnsignedInt32 GetParticipantCount() const
    {
        return m_ParticipationCount;
    }

    /*!
    @brief      マッチメイクセッションの P2P 通信用の共通鍵を取得します

    このマッチメイクセッションの P2P 通信用の共通鍵を取得します。
    @ref MatchmakeExtensionClient::AutoMatchmake() , 
    @ref MatchmakeExtensionClient::FindMatchmakeSessionByGatheringIdDetail() と,
    MatchmakeSession を out 引数に取る @ref MatchmakeExtensionClient::CreateMatchmakeSession() ,
    @ref MatchmakeExtensionClient::JoinMatchmakeSession() で取得した場合のみ有効な値がセットされます。
    鍵長は @ref Network::MAX_P2P_SIGNATURE_KEY_LEN byte です。

    @return     P2P 通信用の共通鍵

    @see Network::SetP2PDataPacketSessionSignatureKey()
    */
    const qVector<qByte>& GetSessionKey() const
    {
        return m_SessionKey;
    }

    /*!
    @brief      ゲーム進行度をセットします。

    このマッチメイクセッションのゲーム進行度をセットします。

    @param[in]     progressScore    アプリケーション定義のゲーム進行度。0 以上 MAX_PROGRESS_SCORE 以下の値を指定できます。
    @return     ゲーム進行度が MAX_PROGRESS_SCORE より大きい場合は、何もせずにfalseを返します。 それ以外の場合は、ゲーム進行度を設定しtrueを返します。
    */
    qBool SetProgressScore(qUnsignedInt8 progressScore);

    /*!
    @brief      ゲーム進行度を取得します。

    このマッチメイクセッションにセットされたゲーム進行度を取得します。

    @return     ゲーム進行度
    */
    qUnsignedInt8 GetProgressScore() const
    {
        return m_ProgressScore;
    }

    /*!
    @brief      マッチメイクセッションのオプションを設定します。

    すでに設定されているオプションに指定したフラグを追加します。

    @param[in] uiFlag MatchmakeFlags::MatchmakeSessionOption0 の値
    
    @see MatchmakeFlags::MatchmakeSessionOption0
    */
    void SetOption0Flag(qUnsignedInt32 uiFlag)
    {
        m_Option0 |= uiFlag;
    }

    /*!
    @brief      マッチメイクセッションのオプションを解除します。

    すでに設定されているオプションから指定したフラグを取り除きます。

    @param[in] uiFlag MatchmakeFlags::MatchmakeSessionOption0 の値
    
    @see MatchmakeFlags::MatchmakeSessionOption0
    */
    void ClearOption0Flag(qUnsignedInt32 uiFlag)
    {
        m_Option0 &= ~uiFlag;
    }
    
    /*!
    @brief      マッチメイクセッションのオプションを取得します

    @param[in] uiFlag MatchmakeFlags::MatchmakeSessionOption0 の値
    
    @return オプション値
    
    @see MatchmakeFlags::MatchmakeSessionOption0
    */
    qBool GetOption0FlagSet(qUnsignedInt32 uiFlag) const
    {
        return (m_Option0 & uiFlag)!=0;
    }

    /*!
    @brief      マッチメイクパラメータをセットします。

    マッチメイクパラメータをセットします。
    マッチメイクパラメータは MATCHMAKE_SELECTION_METHOD_SCORE_BASED によるマッチメイクセッション選択方式で利用されます。

    @param[in]     matchmakeParam    マッチメイクパラメータ。
    */
    void SetMatchmakeParam(const MatchmakeParam& matchmakeParam)
    {
        m_MatchmakeParam = matchmakeParam;
    }

    /*!
    @brief      マッチメイクパラメータを取得します。

    マッチメイクパラメータを取得します。

    @return     マッチメイクパラメータ
    */
    const MatchmakeParam& GetMatchmakeParam() const
    {
        return m_MatchmakeParam;
    }

    /*!
    @brief      マッチメイクセッションユーザーパスワードをセットします。

    マッチメイクセッションユーザーパスワードをセットします。
    マッチメイクセッションユーザーパスワードは最大 MAX_MATCHMAKE_SESSION_USER_PASSWORD_LENGTH 文字までセットできます。

    @param[in]     userPassword    マッチメイクセッションユーザーパスワード。
    */
    void SetUserPassword(const String& userPassword)
    {
        m_UserPassword = userPassword;
    }

    /*!
    @brief      マッチメイクセッションユーザーパスワードを取得します。

    ローカルで MatchmakeSession::SetUserPassword() を使いパスワードをセットした場合のみ値を取得できます。
    サーバーから取得したマッチメイクセッションからはパスワードを取得できず、常に空文字列が返されます。
    マッチメイクセッションユーザーパスワードがセットされているかどうかは MatchmakeSession::GetUserPasswordEnabled() で確認できます。

    @return     マッチメイクセッションユーザーパスワード。
    */
    const String& GetUserPassword() const
    {
        return m_UserPassword;
    }

    /*!
    @internal
    @brief      マッチメイク開始日時をセットします。

    マッチメイクを開始した日時をセットします。
    MatchmakeExtensionClient::UpdateMatchmakeSession() で MatchmakeSession を指定する際に使用できます。
    DateTime::Never を指定した場合は更新は行われません。
    MatchmakeExtensionClient::CreateMatchmakeSession() や MatchmakeExtensionClient::AutoMatchmake()
    でマッチメイクセッションを作成したときは常に現在時刻がセットされ、ここでセットした日時は無視されます。

    @param[in]     startedTime    マッチメイク開始日時。
    */
    void SetStartedTime(const DateTime& startedTime)
    {
        m_StartedTime = startedTime;
    };

    /*!
    @brief      マッチメイク開始日時を取得します。

    マッチメイクを開始した日時を取得します。

    @return     マッチメイク開始日時
    */
    const DateTime& GetStartedTime() const
    {
        return m_StartedTime;
    }

    /*!
    @brief      関連する永続ギャザリングの ID をセットします。

    このマッチメイクセッションを関連付ける永続ギャザリングの ID をセットします。

    @param[in]     referPersistentGatheringId    このマッチメイクセッションを関連付ける永続ギャザリングの ID。
                              INVALID_GATHERINGID は無効値となり、関連する永続ギャザリングはないことを表します。
                              永続ギャザリングに関連付けられたマッチメイクセッションの数は
                              PersistentGathering::GetMatchmakeSessionCount() で取得できます。
                              マッチメイクセッションが破棄されたときは自動的にこの数が減少します。
                              初期値は INVALID_GATHERINGID です。
    */
    void SetReferPersistentGatheringId(GatheringID referPersistentGatheringId)
    {
        m_ReferGid = referPersistentGatheringId;
    }

    /*!
    @brief      関連する永続ギャザリングの ID を取得します。

    このマッチメイクセッションにセットされた関連する永続ギャザリングの ID を取得します。

    @return     関連する ID
    */
    GatheringID GetReferPersistentGatheringId() const
    {
        return m_ReferGid;
    }

    /*!
    @brief      マッチメイクセッションユーザーパスワードがセットされているかどうかを取得します。

    サーバーから取得したマッチメイクセッションからのみ有効な値を取得できます。

    @return     マッチメイクセッションユーザーパスワードがセットされているかどうか。
    */
    qBool GetUserPasswordEnabled() const
    {
        return static_cast<qBool>(m_UserPasswordEnabled);
    }

    /*!
    @brief      マッチメイクセッションシステムパスワードがセットされているかどうかを取得します。

    サーバーから取得したマッチメイクセッションからのみ有効な値を取得できます。

    @return     マッチメイクセッションシステムパスワードがセットされているかどうか。
    */
    qBool GetSystemPasswordEnabled() const
    {
        return static_cast<qBool>(m_SystemPasswordEnabled);
    }

    /*!
    @brief      あいことばをセットします

    空文字列をセットすることにより、あいことばを利用しない設定にすることができます。
    初期状態は空文字列です。


    @param[in]     codeword    あいことば

    @retval        true  成功。あいことばとして使用できる文字列もしくは空文字列です
    @retval        false 失敗。あいことばとして使用できない文字が含まれています
    */
    qBool SetCodeword(const String& codeword);

    /*!
    @brief      あいことばを取得します


    @return   あいことば
    */
    const String& GetCodeword() const
    {
        return m_Codeword;
    };

    /*!
      @internal
      @brief    自身のオブジェクトの参照を返します

      @return     自身のオブジェクトの参照
      */
    const MatchmakeSession& GetRef() const
    {
        return *this;
    }

    /*!
      @internal
      @brief    自身のオブジェクトの参照を返します

      @return     自身のオブジェクトの参照
      */
    MatchmakeSession& GetRef()
    {
        return *this;
    }
};

END_NAMESPACE_NEX

