﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeSessionSearchCriteriaDDL.h>


BEGIN_NAMESPACE_NEX

/*!
@brief      マッチメイクセッションの検索条件を格納するデータクラスです。

AutoMatchmakeParam::SetSearchCriteria() で検索条件を指定した場合は設定にかかわらず、クローズされているもの、満員のもの、パスワードがセットされているものは自動的に除外されます。

クローズされているもの、満員のもの、パスワードがセットされているものを除外するかの設定は、 MatchmakeExtensionClient::BrowseMatchmakeSession() を利用した際に有効になります。

*/
class MatchmakeSessionSearchCriteria: public DDLCLASS(MatchmakeSessionSearchCriteria)
{
public:

    /*!
    @brief      コンストラクタです。

    コンストラクタです。初期状態は不正な検索条件となります。
    最低限 MatchmakeSessionSearchCriteria::SetGameMode() 及び MatchmakeSessionSearchCriteria::SetMatchmakeSystemType()
    でゲームモードとマッチメイクシステムタイプを指定する必要があります。
    指定したゲームモードとマッチメイクシステムタイプと一致し、ホストが決定していないもの、パスワードがセットされているもの、
    参加がロックされているもの、満員のものを除外したマッチメイクセッションにマッチします。
    検索範囲は、 RESULTRANGE_ANY_OFFSET になります。
    */
    MatchmakeSessionSearchCriteria();

    /*!
    @brief      コンストラクタです。

    コンストラクタです。最低限必要なパラメータがセットされた有効なオブジェクトを生成します。
    指定したゲームモードとマッチメイクシステムタイプと一致し、ホストが決定していないもの、パスワードがセットされているもの、
    参加がロックされているもの、満員のものを除外したマッチメイクセッションにマッチします。
    取得件数は 20 件、オフセットはランダム (RESULTRANGE_ANY_OFFSET) になります。
    */
    MatchmakeSessionSearchCriteria(qUnsignedInt32 gameMode, MatchmakeSystemType matchmakeSystemType);

    /*!
     @brief    デストラクタです。
     */
    virtual ~MatchmakeSessionSearchCriteria() {}

    /*!
    @brief      検索するゲームモードを指定します。

    検索するゲームモードを指定します。指定は必須です。

    @param[in]     gameMode    ゲームモード。
    */
    void SetGameMode(qUnsignedInt32 gameMode);

    /*!
    @brief      最少参加人数を指定します。

    最少参加人数を指定します。指定しなかった場合、任意の値にマッチするようになります

    @param[in]     minParticipants    最少参加人数
    */
    void SetMinParticipants(qUnsignedInt16 minParticipants);

    /*!
    @brief      最少参加人数を範囲指定します。

    最少参加人数を指定します。指定しなかった場合、任意の値にマッチするようになります
    min値 <= 最少参加人数 <= max値 で検索します。

    @param[in]     min        最少参加人数の最小値。
    @param[in]     max        最少参加人数の最大値。
    */
    void SetMinParticipantsWithRange(qUnsignedInt16 min, qUnsignedInt16 max);

    /*!
    @brief      最大参加人数を指定します。

    最大参加人数を指定します。指定しなかった場合、任意の値にマッチするようになります

    @param[in]     maxParticipants    最大参加人数
    */
    void SetMaxParticipants(qUnsignedInt16 maxParticipants);

    /*!
    @brief      最大参加人数を範囲指定します。

    最大参加人数を指定します。指定しなかった場合、任意の値にマッチするようになります
    min値 <= 最大参加人数 <= max値 で検索します。

    @param[in]     min        最大参加人数の最小値。
    @param[in]     max        最大参加人数の最大値。
    */
    void SetMaxParticipantsWithRange(qUnsignedInt16 min, qUnsignedInt16 max);

    /*!
    @brief      検索する属性を指定します。

    検索する属性を指定します。最大NUM_MATCHMAKE_SESSION_ATTRIBUTES個の属性を指定できます。指定しなかったインデックスは任意の値にマッチするようになります。

    属性値は、インデックス 0 から NUM_MATCHMAKE_SESSION_ATTRIBUTES - 1 まで一括して評価されます。インデックスの順番は関係ありません。

    @param[in]     uiIndex    セットする属性のインデックス。0以上NUM_MATCHMAKE_SESSION_ATTRIBUTES - 1以下の値を指定できます。
    @param[in]     val    アプリケーション定義の属性値。
    */
    void SetAttribute(qUnsignedInt32 uiIndex, qUnsignedInt32 val);

    /*!
    @brief      検索する属性を指定します。属性値を複数指定できます。

    検索する属性を指定します。最大NUM_MATCHMAKE_SESSION_ATTRIBUTES個の属性を指定できます。指定しなかったインデックスは任意の値にマッチするようになります。

    属性値は、インデックス 0 から NUM_MATCHMAKE_SESSION_ATTRIBUTES - 1 まで一括して評価されます。インデックスの順番は関係ありません。

    @param[in]     uiIndex    セットする属性のインデックス。0以上NUM_MATCHMAKE_SESSION_ATTRIBUTES - 1以下の値を指定できます。
    @param[in]     vals    アプリケーション定義の属性値。要素数は1以上SETATTIBUTE_VECTOR_SIZE_MAX以下としてください。
    @return     属性値の要素数が0、もしくはSETATTIBUTE_VECTOR_SIZE_MAXより大きい場合は、何もせずにfalseを返します。
                それ以外の場合は、属性を指定しtrueを返します。
    */
    qBool SetAttribute(qUnsignedInt32 uiIndex, const qVector<qUnsignedInt32>& vals);

    /*!
    @brief      検索する属性を範囲で指定します。

    検索する属性を範囲で指定します。最大NUM_MATCHMAKE_SESSION_ATTRIBUTES個の属性を指定できます。
    指定しなかったインデックスは任意の値にマッチするようになります。
    min値 <= 属性値 <= max値 で検索します。

    属性値は、インデックス 0 から NUM_MATCHMAKE_SESSION_ATTRIBUTES - 1 まで一括して評価されます。インデックスの順番は関係ありません。

    @param[in]     uiIndex    セットする属性のインデックス。0 以上 NUM_MATCHMAKE_SESSION_ATTRIBUTES - 1以下の値を指定できます。
    @param[in]     min        アプリケーション定義の属性値の最小値。
    @param[in]     max        アプリケーション定義の属性値の最大値。
    */
    void SetAttributeWithRange(qUnsignedInt32 uiIndex, qUnsignedInt32 min, qUnsignedInt32 max);

    /*!
    @brief      検索するマッチメイクシステムタイプを指定します。

    検索するマッチメイクシステムタイプを指定します。指定は必須です。
    MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING を指定した場合、@ref SetAttribute() で 0 番目の属性値に永続ギャザリングのギャザリングIDをセットする必要があります。

    @param[in]     matchmakeSystemType    マッチメイクシステムタイプ。
    */
    void SetMatchmakeSystemType(MatchmakeSystemType matchmakeSystemType);

    /*!
    @brief      すでに参加がロックされているものを除外するかを設定します。

    すでに参加がクローズされているものを除外するかを設定します。
    クローズは MatchmakeExtensionClient::CloseParticipation() で行います。
    デフォルトでは true です。
    
    AutoMatchmakeParam::SetSearchCriteria() で指定した場合は設定にかかわらず除外します。
    MatchmakeExtensionClient::BrowseMatchmakeSession() を利用した際に有効になります。

    @param[in]     excludeLocked    参加がロック、クローズされているものを除外する場合はtrue、含む場合はfalseを指定してください。
    */
    void SetExcludeLocked(bool excludeLocked);

    /*!
    @brief      満員のものを除外するかを設定します。

    満員のものを除外するかを設定します。デフォルトでは除外しません。
    
    AutoMatchmakeParam::SetSearchCriteria() で指定した場合は設定にかかわらず除外します。
    MatchmakeExtensionClient::BrowseMatchmakeSession() を利用した際に有効になります。


    @param[in]     vacantOnly    満員のものを除外する場合はtrue、除外しない場合はfalseを指定してください。
    */
    void SetVacantOnly(bool vacantOnly);

    /*!
    @brief      満員のものを除外するかを設定します。空き人数の条件を指定できます。

    満員のものを除外するかを設定します。デフォルトでは除外しません。
    除外する場合、検索するマッチメイクセッションの空き人数を指定できます。

    MatchmakeExtensionClient::BrowseMatchmakeSession() では、participantsに指定した値以上の空きがあるマッチメイクセッションを検索します。

    AutoMatchmakeParam::SetSearchCriteria() で指定した場合は設定にかかわらず参加する人数分の空きがあるマッチメイクセッションを検索します。

    @param[in]     vacantOnly    満員のものを除外する場合はtrue、除外しない場合はfalseを指定してください。
    @param[in]     participants    空き人数。1以上の値を設定してください。vacantOnlyがfalseのとき、この値は無視されます。

    @retval     true 設定に成功。
    @retval     false 失敗。vacantOnlyがtrueかつ、participantsが0のとき失敗します。
    */
    qBool SetVacantOnly(bool vacantOnly, qUnsignedInt16 participants);

    /*!
    @brief      ホストが決定していないものを除外するかを設定します。

    ホストのプリンシパル ID がマッチメイクセッションに登録されていないものを除外するかを設定します。除外する場合はtrueを指定してください。デフォルトでは除外します。
    MatchmakeExtensionClient::CreateMatchmakeSession() や MatchmakeExtensionClient::AutoMatchmake() でマッチメイクセッションを作成した場合は自動的にオーナーがホストになります。

    @param[in]     excludeNonHostPid_    ホストが決定していないものを除外する場合はtrue、除外しない場合はfalseを指定してください。
    */
    void SetExcludeNonHostPid(bool excludeNonHostPid_);

    /*!
    @brief      マッチメイクセッションユーザーパスワードが設定されたものを除外するかを設定します。

    マッチメイクセッションユーザーパスワードが設定されたものを除外するかを設定します。除外する場合はtrueを指定してください。デフォルトでは除外します。

    AutoMatchmakeParam::SetSearchCriteria() で指定した場合は設定にかかわらず除外します。
    MatchmakeExtensionClient::BrowseMatchmakeSession() を利用した際に有効になります。

    @param[in]     excludeUserPasswordSet_    マッチメイクセッションユーザーパスワードが設定されたものを除外する場合はtrue、除外しない場合はfalseを指定してください。
    */
    void SetExcludeUserPasswordSet(bool excludeUserPasswordSet_);

    /*!
    @brief      検索結果から参加するマッチメイクセッションを選ぶ方法を指定します。

    検索条件に一致した複数のマッチメイクセッションの中から参加するマッチメイクセッションを選ぶ方法を設定します（オートマッチメイク時のみ利用）。
    MATCHMAKE_SELECTION_METHOD_SCORE_BASED 以外を使用する場合はこちらのオーバーロード関数を使用します。

    @param[in]     selectionMethod    選び方
    */
    void SetSelectionMethod(MatchmakeSelectionMethod selectionMethod);

    /*!
    @brief      検索結果から参加するマッチメイクセッションを選ぶ方法を指定します。

    検索条件に一致した複数のマッチメイクセッションの中から参加するマッチメイクセッションを選ぶ方法を設定します（オートマッチメイク時のみ利用）。
    MATCHMAKE_SELECTION_METHOD_SCORE_BASED を使用する場合はこちらのオーバーロード関数を使用します。

    @param[in]     selectionMethod    選び方
    @param[in]     param     選び方に対するパラメータ
    */
    void SetSelectionMethod(MatchmakeSelectionMethod selectionMethod, const MatchmakeParam& param);

    /*!
    @brief      検索する関連する永続ギャザリングの ID をセットします。

    検索する関連する永続ギャザリングの ID をセットします。指定した永続ギャザリングに関連するマッチメイクセッションが検索されます。

    @param[in]     referPersistentGatheringId    関連する永続ギャザリングの ID。
                              INVALID_GATHERING を指定した場合は任意の値にマッチするようになります。デフォルトは INVALID_GATHERING です。
    */
    void SetReferPersistentGatheringId(GatheringID referPersistentGatheringId)
    {
        m_ReferGid = referPersistentGatheringId;
    }

    /*!
    @brief      検索対象のあいことばをセットします

    空文字列をセットすることにより、あいことばを検索しない設定にすることができます。
    初期状態は空文字列です。

    @param[in]     codeword あいことば

    @retval        true  成功。あいことばとして使用できる文字列もしくは空文字列です
    @retval        false 失敗。あいことばとして使用できない文字が含まれています
    */
    qBool SetCodeword(const String& codeword);
    
    /*!
    @brief      検索範囲をセットします。

    MatchmakeExtensionClient::BrowseMatchmakeSession() の ResultRange を引数に取らないオーバーロード関数でのみここでセットした検索範囲が使用されます。
    それ以外のオーバーロード関数を使用した場合は引数で直接指定したものが使用されます。
    また、 MatchmakeExtensionClient::AutoMatchmake() に指定した場合も無視されます。

    @param[in]     resultRange    検索範囲

    @see MatchmakeExtensionClient::BrowseMatchmakeSession()
    */
    void SetResultRange(const ResultRange& resultRange)
    {
        m_ResultRange = resultRange;
    }

    /*!
    @brief      全ての検索条件をリセットします。

    全ての検索条件をリセットし、デフォルトコンストラクタで生成した状態にします。
    */
    void Reset();

    /*!
    @brief      検索条件が有効かどうか調べます。

    ゲームモードとマッチメイクシステムタイプがセットされていれば有効となります。
       
    @return 検索条件が有効かどうか
    */
    qBool IsValid() const
    {
        return m_GameMode != NEX_T("") && m_MatchmakeSystemType != NEX_T("");
    }

};

END_NAMESPACE_NEX

