﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "MatchmakeRefereePersonalRoundResultDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      ラウンドの個人結果を表すクラスです。
*/
class MatchmakeRefereePersonalRoundResult: public DDLCLASS(MatchmakeRefereePersonalRoundResult)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    MatchmakeRefereePersonalRoundResult()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~MatchmakeRefereePersonalRoundResult() {}

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetPrincipalId(0);
        SetPersonalRoundResultFlag(0);
        SetRoundWinLoss(0);
        SetRatingValueChange(0);
        this->buffer.clear();
    }

    /*!
        @brief      プリンシパル ID をセットします。

        @param[in]  pid_ 本オブジェクトが表すユーザのプリンシパル ID
    */
    void SetPrincipalId(PrincipalID pid_)
    {
        this->pid = pid_;
    }

    /*!
        @brief      プリンシパル ID を取得します。

        @return プリンシパル ID。
    */
    PrincipalID GetPrincipalId() const
    {
        return this->pid;
    }

    /*!
        @brief      結果ビットフラグをセットします。

        @param[in]  personalRoundResultFlag_ MatchmakeRefereeConstants::PersonalRoundResultFlag に対するビット単位の論理和です。
    */
    void SetPersonalRoundResultFlag(qUnsignedInt32 personalRoundResultFlag_)
    {
        this->personalRoundResultFlag = personalRoundResultFlag_;
    }

    /*!
        @brief      結果ビットフラグを取得します。

        @return 結果ビットフラグ。
    */
    qUnsignedInt32 GetPersonalRoundResultFlag() const
    {
        return this->personalRoundResultFlag;
    }

    /*!
        @brief      勝敗をセットします。

        @param[in]  roundWinLoss_ MatchmakeRefereeConstants::RoundResult で表す状態です。
    */
    void SetRoundWinLoss(qUnsignedInt32 roundWinLoss_)
    {
        this->roundWinLoss = roundWinLoss_;
    }

    /*!
        @brief      勝敗を取得します。

        @return 勝敗。
    */
    qUnsignedInt32 GetRoundWinLoss() const
    {
        return this->roundWinLoss;
    }

    /*!
        @brief      レートの変化量をセットします。

        @param[in]  ratingValueChange_ レートの変化量。
    */
    void SetRatingValueChange(qInt32 ratingValueChange_)
    {
        this->ratingValueChange = ratingValueChange_;
    }

    /*!
        @brief      レートの変化量を取得します。

        @return レートの変化量。
    */
    qInt32 GetRatingValueChange() const
    {
        return this->ratingValueChange;
    }

    /*!
        @brief      バイナリ値をセットします。

        src に指定された領域から size バイト分の内容をセットします。
        サーバにアップロードする際、size が MatchmakeRefereeConstants::STATS_BUFFER_MAX_SIZE 
       を超えていると処理に失敗します。 エラーの詳細は MatchmakeRefereeClient::EndRound() を参照下さい。

        @param[in]  src セットするメモリ領域の先頭アドレス
        @param[in]  size セットするメモリ領域のサイズ
    */
    void SetBuffer(const void* src, qUnsignedInt32 size)
    {
        this->buffer.clear();
        this->buffer.push_back((const qByte*)src, size);
    }

    /*!
        @brief      バイナリ値を取得します。

        @return バイナリ値。
    */
    const qBuffer& GetBuffer() const
    {
        return this->buffer;
    }

    /*!
        @brief      MatchmakeRefereePersonalRoundResult オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  bShowBufferContents buffer の中身も表示するか。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, qBool bShowBufferContents=false) const
    {
        StringStream ss;
        ss << NEX_T("MatchmakeRefereePersonalRoundResult: (")
           << NEX_T("pid=") << this->pid
           << NEX_T(", personalRoundResultFlag=") << this->personalRoundResultFlag
           << NEX_T(", roundWinLoss=") << roundWinLoss
           << NEX_T(", ratingValueChange=") << this->ratingValueChange
           << NEX_T(", buffer size=") << this->buffer.size();
        if (bShowBufferContents)
        {
            ss << NEX_T(", buffer contents=");
            for (qBuffer::const_iterator it = this->buffer.begin(); it != this->buffer.end(); ++it)
            {
                ss << hex << *it;
            }
        }
        ss << NEX_T(")");
        QTRACE(uiTraceFlags, ss.CStr());
    }
};

END_NAMESPACE_NEX

