﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <RendezVous/Services/Jugem/Ranking/src/Common/RankingCachedResultDDL.h>
#include <RendezVous/Services/Jugem/Ranking/src/Common/RankingResult.h>

BEGIN_NAMESPACE_NEX

class RankingOrderParam;
class RankingScoreData;

#ifdef FOR_DOC_PURPOSE
/*!
    @brief      キャッシュ付きのランキングの情報が格納されるクラスです。
 */
class RankingCachedResult: public RankingResult
#else
class RankingCachedResult: public DDLCLASS(RankingCachedResult)
#endif
{
public:
    /*!
        @brief      コンストラクタです。
    */
    RankingCachedResult()
    {
        this->maxLength = 0;
    };

    /*!
        @brief      デストラクタです。
    */
    virtual ~RankingCachedResult() {};

    /*!
        @brief      コンフィグで設定された最大長を返します

        @return     最大長
    */
    qUnsignedInt32 GetMaxLength() const
    {
        return this->maxLength;
    };

    /*!
        @brief      キャッシュ生成日時を取得します。

        @return     キャッシュ生成日時。 (UTC時間)
    */
    DateTime GetCreatedTime() const
    {
        return this->createdTime;
    };


    /*!
        @brief      キャッシュ有効期限日時を取得します。

        @return     キャッシュ有効期限日時。 (UTC時間)
    */
    DateTime GetExpiredTime() const
    {
        return this->expiredTime;
    };

    /*!
     *  @brief RankingCachedResult::LocalUpdateState() の戻り値を定義します。
     *
     */
    enum LocalUpdateState
    {
        UPDATE_SUCCESS = 0, //!< 成功
        UPDATE_FAILED = 1, //!< スコアの順位が低いなどで、更新に失敗した
        UPDATE_ERROR = 2 //!< 値が不正
    };

    /*!
        @brief      ローカルに保持されたキャッシュ付き Top ランキングを、指定されたスコアで更新します。

        RankingClient::GetCachedTopXRanking() や、 RankingClient::GetCachedTopXRankings() で取得されたデータは、
        更新されるような新しいスコアをアップロードしても、更新期限まで、
        新しいスコアが更新されることがありません。
        ローカルで、新しいスコアを更新することにより、あたかもスコアが更新されたような状態を作るための補助関数となります。
        サーバーへのアクセスは行わないので、スコアはあらかじめ保持しておく必要があります。

        制限事項としては、以下の通りです。

        @li ローカルで更新するので、キャッシュ更新までの間に他のユーザーからのスコア更新があった場合にランキングが結果が異なることがあります。
        @li GetCachedTopXRanking[s]() で渡したものと異なる orderParam を渡した場合の動作の一貫性は保証されません。
        @li orderParam のグループフィルタが、 RankingConstants::FILTER_GROUP_INDEX_NONE もしくは、 追加されるスコアと  orderParam のグループが一緒である必要があります。
        @li orderParam のグループフィルタで検索されない条件でも追加されます。
        @li utcCurrentTime の値にかかわらず、常に最新のデータとして追加されます。
        @li orderParam の offset と length は無視されます。

        @param[in] orderParam GetCachedTopXRanking[s]() で指定した順位計算用パラメータ
        @param[in] scoreData 更新スコア
        @param[in] pid 自分のプリンシパル ID 。 RankingClient::GetCredentials()->GetPrincipalID() より取得できます。
        @param[in] nexUniqueId 自分のNEXユニークID。デフォルトは、 INVALID_UNIQUEID。
        @param[in] utcCurrentTime 更新時に使用する更新時間。デフォルトは、 DateTime::Never 。 更新時間が必要であれば、 NgsFacade::GetGameServerTime() の値を指定してください。
        @param[in] pCommonData 更新時に使用する共通データの先頭ポインタ。 デフォルトは、 NULL で、共通データは追加されない。
        @param[in] commonDataSize 更新時に使用する共通データのサイズ。デフォルトは、0

        @retval        RankingCachedResult::UPDATE_SUCCESS 更新された。
        @retval        RankingCachedResult::UPDATE_FAILED 更新されなかった。( RankingConstants::UPDATE_MODE_NORMAL で更新スコアが既存のスコアより下位、もしくは、 最後に追加しようとしてもスコア数が上限に達していた。)
        @retval        RankingCachedResult::UPDATE_ERROR グループ値が不正。

        @see RankingClient::GetCachedTopXRanking() , RankingClient::GetCachedTopXRankings()
     */
    LocalUpdateState LocalUpdate(const RankingOrderParam& orderParam,
                const RankingScoreData& scoreData, PrincipalID pid,
                UniqueId nexUniqueId = INVALID_UNIQUEID, DateTime utcCurrentTime = DateTime::Never,
                const void * pCommonData = NULL, qByte commonDataSize = 0
                );


};

END_NAMESPACE_NEX

