﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Services/Jugem/Ranking2/src/Common/Ranking2Constants.h>
#include <RendezVous/Services/Jugem/Ranking2/src/Common/Ranking2DDF.h>
#include <RendezVous/Services/Jugem/Ranking2/src/Common/Ranking2ProtocolDDL.h>
#include <RendezVous/Services/Jugem/Ranking2/src/Common/Ranking2Errors.h>
#include <RendezVous/Services/Jugem/Ranking2/src/Common/Ranking2Successes.h>
#include <RendezVous/Services/Jugem/Ranking2/src/Common/Ranking2GetOptionFlags.h>
#include <RendezVous/Services/Jugem/Ranking2/src/Common/Ranking2SortFlags.h>
#include <RendezVous/Services/Jugem/Utility/src/Common/UniqueId.h>

BEGIN_NAMESPACE_NEX

/*!
    @brief Ranking2 機能を利用するためのクラス。
*/
class Ranking2Client : public ServiceClient
{
public:

    /*!
    @brief      コンストラクタです。

       コンストラクタです。
    */
    Ranking2Client();

    /*!
    @brief      デストラクタです。

    デストラクタです。
    */
    virtual ~Ranking2Client();

    void UpdateProtocolsDefaultCredentials(Credentials* pCredentials);

    // qBool Hello(ProtocolCallContext* pContext, const String& name);

    /*!
        @brief      スコアを登録します。

        １つのスコアを１つのカテゴリに登録します。
        ログインしているユーザーのスコアのみを登録可能です。他のユーザーのスコアを登録することはできません。
        カテゴリ設定で指定された正常範囲を外れたスコアが渡された場合は、ランキングには現れない場所に不正スコアとして隔離して保存されます。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Ranking2, InvalidArgument) : パラメータの指定に問題があります。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  scoreData スコアの情報。
        @param[in]  nexUniqueId NEX ユニーク ID。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool PutScore(ProtocolCallContext* pContext, const Ranking2ScoreData& scoreData,
        UniqueId nexUniqueId = INVALID_UNIQUEID);

    /*!
        @brief      スコアを登録します。

        複数のスコアをそれぞれ別のカテゴリに登録します。
        ログインしているユーザーのスコアのみを登録可能です。他のユーザーのスコアを登録することはできません。
        同時に登録できるスコアの数は、最大 Ranking2Constants::MAX_PUT_MULTI_SCORES 個です。
        カテゴリ設定で指定された正常範囲を外れたスコアが渡された場合は、ランキングには現れない場所に不正スコアとして隔離して保存されます。
        正常範囲に収まるスコアと外れるスコアが混在している場合は、正常範囲に収まるスコアのみランキングに登録され、外れるスコアは隔離されます。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Ranking2, InvalidArgument) : パラメータの指定に問題があります。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  scoreDataList 複数のスコアの情報。
        @param[in]  nexUniqueId NEX ユニーク ID。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool PutScore(ProtocolCallContext* pContext, const qVector<Ranking2ScoreData>& scoreDataList,
        UniqueId nexUniqueId = INVALID_UNIQUEID);

    /*!
        @brief      ユーザー共通データを取得します。

        ユーザー名、バイナリデータを格納したユーザー共通データを取得します。
        ユーザー共通データを登録していないユーザーのデータを取得しようとした場合、それぞれの値が空の状態で返ります。

        
        オプションフラグの指定により、Mii データも合わせて取得することができます。
        Mii データは、該当ユーザーの
        「じぶんの Mii」
        のデータが取得されます。
        

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Ranking2, InvalidArgument) : パラメータの指定に問題があります。

        @param[in,out]  pContext 呼び出し情報。
        @param[out] pCommonData ユーザー共通データ。
        @param[in]  optionFlags オプションフラグ。Ranking2GetOptionFlags の値を指定します。
        @param[in]  principalId プリンシパル ID。
        @param[in]  nexUniqueId NEX ユニーク ID。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。

        @see        Ranking2GetOptionFlags
    */
    qBool GetCommonData(ProtocolCallContext* pContext, Ranking2CommonData* pCommonData,
        qUnsignedInt32 optionFlags = 0, PrincipalID principalId = INVALID_PRINCIPALID, UniqueId nexUniqueId = INVALID_UNIQUEID);

    /*!
        @brief      ユーザー共通データを登録します。

        ユーザー名、バイナリデータを格納したユーザー共通データを登録します。
        
        Mii データの登録は行いません。
        
        ログインしているユーザーのデータのみを登録可能です。他のユーザーのデータを登録することはできません。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Ranking2, InvalidArgument) : パラメータの指定に問題があります。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  commonData ユーザー共通データ。
        @param[in]  nexUniqueId NEX ユニーク ID。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool PutCommonData(ProtocolCallContext* pContext, const Ranking2CommonData& commonData,
        UniqueId nexUniqueId = INVALID_UNIQUEID);

    /*!
        @brief      ユーザー共通データを削除します。

        ユーザー共通データを削除します。
        
        Mii データの削除は行いません。
        
        ログインしているユーザーのデータのみを削除可能です。他のユーザーのデータを削除することはできません。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Ranking2, InvalidArgument) : パラメータの指定に問題があります。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  nexUniqueId NEX ユニーク ID。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool DeleteCommonData(ProtocolCallContext* pContext, UniqueId nexUniqueId = INVALID_UNIQUEID);

    /*!
        @brief      ランキングを取得します。

        指定した条件にしたがってランキングを取得します。条件の詳細は Ranking2GetParam を参照してください。

        結果は Ranking2Info オブジェクトに格納されます。

        条件に合致するデータが存在しない場合、
        ランキングリストは空になりますが、その他の情報は取得することができます。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Ranking2, InvalidArgument) : パラメータの指定に問題があります。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  getParam ランキング取得条件の情報。
        @param[out] pRankingInfo ランキング情報。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetRanking(ProtocolCallContext* pContext, const Ranking2GetParam& getParam, Ranking2Info* pRankingInfo);

    /*!
        @brief      プリンシパル ID リストを用いてランキングを取得します。

        指定したプリンシパル ID リスト内の ID で登録されているデータだけを抽出したランキングを取得します。
        その他の条件の詳細は Ranking2GetByListParam を参照してください。

        結果は GetRanking と同様です。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Ranking2, InvalidArgument) : パラメータの指定に問題があります。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  getParam ランキング取得条件の情報。
        @param[in]  principalIdList プリンシパル ID リスト。
        @param[out] pRankingInfo ランキング情報。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetRanking(ProtocolCallContext* pContext, const Ranking2GetByListParam& getParam,
        const qVector<PrincipalID>& principalIdList, Ranking2Info* pRankingInfo);

    /*!
        @brief      カテゴリ設定を取得します。

        指定したカテゴリに設定された値が取得できます。
        設定されていないカテゴリを指定した場合、デフォルト値が取得できます。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  category カテゴリ。
        @param[out] pCategorySetting カテゴリ設定。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetCategorySetting(ProtocolCallContext* pContext,
        qUnsignedInt32 category, Ranking2CategorySetting* pCategorySetting);


    /*!
        @brief      ランキングチャートデータを取得します。

        info で指定されたランキングチャートデータを取得します。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Ranking2, InvalidArgument) : パラメータの指定に問題があります。

        初回更新前や設定ミスがあった場合はランキングチャートデータが生成されません。
        そのような生成されていないランキングチャートデータを取得しようとした場合は pChart->IsValid() が false を返すオブジェクトが返ります。


        @param[in,out]  pContext 呼び出し情報。
        @param[in]  info 取得するランキングチャートデータを指定
        @param[out] pChart ランキングチャートデータ

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetRankingChart(ProtocolCallContext* pContext,
            const Ranking2ChartInfoInput& info, Ranking2ChartInfo* pChart);

    /*!
        @brief      ランキングチャートデータを複数取得します。

        infoArray で指定されたランキングチャートデータを複数取得します。
        一度に最大 Ranking2Constants::MAX_CHART_GET_LENGTH 設定可能です。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Ranking2, InvalidArgument) : パラメータの指定に問題があります。

        初回更新前や設定ミスがあった場合はランキングチャートデータが生成されません。そのような生成されていないチャートデータは結果に含まれません。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  infoArray 取得するランキングチャートデータを指定した qVector 配列
        @param[out] pChartArray ランキングチャートデータの qVector 配列

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetRankingChart(ProtocolCallContext* pContext,
            const qVector<Ranking2ChartInfoInput>& infoArray, qVector<Ranking2ChartInfo>* pChartArray);

    /*!
        @brief      スコアから順位を推定します。最低順位を下回るスコアでも推定値で順位を取得できます。

        最低順位を下回るスコアは、サンプリングされたデータに基づいて推測して算出します。

        データが存在しない場合は、pOutput->rank や pOutput->length が 0 となります。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Ranking2, InvalidArgument) : パラメータの指定に問題があります。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  input スコアと、カテゴリ等の設定
        @param[out] pOutput 推定された順位結果

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetEstimateScoreRank(ProtocolCallContext* pContext, const Ranking2EstimateScoreRankInput& input, Ranking2EstimateScoreRankOutput* pOutput);

private:
    Ranking2ProtocolClient m_oProtocol;
};

END_NAMESPACE_NEX

