﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include "Ranking2ChartInfoDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      生成されたランキングチャートデータを格納するクラスです。
*/
class Ranking2ChartInfo: public DDLCLASS(Ranking2ChartInfo)
{
public:
    /*!
        @brief      コンストラクタ。
    */
    Ranking2ChartInfo()
    {
    }

    /*!
        @brief      デストラクタ。
    */
    virtual ~Ranking2ChartInfo() {}

    /*!
        @brief ランキングチャートデータが生成された日時を取得します。

        @return 生成日時
    */
    const DateTime& GetCreateTime() const
    {
        return createTime;
    }
    
    /*!
        @brief ランキングチャートデータのインデックス番号を取得します。

        @return ランキングチャートデータのインデックス番号
    */
    qUnsignedInt32 GetIndex() const
    {
        return index;
    }
    
    /*!
        @brief ランキングチャートデータ作成の元になったカテゴリを取得します。

        @return カテゴリ
    */
    qUnsignedInt32 GetCategory() const
    {
        return category;
    }
    
    /*!
        @brief シーズンを取得します。

        @return シーズン
    */
    qUnsignedInt GetSeason() const
    {
        return season;
    }
    
    /*!
        @brief 分布図の分割区間数を取得します。

        最大255です。

        @return 分割区間数
    */
    qUnsignedInt8 GetBinsSize() const
    {
        return binsSize;
    }

    /*!
        @brief データが有効か否かを返します。

        @retval true 有効
        @retval false 無効
    */
    qBool IsValid() const
    {
        return binsSize > 0;
    }

    /*!
        @brief 標本比率を取得します。

        実際の標本比率は、(1/ (1 << GetSamplingRate()) となり、
        母集団 * (1 / (1 << GetSamplingRate())) の分の標本データが推定に用いられていることを示します。
        0 が返された場合は、全てのスコアが標本データとなっている事を示します。
        1 が返された場合は、約 50% のスコアが標本データとなっている事を示します。
        2 が返された場合は、約 25% のスコアが標本データとなっている事を示します。

        @return 標本比率の逆数のビットシフト前の値
    */
    qUnsignedInt GetSamplingRate() const
    {
        return static_cast<qUnsignedInt>(samplingRate);
    }

    /*!
        @brief 順番を取得します。

        @retval Ranking2Constants::SCORE_ORDER_ASC 昇順
        @retval Ranking2Constants::SCORE_ORDER_DESC 降順
    */
    
    qBool GetScoreOrder() const
    {
        return scoreOrder;
    }
    
    /*!
        @brief 推定スコア総数を取得します。

        ランキングチャートの設定にかかわらず、アップロードされたスコアの推定総数です。

        @return 推定スコア総数
    */
    qUnsignedInt32 GetEstimateLength() const
    {
        return estimateLength;
    }
    
    /*!
        @brief 推定最高スコアを取得します。

        @return 推定最高スコア
    */
    qUnsignedInt32 GetEstimateHighestScore() const
    {
        return estimateHighestScore;
    }

    /*!
        @brief 推定最低スコアを取得します。

        @return 推定最低スコア
    */
    qUnsignedInt32 GetEstimateLowestScore() const
    {
        return estimateLowestScore;
    }

    /*!
        @brief 推定中央スコアを取得します。

        @return 推定中央スコア
    */
    qUnsignedInt32 GetEstimateMedianScore() const
    {
        return estimateMedianScore;
    }

    /*!
        @brief 推定平均スコアを取得します。

        @return 推定平均スコア
    */
    qDouble GetEstimateAverageScore() const
    {
        return estimateAverageScore;
    }

    /*!
        @brief 区間終了の最高スコアを取得します。

        実際には、この値よりひとつ低位のスコアまでとなります。

        @return 区間終了の最高スコア
    */
    qUnsignedInt32 GetHighestBinsScore() const
    {
        return highestBinsScore;
    }

    /*!
        @brief 区間開始の最低スコアを取得します。

        @return 区間開始の最低スコア
    */
    qUnsignedInt32 GetLowestBinsScore() const
    {
        return lowestBinsScore;
    }

    /*!
        @brief 分布図の区間幅を取得します。

        @return 区間幅
    */
    qUnsignedInt32 GetBinsWidth() const
    {
        return binsWidth;
    }

    /*!
        @brief 任意の設定値 その1を取得します。

        @return 設定値
    */
    qUnsignedInt32 GetAttribute1() const
    {
        return attribute1;
    }

    /*!
        @brief 任意の設定値 その2を取得します。

        @return 設定値
    */
    qUnsignedInt32 GetAttribute2() const
    {
        return attribute2;
    }

    /*!
        @brief 区間ごとの度数を取得します。

        区間最高スコアから区間最低スコアへ向かって度数が数えられます。

        @return 区間ごとの度数
    */
    const qVector<qUnsignedInt32>& GetQuantities() const
    {
        return quantities;
    }
    
    
    /*!
        @brief      Ranking2CategorySetting オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const Ranking2ChartInfo& rch);

    /*!
         @brief デバッグ用にオブジェクトの内容を文字列として出力します。

         @return デバッグ用の出力文字列
     */
    String ToString() const
    {
        StringStream ss;
        ss << *this;
        return String(ss.CStr());
    }
};

inline StringStream& operator<<(StringStream& oStr, const Ranking2ChartInfo& rch)
{
    oStr << NEX_T("Ranking2ChartInfo: (")
         << NEX_T("createTime=") << rch.createTime
         << NEX_T(", chartIndex=") << rch.index
         << NEX_T(", category=") << rch.category
         << NEX_T(", season=") << rch.season
         << NEX_T(", binsSize=") << static_cast<qInt>(rch.binsSize)
         << NEX_T(", samplingRate=") << static_cast<qInt>(rch.samplingRate)
         << NEX_T(", scoreOrder(IsDesc)=") << rch.scoreOrder
         << NEX_T(", estimateLength=") << rch.estimateLength
         << NEX_T(", estimateHighestScore=") << rch.estimateHighestScore
         << NEX_T(", estimateLowestScore=") << rch.estimateLowestScore
         << NEX_T(", estimateMedianScore=") << rch.estimateMedianScore
         << NEX_T(", estimateAverageScore=") << rch.estimateAverageScore
         << NEX_T(", highestBinsScore=") << rch.highestBinsScore
         << NEX_T(", lowestBinsScore=") << rch.lowestBinsScore
         << NEX_T(", binsWidth=") << rch.binsWidth
         << NEX_T(", attribute1=") << rch.attribute1
         << NEX_T(", attribute2=") << rch.attribute2
         << NEX_T(", quantities=[");
    for (qVector<qUnsignedInt32>::const_iterator it = rch.GetQuantities().begin(); it != rch.GetQuantities().end(); ++it)
    {
        oStr << *it << NEX_T(",");
    }
    oStr << NEX_T("]");
    oStr << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX

