﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/Jugem/DataStore/src/Common/DataStoreConstants.h>
#include "ScreeningDataStoreContentParamDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @internal
    @brief 通報対象にするデータストアの情報を保持するクラスです。
 */
class ScreeningDataStoreContentParam: public DDLCLASS(ScreeningDataStoreContentParam)
{
public:
    /*!
        @brief コンストラクタです。
    */
    ScreeningDataStoreContentParam()
    {
        Reset();
    }

    /*!
        @brief デストラクタです。
    */
    virtual ~ScreeningDataStoreContentParam() {}

    /*!
        @brief 通報するデータのデータ ID をセットします。

        違反があった場合に削除が行われるデータです。
        デフォルトは DataStoreConstants::INVALID_DATAID です。

        @param[in] dataId_ 通報するデータのデータ ID。
    */
    void SetDataId(qUnsignedInt64 dataId_)
    {
        this->dataId = dataId_;
    }

    /*!
        @brief 通報するデータのデータ ID を取得します。

        @return 通報するデータのデータ ID。
    */
    qUnsignedInt64 GetDataId() const
    {
        return this->dataId;
    }

    /*!
        @brief 審査で使用するデータのデータ ID をセットします。

        通報対象とは別のデータを審査に使用する場合に設定します。
        デフォルトは DataStoreConstants::INVALID_DATAID で、
        通報対象のデータが審査で使用されます。

        @param[in] contentDataId_ 審査で使用するデータのデータ ID。
    */
    void SetContentDataId(qUnsignedInt64 contentDataId_)
    {
        this->contentDataId = contentDataId_;
    }

    /*!
        @brief 審査で使用するデータのデータ ID を取得します。

        通報対象のデータで審査を行う場合、
        DataStoreConstants::INVALID_DATAID が返ります。

        @return 審査で使用するデータのデータ ID。
    */
    qUnsignedInt64 GetContentDataId() const
    {
        return this->contentDataId;
    }

    /*!
        @brief UGC 企画を識別する文字列をセットします。

        @param[in] ugcType_ UGC 企画を識別する文字列。
    */
    void SetUgcType(const String& ugcType_)
    {
        this->ugcType = ugcType_;
    }

    /*!
        @brief UGC 企画を識別する文字列を取得します。

        @return UGC 企画を識別する文字列。
    */
    const String& GetUgcType() const
    {
        return this->ugcType;
    }

    /*!
        @brief 審査対象のコンテンツの言語をセットします。

        @param[in] language_ 審査対象のコンテンツの言語。
    */
    void SetLanguage(const String& language_)
    {
        this->language = language_;
    }

    /*!
        @brief 審査対象のコンテンツの言語を取得します。

        @return 審査対象のコンテンツの言語。
    */
    const String& GetLanguage() const
    {
        return this->language;
    }

    /*!
        @brief ゲーム内のコンテンツと紐づけるための検索キーをセットします。

        ゲーム内でユーザーからの問い合わせなどで聴取できるキーをセットします。
        設定しない場合、通報対象のデータ ID に対応するデータコード文字列を使用します。

        @param[in] searchKey_ ゲーム内のコンテンツと紐づけるための検索キー。
    */
    void SetSearchKey(const String& searchKey_)
    {
        this->searchKey = searchKey_;
    }

    /*!
        @brief ゲーム内のコンテンツと紐づけるための検索キーを取得します。

        @return ゲーム内のコンテンツと紐づけるための検索キー。
    */
    const String& GetSearchKey() const
    {
        return this->searchKey;
    }

    /*!
        @brief オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetDataId(DataStoreConstants::INVALID_DATAID);
        SetContentDataId(DataStoreConstants::INVALID_DATAID);
        SetUgcType(NEX_T(""));
        SetLanguage(NEX_T(""));
        SetSearchKey(NEX_T(""));
    }

    /*!
        @brief      OrderedGear オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        StringStream ss;
        ss << NEX_T("ScreeningDataStoreContentParam: (")
           << NEX_T("dataId=") << GetDataId()
           << NEX_T(", contentDataId=") << GetContentDataId()
           << NEX_T(", ugcType=") << GetUgcType()
           << NEX_T(", language=") << GetLanguage()
           << NEX_T(", searchKey=") << GetSearchKey()
           << NEX_T(")");
        QTRACE(uiTraceFlags, ss.CStr());
    }
};

END_NAMESPACE_NEX
