﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2018 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/Jugem/Screening/src/Common/ScreeningConstants.h>
#include "ScreeningUgcViolationParamDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @internal
    @brief UGC 通報の通報の情報を表すクラスです。通報対象の情報を含みません。
 */
class ScreeningUgcViolationParam: public DDLCLASS(ScreeningUgcViolationParam)
{
public:
    /*!
        @brief コンストラクタです。
    */
    ScreeningUgcViolationParam()
    {
        Reset();
    }

    /*!
        @brief デストラクタです。
    */
    virtual ~ScreeningUgcViolationParam() {}

    /*!
        @brief 通報カテゴリをセットします。

        通報カテゴリを指定します。
        デフォルトは ScreeningConstants::REPORT_CATEGORY_INVALID です。

        @param[in] category 通報カテゴリ。
        @return セットできた場合は true、不正な値のためセットできなかった場合 false を返します。
    */
    qBool SetCategory(ScreeningConstants::ReportCategory category);

    /*!
        @brief 通報カテゴリを取得します。

        @return 通報カテゴリ。
    */
    ScreeningConstants::ReportCategory GetCategory() const;

    /*!
        @internal
        @brief 通報カテゴリを直値でセットします。

        @param[in] categoryString_ 通報カテゴリ（直値）。
    */
    void SetCategoryString(const String& categoryString_)
    {
            this->categoryString = categoryString_;
    }

    /*!
        @internal
        @brief 通報カテゴリ（直値）を取得します。

        @return 通報カテゴリ（直値）。
    */
    const String& GetCategoryString() const
    {
        return this->categoryString;
    }

    /*!
        @brief 通報理由（自由テキスト）をセットします。

        @param[in] reason_ 通報理由（自由テキスト）。最大 400 文字。
    */
    void SetReason(const String& reason_)
    {
        this->reason = reason_;
    }

    /*!
        @brief 通報理由を取得します。

        @return 通報理由。
    */
    String GetReason() const
    {
        return this->reason;
    }

    /*!
        @internal
        @brief 通報に対して審査を行わない指定を行います。

        初期値は false です。

        @param[in] noscreening_ 審査を行わない指定。true の場合、審査を行わない。
    */
    void SetNoscreening(qBool noscreening_)
    {
        this->noscreening = noscreening_;
    }

    /*!
        @internal
        @brief 通報に対して審査を行わない指定を取得します。

        @return 審査を行わない指定。true の場合、審査を行わない。
    */
    qBool IsNoscreening() const
    {
        return this->noscreening;
    }

    /*!
        @brief 通報の追加情報を設定します。

        通報の状況などを示す任意の追加データを設定できます。

        @param[in] key 追加情報のキー。
        @param[in] value 追加情報の値。
    */
    void SetContextInfo(const String& key, const String& value)
    {
        this->context[key] = value;
    }

    /*!
        @brief 通報の追加情報を取得します。

        @param[in] key 追加情報のキー。
        @return 追加情報の値。設定されていない場合、空文字列を返します。
    */
    const String& GetContextInfo(const String& key) const;

    /*!
        @brief オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetCategoryString(NEX_T(""));
        SetReason(NEX_T(""));
        SetNoscreening(false);
        this->context.clear();
    }

    /*!
        @brief オブジェクトの内容をログに出力します。

        @param[in] uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        StringStream ss;
        ss << NEX_T("ScreeningUgcViolationParam: (")
           << NEX_T(", category=") << GetCategoryString()
           << NEX_T(", reason=") << GetReason()
           << NEX_T(", noscreening=") << IsNoscreening()
           << NEX_T(", context=(");
        for (
            qMap<String, String>::const_iterator it = this->context.begin();
            it != this->context.end();
            ++it
        )
        {
            ss << it->first << NEX_T(":") << it->second << NEX_T(", ");
        }
        ss << NEX_T("))");
        QTRACE(uiTraceFlags, ss.CStr());
    }
};

END_NAMESPACE_NEX
