﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Services/Jugem/ServiceItem/src/Common/ServiceItemDDF.h>
#include <RendezVous/Services/Jugem/ServiceItem/src/Common/ServiceItemProtocolDDL.h>
#include <RendezVous/Services/Jugem/ServiceItem/src/Common/ServiceItemErrors.h>
#include <RendezVous/Services/Jugem/ServiceItem/src/Client/ServiceItemInternalClient.h>
#include <RendezVous/Services/Jugem/ServiceItem/src/Client/ServiceItemAsynchronousRmcTraits.h>
#include <RendezVous/Services/Jugem/ServiceItem/src/Client/ServiceItemClientInterface.h>

BEGIN_NAMESPACE_NEX

/*!
    @brief サービスアイテム機能を利用するためのクラスです。

    典型的な API の発行順序を以下に示します。

    (1) NGS にログインします。 @n
    (2) GetServiceItemRight() で最新の権利情報を取得します。 @n
    (3) サービスアイテムを購入するシーンの場合は以下を行います。

    (3-1) ListServiceItem() で販売しているサービスアイテムの一覧を取得し、購入画面を表示します。 @n
    (3-2) GetPrepurchaseInfo() で購入前情報を取得し、購入の最終確認画面を表示します。 @n
    (3-3) AcquireEcServiceTokenAndPurchaseServiceItem() で購入します。 @n
    (3-4) GetServiceItemRight() で最新の権利情報を取得し直します。

    (4) 消費型サービス利用券を消費するシーンの場合は以下を行います。

    (4-1) UseServiceItemByAccount() で消費型サービス利用券を消費します。 @n
          この時、ユーザサポート向けの管理ツールに表示されるログメッセージをセットします。 @n
          また、保護すべきデータを権利用バイナリデータとしてセットできます。 @n
          セットした権利用バイナリデータは GetServiceItemRight() で権利情報として取得できます。 @n
    (4-2) 消費型サービス利用券を消費した対価となるアプリケーションの処理を行います。 @n
    (4-3) 必要に応じて PostRightBinaryByAccount() でユーザサポート向けの管理ツールに表示されるログメッセージと権利用バイナリをセットします。 @n
          セットした権利用バイナリデータは GetServiceItemRight() で権利情報として取得できます。 @n

    (5) 特別に無料でサービスアイテムを獲得したい場合は AcquireServiceItemByAccount() を実行します。 @n
        この時、ユーザサポート向けの管理ツールに表示されるログメッセージをセットします。 @n
        また、保護すべきデータを権利用バイナリデータとしてセットできます。 @n
        セットした権利用バイナリデータは GetServiceItemRight() で権利情報として取得できます。 @n

    (6) 権利情報の詳細画面を表示する場合は毎回 GetServiceItemRightStatement() で最新の権利情報を取得します。 @n

*/
class ServiceItemClient : public ServiceItemInternalClient, public ServiceItemClientInterface
{
public:

    /*!
        @brief      コンストラクタです。

        コンストラクタです。
    */
    ServiceItemClient();

    /*!
        @brief      デストラクタです。

        デストラクタです。
    */
    virtual ~ServiceItemClient(){};

    /*!
        @internal
        @brief      任意の URL に対して HTTP GET を行います。テスト用。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool HttpGet(ProtocolCallContext *pContext, const ServiceItemHttpGetParam& request, ServiceItemHttpGetResponse* response)
    {
        return AsynchronousRmc<ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_HTTP_GET> >(pContext, request, response);
    }

    /*!
        @brief      サービスアイテムを購入します。

        本関数を使用する場合はアプリケーションで事前に EC サービストークンを取得する必要があります。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool PurchaseServiceItem(ProtocolCallContext *pContext, const ServiceItemPurchaseServiceItemParam& request, ServiceItemPurchaseServiceItemResponse* response)
    {
        return AsynchronousRmc<ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_PURCHASE_SERVICE_ITEM> >(pContext, request, response);
    }

    /*!
        @brief      販売しているサービスアイテムの一覧を取得します。

        サービスイン後の変更を考慮し、サーバから受け取ったカタログの順番・個数をそのままアプリケーション側で表示してください。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool ListServiceItem(ProtocolCallContext *pContext, const ServiceItemListServiceItemParam& request, ServiceItemListServiceItemResponse* response)
    {
        return AsynchronousRmc<ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_LIST_SERVICE_ITEM> >(pContext, request, response);
    }

    /*!
        @brief      アカウントの残高情報を取得します。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetBalance(ProtocolCallContext *pContext, const ServiceItemGetBalanceParam& request, ServiceItemGetBalanceResponse* response)
    {
        return AsynchronousRmc<ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_GET_BALANCE> >(pContext, request, response);
    }

    /*!
        @brief      購入前情報を取得します。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetPrepurchaseInfo(ProtocolCallContext *pContext, const ServiceItemGetPrepurchaseInfoParam& request, ServiceItemGetPrepurchaseInfoResponse* response)
    {
        return AsynchronousRmc<ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_GET_PREPURCHASE_INFO> >(pContext, request, response);
    }

    /*!
        @brief      権利情報を取得します。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetServiceItemRight(ProtocolCallContext *pContext, const ServiceItemGetServiceItemRightParam& request, ServiceItemGetServiceItemRightResponse* response)
    {
        return AsynchronousRmc<ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_GET_SERVICE_ITEM_RIGHT> >(pContext, request, response);
    }

    /*!
      @brief      権利用バイナリを除いた権利情報を取得します。

        本関数は権利用バイナリを返却しない以外は GetServiceItemRight() と同様の結果を返却します。
        権利用バイナリが含まれる qVector<ServiceItemRightBinary> のサイズは 0 となります。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetServiceItemRightStatement(ProtocolCallContext *pContext, const ServiceItemGetServiceItemRightParam& request, ServiceItemGetServiceItemRightResponse* response)
    {
        return AsynchronousRmc<ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_GET_SERVICE_ITEM_RIGHT_STATEMENT> >(pContext, request, response);
    }

    /*!
        @brief      購入履歴を取得します。

        レスポンスは最新の購入履歴から昇順でソートされた状態で返ってきます。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetPurchaseHistory(ProtocolCallContext *pContext, const ServiceItemGetPurchaseHistoryParam& request, ServiceItemGetPurchaseHistoryResponse* response)
    {
        return AsynchronousRmc<ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_GET_PURCHASE_HISTORY> >(pContext, request, response);
    }

    /*!
        @brief      法令に基づく表示のメッセージを取得します。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetLawMessage(ProtocolCallContext *pContext, const ServiceItemGetLawMessageParam& request, ServiceItemGetLawMessageResponse* response)
    {
        return AsynchronousRmc<ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_GET_LAW_MESSAGE> >(pContext, request, response);
    }

    /*!
        @brief      継続課金アイテム一覧を取得します。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool ListAutoBillingServiceItem(ProtocolCallContext *pContext, const ServiceItemListAutoBillingServiceItemParam& request, ServiceItemListAutoBillingServiceItemResponse* response)
    {
        return AsynchronousRmc<ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_LIST_AUTO_BILLING_SERVICE_ITEM> >(pContext, request, response);
    }

    /*!
        @brief      現在契約中の継続課金アイテムを取得します。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetAutoBillingContract(ProtocolCallContext *pContext, const ServiceItemGetAutoBillingContractParam& request, ServiceItemGetAutoBillingContractResponse* response)
    {
        return AsynchronousRmc<ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_GET_AUTO_BILLING_CONTRACT> >(pContext, request, response);
    }

    /*!
        @brief      継続課金を解除します。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool CancelAutoBilling(ProtocolCallContext *pContext, const ServiceItemCancelAutoBillingParam& request, ServiceItemCancelAutoBillingResponse* response)
    {
        return AsynchronousRmc<ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_CANCEL_AUTO_BILLING> >(pContext, request, response);
    }

    /*!
        @brief      引換番号で引換可能なサービスアイテムの一覧を取得します。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool ListRedemptionServiceItem(ProtocolCallContext *pContext, const ServiceItemListRedemptionServiceItemParam& request, ServiceItemListRedemptionServiceItemResponse* response)
    {
        return AsynchronousRmc<ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_LIST_REDEMPTION_SERVICE_ITEM> >(pContext, request, response);
    }

    /*!
        @brief      サービスアイテムを引き換えます。

            本関数を使用する場合はアプリケーションで事前に EC サービストークンを取得する必要があります。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool RedeemServiceItem(ProtocolCallContext *pContext, const ServiceItemRedeemServiceItemParam& request, ServiceItemRedeemServiceItemResponse* response)
    {
        return AsynchronousRmc<ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_REDEEM_SERVICE_ITEM> >(pContext, request, response);
    }

    /*!
        @brief      EC サービストークンを取得し、サービスアイテムを購入します。

        本関数を使用する場合はアプリケーションで事前に EC サービストークンを取得する必要はありません。
        EC サービストークンを並列して取得することができないため、本関数ではレスポンスを受けとるまで並列して実行することはできません。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool AcquireEcServiceTokenAndPurchaseServiceItem(ProtocolCallContext *pContext, const ServiceItemPurchaseServiceItemParam& request,
                                                      ServiceItemPurchaseServiceItemResponse* response);

    /*!
        @brief      EC サービストークンを取得し、サービスアイテムを引き換えます。

            本関数を使用する場合はアプリケーションで事前に EC サービストークンを取得する必要はありません。
            EC サービストークンを並列して取得することができないため、本関数ではレスポンスを受けとるまで並列して実行することはできません。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool AcquireEcServiceTokenAndRedeemServiceItem(ProtocolCallContext *pContext, const ServiceItemRedeemServiceItemParam& request,  ServiceItemRedeemServiceItemResponse* response);

    //qBool PostRightBinaryByAccount(ProtocolCallContext *pContext, const ServiceItemPostRightBinaryByAccountParam& request, ServiceItemPostRightBinaryResponse* response)
    //{
    //    return AsynchronousRmc<ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_POST_RIGHT_BINARY_BY_ACCOUNT> >(pContext, request, response);
    //}

    /*!
        @brief      アカウント単位で消費型サービス利用券を消費すると同時に権利用バイナリデータを登録します。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool UseServiceItemByAccount(ProtocolCallContext *pContext, const ServiceItemUseServiceItemByAccountParam& request, ServiceItemUseServiceItemResponse* response)
    {
        return AsynchronousRmc<ServiceItemAsynchronousRmcTraits<SERVICEITEM_ASYNCHRONOUS_RMC_USE_SERVICE_ITEM_BY_ACCOUNT> >(pContext, request, response);
    }

protected:
    template<class TAsynchronousRmcTraits>
    qBool AsynchronousRmc(ProtocolCallContext *pContext,
                          const typename TAsynchronousRmcTraits::TRequest& request,
                          typename TAsynchronousRmcTraits::TResponse* response);
};

END_NAMESPACE_NEX

