﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemAcquireServiceItemByAccountParamDDL.h"
#include "ServiceItemConstants.h"
#include "ServiceItemConstantsInternal.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      サービスアイテムを獲得すると同時にアカウント単位で利用する権利用バイナリデータを登録するためのパラメータを表すクラスです。

    @copydoc    DescriptionOfRightBinary
*/
class ServiceItemAcquireServiceItemByAccountParam: public DDLCLASS(ServiceItemAcquireServiceItemByAccountParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemAcquireServiceItemByAccountParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemAcquireServiceItemByAccountParam() {}

    /*!
        @brief      獲得したいサービスアイテムのリファレンス ID を取得します。

        @return     獲得したいサービスアイテムのリファレンス ID 。
    */
    const String& GetReferenceIdForAcquisition() const { return referenceIdForAcquisition; }

    /*!
        @brief      [必須パラメタ] 獲得したいサービスアイテムのリファレンス ID をセットします。

        @param[in]  referenceIdForAcquisition_ 獲得したいサービスアイテムのリファレンス ID 。
    */
    void SetReferenceIdForAcquisition(const String& referenceIdForAcquisition_) { this->referenceIdForAcquisition = referenceIdForAcquisition_; }

    /*!
        @brief      権利用バイナリデータを登録したいサービスアイテムのリファレンス ID を取得します。

        @return     権利用バイナリデータを登録したいサービスアイテムののリファレンス ID 。
    */
    const String& GetReferenceIdForRightBinary() const { return referenceIdForRightBinary; }

    /*!
        @brief      [必須パラメタ] 権利用バイナリデータを登録したいサービスアイテムのリファレンス ID をセットします。

        @param[in]  referenceIdForRightBinary_ 権利用バイナリデータを登録したいサービスアイテムのリファレンス ID 。
    */
    void SetReferenceIdForRightBinary(const String& referenceIdForRightBinary_) { this->referenceIdForRightBinary = referenceIdForRightBinary_; }

    /*!
        @brief      権利用バイナリデータの利用タイプを取得します。

        @return     権利用バイナリデータの利用タイプ。
    */
    qByte GetUseType() const { return useType; }

    /*!
        @brief      [必須パラメタ] 権利用バイナリデータの利用タイプをセットします。

        アプリケーションでの権利用バイナリデータの用途に応じて 1-15 までの値をセットできます。

        @param[in]  useType_ 権利用バイナリデータの利用タイプ。
    */
    void SetUseType(qByte useType_)
    {
        SYSTEMCHECK(useType_ >= 1 && useType_ <= 15 && "Invalid useType. Please set between 1 and 15.");
        this->useType = useType_;
    }

    /*!
        @brief      権利種別を取得します。

        @return     権利種別。
    */
    ServiceItemConstants::LimitationType GetLimitationType() const { return static_cast<ServiceItemConstants::LimitationType>(limitationType); }

    /*!
        @brief      [必須パラメタ] 権利種別をセットします。

        獲得したいサービスアイテムの権利種別をセットします。
        現時点では、獲得できるサービスアイテムの種類として消費型サービス利用券のみをサポートしているため、
        必ず ServiceItemConstants::LIMITATION_TYPE_CONSUMPTION をセットしてください。

        @param[in]  limitationType_ 獲得したいサービスアイテムの権利種別。
    */
    void SetLimitationType(ServiceItemConstants::LimitationType limitationType_)
    {
        SYSTEMCHECK(limitationType_ == ServiceItemConstants::LIMITATION_TYPE_CONSUMPTION && "Invalid limitationType. Please set LIMITATION_TYPE_CONSUMPTION.");
        // SYSTEMCHECK(limitationType != ServiceItemConstants::LIMITATION_TYPE_INVALID && "Invalid limitationType. Please set other than LIMITATION_TYPE_INVALID.");
        this->limitationType = static_cast<qUnsignedInt32>(limitationType_);
    }

    /*!
        @brief      権利の値を取得します。

        @return     権利の値。
    */
    qUnsignedInt32 GetLimitationValue() const { return limitationValue; }

    /*!
        @brief      [必須パラメタ] 権利の値をセットします。

        獲得したいサービスアイテムの権利種別に応じた値をセットします。
        権利種別が ServiceItemConstants::LIMITATION_TYPE_PERMANENT の場合はセットする必要はありません。
        ServiceItemConstants::LIMITATION_TYPE_ADDITIONAL_TIME の場合は有効期間 (秒数) をセットします。
        ServiceItemConstants::LIMITATION_TYPE_CONSUMPTION の場合は個数をセットします。

        @param[in]  limitationValue_ 獲得したいサービスアイテムの権利の値。
    */
    void SetLimitationValue(qUnsignedInt32 limitationValue_) { this->limitationValue = limitationValue_; }

    /*!
        @brief      アカウント単位で利用する権利用バイナリデータを取得します。

        @param[out]  rightBinary_ アカウント単位で利用する権利用バイナリデータ。
    */
    void GetRightBinary(qVector<qByte>& rightBinary_) const
    {
        // qBuffer -> qVector<qByte>への変換
        rightBinary_.assign(this->rightBinary.begin(), this->rightBinary.end());
    }

    /*!
        @brief      アカウント単位で利用する権利用バイナリデータをセットします。

        アカウント単位で利用する、権利で保護すべきバイナリデータを指定します。
        0 ～ ServiceItemConstants::RIGHT_BINARY_MAX_SIZE バイトまでセットできます。
        本パラメータの指定は必須ではありません。セットしない場合の初期値は空です。

        指定された権利用バイナリデータがそのままサーバに登録されます。
        空の場合に、削除されたり登録がスキップされることはありません。

        @param[in]  pRightBinary_ アカウント単位で利用する権利用バイナリデータのポインタ。 NULL を指定すると空になります。
        @param[in]  size_ アカウント単位で利用する権利用バイナリデータのデータサイズ (0 ～ ServiceItemConstants::RIGHT_BINARY_MAX_SIZE) 。
    */
    void SetRightBinary(const void* pRightBinary_, qUnsignedInt32 size_)
    {
        SYSTEMCHECK(size_ <= ServiceItemConstants::RIGHT_BINARY_MAX_SIZE && "Invalid pRightBinary. Please set ServiceItemConstants::RIGHT_BINARY_MAX_SIZE or less.");
        this->rightBinary.clear();
        if (pRightBinary_ != NULL)
        {
            const qByte* pb = static_cast<const qByte*>(pRightBinary_);
            this->rightBinary.push_back(pb, size_);
        }
    }

    /*!
        @brief      ログメッセージを取得します。

        @return     ログメッセージ。
    */
    const String& GetLogMessage() const { return logMessage; }

    /*!
        @brief      [必須パラメタ] ログメッセージをセットします。

        ユーザサポート向けの管理ツールに表示されるログメッセージをセットします。
        最大で ServiceItemConstants::SUPPORT_LOG_MESSAGE_MAX_SIZE バイトまでセットできます。

        @param[in]  logMessage_ ログメッセージ。
    */
    void SetLogMessage(const String& logMessage_)
    {
        SYSTEMCHECK(logMessage_.GetLength() > 0 && (sizeof(qChar) * logMessage_.GetLength()) <= ServiceItemConstants::SUPPORT_LOG_MESSAGE_MAX_SIZE && "Invalid logMessage. Please set between 1 and ServiceItemConstants::SUPPORT_LOG_MESSAGE_MAX_SIZE bytes.");
        this->logMessage = logMessage_;
    }

    /*!
        @brief      ユニーク ID を取得します。

        @return     ユニーク ID。
    */
    qUnsignedInt32 GetUniqueId() const { return uniqueId; }

    /*!
        @brief      [必須パラメタ] ユニーク ID をセットします。

        NEX サーバーから接続する eShop サーバーを決定するためのパラメーターです。
        通常はタイトルのユニーク ID を指定します。

        @param[in]  uniqueId_ ユニーク ID。
    */
    void SetUniqueId(qUnsignedInt32 uniqueId_) { this->uniqueId = uniqueId_; }

    /*!
        @brief      有効期限 ID を取得します。

        periodId は、消費型サービス利用券の有効期限を設定するための ID です。
        pid ごとに有効期限を変えたい場合に使用します。

        periodId を指定しない場合の初期値は 0 となり、
        有効期限は、日本のタイムゾーンの 4 ヶ月後の月末となります。

        periodId に紐づいた有効期限の設定値をサーバに登録する必要があるため、
        利用する場合はサーバー担当者にご相談ください。

        @return     有効期限 ID。
    */
    qUnsignedInt16 GetPeriodId() const { return periodId; }

    /*!
        @brief      有効期限 ID をセットします。

        periodId は、消費型サービス利用券の有効期限を設定するための ID です。
        pid ごとに有効期限を変えたい場合に使用します。

        periodId を指定しない場合の初期値は 0 となり、
        有効期限は、日本のタイムゾーンの 4 ヶ月後の月末となります。

        periodId に紐づいた有効期限の設定値をサーバに登録する必要があるため、
        利用する場合はサーバー担当者にご相談ください。

        @param[in]  periodId_ 有効期限 ID。
    */
    void SetPeriodId(qUnsignedInt16 periodId_) { this->periodId = periodId_; }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetReferenceIdForAcquisition(ServiceItemConstants::INVALID_REFERENCE_ID);
        SetReferenceIdForRightBinary(ServiceItemConstants::INVALID_REFERENCE_ID);
        this->useType = ServiceItemConstants::INVALID_USE_TYPE;
        this->limitationType = ServiceItemConstants::LIMITATION_TYPE_INVALID;
        SetLimitationValue(0);
        SetRightBinary(NULL, 0);
        this->logMessage = ServiceItemConstants::INVALID_LOG_MESSAGE;
        SetUniqueId(ServiceItemConstants::INVALID_UNIQUE_ID);
        platform = NEX_SERVICEITEM_GET_PLATFORM;
        SetPeriodId(0);
    }

    /*!
        @brief      ServiceItemAcquireServiceItemByAccountParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        {
            StringStream ss;
            ss << NEX_T("ServiceItemAcquireServiceItemByAccountParam: (")
                        << NEX_T(", useType=") << useType
                        << NEX_T(", limitationType=") << limitationType
                        << NEX_T(", limitationValue=") << limitationValue
                        << NEX_T(", logMessage=") << logMessage
                        << NEX_T(", uniqueId=") << uniqueId
                        << NEX_T(", periodId=") << periodId;
            QTRACE(uiTraceFlags, ss.CStr())
        }

        { // referenceIdは2つあって長いので別でログ出力する
            QTRACE(uiTraceFlags, NEX_T("    referenceIdForAcquisition=") << referenceIdForAcquisition);
            QTRACE(uiTraceFlags, NEX_T("    referenceIdForRightBinary=") << referenceIdForRightBinary);
        }

        {
            // rightBinaryは大きい可能性があるので別でログ出力する
            StringStream ss;
            ss << NEX_T("    rightBinary=<");
            if(!rightBinary.empty())
            {
                ss.BytesDump(&rightBinary[0], rightBinary.size());
            }
            ss << NEX_T(">)");
            QTRACE(uiTraceFlags, ss.CStr())
        }
    }

    /*!
        @internal
        @brief      RMCリクエスト前にパラメタをチェックする関数です。
    */
    qBool RequestPreCheck() const
    {
        qBool success = true;
        if (GetReferenceIdForAcquisition() == ServiceItemConstants::INVALID_REFERENCE_ID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid referenceIdForAcquisition. Please set referenceIdForAcquisition with ServiceItemAcquireServiceItemByAccountParam::SetReferenceIdForAcquisition()."));
            success = false;
        }
        if (GetReferenceIdForRightBinary() == ServiceItemConstants::INVALID_REFERENCE_ID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid referenceIdForRightBinary. Please set referenceIdForRightBinary with ServiceItemAcquireServiceItemByAccountParam::SetReferenceIdForRightBinary()."));
            success = false;
        }
        if (GetUseType() == ServiceItemConstants::INVALID_USE_TYPE)
        {
            QLOG(EventLog::Error, NEX_T("Invalid useType. Please set useType with ServiceItemAcquireServiceItemByAccountParam::SetUseType()."));
            success = false;
        }
        if (GetLimitationType() == ServiceItemConstants::LIMITATION_TYPE_INVALID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid limitationType. Please set limitationType with ServiceItemAcquireServiceItemByAccountParam::SetLimitationType()."));
            success = false;
        }
        if (GetLogMessage() == ServiceItemConstants::INVALID_LOG_MESSAGE)
        {
            QLOG(EventLog::Error, NEX_T("Invalid logMessage. Please set logMessage with ServiceItemAcquireServiceItemByAccountParam::SetLogMessage()."));
            success = false;
        }
        if(GetUniqueId() == ServiceItemConstants::INVALID_UNIQUE_ID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid uniqueId. Please set uniqueId with ServiceItemAcquireServiceItemByAccountParam::SetUniqueId()."));
            success = false;
        }
        // RightBinary は一応消せる仕様なのでチェックしない
        return success;
    }
};

END_NAMESPACE_NEX

