﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/ErrorCodeConverter.h>
#include <RendezVous/Services/Jugem/ServiceItem/src/Common/ServiceItemErrors.h>
#include "ServiceItemEShopResponseDDL.h"
#include "ServiceItemConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      eShop サーバーへアクセスする RMC で使用する、eShop からのレスポンスを表す基底クラスです。
*/
class ServiceItemEShopResponse: public DDLCLASS(ServiceItemEShopResponse)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemEShopResponse()
    {
        errorCode = ErrorCodeConverter::ConvertToNetworkErrorCode(QRESULT_ERROR(ServiceItem, Unknown));
        httpStatus = ServiceItemConstants::ESHOP_HTTP_STATUS_UNKNOWN;
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemEShopResponse() {}

    /*!
        @brief      eShop サーバーから返された HTTP ステータスコードを取得します。

        @return     eShop サーバーから返された HTTP ステータスコード。
    */
    virtual qUnsignedInt32 GetHttpStatus() const { return httpStatus; }

    /*!
        @brief      eShop サーバーから返されたネットワークエラーコードを取得します。

        @return     eShop サーバーから返されたネットワークエラーコード。
    */
    virtual qUnsignedInt32 GetErrorCode() const
    {
        // eShopのエラーコードが成功なのにレスポンスが空なのはおかしい(サーバー側のプログラミングミス)のでUnknownにする。基本的に発生しない。
        if (errorCode == ServiceItemConstants::ESHOP_ERRORCODE_SUCCESS && Get() == NULL)
        {
            QLOG(EventLog::Error, NEX_T("Internal error occurred in ServiceItem service."));
            return ErrorCodeConverter::ConvertToNetworkErrorCode(QRESULT_ERROR(ServiceItem, Unknown));
        }
        return errorCode;
    }

    /*!
        @brief      eShop サーバーから返された 問い合わせ ID (CorrelationId) を取得します。何か問題が生じた場合はこの ID を任天堂のサポート窓口までご連絡下さい。

        @return     eShop サーバーから返された 問い合わせ ID (CorrelationId)。
    */
    virtual const String& GetCorrelationId() const { return correlationId; }

    /*!
        @brief      eShop サーバーへのアクセスが成功したかどうかを取得します。

        @retval true  成功。Get() でレスポンス内容を取得できます。
        @retval false 失敗。
    */
    virtual qBool IsSuccess() const { return GetErrorCode() == ServiceItemConstants::ESHOP_ERRORCODE_SUCCESS; }

    /*!
        @brief      eShop サーバーへのアクセスが失敗したかどうかを取得します。

        @retval true  失敗。
        @retval false 成功。Get() でレスポンス内容を取得できます。
    */
    virtual qBool IsFailure() const { return !IsSuccess(); }

    virtual const RootObject* Get() const = 0;

    /*!
        @brief      ServiceItemEShopResponse オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        StringStream oStr;
        oStr << title << NEX_T("ServiceItemEShopResponse") << *this;
        QTRACE(uiTraceFlags, oStr.CStr());
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const ServiceItemEShopResponse& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const ServiceItemEShopResponse& rhs)
{
    oStr << NEX_T("(httpStatus=") << rhs.httpStatus
         << NEX_T(", errorCode=") << rhs.errorCode
         << NEX_T(", correlationId=") << rhs.correlationId
         << NEX_T(")");

    return oStr;
}

END_NAMESPACE_NEX

