﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemGetLawMessageParamDDL.h"
#include "ServiceItemConstants.h"
#include "ServiceItemConstantsInternal.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      法令に基づく表示のメッセージを取得するためのパラメータを表すクラスです。
*/
class ServiceItemGetLawMessageParam: public DDLCLASS(ServiceItemGetLawMessageParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemGetLawMessageParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemGetLawMessageParam() {}

    /*!
        @brief      言語を取得します。

        @return     言語。
    */
    const String& GetLanguage() const { return language; }

    /*!
        @brief      [必須パラメタ] 言語をセットします。

        言語をセットします。通貨表記のフォーマットを決定するために使用します。
        ISO 639 で規定されている 2 文字の言語コードを使用します。
        
        nn::cfg::CTR::GetLanguageCodeA2() 関数で取得した文字列を指定してください。
        

        @param[in]  language_ 言語。
    */
    void SetLanguage(const String& language_) { this->language = language_; }

    /*!
        @brief      ユニーク ID を取得します。

        @return     ユニーク ID。
    */
    qUnsignedInt32 GetUniqueId() const { return uniqueId; }

    /*!
        @brief      [必須パラメタ] ユニーク ID をセットします。

        NEX サーバーから接続する eShop サーバーを決定するためのパラメーターです。
        通常はタイトルのユニーク ID を指定します。

        @param[in]  uniqueId_ ユニーク ID。
    */
    void SetUniqueId(qUnsignedInt32 uniqueId_) { this->uniqueId = uniqueId_; }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetLanguage(ServiceItemConstants::INVALID_LANGUAGE);
        SetUniqueId(ServiceItemConstants::INVALID_UNIQUE_ID);
    }
    /*!
        @brief      ServiceItemGetLawMessageParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const ServiceItemGetLawMessageParam& rhs);

    /*!
        @internal
        @brief      RMCリクエスト前にパラメタをチェックする関数です。
    */
    qBool RequestPreCheck() const
    {
        qBool success = true;
        if(GetUniqueId() == ServiceItemConstants::INVALID_UNIQUE_ID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid uniqueId. Please set uniqueId with ServiceItemGetLawMessageParam::SetUniqueId()."));
            success = false;
        }
        if(GetLanguage() == ServiceItemConstants::INVALID_LANGUAGE)
        {
            QLOG(EventLog::Error, NEX_T("Invalid language. Please set language with ServiceItemGetLawMessageParam::SetLanguage()."));
            success = false;
        }
        return success;
    }
};

inline StringStream& operator<<(StringStream& oStr, const ServiceItemGetLawMessageParam& rhs)
{
    oStr << NEX_T("ServiceItemGetLawMessageParam: (")
               << NEX_T("language=") << rhs.language
               << NEX_T(", uniqueId=") << rhs.uniqueId
               << NEX_T(")");

    return oStr;
}

END_NAMESPACE_NEX
