﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _SERVICEITEM_LIST_REDEMPTION_SERVICE_ITEM_PARAM_H
#define _SERVICEITEM_LIST_REDEMPTION_SERVICE_ITEM_PARAM_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemListRedemptionServiceItemParamDDL.h"
#include "ServiceItemConstants.h"
#include "ServiceItemConstantsInternal.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      販売しているサービスアイテム情報のリストを取得するためのパラメータを表すクラスです。
*/
class ServiceItemListRedemptionServiceItemParam: public DDLCLASS(ServiceItemListRedemptionServiceItemParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemListRedemptionServiceItemParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemListRedemptionServiceItemParam() {}

    /*!
        @brief      引換番号を取得します。

        @return     引換番号。
    */
    const String& GetCardNumber() const { return cardNumber; }

    /*!
        @brief      [必須パラメタ] 引換番号をセットします。

            引換番号をセットします。クライアントから受け取った引換番号を使用してください。

        @param[in]  cardNumber 引換番号。
    */
    void SetCardNumber(const String& cardNumber_) { this->cardNumber = cardNumber_; }

    /*!
        @brief      言語を取得します。

        @return     言語。
    */
    const String& GetLanguage() const { return language; }

    /*!
        @brief      [必須パラメタ] 言語をセットします。

            言語をセットします。通貨表記のフォーマットを決定するために使用します。
            ISO 639 で規定されている 2 文字の言語コードを使用します。
            nn::cfg::CTR::GetLanguageCodeA2() 関数で取得した文字列を指定してください。

        @param[in]  language 言語。
    */
    void SetLanguage(const String& language_) { this->language = language_; }

    /*!
        @brief      ユニーク ID を取得します。

        @return     ユニーク ID。
    */
    qUnsignedInt32 GetUniqueId() const { return uniqueId; }

    /*!
        @brief      [必須パラメタ] ユニーク ID をセットします。

            NEX サーバーから接続する eShop サーバーを決定するためのパラメーターです。
            通常はタイトルのユニーク ID を指定します。

        @param[in]  uniqueId ユニーク ID。
    */
    void SetUniqueId(qUnsignedInt32 uniqueId_) { this->uniqueId = uniqueId_; }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetCardNumber(NEX_T(""));
        SetLanguage(ServiceItemConstants::INVALID_LANGUAGE);
        SetUniqueId(ServiceItemConstants::INVALID_UNIQUE_ID);
        platform = NEX_SERVICEITEM_GET_PLATFORM;
    }

    /*!
        @brief      ServiceItemListRedemptionServiceItemParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const ServiceItemListRedemptionServiceItemParam& rhs);

    /*!
        @internal
        @brief      RMCリクエスト前にパラメタをチェックする関数です。
    */
    qBool RequestPreCheck() const
    {
        qBool success = true;
        if (GetCardNumber().GetLength() == 0)
        {
            QLOG(EventLog::Error, NEX_T("Invalid card number. Please set card number with ServiceItemListRedemptionServiceItemParam.::SetCardNumber()."));
            success = false;
        }
        if (GetUniqueId() == ServiceItemConstants::INVALID_UNIQUE_ID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid uniqueId. Please set uniqueId with ServiceItemListRedemptionServiceItemParam.::SetUniqueId()."));
            success = false;
        }
        if (GetLanguage() == ServiceItemConstants::INVALID_LANGUAGE)
        {
            QLOG(EventLog::Error, NEX_T("Invalid language. Please set language with ServiceItemListRedemptionServiceItemParam::SetLanguage()."));
            success = false;
        }
        return success;
    }
};

inline StringStream& operator<<(StringStream& oStr, const ServiceItemListRedemptionServiceItemParam& rhs)
{
    oStr << NEX_T("ServiceItemListRedemptionServiceItemParam: (")
               << NEX_T("language=") << rhs.language
               << NEX_T(", cardNumber=") << rhs.cardNumber
               << NEX_T(", uniqueId=") << rhs.uniqueId
               << NEX_T(")");

    return oStr;
}

END_NAMESPACE_NEX

#endif
