﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemTransactionDDL.h"
#include "ServiceItemAmount.h"
#include "ServiceItemConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      一つのトランザクション情報を表すクラスです。
*/
class ServiceItemTransaction: public DDLCLASS(ServiceItemTransaction)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemTransaction()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemTransaction() {}

    /*!
        @brief      eShop サーバで発行されたトランザクション ID を取得します。

        @return     eShop サーバで発行されたトランザクション ID。
    */
    const String& GetTransactionId() const { return transactionId; }

    /*!
        @brief      NGS で発行されたトランザクション ID を取得します。

        @return     NGS で発行されたトランザクション ID。
    */
    const String& GetExtTransactionId() const { return extTransactionId; }

    /*!
        @brief      トランザクションが発生した日時を取得します。

        @return     トランザクションが発生した日時。
    */
    const DateTime& GetTime() const { return time; }

    /*!
        @brief      トランザクション種別を取得します。

        @return     トランザクション種別。
    */
    ServiceItemConstants::TransactionType GetTransactionType() const { return static_cast<ServiceItemConstants::TransactionType>(transactionType); }

    /*!
        @brief      トランザクションの詳細を取得します。

        @return     トランザクションの詳細。
    */
    const String& GetTransactionDescription() const { return transactionDescription; }

    /*!
        @brief      税込の総支払額情報を取得します。

        @return     税込の総支払額情報。
    */
    const ServiceItemAmount& GetTransactionAmount() const { return transactionAmount; }

    /*!
        @brief      アイテムコードを取得します。

        @return     アイテムコード。
    */
    const String& GetItemCode() const { return itemCode; }

    /*!
        @brief      リファレンス ID を取得します。

        @return     リファレンス ID。
    */
    const String& GetReferenceId() const { return referenceId; }

    /*!
        @brief      権利情報を取得します。

        @return     権利情報。
    */
    const ServiceItemLimitation& GetLimitation() const { return limitation; }

    /*!
        @brief      ServiceItemTransaction オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        StringStream ss;
        ss << title << NEX_T("ServiceItemTransaction(");
        ss << NEX_T(" transactionId=") << GetTransactionId();
        ss << NEX_T(" extTransactionId=") << GetExtTransactionId();
        ss << NEX_T(" time=") << GetTime();
        ss << NEX_T(" transactionType=") << GetTransactionType();
        ss << NEX_T(" transactionDescription=") << GetTransactionDescription();
        ss << NEX_T(" itemCode=") << GetItemCode();
        ss << NEX_T(" referenceId=") << GetReferenceId();
        ss << NEX_T(")");
        QTRACE(uiTraceFlags, ss.CStr());

        GetTransactionAmount().Trace(uiTraceFlags, title + NEX_T("    transactionAmount"));
        GetLimitation().Trace(uiTraceFlags, title + NEX_T("    limitation"));
    }
};

END_NAMESPACE_NEX

