﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "SubscriberContentDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      コンテンツに関するパラメータを表すクラスです。
*/
class SubscriberContent: public DDLCLASS(SubscriberContent)
{
public:
    /*!
        @brief      コンストラクタです。無効なコンテンツ生成します。サーバーからコンテンツを取得した後、格納された値を利用してください。
    */
    SubscriberContent()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~SubscriberContent() {}

    /*!
    @brief      コンテンツに付けられる一意の ID を取得します。

    @return コンテンツに付けられる一意の ID。
    */
    qUnsignedInt64 GetContentId() const { return contentId; }

    /*!
    @brief      コンテンツに含まれる任意の文字列情報を取得します。

    @return コンテンツに含まれる任意の文字列情報。
    */
    const String& GetMessage() const { return message; }

    /*!
    @brief      コンテンツに含まれる任意のバイナリ情報を取得します。

    @return     コンテンツに含まれる任意のバイナリ情報。
    */
    qVector<qByte> GetBinary() const
    {
        // qBuffer -> qVector<qByte>への変換
        qVector<qByte> vb;
        vb.assign(this->binary.begin(), this->binary.end());
        return vb;
    }

    /*!
    @brief      コンテンツに含まれる任意のバイナリ情報を取得します。

    こちらのオーバーロード関数の方が効率的です。

    @param[out]  binary_ コンテンツに含まれる任意のバイナリ情報。
    */
    void GetBinary(qVector<qByte>& binary_) const
    {
        // qBuffer -> qVector<qByte>への変換
        binary_.assign(this->binary.begin(), this->binary.end());
    }

    /*!
    @brief      コンテンツを投稿したユーザーのプリンシパル ID を取得します。

    @return コンテンツを投稿したユーザーのプリンシパル ID。
    */
    PrincipalID GetPosterPrincipalId() const { return pid; }

    /*!
    @brief      コンテンツが投稿されたトピックを全て取得します。

    このコンテンツを投稿した時に設定されたトピックを全て取得します。
    トピックは投稿時に一つ以上指定する必要があるため、サーバーから取得したコンテンツには必ず一つ以上のトピックが含まれます。
    トピック文字列から予約済みトピック番号に変換するには SubscriberUtil::TopicStringToReservedTopicNum() を利用してください。

    @return コンテンツが投稿されたトピック。
    */
    const qVector<String>& GetTopic() const { return topics; }

    /*!
    @brief      コンテンツが投稿されたトピックを一つ取得します。

    ショートカット関数です。このコンテンツを投稿した時に設定されたトピックを一つ取得します。
    トピックが複数セットされている可能性がある場合は SubscriberContent::GetTopic() を利用してください。
    トピックは投稿時に一つ以上指定する必要があるため、サーバーから取得したコンテンツには必ず一つ以上のトピックが含まれます。
    アプリケーションがトピックを一つだけ設定する仕様の場合はこのオーバーロード関数が便利です。
    複数のトピックがセットされていた場合、どのトピックが取得されるかは不定です。
    トピック文字列から予約済みトピック番号に変換するには SubscriberUtil::TopicStringToReservedTopicNum() を利用してください。

    @return コンテンツが投稿されたトピック。
    */
    String GetTopicSingle() const { if (topics.empty()) { return NEX_T(""); } return topics.at(0); }

    /*!
    @brief      コンテンツが投稿された日時を取得します。

    @return コンテンツが投稿された日時。
    */
    const DateTime& GetPostDateTime() const { return postTime; }


    /*!
        @brief      SubscriberContent オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const SubscriberContent& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const SubscriberContent& rhs)
{
    oStr << NEX_T("SubscriberContent: (") << dec
               << NEX_T("contentId=") << rhs.contentId
               << NEX_T(", message=") << rhs.message
               << NEX_T(", pid=") << rhs.pid
               << NEX_T(", postTime=") << rhs.postTime;

    oStr << NEX_T(", binary=");
    for (qBuffer::const_iterator it = rhs.binary.begin(); it != rhs.binary.end(); ++it)
    {
        oStr << hex << *it;
    }

    oStr << NEX_T(", topics=(");
    for (qVector<String>::const_iterator it = rhs.topics.begin(); it != rhs.topics.end(); ++it)
    {
        oStr << *it << NEX_T(", ");
    }
    oStr << NEX_T(")");

    oStr << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX

