﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "SubscriberPostContentParamDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      コンテンツの投稿に関するパラメータを表すクラスです。
*/
class SubscriberPostContentParam: public DDLCLASS(SubscriberPostContentParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    SubscriberPostContentParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~SubscriberPostContentParam() {}

    /*!
    @brief      投稿先のトピックを複数設定します。

    トピックには "r/<予約済みトピック番号>" のフォーマットを指定できます。
    予約済みトピック番号は 0 から NUM_RESERVED_TOPICS - 1 の整数を指定できます。
    予約済みトピック番号からトピック文字列を生成するには SubscriberUtil::ReservedTopicNumToTopicString() を利用してください。
    一つ以上のトピックを指定する必要があります。初期値は空です。

    @param[in] topics_ 投稿先のトピック。
    */
    void SetTopic(const qVector<String>& topics_) { this->topics = topics_; }

    /*!
    @brief      投稿先のトピックを一つ設定します。

    ショートカット関数です。一つだけトピックを設定する場合はこちらのオーバーロード関数が便利です。
    トピックには "r/<予約済みトピック番号>" のフォーマットを指定できます。
    予約済みトピック番号は 0 から NUM_RESERVED_TOPICS - 1 の整数を指定できます。
    予約済みトピック番号からトピック文字列を生成するには SubscriberUtil::ReservedTopicNumToTopicString() を利用してください。
    初期値は空です。

    @param[in] topic_ 投稿先のトピック。
    */
    void SetTopic(const String& topic_) { qVector<String> topics_; topics_.push_back(topic_); this->topics = topics_; }

    /*!
    @brief      投稿先のトピックを全て取得します。

    @return 投稿先のトピック。
    */
    const qVector<String>& GetTopic() const { return topics; }

    /*!
    @brief      投稿先のトピックを一つ取得します。

    @return 投稿先のトピック。
    */
    String GetTopicSingle() const { if (topics.empty()) { return NEX_T(""); } return topics.at(0); }

    /*!
    @brief      コンテンツに含む任意の文字列情報を設定します。

    初期値は空文字列です。

    @param[in] message_ コンテンツに含む任意の文字列情報。
    */
    void SetMessage(const String& message_) { this->message = message_; }

    /*!
    @brief      コンテンツに含む任意の文字列情報を取得します。

    @return コンテンツに含む任意の文字列情報。
    */
    const String& GetMessage() const { return message; }

    /*!
    @brief      コンテンツに含む任意のバイナリ情報を設定します。

    初期値は空です。

    @param[in] p コンテンツに含む任意のバイナリ情報へのポインタ。NULLを指定すると空になります。
    @param[in] size コンテンツに含む任意のバイナリ情報のサイズ。
    */
    void SetBinary(const void* p, size_t size)
    {
        this->binary.clear();
        if (p != NULL)
        {
            const qByte* pb = static_cast<const qByte*>(p);
            this->binary.push_back(pb, size);
        }
    }

    /*!
    @brief      コンテンツに含む任意のバイナリ情報を取得します。

    @return     コンテンツに含む任意のバイナリ情報。
    */
    qVector<qByte> GetBinary() const
    {
        // qBuffer -> qVector<qByte>への変換
        qVector<qByte> vb;
        vb.assign(this->binary.begin(), this->binary.end());
        return vb;
    }

    /*!
    @brief      コンテンツに含む任意のバイナリ情報を取得します。

    こちらのオーバーロード関数の方が効率的です。

    @param[out]  binary_ コンテンツに含む任意のバイナリ情報。
    */
    void GetBinary(qVector<qByte>& binary_) const
    {
        // qBuffer -> qVector<qByte>への変換
        binary_.assign(this->binary.begin(), this->binary.end());
    }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetTopic(qVector<String>());
        SetMessage(NEX_T(""));
        SetBinary(NULL, 0);
    }

    /*!
        @brief      SubscriberContent オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const SubscriberPostContentParam& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const SubscriberPostContentParam& rhs)
{
    oStr << NEX_T("SubscriberContent: (")
               << NEX_T("message=") << rhs.message;

    oStr << NEX_T(", binary=");
    for (qBuffer::const_iterator it = rhs.binary.begin(); it != rhs.binary.end(); ++it)
    {
        oStr << hex << *it;
    }

    oStr << NEX_T(", topics=(");
    for (qVector<String>::const_iterator it = rhs.topics.begin(); it != rhs.topics.end(); ++it)
    {
        oStr << *it << NEX_T(", ");
    }
    oStr << NEX_T(")");

    oStr << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX

