﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Services/Jugem/Utility/src/Common/UtilityDDF.h>
#include <RendezVous/Services/Jugem/Utility/src/Common/UtilityProtocolDDL.h>
#include <RendezVous/Services/Jugem/Utility/src/Common/UniqueId.h>

BEGIN_NAMESPACE_NEX

/*!
    @brief NEX サービス全体で利用することのある機能をまとめたクラスです。
*/
class UtilityClient : public ServiceClient
{
public:
    /*!
        @brief         コンストラクタです。
    */
    UtilityClient();

    /*!
        @brief         デストラクタです。
    */
    virtual ~UtilityClient();

    void UpdateProtocolsDefaultCredentials(Credentials* pCredentials);

    /*!
        @brief         NEX ユニーク ID を取得します。

        NEX ユニーク ID は、本関数を呼び出す度に新しい値が取得されます。@n
        取得したユニーク ID はセーブ領域に保存し、保存されていない場合にのみ本関数を呼び出すようにして下さい。

        @param[in,out] pContext   呼び出し情報。
        @param[out]    pNexUniqueId    NEX ユニーク ID。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool AcquireNexUniqueId(ProtocolCallContext* pContext, UniqueId* pNexUniqueId);

    /*!
        @brief         NEX ユニーク ID とパスワードを取得します。

        NEX ユニーク ID とパスワードは、本関数を呼び出す度に新しい値が取得されます。@n
        取得したユニーク ID とパスワードはセーブ領域に保存し、保存されていない場合にのみ本関数を呼び出すようにして下さい。

        @param[in,out] pContext   呼び出し情報。
        @param[out]    pUniqueIdInfo    NEX ユニーク ID 情報

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool AcquireNexUniqueIdWithPassword(ProtocolCallContext* pContext, UniqueIdInfo* pUniqueIdInfo);

    /*!
        @brief         NEX ユニーク ID が妥当かどうかを判定します。

        @param[in]     nexUniqueId    NEX ユニーク ID。

        @return        妥当な場合は true 、不正な場合は false が返ります。
    */
    static qBool IsValidNexUniqueId(UniqueId nexUniqueId);

    /*!
        @brief         1つの NEX ユニーク ID を自分のプリンシパル ID に関連付けます。

        uniqueIdInfo に指定した NEX ユニーク ID をプライマリ関連 NEX ユニーク ID として登録し、
        その他の紐付を解除します。 NEX ユニーク ID に INVALID_UNIQUEID を指定すると全ての紐付が解除されます。
        既に他の人に紐づいている NEX ユニーク ID を指定する場合、その人がオフライン状態である必要があります。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(RendezVous, DuplicateEntry) : 既に他の人に紐づいている NEX ユニーク ID を指定し、その人がオンライン状態である。
        @li QERROR(Rendezvous, InvalidPassword) : パスワードが不正。
        @li QERROR(Rendezvous, InvalidUniqueId) : NEX ユニーク ID が不正。

        @param[in,out] pContext   呼び出し情報。
        @param[in]     uniqueIdInfo   関連付ける NEX ユニーク ID、パスワード

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool AssociateNexUniqueIdWithMyPrincipalId(ProtocolCallContext* pContext, const UniqueIdInfo& uniqueIdInfo);

    /*!
        @brief         複数の NEX ユニーク ID を自分のプリンシパル ID に関連付けます。

        uniqueIdInfo に空のリストを指定すると全ての紐付が解除されます。
        uniqueIdInfo の先頭がプライマリ関連 NEX ユニーク ID となります。
        既に他の人に紐づいている NEX ユニーク ID を指定する場合、その人がオフライン状態である必要があります。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(RendezVous, DuplicateEntry) : 既に他の人に紐づいている NEX ユニーク ID を指定し、その人がオンライン状態である。
        @li QERROR(Rendezvous, InvalidPassword) : パスワードが不正。
        @li QERROR(Rendezvous, InvalidUniqueId) : NEX ユニーク ID が不正。

        @param[in,out] pContext   呼び出し情報。
        @param[in]     uniqueIdInfo   関連付ける NEX ユニーク ID、パスワード。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool AssociateNexUniqueIdWithMyPrincipalId(ProtocolCallContext* pContext, const qVector<UniqueIdInfo>& uniqueIdInfo);

    /*!
        @brief         自分のプリンシパル ID に関連付いたプライマリ関連 NEX ユニーク ID を取得します。

        @param[in,out] pContext   呼び出し情報。
        @param[out]    pUniqueIdInfo   自分のプリンシパル ID に関連付いたプライマリ関連 NEX ユニーク ID、パスワード。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetAssociatedNexUniqueIdWithMyPrincipalId(ProtocolCallContext* pContext, UniqueIdInfo* pUniqueIdInfo);

    /*!
        @brief         自分のプリンシパル ID に関連付いた NEX ユニーク ID を取得します。

        @param[in,out] pContext   呼び出し情報。
        @param[out]    pUniqueIdInfo   自分のプリンシパル ID に関連付いた NEX ユニーク ID、パスワード。先頭がプライマリ関連 NEX ユニーク ID。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetAssociatedNexUniqueIdWithMyPrincipalId(ProtocolCallContext* pContext, qVector<UniqueIdInfo>* pUniqueIdInfo);

    /*!
        @brief         アプリケーション固有の整数の設定値を取得します。

        サーバ側に設定されたアプリケーション固有の整数の設定値を取得します。

        @param[in,out]  pContext   呼び出し情報。
        @param[in]      integerSettingIndex  設定インデックス。
        @param[out]     pIntegerSettings  サーバ側に設定されたアプリケーション固有の整数の設定値。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetIntegerSettings(ProtocolCallContext* pContext, qUnsignedInt32 integerSettingIndex, qMap<qUnsignedInt16, qInt32>* pIntegerSettings);

    /*! @internal
        @brief         アプリケーション固有の文字列の設定値を取得します。

        サーバ側に設定されたアプリケーション固有の文字列の設定値を取得します。

        @param[in,out]  pContext   呼び出し情報。
        @param[in]      stringSettingIndex  設定インデックス。
        @param[out]     pStringSettings  サーバ側に設定されたアプリケーション固有の文字列の設定値。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetStringSettings(ProtocolCallContext* pContext, qUnsignedInt32 stringSettingIndex, qMap<qUnsignedInt16, String>* pStringSettings);

private:
    // プロトコルオブジェクト
    UtilityProtocolClient m_oProtocol;
};

END_NAMESPACE_NEX

