﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

BEGIN_NAMESPACE_NEX

/**
   \ingroup group_rdv_services_gathering_client
   \brief   This class contains the enumeration of participation policies specific to Gathering.
*/
#ifdef SWIG
class BuiltInParticipationPolicies
{
public:
#else
namespace BuiltInParticipationPolicies
{
#endif
    /**
       \brief   Enumerates the types of participation policies that may be set for a Gathering.

        A Participation policy is associated to a Gathering and defines the Principal s
        that may participate in that Gathering. Participation polices do not place restrictions as to
        when a Principal may join the Gathering. If you wish to implement restrictions, /e.g./ Principal s
        cannot join after the gathering has started, then you can either implement your own participation policy
        or implement Hook s on the MatchMakingClient::Participate method.
        Only one Participation policy may be assigned to any given Gathering, which is set via
        Gathering::SetParticipationPolicy or one of the related methods. You cannot modify the built-in
        participation policies however, you may add an almost unlimited number of user-defined participation policies.
        In order to access these types you must include the RendezVous.h file.
    */
    enum ParticipationPolicies
    {

        /**
        This is the default Participation policy.

        Any Principal may participate in the Gathering. Although invitations can still be sent for this
        gathering, it is not necessary to receive an invitation to participate in the Gathering. Gathering s
        with this Participation policy are only removed by the system when the Gathering owner logs out of the
        lobby.
        */
        OpenParticipation = 1,
        /**
        \brief To participate in the Gathering a Principal must belong to a specific group, which is defined
        via BuiltInGroups. Any Principal that is a member of the specified Group will automatically
        participate in the Gathering. As participation is automatic, invitations cannot be sent for this
        Gathering, nor can a Principal choose to join the Gathering (via MatchMakingClient::Participate)
        or cancel its participation (via MatchMakingClient::CancelParticipation).
        */
        GroupParticipation = 2,
        /**
        To participate in the Gathering a Principal must have received an invitation to do so. A gathering's
        owner sends invitations via the MatchMakingClient::Invite method and then the invitee may accept
        or decline the invitation. When this Participation policy is used, Principals confirm their participation
        by accepting the invitation (MatchMakingClient::AcceptInvitation) rather than calling
        MatchMakingClient::Participate.
        */
        OnInvitation = 3,
        /**
        To participate in the Gathering a Principal must use a valid password. The password is passed
        as a parameter of the MatchMakingClient::Participate method.
        */
        PasswordProtected = 4,
        /**
        To participate in the Gathering a Principal must have the specified Capability, which is defined
        via Capabilities.
        */
        CapabilityBased = 5,
        /// @cond FOR_TEST_ONLY
        NullPolicy = 6,
        /// @endcond
        /**
        Obsolete. Use OpenSessionParticipation instead.
        */
        OngoingParticipation = 7,
        /**
        It is the same as the OpenParticipation policy, except that the GatheringStates of the Gathering is
        automatically set to HasStarted|HasFinished when the last Active participant leaves the Gathering session.
        Therefore, even if no participant sets the GatheringStates to end the gathering, such as may occur if all
        participants leave the gathering before the end, then Rendez-Vous will still correctly terminate the Gathering
        and process the appropriate statistics where necessary. Principal s who disconnect from a Gathering with this
        Participation policy are flagged with StatisticFlags::Disconnected.
        */
        OpenSessionParticipation = 8,

        /*
        The ID of the first user-defined Participation policy, which by default is equal to 100.
        */
        FirstUserPolicy = 100
    };
};
END_NAMESPACE_NEX

