﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/MatchMaking/src/Common/InvitationDDL.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringStatsDDL.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringID.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringDDL.h>
#include <RendezVous/Services/MatchMaking/src/Common/ParticipantDetailsDDL.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <RendezVous/Services/MatchMaking/src/Common/BuiltInParticipationPolicies.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringStates.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringFlags.h>

BEGIN_NAMESPACE_NEX

class MatchMakingProtocolClient;
class ProtocolCallContext;

//Group=Matchmaking Service

#ifdef FOR_DOC_PURPOSE

/*
Description: A Gathering is any session, such as game, competition, tournament, statistics and rankings, chat, or forum sessions,
that a Principal may join.
Remarks: A gathering is simply a session that a player can participate in, whether actively ( /e.g./ a game session) or passively
( /e.g./ tournament rankings). A gathering is a very generic, and easily customized, beast and can be defined as any sort of session
including game, competition, tournament, statistics and rankings, chat, or forum sessions. There are virtually no restrictions on
how many different types of gatherings you can define, nor how many gatherings exist on the lobby at any given time.

Gatherings are registered to the Matchmaking Service so that clients can locate various lobby gatherings via the MatchMakingClient. In addition,
services can query gatherings to retrieve pertinent information, such as the current state or statistics of the gathering.

Individual gatherings may operate in isolation or be associated to or part (children) of a larger gathering. These relationships between
gatherings do not imply that related gatherings have to be of the same type. For example, the individual matches in a multi-round
tournament are child gatherings of the tournament gathering, and the tournament statistics gathering is implemented as a separate
gathering that is associated to the tournament gathering.

Via this class a Principal may create a Gathering, set the minimum and maximum number of gathering participants, set the
Participation policy (the default is BuiltInParticipationPolicies::OpenParticipation), send invitations to given players to
join the Gathering, and then launch the session when ready.

In order to facilitate competition creation, Rendez-Vous implements the Gathering hierarchy shown in the figure below.
Any competition objects that you implement ( /e.g./ tournament, ranking, statistics) must belong to Competition
class. Any custom gatherings ( /e.g./ matches, rounds, chats, forums) that you define must inherit from the Gathering class.
The GameSession, Tournament, and Ranking classes are empty classes defined for convenience when creating competitions that
you may or may not choose to use in your implementation.
Details of how to implement your own user-defined gatherings is given in the Custom Gathering Classes section.

{image:GatheringHierarchy}

See Also: GatheringFlags, GatheringStates, GatheringID, ParticipationEvents, BuiltInParticipationPolicies,
Custom Gathering Classes
*/

/*!
 @brief ０人以上のクライアントが参加するセッションをあらわすクラスです。

 ギャザリングは「マッチメイクの部屋」というよりも広い意味を持つ概念ですが、
 NEXライブラリの マッチメイク機能では Gathering クラスを継承した MatchmakeSession クラスのみ
 利用可能です。
 */
class Gathering: public RootObject
{

#else
class Gathering: public DDLCLASS(Gathering)
{
#endif
public:
    Gathering();

    //{secret}
    void Reset();

    /*
    Remarks: If a Gathering is non-persistent then its destructor will automatically be called by
    Rendez-Vous when either the Gathering owner logs out of the lobby or the GatheringStates is set
    to /HasFinished/. On the other hand, if a Gathering is persistent then it will only be
    deleted when its destructor is called.
    */
    virtual ~Gathering();

    //Group=Type Definitions

    //Group=Methods

    /*
    Description: Returns the ID of this Gathering.
    Returns: The GatheringID of the Gathering, which is assigned by the database after the
    Gathering has been registered.
    See Also: GatheringID
    */

    /*!
     @brief ギャザリングの ID を取得します。

     @return ギャザリングの ID
     */
    GatheringID GetID() const
    {
        return m_idMyself;
    }

    /*!
     @brief ギャザリングの ID を設定します。

     MatchmakeExtensionClient::AutoMatchmake() や MatchmakeExtensionClient::CreateMatchmakeSession() などでギャザリングを
     登録する際は必ず INVALID_GATHERINGID を指定してください
     また、 MatchmakeExtensionClient::UpdateMatchmakeSession() を使用する際、
     本関数を用いて更新するマッチメイクセッションのギャザリングIDを指定してください。

     @param[in]    id    ギャザリングの ID

     @return 設定された後のギャザリング ID
     */
    GatheringID SetID(GatheringID id)
    {
        return m_idMyself = id;
    }

#ifdef FOR_DOC_PURPOSE

    /*
    Description: Returns the type of Gathering that this Gathering is.
    Returns: The type of the Gathering as a string.
    Remarks: Valid Gathering types are the same as the names of the Gathering classes
    (classes that inherit from the root Gathering class) defined in the DDL file.
    See Also: Custom Gatherings
    */

    /*!
     @brief    自分のクラス名を返します

     @return    自分のクラス名
     */
    virtual String GetGatheringType() const;

    /*
    Description: Returns whether or not this Gathering belongs to the specified class.
    Parameters:
    strClass: The Gathering class as a string.
    Returns: true if this Gathering is an instance of the specified class, false if not
    or an error occurred.
    Remarks: Gathering classes (classes that inherit from the root Gathering class) are
    defined in the DDL file.
    See Also: IsAKindOf, Custom Gatherings
    */

    /*!
     @brief    引数で与えられたクラス名のオブジェクトであるかを判定します。

     @remark    true を返す場合、そのクラスへキャストすることが可能です。

     @param[in]    strClass    クラス名

     @retval    true    引数で与えられたクラス名のクラスオブジェクトである
     @retval    false   引数で与えられたクラス名のクラスオブジェクトではない
     */
    virtual qBool IsA(const String& strClass) const;

    /*
    Description: Returns whether or not this Gathering belongs to or inherits from the
    specified class.
    Parameters:
    strClass: The Gathering class as a string.
    Returns: true if this Gathering is an instance of or an instance of a child class of the
    specified class, false if not or an error occurred.
    Remarks: Gathering classes (classes that inherit from the root Gathering class) are
    defined in the DDL file.
    See Also: IsA, Custom Gatherings
    */

    /*!
     @brief    引数で与えられたクラス名のクラスか、それを継承したオブジェクトかを判定します。

     @remark    true を返す場合、そのクラスへキャストすることが可能です。

     @param[in]    strClass    クラス名

     @retval    true    引数で与えられたクラス名のクラスまたはそれを継承したオブジェクトである
     @retval    false   引数で与えられたクラス名のクラスまたはそれを継承したオブジェクトではない
     */
    virtual qBool IsAKindOf(const String& strClass) const;

#endif


    /*
    {group:Participation Policy Related Methods}

    Description: Sets the Participation policy of this Gathering.
    Parameters:
    uiParticipationPolicy: The Participation policy of this Gathering, which is defined via the
    BuiltInParticipationPolicies types.
    uiArgument: The policy argument of this Gathering. The default value is zero. This is an optional
    Participation policy specific argument whose exact significance depends on the given Participation policy.
    Remarks: Only one Participation policy may be assigned to a given Gathering. By default the
    Participation policy of a Gathering is /OpenParticipation/. If you wish to use one
    of the BuiltInParticipationPolicies, rather then setting the policy via this method you can use one
    of the "Use<X>ParticipationPolicy" methods.

    The policy argument that should be passed as the uiArgument parameter for each of the
    BuiltInParticipationPolicies is as follows:

    CapabilityBased - The Capabilities of the policy
    GroupPartcipation - The BuiltInGroups of the policy
    OnInvitation - No policy argument
    OpenPartcipation - No policy argument
    PasswordProtected - The password of the policy

    See Also: BuiltInParticipationPolicies, UseGroupParticipationPolicy, UseOnInvitationParticipationPolicy,
    UseOpenParticipationPolicy, UsePasswordProtectedParticipation, UseCapabilityBasedParticipation,
    GetParticipationPolicy
    */
    void SetParticipationPolicy(qUnsignedInt32 uiParticipationPolicy, qUnsignedInt32 uiArgument=0);

    /*
    {group:Participation Policy Related Methods}

    Description: Sets this Gathering to use the built-in /GroupParticipation/ Participation policy.
    Parameters:
    uiGroup: The Group to which a Principal must belong to participate in this Gathering. The valid
    groups are defined via the BuiltInGroups types.
    Remarks: With the /GroupParticipation/ Participation policy, any Principal that belongs to the specified
    uiGroup automatically participates in the Gathering.
    See Also: BuiltInParticipationPolicies, BuiltInGroups
    */
    void UseGroupParticipationPolicy(qUnsignedInt32 uiGroup)
    {
        SetParticipationPolicy(BuiltInParticipationPolicies::GroupParticipation, uiGroup);
    }

    /*
    {group:Participation Policy Related Methods}

    Description: Sets this Gathering to use the built-in /OnInvitation/ Participation policy.
    Remarks: With the /OnInvitation/ Participation policy, a Principal must receive (and accept) an invitation before
    being permitted to participate in the Gathering.
    See Also: BuiltInParticipationPolicies, MatchMakingClient::Invite
    */
    void UseOnInvitationParticipationPolicy()
    {
        SetParticipationPolicy(BuiltInParticipationPolicies::OnInvitation);
    }

    /*
    {group:Participation Policy Related Methods}

    Description: Sets this Gathering to use the built-in /OpenParticipation/ Participation policy.
    Remarks: With the /OpenParticipation/ Participation policy, any Principal may participate in the Gathering whether
    or not an inviation has been received to do so.
    See Also: BuiltInParticipationPolicies
    */
    void UseOpenParticipationPolicy()
    {
        SetParticipationPolicy(BuiltInParticipationPolicies::OpenParticipation);
    }

    /*
    {group:Participation Policy Related Methods}

    Description: Sets this Gathering to use the built-in /PasswordProtected/ Participation policy.
    Parameters:
    szPassword: The valid password for this Gathering.
    Remarks: With the /PasswordProtected/ Participation policy, a Principal must use a valid password to participate
    in the Gathering. The password is passed as the /strMessage/ parameter of the MatchMakingClient::Participate method.
    A Gathering may only have a single valid password that cannot be modified once this method is called.
    See Also: BuiltInParticipationPolicies
    */
    void UsePasswordProtectedParticipation(const qChar8* szPassword);

    /*
    {group:Participation Policy Related Methods}

    Description: Sets this Gathering to use the built-in /CapabilityBased/ Participation policy.
    Parameters:
    uiCapability: The required Capability for this Gathering. Capabilities are defined via the Capabilities types.
    Remarks: With the /CapabilityBased/ Participation policy, a Principal must have the specified Capability in
    order to participate in the Gathering.
    See Also: BuiltInParticipationPolicies, Capabilities
    */
    void UseCapabilityBasedParticipation(qUnsignedInt32 uiCapability)
    {
        SetParticipationPolicy(BuiltInParticipationPolicies::CapabilityBased, uiCapability);
    }

    /*
    {group:Participation Policy Related Methods}

    Description: Returns the Participation policy of this Gathering.
    Returns: The Participation policy of this Gathering.
    Remarks: The Participation policy of a Gathering is set via SetParticipationPolicy.
    See Also: SetParticipationPolicy
    */
    qUnsignedInt32 GetParticipationPolicy() const;

    /*
    {group:Participation Policy Related Methods}

    Description: Returns the policy argument, if any, of this Gathering.
    Returns: The policy argument, if any, of this Gathering.
    Remarks: The policy argument of a Participation policy is an optional parameter that is passed
    when SetParticipationPolicy is called.
    See Also: SetParticipationPolicy
    */
    
    /*! @internal
    @brief 参加ポリシー引数を取得します。

    @return 参加ポリシー引数
    */
    qUnsignedInt32 GetPolicyArgument() const;

    /*
    Description: Assigns a description to this Gathering.
    Parameters:
    strDescription: The description of the Gathering as a string.
    Remarks: There is no limit to the length of a Gathering description.
    See Also: GetDescription
    */

    /*!
     @brief 説明文字列をセットします。

     ギャザリングの説明文字列をセットします。アプリケーションが任意の値を設定できます（最大 MATCHMAKE_STRING_MAX_LENGTH 文字）。

     @param[in]    strDescription    説明文字列
     */
    void SetDescription(const String& strDescription);

    /*
    Description: Returns a description of this Gathering.
    Returns: The description of the Gathering as a string.
    Remarks: The Gathering description is set via SetDescription.
    See Also: SetDescription
    */

    /*!
     @brief 説明文字列を取得します。

     ギャザリングの説明文字列を取得します。

     @return    ギャザリングの説明文字列
     */
    const String& GetDescription() const
    {
        return m_strDescription;
    }

    // {secret}
    // Used by the MatchMaking client, to initialize the value.
    // See also bug 2533 - it is used by some clients.
    GatheringID* GetPtrToID()
    {
        return &m_idMyself;
    }


    // Status.
    // Open,Closed, etc.
    // Scheduler (now, future)
    // Restrictions - nb players, etc.
    // Description

    // And then, game specifics (in derived class).

    // Three options to manage specifics:
    // 1. Put all specifics in a blob.
    // virtual PackageSpecifics(BitStream&) //
    // 2. implement a specific protocol, that complements the
    // default one.
    // 3. Build a new protocol.

    // ... + API to update participants.
    // Join()
    // Leave()

    // How do we define invited participants?
    // Invite(PID);

    // When deleted, does the gathering stays active?  When is is
    // purged from the DB?

    // What is the 'owner' of a gathering?  About FT?

    // qList<PrincipalID>& GetParticipantList() {return m_lstMessages;}

    /*
    Description: Returns the PrincipalID of the Principal that is this Gathering's owner.
    Returns: The PrincipalID of the Principal of this Gathering's owner.
    Remarks: A Gathering may only have one owner at any given time. The owner is the Principal
    that controls the gathering and is the only Principal that may invite players to join the
    gathering or delete the gathering. A gathering's owner is initially the Principal that
    created the gathering. If this Principal leaves the session then the gathering will either
    be deleted or, if permitted, ownership will be transferred to another Principal. The
    Gathering owner should not be confused with the Gathering host, which is the Principal of
    the Process on which the gathering session runs (is hosted), although the owner and host may
    be the same Principal.
    See Also: PrincipalID, GetHostPID, GetOwnerCID
    */

    /*!
     @brief ギャザリングのオーナーの PrincipalID を取得します。

     ギャザリングのオーナーの PrincipalID を取得します。オーナーはギャザリングを変更できる唯一のユーザーで、
     マッチメイクの募集状態の変更や、ギャザリングの削除などを行うことができます。

     初期状態では、ギャザリングの登録ユーザーがオーナーとなりますが、@ref MatchMakingClient::MigrateGatheringOwnership() を
     実行するか、GatheringFlag に MigrateOwner をセットした状態でオーナーが退出することでオーナーを変更することができます。

     @return    ギャザリングのオーナーの PrincipalID

     @see MatchMakingClient::MigrateGatheringOwnership()
     */
    PrincipalID GetOwnerPID() const
    {
        return m_pidOwner;
    }

    /*
    Description: Returns the PrincipalID of the Principal that is this Gathering's host.
    Returns: The PrincipalID of the Principal of this Gathering's host.
    Remarks: A Gathering may only have one host, which is the Principal of the Process on which the
    gathering runs (is hosted), at any given time. If this Principal leaves the session then the gathering
    will either be deleted or, if permitted, hosting duties will be transferred to another Principal.
    The Gathering host should not be confused with the Gathering owner, which is the Principal
    that controls the gathering and which may invite players to join, although the owner and
    host may be the same Principal.
    See Also: PrincipalID, GetOwnerPID
    */

    /*!
     @brief ギャザリングのホストの プリンシパルID を取得します。

     ホストは P2P セッションの管理者で、
     ギャザリングの状態を変更したりすることはできません。他のユーザーが P2P セッションに参加する場合、
     ホストの URL をサーバーから取得する必要があります。

     @return    ギャザリングのホストの プリンシパルID

     @see MatchMakingClient::UpdateSessionHost()
     */
    PrincipalID GetHostPID() const
    {
        return m_pidHost;
    }

    /*
    Description: Returns the minimum number of participants for this Gathering.
    Returns: The minimum number of participants for this Gathering.
    Remarks: The minimum number of participants is set via SetMinParticipants.
    See Also: GetMaxParticipants
    */

    /*!
     @brief ギャザリングの最小参加人数を取得します

     @return    ギャザリングの最小参加人数
     */
    qUnsignedInt16 GetMinParticipants() const
    {
        return m_uiMinParticipants;
    }

    /*
    Description: Returns the maximum number of participants.
    Returns: The maximum number of participants.
    Remarks: The maximum number of participants is set via SetMaxParticipants.
    See Also: GetMinParticipants
    */

    /*!
     @brief ギャザリングの最大参加人数を取得します

     @return    ギャザリングの最大参加人数
     */
    qUnsignedInt16 GetMaxParticipants() const
    {
        return m_uiMaxParticipants;
    }

    //TODO: is this restricted to gathering owner?

    /*
    Description: Sets the minimum number of participants.
    Parameters:
    uiMinParticipants: The minimum number of participants.
    By default no restrictions are placed on this value.
    Remarks: The uiMinParticipants parameter must be a positive integer and less than
    the set maximum number of participants.
    See Also: GetMinParticipants, SetMaxParticipants
    */

    /*!
     @brief ギャザリングの最小参加人数をセットします

     ギャザリングの最小参加人数をセットします。現在、本関数でセットした値はマッチメイクに影響を与えません。

     @param[in]    uiMinParticipants    ギャザリングの最小参加人数
     */
    void SetMinParticipants(qUnsignedInt16 uiMinParticipants)
    {
        m_uiMinParticipants=uiMinParticipants;
    }

    //TODO: is this restricted to gathering owner?

    /*
    Description: Sets the maximum number of participants.
    Parameters:
    uiMaxParticipants: The maximum number of participants.
    The default value is zero which means that the maximum number of participants is unlimited.
    Remarks: The uiMaxParticipants parameter must be a positive integer and greater than the
    set minimum number of participants.
    See Also: GetMaxParticipants, SetMinParticipants
    */

    /*!
     @brief ギャザリングの最大参加人数をセットします

     この人数を越えてギャザリングに参加することはできません。
     0 を指定した場合は最大参加人数が無制限になります。
     MatchmakeExtensionClient::UpdateMatchmakeSession() や MatchmakeExtensionClient::UpdatePersistentGathering() で最大参加人数を更新する場合、
     現在の参加人数未満の値を指定する事もできます。この場合は既存の参加者が退出させられることはありませんが、
     現在の参加人数が最大参加人数を下回るまでそのギャザリングに参加することはできません。

     @param[in]    uiMaxParticipants    ギャザリングの最大参加人数
     */
    void SetMaxParticipants(qUnsignedInt16 uiMaxParticipants)
    {
        m_uiMaxParticipants=uiMaxParticipants;
    }

    /*
    Description: Returns the local gathering state (See GatheringStates enumeration).
    Returns: The local gathering state.
    Remarks: This method only returns the state on the local station.
    If you wish to know the state stored on the server then use the method MatchMakingClient::GetState.
    See Also: SetState, GatheringStates
    */
    qUnsignedInt32 GetState() const
    {
        return m_uiState;
    }

    /*
    Description: Specifies the local gathering state (See GatheringStates enumeration).
    Parameters:
    uiState: The local state to set. The valid states are given by GatheringStates.
    Remarks: This method only sets the gathering state on the local station.
    If you wish to update the state on the server then use the method MatchMakingClient::SetState.
    See Also: GetState, GatheringStates
    */
    void SetState(qUnsignedInt32 uiState)
    {
        m_uiState = uiState;
    }

    // {secret}
    qUnsignedInt32 GetFlags() const
    {
        return m_uiFlags;
    }

    /*
    Description: Returns whether or not the specified flag is set.
    Parameters:
    uiFlag: The GatheringFlags for which it will be determined whether or not it is set.
    Returns: True if the specified flag is set for this Gathering.
    Remarks: The flags that may be set for a Gathering are defined via the GatheringFlags types.
    See Also: SetFlag, ClearFlag, GatheringFlags
    */

    /*!
     @brief     ギャザリングフラグがセットされているかを確認します

     指定されたギャザリングフラグがセットされているかを確認します。

     @param[in]    uiFlag    セットされているかどうか確認するギャザリングフラグ
     @return    フラグがセットされているかどうか

     @see GatheringFlags
     */
    qBool FlagIsSet(qUnsignedInt32 uiFlag) const
    {
        return (m_uiFlags & uiFlag) == uiFlag;
    }

    /*
    Description: Sets a given GatheringFlags.
    Parameters:
    uiFlag: A combination of GatheringFlags enumation values.
    Remarks:
    See Also: ClearFlag, FlagIsSet, GatheringFlags
    */

    /*!
     @brief     GatheringFlags で列挙されているギャザリングフラグをセットします

     指定された ギャザリングフラグをセットします。

     @param[in]    uiFlag    セットするギャザリングフラグ
     @see GatheringFlags
     */
    void SetFlag(qUnsignedInt32 uiFlag)
    {
        m_uiFlags = m_uiFlags | uiFlag;
    }

    /*
    Description: Clears a given GatheringFlags for this Gathering.
    Parameters:
    uiFlag: The GatheringFlags to be cleared for this Gathering.
    Remarks:
    See Also: SetFlag, FlagIsSet, GatheringFlags
    */

    /*!
     @brief     ギャザリングフラグをクリアします

     指定されたギャザリングフラグをクリアします。

     @param[in]    uiFlag    クリアするギャザリングフラグ
     @see GatheringFlags
     */
    void ClearFlag(qUnsignedInt32 uiFlag)
    {
        m_uiFlags = m_uiFlags & (~uiFlag);
    }
    
    /*!
     @internal
     @brief     ギャザリングフラグを全てクリアします

     指定されたギャザリングフラグを全てクリアします。
     PersistentGathering にはクリアしてはいけないフラグがあるので本関数は非公開。

     @see GatheringFlags
     */
    void ClearFlagAll()
    {
        m_uiFlags = GatheringFlags::Nothing;
    }

    /*
    Description: Traces the content of this Gathering object.
    Remarks: This method traces the Gathering type, ID, and description, Gathering owner, Gathering host,
    the Participation policy of the gathering, and any GatheringFlags currently set.
    */
    virtual void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS) const;

    // 最大人数が無制限の場合の空き人数
    static const qInt32 INFINITE_VACANT_PARTICIPANTS = 0x7fffffff;
};

END_NAMESPACE_NEX

