﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

BEGIN_NAMESPACE_NEX

//group=Matchmaking Service

/*
Description: Enumerates the various flags that may be set for a Gathering.
Remarks: GatheringFlags control the persistence of a Gathering, or in other words
when the Gathering will be destroyed. By default a gathering will be destroyed when its
GatheringStates is set to /HasFinished/ or the Gathering owner logs out of the lobby.
If you want a Gathering to persist after either of these two events occurs, you can
set the /Persistent/ flag for a given gathering via Gathering::SetFlag. You may set
multiple flags for a given gathering however, the /CompetitionEvent/ flag should only be
set by Rendez-Vous and not by the user.
*/
#ifdef SWIG
class GatheringFlags
{
public:
#else  // Not SWIG
/*!
 @brief GatheringFlags の 名前空間です。
*/
namespace GatheringFlags
{
#endif // End of SWIG

    // group=Type Definitions

    //Description: Enumerates the various flags that may be set for a Gathering.
    //Remarks: GatheringFlags control the persistence of a Gathering, or in other words
    //when the Gathering will be destroyed. By default a gathering will be destroyed when its
    //GatheringStates is set to /HasFinished/ or the Gathering owner logs out of the lobby.
    //If you want a Gathering to persist after either of these two events occurs, you can
    //set the /Persistent/ flag for a given gathering via Gathering::SetFlag. You may set
    //multiple flags for a given gathering however, the /CompetitionEvent/ flag should only be
    //set by Rendez-Vous and not by the user.

    /*!
     @brief    ゲームサーバーに登録されたギャザリングの振る舞いを定義する列挙子です。

     ゲームサーバーに登録されたギャザリングの振る舞いを定義する列挙子です。
     Gathering::SetFlag() でギャザリングに指定できます。ビットフラグとなっているため、
     複数のフラグを指定することが可能です。
     */
    enum GatheringFlags
    {
        /*!
         @brief    フラグを指定しません。
         */
        Nothing = 0x00000000,
        /*
        The Gathering is persistent and therefore, it will not automatically be destroyed when
        either the Gathering owner logs out of or disconnects from the lobby or the GatheringStates
        is set to /HasFinished/. A gathering with this flag set will only be destroyed when the Gathering
        is unregistered from the Matchmaking service, via MatchMakingClient::UnregisterGathering.
        */

        /*!
         @brief    使用されません
         */
        Persistent = 0x00000001,
        /*
        The Gathering is a Competition event, which is a sub-gathering of a Competition
        gathering. This flag should not be set by the user as it is automatically set when
        a competition event is registered to a competition via CompetitionClient::RegisterCompetitionEvent.
        A Gathering with this flag set will not be destroyed until after its associated Competition has
        finished all necessary processing regarding the Gathering. For example, a game session gathering
        will only be destroyed once its associated Competition has processed any relevant statistics.
        */

        /*!
         @brief    使用されません
         */
        CompetitionEvent = 0x00000002,
        /*
        All sub-gatherings of the Gathering for which this flag is set are persistent. In other words,
        all sub-gatherings of the gathering will not be destroyed until the parent gathering is destroyed.
        */

        /*!
         @brief    使用されません
         */
        PersistentEvents = 0x00000004,
        /*
        The gathering is private and will therefore, not be found by some FindBy functions. Note that this
        flag is currently not implemented.
        */

        /*!
         @brief    使用されません
         */
        Private = 0x00000008,
        /*
        When the current Gathering owner of a non-persistent gathering disconnects from the lobby, ownership of
        the Gathering will be transferred to one of the other participants, with preference given to participants
        who are active and online. If no participant is found, /i.e./ all participants have left the session, then the
        gathering will be destroyed. If you wish to control to which Principal a Gathering is migrated, then you may
        use the MatchMakingClient::MigrateGatheringOwnership method.
        */

        /*!
         @brief    このフラグが指定されている場合、ギャザリングのオーナーがマッチメイクセッションから退出した際に、ゲームサーバーは
         ギャザリングの参加者の中から新しいオーナーを選びます。オーナーになれる参加者がいない場合はギャザリングが
         削除されます。任意のタイミングでオーナー変更をする場合は MatchmakeExtensionClient::MigrateGatheringOwnership() を使用してください。
         */
        MigrateOwner = 0x00000010,
#ifndef FOR_DOC_PURPOSE
        /*! @internal
          @brief    使用されません
         */
        _UseBlockList = 0x00000020,
#endif
        /*!
          @brief    このフラグが指定されている場合、ゲームサーバーからのログアウトやゲームサーバーとの切断によって永続ギャザリングから退出します。

                    このフラグは永続ギャザリングのみに効果があります。
         */
        NoPersistentParticipation = 0x00000040,
        /*!
          @brief    このフラグが指定されている場合、参加者が0人になっても永続ギャザリングは削除されません。
                    また、サーバー側で登録されるクライアントアプリケーション以外の参加者の登録を抑止します。
                    例えば、公式永続ギャザリングでは全てのクライアントアプリケーションが退出しても、
                    永続ギャザリングが削除されないようにサーバー側で用意された固定の Principal ID の参加者が登録されます。
                    本フラグが指定されている場合は、このようなクライアントアプリケーション以外の参加者の登録は不要なため抑止されます。

                    本フラグが指定されている場合、ギャザリングの削除は明示的に @ref MatchMakingClient::UnregisterGathering() を呼ぶ必要があります。

                    このフラグは永続ギャザリングに対してのみ指定可能です。
         */
        // 永続ギャザリング以外に指定しても動作するが保証しない。
        AllowNoParticipant = 0x00000080,
#ifndef FOR_DOC_PURPOSE
        /*! @internal
          @brief    このフラグは、サーバーで登録する永続ギャザリングに対して設定します。
                    サーバーで登録された永続ギャザリングかどうかの判定に使用されます。
                    クライアントアプリケーション側では設定しないでください。

                    このフラグは永続ギャザリングに対してのみ指定可能です。
         */
        RegistrationByServer = 0x00000100,
#endif
        /*!
         * @brief このフラグは、マッチメイクセッションのオーナーを他の参加者から MatchMakingClient::UpdateSessionHost(), MatchMakingClient::UpdateGatheringOwnership() により
         * 奪うことができるかどうかを設定するフラグです。
         *
         * NEX 3.7以降、デフォルトONです。
         *
         *  @see MatchMakingClient::UpdateSessionHost(), MatchMakingClient::UpdateGatheringOwnership()
         */
        ChangeOwnerByOtherHost = 0x00000200,

        /*!
         * @brief マッチメイクセッションにおいて ParticipationEvents::Participate、ParticipationEvents::Disconnect、ParticipationEvents::EndParticipation
                  通知を自分自身を含む参加者全員に対して行います。

                  NotifyParticipationEventsToAllParticipantsReproducibly が同時に指定された場合は NotifyParticipationEventsToAllParticipantsReproducibly が優先されます。
         */
        NotifyParticipationEventsToAllParticipants = 0x00000400,

        /*!
         * @brief マッチメイクセッションにおいて ParticipationEvents::Participate、ParticipationEvents::Disconnect、ParticipationEvents::EndParticipation
                  通知を自分自身を含む参加者全員に対して行います。
                  加えて、ユーザーがマッチメイクセッションに参加した時にそのユーザーに対してすでに参加しているユーザーの ParticipationEvents::Participate が通知されます。

                  NotifyParticipationEventsToAllParticipants が同時に指定された場合は NotifyParticipationEventsToAllParticipantsReproducibly が優先されます。
         */
        NotifyParticipationEventsToAllParticipantsReproducibly = 0x00000800
    };

#ifdef SWIG
};
#else // Not SWIG
}
#endif // End of SWIG

END_NAMESPACE_NEX

