﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/MatchMaking/src/Common/Gathering.h>
#include <RendezVous/Core/src/Common/AnyObjectHolder.h>
#include <RendezVous/Services/MatchMaking/src/Common/AnyGatheringAdapter.h>

BEGIN_NAMESPACE_NEX

#ifndef FOR_DOC_PURPOSE

typedef AnyObjectHolder<Gathering, String> GatheringHolder;

#endif

/*
    \ingroup group_rdv_services_gathering_client
    \brief      This template class is used to customize a GatheringHolder for a particular type of Gathering.

    Use this template class to customize a GatheringHolder for a particular type of Gathering. The
    custom Gathering type is designated by T.
*/
template <class T=Gathering>
class CustomGatheringHolder: public AnyObjectHolder<Gathering, String>
{
public:

    /*
        \param[in]  pGathering   A pointer to the type of Gathering that this CustomGatheringHolder
        will refer to.
    */
    CustomGatheringHolder(T* pGathering=NULL): AnyObjectHolder<Gathering, String>(pGathering) {}

    /*
        \brief      Returns a pointer to the Gathering object that is referenced by this CustomGatheringHolder.
        \returns    A pointer to the Gathering object that is referenced by this CustomGatheringHolder.
        If no Gathering object is referenced, a NULL pointer is returned.
        \see        Release
    */
    T* Get() const
    {
        return static_cast<T*>(AnyObjectHolder<Gathering, String>::Get());
    }
    T& operator* () const
    {
        return *Get();
    }
    T* operator-> () const
    {
        return Get();
    }

    /*
        \brief  Releases ownership of the Gathering object that this CustomGatheringHolder references.
        \see    Get
    */
    T* Release()
    {
        return static_cast<T*>(AnyObjectHolder<Gathering, String>::Release());
    }
};

#ifdef FOR_DOC_PURPOSE

//Group=Matchmaking Service

/*
Description: The root class used to safely hold a Gathering object while operations
are performed on it.
Remarks: GatheringHolder s are used to safely hold Gathering objects while operations are performed
on the objects and to prevent memory leaks due to objects not being deallocated correctly. This
class operates in a similar manner to the Standard C++ Library auto_ptr class, which is a smart pointer
that provides strict ownership semantics. In other words, a GatheringHolder is like a Gathering pointer
however, it also destroys the Gathering object to which it points if the GatheringHolder object is itself
destroyed or if another pointer is assigned to it.

A GatheringHolder is used to "create" a Gathering on the local client process where the required flags
and parameters for the gathering can be set. For the gathering to then be actually created on the
server, the gathering has to be registered to the Matchmaking Service via MatchMakingClient::RegisterGathering.
After this method has completed, the gathering will then be available to other clients.

Customization of GatheringHolder s should be performed via the CustomGatheringHolder class.
See Also: CustomGatheringHolder, MatchMakingClient::RegisterGathering
*/

/*!
 @brief    ギャザリングのホルダークラスです

 @ref Gathering クラス及び Gathering の派生クラス( @ref MatchmakeSession ) をゲームサーバーに送受信するために使用されます。
 内部で @ref Gathering クラスオブジェクトのポインタを保持しており、代入やデストラクタ呼び出し時に保持しているオブジェクトのデストラクタも呼び出します。
 */
class GatheringHolder: public RootObject
{
public:

    //group=Methods

    /*
    Parameters:
    p: A pointer that determines to where the GatheringHolder will refer.

    Remarks: If you use the first (default) constructor the GatheringHolder will not refer to
    anything, whereas  if you use the second constructor the GatheringHolder will refer to where
    the pointer refers.
    */

    /*!
     @brief    コンストラクタです。

     初期値として NULL ポインタがセットされます
     */
    GatheringHolder() : m_ptr(NULL)
    {
    }

    /*!
     @brief    コンストラクタです。

     引数で与えられた Gathering クラスオブジェクトをセットします。

     @param[in]    p    Gathering クラスオブジェクトのポインタ
     */
    explicit GatheringHolder (Gathering* p) : m_ptr(p)
    {
    }

    // Remarks: When a GatheringHolder is deleted it releases the object, if any, to which it refers.
    /*!
     @brief    デストラクタです。

     保持している Gathering クラスオブジェクトをデストラクトします。
     GatheringHolder のデストラクト後も Data クラスオブジェクトを使用する場合は、
     @ref Release() を呼び出してオブジェクトの所有権を変更してください。
     */
    ~GatheringHolder()
    {
        if (m_ptr!=NULL)
        {
            delete m_ptr;
        }
    }

    /*!
      @brief   コピーコンストラクタです。

      other が保持しているGathering クラスオブジェクトの所有権が this に移ります。
      @param[in]    other    GatheringHolder クラスオブジェクト
     */
    GatheringHolder(GatheringHolder const &other);

    /*!
      @brief   代入演算子です。

      other が保持している Gathering クラスオブジェクトの所有権が this に移ります。

      this が既に Gathering クラスオブジェクトを保持していた場合は削除されます。
      @param[in]    other    GatheringHolder クラスオブジェクト
      @return     自分自身への参照
     */
    GatheringHolder& operator= (GatheringHolder const &other);

    // Remarks: Assigns a new pointer to the GatheringHolder. This method automatically deletes
    // and deallocates the previously held object, if any.
    /*!
     @brief    代入演算子です。

     保持している Gathering クラスオブジェクトをデストラクトした上で、引数で与えられた Gathering クラスオブジェクトをセットします。

     @param[in]    p    Gathering クラスオブジェクトのポインタ
     @return     自分自身への参照
     */
    GatheringHolder& operator= (Gathering* p)
    {
        if (m_ptr!=NULL)
        {
            delete m_ptr;
        }
        m_ptr = p;
        return *this;
    }

    /*!
      @brief    保持しているオブジェクトの参照を返します

      何も保持していない、または所有権を失った GatheringHolder クラスオブジェクトに対して呼ばないでください。

      @return     保持しているオブジェクトの参照
      */
    Gathering& operator* () const
    {
        return *m_ptr;
    }

    /*
      @brief    保持しているオブジェクトの参照を返します

      何も保持していない、または所有権を失った GatheringHolder クラスオブジェクトに対して呼ばないでください。

      @return     保持しているオブジェクトの参照
      */
    Gathering& GetRef() const
    {
        return *m_ptr;
    }


    /*!
      @brief    保持しているオブジェクトのポインタを返します

      @return     保持しているオブジェクトのポインタ。セットされていない場合 NULL を返します。
      */
    Gathering* operator-> () const
    {
        return m_ptr;
    }

    /*
    Description: Returns a pointer to the Gathering object that is referenced by this GatheringHolder.
    Returns: A pointer to the Gathering object that is referenced by this GatheringHolder.
    If no Gathering object is referenced, a NULL pointer is returned.
    See Also: Release, ExchangeWith
    */

    /*!
     @brief    保持しているオブジェクトのポインタを返します。

     保持している Gathering クラスオブジェクトを返します。セットされていない場合 NULL を返します。

     @return     保持しているオブジェクトのポインタ
     */
    Gathering* Get() const
    {
        return m_ptr;
    }

    /*
    Description: Releases ownership of the Gathering object that this GatheringHolder references.
    See Also: Get, ExchangeWith
    */

    /*!
     @brief    保持している Gathering クラスオブジェクトの所有権を解放します。

     保持している Gathering クラスオブジェクトの所有権を解放し、そのポインタを返します。

     @return     保持していた Gathering クラスオブジェクトのポインタ
     */
    Gathering* Release()
    {
        Gathering* ret=m_ptr;
        m_ptr = 0;
        return ret;
    }

    /*
    Description: Exchanges ownership of a Gathering object with another GatheringHolder.
    Parameters:
    h: The GatheringHolder with which Gathering object ownership will be exchanged.
    p: A pointer to the Gathering object that will be exchanged with the current Gathering
    object

    Remarks: This method enables the held Gathering object to be replaced without destroying the
    object. The object that will replace the current object can be specified via either its
    current GatheringHolder owner or via a pointer to the said Gathering object.
    See Also: Get, Release
    */
    void ExchangeWith (GatheringHolder<T>& h)
    {
        std::swap(m_ptr,h.m_ptr);
    }
    void ExchangeWith (Gathering*& p)
    {
        std::swap(m_ptr,p);
    }
};
#endif

END_NAMESPACE_NEX

