﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <RendezVous/Core/src/Common/DataHolder.h>
#include <RendezVous/Core/src/Common/ResultRange.h>
#include <RendezVous/Services/Messaging/src/Client/MessageDeliveryServer.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Services/Messaging/src/Common/MessagingProtocolDDL.h>
#include <RendezVous/Core/src/Common/Constants/ServiceID.h>
#include <RendezVous/Core/src/Common/Constants/IdentifierTypes.h>
#include <RendezVous/Services/Messaging/src/Common/MessagingFlags.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringID.h>
#include <RendezVous/Services/Messaging/src/Client/MessagingNotificationHandler.h>


BEGIN_NAMESPACE_NEX

class StreamManager;
//group=Messaging Service

/*
Description: A unique positive integer to identify a message entry
in the database.
*/
typedef qUnsignedInt32  MessageID;

/*
Description: Default message lifetime, used by the server to control
message expiration date/time.
*/
#define DEFAULT_MESSAGE_LIFE_TIME   (3600 * 24 * 365 * 10)

/*
Description: This is the data holder implementation for all classes
deriving from UserMessage.
*/

/*!
 @brief    UserMessage, TextMessage, BinaryMessage 用の DataHolder です。
 */
typedef CustomDataHolder<UserMessage> UserMessageDataHolder;

/*!
@brief    サーバーに送信できる文字列の最大長。サーバーと通信する非同期関数にこれより大きい文字列を
          引数として与えると、非同期処理の結果が QERROR(Core, InvalidArgument) となります。

*/
static const qUnsignedInt32 MESSAGING_STRING_MAX_LENGTH = 256;
/*!
@brief    サーバに送信できるバイナリデータの最大サイズ。サーバと通信する非同期関数にこれより大きいバイナリデータを
          引数として与えると、非同期処理の結果が QERROR(Core, InvalidArgument) となります。

*/
static const qUnsignedInt32 MESSAGING_BINARY_MAX_SIZE = 512;

/*
Description: This service enables players to send UserMessages to each other and participate in forums.
Remarks: Using this service, players can send messages to each other and post messages in forums. Because all
messages sent using this service pass through the server, messages can be sent to any Principal, regardless
of whether that Principal is online or not.

See Also: UserMessage, TextMessage, BinaryMessage
*/

/*!
 @brief    インスタントメッセージ機能を利用するためのクラスです
 */
class MessagingClient: public ServiceClient
{
public:

#ifdef FOR_DOC_PURPOSE

    // Default Constructor
    /*!
     @brief コンストラクタです
     */
    MessagingClient();

#else
    MessagingClient(qBool bClientUse = true);

#endif

    // Destructor
    virtual ~MessagingClient();

    /*
    Description: Delivers a message to a given principal.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    oUserMessage: DataHolder to object deriving from the UserMessage DDL
    class (see example).
    pidRecipient: Principal ID of the intended message recipient.
    uiFlags: Bitwise combination of flags defined in MessagingFlags.h.
    uiMsgLifeTime: Delay (in seconds) from message reception to the time that it is
    deleted from database. If a message recipient is offline longer than the uiMsgLifeTime
    value, the message will never be delivered.

    Note: This function returns false if oUserMessage refers to an object that is not
    derived from the UserMessage DDL class.

    Example:
    UserMessage derived DDL class SomeLobby.ddl

    class TextMessage: UserMessage {
        string m_strTextBody;
    } concrete;

    Note: If MessagingFlags::DirectDelivery is set on uiFlags, the target principal's
    MessagingClient may invoke CheckLiveMessages to capture the message ASAP.
    */
    // {secret}
    qBool DeliverMessageToPrincipal(ProtocolCallContext *pContext,
                                    const UserMessageDataHolder&    oUserMessage,
                                    PrincipalID         pidRecipient,
                                    qUnsignedInt32      uiFlags,
                                    qUnsignedInt32      uiMsgLifeTime
                                   )
    {
        return DeliverMessage(pContext, MessageRecipient(IdentifierTypes::PIDType, pidRecipient, INVALID_GATHERINGID), oUserMessage, uiFlags, uiMsgLifeTime);
    }


    /*
    Description: Delivers a message to a given gathering, such as a chat room or forum. Gatherings
    are treated as Principals.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    oUserMessage: DataHolder to object deriving from the UserMessage DDL
    class (see example).
    gidRecipient: Principal ID of the gathering that is the intended message recipient.
    uiFlags: Bitwise combination of flags defined in MessagingFlags.h.
    uiMsgLifeTime: Delay (in seconds) from message reception to the time that it is
    deleted from database. If a message recipient is offline longer than the uiMsgLifeTime
    value, the message will never be delivered.

    Note: This function returns false if oUserMessage refers to an object that is not
    derived from the UserMessage DDL class.

    Example:
    UserMessage derived DDL class SomeLobby.ddl

    class TextMessage: UserMessage {
        string m_strTextBody;
    } concrete;

    Note: If MessagingFlags::DirectDelivery is set on uiFlags, the active participants
    of the gathering may invoke MessagingClient::CheckLiveMessages to capture the message
    ASAP.
    */
    // {secret}
    qBool DeliverMessageToGathering(ProtocolCallContext *pContext,
                                    const UserMessageDataHolder&    oUserMessage,
                                    GatheringID         gidRecipient,
                                    qUnsignedInt32      uiFlags,
                                    qUnsignedInt32      uiMsgLifeTime
                                   )
    {
        return DeliverMessage(pContext, MessageRecipient(IdentifierTypes::GIDType, INVALID_PRINCIPALID, gidRecipient), oUserMessage, uiFlags, uiMsgLifeTime);
    }

    // {secret}
    qBool DeliverMessage(ProtocolCallContext *pContext,
                         const MessageRecipient& oRecipient,
                         const UserMessageDataHolder&   oUserMessage,
                         qUnsignedInt32     uiFlags,
                         qUnsignedInt32     uiMsgLifeTime
                        );

    /*
    Description: Sends a message to a target principal. The
    message is simply registered in the database (assigned a MessageID and stored).
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    pid: The ID of the target principal.
    oUserMessage: UserMessageDataHolder to object deriving from the UserMessage DDL
    class.
    uiMsgLifeTime: Delay (in seconds) from message reception to the time that it is
    deleted from the database. If a message recipient is offline longer than the uiMsgLifeTime
    value, the message will never be delivered.
    Returns: true if the call was successfully initiated, false if not or an error occurred.
    Note: UserMessageDataHolder is a smart pointer; thus, it must be initialized by assigning
    to it an object allocated on heap.
    Example:
    UserMessageDataHolder hMsg = qNew UserMessage;
    hMsg->SetSubject(strMessage);
    Example:
    TextMessage* pTextMessage = qNew TextMessage;
    pTextMessage->SetSubject(strSubject);
    pTextMessage->SetTextBody(strMessage);
    UserMessageDataHolder hMsg = pTextMessage;
    Remarks: The call succeeds (pContext->GetState()==CallContext::CallSuccess) even though the target recipient does not exist.
    */
    qBool Send(ProtocolCallContext *pContext,
               PrincipalID pid,
               const UserMessageDataHolder& oUserMessage,
               qUnsignedInt32 uiMsgLifeTime=DEFAULT_MESSAGE_LIFE_TIME)
    {
        return DeliverMessage(pContext, MessageRecipient(IdentifierTypes::PIDType, pid, INVALID_GATHERINGID), oUserMessage, MessagingFlags::PersistentMessaging, uiMsgLifeTime);
    }

    /*
    Description: Sends an instant message to a target principal. The
    message is delivered directly to the principal without being
    stored in the database.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    pid: The ID of the target principal.
    oUserMessage: UserMessageDataHolder to object deriving from the UserMessage DDL
    class.
    Returns: true if the call was successfully initiated, false if not or an error occurred.
    Note: UserMessageDataHolder is a smart pointer; thus, it must be initialized by assigning
    to it an object allocated on heap.
    Example:
    UserMessageDataHolder hMsg = qNew UserMessage;
    hMsg->SetSubject(strMessage);
    Example:
    TextMessage* pTextMessage = qNew TextMessage;
    pTextMessage->SetSubject(strSubject);
    pTextMessage->SetTextBody(strMessage);
    UserMessageDataHolder hMsg = pTextMessage;
    Remarks: The call succeeds (pContext->GetState()==CallContext::CallSuccess) even though the target recipient does not exist.
    See also: MessagingNotificationHandler
    */

    /*!
    @brief    インスタントメッセージを送信します

    Reliable 通信を使用して、指定したユーザーにゲームサーバー経由でインスタントメッセージを送信します。
    非同期処理は、インスタントメッセージがゲームサーバーまで到達した段階で成功を返します。
    そのため、メッセージが送信先に到達したかは非同期処理の結果からは分かりません。
    ゲームサーバーまで到達したメッセージは、送信先がゲームサーバーと切断されていなければ到達性が保証されます。

    @param[in]     pContext    呼び出し情報。
    @param[in]     pid   送信先の プリンシパル ID 
    @param[in]    oUserMessage メッセージデータ

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。

    @see MessagingNotificationHandler
     */
    qBool SendInstant(ProtocolCallContext *pContext,
                      PrincipalID pid,
                      const UserMessageDataHolder&  oUserMessage)
    {
        return DeliverMessage(pContext, MessageRecipient(IdentifierTypes::PIDType, pid, INVALID_GATHERINGID), oUserMessage, MessagingFlags::InstantMessaging, 0);
    }

    /*
    Description: Sends a message to a target gathering. The
    message is simply registered in the database (assigned a MessageID and stored).
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    gid: The ID of the target gathering.
    oUserMessage: UserMessageDataHolder to object deriving from the UserMessage DDL
    class.
    uiMsgLifeTime: Delay (in seconds) from message reception to the time that it is
    deleted from the database. If a message recipient is offline longer than the uiMsgLifeTime
    value, the message will never be delivered.
    Returns: true if the call was successfully initiated, false if not or an error occurred.
    Remarks: This method can be used to post a message to a
    gathering used as a forum, for example.
    Note: UserMessageDataHolder is a smart pointer; thus, it must be initialized by assigning
    to it an object allocated on heap.
    Example:
    UserMessageDataHolder hMsg = qNew UserMessage;
    hMsg->SetSubject(strMessage);
    Example:
    TextMessage* pTextMessage = qNew TextMessage;
    pTextMessage->SetSubject(strSubject);
    pTextMessage->SetTextBody(strMessage);
    UserMessageDataHolder hMsg = pTextMessage;

    The call succeeds (pContext->GetState()==CallContext::CallSuccess) even though the target recipient does not exist.

    See also: MessagingNotificationHandler
    */
    qBool Post(ProtocolCallContext *pContext,
               GatheringID gid,
               const UserMessageDataHolder& oUserMessage,
               qUnsignedInt32 uiMsgLifeTime=DEFAULT_MESSAGE_LIFE_TIME)
    {
        return DeliverMessage(pContext, MessageRecipient(IdentifierTypes::GIDType, INVALID_PRINCIPALID, gid), oUserMessage, MessagingFlags::PersistentMessaging, uiMsgLifeTime);
    }

    /*
    Description: Sends an instant message to a target gathering. The
    message is delivered directly to active participants of the gathering, without being
    stored in the database.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    gid: The ID of the target gathering.
    oUserMessage: UserMessageDataHolder to object deriving from the UserMessage DDL
    class.
    Returns: true if the call was successfully initiated, false if not or an error occurred.
    Remarks: This method can be used to send a message instantly to
    all active participants of a gathering used as a chat room, for
    example.
    Note: UserMessageDataHolder is a smart pointer; thus, it must be initialized by assigning
    to it an object allocated on heap.
    Example:
    UserMessageDataHolder hMsg = qNew UserMessage;
    hMsg->SetSubject(strMessage);
    Example:
    TextMessage* pTextMessage = qNew TextMessage;
    pTextMessage->SetSubject(strSubject);
    pTextMessage->SetTextBody(strMessage);
    UserMessageDataHolder hMsg = pTextMessage;
    The call succeeds (pContext->GetState()==CallContext::CallSuccess) even though the target recipient does not exist.
    */

    /*!
    @brief    ギャザリングの参加者全員にインスタントメッセージを送信します

    Reliable 通信を使用して、指定したギャザリングの参加者全員にゲームサーバー経由でインスタントメッセージを送信します。
    非同期処理は、インスタントメッセージがゲームサーバーまで到達した段階で成功を返します。
    そのため、メッセージが送信先に到達したかは非同期処理の結果からは分かりません。
    ゲームサーバーまで到達したメッセージは、送信先がゲームサーバーと切断されていなければ到達性が保証されます。

    永続ギャザリングに対して本関数を使用する場合は、永続ギャザリングの最大参加人数を PERSISTENT_GATHERING_CHAT_PARTICIPANTS_MAX 以下に設定する必要があります。

    @param[in]     pContext    呼び出し情報。
    @param[in]     gid   送信先のギャザリングの ID
    @param[in]    oUserMessage メッセージデータ

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。

    @see MessagingNotificationHandler
     */
    qBool SendChat(ProtocolCallContext *pContext,
                   GatheringID gid,
                   const UserMessageDataHolder& oUserMessage)
    {
        return DeliverMessage(pContext, MessageRecipient(IdentifierTypes::GIDType, INVALID_PRINCIPALID, gid), oUserMessage, MessagingFlags::InstantMessaging, 0);
    }


    // {secret}
    //n2361 not used
    //qBool DeliverMessageFromServer(ProtocolCallContext *pContext,
    //const DataHolder&   oUserMessage,
    //DataHolder*           pModifiedMessage,
    //qList<PrincipalID>* lstParticipants);

    // {secret}
    qBool GetNbMessages( ProtocolCallContext *pContext, qUnsignedInt32* puiNbMessages );


    // {secret}
    qBool GetNbGatheringMessages( ProtocolCallContext *pContext, GatheringID idGathering, qUnsignedInt32* puiNbMessages );

    // {secret}
    qBool RetrieveAllMessagesWithinRange( ProtocolCallContext*  pContext,
                                          const ResultRange&      oRange,
                                          qList<DataHolder>*      pListMessages );


    /*
    Description: Fetches the messages from the database, whose IDs
    correspond to items included into lstIDs.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    lstIDs: A list of message IDs.
    pListMessages: Returned list of messages.
    bLeaveOnServer: If true, the messages are not removed from
    database when read. Default: false.
    Returns: true if the call was successfully initiated, false if not or an error occurred.
    Remarks: Call GetMessageHeaders, first, to read the list of all
    available messages whose target is the current principal. Then,
    you can invoke UserMessage::GetID for each item you want to
    retrieve, and build listIDs accordingly.
    */
    qBool RetrieveMessages(ProtocolCallContext*     pContext,
                           const qList<MessageID>&  lstIDs,
                           qList<DataHolder>*       pListMessages,
                           qBool bLeaveOnServer = false);


    /*
    Description: Fetches the messages from database, for a registered
    gathering, whose IDs correspond to items included into lstIDs.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    gid: The ID of a registered gathering.
    lstIDs: A list of message IDs.
    pListMessages: Returned list of messages.
    Returns: true if the call was successfully initiated, false if not or an error occurred.
    Remarks: Call GetGatheringMessageHeaders, first, to read the list of all
    available messages whose target is the given gathering. Then,
    you can invoke UserMessage::GetID for each item you want to
    retrieve, and build listIDs accordingly.
    */
    qBool RetrieveGatheringMessages(ProtocolCallContext*    pContext,
                                    GatheringID             gid,
                                    const qList<MessageID>& lstIDs,
                                    qList<DataHolder>*      pListMessages);



    // {secret}
    qBool RetrieveAllGatheringMessagesWithinRange( ProtocolCallContext* pContext,
            GatheringID            gid,
            const ResultRange&     oRange,
            qList<DataHolder>*     pListMessages );


    /*
    Description: Reads the message headers of the current
    principal, using the range parameters.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    oRange: ResultRange object.
    pListMessages: Returned list of message headers.
    Returns: true if the call was successfully initiated, false if not or an error occurred.
    */
    qBool GetMessageHeaders(ProtocolCallContext*    pContext,
                            const ResultRange&      oRange,
                            qList<UserMessage>*     pListMessages);

    /*
    Description: Reads the message headers of the given gathering,
    using the range parameters.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    gid: ID of a registered gathering.
    oRange: ResultRange object.
    pListMessages: Returned list of message headers.
    Returns: true if the call was successfully initiated, false if not or an error occurred.
    */
    qBool GetGatheringMessageHeaders(ProtocolCallContext*   pContext,
                                     GatheringID            gid,
                                     const ResultRange&     oRange,
                                     qList<UserMessage>*    pListMessages);


    // {secret}
    qBool DeleteAllMessages( ProtocolCallContext *pContext, qUnsignedInt32* puiNbMessages );


    /*
    Description: Deletes a subset of the current Principal's messages.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    lstMessagesToDelete: List of message IDs to be deleted from the database.
    Returns: true if the call was successfully initiated, false if not or an error occurred.
    Remarks: Call GetMessageHeaders, first, to read the list of all
    available messages whose target is the current principal. Then,
    you can invoke UserMessage::GetID for each item you want to
    delete, and build listIDs accordingly.
    */
    qBool DeleteMessages( ProtocolCallContext *pContext, const qList<MessageID>& lstMessagesToDelete );

    // {secret}
    qBool DeleteAllGatheringMessages( ProtocolCallContext *pContext, GatheringID gid, qUnsignedInt32* puiNbMessages );

    /*
    Description: Deletes a subset of messages from a given registered gathering.
    Parameters:
    pContext: A pointer to the ProtocolCallContext for this call.
    gid: The ID of a registered gathering.
    lstMessagesToDelete: List of message IDs to be deleted from the database.
    Returns: true if the call was successfully initiated, false if not or an error occurred.
    Remarks: Call GetGatheringMessageHeaders, first, to read the list of all
    available messages whose target is the given gathering. Then,
    you can invoke UserMessage::GetID for each item you want to
    delete, and build listIDs accordingly.
    */
    qBool DeleteGatheringMessages( ProtocolCallContext *pContext, GatheringID gid, const qList<MessageID>& lstMessagesToDelete );


    /*
    Description: Attaches to the object a notification handler defined by the user
    to capture instant messages.
    Parameters:
    pHandler: Pointer to an object of a class deriving from
    MessagingNotificationHandler.
    */

    /*!
     @brief    メッセージハンドラを登録します

     メッセージハンドラを登録します。ハンドラーが登録されると通知メッセージを受信するたびに
     メッセージがハンドラーへ送られ、@ref MessagingNotificationHandler::DeliverInstantMessage() が呼び出されます。

     @param[in]    pHandler    メッセージハンドラ
     */
    void RegisterMessagingNotificationHandler(MessagingNotificationHandler* pHandler);

    /*
    Description: Removes the current user-defined notification
    handler to capture instant messages.
    */

    /*!
     @brief    登録されているメッセージハンドラを削除します。
     */
    void UnregisterMessagingNotificationHandler();

protected:

    virtual void UpdateProtocolsDefaultCredentials(Credentials* pCredentials);

private:
    static qUnsignedInt16 s_uiMessagingClientCount;
    static CriticalSection s_csRefCount;

    // {secret}
    // I believe that this can be removed now.
    MessagingProtocolClient         m_oMessagingProtocolClient;
    // {secret}
    MessageDeliveryServer           m_oMessageDeliveryServer;
    // {secret}
    MessageDeliveryProtocolClient   m_oMessageDeliveryClient;
    // {secret}
    qBool                           m_bClientUse;
};

END_NAMESPACE_NEX

