﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2018, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2018 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <RendezVous/Services/Messaging/src/Common/BinaryMessageDDL.h>

BEGIN_NAMESPACE_NEX


#ifdef FOR_DOC_PURPOSE
/*
Description: Construct a message containing essentially a binary body.
See Also: Messaging Service
*/

/*!
 @brief    バイナリデータを格納するクラスです

 インスタントメッセージング機能を利用する場合は本クラス、@ref UserMessage 、@ref TextMessage のいずれかのクラスを利用してください。
 */
class BinaryMessage: public UserMessage
{

#else
class BinaryMessage: public DDLCLASS(BinaryMessage)
{

#endif

public:

    /*
     Description: Default constructor.
     */

    /*!
     @brief コンストラクタです
     */
    BinaryMessage()
    {
    }

    /*
     Description: Assignment constructor.
     Parameters:
     binaryBody: Binary body to attach to message.
     */

    /*!
     @brief コンストラクタです

     @param[in]    pBuffer   バッファアドレス
     @param[in]    bufferSize    バッファサイズ
     */
    BinaryMessage(const void* pBuffer, qUnsignedInt32 bufferSize)
    {
        SetBuffer(pBuffer, bufferSize);
    }

    /*
     Description: Accessor to set binary body.
     Parameters:
     binaryBody: Binary body to attach to message.
     */

    /*!
     @brief 指定したバッファをコピーしてバイナリデータをセットします。
            MESSAGING_BINARY_MAX_SIZE を超えるデータは、インスタントメッセージング機能では使用できません。
            サーバー送信時にエラーとなります。

     @param[in]    pBuffer    バッファアドレス
     @param[in]    bufferSize    バッファサイズ
     */
    void SetBuffer(const void* pBuffer, size_t bufferSize)
    {
        SYSTEMCHECK(pBuffer != NULL && NEX_T("BinaryMessage::SetBuffer() pBuffer is NULL"));
        SYSTEMCHECK(bufferSize > 0 && NEX_T("BinaryMessage::SetBuffer() bufferSize is 0"));
        if (!pBuffer || bufferSize == 0)
        {
            // Release版は何もしない
            return;
        }

        m_binaryBody.clear();
        m_binaryBody.push_back(static_cast<const qByte*>(pBuffer), bufferSize);
    }

    /*
     Description: Accessor to get binary body.
     Returns: Message binary body
     */

    /*!
     @brief バイナリデータのバッファアドレスを取得します。

     バッファ未設定の場合は、NULLが返ります。

     @return    バッファアドレス
     */
    const void* GetBufferPtr() const
    {
        // front()はempty()==trueでSYSTEMCHECKに引っかかるので、empty()で分岐。
        if (m_binaryBody.empty())
        {
            return NULL;
        }
        return static_cast<const void*>(&m_binaryBody.front());
    }
    /*!
     @brief バイナリデータのバッファサイズを取得します。

     バッファ未設定の場合は、0が返ります。

     @return    バイナリサイズ
     */
    size_t GetBufferSize() const
    {
        return m_binaryBody.size();
    }

    /*
     for debug
     */
    virtual void Trace(qUnsignedInt64 traceFlag)
    {
        UserMessage::Trace(traceFlag);
        QTRACE(traceFlag, NEX_T("BinaryMessage:") << this);
        QTRACE(traceFlag, NEX_T("  BinaryBody Size:") << m_binaryBody.size());
        QTRACE(traceFlag, NEX_T("  BinaryBody:"));
        StringStream ss;
        ss << noshowbase; // 0xを付けない
        ss << zerohex;    // 足りない上位桁は0で埋める
        for (qUnsignedInt i = 0; i < m_binaryBody.size(); i++)
        {
            if (i % 64 == 0)
            {
                if (ss.GetLength() > 0)
                {
                    QTRACE(traceFlag, ss.CStr());
                }
                ss.Clear();
                ss << NEX_T("    ");
            }
            ss << m_binaryBody[i];
        }
        if (ss.GetLength() > 0)
        {
            QTRACE(traceFlag, ss.CStr());
        }
    }
};

END_NAMESPACE_NEX

